Package: TrialSimulator
Type: Package
Title: Clinical Trial Simulator
Version: 1.20.1
Description: Simulate phase II and/or phase III clinical trials. It supports various types of endpoints and adaptive strategies. Tools for carrying out graphical testing procedure and combination test under group sequential design are also provided. 
License: MIT + file LICENSE
Encoding: UTF-8
Imports: base64enc, dplyr, emmeans, ggplot2, gMCPLite, htmltools,
        mvtnorm, R6, Rcpp, rlang, rpact, rstudioapi, survival, utils
LinkingTo: Rcpp
Authors@R: 
    person("Han", "Zhang", , "zhangh.ustc@gmail.com", role = c("cre", "aut"))
RoxygenNote: 7.3.3
Suggests: data.table, DoseFinding, graphicalMCP, kableExtra, knitr,
        mirai, PWEALL, PWEXP, rmarkdown, simdata, survminer, testthat
        (>= 3.0.0)
VignetteBuilder: knitr
URL: https://zhangh12.github.io/TrialSimulator/
BugReports: https://github.com/zhangh12/TrialSimulator/issues
Depends: R (>= 4.1.0)
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-06-15 07:05:08 UTC; zhhan
Author: Han Zhang [cre, aut]
Maintainer: Han Zhang <zhangh.ustc@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-15 07:40:08 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-06-15 10:38:37 UTC; unix
Archs: TrialSimulator.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("&",Condition)
S3method("|",Condition)
S3method(plot,milestone_time_summary)
S3method(plot,three_state_model)
S3method(print,dynamic_rng_function)
export(Arms)
export(Controllers)
export(CorrelatedPfsAndOs2)
export(CorrelatedPfsAndOs3)
export(CorrelatedPfsAndOs4)
export(DynamicRNGFunction)
export(Endpoints)
export(GraphicalTesting)
export(GroupSequentialTest)
export(Listeners)
export(Milestones)
export(PiecewiseConstantExponentialRNG)
export(Regimens)
export(StaggeredRecruiter)
export(Surv)
export(Trials)
export(add_arms)
export(arm)
export(calendarTime)
export(controller)
export(crossover)
export(doNothing)
export(endpoint)
export(enrollment)
export(eventNumber)
export(expandRegimen)
export(fitCoxph)
export(fitFarringtonManning)
export(fitLinear)
export(fitLogistic)
export(fitLogrank)
export(listener)
export(milestone)
export(qPiecewiseExponential)
export(rconst)
export(regimen)
export(remove_arms)
export(resize)
export(set_duration)
export(solveMixtureExponentialDistribution)
export(solvePiecewiseConstantExponentialDistribution)
export(solveThreeStateModel)
export(strata)
export(summarizeDataFrame)
export(summarizeMilestoneTime)
export(trial)
export(update_generator)
export(update_sample_ratio)
export(weibullDropout)
import(ggplot2)
import(survival)
importFrom(R6,R6Class)
importFrom(Rcpp,sourceCpp)
importFrom(base64enc,base64encode)
importFrom(dplyr,"%>%")
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,contains)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(emmeans,contrast)
importFrom(emmeans,emmeans)
importFrom(emmeans,regrid)
importFrom(gMCPLite,hGraph)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(htmltools,HTML)
importFrom(mvtnorm,pmvnorm)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,enquos)
importFrom(rlang,expr_text)
importFrom(rlang,sym)
importFrom(rpact,getDesignGroupSequential)
importFrom(rstudioapi,isAvailable)
importFrom(rstudioapi,viewer)
importFrom(stats,IQR)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,glm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.response)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,rexp)
importFrom(stats,runif)
importFrom(stats,rweibull)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(survival,Surv)
importFrom(survival,strata)
importFrom(utils,head)
importFrom(utils,stack)
importFrom(utils,tail)
useDynLib(TrialSimulator, .registration = TRUE)

# TrialSimulator <img src="man/figures/logo.svg" align="right" width="175" />

<!-- badges: start -->
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![Code Coverage](https://codecov.io/gh/zhangh12/TrialSimulator/branch/main/graph/badge.svg)](https://app.codecov.io/gh/zhangh12/TrialSimulator)
<!-- badges: end -->

**&#x2695; No `for` loop in clinical trial simulation &#x2695;**

`TrialSimulator` is a system for declaratively implementing clinical trial simulations, inspired by the modular grammar of trial design. You provide the components—arms, endpoint-generation rules, milestones, and analysis methods—specify how they are combined under fixed or adaptive designs, and `TrialSimulator` takes care of the details, allowing you to focus on the design while it handles the mechanics.

`TrialSimulator` provides a suite of helper functions to support the full trial life cycle. 

- For **data generation**, it offers utilities to sample endpoints from common distributions. 
- For **adaptation**, it includes execution functions that modify trial settings *in place*. 
- For **analysis**, it implements methods such as regression models, group sequential tests, graphical multiple testing procedures, combination tests, and closed tests. These tools are entirely optional, but they make it simple to construct and evaluate complex trial designs without sacrificing flexibility.

## Shiny App

An online version of this app, `TrialDesigner`, is hosted on [shinyapps.io](https://bx7ttm-han-zhang.shinyapps.io/TrialDesigner/). This app is still under development. 

## Validation

The validation documents for this R package are hosted [here](https://github.com/zhangh12/TrialSimulatorDocuments) and are continuously updated.

## Installation

### Release

You can install the current release version from `CRAN` with: 

``` r
install.packages("TrialSimulator")
```

### Development

You can install the development version from 
[GitHub](https://github.com/zhangh12/TrialSimulator) with:

``` r
if(!requireNamespace("remotes", quietly = TRUE)){
  install.packages("remotes")
}
remotes::install_github(
  "zhangh12/TrialSimulator", 
  dependencies = TRUE
)
```

## Getting Started

`TrialSimulator` is designed with a modular architecture, allowing users to systematically construct and simulate clinical trials in a structured and flexible manner. The simulation process is broken down into three key steps:

1. **Defining Endpoints**: Specify the treatment endpoints that will be evaluated throughout the trial.
2. **Specifying Milestone**: Establish the conditions under which specific milestones (e.g., interim analysis, futility assessment, or stopping rules) will be triggered.
3. **Defining Actions for Milestones**: Determine the appropriate actions to be taken once a milestone is triggered, such as adjusting enrollment, terminating treatment arms, or conducting statistical analysis.

To efficiently utilize `TrialSimulator` for clinical trial simulations, we recommend reading the following vignettes in order:

- Define endpoints and arms
  - [Time-to-event (TTE) endpoints](https://zhangh12.github.io/TrialSimulator/articles/defineTimeToEventEndpoints.html)
    - [Correlated PFS and OS using a Gumbel copula](https://zhangh12.github.io/TrialSimulator/articles/simulatePfsAndOsGumbel.html)
    - [Correlated PFS and OS under an illness-death model](https://zhangh12.github.io/TrialSimulator/articles/simulatePfsAndOsIdm.html)
  - [Non-TTE endpoints](https://zhangh12.github.io/TrialSimulator/articles/defineNonTimeToEventEndpoints.html)
    - [Special case: longitudinal endpoints](https://zhangh12.github.io/TrialSimulator/articles/defineLongitudinalEndpoints.html)
  - [Define and summarize arms](https://zhangh12.github.io/TrialSimulator/articles/defineArms.html)
- Condition system for milestones
  - [Triggering trial milestones](https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html)
- Define actions for milestones
  - [Action function](https://zhangh12.github.io/TrialSimulator/articles/actionFunctions.html)
- Examples: fixed design
  - [Design with correlated endpoints, and custom data generator](https://zhangh12.github.io/TrialSimulator/articles/fixedDesign.html)
- Examples: adaptive design
  - [Seamless design with dose selection, interim, and multiple endpoints](https://zhangh12.github.io/TrialSimulator/articles/adaptiveDesign.html)
  - [Response-adaptive design](https://zhangh12.github.io/TrialSimulator/articles/responseAdaptive.html)
  - [Dose-ranging study](https://zhangh12.github.io/TrialSimulator/articles/doseRanging.html)
  - [Enrichment design] vignette is under development. 
  - [Platform trial] vignette is under development. 
  - [Basket trial] vignette is under development. 
- Examples: crossover
  - [Crossover trial with wash-out periods](https://zhangh12.github.io/TrialSimulator/articles/crossoverWashout.html)
  - [Dynamic treatment switching](https://zhangh12.github.io/TrialSimulator/articles/dynamicTreatmentSwitching.html)
  - [Crossover at specified time points] vignette is coming!
- Built-in methods supported in `TrialSimulator`
  - [Wrapper functions of common statistical methods for estimating treatment effect](https://zhangh12.github.io/TrialSimulator/articles/wrappers.html)
  - [Group sequential test] See `?GroupSequentialTest`. 
  - [Graphical testing procedure] See `?GraphicalTesting`. 
  - [Combination test based on independent increment] extended from [Jorgens et al. 2019](https://onlinelibrary.wiley.com/doi/10.1002/pst.1926). Vignette is under development. 

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/crossoverWashout/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(TrialSimulator)
library(mvtnorm)
library(dplyr)
library(kableExtra)

## ----dadfaiq------------------------------------------------------------------
knitr::include_graphics('crossover_readout.png')

## ----rng, echo=FALSE----------------------------------------------------------
rng <- function(n, means, vcov = diag(1, 8)){
  ret <- as.data.frame(rmvnorm(n, mean = means, sigma = vcov))
  colnames(ret) <- c('baseline1', 'ep1', 
                     'baseline2', 'ep2', 
                     'baseline3', 'ep3', 
                     'baseline4', 'ep4')
  ret
}

## ----results='asis'-----------------------------------------------------------
all_endpoint_name <- c('baseline1', 'ep1', 
                       'baseline2', 'ep2', 
                       'baseline3', 'ep3', 
                       'baseline4', 'ep4')

readouts <- c(baseline1 = 0, ep1 = 3, 
              baseline2 = 4, ep2 = 8, 
              baseline3 = 10, ep3 = 15, 
              baseline4 = 17.5, ep4 = 19.5)

eps <- endpoint(
  name = all_endpoint_name,
  type = rep('non-tte', 8), 
  readout = readouts, 
  generator = rng, means = rep(c(0, .5), 4)
)

arm1 <- arm(name = 'ABCD')
arm1$add_endpoints(eps)
arm1

## ----ref.label='rng', eval=FALSE----------------------------------------------
# rng <- function(n, means, vcov = diag(1, 8)){
#   ret <- as.data.frame(rmvnorm(n, mean = means, sigma = vcov))
#   colnames(ret) <- c('baseline1', 'ep1',
#                      'baseline2', 'ep2',
#                      'baseline3', 'ep3',
#                      'baseline4', 'ep4')
#   ret
# }

## -----------------------------------------------------------------------------
eps <- endpoint(
  name = all_endpoint_name,
  type = rep('non-tte', 8), 
  readout = readouts, 
  generator = rng, means = rep(c(0, .6), 4) # diff means
)

arm2 <- arm(name = 'BDAC')
arm2$add_endpoints(eps)

eps <- endpoint(
  name = all_endpoint_name,
  type = rep('non-tte', 8), 
  readout = readouts, 
  generator = rng, means = rep(c(0, .2), 4), vcov = diag(1.2, 8) # diff means/vcov
)

arm3 <- arm(name = 'CADB')
arm3$add_endpoints(eps)

eps <- endpoint(
  name = all_endpoint_name,
  type = rep('non-tte', 8), 
  readout = readouts, 
  generator = rng, means = rep(c(0, 0), 4) # diff means
)

arm4 <- arm(name = 'DCBA')
arm4$add_endpoints(eps)

## -----------------------------------------------------------------------------
accrual_rate <- data.frame(end_time = c(6, Inf),
                           piecewise_rate = c(10, 10))

trial <- trial(name = 'crossover-trial', 
               n_patients = 60, 
               duration = 28,
               enroller = StaggeredRecruiter, accrual_rate = accrual_rate, 
               silent = TRUE)

trial$add_arms(sample_ratio = c(1, 1, 1, 1), arm1, arm2, arm3, arm4)
trial

## ----action_function, echo=FALSE----------------------------------------------
action <- function(trial){
  locked_data <- trial$get_locked_data('final')
  ## omit statistical analysis
  
  trial$save(value = 'anything', name = 'result')
  ## save more results for summary of simulation
  # trial$save(value = ..., name = ...)
}

## -----------------------------------------------------------------------------
final <- milestone(name = 'final', 
                   when = calendarTime(time = 25.5), 
                   action = action)

## -----------------------------------------------------------------------------
final <- milestone(name = 'final', 
                   when = eventNumber(endpoint = 'ep4', n = 60), 
                   action = action)

## -----------------------------------------------------------------------------
final <- milestone(name = 'final', 
                   when = enrollment(n = 60, min_treatment_duration = 19.5), 
                   action = action)

## ----ref.label='action_function', eval=FALSE----------------------------------
# action <- function(trial){
#   locked_data <- trial$get_locked_data('final')
#   ## omit statistical analysis
# 
#   trial$save(value = 'anything', name = 'result')
#   ## save more results for summary of simulation
#   # trial$save(value = ..., name = ...)
# }

## -----------------------------------------------------------------------------
listener <- listener()
listener$add_milestones(final)

controller <- controller(trial, listener)
controller$run(n = 1, plot_event = TRUE)

## -----------------------------------------------------------------------------
output <- controller$get_output()

output %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache = TRUE,
  cache.path = 'cache/defineNonTimeToEventEndpoints/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, message = FALSE, echo=FALSE---------------------------------------
library(dplyr)
library(TrialSimulator)
set.seed(12345)

## ----lljalk-------------------------------------------------------------------
## endpoints in placebo arm
tumor_cfb_pbo <- endpoint(name = 'cfb', type = 'non-tte', 
                          readout = c(cfb = 6),
                          generator = rnorm, mean = .8, sd = 3.2)
orr_pbo <- endpoint(name = 'orr', type = 'non-tte', 
                    readout = c(orr = 2),
                    generator = rbinom, size = 1, prob = .1)

## define the placebo arm
pbo <- arm(name = 'placebo')
pbo$add_endpoints(tumor_cfb_pbo, orr_pbo)

## endpoints in treatment arm
tumor_cfb_trt <- endpoint(name = 'cfb', type = 'non-tte', 
                          readout = c(cfb = 6), 
                          generator = rnorm, mean = -2.3, sd = 1.5)
orr_trt <- endpoint(name = 'orr', type = 'non-tte', 
                    readout = c(orr = 2), 
                    generator = rbinom, size = 1, prob = .25)

## define the treatment arm
trt <- arm(name = 'treatment')
trt$add_endpoints(tumor_cfb_trt, orr_trt)

## ----baseline-ep--------------------------------------------------------------
## a baseline biomarker available at randomization
biomarker <- endpoint(name = 'biomarker', type = 'baseline',
                      generator = rbinom, size = 1, prob = .3)

## ----baseline-equiv, eval = FALSE---------------------------------------------
# ## these are equivalent
# endpoint(name = 'biomarker', type = 'baseline',
#          generator = rbinom, size = 1, prob = .3)
# endpoint(name = 'biomarker', type = 'non-tte', readout = c(biomarker = 0),
#          generator = rbinom, size = 1, prob = .3)

## ----oheahs-------------------------------------------------------------------
dropout_pars <- weibullDropout(c(12, 18), c(.15, .30))
dropout_pars

## ----lihoehf------------------------------------------------------------------
accrual_rate <- data.frame(end_time = c(6, Inf), 
                           piecewise_rate = c(10, 20))

trial <- trial(
  name = 'Trial-31415', description = 'Example Clinical Trial', 
  n_patients = 420, duration = 30, 
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate, 
  dropout = rweibull, scale = 30.636, shape = 1.939
)

## add arms to the trial
trial$add_arms(sample_ratio = c(1, 1), trt, pbo)
trial

## ----llieh--------------------------------------------------------------------
interim <- milestone(name = 'interim', 
                     when = eventNumber(endpoint = 'orr', n = 60), 
                     action = doNothing)

random <- milestone(name = 'random', 
                    when = 
                      calendarTime(time = 10) & 
                      (eventNumber(endpoint = 'cfb', n = 100) | 
                         eventNumber(endpoint = 'orr', n = 180)
                       ), 
                    action = doNothing)

final <- milestone(name = 'final', 
                   when = calendarTime(time = 30), 
                   action = doNothing)

## ----ihhf---------------------------------------------------------------------
## register milestones to the listener
listener <- listener()
listener$add_milestones(interim, random, final)

## run the trial
controller <- controller(trial, listener)
controller$run()

## ----ihegea-------------------------------------------------------------------
interim_data <- trial$get_locked_data(milestone_name = 'interim')
random_data <- trial$get_locked_data(milestone_name = 'random')
final_data <- trial$get_locked_data(milestone_name = 'final')
head(interim_data)

## ----eqfee--------------------------------------------------------------------
not_ready_at_interim <- 
  interim_data %>% 
  dplyr::filter(is.na(cfb) & 
                  is.na(orr) & 
                  enroll_time + 6 < dropout_time) %>% 
  head() %>% 
  print()

random_data %>% 
  dplyr::filter(patient_id %in% not_ready_at_interim$patient_id) %>% 
  print()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/doseRanging/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(TrialSimulator)
library(DoseFinding)
library(dplyr)
library(kableExtra)
set.seed(12345)

## ----flaf---------------------------------------------------------------------
mods <- DoseFinding::Mods(sigEmax = rbind(c(1, 3)), 
                          placEff = log(.1/(1 - .1)), 
                          maxEff = log(.25/(1 - .25)) - log(.1/(1 - .1)),
                          doses = c(0, 0.5, 1.5, 2.5, 4))

DoseFinding::plotMods(mods, trafo = function(x) 1/(1+exp(-x)))

## response rates on curve
x <- DoseFinding::getResp(mods, doses = c(0, 0.5, 1.5, 2.5, 4))
1 / (1 + exp(-unclass(x)))

## -----------------------------------------------------------------------------
ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
               generator = rbinom, size = 1, prob = 0.1)
pbo <- arm(name = 'dose = 0.0')
pbo$add_endpoints(ep)

ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
               generator = rbinom, size = 1, prob = 0.25)
trt4 <- arm(name = 'dose = 4.0')
trt4$add_endpoints(ep)

## -----------------------------------------------------------------------------
accrual_rate <- data.frame(end_time = c(7, Inf),
                           piecewise_rate = c(5, 20))

trial <- trial(name = '123', n_patients = 150, duration = 14,
               enroller = StaggeredRecruiter, accrual_rate = accrual_rate, 
               silent = TRUE)

trial$add_arms(sample_ratio = c(1, 1), pbo, trt4)
trial

## ----go_nogo, echo=FALSE------------------------------------------------------
go_nogo <- function(data){

  ## candidate models for MCPMod
  doses <- c(0, 0.5, 1.5, 2.5, 4)
  candidates <- Mods(emax = c(.25, 1),
                     sigEmax = rbind(c(1, 3), c(2.5, 4)),
                     betaMod = c(1.1, 1.1),
                     placEff = log(.1/(1 - .1)),
                     maxEff = log(.25/(1 - .25)) - log(.1/(1 - .1)),
                     doses = doses)

  fit <- glm(ep ~ factor(arm) + 0, data = data, family = binomial)
  mu_hat <- coef(fit)
  S_hat <- vcov(fit)

  ## multiple contrast test
  test <- DoseFinding::MCTtest(dose = doses,
                               mu_hat, S = S_hat,
                               models = candidates, type = "general")

  ## model averaging
  model <- DoseFinding::maFitMod(dose = doses,
                                 mu_hat, S = S_hat,
                                 models = c("emax", "sigEmax", "betaMod"))
  
  ## predict response rate per dose
  prd <- predict(model, summaryFct = median, doseSeq = doses)
  
  ## convert to scale of probability
  prd_rate <- 1 / (1 + exp(-prd))

  ## go/no-go rule: MCP test p-value < 0.05 and estimated effect > 10%
  ifelse(min(attr(test$tStat, 'pVal')) < .05 &
           max(prd_rate - prd_rate[1]) > .1, 'go', 'no-go')

}

## ----action_at_interim, echo=FALSE--------------------------------------------
action_at_interim <- function(trial){

  ## get data snapshot
  locked_data <- trial$get_locked_data('interim')

  ## compare two arms
  ## Risk difference = response rate in high dose - response rate in placebo
  fit <- fitLogistic(ep ~ arm, placebo = 'dose = 0.0',
                     data = locked_data, alternative = 'greater',
                     scale = 'risk difference')

  ## for summary of early termination
  trial$save(value = fit$z, name = 'z_value')
  trial$save(value = ifelse(fit$z > 1.64, 'add dose arms', 'stop trial'), 
             name = 'interim_decision')

  ## create three dose arms
  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
                 generator = rbinom, size = 1, prob = .112)
  trt1 <- arm(name = 'dose = 0.5')
  trt1$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
                 generator = rbinom, size = 1, prob = .208)
  trt2 <- arm(name = 'dose = 1.5')
  trt2$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
                 generator = rbinom, size = 1, prob = .241)
  trt3 <- arm(name = 'dose = 2.5')
  trt3$add_endpoints(ep)

  ## add three new arms to trial
  trial$add_arms(sample_ratio = c(2, 2, 2), trt1, trt2, trt3)
  
}

## ----action_at_final, echo=FALSE----------------------------------------------
action_at_final <- function(trial){

  locked_data <- trial$get_locked_data('final')

  trial$save(value = go_nogo(locked_data), name = 'decision')

}

## -----------------------------------------------------------------------------
interim <- milestone(name = 'interim',
                     when = eventNumber(endpoint = 'ep', n = 30),
                     action = action_at_interim)

final <- milestone(name = 'final',
                   when = eventNumber(endpoint = 'ep', n = 150),
                   action = action_at_final)

## ----ref.label='action_at_interim', eval=FALSE--------------------------------
# action_at_interim <- function(trial){
# 
#   ## get data snapshot
#   locked_data <- trial$get_locked_data('interim')
# 
#   ## compare two arms
#   ## Risk difference = response rate in high dose - response rate in placebo
#   fit <- fitLogistic(ep ~ arm, placebo = 'dose = 0.0',
#                      data = locked_data, alternative = 'greater',
#                      scale = 'risk difference')
# 
#   ## for summary of early termination
#   trial$save(value = fit$z, name = 'z_value')
#   trial$save(value = ifelse(fit$z > 1.64, 'add dose arms', 'stop trial'),
#              name = 'interim_decision')
# 
#   ## create three dose arms
#   ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
#                  generator = rbinom, size = 1, prob = .112)
#   trt1 <- arm(name = 'dose = 0.5')
#   trt1$add_endpoints(ep)
# 
#   ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
#                  generator = rbinom, size = 1, prob = .208)
#   trt2 <- arm(name = 'dose = 1.5')
#   trt2$add_endpoints(ep)
# 
#   ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
#                  generator = rbinom, size = 1, prob = .241)
#   trt3 <- arm(name = 'dose = 2.5')
#   trt3$add_endpoints(ep)
# 
#   ## add three new arms to trial
#   trial$add_arms(sample_ratio = c(2, 2, 2), trt1, trt2, trt3)
# 
# }

## ----ref.label='action_at_final', eval=FALSE----------------------------------
# action_at_final <- function(trial){
# 
#   locked_data <- trial$get_locked_data('final')
# 
#   trial$save(value = go_nogo(locked_data), name = 'decision')
# 
# }

## ----message=FALSE------------------------------------------------------------
listener <- listener()
listener$add_milestones(interim, final)

controller <- controller(trial, listener)
controller$run(n = 1, plot_event = TRUE)

## -----------------------------------------------------------------------------
output <- controller$get_output()

output %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")

## ----eval=FALSE, message=FALSE------------------------------------------------
# ## important: reset before calling run() again
# controller$reset()
# controller$run(n = 10, plot_event = FALSE, silent = TRUE)
# res <- controller$get_output()

## ----echo=FALSE---------------------------------------------------------------
res <- TrialSimulator:::getDoseRangingOutput()

## -----------------------------------------------------------------------------
res %>%
  kable(escape = FALSE) %>%
  kable_styling(bootstrap_options = "striped",
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")

## ----message=FALSE------------------------------------------------------------
## Number of overall Go
sum(res$interim_decision == 'add dose arms' & 'decision' == 'go')

## Number of early trial termination
sum(res$interim_decision == 'stop trial')

## ----ref.label='go_nogo', eval=FALSE------------------------------------------
# go_nogo <- function(data){
# 
#   ## candidate models for MCPMod
#   doses <- c(0, 0.5, 1.5, 2.5, 4)
#   candidates <- Mods(emax = c(.25, 1),
#                      sigEmax = rbind(c(1, 3), c(2.5, 4)),
#                      betaMod = c(1.1, 1.1),
#                      placEff = log(.1/(1 - .1)),
#                      maxEff = log(.25/(1 - .25)) - log(.1/(1 - .1)),
#                      doses = doses)
# 
#   fit <- glm(ep ~ factor(arm) + 0, data = data, family = binomial)
#   mu_hat <- coef(fit)
#   S_hat <- vcov(fit)
# 
#   ## multiple contrast test
#   test <- DoseFinding::MCTtest(dose = doses,
#                                mu_hat, S = S_hat,
#                                models = candidates, type = "general")
# 
#   ## model averaging
#   model <- DoseFinding::maFitMod(dose = doses,
#                                  mu_hat, S = S_hat,
#                                  models = c("emax", "sigEmax", "betaMod"))
# 
#   ## predict response rate per dose
#   prd <- predict(model, summaryFct = median, doseSeq = doses)
# 
#   ## convert to scale of probability
#   prd_rate <- 1 / (1 + exp(-prd))
# 
#   ## go/no-go rule: MCP test p-value < 0.05 and estimated effect > 10%
#   ifelse(min(attr(test$tStat, 'pVal')) < .05 &
#            max(prd_rate - prd_rate[1]) > .1, 'go', 'no-go')
# 
# }

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache = TRUE,
  cache.path = 'cache/defineArms/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, message = FALSE, echo = FALSE-------------------------------------
library(dplyr)
library(simdata)
library(TrialSimulator)
set.seed(12345)

## ----garalbaiola--------------------------------------------------------------
rng <- function(n, pfs_rate, os_rate, psa_mean, psa_sd, corr_matrix){
  
  dist <- list()
  dist[['PFS']] <- function(x) qexp(x, rate = pfs_rate)
  dist[['OS']] <- function(x) qexp(x, rate = os_rate)
  dist[['PSA_baseline']] <- function(x) qnorm(x, mean = psa_mean, sd = psa_sd)
  dist[['PSA_year1']] <- function(x) qnorm(x, mean = psa_mean - 12, sd = psa_sd)
  dsgn = simdata::simdesign_norta(cor_target_final = corr_matrix, 
                                dist = dist, 
                                transform_initial = data.frame,
                                names_final = names(dist), 
                                seed_initial = 1)
  
  simdata::simulate_data(dsgn, n_obs = n) %>% 
    mutate(PFS = pmin(PFS, OS)) %>% 
    mutate(PFS_event = 1, OS_event = 1)
  
}

## ----adldaieafg, results='asis'-----------------------------------------------
ep1 <- endpoint(name = c('PSA_baseline', 'PSA_year1', 'OS', 'PFS'), 
                type = c('non-tte', 'non-tte', 'tte', 'tte'), 
                readout = c(PSA_baseline = 0, PSA_year1 = 1), 
                generator = rng, 
                pfs_rate = log(2)/2.5, os_rate = log(2)/4.5, 
                psa_mean = 20, psa_sd = 4, 
                corr_matrix = matrix(c(1, .6, -.5, -.4, 
                                       .6, 1, -.4, -.3, 
                                       -.5, -.4, 1, .7, 
                                       -.4, -.3, .7, 1), nrow = 4))

ep1

## ----ioagjie, results='asis'--------------------------------------------------
ep2 <- endpoint(name = 'biomarker', 
                type = 'non-tte', 
                readout = c(biomarker = 0), 
                generator = rbinom, 
                size = 1, prob = .3)
ep2

## ----dlaieafj, results='asis'-------------------------------------------------
trt <- arm(name = 'treated')
trt$add_endpoints(ep1, ep2)
trt

## ----eiaojda, results='asis'--------------------------------------------------
trt <- arm(name = 'treated', PSA_baseline > 10 & PSA_year1 > 0)
trt$add_endpoints(ep1, ep2)
trt

## ----eiaojfa------------------------------------------------------------------
## not recommended
tmp <- trt$generate_data(100)
head(tmp)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/responseAdaptive/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(dplyr)
library(kableExtra)
library(DoseFinding)
library(TrialSimulator)
set.seed(12345)

## ----eaioajf------------------------------------------------------------------
rng <- function(n, dose){

  model <- DoseFinding::Mods(
    emax = c(2.6, 12.5),
    placEff = 1.25, maxEff = 0.1,
    doses = c(0, 20, 25, 50, 100))

  data.frame(
    fev1 = rnorm(n, mean = DoseFinding::getResp(model, doses = dose), sd = .05)
  )

}

## ----lalglafj-----------------------------------------------------------------
fev1 <- endpoint(name = 'fev1', type = 'non-tte', readout = c(fev1 = 4),
                 generator = rng, dose = 0)
pbo <- arm(name = '0.0')
pbo$add_endpoints(fev1)

fev1 <- endpoint(name = 'fev1', type = 'non-tte', readout = c(fev1 = 4),
                 generator = rng, dose = 20.0)
dose1 <- arm(name = '20.0')
dose1$add_endpoints(fev1)

fev1 <- endpoint(name = 'fev1', type = 'non-tte', readout = c(fev1 = 4),
                 generator = rng, dose = 25.0)
dose2 <- arm(name = '25.0')
dose2$add_endpoints(fev1)

fev1 <- endpoint(name = 'fev1', type = 'non-tte', readout = c(fev1 = 4),
                 generator = rng, dose = 30.0)
dose3 <- arm(name = '30.0')
dose3$add_endpoints(fev1)

fev1 <- endpoint(name = 'fev1', type = 'non-tte', readout = c(fev1 = 4),
                 generator = rng, dose = 35.0)
dose4 <- arm(name = '35.0')
dose4$add_endpoints(fev1)

## ----weoieaf------------------------------------------------------------------
accrual_rate <- data.frame(end_time = c(24, Inf),
                           piecewise_rate = c(100/24, 100/12))
trial <- trial(
  name = 'Trial-3415', n_patients = 200,
  seed = 1727811904, duration = 40,
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
  silent = TRUE
)

trial$add_arms(sample_ratio = rep(1, 5), pbo, dose1, dose2, dose3, dose4)
trial

## ----helpers, echo=FALSE------------------------------------------------------
compute_sample_ratio <- function(data){

  data$dose <- as.numeric(data$arm)
  fit <- lm(fev1 ~ factor(dose) - 1, data = data)
  dose <- unique(sort(data$dose))
  mu_hat <- coef(fit)
  S_hat <- vcov(fit)

  suppressMessages(
    ma_fit <- DoseFinding::maFitMod(dose, mu_hat, S = S_hat,
                                    models = c("emax", "sigEmax", "quadratic"))
  )

  pred <- predict(ma_fit, doseSeq = c(0, 20, 25, 30, 35), summaryFct = NULL)
  prob <- apply(pred[, -1] - pred[, 1], 2, function(x){mean(x > .08)})
  sample_ratio <- c(.2, (1 - .2) * prob / sum(prob)) %>% unname()

  sample_ratio
}

multiple_contrast_test <- function(data){
  
  candidate_models <- DoseFinding::Mods(
    emax = c(2.6, 12.5), sigEmax = c(30.5, 3.5), quadratic = -0.00776,
    placEff = 1.25, maxEff = 0.15, doses = c(0, 20, 25, 30, 35))

  data$dose <- as.numeric(data$arm)
  test <- DoseFinding::MCTtest(dose = dose, resp = fev1,
                               models = candidate_models, data = data)
  
  ## at least one dose shows significant non-flatten pattern
  any(attr(test$tStat, 'pVal') < .05)

}

## ----stage_action, echo=FALSE-------------------------------------------------
stage_action <- function(trial, milestone_name){

  locked_data <- trial$get_locked_data(milestone_name)
  new_sample_ratio <- compute_sample_ratio(locked_data)

  trial$update_sample_ratio(arm_names = c('0.0', '20.0', '25.0', '30.0', '35.0'),
                            sample_ratios = new_sample_ratio)
  
  message(milestone_name, ': ')
  data.frame(table(locked_data$arm), new_sample_ratio) %>%
    setNames(c('dose', 'total_n', 'new_ratio')) %>% print()

}

## ----final_action, echo=FALSE-------------------------------------------------
final_action <- function(trial){

  locked_data <- trial$get_locked_data('final')
  
  message('final: ')
  data.frame(table(locked_data$arm)) %>%
    setNames(c('dose', 'total_n')) %>% print()

  trial$save(value = multiple_contrast_test(locked_data),
             name = 'MC_test')
}

## ----gfdassfl-----------------------------------------------------------------
stage1 <- milestone(name = 'stage 1',
                    when = eventNumber('fev1', n = 50),
                    action = stage_action, milestone_name = 'stage 1')

stage2 <- milestone(name = 'stage 2',
                    when = eventNumber('fev1', n = 120),
                    action = stage_action, milestone_name = 'stage 2')

final <- milestone(name = 'final',
                   when = eventNumber('fev1', n = 200),
                   action = final_action)

## ----ref.label='stage_action', eval=FALSE-------------------------------------
# stage_action <- function(trial, milestone_name){
# 
#   locked_data <- trial$get_locked_data(milestone_name)
#   new_sample_ratio <- compute_sample_ratio(locked_data)
# 
#   trial$update_sample_ratio(arm_names = c('0.0', '20.0', '25.0', '30.0', '35.0'),
#                             sample_ratios = new_sample_ratio)
# 
#   message(milestone_name, ': ')
#   data.frame(table(locked_data$arm), new_sample_ratio) %>%
#     setNames(c('dose', 'total_n', 'new_ratio')) %>% print()
# 
# }

## ----ref.label='final_action', eval=FALSE-------------------------------------
# final_action <- function(trial){
# 
#   locked_data <- trial$get_locked_data('final')
# 
#   message('final: ')
#   data.frame(table(locked_data$arm)) %>%
#     setNames(c('dose', 'total_n')) %>% print()
# 
#   trial$save(value = multiple_contrast_test(locked_data),
#              name = 'MC_test')
# }

## ----sdlatiow-----------------------------------------------------------------
listener <- listener()
listener$add_milestones(stage1, stage2, final)

controller <- controller(trial, listener)
controller$run(n = 1, silent = TRUE)

output <- controller$get_output()

output %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")

## ----qeaeal-------------------------------------------------------------------
output[, 'n_events_<stage 1>_<arms>']

output[, 'n_events_<stage 2>_<arms>']

output[, 'n_events_<final>_<arms>']

## ----ref.label='helpers', eval=FALSE------------------------------------------
# compute_sample_ratio <- function(data){
# 
#   data$dose <- as.numeric(data$arm)
#   fit <- lm(fev1 ~ factor(dose) - 1, data = data)
#   dose <- unique(sort(data$dose))
#   mu_hat <- coef(fit)
#   S_hat <- vcov(fit)
# 
#   suppressMessages(
#     ma_fit <- DoseFinding::maFitMod(dose, mu_hat, S = S_hat,
#                                     models = c("emax", "sigEmax", "quadratic"))
#   )
# 
#   pred <- predict(ma_fit, doseSeq = c(0, 20, 25, 30, 35), summaryFct = NULL)
#   prob <- apply(pred[, -1] - pred[, 1], 2, function(x){mean(x > .08)})
#   sample_ratio <- c(.2, (1 - .2) * prob / sum(prob)) %>% unname()
# 
#   sample_ratio
# }
# 
# multiple_contrast_test <- function(data){
# 
#   candidate_models <- DoseFinding::Mods(
#     emax = c(2.6, 12.5), sigEmax = c(30.5, 3.5), quadratic = -0.00776,
#     placEff = 1.25, maxEff = 0.15, doses = c(0, 20, 25, 30, 35))
# 
#   data$dose <- as.numeric(data$arm)
#   test <- DoseFinding::MCTtest(dose = dose, resp = fev1,
#                                models = candidate_models, data = data)
# 
#   ## at least one dose shows significant non-flatten pattern
#   any(attr(test$tStat, 'pVal') < .05)
# 
# }

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/crossoverAtMilestone/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(TrialSimulator)

## ----eval=FALSE---------------------------------------------------------------
# action <- function(trial){
#   locked_data <- trial$get_locked_data('interim')
#   ## ... interim analysis / decision making ...
#   trial$crossover(what = what_fn, how = how_fn)
# }
# 
# interim <- milestone(name = 'interim',
#                      when = eventNumber(endpoint = 'pfs', n = 300),
#                      action = action)

## ----eval=FALSE---------------------------------------------------------------
# crossover(trial, what = what_fn, how = how_fn)

## ----eval=FALSE---------------------------------------------------------------
# what(patient_data)   # -> patient_id, new_treatment
# when(patient_data)   # -> patient_id, switch_time
# how(patient_data)    # -> patient_id, <modified endpoints>

## ----funnel, echo=FALSE, fig.width=7.2, fig.height=4.2, out.width="95%", fig.alt="Funnel: trial_data is filtered to eligible patients (passed to what()), then to switchers (passed to when()), then to switchers with their switch_time (passed to how())."----
op <- par(mar = c(0, 0, 0, 0))
plot.new(); plot.window(xlim = c(0, 10), ylim = c(0, 10))
cx <- 5
yb <- c(9.3, 7.1, 4.9, 2.7, 0.6)            # band boundaries (top -> bottom)
wt <- c(3.4, 2.6, 1.8, 1.0)                 # band top widths
wd <- c(2.6, 1.8, 1.0, 0.5)                 # band bottom widths
for (i in 1:4) {
  polygon(cx + c(-wt[i], wt[i], wd[i], -wd[i]) / 2,
          c(yb[i], yb[i], yb[i + 1], yb[i + 1]),
          col = "#e8eef5", border = "#33618f", lwd = 1.4)
}
ymid <- (head(yb, -1) + tail(yb, -1)) / 2
left <- c("all patients\n(trial_data)",
          "eligible\n(open endpoint at T)",
          "switchers\n(what() selects)",
          "+ switch_time")
text(cx - 2.0, ymid, left, adj = 1, cex = 0.76)
fn <- c("what()", "when()", "how()")
for (i in 1:3) {
  arrows(cx + wt[i + 1] / 2 + 0.1, ymid[i + 1], cx + 2.0, ymid[i + 1],
         length = 0.07, col = "#33618f")
  text(cx + 2.1, ymid[i + 1], paste0("passed to ", fn[i]), adj = 0, cex = 0.82)
}
par(op)

## ----eval=FALSE---------------------------------------------------------------
# time_selector <- function(patient_data){
#   data.frame(
#     patient_id  = patient_data$patient_id,
#     switch_time = pmax(patient_data$pfs,
#                        patient_data$earliest_crossover_time_from_enrollment)
#   )
# }

## ----eval=FALSE---------------------------------------------------------------
# data_modifier <- function(patient_data){
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## extend only the residual (post-switch) survival; leave os unchanged
#     ## for patients whose event is at or before the switch
#     os = ifelse(patient_data$os > patient_data$switch_time,
#                 patient_data$switch_time +
#                   1.2 * (patient_data$os - patient_data$switch_time),
#                 patient_data$os)
#   )
# }

## ----eval=FALSE---------------------------------------------------------------
# what <- function(patient_data){
#   browser()                       # pause here with the eligible pool in scope
#   switchers <- patient_data[patient_data$arm == 'control', ]
#   data.frame(patient_id = switchers$patient_id, new_treatment = 'experimental')
# }

## -----------------------------------------------------------------------------
what <- function(patient_data){
  ## return only the patients who switch (here, everyone on control)
  switchers <- patient_data[patient_data$arm == 'control', ]
  data.frame(
    patient_id    = switchers$patient_id,
    new_treatment = 'experimental'
  )
}

when <- function(patient_data){
  data.frame(
    patient_id  = patient_data$patient_id,
    switch_time = pmax(patient_data$pfs,
                       patient_data$earliest_crossover_time_from_enrollment)
  )
}

how <- function(patient_data){
  data.frame(
    patient_id = patient_data$patient_id,
    os = ifelse(patient_data$os > patient_data$switch_time,
                patient_data$switch_time +
                  1.3 * (patient_data$os - patient_data$switch_time),
                patient_data$os)
  )
}

crossover_action <- function(trial){
  trial$get_locked_data('interim')        # interim decision making can go here
  ## delay = 0 opens crossover at the interim; use delay > 0 for a wash-out
  trial$crossover(what = what, when = when, how = how, delay = 0)
}

## ----echo=FALSE---------------------------------------------------------------
os_e  <- endpoint(name = 'os',  type = 'tte', generator = rexp, rate = log(2) / 12)
pfs_e <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2) / 6)
control <- arm(name = 'control', pfs <= os); control$add_endpoints(pfs_e, os_e)
trt     <- arm(name = 'trt',     pfs <= os); trt$add_endpoints(pfs_e, os_e)

trial <- trial(name = 'demo', n_patients = 200, seed = 123, duration = 36,
               enroller = StaggeredRecruiter,
               accrual_rate = data.frame(end_time = Inf, piecewise_rate = 15),
               silent = TRUE)
trial$add_arms(sample_ratio = c(1, 1), control, trt)

lst <- listener(silent = TRUE)
lst$add_milestones(
  milestone(name = 'interim', when = calendarTime(time = 12), action = crossover_action),
  milestone(name = 'final',   when = calendarTime(time = 36))
)
invisible(controller(trial, lst)$run(n = 1, silent = TRUE, plot_event = FALSE))

## -----------------------------------------------------------------------------
final <- trial$get_locked_data('final')
head(final[final$arm == 'control',
           c('patient_id', 'arm', 'regimen_trajectory', 'n_switches')])

## ----eval=FALSE---------------------------------------------------------------
# trial$crossover(what = what, how = how, delay = 2) # opens 2 time units later

## -----------------------------------------------------------------------------
head(expandRegimen(final))

## ---- regimen_trajectory encoding --------------------------------------------

test_that('regimen_trajectory is "arm@0" when no patient switches', {

  skip_if_not_installed('TrialSimulator')

  pfs_e <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/8)
  os_e  <- endpoint(name = 'os',  type = 'tte', generator = rexp, rate = log(2)/16)
  pbo   <- arm(name = 'placebo'); pbo$add_endpoints(pfs_e, os_e)
  trt   <- arm(name = 'trt');     trt$add_endpoints(pfs_e, os_e)

  # what() returns NA for everyone — nobody switches
  what_fn <- function(patient_data) data.frame(patient_id = patient_data$patient_id, new_treatment = NA_character_)
  when_fn <- function(patient_data) data.frame(patient_id = patient_data$patient_id, switch_time = patient_data$pfs)
  how_fn  <- function(patient_data) data.frame(patient_id = patient_data$patient_id)

  tr <- trial(name = 't', n_patients = 60, seed = 1, duration = 30,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 10),
              silent = TRUE)
  tr$add_regimen(regimen(what_fn, when_fn, how_fn))
  tr$add_arms(sample_ratio = c(1, 1), pbo, trt)

  lst <- listener(silent = TRUE)
  lst$add_milestones(milestone(name = 'final', when = calendarTime(time = 30)))
  controller(tr, lst)$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data('final')
  expect_equal(d$regimen_trajectory, paste0(d$arm, '@0'))
})

test_that('regimen_trajectory encodes a single switching round correctly', {

  skip_if_not_installed('TrialSimulator')

  pfs_e <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/8)
  os_e  <- endpoint(name = 'os',  type = 'tte', generator = rexp, rate = log(2)/16)
  pbo   <- arm(name = 'placebo'); pbo$add_endpoints(pfs_e, os_e)
  trt   <- arm(name = 'trt');     trt$add_endpoints(pfs_e, os_e)

  # all placebo patients switch to trt at their pfs time
  what_fn <- function(patient_data)
    data.frame(patient_id    = patient_data$patient_id,
               new_treatment = ifelse(patient_data$arm == 'placebo', 'trt', NA_character_))
  when_fn <- function(patient_data) data.frame(patient_id = patient_data$patient_id, switch_time = patient_data$pfs)
  how_fn  <- function(patient_data) data.frame(patient_id = patient_data$patient_id)

  tr <- trial(name = 't', n_patients = 100, seed = 2, duration = 200,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 10),
              silent = TRUE)
  tr$add_regimen(regimen(what_fn, when_fn, how_fn))
  tr$add_arms(sample_ratio = c(1, 1), pbo, trt)

  lst <- listener(silent = TRUE)
  lst$add_milestones(milestone(name = 'final', when = calendarTime(time = 200)))
  controller(tr, lst)$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data('final')

  # trt arm: no switch
  expect_equal(d$regimen_trajectory[d$arm == 'trt'], rep('trt@0', sum(d$arm == 'trt')))

  # placebo arm: trajectory starts with 'placebo@0;trt@' and switch_time == pfs
  pbo_d <- d[d$arm == 'placebo', ]
  expect_true(all(startsWith(pbo_d$regimen_trajectory, 'placebo@0;trt@')))

  long <- expandRegimen(pbo_d)
  switched <- long[long$regimen == 'trt', ]
  expect_equal(switched$switch_time_from_enrollment,
               pbo_d$pfs[match(switched$patient_id, pbo_d$patient_id)])
})

test_that('regimen_trajectory accumulates two switching rounds for the same patients', {

  skip_if_not_installed('TrialSimulator')

  pfs_e <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/6)
  os_e  <- endpoint(name = 'os',  type = 'tte', generator = rexp, rate = log(2)/15)
  pbo   <- arm(name = 'placebo'); pbo$add_endpoints(pfs_e, os_e)
  trt   <- arm(name = 'trt');     trt$add_endpoints(pfs_e, os_e)

  # round 1: placebo → low dose at pfs
  what1 <- function(patient_data) data.frame(patient_id    = patient_data$patient_id,
                                   new_treatment = ifelse(patient_data$arm == 'placebo', 'low dose', NA_character_))
  when1 <- function(patient_data) data.frame(patient_id = patient_data$patient_id, switch_time = patient_data$pfs)
  how1  <- function(patient_data) data.frame(patient_id = patient_data$patient_id)

  # round 2: same placebo patients → high dose at os/2
  what2 <- function(patient_data) data.frame(patient_id    = patient_data$patient_id,
                                   new_treatment = ifelse(patient_data$arm == 'placebo', 'high dose', NA_character_))
  when2 <- function(patient_data) data.frame(patient_id = patient_data$patient_id, switch_time = patient_data$os / 2)
  how2  <- function(patient_data) data.frame(patient_id = patient_data$patient_id)

  tr <- trial(name = 't', n_patients = 60, seed = 3, duration = 200,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 10),
              silent = TRUE)
  tr$add_regimen(regimen(list(what1, what2), list(when1, when2), list(how1, how2)))
  tr$add_arms(sample_ratio = c(1, 1), pbo, trt)

  lst <- listener(silent = TRUE)
  lst$add_milestones(milestone(name = 'final', when = calendarTime(time = 200)))
  controller(tr, lst)$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data('final')
  pbo_d <- d[d$arm == 'placebo', ]

  # every placebo patient must have exactly 3 segments
  expect_true(all(startsWith(pbo_d$regimen_trajectory, 'placebo@0;low dose@')))
  n_segments <- lengths(strsplit(pbo_d$regimen_trajectory, ';', fixed = TRUE))
  expect_true(all(n_segments == 3L))
})

test_that('when() returning rows in shuffled order still assigns correct switch times', {

  skip_if_not_installed('TrialSimulator')

  # when() reverses its output rows — if the framework used row position instead
  # of matching on patient_id, each patient would get the wrong switch time.
  # switch_time is set to patient_id * 0.01 so every patient has a unique,
  # patient_id-dependent value we can verify after the run.
  pfs_e <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/8)
  pbo <- arm(name = 'placebo'); pbo$add_endpoints(pfs_e)
  trt <- arm(name = 'trt');     trt$add_endpoints(pfs_e)

  tr <- trial(name = 't', n_patients = 40, seed = 7, duration = 200,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 20),
              silent = TRUE)
  tr$add_regimen(regimen(
    what = function(patient_data)
      data.frame(patient_id = patient_data$patient_id, new_treatment = 'switched'),
    when = function(patient_data) {
      df <- data.frame(patient_id = patient_data$patient_id,
                       switch_time = patient_data$patient_id * 0.01)
      df[rev(seq_len(nrow(df))), , drop = FALSE]   # deliberately reversed
    },
    how = function(patient_data) data.frame(patient_id = patient_data$patient_id)
  ))
  tr$add_arms(sample_ratio = c(1, 1), pbo, trt)

  lst <- listener(silent = TRUE)
  lst$add_milestones(milestone(name = 'final', when = calendarTime(time = 200)))
  controller(tr, lst)$run(n = 1, silent = TRUE, plot_event = FALSE)

  long <- expandRegimen(tr$get_locked_data('final'))
  switched <- long[long$switch_time_from_enrollment > 0, ]

  # each patient's switch_time must equal their patient_id * 0.01
  expect_equal(switched$switch_time_from_enrollment,
               switched$patient_id * 0.01)
})

test_that('lock_data trims switch entries that fall after the calendar lock time', {

  skip_if_not_installed('TrialSimulator')

  pfs_e <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/5)
  os_e  <- endpoint(name = 'os',  type = 'tte', generator = rexp, rate = log(2)/20)
  pbo   <- arm(name = 'placebo'); pbo$add_endpoints(pfs_e, os_e)

  # switch time is os/2 — often exceeds a short lock window
  what_fn <- function(patient_data) data.frame(patient_id = patient_data$patient_id, new_treatment = 'trt')
  when_fn <- function(patient_data) data.frame(patient_id = patient_data$patient_id, switch_time = patient_data$os / 2)
  how_fn  <- function(patient_data) data.frame(patient_id = patient_data$patient_id)

  lock_time <- 5
  tr <- trial(name = 't', n_patients = 200, seed = 4, duration = lock_time,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 100),
              silent = TRUE)
  tr$add_regimen(regimen(what_fn, when_fn, how_fn))
  tr$add_arms(sample_ratio = 1, pbo)

  lst <- listener(silent = TRUE)
  lst$add_milestones(milestone(name = 'final', when = calendarTime(time = lock_time)))
  controller(tr, lst)$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data('final')

  # any switch entry that survived must satisfy enroll_time + switch_time <= lock_time
  long <- expandRegimen(d)
  switched <- long[long$switch_time_from_enrollment > 0, ]
  if(nrow(switched) > 0){
    enroll_times <- d$enroll_time[match(switched$patient_id, d$patient_id)]
    expect_true(all(enroll_times + switched$switch_time_from_enrollment <= lock_time + 1e-9))
  }

  # with a short lock window, at least some patients should have had their switch trimmed
  expect_true(any(d$regimen_trajectory == 'placebo@0'))
})

## ---- Argument routing -------------------------------------------------------

test_that('regimen() routes ... to the correct function only (no overlap)', {

  what_fn <- function(patient_data, prob) NULL
  when_fn <- function(patient_data, delay = 0) NULL
  how_fn  <- function(patient_data, factor = 1) NULL

  reg <- regimen(what_fn, when_fn, how_fn, prob = 0.3, delay = 5, factor = 1.2)

  expect_equal(reg$get_treatment_allocator_args(1), list(prob = 0.3))
  expect_equal(reg$get_time_selector_args(1),        list(delay = 5))
  expect_equal(reg$get_data_modifier_args(1),        list(factor = 1.2))
})

test_that('regimen() routes a shared arg to all matching functions across slots', {

  what_fn <- function(patient_data, scale) NULL
  when_fn <- function(patient_data) NULL
  how_fn  <- function(patient_data, scale) NULL

  reg <- regimen(what_fn, when_fn, how_fn, scale = 2.0)

  expect_equal(reg$get_treatment_allocator_args(1), list(scale = 2.0))
  expect_equal(reg$get_time_selector_args(1),        list())
  expect_equal(reg$get_data_modifier_args(1),        list(scale = 2.0))
})

test_that('regimen() routes shared arg across different slots (cross-slot)', {

  ## a is required in what, optional-with-default in when
  what_fn <- function(patient_data, a) NULL
  when_fn <- function(patient_data, a = 3) NULL
  how_fn  <- function(patient_data) NULL

  reg <- regimen(what_fn, when_fn, how_fn, a = 'aaaaa')

  expect_equal(reg$get_treatment_allocator_args(1)$a, 'aaaaa')
  expect_equal(reg$get_time_selector_args(1)$a,        'aaaaa')
  expect_equal(reg$get_data_modifier_args(1),          list())
})

## ---- List-of-functions form (mirrors test3.r) --------------------------------

test_that('regimen() routes args correctly across lists of functions', {

  ## Mirrors test3.r:
  ##   what:  f(patient_data, a, b='b')     x2  a required, b optional
  ##   when:  f(patient_data, a=3, b=4)         a and b both optional
  ##          f(patient_data, b=3, c)           b optional, c required
  ##   how:   f(patient_data, c, d=2)       x2  c required, d optional
  ##   dots:  a='aaaaa', c='ccccc'
  ##
  ##   b has defaults everywhere -> not required in dots, not an error
  ##   d has a default           -> not required in dots, not an error

  what1 <- function(patient_data, a, b = 'b') NULL
  what2 <- function(patient_data, a, b = 'b') NULL
  when1 <- function(patient_data, a = 3, b = 4) NULL
  when2 <- function(patient_data, b = 3, c) NULL
  how1  <- function(patient_data, c, d = 2) NULL
  how2  <- function(patient_data, c, d = 2) NULL

  reg <- regimen(
    list(what1, what2),
    list(when1, when2),
    list(how1,  how2),
    a = 'aaaaa', c = 'ccccc'
  )

  expect_equal(reg$get_treatment_allocator_args(1), list(a = 'aaaaa'))
  expect_equal(reg$get_treatment_allocator_args(2), list(a = 'aaaaa'))
  expect_equal(reg$get_time_selector_args(1),        list(a = 'aaaaa'))
  expect_equal(reg$get_time_selector_args(2),        list(c = 'ccccc'))
  expect_equal(reg$get_data_modifier_args(1),        list(c = 'ccccc'))
  expect_equal(reg$get_data_modifier_args(2),        list(c = 'ccccc'))
})

test_that('regimen() does not require args that have defaults everywhere', {

  ## b has defaults in what and when — omitting b from dots is fine
  what_fn <- function(patient_data, a, b = 'default_b') NULL
  when_fn <- function(patient_data, b = 4) NULL
  how_fn  <- function(patient_data) NULL

  expect_no_error(regimen(what_fn, when_fn, how_fn, a = 'x'))
})

## ---- Variable vs constant in ... --------------------------------------------

test_that('regimen() captures variable values eagerly', {

  p <- 0.7
  what_fn <- function(patient_data, prob) NULL
  when_fn <- function(patient_data) NULL
  how_fn  <- function(patient_data) NULL

  reg <- regimen(what_fn, when_fn, how_fn, prob = p)
  p <- 0.1   # mutate after construction

  expect_equal(reg$get_treatment_allocator_args(1)$prob, 0.7)
})

## ---- Function with ... in formals -------------------------------------------

test_that('regimen() passes all dots to a function that declares ...', {

  what_fn <- function(patient_data, ...) NULL
  when_fn <- function(patient_data) NULL
  how_fn  <- function(patient_data) NULL

  reg <- regimen(what_fn, when_fn, how_fn, prob = 0.5, delay = 3)

  expect_equal(reg$get_treatment_allocator_args(1), list(prob = 0.5, delay = 3))
  expect_equal(reg$get_time_selector_args(1),        list())
})

## ---- expandRegimen ----------------------------------------------------------

test_that('expandRegimen errors clearly when required columns are missing', {

  expect_error(expandRegimen(data.frame(x = 1)),
               regexp = 'expandRegimen:.*patient_id.*regimen_trajectory')

  expect_error(expandRegimen(data.frame(patient_id = 1)),
               regexp = 'expandRegimen:.*regimen_trajectory')

  expect_error(expandRegimen(data.frame(regimen_trajectory = 'a@0')),
               regexp = 'expandRegimen:.*patient_id')
})

test_that('expandRegimen returns correct long format for non-switchers', {

  d <- data.frame(patient_id = 1:3,
                  regimen_trajectory = c('placebo@0', 'placebo@0', 'placebo@0'),
                  stringsAsFactors = FALSE)
  out <- expandRegimen(d)

  expect_equal(nrow(out), 3L)
  expect_equal(names(out), c('patient_id', 'regimen', 'switch_time_from_enrollment'))
  expect_true(all(out$regimen == 'placebo'))
  expect_true(all(out$switch_time_from_enrollment == 0))
})

test_that('expandRegimen expands switchers into multiple rows per patient', {

  d <- data.frame(
    patient_id = c(1L, 2L),
    regimen_trajectory = c('placebo@0;low dose@5.5', 'placebo@0'),
    stringsAsFactors = FALSE
  )
  out <- expandRegimen(d)

  expect_equal(nrow(out), 3L)
  expect_equal(out$patient_id,                  c(1L, 1L, 2L))
  expect_equal(out$regimen,                     c('placebo', 'low dose', 'placebo'))
  expect_equal(out$switch_time_from_enrollment, c(0, 5.5, 0))
})

test_that('expandRegimen accepts data.table input without error', {

  skip_if_not_installed('data.table')
  d <- data.table::data.table(
    patient_id = 1:2,
    regimen_trajectory = c('placebo@0;high dose@3', 'trt@0')
  )
  out <- expandRegimen(d)

  expect_equal(nrow(out), 3L)
  expect_equal(names(out), c('patient_id', 'regimen', 'switch_time_from_enrollment'))
})

## ---- Backward compatibility -------------------------------------------------

test_that('regimen() with no ... is backward compatible', {

  what_fn <- function(patient_data) NULL
  when_fn <- function(patient_data) NULL
  how_fn  <- function(patient_data) NULL

  expect_no_error(regimen(what_fn, when_fn, how_fn))

  reg <- regimen(what_fn, when_fn, how_fn)
  expect_equal(reg$get_treatment_allocator_args(1), list())
  expect_equal(reg$get_time_selector_args(1),        list())
  expect_equal(reg$get_data_modifier_args(1),        list())
})

## ---- Error cases ------------------------------------------------------------

test_that('regimen() errors on unnamed positional arg', {

  what_fn <- function(patient_data, prob) NULL
  when_fn <- function(patient_data) NULL
  how_fn  <- function(patient_data) NULL

  expect_error(regimen(what_fn, when_fn, how_fn, 0.5), regexp = 'must be named')
})

test_that('regimen() errors on arg unknown to all functions', {

  what_fn <- function(patient_data) NULL
  when_fn <- function(patient_data) NULL
  how_fn  <- function(patient_data) NULL

  expect_error(regimen(what_fn, when_fn, how_fn, xyz = 1), regexp = 'Unknown argument')
})

test_that('regimen() errors when a required arg is not in ...', {

  what_fn <- function(patient_data, prob) NULL  # prob required, no default
  when_fn <- function(patient_data) NULL
  how_fn  <- function(patient_data) NULL

  expect_error(regimen(what_fn, when_fn, how_fn), regexp = 'Missing required argument')
})

test_that('regimen() errors when a required arg in a list function is not in ...', {

  ## All three must be lists when using the list form
  what1 <- function(patient_data) NULL
  what2 <- function(patient_data) NULL
  when1 <- function(patient_data) NULL
  when2 <- function(patient_data) NULL
  how1  <- function(patient_data, d = 2) NULL  # d optional — fine
  how2  <- function(patient_data, c) NULL       # c required — must be in dots

  expect_error(
    regimen(list(what1, what2), list(when1, when2), list(how1, how2)),
    regexp = 'Missing required argument'
  )
})

## ---- Runtime output validation ----------------------------------------------

# Shared minimal trial factory used by runtime-validation tests.
make_trial_with_regimen <- function(reg){
  pfs_e <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/8)
  pbo   <- arm(name = 'placebo'); pbo$add_endpoints(pfs_e)
  trt   <- arm(name = 'trt');     trt$add_endpoints(pfs_e)
  tr <- trial(name = 't', n_patients = 20, seed = 9, duration = 50,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 20),
              silent = TRUE)
  tr$add_regimen(reg)
  tr$add_arms(sample_ratio = c(1, 1), pbo, trt)
  lst <- listener(silent = TRUE)
  lst$add_milestones(milestone(name = 'final', when = calendarTime(time = 50)))
  controller(tr, lst)
}

test_that('runtime error when what() does not return a data frame', {

  skip_if_not_installed('TrialSimulator')

  reg <- regimen(
    what = function(patient_data) 'not a data frame',
    when = function(patient_data) data.frame(patient_id = patient_data$patient_id, switch_time = 1),
    how  = function(patient_data) data.frame(patient_id = patient_data$patient_id)
  )
  expect_error(make_trial_with_regimen(reg)$run(n = 1, silent = TRUE, plot_event = FALSE),
               regexp = 'what\\(\\)')
})

test_that('runtime error when what() is missing required columns', {

  skip_if_not_installed('TrialSimulator')

  reg <- regimen(
    what = function(patient_data) data.frame(patient_id = patient_data$patient_id),  # new_treatment missing
    when = function(patient_data) data.frame(patient_id = patient_data$patient_id, switch_time = 1),
    how  = function(patient_data) data.frame(patient_id = patient_data$patient_id)
  )
  expect_error(make_trial_with_regimen(reg)$run(n = 1, silent = TRUE, plot_event = FALSE),
               regexp = 'new_treatment')
})

test_that('runtime error when what() returns duplicate patient IDs', {

  skip_if_not_installed('TrialSimulator')

  reg <- regimen(
    what = function(patient_data) data.frame(
      patient_id    = c(patient_data$patient_id, patient_data$patient_id[1]),
      new_treatment = c(rep('trt', nrow(patient_data)), 'trt')
    ),
    when = function(patient_data) data.frame(patient_id = patient_data$patient_id, switch_time = 1),
    how  = function(patient_data) data.frame(patient_id = patient_data$patient_id)
  )
  expect_error(make_trial_with_regimen(reg)$run(n = 1, silent = TRUE, plot_event = FALSE),
               regexp = 'duplicated')
})

test_that('runtime error when when() returns duplicate patient IDs', {

  skip_if_not_installed('TrialSimulator')

  reg <- regimen(
    what = function(patient_data) data.frame(patient_id = patient_data$patient_id, new_treatment = 'trt'),
    when = function(patient_data) data.frame(
      patient_id  = c(patient_data$patient_id, patient_data$patient_id[1]),
      switch_time = 1
    ),
    how  = function(patient_data) data.frame(patient_id = patient_data$patient_id)
  )
  expect_error(make_trial_with_regimen(reg)$run(n = 1, silent = TRUE, plot_event = FALSE),
               regexp = 'duplicated')
})

test_that('runtime error when how() returns duplicate patient IDs', {

  skip_if_not_installed('TrialSimulator')

  reg <- regimen(
    what = function(patient_data) data.frame(patient_id = patient_data$patient_id, new_treatment = 'trt'),
    when = function(patient_data) data.frame(patient_id = patient_data$patient_id, switch_time = 1),
    how  = function(patient_data) data.frame(
      patient_id = c(patient_data$patient_id, patient_data$patient_id[1]),
      os         = 1
    )
  )
  expect_error(make_trial_with_regimen(reg)$run(n = 1, silent = TRUE, plot_event = FALSE),
               regexp = 'duplicated')
})

test_that('runtime error when when() does not return a data frame', {

  skip_if_not_installed('TrialSimulator')

  reg <- regimen(
    what = function(patient_data) data.frame(patient_id = patient_data$patient_id, new_treatment = 'trt'),
    when = function(patient_data) 42,
    how  = function(patient_data) data.frame(patient_id = patient_data$patient_id)
  )
  expect_error(make_trial_with_regimen(reg)$run(n = 1, silent = TRUE, plot_event = FALSE),
               regexp = 'when\\(\\)')
})

test_that('runtime error when when() does not cover all patients returned by what()', {

  skip_if_not_installed('TrialSimulator')

  reg <- regimen(
    what = function(patient_data) data.frame(patient_id = patient_data$patient_id, new_treatment = 'trt'),
    when = function(patient_data) data.frame(patient_id = patient_data$patient_id[1], switch_time = 1),
    how  = function(patient_data) data.frame(patient_id = patient_data$patient_id)
  )
  expect_error(make_trial_with_regimen(reg)$run(n = 1, silent = TRUE, plot_event = FALSE),
               regexp = 'when\\(\\)')
})

test_that('runtime error when how() tries to modify a protected column', {

  skip_if_not_installed('TrialSimulator')

  reg <- regimen(
    what = function(patient_data) data.frame(patient_id = patient_data$patient_id, new_treatment = 'trt'),
    when = function(patient_data) data.frame(patient_id = patient_data$patient_id, switch_time = 1),
    how  = function(patient_data) data.frame(patient_id = patient_data$patient_id, arm = 'new_arm')
  )
  expect_error(make_trial_with_regimen(reg)$run(n = 1, silent = TRUE, plot_event = FALSE),
               regexp = 'must not be modified')
})

## ---- Integration: args actually reach functions during execution -------------

test_that('regimen() ... args are passed to what/when/how at execution time', {

  ## Use <<- to capture the arg value received inside each function.
  ## If do.call does not inject the args, the function call would error
  ## (missing required arg) or received_* would remain NULL.

  received_prob  <- NULL
  received_accel <- NULL

  pfs_e <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/5)
  os_e  <- endpoint(name = 'os',  type = 'tte', generator = rexp, rate = log(2)/14)
  pbo   <- arm(name = 'placebo')
  pbo$add_endpoints(pfs_e, os_e)

  pfs_e2 <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/7)
  os_e2  <- endpoint(name = 'os',  type = 'tte', generator = rexp, rate = log(2)/18)
  trt    <- arm(name = 'trt')
  trt$add_endpoints(pfs_e2, os_e2)

  what_fn <- function(patient_data, prob){
    received_prob <<- prob
    data.frame(
      patient_id    = patient_data$patient_id,
      new_treatment = ifelse(patient_data$arm == 'placebo', 'trt', NA_character_)
    )
  }

  when_fn <- function(patient_data){
    data.frame(patient_id = patient_data$patient_id, switch_time = patient_data$pfs)
  }

  how_fn <- function(patient_data, accel){
    received_accel <<- accel
    data.frame(
      patient_id = patient_data$patient_id,
      os = patient_data$switch_time + accel * pmax(patient_data$os - patient_data$switch_time, 0)
    )
  }

  reg <- regimen(what_fn, when_fn, how_fn, prob = 0.42, accel = 1.3)

  accrual_rate <- data.frame(end_time = Inf, piecewise_rate = 10)
  tr <- trial(
    name = 'test', n_patients = 50, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )
  tr$add_regimen(reg)
  tr$add_arms(sample_ratio = c(1, 1), pbo, trt)

  final <- milestone(name = 'final', when = calendarTime(time = 40))
  lst   <- listener(silent = TRUE)
  lst$add_milestones(final)

  controller(tr, lst)$run(n = 1, silent = TRUE, plot_event = FALSE)

  expect_equal(received_prob,  0.42)
  expect_equal(received_accel, 1.3)
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/fixedDesign/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(dplyr)
library(kableExtra)
library(TrialSimulator)
set.seed(12345)

## ----iioeal-------------------------------------------------------------------
pars_soc <- solveThreeStateModel(median_pfs = 7, median_os = 15, corr = .68, 
                                 h12 = seq(.07, .10, length.out = 50))
pars_soc

## ----eaioie-------------------------------------------------------------------
pars_low <- solveThreeStateModel(median_pfs = 9, median_os = 18.5, corr = .65, 
                                 h12 = seq(.04, .07, length.out = 50))
pars_low

## ----aealga-------------------------------------------------------------------
pars_high <- solveThreeStateModel(median_pfs = 10, median_os = 20, corr = .60, 
                                  h12 = seq(.02, .06, length.out = 50))
pars_high

## ----eaald, echo=FALSE, eval=TRUE---------------------------------------------
rbind(pars_soc, pars_low, pars_high) %>% 
  structure(class = 'data.frame') %>% 
  mutate(arm = c('soc', 'low', 'high')) %>% 
  select(arm, h01, h02, h12) %>% 
  kable(format = 'html', digits = 3, 
        caption = 'Transition hazards in three treatment arms')

## ----ieaof--------------------------------------------------------------------
#' define SoC
pfs_os_in_soc <- endpoint(name = c('pfs', 'os'), 
                          type = c('tte', 'tte'), 
                          generator = CorrelatedPfsAndOs3, 
                          h01 = 0.075, h02 = 0.024, h12 = 0.090)

soc <- arm(name = 'soc')
soc$add_endpoints(pfs_os_in_soc)

#' define low dose arm
pfs_os_in_low <- endpoint(name = c('pfs', 'os'), 
                          type = c('tte', 'tte'), 
                          generator = CorrelatedPfsAndOs3, 
                          h01 = 0.051, h02 = 0.026, h12 = 0.062)

low <- arm(name = 'low')
low$add_endpoints(pfs_os_in_low)

#' define high dose arm
pfs_os_in_high <- endpoint(name = c('pfs', 'os'), 
                           type = c('tte', 'tte'), 
                           generator = CorrelatedPfsAndOs3, 
                          h01 = 0.040, h02 = 0.030, h12 = 0.047)

high <- arm(name = 'high')
high$add_endpoints(pfs_os_in_high)

## ----ioeafaea, results='asis'-------------------------------------------------
high

## ----ioeeaf-------------------------------------------------------------------
accrual_rate <- data.frame(end_time = c(10, Inf),
                           piecewise_rate = c(30, 50))
trial <- trial(
  name = 'Trial-3415', n_patients = 1000,
  seed = 1727811904, duration = 500,
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
  dropout = rexp, rate = -log(1 - 0.1)/18, ## 10% by month 18
  silent = TRUE
)

trial$add_arms(sample_ratio = c(1, 1, 1), soc, low, high) ## 1:1:1
trial

## ----wewfal-------------------------------------------------------------------
action <- function(trial){
  
  locked_data <- trial$get_locked_data('final')
  
  pfs <- fitCoxph((Surv(pfs, pfs_event) ~ arm), placebo = 'soc', 
                  data = locked_data, alternative = 'less', 
                  scale = 'hazard ratio')
  
  os <- fitLogrank((Surv(os, os_event) ~ arm), placebo = 'soc', 
                   data = locked_data, alternative = 'less')
  
  ## Bonferroni test is applied to four hypotheses: 
  ## PFS_low, PFS_high, OS_low, and OS_high
  pfs$decision <- ifelse(pfs$p < .05/4, 'reject', 'accept')
  os$decision <- ifelse(os$p < .05/4, 'reject', 'accept')
  
  trial$save(
    value = pfs %>% filter(arm == 'low') %>% select(estimate, decision, info), 
    name = 'pfs_low')
  
  trial$save(
    value = pfs %>% filter(arm == 'high') %>% select(estimate, decision, info), 
    name = 'pfs_high')
  
  trial$save(
    value = os %>% filter(arm == 'low') %>% select(decision, info), 
    name = 'os_low')
  
  trial$save(
    value = os %>% filter(arm == 'high') %>% select(decision, info), 
    name = 'os_high')
  
}

## ----tuyo---------------------------------------------------------------------
final <- milestone(name = 'final', action = action, 
                   when = eventNumber(endpoint = 'pfs', n = 450, 
                                      arms = c('soc', 'high')) & 
                     eventNumber(endpoint = 'os', n = 550)
                   )

listener <- listener()
listener$add_milestones(final)

controller <- controller(trial, listener)

## ----hgajffal, eval=FALSE, cache=TRUE, message=FALSE, warning=FALSE, results='hide'----
# controller$run(n = 1000, plot_event = FALSE, silent = TRUE)
# output <- controller$get_output()

## ----eaioalafj, echo=FALSE----------------------------------------------------
output <- TrialSimulator:::getFixedDesignOutput()

## ----eeioafla-----------------------------------------------------------------
output %>% 
  head(5) %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")

## ----hghgale------------------------------------------------------------------
output %>% 
  summarise(
    across(matches('_<decision>$'), ~ mean(. == 'reject') * 100, .names = 'Power_{.col}'), 
    across(matches('_<estimate>$'), ~ mean(.x), .names = 'HR_{.col}')
  ) %>%
  rename_with(~ sub('_<decision>$', '', .), starts_with('Power_')) %>%
  rename_with(~ sub('_<estimate>$', '', .), starts_with('HR_')) %>% 
  kable(col.name = NULL, digits = 3, align = 'r') %>% 
  add_header_above(c('Low', 'High', 'Low', 'High', 'Low', 'High'), align = 'r') %>% 
  add_header_above(c('PFS' = 2, 'OS' = 2, 'PFS' = 2)) %>% 
  add_header_above(c('Power (%)' = 4, 'Hazard Ratio' = 2)) %>% 
  kable_styling(full_width = TRUE)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/dynamicTreatmentSwitching/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(TrialSimulator)
library(mvtnorm)
library(kableExtra)

## ----eval=FALSE---------------------------------------------------------------
# treatment_allocator <- function(patient_data){...}
# time_selector <- function(patient_data){...}
# data_modifier <- function(patient_data){...}
# 
# regimen <- regimen(treatment_allocator, time_selector, data_modifier)

## ----eval=FALSE---------------------------------------------------------------
# trial <- trial(...)
# trial$add_regime(regimen)

## ----eval=FALSE---------------------------------------------------------------
# trial <- trial(...)
# trial$add_arms(sample_ratio, soc, low_dose, high_dose)
# trial$add_regime(regimen)

## ----echo=FALSE, error=TRUE---------------------------------------------------
try({
msg <- tryCatch(
  {
    stop(' Member function trial$add_regimen() must be called before trial$add_arms(). ', 
         'A good practice is to call trial$add_regimen() immediately after trial() is executed. ')
    NULL
  },
  error = function(e) {
    cat('Error in trial$add_regimen(regimen) :\n', 
        e$message)
  }
)
})

## ----eval=FALSE---------------------------------------------------------------
# treatment_allocator <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   switch_to <- sample(c('low', 'high', 'stay'), nrow(patient_data),
#                         replace = TRUE, prob = c(.3, .4, .3))
#   ## placebo patients who progressed before death (pfs < os) and were
#   ## assigned a dose are the switchers; the rest are simply not returned
#   is_switcher <- patient_data$arm == 'placebo' &
#     patient_data$pfs < patient_data$os & switch_to != 'stay'
#   sw <- patient_data[is_switcher, ]
#   data.frame(
#     patient_id    = sw$patient_id,
#     new_treatment = ifelse(switch_to[is_switcher] == 'low', 'low dose', 'high dose')
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# time_selector <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## all patient in patient_data progress before die
#     ## thus pfs < os and can switch.
#     ## See treatment_allocator()
#     switch_time = patient_data$pfs
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# time_selector <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## all patient in patient_data progress before die
#     ## thus pfs < os and can switch.
#     ## See treatment_allocator()
#     switch_time = runif(nrow(patient_data), min = patient_data$pfs, max = patient_data$os)
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# data_modifier <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   f <- ifelse(patient_data$new_treatment == 'low dose', 1.1, 1.15)
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## other_endpoint = ...,
#     os = patient_data$switch_time + f * (patient_data$os - patient_data$switch_time)
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# treatment_allocator <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   ## low-dose non-responders switch to high dose
#   sw <- patient_data[patient_data$arm == 'low dose' & patient_data$response == 0, ]
#   data.frame(
#     patient_id    = sw$patient_id,
#     new_treatment = 'high dose'
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# time_selector <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## all patient in patient_data progress before die
#     ## thus pfs < os and can switch
#     switch_time = patient_data$response_readout
#   )
# 
# }

## -----------------------------------------------------------------------------
treatment_allocator <- function(patient_data){
  ## add break point to develop and debug
  # browser()
  ## all placebo patients switch
  sw <- patient_data[patient_data$arm == 'placebo', ]
  data.frame(
    patient_id    = sw$patient_id,
    new_treatment = 'new treatment'
  )

}

## ----eval=FALSE---------------------------------------------------------------
# time_selector <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## all patient in patient_data progress before die
#     ## thus pfs < os and can switch
#     switch_time = ifelse(patient_data$os <= 1, .9 * patient_data$os, patient_data$os - 1)
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# what <- list(allocator1, allocator2, allocator3)
# when <- list(selector1, selector2, selector3)
# how <- list(modifier1, modifier2, modifier3)
# 
# regimen <- regimen(what, when, how)
# 
# trial <- trial(...)
# trial$add_regimen(regimen)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/simulatePfsAndOsGumbel/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(dplyr)
library(knitr)
library(ggplot2)
library(TrialSimulator)


## ----example-endpoint, results = 'asis'---------------------------------------
pfs_and_os <- endpoint(name = c('pfs', 'os'),
                       type = c('tte', 'tte'),
                       generator = CorrelatedPfsAndOs2,
                       median_pfs = 5,
                       median_os = 11,
                       kendall = 0.6,
                       pfs_name = 'pfs',
                       os_name = 'os')

pfs_and_os

## ----direct-generator---------------------------------------------------------
set.seed(123)
dat <- CorrelatedPfsAndOs2(n = 10000,
                           median_pfs = 5,
                           median_os = 11,
                           kendall = 0.6)

head(dat, 2)

## ----validation---------------------------------------------------------------
with(dat, median(pfs))
with(dat, median(os))
with(dat, cor(pfs, os, method = 'kendall'))
with(dat, all(pfs <= os))

#' Class of GroupSequentialTest
#' @description
#' Perform group sequential test for a single endpoint based on sequential
#' one-sided p-values at each stages. Selected alpha spending functions,
#' including user-defined functions, are supported. Boundaries are calculated
#' with `rpact`. At the final analysis, adjustment can be applied for
#' over-running or under-running trial where observed final information is
#' greater or lower than the planned maximum information. See
#' Wassmer & Brannath, 2016, p78f. The test is based on p-values not z
#' statistics because it is easier to not handling direction of alternative
#' hypothesis in current implementation. In addition, only one-sided test is
#' supported which should be sufficient for common use in clinical design.
#'
#' @docType class
#'
#' @examples
#' ## Note: examples showed here replicate the results from
#' ## https://www.rpact.org/vignettes/planning/rpact_boundary_update_example/
#'
#' ## Example 1. Generate boundaries for a pre-fix group sequential design
#' gst <- GroupSequentialTest$new(
#'   alpha = .025, alpha_spending = 'asOF',
#'   planned_max_info = 387)
#'
#' ## without giving p-values, boundaries are returned without actual testing
#' gst$test(observed_info = c(205, 285, 393), is_final = c(FALSE, FALSE, TRUE))
#' gst
#'
#' ## Example 2. Calculate boundaries with observed information at stages
#' ## No p-values are provided
#'
#' ## get an error without resetting an used object
#' try( gst$test(observed_info = 500, is_final = FALSE) )
#'
#' ## reset the object for re-use
#' gst$reset()
#' gst$test(observed_info = c(205, 285, 393), is_final = c(FALSE, FALSE, TRUE))
#' gst
#'
#' ## Example 3. Test stagewise p-values sequentially
#' gst$reset()
#'
#' gst$test(observed_info = 205, is_final = FALSE, p_values = .09)
#' gst$test(285, FALSE, .006)
#'
#' ## print testing trajectory by now
#' gst
#'
#' gst$test(393, TRUE, .002)
#'
#' ## print all testing trajectory
#' gst
#'
#' ## you can also test all stages at once
#' ## the result is the same as calling test() for each of the stages
#' gst$reset()
#' gst$test(c(205, 285, 393), c(FALSE, FALSE, TRUE), c(.09, .006, .002))
#' gst
#'
#' ## Example 4. use user-define alpha spending
#' gst <- GroupSequentialTest$new(
#'   alpha = .025, alpha_spending = 'asUser',
#'   planned_max_info = 387)
#'
#' gst$test(
#'   observed_info = c(205, 285, 393),
#'   is_final = c(FALSE, FALSE, TRUE),
#'   alpha_spent = c(.005, .0125, .025))
#' gst
#'
#' @export
#'
GroupSequentialTest <- R6::R6Class(
  'GroupSequentialTest',

  public = list(

    #' @description
    #' initialize a group sequential test. Now only support one-sided test
    #' based on p-values.
    #' @param alpha familywise error rate
    #' @param alpha_spending alpha spending function. Use \code{"asUser"}
    #' if custom alpha spending schedule is used.
    #' @param planned_max_info integer. Planned maximum number of patients for
    #' non-tte endpoints or number of events for tte endpoints
    #' @param name character. Name of the hypothesis, e.g. endpoint, subgroup,
    #' etc. Optional.
    #' @param silent \code{TRUE} if muting all messages.
    initialize =
      function(alpha = .025,
               alpha_spending = c('asP', 'asOF', 'asUser'),
               planned_max_info,
               name = 'H0',
               silent = TRUE){

        sided <- 1
        stopifnot(is.numeric(alpha) && length(alpha) == 1 &&
                    alpha > 0 && alpha < 1)

        stopifnot(is.wholenumber(planned_max_info))
        stopifnot(length(name) == 1 && is.character(name))

        stopifnot(is.logical(silent) && length(silent) == 1)
        private$silent <- silent
        private$update_max_info <- FALSE
        private$info_fraction <- NULL
        private$observed_info <- NULL
        private$complete <- FALSE
        private$stage <- 1
        private$info_fraction <- NULL
        private$planned_max_info <- planned_max_info
        private$alpha <- alpha
        private$alpha_spending <- match.arg(alpha_spending)
        private$alpha_spent <- 1e-6
        private$always_asUser <- ifelse(private$alpha_spending == 'asUser', TRUE, FALSE)
        private$name <- name
        private$sided <- 1 # sided
        private$original_planned_max_info <- planned_max_info
        private$original_alpha_spending <- match.arg(alpha_spending)
        private$original_silent <- silent

        asf <- c(asOF = "O'Brien & Fleming",
                 asP = 'Pocock',
                 asUser = 'custom')

        if(!private$silent){
          message('A group sequential design with overall alpha = <',
                  private$alpha, '> and alpha spending function <',
                  asf[private$alpha_spending],
                  '> is initialized for the hypothesis <', private$name, '>. \n',
                  'Call $test() to compute boundaries or test the hypothesis. \n',
                  ifelse(private$alpha_spending %in% 'asUser',
                         'You need to specifiy alpha_spent to use "asUser". ',
                         "You don't need to specify alpha_spent. ")
                  )
        }

      },

    #' @description
    #' get name of hypothesis
    get_name = function(){
      private$name
    },

    #' @description
    #' get overall alpha
    get_alpha = function(){
      private$alpha
    },

    #' @description
    #' set alpha spending function. This is useful when set 'asUser' at the
    #' final stage to adjust for an under- or over-running trial.
    #' @param asf character of alpha spending function.
    set_alpha_spending = function(asf){
      stopifnot(asf %in% c('asP', 'asOF', 'asUser'))
      private$alpha_spending <- asf
    },

    #' @description
    #' return character of alpha spending function
    get_alpha_spending = function(){
      private$alpha_spending
    },

    #' @description
    #' return planned maximum information
    get_max_info = function(){
      private$planned_max_info
    },

    #' @description
    #' set planned maximum information. This is used at the final stage
    #' to adjust for an under- or over-running trial.
    #' @param obs_max_info integer. Maximum information, which could be
    #' observed number of patients or events at the final stage.
    set_max_info = function(obs_max_info){
      tmp <- private$planned_max_info
      private$planned_max_info <- obs_max_info
      if(!private$silent){
        message('Maximum information is updated at stage ', self$get_stage(),
                ' (', tmp, ' -> ', obs_max_info, '). ')
      }
    },

    #' @description
    #' get current stage.
    get_stage = function(){
      private$stage
    },

    #' @description
    #' an object of class \code{GroupSequentialTest} is designed to be used
    #' sequentially by calling \code{GroupSequentialTest$test}. When all
    #' planned tests are performed, no further analysis could be done. In that
    #' case keep calling \code{GroupSequentialTest$test} will trigger an error.
    #' To reuse the object for a new set of staged p-values, call this function
    #' to reset the status to stage 1. See examples. This implementation can
    #' prevent the error that more than the planned number of stages are tested.
    reset = function(){

      private$update_max_info <- FALSE
      private$info_fraction <- NULL
      private$observed_info <- NULL
      private$complete <- FALSE
      private$planned_max_info <- private$original_planned_max_info
      private$info_fraction <- NULL
      self$set_alpha_spending(private$original_alpha_spending)
      private$always_asUser <- ifelse(private$original_alpha_spending == 'asUser', TRUE, FALSE)
      private$stage <- 1
      private$alpha_spent <- 1e-6
      private$trajectory <- NULL
      private$silent <- private$original_silent
      message('GroupSequentialTest object <', private$name,
              '> has been reset and is ready to use. ')
    },

    #' @description
    #' save testing result at current stage
    #' @param result a data frame storing testing result at a stage.
    #' @param is_final logical. \code{TRUE} if final test for the hypothesis,
    #' \code{FALSE} otherwise.
    set_trajectory = function(result, is_final = FALSE){
      private$trajectory <- dplyr::bind_rows(private$trajectory, result)
      if(is_final){
        ## information fraction is finalized only at the final stage
        private$trajectory$informationRates <- private$info_fraction[-1]
      }
    },

    #' @description
    #' return testing trajectory until current stage. This function can be
    #' called at any stage. See examples.
    get_trajectory = function(){
      private$trajectory
    },

    #' @description
    #' compute boundaries given current (potentially updated) settings. It
    #' returns different values if settings are changed over time.
    get_stage_level = function(){

      if(!private$always_asUser && self$get_alpha_spending() == 'asUser'){
        private$alpha_spent <- c(private$alpha_spent, self$get_alpha())
      }

      suppressWarnings(
        design <- rpact::getDesignGroupSequential(
          sided = private$sided,
          alpha = private$alpha,
          informationRates = private$info_fraction,
          typeOfDesign = self$get_alpha_spending(),
          userAlphaSpending = private$alpha_spent
        )
      )

      if(private$update_max_info){
        self$set_alpha_spending(private$original_alpha_spending)
      }

      ## will be used in next test if max_info is updated at the final stage
      ## in that case, alpha_spending will be set to 'asUser'
      private$alpha_spent <- design$alphaSpent

      design <- design %>%
        as.data.frame() %>%
        mutate(stages = row_number() - 1) %>%
        dplyr::filter(stages %in% self$get_stage())

      level <- design$stageLevels[1]
      attr(level, 'details') <-
        design %>%
        dplyr::select(typeOfDesign, stages, informationRates, alpha,
                      sided, alphaSpent, criticalValues, stageLevels) %>%
        as.data.frame()
      level

    },

    #' @description
    #' test a hypothesis with the given p-value at current stage
    #' @param p_value numeric. A p-value.
    #' @param is_final logical. \code{TRUE} if this test is carried out for
    #' the final analysis.
    #' @param observed_info integer. Observed information at current stage. It
    #' can be the number of samples (non-tte) or number of events (tte) at test.
    #' If the current stage is final, observed_info will be used to update
    #' planned_max_info, the alpha spending function (\code{typeOfDesign}
    #' in \code{rpact}) will be updated to \code{'asUser'}, and the argument
    #' \code{userAlphaSpending} will be used when calling
    #' \code{rpact::getDesignGroupSequential}.
    #' @param alpha_spent numeric if \code{alpha_spending = "asUser"}. It must
    #' be between 0 and \code{alpha}, the overall alpha of the test.
    #' \code{NA_real_} for other alpha spending functions \code{"asOF"} and
    #' \code{"asP"}.
    test_one = function(p_value, is_final, observed_info, alpha_spent = NA_real_){

      if(missing(p_value)){
        p_value <- NA

        if(!private$silent){
          message('p_value is missing so a dummy test is performed by setting it to NA. ')
        }
      }

      private$observed_info <- c(private$observed_info, observed_info)

      if(is_final){ # at final stage
        if(observed_info != self$get_max_info()){ # under or over running a trial
          self$set_max_info(observed_info)
          self$set_alpha_spending('asUser')
          private$update_max_info <- TRUE
        }
      }else{
        if(observed_info >= self$get_max_info()){
          stop('observed information <',
               observed_info, '> is greater than planned_max_info <',
               self$get_max_info(),
               '>. This can only happened in the final stage. ')
        }
        private$update_max_info <- FALSE
      }

      ## set first information fraction to be 1e-6 for easy implementation
      private$info_fraction <- c(1e-6, private$observed_info / self$get_max_info())
      private$alpha_spent <- sort(unique(c(private$alpha_spent, alpha_spent)))

      stage_level <- self$get_stage_level()
      private$stage <- private$stage + 1

      test_result <-
        attr(stage_level, 'details') %>%
        mutate(obs_p_value = p_value) %>%
        mutate(decision = ifelse(p_value < stage_level, 'reject', 'accept')) %>%
        mutate(hypothesis = private$name)

      self$set_trajectory(test_result, is_final)

      if(is_final){
        private$complete <- TRUE
      }

      test_result

    },

    #' @description
    #' Carry out test based on group sequential design. If \code{p_values}
    #' is \code{NULL}, dummy values will be use and boundaries are calculated
    #' for users to review.
    #' @param observed_info a vector of integers, observed information at stages.
    #' @param is_final logical vector. \code{TRUE} if the test is for the final
    #' analysis.
    #' @param p_values a vector of p-values. If specified, its length should
    #' equal to the length of \code{observed_info}.
    #' @param alpha_spent accumulative alpha spent at observed information.
    #' It is a numeric vector of values between 0 and 1, and of length that
    #' equals \code{length(observed_info)} if alpha-spending
    #' function is \code{"asUser"}. Otherwise \code{NULL}.
    test = function(observed_info, is_final, p_values = NULL, alpha_spent = NULL){

      if(private$complete){
        stop('Group sequential test has been completed. \n',
             'No further test is available. \n',
             'Run GroupSequentialTest$reset() and try again. ')
      }

      if(any(is.na(observed_info))){
        stop('No NA is allowed in observed_info. ')
      }

      if(!all(is.wholenumber(observed_info)) || any(observed_info <= 0)){
        stop('observed_info should be a vector of positive integers. ')
      }

      if(any(is.na(is_final))){
        stop('No NA is allowed in logical vector is_final. ')
      }

      stopifnot(is.logical(is_final))
      if(length(observed_info) != length(is_final)){
        stop('observered_info and is_final must of same length. ')
      }

      if(!is.null(p_values)){
        if(length(p_values) != length(observed_info)){
          stop('p_values and observed_info should be of same length. ')
        }
      }else{
        p_values <- rep(NA, length(observed_info))
        if(!private$silent){
          message('No p-values are provided. Only decision boundaries are calculated. ')
        }
      }

      if(private$always_asUser){
        if(is.null(alpha_spent)){
          stop('alpha_spent cannot be NULL as custom alpha spending function is in use. ')
        }

        if(length(alpha_spent) != length(observed_info)){
          stop('alpha_spent and observed_info should be of same length. ')
        }

        if(any(alpha_spent <= 0 | alpha_spent > self$get_alpha())){
          stop('alpha_spent should be of values between 0 and ', self$get_alpha())
        }

        if(length(alpha_spent) > 1 && any(diff(alpha_spent) <= 0)){
          stop('alpha_spent should be monotonically increasing. ')
        }

        for(i in seq_along(alpha_spent)){
          if(is_final[i] && abs(alpha_spent[i] - self$get_alpha()) > 1e-6){
            stop('At final test, the accumulated alpha_spent should be ',
                 self$get_alpha())
          }
        }

      }else{ ## asOF, asP
        if(!is.null(alpha_spent)){
          stop('alpha_spent should not be specified as alpha spending function is <',
               self$get_alpha_spending(), '>. ')
        }

        alpha_spent <- rep(NA_real_, length(observed_info))
      }



      for(i in seq_along(p_values)){
        self$test_one(p_value = p_values[i],
                      is_final = is_final[i],
                      observed_info = observed_info[i],
                      alpha_spent = alpha_spent[i])
      }
    },

    #' @description
    #' generic function for \code{print}
    print = function(){
      print(self$get_trajectory())
    }


  ),

  private = list(
    sided = 1, ## always one-sided for now
    alpha = NULL, ## overall FWER
    name = 'H0', ## name of hypothesis
    alpha_spending = NULL, ## 'asOF', 'asP', 'asUser'
    planned_max_info = NULL, ## integer to be updated at final stage

    silent = TRUE,

    always_asUser = NULL,
    observed_info = NULL,
    info_fraction = NULL,
    update_max_info = FALSE,

    original_planned_max_info = NULL,
    original_alpha_spending = NULL,
    original_silent = NULL,

    stage = 1, ## integer, current stage, change over time

    alpha_spent = NULL,
    trajectory = NULL,
    complete = FALSE
  )
)

# GroupSequentialTest R6 class
#
# Covers:
#   - constructor and accessors
#   - test() with only observed_info returns critical values / boundaries
#   - test() with p-values populates decisions and stage trajectory
#   - calling test() after completion errors; reset() restores state
#   - input validation (NA info, wrong length, non-whole numbers)
#   - asUser branch: alpha_spent length/monotonicity/final-sum checks

make_gst <- function(alpha = 0.025, spending = "asOF", max_info = 400L,
                     silent = TRUE) {
  GroupSequentialTest$new(alpha = alpha, alpha_spending = spending,
                          planned_max_info = max_info, silent = silent)
}


test_that("GroupSequentialTest accessors reflect initialization", {

  gst <- make_gst(alpha = 0.025, spending = "asOF", max_info = 400)
  expect_equal(gst$get_alpha(), 0.025)
  expect_equal(gst$get_alpha_spending(), "asOF")
  expect_equal(gst$get_max_info(), 400)
  expect_equal(gst$get_stage(), 1)
  expect_equal(gst$get_name(), "H0")
})


test_that("GroupSequentialTest computes boundaries without p-values", {

  skip_if_not_installed("rpact")

  gst <- make_gst(alpha = 0.025, spending = "asOF", max_info = 393)
  gst$test(observed_info = c(205L, 285L, 393L),
           is_final = c(FALSE, FALSE, TRUE))

  tr <- gst$get_trajectory()
  expect_equal(nrow(tr), 3)
  expect_named(tr, c("typeOfDesign", "stages", "informationRates",
                     "alpha", "sided", "alphaSpent", "criticalValues",
                     "stageLevels", "obs_p_value", "decision", "hypothesis"))
  # no p-values -> decision is NA or accept; critical values strictly decreasing
  expect_true(all(diff(tr$criticalValues) < 0))
  expect_true(all(is.na(tr$obs_p_value)))
})


test_that("GroupSequentialTest resets and then accepts staged p-values", {

  gst <- make_gst(alpha = 0.025, spending = "asOF", max_info = 393)
  gst$test(c(205L, 285L, 393L), c(FALSE, FALSE, TRUE))

  # second call errors until reset
  expect_error(gst$test(observed_info = 500L, is_final = FALSE),
               "completed")

  suppressMessages(gst$reset())
  expect_equal(gst$get_stage(), 1)

  gst$test(observed_info = c(205L, 285L, 393L),
           is_final = c(FALSE, FALSE, TRUE),
           p_values = c(0.09, 0.006, 0.002))

  tr <- gst$get_trajectory()
  expect_equal(tr$obs_p_value, c(0.09, 0.006, 0.002))
  expect_true(all(tr$decision %in% c("reject", "accept")))
  # last p-value is small enough to reject under OF at the final stage
  expect_equal(tail(tr$decision, 1), "reject")
})


test_that("GroupSequentialTest validates test() inputs", {

  gst <- make_gst()
  expect_error(gst$test(observed_info = NA_integer_, is_final = FALSE),
               "No NA is allowed in observed_info")

  gst <- make_gst()
  expect_error(gst$test(observed_info = 1.5, is_final = FALSE),
               "positive integers")

  gst <- make_gst()
  expect_error(gst$test(observed_info = c(100L, 200L), is_final = FALSE),
               "same length")

  gst <- make_gst()
  expect_error(gst$test(observed_info = 100L, is_final = NA),
               "No NA is allowed")

  gst <- make_gst(max_info = 100L)
  expect_error(gst$test(observed_info = 150L, is_final = FALSE),
               "greater than planned_max_info")

  gst <- make_gst()
  expect_error(gst$test(observed_info = c(100L, 200L),
                        is_final = c(FALSE, TRUE),
                        p_values = 0.1),
               "same length")
})


test_that("GroupSequentialTest 'asUser' requires consistent alpha_spent", {

  # asUser without alpha_spent -> error
  gst <- make_gst(spending = "asUser", max_info = 100L)
  expect_error(gst$test(observed_info = c(40L, 100L),
                        is_final = c(FALSE, TRUE),
                        p_values = c(.2, .02)),
               "alpha_spent cannot be NULL")

  # asOF with alpha_spent -> error
  gst <- make_gst(spending = "asOF", max_info = 100L)
  expect_error(gst$test(observed_info = c(40L, 100L),
                        is_final = c(FALSE, TRUE),
                        p_values = c(.2, .02),
                        alpha_spent = c(.01, .025)),
               "alpha_spent should not be specified")

  gst <- make_gst(spending = "asUser", max_info = 100L)
  # non-monotone alpha_spent
  expect_error(gst$test(observed_info = c(40L, 100L),
                        is_final = c(FALSE, TRUE),
                        p_values = c(.2, .02),
                        alpha_spent = c(.02, .01)),
               "monotonically increasing")

  gst <- make_gst(spending = "asUser", max_info = 100L)
  # out-of-range alpha_spent
  expect_error(gst$test(observed_info = c(40L, 100L),
                        is_final = c(FALSE, TRUE),
                        p_values = c(.2, .02),
                        alpha_spent = c(.01, .5)),
               "values between 0")

  # wrong length
  gst <- make_gst(spending = "asUser", max_info = 100L)
  expect_error(gst$test(observed_info = c(40L, 100L),
                        is_final = c(FALSE, TRUE),
                        p_values = c(.2, .02),
                        alpha_spent = 0.01),
               "same length")

  # final alpha_spent mismatches alpha
  gst <- make_gst(alpha = 0.025, spending = "asUser", max_info = 100L)
  expect_error(gst$test(observed_info = c(40L, 100L),
                        is_final = c(FALSE, TRUE),
                        p_values = c(.2, .02),
                        alpha_spent = c(.01, .02)),
               "accumulated alpha_spent")
})


test_that("GroupSequentialTest constructor validates alpha and planned_max_info", {
  expect_error(GroupSequentialTest$new(alpha = 1.5,
                                       planned_max_info = 100L))
  expect_error(GroupSequentialTest$new(alpha = 0.025,
                                       planned_max_info = 100.5))
  expect_error(GroupSequentialTest$new(alpha = 0.025,
                                       planned_max_info = 100L,
                                       alpha_spending = "bad"))
})


test_that("GroupSequentialTest asUser succeeds with valid alpha_spent", {

  alpha <- 0.025
  gst <- make_gst(alpha = alpha, spending = "asUser", max_info = 387L)
  gst$test(observed_info = c(205L, 285L, 387L),
           is_final = c(FALSE, FALSE, TRUE),
           alpha_spent = c(.005, .0125, alpha))
  tr <- gst$get_trajectory()
  expect_equal(nrow(tr), 3)
  expect_true(all(tr$alphaSpent >= 0))
})


test_that("GroupSequentialTest can adjust planned_max_info at final stage", {

  gst <- make_gst(alpha = 0.025, spending = "asOF", max_info = 393L)
  # over-running trial: final observed_info exceeds planned
  gst$test(observed_info = c(205L, 285L, 420L),
           is_final = c(FALSE, FALSE, TRUE),
           p_values = c(0.1, 0.01, 0.001))
  expect_equal(gst$get_max_info(), 420L)
  tr <- gst$get_trajectory()
  expect_equal(nrow(tr), 3)
})

CalendarTimeCondition <- R6::R6Class(
  'CalendarTimeCondition',

  inherit = Condition,

  public = list(
    time = NULL,
    arms = NULL,

    initialize = function(time, arms = NULL){

      stopifnot(is.numeric(time))
      stopifnot(length(time) == 1)
      stopifnot(time > 0)

      stopifnot(is.null(arms) || is.character(arms))

      self$time <- time
      self$arms <- arms
    },

    get_trigger_time = function(trial){

      milestone_time <- trial$get_data_lock_time_by_calendar_time(
        calendar_time = self$time)

      milestone_time
    },

    print = function(){
      cat('Calendar time >= ',
          self$time, '\n')

      invisible(self)
    }
  )
)

#' Generate Correlated PFS and OS Using Gumbel Copula
#'
#' @description
#'
#' Generate correlated PFS and OS endpoints using the Gumbel copula. Marginally,
#' both PFS and OS follow exponential distributions. This function can be used
#' as custom \code{generator} in the function \code{endpoint()}.
#'
#' Note that the Gumbel copula is applied to the survival functions of OS and
#' time-to-progression (TTP). PFS is defined as min(TTP, OS), which also
#' follows an exponential distribution.
#'
#' For more information, refer to
#' \href{https://zhangh12.github.io/TrialSimulator/articles/simulatePfsAndOsGumbel.html}{this vignette}.
#'
#' @param n integer. Number of observations.
#' @param median_pfs numeric. Median of PFS.
#' @param median_os numeric. Median of OS.
#' @param kendall numeric. Kendall's tau between observed, uncensored PFS and OS.
#' Must be positive and usually away from zero. Note that
#' this argument is not the Kendall's tau between TTP and OS.
#' @param pfs_name column name of PFS in returned data frame. It must be
#' consistent with name in the function \code{endpoint()}.
#' @param os_name column name of OS in returned data frame. It must be
#' consistent with name in the function \code{endpoint()}.
#'
#' @returns
#' A data frame of \code{n} rows and four columns, including PFS, OS and their
#' event indicators. The event indicators are all 1s. The column names are
#' \code{<pfs_name>}, \code{<pfs_name>_event}, \code{<os_name>},
#' and \code{<os_name>_event}.
#'
#' @export
#'
#' @examples
#' pfs_and_os <- endpoint(name = c('PFS', 'Os'),
#'                        type = c('tte', 'tte'),
#'                        generator = CorrelatedPfsAndOs2,
#'                        median_pfs = 5,
#'                        median_os = 11,
#'                        kendall = .6,
#'                        pfs_name = 'PFS',
#'                        os_name = 'Os')
#'
#' pfs_and_os # run it in console to see summary report
#'
#' ## for validation purpose only
#' ## not the recommended way to use TrialSimulator
#' dat <- pfs_and_os$test_generator(n = 1e4)
#' cor(dat[, 1:2], method = 'kendall') ## close to 0.6
#'
CorrelatedPfsAndOs2 <- function(n, median_pfs, median_os, kendall, pfs_name = 'pfs', os_name = 'os'){

  if(kendall < 0){
    stop("Kendall's tau cannot be negative in CorrelatedPfsAndOs2(). ")
  }

  if(median_os <= 0){
    stop('Median of OS must be positive. ')
  }

  if(median_pfs <= 0){
    stop('Median of PFS must be positive. ')
  }

  f <- function(tau, kendall, median_pfs, median_os){
    kendall - 1 + (1 - tau) * (1 - (median_pfs / median_os)^(1/(1-tau)))
  }

  fit <- try(uniroot(f, c(0, 1),
                     kendall = kendall,
                     median_pfs = median_pfs,
                     median_os = median_os),
             silent = TRUE)

  if('try-error' %in% class(fit)){
    stop("Kendall's tau (", kendall, ') between OS and PFS is too small given the two medians ',
         median_pfs, ' and ', median_os, '. ')
  }

  ## Kendall's tau between TTP and OS
  tau <- fit$root
  theta <- 1 / (1 - tau)

  dsgn <- copula::gumbelCopula(param = theta, dim = 2)
  u <- copula::rCopula(n, dsgn)

  rate_pfs <- log(2) / median_pfs
  rate_os <- log(2) / median_os

  rate_ttp <- (rate_pfs^theta - rate_os^theta)^(1/theta)

  ttp <- -log(u[, 1]) / rate_ttp
  os <- -log(u[, 2]) / rate_os

  pfs <- ifelse(ttp > os, os, ttp)

  ## Kendall's tau between OS and PFS (i.e. min(TTP, OS))
  tau_ <- 1 - 1/theta * (1 - (median_pfs / median_os)^theta)

  data.frame(pfs, os) %>%
    mutate(pfs_event = 1) %>%
    mutate(os_event = 1) %>%
    rename(!!paste0(pfs_name, '_event') := .data$pfs_event) %>%
    rename(!!pfs_name := .data$pfs) %>%
    rename(!!paste0(os_name, '_event') := .data$os_event) %>%
    rename(!!os_name := .data$os)

}

#' Expand regimen trajectory into long format
#'
#' @description
#' Expands the compact \code{regimen_trajectory} column in locked data
#' (returned by \code{trial$get_locked_data()}) into a long-format data frame
#' with one row per regimen segment per patient.
#'
#' The \code{regimen_trajectory} column stores each patient's treatment history
#' as a semicolon-separated string of \code{"name\@time"} entries, e.g.
#' \code{"placebo\@0;low dose\@8.5"}.  \code{expandRegimen} parses this into
#' two additional columns:
#' \itemize{
#'   \item \code{regimen} — name of the treatment regimen
#'   \item \code{switch_time_from_enrollment} — time from enrollment at which
#'         the patient switched to this regimen
#' }
#' The \code{regimen_trajectory} column is dropped from the result.
#'
#' @param data a data frame returned by \code{trial$get_locked_data()}.
#'
#' @return a long-format data frame: one row per regimen segment per patient,
#' with \code{regimen} and \code{switch_time_from_enrollment} appended and
#' \code{regimen_trajectory} removed.
#'
#' @examples
#' \dontrun{
#' locked <- trial$get_locked_data('final')
#' long   <- expandRegimen(locked)
#' }
#'
#' @export
expandRegimen <- function(data){

  required_cols <- c('patient_id', 'regimen_trajectory')
  missing_cols  <- setdiff(required_cols, names(data))
  if(length(missing_cols) > 0){
    stop('expandRegimen: column(s) not found in `data`: ',
         paste(missing_cols, collapse = ', '), '. ',
         '`data` does not look like it was returned by trial$get_locked_data(). ',
         'Also make sure the trial was run with a regimen registered via trial$add_regimen().',
         call. = FALSE)
  }

  if(inherits(data, 'data.table')) data <- as.data.frame(data)

  rows <- lapply(seq_len(nrow(data)), function(i){
    entries  <- strsplit(data$regimen_trajectory[i], ';', fixed = TRUE)[[1]]
    data.frame(
      patient_id                  = data$patient_id[i],
      regimen                     = sub('@.*',  '',  entries),
      switch_time_from_enrollment = as.numeric(sub('.*@', '', entries)),
      stringsAsFactors            = FALSE
    )
  })

  result <- do.call(rbind, rows)
  rownames(result) <- NULL
  result

}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache = TRUE,
  cache.path = 'cache/defineNonTimeToEventEndpoints/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, message = FALSE, echo=FALSE---------------------------------------
library(dplyr)
library(TrialSimulator)
set.seed(12345)

## ----lljalk-------------------------------------------------------------------
## endpoints in placebo arm
tumor_cfb_pbo <- endpoint(name = 'cfb', type = 'non-tte', 
                          readout = c(cfb = 6),
                          generator = rnorm, mean = .8, sd = 3.2)
orr_pbo <- endpoint(name = 'orr', type = 'non-tte', 
                    readout = c(orr = 2),
                    generator = rbinom, size = 1, prob = .1)

## define the placebo arm
pbo <- arm(name = 'placebo')
pbo$add_endpoints(tumor_cfb_pbo, orr_pbo)

## endpoints in treatment arm
tumor_cfb_trt <- endpoint(name = 'cfb', type = 'non-tte', 
                          readout = c(cfb = 6), 
                          generator = rnorm, mean = -2.3, sd = 1.5)
orr_trt <- endpoint(name = 'orr', type = 'non-tte', 
                    readout = c(orr = 2), 
                    generator = rbinom, size = 1, prob = .25)

## define the treatment arm
trt <- arm(name = 'treatment')
trt$add_endpoints(tumor_cfb_trt, orr_trt)

## ----oheahs-------------------------------------------------------------------
dropout_pars <- weibullDropout(c(12, 18), c(.15, .30))
dropout_pars

## ----lihoehf------------------------------------------------------------------
accrual_rate <- data.frame(end_time = c(6, Inf), 
                           piecewise_rate = c(10, 20))

trial <- trial(
  name = 'Trial-31415', description = 'Example Clinical Trial', 
  n_patients = 420, duration = 30, 
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate, 
  dropout = rweibull, scale = 30.636, shape = 1.939
)

## add arms to the trial
trial$add_arms(sample_ratio = c(1, 1), trt, pbo)
trial

## ----llieh--------------------------------------------------------------------
interim <- milestone(name = 'interim', 
                     when = eventNumber(endpoint = 'orr', n = 60), 
                     action = doNothing)

random <- milestone(name = 'random', 
                    when = 
                      calendarTime(time = 10) & 
                      (eventNumber(endpoint = 'cfb', n = 100) | 
                         eventNumber(endpoint = 'orr', n = 180)
                       ), 
                    action = doNothing)

final <- milestone(name = 'final', 
                   when = calendarTime(time = 30), 
                   action = doNothing)

## ----ihhf---------------------------------------------------------------------
## register milestones to the listener
listener <- listener()
listener$add_milestones(interim, random, final)

## run the trial
controller <- controller(trial, listener)
controller$run()

## ----ihegea-------------------------------------------------------------------
interim_data <- trial$get_locked_data(milestone_name = 'interim')
random_data <- trial$get_locked_data(milestone_name = 'random')
final_data <- trial$get_locked_data(milestone_name = 'final')
head(interim_data)

## ----eqfee--------------------------------------------------------------------
not_ready_at_interim <- 
  interim_data %>% 
  dplyr::filter(is.na(cfb) & 
                  is.na(orr) & 
                  enroll_time + 6 < dropout_time) %>% 
  head() %>% 
  print()

random_data %>% 
  dplyr::filter(patient_id %in% not_ready_at_interim$patient_id) %>% 
  print()

#' Generate Constant Variable
#' @description
#' A random number generator returning only a constant. This can be used to
#' set dropout time. Currently it is the default value of dropout time, with
#' \code{value = Inf}.
#'
#' This function can also be used as a generator of \code{endpoint()} if a
#' constant endpoint is needed.
#'
#' @param n integer. Number of observations.
#' @param value scalar. Value of constant observations.
#'
#' @export
rconst <- function(n, value){
  rep(value, n)
}

#' Generate Enrollment Time from Piecewise Constant Uniform Distribution
#' @description
#' It assumes a uniform enrollment with constant rate in each of the time
#' windows. This function can be used as the \code{enroller} when calling
#' \code{trial()} to define a trial.
#'
#' @param n integer. Number of random numbers.
#' @param accrual_rate a data frame of columns
#' \describe{
#'       \item{\code{end_time}}{
#'           End time for a constant rate in a time window. The start time of
#'           the first time window is 0.}
#'       \item{\code{piecewise_rate}}{
#'           A constant rate in a time window. So the number of patients
#'           being recruited in that window is window length x
#'           \code{piecewise_rate}. }
#' }
#' @examples
#' accrual_rate <- data.frame(
#'   end_time = c(12, 13:17, Inf),
#'   piecewise_rate = c(15, 15 + 6 * (1:5), 45)
#' )
#'
#' StaggeredRecruiter(30, accrual_rate)
#'
#' accrual_rate <- data.frame(
#'   end_time = c(3, 4, 5, 8, Inf),
#'   piecewise_rate = c(1, 2, 2, 3, 4)
#' )
#'
#' StaggeredRecruiter(30, accrual_rate)
#' @export
StaggeredRecruiter <- function(n, accrual_rate){

  start <- 0
  enroll_time <- NULL
  for(i in seq_along(accrual_rate$end_time)){
    end <- accrual_rate$end_time[i]
    rate <- accrual_rate$piecewise_rate[i]
    if(is.infinite(end)){
      ## figure out how many more patients are needed
      ## because accrual rate is flat from now on
      n_ <- n - length(enroll_time)
      stopifnot(n_ > 0)
      end <- start + 1/rate * n_
    }

    new_time <- seq(from = start, to = end, by = 1/rate)
    start <- tail(new_time, 1)
    enroll_time <- c(enroll_time, head(new_time, -1))
    rm(new_time)

    if(length(enroll_time) >= n){
      if(!all(diff(enroll_time) > 0)){
        stop('Debug StaggeredRecruiter. ')
      }
      return(head(enroll_time, n))
    }

  }

  stop('No sufficient patients can be recruited from the accrual rate. ',
       'Check your setting or debug StaggeredRecruiter. ')

}

#' Fit logistic regression model
#'
#' @description
#' Fit logistic regression model on an binary endpoint.
#'
#' Refer to \href{https://zhangh12.github.io/TrialSimulator/articles/wrappers.html}{this vignette}
#' for more information and examples.
#'
#' @param formula An object of class \code{formula}. Must include \code{arm}
#' and endpoint in \code{data}. Covariates can be adjusted.
#' @param placebo Character. String indicating the placebo in \code{data$arm}.
#' @param data Data frame. Usually it is a locked data set.
#' @param alternative a character string specifying the alternative hypothesis,
#' must be one of \code{"greater"} or \code{"less"},
#' i.e., one-sided test is enforced. No default value.
#' \code{"greater"} means superiority of treatment over placebo is established
#' by an odds ratio greater than 1.
#' @param scale character. The type of estimate in the output. Must be one
#' of \code{"coefficient"}, \code{"log odds ratio"}, \code{"odds ratio"}, \code{"risk ratio"},
#' or \code{"risk difference"}. No default value.
#' @param ... Subset conditions compatible with \code{dplyr::filter}.
#' \code{glm} will be fitted on this subset only. This argument can be useful
#' to create a subset of data for analysis when a trial consists of more
#' than two arms. By default, it is not specified,
#' all data will be used to fit the model. More than one condition can be
#' specified in \code{...}, e.g.,
#' \code{fitLogistic(remission ~ arm, 'pbo', data, 'greater', 'odds ratio', arm \%in\% c('pbo', 'low dose'), cfb > 0.5)},
#' which is equivalent to:
#' \code{fitLogistic(remission ~ arm, 'pbo', data, 'greater', 'odds ratio', arm \%in\% c('pbo', 'low dose') & cfb > 0.5)}.
#' Note that if more than one treatment arm are present in the data after
#' applying filter in \code{...}, models are fitted for placebo verse
#' each of the treatment arms.
#'
#' @returns a data frame with columns:
#' \describe{
#' \item{\code{arm}}{name of the treatment arm. }
#' \item{\code{placebo}}{name of the placebo arm. }
#' \item{\code{estimate}}{estimate depending on \code{scale}. }
#' \item{\code{p}}{one-sided p-value for log odds ratio (treated vs placebo). }
#' \item{\code{info}}{sample size used in model with \code{NA} being removed. }
#' \item{\code{z}}{z statistics of log odds ratio (treated vs placebo). }
#' }
#'
#' @export
#'
fitLogistic <- function(formula, placebo, data, alternative, scale, ...) {

  if(!inherits(formula, 'formula')){
    stop('formula must be a formula object with "arm" indicating the column arm in data. ')
  }

  if(!is.character(placebo) || length(placebo) != 1){
    stop("placebo must be a single character string")
  }

  if(!is.data.frame(data)){
    stop("data must be a data frame")
  }

  alternative <- match.arg(alternative, choices = c('greater', 'less'))

  valid_scales <- c('coefficient', 'log odds ratio', 'odds ratio', 'risk ratio', 'risk difference')
  if(!is.character(scale) || length(scale) != 1 || !(scale %in% valid_scales)){
    stop('scale must be one of ', paste0(valid_scales, collapse = ', '))
  }

  vars_in_formula <- all.vars(formula)
  missing_vars <- setdiff(vars_in_formula, names(data))
  if(length(missing_vars) > 0){
    stop('The following variable(s) used in formula are missing from data: \n',
         paste0(missing_vars, collapse = ', '))
  }

  if(!'arm' %in% vars_in_formula){
    stop('formula must include main effect term for arm. ')
  }

  # Prepare the data based on condition ...
  filtered_data <- if(...length() == 0){
    data
  }else{
    tryCatch({
      data %>% dplyr::filter(...)
    },
    error = function(e){
      stop('Error in filtering data for fitting a logistic regression model. ',
           'Please check condition in ..., ',
           'which should be compatible with dplyr::filter. ')
    })
  }

  # Check if any data remains after filtering
  if (nrow(filtered_data) == 0) {
    stop("No data remaining after applying subset condition. ")
  }

  treatment_arms <- setdiff(unique(filtered_data$arm), placebo) %>% sort()

  ret <- NULL

  for(trt_arm in treatment_arms){
    sub_data <- filtered_data %>% dplyr::filter(.data$arm %in% c(placebo, trt_arm))

    # Ensure arm is a factor with placebo and treatment
    sub_data$arm <- factor(sub_data$arm, levels = c(placebo, trt_arm))

    # Fit the linear regression model
    # Fit the Cox model
    fit <- tryCatch({
      glm(formula, data = sub_data, family = 'binomial')
    }, error = function(e){
      stop('logistic regression model fitting failed: ', e$message)
    })

    if(scale == 'coefficient'){

      sfit <- summary(fit)

      ## identify coefficient of treatment arm (main effect)
      coef_row <- grep(paste0('^arm', trt_arm, '$'), rownames(sfit$coef))
      if(length(coef_row) != 1){
        stop('fixLogistic: unable to identify arm main effect coefficient for arm <', trt_arm,
             '>. Please use <arm> in formula explicitly. ')
      }

      estimate <- sfit$coef[coef_row, 'Estimate']
      z <- sfit$coef[coef_row, 'z value']

    }else{

      type1 <- c('log odds ratio' = 'logit',
                 'odds ratio' = 'response',
                 'risk ratio' = 'response',
                 'risk difference' = 'response')[scale]

      type2 <- c('log odds ratio' = 'none',
                 'odds ratio' = 'none',
                 'risk ratio' = 'log',
                 'risk difference' = 'response')[scale]

      estimate_index <- c('log odds ratio' = 'estimate',
                          'odds ratio' = 'odds.ratio',
                          'risk ratio' = 'ratio',
                          'risk difference' = 'estimate')[scale]

      res <- fit %>%
        emmeans(~ arm, type = type1) %>%
        regrid(type2) %>%
        contrast(method = list('trt_vs_pbo' = setNames(c(-1, 1), c(placebo, trt_arm)))) %>%
        summary()

      estimate <- res[[estimate_index]]
      z <- res$z.ratio
    }

    p <- ifelse(alternative == 'greater', 1 - pnorm(z), pnorm(z))

    info <- fit$df.residual + fit$rank

    ret <- rbind(ret, data.frame(arm = trt_arm, placebo = placebo,
                                 estimate = estimate,
                                 p = p, info = info, z = z
                                 )
                 )
  }

  rownames(ret) <- NULL
  class(ret) <- c('fit_logistic', class(ret))
  ret
}

#' An Action Function that Does Nothing
#' @description
#' This is an action function that does nothing when the corresponding milestone
#' is triggered. When the listener is monitoring a trial and determining the
#' time to trigger a milestone, data is automatically locked with other necessary
#' data manipulations (censoring, truncation, etc.) are executed.
#' If the users have no intent to modify the
#' trial adaptively at the milestone, e.g., adding (\code{add_arms()}) or
#' removing (\code{remove_arms()}) arm(s),
#' changing sampling ratio(s) (\code{update_sample_ratio()}),
#' modifying trial duration (\code{set_duration()}), carrying out statistical
#' testing, or saving intermediate results (\code{save()}, etc.),
#' then this function
#' can be used to set the argument \code{action} when creating a new milestone.
#' Note that the triggering time and number of observations/events of endpoints
#' at a milestone with \code{action = doNothing} is still recorded in output
#' automatically.
#'
#' @param trial an object returned from \code{trial()}.
#' @param ... (optional) arguments. This is for capturing redundant arguments
#' in \code{milestone()} only.
#'
#' @returns
#' This function returns \code{NULL}. Actually, nothing is done in this function.
#'
#' @export
#'
doNothing <- function(trial, ...){

  dots <- list(...)
  if(length(dots) > 0){
    stop('Argument(s) <',
         paste0(names(dots), collapse = ', '),
         '> should not be specified in milestone() ',
         'if no action function is specified. ')
  }

  invisible(NULL)

}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/simulatePfsAndOsGumbel/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(dplyr)
library(knitr)
library(ggplot2)
library(TrialSimulator)

if (!requireNamespace('copula', quietly = TRUE)) {
  knitr::opts_chunk$set(eval = FALSE)
}

## ----example-endpoint, results = 'asis'---------------------------------------
pfs_and_os <- endpoint(name = c('pfs', 'os'),
                       type = c('tte', 'tte'),
                       generator = CorrelatedPfsAndOs2,
                       median_pfs = 5,
                       median_os = 11,
                       kendall = 0.6,
                       pfs_name = 'pfs',
                       os_name = 'os')

pfs_and_os

## ----direct-generator---------------------------------------------------------
set.seed(123)
dat <- CorrelatedPfsAndOs2(n = 10000,
                           median_pfs = 5,
                           median_os = 11,
                           kendall = 0.6)

head(dat, 2)

## ----validation---------------------------------------------------------------
with(dat, median(pfs))
with(dat, median(os))
with(dat, cor(pfs, os, method = 'kendall'))
with(dat, all(pfs <= os))

#' Define a Milestone
#'
#' @description
#' Define a milestone of a trial. This is a user-friendly wrapper for
#' the class constructor \code{Milestones$new()}. Users who are not familiar with
#' the concept of classes may consider using this wrapper directly.
#'
#' A milestone means the time point to take an action,
#' e.g., carrying out (futility, interim, final) analysis for
#' adding/removing arms, or stopping a trial early.
#' It can also be any more general time point where trial
#' data is used in decision making or adaptation. For example, one can define a
#' milestone for changing randomization scheme, sample size re-assessment, trial
#' duration extension etc.
#'
#' Refer to the
#' \href{https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html}{vignette}
#' to learn how to define milestones when performing simulation using
#' \code{TrialSimulator}.
#'
#' @param name character. Name of milestone.
#' @param when condition to check if this milestone should be
#' triggered. It taks value returned from functions \code{calendarTime()},
#' \code{enrollment()}, \code{eventNumber()} or their logic combinations.
#' @param action function to execute when the milestone triggers.
#' If no action to be executed but simply need to record triggering time and
#' number of events/non-missing observations of endpoints at
#' a milestone, \code{action} can be its default value, a built-in function
#' \code{doNothing}.
#' @param ... (optional) arguments of \code{action}.
#'
#' @examples
#' ## See vignette('conditionSystem')
#'
#'
#' @export
#'
milestone = function(name, when, action = doNothing, ...){

  Milestones$new(
    name = name,
    type = name,
    trigger_condition = when,
    action = action,
    ...
  )

}

#' Class of GraphicalTesting
#' @description
#' Perform graphical testing under group sequential design for one or multiple
#' endpoints. See Maurer & Bretz (2013).
#'
#' @docType class
#'
#' @examples
#'
#' ## Example 1
#' ## dry-run to study the behavior of a graph
#' ## without group sequential design
#' if(interactive()){
#' eps <- .01
#' alpha <- c(.01, .04, 0, 0, 0)
#' transition <- matrix(c(
#'   0, 0, 0, 0, 1,
#'   0, 0, .75, 0, .25,
#'   0, 1/2-eps/2, 0, eps, 1/2-eps/2,
#'   0, 0, 0, 0, 0,
#'   0, 1/2, 1/2, 0, 0
#' ), nrow = 5, byrow = TRUE)
#'
#' ## dummy can be anything, we don't actually use it
#' asf <- rep('asOF', 5)
#' ## dummy can be anything, we don't actually use it
#' max_info <- c(300, 1100, 1100, 1100, 500)
#'
#' hs <- c('H1: UPCR IgA', 'H2: eGFR GN', 'H3: eGFR GN 10wk', 'H5: 2nd Endpoints', 'H4: eGFR IgA')
#'
#' ## initialize an object
#' gt <- GraphicalTesting$new(alpha, transition, asf, max_info, hs)
#' print(gt)
#'
#' ## reject hypotheses based on customized order
#' ## to understand the behavior of a testing strategy
#' ## Any other rejection order is possible
#' gt$reject_a_hypothesis('H1: UPCR IgA')
#' print(gt)
#'
#' gt$reject_a_hypothesis('H2: eGFR GN')
#' print(gt)
#'
#' gt$reject_a_hypothesis('H4: eGFR IgA')
#' print(gt)
#'
#' gt$reject_a_hypothesis('H3: eGFR GN 10wk')
#' print(gt)
#'
#' gt$reset()
#' }
#'
#' ## Example 2
#' ## Example modified from vignettes in gMCPLite:
#' ## Graphical testing for group sequential design
#' if(interactive()){
#' ## initial alpha split to each of the hypotheses
#' alpha <- c(.01, .01, .004, .0, .0005, .0005)
#'
#' ## transition matrix of the initial graph
#' transition <- matrix(c(
#'   0, 1, 0, 0, 0, 0,
#'   0, 0, .5, .5, 0, 0,
#'   0, 0, 0, 1, 0, 0,
#'   0, 0, 0, 0, .5, .5,
#'   0, 0, 0, 0, 0, 1,
#'   .5, .5, 0, 0, 0, 0
#' ), nrow = 6, byrow = TRUE)
#'
#' ## alpha spending functions per hypothesis
#' asf <- c('asUser', 'asOF', 'asUser', 'asOF', 'asOF', 'asOF')
#'
#' ## planned maximum number of events per hypothesis
#' max_info <- c(295, 800, 310, 750, 500, 1100)
#'
#' ## name of hypotheses
#' hs <- c('H1: OS sub',
#'         'H2: OS all',
#'         'H3: PFS sub',
#'         'H4: PFS all',
#'         'H5: ORR sub',
#'         'H6: ORR all')
#'
#' gt <- GraphicalTesting$new(alpha, transition, asf, max_info, hs)
#'
#' ## print initial graph
#' gt
#'
#' ## nominal p-values at each stage
#' ## Note: p-values with same order are calculated with the same locked data
#' ## Note: alpha_spent is only specified for hypotheses using custom alpha
#' ##       spending function "asUser"
#' stats <-
#'   data.frame(
#'     order = c(1:3, 1:3, 1:2, 1:2, 1, 1),
#'     hypotheses = c(rep('H1: OS sub', 3), rep('H2: OS all', 3),
#'                    rep('H3: PFS sub', 2), rep('H4: PFS all', 2),
#'                    'H5: ORR sub', 'H6: ORR all'),
#'     p = c(.03, .0001, .000001, .2, .15, .1, .2, .001, .3, .2, .00001, .1),
#'     info = c(185, 245, 295, 529, 700, 800, 265, 310, 675, 750, 490, 990),
#'     is_final = c(F, F, T, F, F, T, F, T, F, T, T, T),
#'     max_info = c(rep(295, 3), rep(800, 3), rep(310, 2), rep(750, 2), 490, 990),
#'     alpha_spent = c(c(.1, .4, 1), rep(NA, 3), c(.3, 1), rep(NA, 2), NA, NA)
#'   )
#'
#' ## test the p-values from the first analysis, plot the updated graph
#' gt$test(stats %>% dplyr::filter(order==1))
#'
#' ## test the p-values from the second analysis, plot the updated graph
#' gt$test(stats %>% dplyr::filter(order==2))
#'
#' ## test the p-values from the third analysis, plot the updated graph
#' ## because no futher test would be done, displayed results are final
#' gt$test(stats %>% dplyr::filter(order==3))
#'
#'
#' ## plot the final status of the graph
#' print(gt, trajectory = FALSE)
#'
#' ## you can get final testing results as follow
#' gt$get_current_testing_results()
#'
#' ## if you want to see step-by-step details
#' print(gt$get_trajectory())
#'
#' ## equivalently, you can call gt$test(stats) for only once to get same results.
#' gt$reset()
#' gt$test(stats)
#'
#' ## if you only want to get the final testing results
#' gt$get_current_decision()
#' }
#'
#' @export
#'
GraphicalTesting <- R6::R6Class(
  'GraphicalTestingProcedure',

  public = list(

    #' @description
        #' Initialize an object for graphical testing procedure.
        #' Group sequential design is also supported.
        #' @param alpha initial alpha allocated to each of the hypotheses.
        #' @param transition matrix of transition weights. Its diagonals should
        #' be all 0s. The row sums should be 1s (for better power) or
        #' 0s (if no outbound edge from a node).
        #' @param alpha_spending character vector of same length of \code{alpha}.
        #' Currently it supports \code{'asP'}, \code{'asOF'}, and \code{'asUser'}.
        #' @param planned_max_info vector of integers. Maximum numbers of
        #' events (tte endpoints) or patients (non-tte endpoints) at the final
        #' analysis of each hypothesis when planning a trial. The actual numbers
        #' could be different, which can be specified elsewhere.
        #' @param hypotheses vector of characters. Names of hypotheses.
        #' @param silent \code{TRUE} if muting all messages and not generating
        #' plots.
    initialize = function(alpha,
                          transition,
                          alpha_spending,
                          planned_max_info,
                          hypotheses = NULL,
                          silent = FALSE){

      private$validate_arguments(alpha,
                                 transition,
                                 alpha_spending,
                                 planned_max_info,
                                 hypotheses)

      stopifnot(is.logical(silent))
      private$silent <- silent

      if(is.null(hypotheses)){
        hypotheses <- paste0('H', 1:length(alpha))
      }
      private$hypotheses <- hypotheses
      private$hids_in_graph <- 1:length(hypotheses)
      private$transition <- transition
      private$alpha <- alpha

      private$gst <- list()
      for(i in seq_along(private$hypotheses)){
        private$gst[[i]] <-
          list(
            alpha = private$alpha[i],
            alpha_spending = alpha_spending[i],
            info = NULL,
            is_final = NULL,
            p = NULL,
            planned_max_info = planned_max_info[i],
            name = private$hypotheses[i],
            alpha_spent = NULL,
            critical_values = NULL
          )
      }

      if(!private$silent){
        message('A graph is initialized for ', length(private$hypotheses),
                ' hypotheses at FWER = ', sum(private$alpha), '. ')
      }

      private$trajectory <- NULL

      private$original_silent <- private$silent
      private$original_hypotheses <- private$hypotheses
      private$original_hids_in_graph <- private$hids_in_graph
      private$original_transition <- private$transition
      private$original_alpha <- private$alpha
      private$original_gst <- private$gst
      private$original_trajectory <- private$trajectory

    },

    #' @description
    #' reset an object of class \code{GraphicalTesting} to original status
    #' so that it can be reused.
    reset = function(){
      private$silent <- private$original_silent
      private$hypotheses <- private$original_hypotheses
      private$hids_in_graph <- private$original_hids_in_graph
      private$transition <- private$original_transition
      private$alpha <- private$original_alpha
      private$gst <- private$original_gst

      private$trajectory <- private$original_trajectory

      message('GraphicalTesting object has been reset and is ready to use. ')
    },

    #' @description
    #' determine if index of a hypothesis is valid
    #' @param hid an integer
    is_valid_hid = function(hid){
      stopifnot(hid > 0 && hid <= nrow(private$transition))
    },

    #' @description
    #' get name of a hypothesis given its index.
    #' @param hid an integer
    get_hypothesis_name = function(hid){
      self$is_valid_hid(hid)
      private$hypotheses[hid]
    },

    #' @description
    #' return weight between two nodes
    #' @param hid1 an integer
    #' @param hid2 an integer
    get_weight = function(hid1, hid2){
      self$is_valid_hid(hid1)
      self$is_valid_hid(hid2)
      private$transition[hid1, hid2]
    },

    #' @description
    #' update weight between two nodes
    #' @param hid1 an integer
    #' @param hid2 an integer
    #' @param value numeric value to be set as a weight two nodes
    set_weight = function(hid1, hid2, value){
      self$is_valid_hid(hid1)
      self$is_valid_hid(hid2)
      if(hid1 == hid2){
        stopifnot(value == 0)
      }

      if(value < 0 && value > -1e-6){
        value <- .0
        if(!private$silent){
          warning('A slightly less-than-zero weight value is reset to 0. ')
        }
      }

      if(value > 1 && value - 1 < 1e-6){
        if(!private$silent){
          warning('A slightly greater-than-1 weight value is reset to 1. ')
        }
        value <- 1.0
      }

      stopifnot(value >= 0 && value <= 1)
      private$transition[hid1, hid2] <- value
    },

    #' @description
        #' return alpha allocated to a hypothesis when calling this function.
        #' Note that a function can be called several time with the graph is
        #' updated dynamically. Thus, returned alpha can be different even for
        #' the same \code{hid}.
        #' @param hid an integer
    get_alpha = function(hid){
      self$is_valid_hid(hid)
      private$alpha[hid]
    },

    #' @description
    #' update alpha of a hypothesis
    #' @param hid integer. Index of a hypothesis
    #' @param value numeric value to be allocated
    set_alpha = function(hid, value){
      self$is_valid_hid(hid)
      stopifnot(value >= 0 && value <= 1)
      private$alpha[hid] <- value
    },


    #' @description
    #' return all valid \code{hid}
    get_hypotheses_ids = function(){
      1:nrow(private$transition)
    },

    #' @description
        #' return number of hypotheses, including those been rejected.
    get_number_hypotheses = function(){
      nrow(private$transition)
    },

    #' @description
        #' return index of hypotheses that are rejected.
    get_hids_not_in_graph = function(){
      setdiff(self$get_hypotheses_ids(), private$hids_in_graph)
    },

    #' @description
        #' return index of hypotheses with non-zero alphas, thus can be tested
        #' at the current stage.
    get_testable_hypotheses = function(){
      which(private$alpha > 0)
    },

    #' @description
        #' determine whether at least one hypothesis is testable.
        #' If return \code{FALSE}, the testing procedure is completed.
    has_testable_hypotheses = function(){
      length(self$get_testable_hypotheses()) > 0
    },

    #' @description
    #' determine whether a hypothesis is not yet rejected (in graph).
    #' @param hid integer. Index of a hypothesis
    is_in_graph = function(hid){
      hid %in% private$hids_in_graph
    },

    #' @description
    #' determine whether a hypothesis has a non-zero alpha allocated.
    #' @param hid integer. Index of a hypothesis
    is_testable = function(hid){
      hid %in% self$get_testable_hypotheses()
    },

    #' @description
    #' convert hypothesis's name into (unique) index.
    #' @param hypothesis character. Name of a hypothesis. It is different from
    #' \code{hid}, which is an index.
    get_hid = function(hypothesis){
      stopifnot(hypothesis %in% private$hypotheses)
      hid <- which(private$hypotheses %in% hypothesis)
      stopifnot(length(hid) == 1 && !is.na(hid))
      hid
    },

    #' @description
    #' remove a node from graph when a hypothesis is rejected
    #' @param hypothesis name of a hypothesis. It is different from
    #' \code{hid}, which is an index.
    reject_a_hypothesis = function(hypothesis){

      hid <- self$get_hid(hypothesis)

      self$is_valid_hid(hid)
      if(!self$is_in_graph(hid)){
        stop('Hypothesis ', self$get_hypothesis_name(hid),
             ' is not in the graph or already been rejected. ')
      }

      if(!self$is_testable(hid)){
        stop('Hypothesis ', self$get_hypothesis_name(hid),
             ' cannot be rejected because no alpha is allocated to it. ')
      }

      private$hids_in_graph <- setdiff(private$hids_in_graph, hid)

      for(l in private$hids_in_graph){
        alp <- self$get_alpha(l) + self$get_alpha(hid) * self$get_weight(hid, l)
        if(alp < 1e-5 && self$get_weight(hid, l) != 0){
          alp <- 1e-5 ## numeric rounding error existing when computing integral for spent alpha. try to avoid too small allocated alpha
        }
        stopifnot(alp >= self$get_alpha(l))
        if(alp > self$get_alpha(l)){
          if(!private$silent && TRUE){
            message('alpha of hypothesis <', self$get_hypothesis_name(l),
                    '> is updated (',
                    signif(self$get_alpha(l), 2), ' -> ', signif(alp, 2), '). ')
          }
        }
        self$set_alpha(l, alp)
      }

      for(l in self$get_hids_not_in_graph()){
        self$set_alpha(l, .0)
      }

      self$set_alpha(hid, .0) # not necessary

      to_be_zero <- list()
      for(l in self$get_hypotheses_ids()){
        for(m in self$get_hypotheses_ids()){

          if(self$is_in_graph(l) && self$is_in_graph(m) && (l != m) &&
             self$get_weight(l, hid) * self$get_weight(hid, l) < 1){
            wt <- (self$get_weight(l, m) +
                     self$get_weight(l, hid) * self$get_weight(hid, m)) /
              (1 - self$get_weight(l, hid) * self$get_weight(hid, l))

            self$set_weight(l, m, wt)
          }else{
            to_be_zero <- append(to_be_zero, list(list(l = l, m = m)))
            next
          }
        }
      }

      for(lst in to_be_zero){
        self$set_weight(lst$l, lst$m, .0)
      }

      if(!private$silent){
        message('Hypothesis <', hypothesis, '> is rejected. ')
      }

    },

    #' @description
    #' save new testing results at current stage
    #' @param result a data frame of specific columns.
    set_trajectory = function(result){
      private$trajectory <- rbind(private$trajectory, result)
    },

    #' @description
        #' return testing results by the time this function is called.
        #' Note that graphical test is carried out in a sequential manner.
        #' Users may want to review the results anytime. Value returned
        #' by this function can possibly vary over time.
    get_trajectory = function(){
      private$trajectory
    },

    #' @description
        #' test hypotheses using p-values (and other information in \code{stats})
        #' base on the current graph. All rows should have the same order
        #' number.
        #' @param stats a data frame. It must contain the following columns:
        #' \describe{
        #' \item{\code{order}}{integer. P-values (among others) of hypotheses that
        #' can be tested at the same time (e.g., an interim, or final analysis)
        #' should be labeled with the same order number.
        #' If a hypothesis is not tested at a stage,
        #' simply don't put it in \code{stats} with that order number.}
        #' \item{\code{hypotheses}}{character. Name of hypotheses to be tested. They
        #' should be identical to those when calling \code{GraphicalTesting$new}.}
        #' \item{\code{p}}{nominal p-values.}
        #' \item{\code{info}}{observed number of events or samples at test. These will
        #' be used to compute information fractions in group sequential design.}
        #' \item{\code{max_info}}{integers. Maximum information at test. At interim,
        #' \code{max_info} should be equal to \code{planned_max_info} when
        #' calling \code{GraphicalTesting$new}. At the final stage of a
        #' hypothesis, one can update it with observed numbers.}
        #' }
    test_hypotheses = function(stats){

      if(nrow(stats) == 0){
        message('No hypothesis is given to be tested. ')
        return(invisible(NULL))
      }

      if(length(unique(stats$order)) != 1){
        stop('test_hypotheses expect p-values of hypotheses of the same analysis. ',
             'Debug it. ')
      }

      ## for a given set of hypotheses to be tested together, once a hypothesis
      ## is rejected, the remaining hypotheses should be tested again with
      ## updated graph
      tested <- rep(FALSE, length(stats$hypotheses))
      names(tested) <- stats$hypotheses
      test_again <- TRUE
      while(test_again){
        test_again <- FALSE

        ## TRUE if at least one hypothesis has alpha > 0
        at_least_one_testable <- FALSE
        for(h in stats$hypotheses){

          hid <- self$get_hid(h)

          stat <- stats %>% dplyr::filter(hypotheses == self$get_hypothesis_name(hid))
          stopifnot(private$gst[[hid]]$name == stat$hypotheses)

          if(!is.na(stat$max_info)){
            if(stat$max_info != private$gst[[hid]]$planned_max_info){
              if(!private$silent){
                message('Planned maximum information for hypothesis <',
                        private$gst[[hid]]$name,
                        '> is changed from <',
                        private$gst[[hid]]$planned_max_info,
                        '> -> <', stat$max_info, '>. ')
              }
            }
            private$gst[[hid]]$planned_max_info <- stat$max_info
          }

          stopifnot(is.null(private$gst[[hid]]$info) || stat$info >= max(private$gst[[hid]]$info))

          if(!tested[h]){
            tested[h] <- TRUE
            private$gst[[hid]]$info <- c(private$gst[[hid]]$info, stat$info)
            private$gst[[hid]]$is_final <- c(private$gst[[hid]]$is_final, stat$is_final)
            private$gst[[hid]]$p <- c(private$gst[[hid]]$p, stat$p)

            if(private$gst[[hid]]$alpha_spending == 'asUser'){
              if(is.na(stat$alpha_spent)){
                stop('alpha_spent cannot be NA as alpha spending function is <asUser> ',
                     'for hypothesis <', private$gst[[hid]]$name, '>. ')
              }

              if(stat$alpha_spent <=0 || stat$alpha_spent > 1){
                stop('alpha_spent for hypothesis <', private$gst[[hid]]$name,
                     '> is out of range (should be within (0, 1]). ')
              }

              if(stat$is_final && 1 - stat$alpha_spent > 1e-4){
                stop('At final stage, alpha_spent for hypothesis <',
                     private$gst[[hid]]$name,
                     '> should be 1. ')
              }

              if(!is.null(private$gst[[hid]]$alpha_spent) &&
                 stat$alpha_spent <= max(private$gst[[hid]]$alpha_spent)){
                stop('At final stage, alpha_spent for hypothesis <',
                     private$gst[[hid]]$name,
                     '> should be increasing. ')
              }
              private$gst[[hid]]$alpha_spent <- c(private$gst[[hid]]$alpha_spent, stat$alpha_spent)
            }
          }

          stopifnot(private$gst[[hid]]$planned_max_info >= max(private$gst[[hid]]$info))

          current_stage <- length(private$gst[[hid]]$info)

          args <- private$gst[[hid]]

          if(self$get_alpha(hid) == 0){
            if(length(private$gst[[hid]]$critical_values) < length(private$gst[[hid]]$info)){
              private$gst[[hid]]$critical_values <- c(private$gst[[hid]]$critical_values, Inf)
              stopifnot(length(private$gst[[hid]]$critical_values) == length(private$gst[[hid]]$info))
            }
            if(!private$silent){
              message('<', args$name, ', order = ', stat$order,
                      '> cannot be tested with alpha = 0. ')
            }
            next
          }else{
            at_least_one_testable <- TRUE
          }

          if(length(args$is_final) == 1 && tail(args$is_final, 1)){
            ## this hypothesis can only be tested once, no final adjustment is needed
            gst <- GroupSequentialTest$new(alpha = self$get_alpha(hid),
                                           alpha_spending = args$alpha_spending,
                                           planned_max_info = args$planned_max_info,
                                           name = args$name,
                                           silent = private$silent)

            gst$test(observed_info = args$info,
                     is_final = args$is_final,
                     p_values = args$p)
          }else{
            if(length(args$is_final) > 1 && tail(args$is_final, 1)){
              ## this hypothesis can be tested more than once
              ## and it is the final test
              ## change alpha_spending to be 'asUser' to make adjustment
              gst <- GroupSequentialTest$new(alpha = self$get_alpha(hid),
                                             alpha_spending = 'asUser',
                                             planned_max_info = args$planned_max_info,
                                             name = args$name)

              info_frac_ <- args$info/args$planned_max_info

              ##################################################################
              ## a very tricky bug was fixed here
              ## Note that this function test all hypotheses that have their
              ## p-values ready for being tested at the same time (stage)
              ## A hypothesis A can possibly be tested for more than once.
              ## This could happen when it was accepted first, and then
              ## another hypothesis, say B, was rejected with its its alpha
              ## being passed to A. Now A will be tested again, which can
              ## trigger this bug. Fixed. An example to trigger the bug is
              ##   order hypotheses            p info is_final max_info
              ##    1    pfs low 1.000000e+00  188    FALSE      352
              ##    2    pfs low 1.000000e+00  352     TRUE      352
              ##    1   pfs high 1.045526e-03  188    FALSE      352
              ##    2   pfs high 5.721944e-06  352     TRUE      352
              ##    2     os low 1.000000e+00  352    FALSE      430
              ##    3     os low 1.000000e+00  430     TRUE      430
              ##    2    os high 3.211445e-04  352    FALSE      430
              ##    3    os high 1.142131e-05  430     TRUE      430
              ## where the graphical testing object is defined as
              ## alpha <- c(.01/2, .01/2, .015/2, .015/2)
              ## transition <- matrix(1/3, nrow = 4, ncol = 4)
              ## diag(transition) <- 0
              ## asf <- rep('asOF', 4)
              ## max_info <- c(352, 352, 430, 430)
              ## hs <- c('pfs low', 'pfs high', 'os low', 'os high')
              ## gt <- GraphicalTesting$new(alpha, transition, asf, max_info, hs)
              ## gt$test(graph_stats) ## this line will trigger issues

              ## add this assert to prevent issue like this
              stopifnot(length(info_frac_) - length(args$critical_values) <= 1)

              ## add this assert to prevent issue like this
              if(length(info_frac_) == length(args$critical_values)){
                stopifnot(tested[h])
              }

              alpha_spent_ <-
                computeCumulativeAlphaSpent(
                  args$critical_values[1:(length(info_frac_) - 1)], ## this line fixes the bug
                  info_frac_[-length(info_frac_)])

              ##################################################################

              alpha_spent_[alpha_spent_ < min(1e-5, gst$get_alpha())] <- min(1e-5, gst$get_alpha()/2)

              gst$test(observed_info = args$info,
                       is_final = args$is_final,
                       p_values = args$p,
                       alpha_spent = c(alpha_spent_, gst$get_alpha()))
            }else{
              ## this is not the final test
              ## alpha_spending should not be changed,
              ## no adjustment is needed
              gst <- GroupSequentialTest$new(alpha = self$get_alpha(hid),
                                             alpha_spending = args$alpha_spending,
                                             planned_max_info = args$planned_max_info,
                                             name = args$name)

              if(args$alpha_spending %in% 'asUser'){
                ## asUser, so custom alpha_spent * allocated alpha is used
                gst$test(observed_info = args$info,
                         is_final = args$is_final,
                         p_values = args$p,
                         alpha_spent = args$alpha_spent * self$get_alpha(hid))
              }else{
                ## asOF or asP, alpha_spent should not be used
                gst$test(observed_info = args$info,
                         is_final = args$is_final,
                         p_values = args$p)
              }
            }
          }

          gst <- gst$get_trajectory() %>%
            dplyr::filter(stages == current_stage)

          if(length(private$gst[[hid]]$critical_values) == length(private$gst[[hid]]$info)){
            k_ <- length(private$gst[[hid]]$critical_values)
            private$gst[[hid]]$critical_values[k_] <- gst$criticalValues
          }else{
            private$gst[[hid]]$critical_values <- c(private$gst[[hid]]$critical_values, gst$criticalValues)
            stopifnot(length(private$gst[[hid]]$critical_values) == length(private$gst[[hid]]$info))
          }


          decision_ <- ifelse(stat$p < gst$stageLevels, 'reject', 'accept')

          if(!private$silent){
            message('<', args$name, ', order = ', stat$order,
                    '> is ', decision_,
                    'ed (obs = ', signif(stat$p, 2),
                    ', level = ', signif(gst$stageLevels, 2), '). ')
          }

          if(decision_ == 'reject'){
            self$reject_a_hypothesis(self$get_hypothesis_name(hid))

            self$print(graph = !private$silent, trajectory = FALSE)
          }

          gst <- gst %>%
            mutate(obs_p_value = stat$p) %>%
            mutate(decision = decision_) %>%
            mutate(order = stat$order)

          self$set_trajectory(gst)
          private$gst[[hid]]$info_fraction <- NULL

          if(decision_ == 'reject'){
            test_again <- TRUE
            break
          }

        }

        ## Consider to deprecate at_least_one_testable
        ## This can cause an error when all hypotheses are rejected and
        ## thus all nodes have alpha = 0. This shouldn't be an issue at all.
        ## Initially, at_least_one_testable is used to throw an error message
        ## if the initial graph has no node with alpha > 0. Maybe we can
        ## check this when initialize a graph.
        if(!at_least_one_testable){
          # stop('None of the hypotheses has non-zero alpha at this stage. ',
          #     'Check your initial alpha split and transition matrix. ')
        }

        if(!test_again){
          if(!private$silent){
            message('No further hypothesis can be rejected (order = ',
                    stats$order[1], '). ')
          }
        }
      }

    },

    #' @description
    #' test hypotheses using p-values (and other information in \code{stats})
    #' base on the current graph. Users can call this function multiple times.
    #' P-values of the same order should be passed through \code{stats}
    #' together. P-values of multiple orders can be passed together as well.
    #' For example, if users only have p-values at current stage, they can call
    #' this function and update the graph accordingly. In this case, column
    #' \code{order} in \code{stats} is a constant. They can call this
    #' function again when p-values of next stage is available, where
    #' \code{order} is another integer. In simulation, if p-values of all
    #' stages are on hand, users can call this function to
    #' test them all in a single pass. In this case, column \code{order} in
    #' \code{stats} can have different values.
    #' @param stats a data frame. It must contain the following columns:
    #' \describe{
    #' \item{\code{order}}{integer. P-values (among others) of hypotheses that
    #' can be tested at the same time (e.g., an interim, or final analysis)
    #' should be labeled with the same order number.
    #' If a hypothesis is not tested at a stage,
    #' simply don't put it in \code{stats} with that order number.
    #' If all p-values in \code{stats} are tested at the same stage, \code{order}
    #' can be absent.}
    #' \item{\code{hypotheses}}{character. Name of hypotheses to be tested. They
    #' should be identical to those when calling
    #' \code{GraphicalTesting$new}.}
    #' \item{\code{p}}{nominal p-values.}
    #' \item{\code{info}}{observed number of events or samples at test. These will
    #' be used to compute information fractions in group sequential design.}
    #' \item{\code{max_info}}{integers. Maximum information at test. At interim,
    #' \code{max_info} should be equal to \code{planned_max_info} when
    #' calling \code{GraphicalTesting$new}. At the final stage of a
    #' hypothesis, one can update it with observed numbers.}
    #' \item{\code{alpha_spent}}{accumulative proportion of allocated alpha
    #' to be spent if \code{alpha_spending = "asUser"}. Set it to
    #' \code{NA_real_} otherwise. If no hypothesis uses \code{"asUser"} in
    #' \code{stats}, this column could be ignored. }
    #' }
    #'
    #' @return a data frame returned by \code{get_current_testing_results}.
    #' It contains details of each of the testing steps.
    test = function(stats){

      if(!self$has_testable_hypotheses()){
        message('All hypotheses in graph have been tested and completed. ')
        return(invisible(NULL))
      }

      if(!is.data.frame(stats)){
        stop('stats of test should be a data frame. ')
      }

      if(is.null(stats$order)){
        stats$order <- 0
      }

      if(!all(c('hypotheses', 'p', 'info', 'is_final', 'max_info') %in% names(stats))){
        stop('Columns <',
             paste0(setdiff(c('hypotheses', 'p', 'info', 'is_final', 'max_info'),
                            names(stats)),
                    collapse = ', '),
             '> are missing in stats. ')
      }

      if(is.null(stats$alpha_spent)){
        stats$alpha_spent <- NA_real_
      }

      if(!all(is.wholenumber(stats$order))){
        stop('Column <order> in stats should be integers. ')
      }

      self$print(graph = !private$silent, trajectory = FALSE)

      for(ord in sort(unique(stats$order))){
        ## hypotheses to be tested together
        stats_ <- stats %>% dplyr::filter(order == ord)

        self$test_hypotheses(stats_)

      }

      self$get_current_testing_results()

    },

    #' @description
    #' return testing results with details by the time this function
    #' is called. This function can be called by users by multiple
    #' times, thus the returned value varies over time.
    #' This function is called by \code{GraphicalTesting::test},
    #' and returns a data frame consisting of columns
    #' \describe{
    #' \item{\code{hypothesis}}{name of hypotheses.}
    #' \item{\code{obs_p_value}}{observed p-values.}
    #' \item{\code{max_allocated_alpha}}{maximum allocated alpha for the hypothesis.}
    #' \item{\code{decision}}{\code{'reject'} or \code{'accept'} the hypotheses.}
    #' \item{\code{stages}}{stage of a hypothesis. }
    #' \item{\code{order}}{order number that this hypothesis is tested for the last time.
    #' It is different from \code{stages}.}
    #' \item{\code{typeOfDesign}}{name of alpha spending functions.}
    #' }
    get_current_testing_results = function(){

      if(is.null(self$get_trajectory())){
        return(NULL)
      }

      current_results <- self$get_trajectory() %>%
        group_split(hypothesis) %>%
        lapply(
          function(h){
            max_allocated_alpha <- max(h$alpha)
            if('reject' %in% h$decision){
              h <- h %>%
                dplyr::filter(decision %in% 'reject') %>%
                distinct()
              if(nrow(h) > 1){
                stop('A hypothesis has been rejected more than once. ',
                     'Debug it. ')
              }
            }else{
              h <- h %>%
                dplyr::filter(order %in% max(order, na.rm = TRUE)) %>%
                distinct() %>%
                arrange(desc(alpha)) %>%
                head(1)
            }
            h$max_allocated_alpha <- max_allocated_alpha
            return(h)
          }
        ) %>%
        do.call(rbind, .) %>%
        dplyr::select(hypothesis,
                      obs_p_value,
                      max_allocated_alpha,
                      decision,
                      stages,
                      order,
                      typeOfDesign) %>%
        arrange(order, stages, desc(max_allocated_alpha), obs_p_value) %>%
        as.data.frame()


      current_results
    },

    #' @description
    #' get current decisions for all hypotheses. Returned value could
    #' changes over time because it depends on the stages being tested already.
    #' @return a named vector of values \code{"accept"} or \code{"reject"}.
    #' Note that if a hypothesis is not yet tested when calling this function,
    #' the decision for that hypothesis would be \code{"accept"}.
    get_current_decision = function(){

      hypotheses <- private$hypotheses

      current <- self$get_current_testing_results()

      ret <- rep('accept', length(hypotheses))
      names(ret) <- hypotheses

      if(is.null(current)){
        return(ret)
      }

      stopifnot(all(current$hypothesis %in% hypotheses))

      for(h in current$hypothesis){
        ret[h] <- current$decision[current$hypothesis %in% h]
      }

      ret

    },

    #' @description
    #' generic function for \code{print}
    #' @param graph logic. \code{TRUE} if visualizing the current graph,
    #' which can vary over time.
    #' @param trajectory logic. \code{TRUE} if print the current data frame of
    #' trajectory, which can vary over time.
    #' @param ... other arguments supported in \code{gMCPLite::hGraph},
    #' e.g., \code{trhw} and \code{trhh} to control the size of transition box,
    #' and \code{trdigits} to control the digits displayed for transition
    #' weights.
    print = function(graph = TRUE, trajectory = TRUE, ...){

      gplot <-
        gMCPLite::hGraph(nHypotheses = self$get_number_hypotheses(),
                         nameHypotheses = private$hypotheses,
                         alphaHypotheses = private$alpha,
                         m = private$transition,
                         palette = '#56B4E9',
                         trhw = .15,
                         trhh = .1125,
                         trdigits = 3, ...)
      if(graph){
        print(gplot)
      }

      if(trajectory){
        # View(self$get_trajectory())
        print(self$get_trajectory())
      }

    }

  ),

  private = list(
    hypotheses = NULL,
    transition = NULL,
    alpha = NULL,

    hids_in_graph = NULL,
    gst = NULL, ## arguments for defining a group sequential test object,
    trajectory = NULL,

    silent = NULL,

    original_silent = NULL,
    original_hypotheses = NULL,
    original_hids_in_graph = NULL,
    original_transition = NULL,
    original_alpha = NULL,
    original_gst = NULL,
    original_trajectory = NULL,

    validate_arguments = function(alpha,
                                  transition,
                                  alpha_spending,
                                  planned_max_info,
                                  hypotheses){

      stopifnot(is.matrix(transition))
      stopifnot(is.vector(alpha) && is.numeric(alpha))
      stopifnot(nrow(transition) == ncol(transition))
      if(!is.null(hypotheses)){
        stopifnot(nrow(transition) == length(hypotheses))
      }
      stopifnot(nrow(transition) == length(alpha))

      if(any(is.na(transition))){
        stop('NA is not allowed in transition matrix. ')
      }

      if(any(is.na(alpha))){
        stop('NA is not allowed in alpha of hypotheses. ')
      }
      stopifnot(all(transition >= 0))
      stopifnot(all(transition <= 1))
      stopifnot(all(diag(transition) == 0))

      stopifnot(all(alpha >= 0))
      stopifnot(sum(alpha) <= 1)

      stopifnot(all(abs(rowSums(transition) - 1)< 1e-6 |
                      abs(rowSums(transition) - 0)< 1e-6))

      stopifnot(all(alpha_spending %in% c('asP', 'asOF', 'asUser')))
      stopifnot(is.vector(alpha_spending) && is.character(alpha_spending))
      stopifnot(length(alpha_spending) == nrow(transition))

      stopifnot(all(is.wholenumber(planned_max_info)))
      stopifnot(all(planned_max_info > 0))
      stopifnot(length(planned_max_info) == nrow(transition))

    }
  )

)

# GraphicalTesting R6 class
#
# Covers:
#   - constructor validation (matrix shape, row sums, alpha range, lengths)
#   - accessors: get_alpha, get_weight, get_hypothesis_name, get_hid, is_in_graph,
#     has_testable_hypotheses
#   - reject_a_hypothesis propagates alpha and updates the transition matrix
#   - test() with a small 3-hypothesis Bonferroni-like graph produces correct
#     accept/reject decisions
#   - get_current_decision returns a named vector over all hypotheses
#   - reset restores initial state

make_gt <- function(alpha = c(.01, .01, .005),
                    transition = NULL,
                    asf = rep("asOF", 3),
                    max_info = c(200L, 200L, 200L),
                    hs = c("H1", "H2", "H3"),
                    silent = TRUE) {

  if(is.null(transition)){
    transition <- matrix(c(0,  .5, .5,
                           .5, 0,  .5,
                           .5, .5, 0),
                         nrow = 3, byrow = TRUE)
  }

  GraphicalTesting$new(alpha, transition, asf, max_info, hs, silent = silent)
}


test_that("GraphicalTesting initializes and exposes accessors", {

  gt <- make_gt()
  expect_equal(gt$get_number_hypotheses(), 3)
  expect_equal(gt$get_hypothesis_name(1), "H1")
  expect_equal(gt$get_hid("H2"), 2)
  expect_equal(gt$get_alpha(1), 0.01)
  expect_equal(gt$get_weight(1, 2), 0.5)
  expect_true(gt$is_in_graph(1))
  expect_true(gt$has_testable_hypotheses())
  expect_equal(gt$get_hypotheses_ids(), 1:3)
  expect_equal(gt$get_testable_hypotheses(), 1:3)
  expect_equal(gt$get_hids_not_in_graph(), integer(0))
})


test_that("GraphicalTesting validates arguments at construction", {

  good_tx <- matrix(c(0, .5, .5,
                      .5, 0, .5,
                      .5, .5, 0), nrow = 3, byrow = TRUE)

  # non-square transition
  expect_error(GraphicalTesting$new(alpha = c(.01, .01),
                                    transition = matrix(0, 2, 3),
                                    alpha_spending = rep("asOF", 2),
                                    planned_max_info = c(100L, 100L),
                                    silent = TRUE))

  # row sums not 0 or 1
  bad_rows <- good_tx
  bad_rows[1, ] <- c(0, .3, .3)
  expect_error(GraphicalTesting$new(alpha = c(.01, .01, .005),
                                    transition = bad_rows,
                                    alpha_spending = rep("asOF", 3),
                                    planned_max_info = c(100L, 100L, 100L),
                                    silent = TRUE))

  # NA in alpha
  expect_error(GraphicalTesting$new(alpha = c(NA, .01, .005),
                                    transition = good_tx,
                                    alpha_spending = rep("asOF", 3),
                                    planned_max_info = c(100L, 100L, 100L),
                                    silent = TRUE),
               "NA is not allowed in alpha")

  # NA in transition
  bad_tx <- good_tx
  bad_tx[1, 2] <- NA
  expect_error(GraphicalTesting$new(alpha = c(.01, .01, .005),
                                    transition = bad_tx,
                                    alpha_spending = rep("asOF", 3),
                                    planned_max_info = c(100L, 100L, 100L),
                                    silent = TRUE),
               "NA is not allowed in transition")

  # total alpha > 1
  expect_error(GraphicalTesting$new(alpha = c(.5, .5, .5),
                                    transition = good_tx,
                                    alpha_spending = rep("asOF", 3),
                                    planned_max_info = c(100L, 100L, 100L),
                                    silent = TRUE))

  # bad alpha spending function
  expect_error(GraphicalTesting$new(alpha = c(.01, .01, .005),
                                    transition = good_tx,
                                    alpha_spending = c("asOF", "asOF", "bad"),
                                    planned_max_info = c(100L, 100L, 100L),
                                    silent = TRUE))
})


test_that("reject_a_hypothesis transfers alpha along the graph", {

  gt <- make_gt()
  a1 <- gt$get_alpha(1)
  a2 <- gt$get_alpha(2)
  a3 <- gt$get_alpha(3)
  w12 <- gt$get_weight(1, 2)
  w13 <- gt$get_weight(1, 3)

  gt$reject_a_hypothesis("H1")

  # H1 should be out of graph and have alpha 0
  expect_false(gt$is_in_graph(1))
  expect_equal(gt$get_alpha(1), 0)

  # alpha of H2 and H3 should increase by a1 * w12 and a1 * w13, respectively
  expect_equal(gt$get_alpha(2), a2 + a1 * w12, tolerance = 1e-10)
  expect_equal(gt$get_alpha(3), a3 + a1 * w13, tolerance = 1e-10)
})


test_that("reject_a_hypothesis errors if hypothesis already out of graph", {

  gt <- make_gt()
  gt$reject_a_hypothesis("H1")
  expect_error(gt$reject_a_hypothesis("H1"),
               "not in the graph")
})


test_that("set_alpha and set_weight enforce value ranges", {
  gt <- make_gt()
  expect_error(gt$set_alpha(1, -0.1))
  expect_error(gt$set_alpha(1, 1.5))
  expect_error(gt$set_weight(1, 1, 0.5))   # diagonal must be 0
  gt$set_weight(1, 2, 0)
  expect_equal(gt$get_weight(1, 2), 0)
})


test_that("test() runs to completion on a small independent design", {

  # Three totally independent, one-stage hypotheses (no transition needed for
  # independence, but constructor requires row sums of 0 or 1 so give trivial
  # transitions).
  transition <- matrix(c(0,  1,  0,
                         0,  0,  1,
                         1,  0,  0),
                       nrow = 3, byrow = TRUE)
  gt <- make_gt(alpha = c(.01, .01, .005),
                transition = transition)

  stats <- data.frame(
    order      = c(1L, 1L, 1L),
    hypotheses = c("H1", "H2", "H3"),
    p          = c(0.0005, 0.2, 0.4),
    info       = c(200L, 200L, 200L),
    is_final   = c(TRUE, TRUE, TRUE),
    max_info   = c(200L, 200L, 200L),
    alpha_spent = c(NA_real_, NA_real_, NA_real_)
  )

  res <- suppressMessages(gt$test(stats))
  expect_s3_class(res, "data.frame")
  expect_true("hypothesis" %in% names(res))

  decisions <- gt$get_current_decision()
  expect_named(decisions, c("H1", "H2", "H3"))
  expect_equal(decisions["H1"], c(H1 = "reject"))
  expect_equal(decisions["H2"], c(H2 = "accept"))
  expect_equal(decisions["H3"], c(H3 = "accept"))

  # H1 should be out of graph after rejection; H2 inherits its alpha
  expect_false(gt$is_in_graph(1))
  expect_true(gt$is_in_graph(2))
  expect_true(gt$get_alpha(2) > 0.01 - 1e-10)  # at least the original
})


test_that("test() errors if stats lacks required columns", {
  gt <- make_gt()
  expect_error(suppressMessages(gt$test(data.frame(hypotheses = "H1",
                                                   p = 0.01,
                                                   info = 100L))),
               "missing in stats")
})


test_that("reset restores GraphicalTesting to initial state", {

  gt <- make_gt()
  a1_before <- gt$get_alpha(1)
  gt$reject_a_hypothesis("H1")
  expect_equal(gt$get_alpha(1), 0)

  suppressMessages(gt$reset())
  expect_equal(gt$get_alpha(1), a1_before)
  expect_true(gt$is_in_graph(1))
})


test_that("print respects silent=TRUE at construction", {
  gt <- make_gt()
  # silent => graph=FALSE, so no device is opened; just trajectory (NULL)
  pdf(NULL); on.exit(dev.off())
  expect_no_error(gt$print(graph = FALSE, trajectory = FALSE))
})

#' Triggering Condition by Calendar Time
#'
#' @description
#' Define a condition to trigger trial milestone by calendar time. The
#' milestone will be triggered when a trial has been running for at least the
#' specified duration since the first patient is enrolled.
#' It can be used combined with conditions specified by
#' \link[TrialSimulator]{enrollment} and \link[TrialSimulator]{eventNumber}.
#'
#' Refer to the
#' \href{https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html}{vignette}
#' to learn how to define milestones when performing simulation using
#' \code{TrialSimulator}.
#'
#' @param time numeric. Calendar time to trigger a milestone of a trial.
#'
#' @returns an object of class `Condition`
#'
#' @examples
#' milestone(name = 'end of trial', when = calendarTime(time = 12))
#'
#' @export
calendarTime <- function(time){

  CalendarTimeCondition$new(time)

}

#' Generate Correlated PFS and OS Using the Three-States Illness-Death Model
#'
#' @description
#' Generate correlated PFS and OS endpoints using the three-states model. This
#' function can be used as custom \code{generator} in the function
#' \code{endpoint()}.
#'
#' @param n integer. Number of observations.
#' @param h01 constant transition hazard from state "initial" to state "progression".
#' @param h02 constant transition hazard from state "initial" to state "death".
#' @param h12 constant transition hazard from state "progression" to state "death".
#' @param pfs_name column name of PFS in returned data frame. It must be
#' consistent with \code{name} in the function \code{endpoint()}.
#' @param os_name column name of OS in returned data frame. It must be
#' consistent with \code{name} in the function \code{endpoint()}.
#'
#' @returns
#' A data frame of \code{n} rows and four columns, including PFS, OS and their
#' event indicators. The event indicators are all 1s. The column names are
#' \code{<pfs_name>}, \code{<pfs_name>_event}, \code{<os_name>},
#' and \code{<os_name>_event}.
#'
#' @export
#'
#'
#' @examples
#' ## use as function (if you don't use TrialSimulator for simulation)
#' pfs_and_os_trt <- CorrelatedPfsAndOs3(1e4, 0.06, 0.30, 0.30, 'PFS', 'OS')
#' pfs_and_os_pbo <- CorrelatedPfsAndOs3(1e4, 0.10, 0.40, 0.30, 'PFS', 'OS')
#'
#' ## use as generator (if you use TrialSimulator for simulation)
#'
#' pfs_and_os <- endpoint(name = c('PFS', 'os'),
#'                        type = c('tte', 'tte'),
#'                        generator = CorrelatedPfsAndOs3,
#'                        h01 = .06, h02 = .30, h12 = .30,
#'                        pfs_name = 'PFS', os_name = 'os')
#'
#' pfs_and_os # run it in console to see summary report
#'
CorrelatedPfsAndOs3 <- function(n, h01, h02, h12,
                                pfs_name = 'pfs',
                                os_name = 'os'){

  ## assuming constant transition hazards

  ## stable time follows exponential(h01 + h02)
  stable_time <- rexp(n, rate = h01 + h02)

  ## given PFS, time from progression to death follows exponential(h12)
  t <- rexp(n, rate = h12)

  state_at_t <- sample(c('progression', 'death'), size = n,
                       replace = TRUE, prob = c(h01, h02))

  pfs <- stable_time

  os <- ifelse(state_at_t == 'death', stable_time, stable_time + t)

  data.frame(pfs, os) %>%
    mutate(pfs_event = 1) %>%
    mutate(os_event = 1) %>%
    rename(!!paste0(pfs_name, '_event') := .data$pfs_event) %>%
    rename(!!pfs_name := .data$pfs) %>%
    rename(!!paste0(os_name, '_event') := .data$os_event) %>%
    rename(!!os_name := .data$os)

}

# Endpoint data generators and helpers
#
# Covers:
#   - weibullDropout: parameter recovery at the two reference times; input
#     validation branches
#   - PiecewiseConstantExponentialRNG: shape of returned data, event flag at
#     last window, recovery of piecewise hazard via exponential MLE
#   - CorrelatedPfsAndOs3: PFS <= OS pointwise; sample correlation roughly at
#     expected level; column renaming through (pfs_name, os_name)
#   - CorrelatedPfsAndOs4: monotone ordering response <= progression <= death,
#     censoring at `duration`, rejection of malformed transition matrices
#   - solveThreeStateModel: output class, columns, and rough recovery of
#     (h01, h02) when true hazards generated the reference data

test_that("weibullDropout recovers parameters at reference time points", {

  set.seed(1)
  for(i in 1:5){
    t <- sort(runif(2, 1, 30))
    r <- sort(runif(2, 0.05, 0.5))   # increasing rates over time
    ps <- weibullDropout(time = t, dropout_rate = r)

    expect_named(ps, c("shape", "scale"))
    expect_equal(1 - exp(-(t[1] / ps["scale"])^ps["shape"]),
                 r[1], tolerance = 1e-4, ignore_attr = TRUE)
    expect_equal(1 - exp(-(t[2] / ps["scale"])^ps["shape"]),
                 r[2], tolerance = 1e-4, ignore_attr = TRUE)
  }
})

test_that("weibullDropout input validation", {

  expect_error(weibullDropout(time = 1, dropout_rate = c(.1, .2)),
               "time should be of length 2")
  expect_error(weibullDropout(time = c(12, 18), dropout_rate = .2),
               "dropout_rate should be of length 2")
  expect_error(weibullDropout(time = c(-1, 18), dropout_rate = c(.1, .2)),
               "time should be positive")
  expect_error(weibullDropout(time = c(12, 18), dropout_rate = c(0, .2)),
               "dropout_rate should be between 0 and 1")
  expect_error(weibullDropout(time = c(12, 18), dropout_rate = c(.2, 1)),
               "dropout_rate should be between 0 and 1")
})

test_that("PiecewiseConstantExponentialRNG returns expected columns and events", {

  risk <- data.frame(
    end_time = c(1, 4.33, 26.0, 52.0),
    piecewise_risk = c(1, 1.01, 0.381, 0.150) * exp(-4.01)
  )

  set.seed(5)
  d <- PiecewiseConstantExponentialRNG(1000, risk, 'PFS')
  expect_named(d, c('PFS', 'PFS_event'))
  expect_equal(nrow(d), 1000)
  expect_true(all(d$PFS_event %in% c(0, 1)))
  expect_true(all(d$PFS <= max(risk$end_time) + 1e-8))
  # PFS_event==0 <=> PFS was censored at last end_time
  expect_true(all(d$PFS[d$PFS_event == 0] == max(risk$end_time)))
})

test_that("PiecewiseConstantExponentialRNG approximates a constant-hazard exponential", {

  skip_on_cran()

  risk <- data.frame(end_time = c(1000), piecewise_risk = 0.1)

  set.seed(101)
  d <- PiecewiseConstantExponentialRNG(5000, risk, 'ep')

  # with one window and rate 0.1, mean ~ 1/0.1 = 10 for uncensored samples
  # observed events should dominate since end_time is very large
  expect_true(mean(d$ep_event) > 0.99)
  expect_equal(mean(d$ep[d$ep_event == 1]), 10, tolerance = 0.5)
})

test_that("PiecewiseConstantExponentialRNG honors hazard_ratio column", {

  risk1 <- data.frame(end_time = 500,
                      piecewise_risk = 0.05,
                      hazard_ratio = 2)
  risk2 <- data.frame(end_time = 500,
                      piecewise_risk = 0.10)

  set.seed(77)
  d1 <- PiecewiseConstantExponentialRNG(4000, risk1, 'ep')
  set.seed(77)
  d2 <- PiecewiseConstantExponentialRNG(4000, risk2, 'ep')
  expect_equal(d1$ep, d2$ep, tolerance = 1e-10)
})

test_that("PiecewiseConstantExponentialRNG rejects bad risk tables", {

  expect_error(PiecewiseConstantExponentialRNG(10, list(end_time = 1), 'ep'),
               regexp = NULL)
  expect_error(PiecewiseConstantExponentialRNG(10,
                                               data.frame(end_time = 1), 'ep'),
               "Missed column")
  expect_error(PiecewiseConstantExponentialRNG(10,
                                               data.frame(piecewise_risk = .1),
                                               'ep'),
               "Missed column")
})

test_that("CorrelatedPfsAndOs3 produces PFS <= OS and renames columns", {

  skip_on_cran()

  set.seed(2024)
  d <- CorrelatedPfsAndOs3(5000, h01 = 0.10, h02 = 0.05, h12 = 0.12,
                           pfs_name = 'PFS', os_name = 'OS')
  expect_named(d, c('PFS', 'OS', 'PFS_event', 'OS_event'),
               ignore.order = TRUE)
  expect_true(all(d$PFS <= d$OS + 1e-10))
  expect_true(all(d$PFS_event == 1))
  expect_true(all(d$OS_event == 1))

  # median of PFS follows exp(h01 + h02) = 0.15 => median = log(2)/0.15 ~ 4.62
  expect_equal(median(d$PFS), log(2) / 0.15, tolerance = 0.3)

  # rough correlation in expected range (0.5-0.8 for these hazards)
  rho <- cor(d$PFS, d$OS)
  expect_gt(rho, 0.3)
  expect_lt(rho, 0.95)
})

test_that("CorrelatedPfsAndOs4 rejects bad transition matrices", {

  ok <- matrix(c(0.99, 0.0035, 0.0055, 0.0010,
                 0,    0.9900, 0.0052, 0.0048,
                 0,    0,      0.9960, 0.0040,
                 0,    0,      0,      1),
               nrow = 4, byrow = TRUE)

  expect_error(CorrelatedPfsAndOs4(10, matrix(0, 3, 3), duration = 10),
               "4x4 matrix")
  bad_neg <- ok
  bad_neg[1, 2] <- -0.1
  expect_error(CorrelatedPfsAndOs4(10, bad_neg, 10), "non-negative")

  bad_rows <- ok
  bad_rows[1, 1] <- 0.5
  expect_error(CorrelatedPfsAndOs4(10, bad_rows, 10), "sum to 1")

  bad_lower <- ok
  bad_lower[2, 1] <- 0.05
  bad_lower[2, 2] <- bad_lower[2, 2] - 0.05   # keep row sum = 1
  expect_error(CorrelatedPfsAndOs4(10, bad_lower, 10), "lower triangle")
})

test_that("CorrelatedPfsAndOs4 produces ordered states with censoring at duration", {

  m <- matrix(c(0.99, 0.0035, 0.0055, 0.0010,
                0,    0.9900, 0.0052, 0.0048,
                0,    0,      0.9960, 0.0040,
                0,    0,      0,      1),
              nrow = 4, byrow = TRUE)

  set.seed(42)
  d <- CorrelatedPfsAndOs4(500, m, duration = 200,
                           death_name = 'os',
                           progression_name = 'pfs',
                           response_name = 'or')

  expect_named(d,
               c('or', 'pfs', 'os', 'or_event', 'pfs_event', 'os_event'),
               ignore.order = TRUE)
  expect_equal(nrow(d), 500)

  # state times must be <= duration
  expect_true(all(d$or <= 200))
  expect_true(all(d$pfs <= 200))
  expect_true(all(d$os <= 200))

  # ordering when all three events happened
  full <- d[d$or_event == 1 & d$pfs_event == 1 & d$os_event == 1, ]
  if(nrow(full) > 0){
    expect_true(all(full$or <= full$pfs))
    expect_true(all(full$pfs <= full$os))
  }

  # censored rows should sit at duration
  expect_true(all(d$pfs[d$pfs_event == 0] == 200))
  expect_true(all(d$os[d$os_event == 0] == 200))
})

test_that("solveThreeStateModel returns an object with expected shape", {

  set.seed(9)
  ret <- solveThreeStateModel(median_pfs = 4.6, median_os = 9.6,
                              corr = c(0.5, 0.6, 0.7),
                              h12 = seq(.08, .2, length.out = 10))

  expect_s3_class(ret, "three_state_model")
  expect_named(ret, c("corr", "h01", "h02", "h12", "error"))
  expect_equal(nrow(ret), 3)
  expect_true(all(ret$h01 > 0))
  expect_true(all(ret$h02 > 0))
  expect_true(all(ret$h12 > 0))
  expect_true(all(ret$error >= 0))
  # error is |target - achieved|, should be reasonably small after grid search
  expect_true(all(ret$error < 0.1))
})

test_that("solveThreeStateModel validates inputs", {
  expect_error(solveThreeStateModel(median_pfs = c(1, 2),
                                    median_os = 9,
                                    corr = .5))
  expect_error(solveThreeStateModel(median_pfs = 4,
                                    median_os = 9,
                                    corr = 0))
  expect_error(solveThreeStateModel(median_pfs = 4,
                                    median_os = 9,
                                    corr = 1.1))
  expect_error(solveThreeStateModel(median_pfs = 4,
                                    median_os = 9,
                                    corr = .5,
                                    h12 = c(-.1, .2)))
})

#' Compute Constant Rates of Piecewise Exponential Distribution
#'
#' @description
#' This function computes the rate parameters \code{lambda} in each of the time
#' windows. \code{lambda} are natural parameters of piecewise exponential
#' distribution, but in practice, users may define the distribution by
#' specifying the survival probabilities at time points where event rates
#' change.
#'
#' This function returns a data frame, which can be used as input of the
#' argument \code{risk} of the data generator \code{PiecewiseConstantExponentialRNG}.
#'
#' @param surv_prob numeric. A vector of survival probabilities at \code{times}.
#' @param times numeric. A vector of time points where event rates change.
#' \code{times} and \code{surv_prob} must be of equal length.
#'
#' @returns
#' a data frame of two columns
#' \describe{
#' \item{\code{end_time}}{End time for a constant event rate. The start time of
#' the first time window is 0. }
#' \item{piecewise_risk}{A constant event rate in the time window ending with
#' \code{end_time} on the same row. }
#' }
#' @export
#'
#' @examples
#' solvePiecewiseConstantExponentialDistribution(
#'   surv_prob = c(.9, .75, .64, .42, .28),
#'   times = c(.4, 1.2, 4, 5.5, 9)
#' )
#'
solvePiecewiseConstantExponentialDistribution <- function(surv_prob, times){

  if(length(times) != length(surv_prob)){
    stop('times and surv_prob must be of the same length. ')
  }

  if(any(surv_prob <= 0) || any(surv_prob > 1)){
    stop('surv_prob must be within (0, 1]. ')
  }

  if(any(times < 0)){
    stop('times must be non-negative. ')
  }

  surv_prob <- surv_prob[order(times, decreasing = FALSE)]
  times <- sort(times, decreasing = FALSE)

  if(any(diff(surv_prob) >= 0)){
    stop('surv_prob must be increasing over time')
  }

  if(times[1] != 0){
    times <- c(0, times)
    surv_prob <- c(1.0, surv_prob)
  }

  lambda <- -diff(log(surv_prob))/diff(times)
  end_time <- times[-1]
  data.frame(end_time, piecewise_risk = lambda)

}

#' Default Action Function for Dry Run of a Trial
#
#' No meaningful action is taken when the argument \code{action} of
#' \code{milestone()} is set to be \code{default_action}. When milestones
#' are triggered, milestone time and number of observations/events are recorded.
#' but no custom information is saved.
#'
#' @keywords internal
#' @noRd
.default_action <- function(){

  action <- 'This is a dry run to determine milestone time and number of endpoint events. '
  action

}

# Internal helpers
#
# Covers:
#   - computeCumulativeAlphaSpent: alpha spent is monotone non-decreasing, lies
#     in (0, 1], and matches rpact reference values for Pocock/O'Brien-Fleming
#   - .default_action: returns a non-empty character describing the dry run
#   - createSticker: runs without error and returns NULL invisibly
#   - getAdaptiveDesignOutput / getFixedDesignOutput: return the internal
#     precomputed data frames with the expected shape

test_that("computeCumulativeAlphaSpent is monotone and bounded", {

  info <- c(0.25, 0.5, 0.75, 1.0)

  # O'Brien-Fleming-like decreasing boundaries (large then small critical values)
  cv <- c(4.33, 2.96, 2.36, 2.01)
  a <- TrialSimulator:::computeCumulativeAlphaSpent(cv, info)

  expect_length(a, length(cv))
  expect_true(all(diff(a) >= -1e-10))   # non-decreasing
  expect_true(all(a > 0))
  expect_true(all(a < 1))
})

test_that("computeCumulativeAlphaSpent matches rpact for an OBF design", {

  skip_if_not_installed("rpact")

  info <- c(1/3, 2/3, 1)
  design <- rpact::getDesignGroupSequential(kMax = length(info),
                                            typeOfDesign = "OF",
                                            alpha = 0.025,
                                            sided = 1,
                                            informationRates = info)

  cv <- as.numeric(design$criticalValues)
  target <- as.numeric(design$alphaSpent)

  got <- TrialSimulator:::computeCumulativeAlphaSpent(cv, info)
  expect_equal(as.numeric(got), target, tolerance = 1e-3)
  # final cumulative alpha is alpha (tolerance from pmvnorm quadrature)
  expect_equal(tail(as.numeric(got), 1), 0.025, tolerance = 1e-3)
})

test_that(".default_action returns a descriptive message", {
  msg <- TrialSimulator:::.default_action()
  expect_type(msg, "character")
  expect_length(msg, 1)
  expect_match(msg, "dry run", ignore.case = TRUE)
})

test_that("createSticker runs without error and returns NULL invisibly", {
  expect_null(TrialSimulator:::createSticker())
})

test_that("getAdaptiveDesignOutput and getFixedDesignOutput return data frames", {
  ad <- TrialSimulator:::getAdaptiveDesignOutput()
  expect_s3_class(ad, "data.frame")
  expect_gt(nrow(ad), 0)

  fd <- TrialSimulator:::getFixedDesignOutput()
  expect_s3_class(fd, "data.frame")
  expect_gt(nrow(fd), 0)
})

#' Solve Parameters in a Three-State Illness-death Model
#'
#' @description
#' The illness-death model consists of three states, \code{initial}, \code{progression},
#' and \code{death}. It can be used to model the progression-free survival (PFS)
#' and overall survival (OS) in clinical trial simulation. It models the
#' correlation between PFS and OS without assumptions on latent status and copula.
#' Also, it does not assume PFS and OS satisfy the proportional hazard assumption
#' simultaneously. The three-state illness-death model ensures a nice property that
#' PFS <= OS with probability one. However, it requires three hazard parameters
#' under the homogeneous Markov assumption. In practice, hazard parameters are
#' hard to specify intuitively especially when no trial data is available at
#' the planning stage.
#'
#' This function reparametrizes the illness-death model in term of three parameters,
#' i.e. median of PFS, median of OS, and correlation between PFS and OS. The
#' output of this function, which consists of the three hazard parameters, can
#' be used to generate PFS and OS with desired property. It can be used with
#' the built-in data generator \code{CorrelatedPfsAndOs3()} when defining
#' endpoints in \code{TrialSimulator}.
#'
#' For more information, refer to
#' \href{https://zhangh12.github.io/TrialSimulator/articles/simulatePfsAndOsIdm.html}{this vignette}.
#'
#' @param median_pfs numeric. Median of PFS.
#' @param median_os numeric. Median of OS.
#' @param corr numeric vector. Pearson correlation coefficients between PFS and OS.
#' @param h12 numeric vector. A set of hazard from progression to
#' death that may induce the target correlation \code{corr} given \code{median_pfs}
#' and \code{median_os}. \code{solveThreeStateModel()} will do a grid search to
#' find the best hazard parameters that matches to the medians of PFS and OS,
#' and their correlations.
#'
#' @returns a data frame with columns:
#' \describe{
#' \item{\code{corr}}{target Peason's correlation coefficients. }
#' \item{\code{h01}}{hazard from stable to progression. }
#' \item{\code{h02}}{hazard from stable to death.  }
#' \item{\code{h12}}{hazard from progression to death. }
#' \item{\code{error}}{absolute error between target correlation and correlation
#' derived from \code{h01}, \code{h02}, and \code{h12}. }
#' }
#'
#' @export
#'
#' @examples
#'
#' dat <- CorrelatedPfsAndOs3(1e6, h01 = .1, h02 = .05, h12 = .12)
#'
#' cor(dat$pfs, dat$os) ## 0.65
#'
#' median(dat$pfs) ## 4.62
#'
#' median(dat$os) ## 9.61
#'
#' ## find h01, h02, h12 that can match to median_pfs, median_os and corr
#' ## should be close to h01 = 0.10, h02 = 0.05, h12 = 0.12 when corr = 0.65
#' ret <- solveThreeStateModel(median_pfs = 4.6, median_os = 9.6,
#'                             corr = seq(.5, .7, length.out=5))
#' ret
#'
solveThreeStateModel <- function(median_pfs, median_os, corr,
                                 h12 = seq(.05, .2, length.out = 50)){


  stopifnot(is.numeric(median_pfs) && length(median_pfs) == 1)
  stopifnot(is.numeric(median_os) && length(median_os) == 1)
  stopifnot(is.numeric(corr))
  stopifnot(all(corr > 0) && all(corr < 1))
  stopifnot(all(h12 > 0))

  .compute_h01_given_h12 <- function(h12, median_pfs, median_os){

    all_cause_hazard <- log(2)/median_pfs
    (1/2 - exp(-all_cause_hazard * median_os)) * (h12 - all_cause_hazard) /
      (exp(-all_cause_hazard * median_os) - exp(-h12 * median_os))

  }

  h01 <- .compute_h01_given_h12(h12, median_pfs, median_os)
  h02 <- log(2)/median_pfs - h01
  corr_ <- NULL
  h12_ <- NULL
  h01_ <- NULL
  h02_ <- NULL
  for(idx in seq_along(h12)){
    if(h01[idx] < 0 || h02[idx] < 0){
      next
    }

    dat <- CorrelatedPfsAndOs3(1e5, h01 = h01[idx], h02 = h02[idx], h12 = h12[idx])
    corr_ <- c(corr_, cor(dat$pfs, dat$os))
    h12_ <- c(h12_, h12[idx])
    h01_ <- c(h01_, h01[idx])
    h02_ <- c(h02_, h02[idx])
  }

  .find_h12_given_corr <- function(res, corr){

    stopifnot(all(corr > 0) && all(corr < 1))

    h12 <- NULL
    h01 <- NULL
    h02 <- NULL
    error <- NULL
    for(corr_ in corr){
      idx <- which.min(abs(res$corr - corr_))[1]
      h12 <- c(h12, res$h12[idx])
      h01 <- c(h01, res$h01[idx])
      h02 <- c(h02, res$h02[idx])
      error <- c(error, abs(res$corr - corr_)[idx])
    }

    ret <- data.frame(corr, h01, h02, h12, error)

    attr(ret, 'data') <- res

    ret

  }

  ret <- .find_h12_given_corr(data.frame(h01 = h01_, h02 = h02_, h12 = h12_, corr = corr_), corr)

  class(ret) <- c('three_state_model', class(ret))
  ret

}

#' Plot result of three-state illness-death model
#'
#' @param x an object returned by \code{solveThreeStateModel()}.
#' @param ... currently not supported.
#'
#' @export
plot.three_state_model <- function(x, ...){

  data_points <- attr(x, 'data')
  data_lines <- x %>%
    mutate(label = sprintf("(%.2f, %.4f)", .data$corr, .data$h12))

  suppressWarnings(
  p <- ggplot(data_points,
              aes(x = .data$corr, y = .data$h12)) +
    geom_point() +

    geom_segment(data_lines,
                 mapping =
                   aes(x = .data$corr, xend = .data$corr,
                       y = 0, yend = .data$h12),
                 linetype = 'dashed',
                 color = 'blue', inherit.aes = FALSE) +

    geom_segment(data_lines,
                 mapping =
                   aes(x = 0, xend = .data$corr,
                       y = .data$h12, yend = .data$h12),
                 linetype = 'dashed',
                 color = 'blue', inherit.aes = FALSE) +

    geom_text(data_lines,
              mapping =
                aes(x = .data$corr, y = .data$h12, label = .data$label),
              vjust = 2, hjust = -.2, inherit.aes = FALSE) +

    xlim(min(data_points$corr)*.9, max(data_points$corr)*1.1) +
    xlab("Pearson Correlation") +
    ylab(expression(h[12]))
  )

  print.data.frame(x)

  print(p)

  invisible(NULL)
}

#' Generate Time-to-Event Endpoint from Piecewise Constant Exponential Distribution
#'
#' @description
#' This function can be used as generator to define endpoint.
#' Implementation is based on
#' \href{https://www.demogr.mpg.de/papers/technicalreports/tr-2010-003.pdf}{this algorithm}.
#' This distribution can be used to simulate delayed treatment effect.
#'
#' @param n integer. Number of random numbers
#' @param risk a data frame of columns
#' \describe{
#' \item{\code{end_time}}{ End time for a constant risk in a time window.
#' The start time of the first time window is 0.}
#' \item{\code{piecewise_risk}}{ A constant risk in a time window, which is
#' absolute risk * relative risk, or (h0 * g) in the link.}
#' \item{\code{hazard_ratio}}{ An optional column for simulating an active arm.
#' If absent, a column of 1s will be added. Equivalently, user can multiply
#' \code{piecewise_risk} by \code{hazard_ratio} manually and ignore this column.}
#' }
#' @param endpoint_name character. Name of endpoint. This should be the same as
#' the \code{name} argument when calling function \code{endpoint()}.
#'
#' @return a data frame of \code{n} rows and two columns
#' \describe{
#' \item{\code{<endpoint_name>}}{name of endpoint specified by users in
#' \code{endpoint_name}. }
#' \item{<endpoint_name>_event}{event indicator with 0/1 as censoring and
#' event, respectively. Note that due to the nature of the algorithm to
#' generate data from this distribution, it is possible to have the endpoint
#' being censoring at the last \code{end_time} unless it is set to \code{Inf}.}
#' }
#'
#' @examples
#' # example code
#' # In this example, absolute risk in [0, 1) and [26, 52] are 0.0181 and
#' # 0.0027, respectively.
#' risk <- data.frame(
#'   end_time = c(1, 4.33, 26.0, 52.0),
#'   piecewise_risk = c(1, 1.01, 0.381, 0.150) * exp(-4.01)
#' )
#' PiecewiseConstantExponentialRNG(10, risk, 'PFS')
#'
#' @export
PiecewiseConstantExponentialRNG <- function(n, risk, endpoint_name){

  stopifnot(is.data.frame(risk))

  required_columns <- c('end_time', 'piecewise_risk')
  missed_columns <- setdiff(required_columns, names(risk))
  if(length(missed_columns) > 0){
    stop('Missed column(s) in risk table: <', paste0(missed_columns, collapse = ', '), '>. ')
  }

  if('hazard_ratio' %in% names(risk)){
    risk <- risk %>%
      mutate(piecewise_risk = .data$piecewise_risk * .data$hazard_ratio) %>%
      dplyr::select(-.data$hazard_ratio)
  }

  n_windows <- nrow(risk)
  tau <- c(0, risk$end_time)
  delta_tau <- tau[-1] - tau[-length(tau)]
  h0_g <- risk$piecewise_risk

  m <- matrix(0, nrow = n_windows + 1, ncol = n_windows)
  m[lower.tri(m)] <- 1

  logS <- log(1 - runif(n))

  tmp <- -as.vector(m %*% (h0_g * delta_tau))

  tte <- rep(NA, n)
  for(i in 1:n_windows){
    tte <- ifelse(
      tmp[i] >= logS & logS > tmp[i + 1],
      tau[i] + (tmp[i] - logS) / h0_g[i],
      tte
    )
  }

  data.frame(tte = tte) %>%
    mutate(event = ifelse(is.na(tte), 0, 1)) %>%
    mutate(tte = ifelse(is.na(tte), tau[length(tau)], tte)) %>%
    rename(!!paste0(endpoint_name, '_event') := .data$event) %>%
    rename(!!endpoint_name := tte)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/adaptiveDesign/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(dplyr)
library(R6)
library(kableExtra)
library(rlang)
library(survival)
library(survminer)
library(ggplot2)
library(TrialSimulator)

## ----daoif, class.source='fold-show'------------------------------------------
weibullDropout(time = c(12, 18), dropout_rate = c(.08, .18))

## ----ljgai, class.source="fold-show"------------------------------------------
#' define three arms
pbo <- arm(name = 'placebo')
low <- arm(name = 'low dose')
high <- arm(name = 'high dose')

#' define endpoints in placebo
pfs <- endpoint(name = 'pfs', type = 'tte',
                generator = rexp, rate = log(2) / 5)

os <- endpoint(name = 'os', type = 'tte',
               generator = rexp, rate = log(2) / 14)

five_weeks <- 5 / 52 * 12 ## convert it in months
surrogate <- endpoint(name = 'surrogate', type = 'non-tte',
                      readout = c(surrogate = five_weeks),
                      generator = rbinom, size = 1, prob = .05)
pbo$add_endpoints(pfs, os, surrogate)

#' define endpoints in low dose arm
pfs <- endpoint(name = 'pfs', type = 'tte',
                generator = rexp, rate = log(2) / 6.7)

os <- endpoint(name = 'os', type = 'tte',
               generator = rexp, rate = log(2) / 17.5)

surrogate <- endpoint(name = 'surrogate', type = 'non-tte',
                      readout = c(surrogate = five_weeks),
                      generator = rbinom, size = 1, prob = .12)
low$add_endpoints(pfs, os, surrogate)

#' define endpoints in high dose arm
pfs <- endpoint(name = 'pfs', type = 'tte',
                generator = rexp, rate = log(2) / 7.1)

os <- endpoint(name = 'os', type = 'tte',
               generator = rexp, rate = log(2) / 18.2)

surrogate <- endpoint(name = 'surrogate', type = 'non-tte',
                      readout = c(surrogate = five_weeks),
                      generator = rbinom, size = 1, prob = .13)
high$add_endpoints(pfs, os, surrogate)

## ----lagieg, class.source="fold-show"-----------------------------------------
accrual_rate <- data.frame(end_time = c(10, Inf),
                           piecewise_rate = c(30, 50))
trial <- trial(
  name = 'Trial-3415', n_patients = 1000,
  seed = 1727811904, duration = 40,
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
  dropout = rweibull, shape = 2.139, scale = 38.343
)

trial$add_arms(sample_ratio = c(1, 1, 1), low, high, pbo)

## ----ieaong, class.source='fold-show'-----------------------------------------
action1 <- function(trial){
  
  locked_data <- trial$get_locked_data('dose selection')
  
  fit <- fitFarringtonManning(endpoint = 'surrogate', placebo = 'placebo',
                              data = locked_data, alternative = 'greater')
  
  # browser() ## if you want to see what does fit look like
  z_l <- fit$z[fit$arm == 'low dose']
  z_h <- fit$z[fit$arm == 'high dose']
  if(z_l > 1.28){
    trial$remove_arms('high dose')
    trial$save(value = 'low', name = 'kept_arm')
  }else if(z_h > 1.28){
    trial$remove_arms('low dose')
    trial$save(value = 'high', name = 'kept_arm')
  }else{
    trial$save(value = 'both', name = 'kept_arm')
  }

}

## ----aotel, class.source="fold-show"------------------------------------------
action2 <- function(trial){
  
  locked_data <- trial$get_locked_data('interim')

  fit <- fitLogrank(Surv(pfs, pfs_event) ~ arm, placebo = 'placebo', 
                    data = locked_data, alternative = 'less')

  ## futility analysis
  if(max(fit$z) < .5){
    trial$save(value = 'negative', name = 'futility')
    
    ## extend duration
    ## trial$set_duration(45)
  }else{
    trial$save(value = 'positive', name = 'futility')
  }

}

## ----alkdae, class.source="fold-show"-----------------------------------------
action3 <- function(trial){
  
  locked_data <- trial$get_locked_data('final')

  ## test PFS
  dt_pfs <- trial$dunnettTest(Surv(pfs, pfs_event) ~ arm, placebo = 'placebo',
                              treatments = c('high dose', 'low dose'),
                              milestones = c('dose selection', 'interim', 'final'),
                              alternative = 'less',
                              planned_info = 'default')
  
  ct_pfs <- trial$closedTest(dt_pfs, treatments = c('high dose', 'low dose'),
                             milestones = c('interim', 'final'),
                             alpha = .005, alpha_spending = 'asOF')

  ## test OS
  dt_os <- trial$dunnettTest(Surv(os, os_event) ~ arm, placebo = 'placebo',
                             treatments = c('high dose', 'low dose'),
                             milestones = c('dose selection', 'final'),
                             alternative = 'less', 
                             planned_info = 'default')
  
  ct_os <- trial$closedTest(dt_os, treatments = c('high dose', 'low dose'),
                            milestones = c('final'),
                            alpha = .02, alpha_spending = 'asOF')
  
  ## we only save testing decision here
  ## You can save whatever you want for summarizing things later, e.g. reject time
  trial$save(value = ct_pfs$decision[ct_pfs$arm == 'high dose'], 
             name = 'pfs_high_dose_decision')
  
  trial$save(value = ct_pfs$decision[ct_pfs$arm == 'low dose'], 
             name = 'pfs_low_dose_decision')
  
  trial$save(value = ct_os$decision[ct_os$arm == 'high dose'], 
             name = 'os_high_dose_decision')
  
  trial$save(value = ct_os$decision[ct_os$arm == 'low dose'], 
             name = 'os_low_dose_decision')
}

## -----------------------------------------------------------------------------
dose_selection <- milestone(name = 'dose selection', action = action1,   
                            when = eventNumber(endpoint = 'surrogate', n = 300)
                            )

interim <- milestone(name = 'interim', action = action2, 
                     when = eventNumber(endpoint = 'pfs', n = 300)
                     )

final <- milestone(name = 'final', action = action3, 
                   when = enrollment(n = 1000, arms = c('placebo', 'low dose', 'high dose')) & 
                     eventNumber(endpoint = 'os', n = 300) & (
                       calendarTime(time = 28) | 
                         eventNumber(endpoint = 'pfs', n = 520)
                       )
                   )

listener <- listener()
#' register milestones with listener
listener$add_milestones(
  dose_selection,
  interim,
  final
)

## ----eiaaf, dpi = 1200--------------------------------------------------------
controller <- controller(trial, listener)
controller$run(plot_event = TRUE)

## ----alojfqoitl, class.source="fold-show"-------------------------------------
controller$get_output() %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")

## ----aief, class.source='fold-show'-------------------------------------------
## test PFS
dt_pfs <- trial$dunnettTest(Surv(pfs, pfs_event) ~ arm, placebo = 'placebo',
                            treatments = c('high dose', 'low dose'),
                            milestones = c('dose selection', 'interim', 'final'),
                            alternative = 'less', 
                            planned_info = 'default')
ct_pfs <- trial$closedTest(dt_pfs, treatments = c('high dose', 'low dose'),
                           milestones = c('interim', 'final'),
                           alpha = .005, alpha_spending = 'asOF')

## test OS
dt_os <- trial$dunnettTest(Surv(os, os_event) ~ arm, placebo = 'placebo',
                           treatments = c('high dose', 'low dose'),
                           milestones = c('dose selection', 'final'),
                           alternative = 'less', 
                           planned_info = 'default')
ct_os <- trial$closedTest(dt_os, treatments = c('high dose', 'low dose'),
                          milestones = c('final'),
                          alpha = .02, alpha_spending = 'asOF')

print(ct_pfs)
print(ct_os)

## ----ioeinaf, eval=FALSE, cache=TRUE, message=FALSE, warning=FALSE, results='hide'----
# ## reset a controller if $run has been executed before
# controller$reset()
# controller$run(n = 1000, plot_event = FALSE, silent = TRUE)
# output <- controller$get_output()

## ----eaiofj, echo=FALSE-------------------------------------------------------
output <- TrialSimulator:::getAdaptiveDesignOutput()

## ----liefa--------------------------------------------------------------------
output %>% 
  head(5) %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")

## ----iojalf, class.source = "fold-hide"---------------------------------------
output %>% 
  summarise(
    time_dose_selection = mean(`milestone_time_<dose selection>`), 
    time_interim = mean(`milestone_time_<interim>`), 
    time_final = mean(`milestone_time_<final>`), 
    n_dose_selection = mean(`n_events_<dose selection>_<patient_id>`), 
    n_interim = mean(`n_events_<interim>_<patient_id>`), 
    n_final = mean(`n_events_<final>_<patient_id>`), 
    low = mean(kept_arm == 'low') * 100, 
    high = mean(kept_arm == 'high') * 100, 
    both = mean(kept_arm == 'both') * 100
  ) %>% 
  kable(col.names = NULL, digits = 1, align = 'r', 
        caption = 'Number of Randomized Patients at Stages') %>% 
  add_header_above(c(rep(c('Dose Selection', 'Interim', 'Final'), 2), 
                     'Low Dose', 'High Dose', 'Both'), align = 'r') %>% 
  add_header_above(c('Time' = 3, 'Number of Patients' = 3, 'Selected Dose (%)' = 3)) %>% 
  kable_styling(full_width = TRUE)

## ----adalf, class.source = "fold-hide"----------------------------------------
output %>% 
  summarise(
    n_pfs_dose_selection = mean(`n_events_<dose selection>_<pfs>`), 
    n_pfs_interim = mean(`n_events_<interim>_<pfs>`),
    n_pfs_final = mean(`n_events_<final>_<pfs>`),
    n_os_dose_selection = mean(`n_events_<dose selection>_<os>`), 
    n_os_interim = mean(`n_events_<interim>_<os>`),
    n_os_final = mean(`n_events_<final>_<os>`)
  ) %>% 
  kable(col.names = NULL, digits = 1, align = 'r', 
        caption = 'Number of Events of PFS and OS at Stages') %>% 
  add_header_above(rep(c('Dose Selection', 'Interim', 'Final'), 2), align = 'r') %>% 
  add_header_above(c('PFS' = 3, 'OS' = 3)) %>% 
  kable_styling(full_width = TRUE)

## ----dgslja, class.source = "fold-hide"---------------------------------------
output %>% 
  summarise(
    power_pfs_low = mean(pfs_low_dose_decision == 'reject') * 100, 
    power_pfs_high = mean(pfs_high_dose_decision == 'reject') * 100, 
    power_pfs_or = mean(pfs_low_dose_decision == 'reject' | pfs_high_dose_decision == 'reject') * 100, 
    power_pfs_and = mean(pfs_low_dose_decision == 'reject' & pfs_high_dose_decision == 'reject') * 100, 
    power_os_low = mean(os_low_dose_decision == 'reject') * 100, 
    power_os_high = mean(os_high_dose_decision == 'reject') * 100, 
    power_os_or = mean(os_low_dose_decision == 'reject' | os_high_dose_decision == 'reject') * 100,
    power_os_and = mean(os_low_dose_decision == 'reject' & os_high_dose_decision == 'reject') * 100
  ) %>% 
  kable(col.names = NULL, digits = 1, align = 'r', 
        caption = 'Power of Testing PFS and OS') %>% 
  add_header_above(rep(c('Low Dose', 'High Dose', 'Low or High', 'Low and High'), 2), align = 'r') %>% 
  add_header_above(c('PFS (%)' = 4, 'OS (%)' = 4)) %>% 
  kable_styling(full_width = TRUE)

## ----eioajf, class.source = "fold-hide"---------------------------------------
output %>% 
  summarise(
    power_pfs_not_os = mean((pfs_low_dose_decision == 'reject' | pfs_high_dose_decision == 'reject') & 
                              os_low_dose_decision == 'accept' & os_high_dose_decision == 'accept') * 100, 
    power_os_not_pfs = mean((os_low_dose_decision == 'reject' | os_high_dose_decision == 'reject') & 
                              pfs_low_dose_decision == 'accept' & pfs_high_dose_decision == 'accept') * 100, 
    power_pfs_and_os = mean((pfs_low_dose_decision == 'reject' | pfs_high_dose_decision == 'reject') & 
                              (os_low_dose_decision == 'reject' | os_high_dose_decision == 'reject')) * 100,
    power_pfs_or_os = mean((pfs_low_dose_decision == 'reject' | pfs_high_dose_decision == 'reject') | 
                              (os_low_dose_decision == 'reject' | os_high_dose_decision == 'reject')) * 100
  ) %>% 
  kable(col.names = NULL, digits = 1, align = 'r', 
        caption = 'Power of Testing PFS and OS (Cont.)') %>% 
  add_header_above(c('Reject PFS and Accept OS', 'Accept PFS and Reject OS', 
                     'Reject PFS and OS', 'Reject PFS or OS'), align = 'r') %>% 
  kable_styling(full_width = TRUE)

#' @import survival ggplot2
#'
#' @importFrom dplyr %>% mutate rename bind_rows arrange row_number case_when
#' @importFrom dplyr filter select group_by summarise contains all_of matches
#' @importFrom rlang expr_text sym .data enquos :=
#' @importFrom mvtnorm pmvnorm
#' @importFrom stats as.formula pnorm model.frame model.response rexp cor runif
#' @importFrom stats IQR complete.cases median sd setNames glm pt terms uniroot
#' @importFrom stats rweibull
#' @importFrom R6 R6Class
#' @importFrom utils head tail stack
#' @importFrom emmeans emmeans contrast regrid
#' @importFrom gMCPLite hGraph
#' @importFrom rpact getDesignGroupSequential
#' @importFrom base64enc base64encode
#' @importFrom htmltools HTML
#' @importFrom grDevices dev.off png
#' @importFrom graphics axis barplot hist par
#' @importFrom rstudioapi isAvailable viewer
NULL

#' Define a Trial
#'
#' @description
#' Define a trial. This is a user-friendly wrapper for
#' the class constructor \code{Trial$new()}. Users who are not familiar with
#' the concept of classes may consider using this wrapper directly.
#'
#' Trial's name, planned size/duration, enrollment plan,
#' dropout mechanism and seeding are specified in this function. Note that
#' many of these parameters can be altered adaptively during a trial.
#'
#' Note that it is users' responsibility to assure that the units of dropout
#' time, trial duration, and readout of non-tte endpoints are consistent.
#'
#' @param name character. Name of trial. Usually, hmm..., useless.
#' @param n_patients integer. Maximum (and initial) number of patients
#' could be enrolled when planning the trial. It can be altered adaptively
#' during a trial.
#' @param duration Numeric. Trial duration. It can be altered adaptively
#' during a trial.
#' @param description character. Optional for description of the trial. By
#' default it is set to be trial's \code{name}. Usually useless.
#' @param seed random seed. If \code{NULL}, seed is set for each simulated
#' trial automatically and saved in output. It can be retrieved in the
#' \code{seed} column in \code{$get_output()}. Setting it to be \code{NULL}
#' is recommended. For debugging, set it to a specific integer.
#' @param enroller a function returning a vector enrollment time for
#' patients. Its first argument \code{n} is the number of enrolled patients.
#' Set it to \code{StaggeredRecruiter} can handle most of the use cases.
#' See \code{?TrialSimulator::StaggeredRecruiter} for more information.
#' @param dropout a function returning a vector of dropout time for patients.
#' It can be any random number generator with first argument \code{n},
#' the number of enrolled patients. Usually \code{rexp} if dropout rate
#' is set at a single time point, or \code{rweibull} if dropout rates are
#' set at two time points. See \code{?TrialSimulator::weibullDropout}.
#' @param stratification_factors character. Names of baseline characteristics
#' to define stratums in stratified permuted block randomization.
#' Stratification factors must be defined in \code{endpoint()} with
#' \code{readout = 0}. As a natural assumption for randomized trial,
#' \code{TrialSimulator} assumes that the baseline
#' characteristics share the same distribution across arms, but endpoints
#' can have same or different distributions given baseline characteristics.
#' \code{NULL} by default, i.e., unstratified permuted block randomization is
#' executed.
#' @param silent logical. \code{TRUE} to mute messages. However, warning
#' message is still displayed. Usually set it to \code{TRUE} in formal
#' simulation. Default: \code{FALSE}.
#' @param ... (optional) arguments of \code{enroller} and \code{dropout}.
#'
#' @examples
#' risk1 <- data.frame(
#'   end_time = c(1, 10, 26.0, 52.0),
#'   piecewise_risk = c(1, 1.01, 0.381, 0.150) * exp(-3.01)
#' )
#'
#' pfs1 <- endpoint(name = 'pfs', type='tte',
#'           generator = PiecewiseConstantExponentialRNG,
#'           risk = risk1, endpoint_name = 'pfs')
#'
#' orr1 <- endpoint(
#'   name = 'orr', type = 'non-tte',
#'   readout = c(orr=1), generator = rbinom,
#'   size = 1, prob = .4)
#'
#' placebo <- arm(name = 'pbo')
#'
#' placebo$add_endpoints(pfs1, orr1)
#'
#' risk2 <- risk1
#' risk2$hazard_ratio <- .8
#'
#' pfs2 <- endpoint(name = 'pfs', type='tte',
#'           generator = PiecewiseConstantExponentialRNG,
#'           risk = risk2, endpoint_name = 'pfs')
#'
#' orr2 <- endpoint(
#'   name = 'orr', type = 'non-tte',
#'   generator = rbinom, readout = c(orr=3),
#'   size = 1, prob = .6)
#'
#' active <- arm(name = 'ac')
#'
#' active$add_endpoints(pfs2, orr2)
#'
#' ## Plan a trial, Trial-3415, of up to 100 patients.
#' ## Enrollment time follows an exponential distribution, with median 5
#' trial <- trial(
#'   name = 'Trial-3415', n_patients = 100,
#'   seed = 31415926, duration = 100,
#'   enroller = rexp, rate = log(2) / 5)
#'
#' trial
#'
#' trial$add_arms(sample_ratio = c(1, 2), placebo, active)
#'
#' ## updated information after arms are registered
#' trial
#'
#' @export
#'
trial =
  function(
    name,
    n_patients,
    duration,
    description = name,
    seed = NULL,
    enroller,
    dropout = NULL,
    stratification_factors = NULL,
    silent = FALSE,
    ...
  ){

    Trials$new(
      name = name,
      n_patients = n_patients,
      duration = duration,
      description = description,
      seed = seed,
      enroller = enroller,
      dropout = dropout,
      stratification_factors = stratification_factors,
      silent = silent,
      ...
    )

  }

#' Class of Controller
#' @description
#' Create a class of controller to run a trial.
#'
#' Public methods in this R6 class are used in developing
#' this package. Thus, we have to export the whole R6 class which exposures all
#' public methods. However, only the public methods in the list below are
#' useful to end users.
#'
#' \itemize{
#' \item \code{$run()}
#' \item \code{$get_output()}
#' \item \code{$reset()}
#' }
#'
#' @docType class
#' @examples
#' ##
#' @export
Controllers <- R6::R6Class(
  'Controllers',

  private = list(
    trial = NULL,
    listener = NULL,
    silent = FALSE,
    dry_run = FALSE,
    output = NULL,

    run_ = function(plot_event = TRUE, silent = FALSE, dry_run = FALSE){

      private$silent <- silent
      private$dry_run <- dry_run
      self$mute()

      self$get_listener()$monitor(self$get_trial(), private$dry_run)
      if(plot_event){
        self$get_trial()$event_plot()
      }

    },

    run_sequential_ = function(n, plot_event, silent, dry_run){

      for(idx in 1:n){
        tryCatch(
          expr = {
            private$run_(plot_event, silent, dry_run)
          },

          error = function(e){
            self$get_trial()$save(e$message, 'error_message', overwrite = TRUE)
            private$output <- bind_rows(private$output, self$get_trial()$get_output())
            stop(e$message)
          }
        )

        private$output <- bind_rows(private$output, self$get_trial()$get_output())

        if(idx < n){
          self$reset()
        }
      }

    },

    run_parallel_ = function(n, n_workers, silent, dry_run){

      if(!requireNamespace("mirai", quietly = TRUE)){
        stop('Package "mirai" is required for parallel execution (n_workers > 1). ',
             'Install it with install.packages("mirai").')
      }

      ## Serialize trial and listener once; workers will deserialize
      ## independent copies. make_arms_snapshot() has already been called
      ## in run(), so the snapshot is baked into the serialized bytes.
      trial_raw <- serialize(self$get_trial()$clone(deep = TRUE), NULL)
      listener_raw <- serialize(self$get_listener()$clone(deep = TRUE), NULL)

      ## Distribute replicates evenly across workers
      reps_per_worker <- rep(floor(n / n_workers), n_workers)
      remainder <- n %% n_workers
      if(remainder > 0){
        reps_per_worker[1:remainder] <- reps_per_worker[1:remainder] + 1L
      }

      with(
        mirai::daemons(n_workers),
        {
          ## Submit all tasks
          tasks <- vector("list", n_workers)
          for(i in seq_len(n_workers)){
            tasks[[i]] <- mirai::mirai(
              {
                library(TrialSimulator)

                trial <- unserialize(trial_raw)
                listener <- unserialize(listener_raw)

                output <- NULL
                error_msg <- NULL

                ## R uses Mersenne-Twister streams by default to generate
                ## random numbers, which theoretically is not safe for parallel,
                ## but the chance of getting wrong is low given the extremely
                ## long period it has (2^19937 - 1).
                ## The packages parallel and mirai use L'Ecuyer-CMRG streams.
                ## The reason why I use Mersenne-Twister:
                ## 1.
                RNGkind('default')
                for(j in seq_len(reps)){
                  trial$reset()
                  listener$reset()

                  trial$mute(silent)
                  listener$mute(silent)

                  run_ok <- tryCatch(
                    {
                      listener$monitor(trial, dry_run)
                      TRUE
                    },
                    error = function(e){
                      trial$save(e$message, 'error_message', overwrite = TRUE)
                      error_msg <<- e$message
                      FALSE
                    }
                  )

                  output <- dplyr::bind_rows(output, trial$get_output())

                  if(!run_ok) break
                }

                list(output = output, error = error_msg)
              },
              trial_raw = trial_raw,
              listener_raw = listener_raw,
              #worker_seed = worker_seeds[i],
              reps = reps_per_worker[i],
              silent = silent,
              dry_run = dry_run
            )
          }

          ## Collect results; fail-fast on first error
          for(i in seq_len(n_workers)){
            result <- mirai::call_mirai(tasks[[i]])$data

            ## Unexpected worker-level failure (e.g. package not found,
            ## serialization issue)
            if(mirai::is_error_value(result)){
              stop('Worker ', i, ' failed unexpectedly: ', result)
            }

            private$output <- bind_rows(private$output, result$output)

            ## Expected simulation error (from stop() in user action functions)
            if(!is.null(result$error)){
              stop(result$error)
            }
          }
        }
      )

      RNGkind('default')

    }

  ),

  public = list(

    #' @description
    #' initialize a controller of the trial
    #' @param trial a trial object returned from \code{trial()}.
    #' @param listener a listener object returned from \code{listener()}.
    initialize = function(trial, listener){
      stopifnot(inherits(trial, 'Trials'))
      stopifnot(inherits(listener, 'Listeners'))
      private$trial <- trial
      private$listener <- listener
      private$silent <- FALSE
      private$dry_run <- FALSE
    },

    #' @description
    #' return listener in a controller.
    get_listener = function(){
      private$listener
    },

    #' @description
    #' return trial in a controller.
    get_trial = function(){
      private$trial
    },

    #' @description
    #' mute all messages (not including warnings).
    #' @param silent logical.
    mute = function(){
      self$get_trial()$mute(private$silent)
      self$get_listener()$mute(private$silent)
    },

    #' @description
    #' reset the trial and listener registered to the controller before running
    #' additional replicate of simulation. This is usually done between two
    #' calls of \code{controller$run()}.
    #'
    reset = function(){
      self$get_trial()$reset()
      self$get_listener()$reset()
    },

    #' @description
    #' return a data frame of all current outputs saved by calling \code{save()}.
    #' @param cols character vector. Columns to be returned from the data frame
    #' of simulation outputs. If \code{NULL}, all columns are returned.
    #' @param simplify logical. Return vector rather than a data frame of one
    #' column when \code{length(cols) == 1} and \code{simplify == TRUE}.
    #' @param tidy logical. \code{TrialSimulator} automatically records a set
    #' of standard outputs at milestones, even when \code{doNothing} is used
    #' as action functions. These includes time of triggering milestones,
    #' number of observed events for time-to-event endpoints, and number of
    #' non-missing readouts for non-TTE endpoints
    #' (see \code{vignette('actionFunctions')}). This usually mean a large
    #' number of columns in outputs. If users have no intent to summarize a
    #' trial on these columns, setting \code{tidy = TRUE} can eliminate these
    #' columns from \code{get_output()}. This is useful to reduced the size of
    #' output data frame when a large number of replicates are done for
    #' simulation. Note that currently we use regex
    #' \code{"^n_events_<.*?>_<.*?>$"} and
    #' \code{"^milestone_time_<.*?>$"} to match columns to be eliminated.
    #' If users plan to use \code{tidy = TRUE}, caution is needed when naming
    #' custom outputs in \code{save()}. Default \code{FALSE}.
    get_output = function(cols = NULL, simplify = TRUE, tidy = FALSE){
      if(is.null(cols)){
        cols <- colnames(private$output)
      }

      if(!all(cols %in% names(private$output))){
        stop('Columns <', paste0(setdiff(cols, names(private$output)), collapse = ', '),
             '> are not found in trial$output. Check if there is a typo. ')
      }

      ret <- private$output[, cols, drop = FALSE]

      if(tidy){
        ret <- ret %>%
          select(!matches("^n_events_<.*?>_<.*?>$")) %>%
          select(!matches("^milestone_time_<.*?>$"))
      }

      if(simplify && ncol(ret) == 1){
        return(ret[, 1])
      }else{
        return(ret)
      }
    },

    #' @description
    #' run trial simulation.
    #' @param n integer. Number of replicates of simulation.
    #' \code{n = 1} by default. Simulation results can be accessed by
    #' \code{controller$get_output()}.
    #' @param n_workers integer. Number of parallel workers. When
    #' \code{n_workers = 1} (default), replicates are run sequentially.
    #' When \code{n_workers > 1}, replicates are distributed across
    #' parallel workers using the \code{mirai} package, which must be
    #' installed separately. Each worker receives a serialized copy of
    #' the trial and listener objects and runs its share of replicates
    #' independently. If any replicate encounters an error, execution
    #' stops and already-collected results are preserved in
    #' \code{$get_output()}. To debug, manually set \code{seed} in
    #' \code{trial()} and \code{n_workers = 1} in \code{run()} for reproduced
    #' results. Note that optimal \code{n_workers} may not be
    #' \code{parallel::detectCores()}. For example, Macbook with M1/M2/M3 chips
    #' may have performance cores and efficiency cores. To achieve the best
    #' parallel performance, one may want to use the performance cores only.
    #' For a M1 laptop with 4 performance cores, \code{n_workers = 3} may give
    #' the best performance.
    #' @param plot_event logical. Create event plot if \code{TRUE}. Forced to
    #' \code{FALSE} when \code{n > 1} or \code{n_workers > 1}.
    #' @param silent logical. \code{TRUE} if muting all messages during a
    #' trial. Note that warning messages are still displayed.
    #' @param dry_run logical. We are considering retire this argument.
    #' \code{TRUE} if action function provided by users is
    #' ignored and an internal default action \code{.default_action} is called
    #' instead. This default function only locks data when the milestone is
    #' triggered. Milestone time and number of endpoints' events or sample sizes
    #' are saved. It is suggested to set \code{dry_run = TRUE} to estimate
    #' distributions of triggering time and number of events before formally
    #' using custom action functions if a fixed design is in use.
    #' This helps determining planned maximum
    #' information for group sequential design and reasonable time of milestone
    #' of interest when planning a trial. Set it to \code{FALSE} for formal
    #' simulations. However, for an adaptive design where arm(s) could
    #' possibly be added or removed, setting \code{dry_run} to \code{TRUE}
    #' is usually not helpful because adaption should be executed
    #' before estimating the milestone time.
    run = function(n = 1, n_workers = 1, plot_event = TRUE, silent = FALSE, dry_run = FALSE){

      self$get_trial()$make_arms_snapshot()
      private$output <- NULL

      if(plot_event){
        if(n > 1 || n_workers > 1){
          plot_event <- FALSE
          if(!silent){
            warning('plot_event is forced to FALSE for parallel execution or when n > 1.',
                    immediate. = TRUE)
          }
        }
      }

      if(n_workers == 1){
        private$run_sequential_(n, plot_event, silent, dry_run)
      }else{
        private$run_parallel_(n, n_workers, silent, dry_run)
      }
    }
  )

)

#' Fit Cox Proportional Hazard Ratio model
#'
#' @description
#' Fit Cox proportional hazards model on an time-to-event endpoint.
#'
#' Refer to \href{https://zhangh12.github.io/TrialSimulator/articles/wrappers.html}{this vignette}
#' for more information and examples.
#'
#' @param formula An object of class \code{formula} that can be used with
#' \code{survival::coxph}. The data frame \code{data} must consist a column
#' \code{arm} and a column of the endpoint specified in \code{formula}.
#' Covariates can be adjusted. Interactions between \code{arm} and covariates are
#' allowed in \code{formula}, but \code{arm} must has a term of main effect,
#' and only estimate of that main effect is tested.
#' @param placebo Character. String indicating the placebo in \code{data$arm}.
#' @param data Data frame. Usually it is a data snapshot locked at a milestone.
#' @param alternative a character string specifying the alternative hypothesis,
#' must be one of \code{"greater"} or \code{"less"}, i.e., one-sided test is
#' enforced. No default value.
#' \code{"greater"} means superiority of treatment over placebo is established
#' by an hazard ratio greater than 1.
#' @param scale character. The type of estimate in the output. Must be one
#' of \code{"log hazard ratio"} or \code{"hazard ratio"}. No default value.
#' @param tidy logical. \code{FALSE} if more information are returned.
#' Default: \code{TRUE}.
#' @param ... (optional) subset conditions compatible with \code{dplyr::filter}.
#' \code{coxph} will be fitted on this subset only. This argument can be useful
#' to create a subset of data for analysis when a trial consists of more
#' than two arms. By default, it is not specified,
#' all data will be used to fit the model. More than one condition can be
#' specified in \code{...}, e.g.,
#' \code{fitCoxph(formula, 'pbo', data, 'less', 'log hazard ratio', arm \%in\% c('pbo', 'low dose'), x > 0.5)},
#' which is equivalent to:
#' \code{fitCoxph(formula, 'pbo', data, 'less', 'log hazard ratio', arm \%in\% c('pbo', 'low dose') & x > 0.5)}.
#' Note that if more than one treatment arm are present in the data after
#' applying filter in \code{...}, models are fitted and tested for placebo verse
#' each of the treatment arms.
#'
#' @returns a data frame with three columns:
#' \describe{
#' \item{\code{arm}}{name of the treatment arm. }
#' \item{\code{placebo}}{name of the placebo arm. }
#' \item{\code{estimate}}{estimate of main effect of arm, depending on \code{scale}. }
#' \item{\code{p}}{one-sided p-value for log hazard ratio (treated vs placebo). }
#' \item{\code{info}}{the number of events of the endpoint in the subset. }
#' \item{\code{z}}{the z statistics of log hazard ratios. }
#' }
#'
#' @export
#'
fitCoxph <- function(formula, placebo, data, alternative, scale, ..., tidy = TRUE) {

  if(!inherits(formula, 'formula')){
    stop('formula must be a formula object with "arm" indicating the column arm in data. ')
  }

  if(!is.character(placebo) || length(placebo) != 1){
    stop("placebo must be a single character string")
  }

  if(!is.data.frame(data)){
    stop("data must be a data frame")
  }

  alternative <- match.arg(alternative, choices = c('greater', 'less'))

  valid_scales <- c('log hazard ratio', 'hazard ratio')
  if(!is.character(scale) || length(scale) != 1 || !(scale %in% valid_scales)){
    stop('scale must be one of ', paste0(valid_scales, collapse = ', '))
  }

  vars_in_formula <- all.vars(formula)
  missing_vars <- setdiff(vars_in_formula, names(data))
  if(length(missing_vars) > 0){
    stop('The following variable(s) used in formula are missing from data: \n',
         paste0(missing_vars, collapse = ', '))
  }

  if(!'arm' %in% vars_in_formula){
    stop('formula must include main effect term for arm. ')
  }


  # Prepare the data based on condition ...
  filtered_data <- if(...length() == 0){
    data
  }else{
    tryCatch({
      data %>% dplyr::filter(...)
    },
    error = function(e){
      stop('Error in filtering data for coxph test. ',
           'Please check condition in ..., ',
           'which should be compatible with dplyr::filter. ')
    })
  }

  # Check if any data remains after filtering
  if (nrow(filtered_data) == 0) {
    stop("No data remaining after applying subset condition. ")
  }

  treatment_arms <- setdiff(unique(filtered_data$arm), placebo) %>% sort()

  ret <- NULL

  for(trt_arm in treatment_arms){
    sub_data <- filtered_data %>% dplyr::filter(.data$arm %in% c(placebo, trt_arm))

    # Ensure arm is a factor with placebo and treatment
    sub_data$arm <- factor(sub_data$arm, levels = c(placebo, trt_arm))

    # Fit the Cox model
    fit <- tryCatch({
      coxph(formula, data = sub_data, model = TRUE)
    }, error = function(e){
      stop('coxph model fitting failed: ', e$message)
    })

    sfit <- summary(fit)

    ## identify coefficient of treatment arm (main effect)
    coef_row <- grep(paste0('^arm', trt_arm, '$'), rownames(sfit$coef))
    if(length(coef_row) != 1){
      stop('fixCoxph: unable to identify arm main effect coefficient for arm <', trt_arm,
           '>. Please use <arm> in formula explicitly. ')
    }

    estimate <- sfit$coef[coef_row, 'coef']
    se <- sfit$coef[coef_row, 'se(coef)']
    z <- estimate / se
    p <- ifelse(alternative == 'greater', 1 - pnorm(z), pnorm(z))

    model_data <- model.frame(fit)
    surv_obj <- model.response(model_data)
    info <- sum(surv_obj[, 'status'] == 1)

    res <- data.frame(arm = trt_arm, placebo = placebo,
                      estimate = ifelse(scale == 'log hazard ratio',
                                        estimate, exp(estimate)),
                      p = p, info = info, z = z
                    )

    ## return only if tidy = FALSE
    if(!tidy){
      stopifnot(length(surv_obj) == nrow(model_data))
      res$info_pbo <- sum(surv_obj[, 'status'] == 1 & model_data$arm %in% placebo)
      res$info_trt <- sum(surv_obj[, 'status'] == 1 & model_data$arm %in% trt_arm)
      stopifnot(res$info_pbo + res$info_trt == res$info)

      res$n_pbo <- sum(model_data$arm %in% placebo)
      res$n_trt <- sum(model_data$arm %in% trt_arm)
      stopifnot(res$n_trt + res$n_pbo == nrow(model_data))
    }

    ret <- rbind(ret, res)

  }

  rownames(ret) <- NULL
  class(ret) <- c('fit_coxph', class(ret))
  ret
}

#' Solve Parameters in a Mixture Exponential Distribution
#'
#' @description
#' This is a helper function to explore parameters for endpoint generator,
#' likely in an enrichment design.
#'
#' Assume that the overall population in an arm is a mixture of two exponential
#' distributions with medians \code{median1} (\eqn{m_1}) and
#' \code{median2} (\eqn{m_2}). Given the proportion of the first component
#' (\eqn{p_1}) and the overall median \eqn{m}, we have
#'
#' \deqn{p_1 (1 - e^{-\log(2)m/m_1}) + (1 - p_1) (1 - e^{-\log(2)m/m_2}) = 1/2}
#'
#' This function computes \eqn{m_2} or \eqn{m} given \eqn{p_1} and \eqn{m_1}.
#' These parameters can be used in custom random number generator to define
#' exponential distributed endpoints.
#'
#' Note that the math formula above may not be displayed correctly on a
#' html page. You can read it with better format by running
#' \code{?solveMixtureExponentialDistribution}.
#'
#' @param weight1 numeric. The proportion of the first component.
#' @param median1 numeric. Median of the first component.
#' @param median2 numeric. Median of the second component. If \code{NULL},
#' then \code{overall_median} must be specified, and this function will
#' calculate and return \code{median2}.
#' @param overall_median numeric. Median of the overall population. If
#' \code{NULL}, then \code{median2} must be specified, and this function
#' will calculate and return \code{overall_median}.
#'
#' @returns a named vector of \code{median2} or \code{overall_median}.
#'
#'
#' @examples
#'
#' library(dplyr)
#'
#' median2 <-
#'   solveMixtureExponentialDistribution(
#'     weight1 = .3,
#'     median1 = 10,
#'     overall_median = 8)
#'
#' median2
#'
#' n <- 1e6
#' ifelse(
#'   runif(n) < .3,
#'   rexp(n, rate=log(2)/10),
#'   rexp(n, rate=log(2)/median2)) %>%
#'   median() ## should be close to 8
#'
#' overall_median <-
#'   solveMixtureExponentialDistribution(
#'     weight1 = .4,
#'     median1 = 12,
#'     median2 = 4)
#'
#' overall_median
#'
#' ifelse(
#'   runif(n) < .4,
#'   rexp(n, rate=log(2)/12),
#'   rexp(n, rate=log(2)/4)) %>%
#'   median() ## should be close to overall_median
#'
#' @export
#'
solveMixtureExponentialDistribution <- function(weight1, median1, median2 = NULL, overall_median = NULL){

  if(is.null(median2) && is.null(overall_median)){
    stop('Only one value of median2 or overall_median needs to be specified. ',
         'The other value is returned after calculation in function. ')
  }

  if(!is.null(median2) && !is.null(overall_median)){
    stop('Only one value of median2 or overall_median needs to be specified. ',
         'The other value is returned after calculation in function. ')
  }

  if(is.null(median2)){
    ret <- -log(2) * overall_median /
      log(1 - (0.5 - weight1 * (1 - exp(-log(2)/median1 * overall_median))) / (1 - weight1))

    names(ret) <- 'median2'

  }else{
    object_function <- function(overall_median, weight1, median1, median2){
      weight1 * (1 - exp(-log(2) / median1 * overall_median)) +
        (1 - weight1) * (1 - exp(-log(2) / median2 * overall_median)) - 0.5
    }

    ret <- uniroot(interval = c(0, max(median1, median2)),
                   f = object_function,
                   tol = 1e-5,
                   weight1 = weight1,
                   median1 = median1,
                   median2 = median2)$root

    names(ret) <- 'overall_median'
  }

  ret

}

#' Get simulation output in the vignette doseRanging.Rmd
#'
#' Internal function that retrieves precomputed simulation results.
#' Not meant for use by package users.
#'
#' @return A data frame containing simulation results of 10 replicates.
#'
getDoseRangingOutput <- function(){
  ## this is saved by calling
  ## usethis::use_data(dose_ranging_output, internal = TRUE, overwrite = TRUE)
  return(dose_ranging_output)
}

#' Quantile Function of Piecewise Exponential Distribution
#'
#' @description
#' To generate endpoint that follows a piecewise exponential distribution and
#' is correlated with other endpoints, the copula method is commonly used. The
#' quantile function needs to be specified (e.g., in the \code{simdata} package).
#' If a piecewise exponential distributed endpoint is independent to other
#' endpoints, one can simply use
#' \code{TrialSimulator::PiecewiseConstantExponentialRNG()}
#' to specify the \code{generator} argument in \code{endpoint()}.
#'
#' There are many R packages implementing the quantile function of the
#' piecewise exponential distributed random variable. Why do I implement it
#' again? The reason is that this function is extremely important for simulating
#' time-to-event endpoint in clinical trial simulation, thus the speed matters.
#' \code{qPiecewiseExponential()} is implemented purely in R for
#' code transparency, and is much faster than other packages.
#'
#' @param p numeric. A vector of probabilities.
#' @param times numeric. A vector of time points where risk (event
#' rates) change. 0 shouldn't be in \code{times}.
#' @param piecewise_risk numeric. A vector of constant risk (event rates) in
#' a time window. \code{length(piecewise_risk) = length(times) + 1}. To fully
#' specify a piecewise exponential distribution, the number of risk parameters
#' is one greater than the number of changepoints in \code{times}.
#'
#' @returns A vector of quantiles.
#' @export
#'
#' @examples
#' ## This code snippet can take > 10s to execute
#' if(interactive()){
#' library(TrialSimulator)
#'
#' run <- TRUE
#' if(!requireNamespace("rpact", quietly = TRUE)){
#'   run <- FALSE
#'   message("Please install 'rpact' to run this example.")
#' }
#'
#' if(!requireNamespace("PWEXP", quietly = TRUE)){
#'   run <- FALSE
#'   message("Please install 'PWEXP' to run this example.")
#' }
#'
#' if(!requireNamespace("PWEALL", quietly = TRUE)){
#'   run <- FALSE
#'   message("Please install 'PWEALL' to run this example.")
#' }
#'
#' if(run){
#'
#' x <- solvePiecewiseConstantExponentialDistribution(
#'   surv_prob = c(.9, .75, .64, .42, .28),
#'   times = c(.4, 1.2, 4, 5.5, 9)
#' )
#'
#' p <- stats::runif(1e5)
#'
#' ## fast, and only five lines of R codes
#' message('TrialSimulator::qPiecewiseExponential(): ')
#' system.time(
#'   a <- qPiecewiseExponential(
#'     p, times = x$end_time, piecewise_risk = c(x$piecewise_risk, .1)
#'   )
#' ) |> print()
#'
#' ## > 10s
#' message('rpact::getPiecewiseExponentialQuantile(): ')
#' system.time(
#'   b <- rpact::getPiecewiseExponentialQuantile(
#'     p, piecewiseSurvivalTime = c(0, x$end_time),
#'     piecewiseLambda=c(x$piecewise_risk, .1)
#'   )
#' ) |> print()
#'
#' ## equally fast, but implemented in Fortran
#' message('PWEALL::qpwe(): ')
#' system.time(
#'   c <- PWEALL::qpwe(p, rate = c(x$piecewise_risk, .1), tchange = c(0, x$end_time))$q
#' ) |> print()
#'
#' ## equally fast, long codes in R (maybe more versatile?)
#' message('PWEXP::qpwexp(): ')
#' system.time(
#'   d <- PWEXP::qpwexp(p, rate = c(x$piecewise_risk, .1), breakpoint = x$end_time)
#' ) |> print()
#'
#' message('a == b: ')
#' all.equal(a, b) |> print()
#' message('a == c: ')
#' all.equal(a, c) |> print()
#' message('a == d: ')
#' all.equal(a, d) |> print()
#'
#' }
#' }
qPiecewiseExponential <- function(p, times, piecewise_risk){

  if(length(piecewise_risk) != length(times) + 1){
    stop('length(piecewise_risk) must equal length(times) + 1. ')
  }

  if(any(times <= 0)){
    stop('times must be positive. ')
  }

  if(any(piecewise_risk < 0)){
    stop('piecewise_risk must be non-negative. ')
  }

  if(any(diff(times) <= 0)){
    stop('times must be strictly monotonous increasing. ')
  }

  if(0 %in% times){
    stop('times are time points where event rate change. Do not put 0 in times. ')
  }

  d <- c(0, times) # index: d_0, d_1, ..., d_r
  lam <- c(0, piecewise_risk) # index: lam_0, lam_1, ..., lam_{r+1}

  tmp <- cumsum(diff(lam) * d)
  idx <- findInterval(p, 1 - exp(-lam[-1] * d + tmp), rightmost.closed = TRUE)

  (tmp[idx] - log(1 - p))/lam[-1][idx]

}

#' Define a Regimen
#'
#' @description
#' Define a regimen of a trial. This is a user-friendly wrapper for
#' the class constructor \code{Regimens$new()}. Users who are not familiar with
#' the concept of classes may consider using this wrapper directly.
#'
#' A regimen defines the rules to select patients who
#' switch treatments, to determine the time of switching, and to update patients'
#' endpoint data.
#'
#' @param what a function determining whether patients' data would be
#' updated due to switching treatment. It takes \code{patient_data},
#' a data frame as argument, and returns a data frame of two columns
#' \code{patient_id} and \code{new_treatment}. Patients with \code{NA} in
#' \code{new_treatment} will be skipped.
#' The number of rows in the returned data frame may be smaller than the
#' number of patients in the input data frame. This indicates that some
#' patients' data will not be modifier.
#' Note that the returned object will be passed into function `how()`, which
#' is also provide by users. This argument can also be a
#' list of functions that will be executed sequentially. No default value.
#' @param when a function determining the time at which a patient switches
#' to another treatment regimen, measured from the time of enrollment.
#' It takes \code{patient_data}, a data frame as
#' argument, and returns a data frame of two columns \code{patient_id} and
#' \code{switch_time} (from \code{enroll_time}). No \code{NA} is allowed
#' in \code{switch_time} and the number of rows in the returned data frame
#' must equal the number of rows in \code{patient_data}, i.e., switching
#' time must be specified to every patients.
#' Note that the returned object will be passed into function `how()`, which
#' is also provided by users. This argument can also be a
#' list of functions that will be executed sequentially. No default value.
#' @param how a function updating patients' data after treatment switching.
#' Only modified columns and \code{patient_id} are returned. A cell will
#' be omitted if \code{NA}, meaning no change to that patient for the endpoint
#' or other variables. Equivalently, users can also fill the cell with
#' its original value. This argument can also be a list of functions that
#' will be executed sequentially. No default value.
#' @param ... (optional) named arguments to be passed to one or more of
#' \code{what}, \code{when}, and \code{how}. Each argument is routed to every
#' function whose formal parameter list contains that name. All arguments must
#' be named, and every name must match at least one parameter of at least one
#' function in \code{what}, \code{when}, or \code{how}.
#'
#' @export
#'
regimen <- function(what, when, how, ...){

  Regimens$new(
    what = what,
    when = when,
    how = how,
    ...
  )

}

#' Adding One or More Arms to a Trial
#'
#' @description
#'
#' add one or more arms to a trial. This function can be used in two
#' scenarios: (1) adding arms right after a trial is created
#' (i.e., \code{trial(...)}). \code{sample_ratio} and arms added through
#' \code{...} must be of same length; (2) adding arms to an existing trial in
#' action functions of milestones.
#'
#' This is a user-friendly wrapper of the member function of trial, i.e.,
#' \code{Trials$add_arms()}, which is used in vignettes. Users who are not
#' familiar with the concept of classes may consider using this wrapper
#' directly.
#'
#' @param trial a trial object returned by \code{trial()}.
#' @param sample_ratio integer vector. Sample ratio for permuted block
#' randomization. It will be appended to existing sample ratio in the trial.
#' @param ... one or more objects returned from \code{arm()}.
#' Randomization is carried out with updated
#' sample ratio of newly added arm.
#' Note that we can run \code{Trials$add_arm(sample_ratio1, arm1)} followed
#' by \code{add_arms(trial, sample_ratio2, arm2)}.
#' We would expected similar result with
#' \code{add_arms(trial, c(sample_ratio1, sample_ratio2), arm1, arm2)}. Note
#' that these two methods won't return exactly the same trial because
#' randomization queue are generated twice in the first approach but only
#' once in the second approach. But statistically, they are equivalent and
#' of the same distribution.
#'
#' @export
#'
add_arms <- function(trial, sample_ratio, ...){

  trial$add_arms(sample_ratio = sample_ratio, ...)

}

#' Class of Arm
#' @description
#' Create a class of arm.
#'
#' Public methods in this R6 class are used in developing
#' this package. Thus, we have to export the whole R6 class which exposures all
#' public methods. However, only the public methods in the list below are
#' useful to end users.
#'
#' \itemize{
#' \item \code{$add_endpoints()}
#' \item \code{$print()}
#' }
#'
#' @docType class
#'
#' @examples
#' # Instead of using Arms$new(), please use arm(), a user-friendly
#' # wrapper. See examples in ?arm
#'
#' @export
Arms <- R6::R6Class(
  'Arms',

  public = list(
    #' @description
    #' initialize an arm
    #' @param name name of arm, which is the arm's label in generated data
    #' @param ... subset condition that is compatible with \code{dplyr::filter}.
    #' This can be used to specify inclusion criteria of an arm.
    #' By default it is not specified, i.e. all data generated by the generator
    #' will be used as trial data. More than one conditions can be
    #' specified in \code{...}.
    initialize = function(name, ...){
      stopifnot(is.character(name))

      private$name <- name
      private$endpoints <- list()
      private$inclusion_filters <- enquos(...)
    },

    #' @description
    #' add one or multiple endpoints to the arm.
    #' @param ... one or more objects returned from \code{endpoint()}.
    #'
    #' @examples
    #'
    #' a <- arm(name = 'trt')
    #' x <- endpoint(name = 'x', type = 'tte',
    #'               generator = rexp) # median = log(2)/1 = 0.7
    #' y <- endpoint(name = 'y', type = 'non-tte', readout = c(y = 0),
    #'               generator = rnorm, sd = 1.4, mean = 0.7)
    #'
    #' a$add_endpoints(y, x)
    #'
    #' ## run it in console to see the summary report
    #' a
    #'
    #' print(a) # use the print method
    #'
    add_endpoints = function(...){
      endpoint_list <- list(...)

      for(ep in endpoint_list){
        stopifnot(inherits(ep, 'Endpoints'))
        if(ep$get_uid() %in% names(private$endpoints)){
          stop('Endpoint <', ep$get_uid(), '> is already in the arm <',
               self$get_name(), '>. ')
        }
        private$endpoints[[ep$get_uid()]] <- ep
      }
    },

    #' @description
    #' return name of arm.
    get_name = function(){
      private$name
    },

    #' @description
    #' return number of endpoints in the arm.
    get_number_endpoints = function(){

      if(length(private$endpoints) == 0){
        return(0)
      }

      sapply(
        private$endpoints,
        function(ep){
          length(ep$get_name())
        }
      ) %>%
        sum()
    },

    #' @description
    #' check if the arm has any endpoint. Return \code{TRUE} or \code{FALSE}.
    has_endpoint = function(){
      self$get_number_endpoints() > 0
    },

    #' @description
    #' return a list of endpoints in the arm.
    get_endpoints = function(){
      private$endpoints
    },

    #' @description
    #' return name of endpoints registered to the arm.
    get_endpoints_name = function(){
      lapply(
        self$get_endpoints(),
        function(ep){
          ep$get_name()
        }
      ) %>%
        unlist() %>%
        unname()
    },

    #' @description
    #' update generator of an endpoint object
    #'
    #' @param endpoint_name character. A vector of endpoint names whose
    #' generator is updated.
    #' @param generator a random number generation (RNG) function.
    #' See \code{generator} of \code{endpoint()}.
    #' @param ... optional arguments for \code{generator}.
    update_endpoint_generator = function(endpoint_name, generator, ...){

      private$endpoints[[paste0(endpoint_name, collapse = '/')]]$update_generator(generator, ...)

    },

    #' @description
    #' generate arm data.
    #'
    #' @param n_patients_in_arm integer.
    #' Number of patients randomized to the arm.
    generate_data = function(n_patients_in_arm){

      arm_data <- NULL
      while(is.null(arm_data) || nrow(arm_data) < n_patients_in_arm){
        dat <- NULL
        for(ep in self$get_endpoints()){
          if(is.null(dat)){
            dat <- ep$get_generator()(n_patients_in_arm)
          }else{
            dat <- cbind(dat, ep$get_generator()(n_patients_in_arm))
          }
        }

        filter_str <- paste0("(", sapply(private$inclusion_filters, expr_text), ")", collapse = " & ")

        dat <- if(length(private$inclusion_filters) == 0){
          dat
        }else{
          tryCatch({
            dat %>% dplyr::filter(!!!private$inclusion_filters)
          }, error = function(e){
            stop(
              'Error in filtering data for arm <', self$get_name(), '>: \n',
              'Inclusion criteria: \n', filter_str, '\n',
              'Error message: \n', e$message
            )
          })
        }

        if(nrow(dat) == 0){
          stop('No data meets inclusion criteria of arm <',
               self$get_name(), '>: \n',
               filter_text
          )
        }

        arm_data <- rbind(arm_data, dat)
      }

      head(arm_data, n_patients_in_arm)
    },

    #' @description
    #' print an arm.
    #'
    #' @param categorical_vars character vector of categorical variables.
    #' This can be used to specify variables with limited distinct (numeric)
    #' values as categorical variables in summary report.
    print = function(categorical_vars = NULL){
      white_text_blue_bg <- "" ## "\033[37;44m"
      reset <- "" ## "\033[0m"  # Reset to default color
      logo <- '\u2695\u2695' ## stringi::stri_escape_unicode('⚕')

      # cat(white_text_blue_bg, logo, 'Arm Name: ', self$get_name(), reset, '\n')
      # cat(white_text_blue_bg, logo, '# of Endpoints: ', self$get_number_endpoints(), reset, '\n')
      # cat(white_text_blue_bg, logo, 'Registered Endpoints: ',
      #     paste0(self$get_endpoints_name(), collapse = ', '), reset, '\n')

      title <- paste0('Arm Name: ', self$get_name())
      sub_title <- paste0('Endpoints (',
                          self$get_number_endpoints(), '):',
                          paste0(self$get_endpoints_name(), collapse = ', '))

      dat <- self$generate_data(n_patients_in_arm = 1e4)
      vars <- self$get_endpoints_name()
      event_vars <- intersect(paste0(vars, '_event'), names(dat))
      tte_vars <- gsub('_event$', '', event_vars)
      exclude_vars <- grep('_readout$', names(dat), value = TRUE)


      if(requireNamespace("knitr", quietly = TRUE) &&
         isTRUE(getOption('knitr.in.progress'))) {
        summary_html <- summarizeDataFrame(dat, exclude_vars = exclude_vars,
                                           tte_vars = tte_vars, event_vars = event_vars,
                                           categorical_vars = categorical_vars,
                                           title = title, sub_title = sub_title)

        temp_file <- tempfile(fileext = ".html")
        writeLines(summary_html, temp_file, useBytes = TRUE)

        if(requireNamespace("htmltools", quietly = TRUE)) {
          iframe_html <- htmltools::tags$iframe(
            src = paste0("data:text/html;charset=utf-8;base64,", base64enc::base64encode(temp_file)),
            width = "100%",
            height = "500px",
            style = "border: 1px solid #ccc; border-radius: 4px;"
          )
          cat(as.character(iframe_html))
        } else {
          file_content <- paste(readLines(temp_file), collapse = "\n")
          file_b64 <- base64enc::base64encode(charToRaw(file_content))
          cat('<iframe src="data:text/html;charset=utf-8;base64,', file_b64,
              '" width="100%" height="500px" style="border: 1px solid #ccc;"></iframe>')
        }
      } else {
        summarizeDataFrame(dat, exclude_vars = exclude_vars,
                           tte_vars = tte_vars, event_vars = event_vars,
                           categorical_vars = categorical_vars,
                           title = title, sub_title = sub_title)
      }

      invisible(self)

    }
  ),

  private = list(
    name = NULL,
    inclusion_filters = NULL,
    endpoints = list()
  )
)

# Targeted tests for files with partial coverage
#
# Covers:
#   - CorrelatedPfsAndOs2: generator works when copula is installed, and
#     rejects negative Kendall / non-positive medians
#   - doNothing: returns NULL, errors on extra args
#   - solveMixtureExponentialDistribution: both branches and error paths
#   - EnrollmentCountCondition: print() variants (arms, min_treatment_duration,
#     filter conditions)
#   - Condition: abstract class print() and abstract method errors;
#     & and | operator overloading wire into ConditionCombiner
#   - Endpoints: test_generator, get_name/get_type, print
#   - Arms: add_endpoints uniqueness check, get_number_endpoints, has_endpoint,
#     print


## CorrelatedPfsAndOs2 -----------------------------------------------------

test_that("CorrelatedPfsAndOs2 returns a properly named data frame", {

  skip_if_not_installed("copula")

  set.seed(9)
  d <- CorrelatedPfsAndOs2(500, median_pfs = 5, median_os = 11,
                           kendall = 0.6,
                           pfs_name = "PFS", os_name = "OS")

  expect_equal(nrow(d), 500)
  expect_named(d, c("PFS", "OS", "PFS_event", "OS_event"), ignore.order = TRUE)
  expect_true(all(d$PFS <= d$OS + 1e-10))
  expect_true(all(d$PFS_event == 1))
  expect_true(all(d$OS_event == 1))
})

test_that("CorrelatedPfsAndOs2 rejects bad inputs", {
  expect_error(CorrelatedPfsAndOs2(100, 5, 10, kendall = -0.1), "negative")
  expect_error(CorrelatedPfsAndOs2(100, -1, 10, kendall = 0.5), "Median of PFS")
  expect_error(CorrelatedPfsAndOs2(100, 5, -5, kendall = 0.5), "Median of OS")
  # infeasible Kendall for these medians
  expect_error(CorrelatedPfsAndOs2(100, 5, 10, kendall = 0.0001),
               "too small")
})


## doNothing ---------------------------------------------------------------

test_that("doNothing returns NULL and rejects extra arguments", {
  expect_null(doNothing(trial = NULL))
  expect_error(doNothing(trial = NULL, foo = 1), "should not be specified")
})


## solveMixtureExponentialDistribution -------------------------------------

test_that("solveMixtureExponentialDistribution computes median2 and overall_median", {

  # Given overall_median, compute median2
  m2 <- solveMixtureExponentialDistribution(weight1 = 0.3,
                                            median1 = 10,
                                            overall_median = 8)
  expect_named(m2, "median2")
  expect_gt(as.numeric(m2), 0)

  # Round-trip: feed median2 back, get overall_median close to the original
  om <- solveMixtureExponentialDistribution(weight1 = 0.3,
                                            median1 = 10,
                                            median2 = as.numeric(m2))
  expect_named(om, "overall_median")
  expect_equal(as.numeric(om), 8, tolerance = 1e-3)
})

test_that("solveMixtureExponentialDistribution input validation", {

  expect_error(solveMixtureExponentialDistribution(weight1 = 0.3,
                                                   median1 = 10),
               "Only one value")
  expect_error(solveMixtureExponentialDistribution(weight1 = 0.3,
                                                   median1 = 10,
                                                   median2 = 4,
                                                   overall_median = 8),
               "Only one value")
})


## EnrollmentCountCondition print ------------------------------------------

test_that("enrollment() prints a human-readable description", {

  cond <- enrollment(n = 100)
  out <- capture.output(print(cond))
  expect_true(any(grepl("Number of randomized patients >= +100", out)))

  cond2 <- enrollment(n = 100, arms = c("pbo", "trt"))
  out2 <- capture.output(print(cond2))
  expect_true(any(grepl("arms", out2)))
  expect_true(any(grepl("pbo", out2)))

  cond3 <- enrollment(n = 100, min_treatment_duration = 5)
  out3 <- capture.output(print(cond3))
  expect_true(any(grepl("treated", out3)))

  cond4 <- enrollment(n = 100, sex == "F")
  out4 <- capture.output(print(cond4))
  expect_true(any(grepl("conditions", out4)))
})

test_that("enrollment() validates n and min_treatment_duration", {
  expect_error(enrollment(n = 1.5))
  expect_error(enrollment(n = 10, min_treatment_duration = -1))
})


## Condition abstract class ------------------------------------------------

test_that("Condition abstract class exposes a generic print and errors on get_trigger_time", {

  cond <- TrialSimulator:::Condition$new()
  out <- capture.output(print(cond))
  expect_true(any(grepl("Condition object", out)))

  expect_error(cond$get_trigger_time(trial = NULL),
               "Abstract method")
})


test_that("Condition & / | operators route through ConditionCombiner", {

  c1 <- calendarTime(time = 10)
  c2 <- calendarTime(time = 20)

  combined_or  <- c1 | c2
  combined_and <- c1 & c2
  expect_s3_class(combined_or, "ConditionCombiner")
  expect_s3_class(combined_and, "ConditionCombiner")
})


## Endpoints / Arms --------------------------------------------------------

test_that("Endpoints exposes test_generator, accessors and print", {

  ep <- endpoint(name = "pfs", type = "tte",
                 generator = rexp, rate = log(2) / 5)

  expect_equal(ep$get_name(), "pfs")
  expect_equal(ep$get_type(), "tte")
  expect_null(ep$get_readout())

  d <- ep$test_generator(200)
  expect_true(nrow(d) == 200)

  out <- capture.output(print(ep))
  expect_true(length(out) > 0)
})

test_that("Endpoints name/type/readout checks fire on bad inputs", {

  # type must be tte or non-tte
  expect_error(endpoint(name = "pfs", type = "bad", generator = rexp))

  # non-tte requires readout
  expect_error(endpoint(name = "orr", type = "non-tte", generator = rnorm))

  # readout names must match endpoint names
  expect_error(endpoint(name = "orr", type = "non-tte",
                        readout = c(other = 0), generator = rnorm))
})

test_that("Arms rejects duplicate endpoints and reports counts", {

  a <- arm(name = "trt")
  expect_false(a$has_endpoint())
  expect_equal(a$get_number_endpoints(), 0)

  ep <- endpoint(name = "pfs", type = "tte", generator = rexp, rate = 0.1)
  a$add_endpoints(ep)
  expect_true(a$has_endpoint())
  expect_equal(a$get_number_endpoints(), 1)

  # same endpoint twice -> error
  expect_error(a$add_endpoints(ep), "already in the arm")

  # print exercises its own path
  out <- capture.output(print(a))
  expect_true(length(out) > 0)
})

test_that("Arms inclusion filter restricts generated data", {

  set.seed(1)
  rng <- function(n) data.frame(x = rnorm(n), sex = sample(c("M", "F"), n, replace = TRUE))
  ep <- endpoint(name = c("x", "sex"), type = c("non-tte", "non-tte"),
                 readout = c(x = 0, sex = 0), generator = rng)

  a <- arm(name = "trt", x > 0, sex == "F")
  a$add_endpoints(ep)

  d <- a$generate_data(2000)
  expect_true(all(d$x > 0))
  expect_true(all(d$sex == "F"))
})

#' Removing One or More Arms From a Trial
#'
#' @description
#'
#' remove arms from a trial. The application of this function includes, but is
#' not limited to, dose selection, enrichment analysis
#' (select sub-population).
#'
#' Note that this function should only be called within action functions of
#' milestones. It is users' responsibility to ensure that and
#' \code{TrialSimulator} has no way to track it. In addition, data of the
#' removed arms are censored or truncated by the time of arm removal.
#'
#' This is a user-friendly wrapper of the member function of trial, i.e.,
#' \code{Trials$remove_arms()}, which is used in vignettes. Users who are not
#' familiar with the concept of classes may consider using this wrapper
#' directly.
#'
#' @param trial a trial object returned by \code{trial()}.
#' @param arms_name character vector. Name of arms to be removed.
#'
#' @export
#'
remove_arms <- function(trial, arms_name){

  trial$remove_arms(arms_name = arms_name)

}

#' A wrapper of random number generator.
#'
#' @description
#' This function may be useful to advanced users of \code{TrialSimulator}. It
#' creates a wrapper function of a random number generator, while fixing a
#' subset or all of arguments. This function is design to prevent inadvertent
#' changing to arguments of random number generator. See examples below.
#'
#' @param fn random number generator, e.g., \code{rnorm}, \code{rchisq},
#' etc. It can be user-defined random number generator as well, e.g.,
#' \code{PiecewiseConstantExponentialRNG}.
#' @param ... arguments for \code{fn}. Specifying invalid arguments can trigger error and
#' be stopped. There are three exceptions. (1) \code{rng} can be passed through
#' \code{...} to give true name of \code{fn}. This could be necessary as it may be
#' hard to parse it accurately in \code{DynamicRNGFunction}, or simply for a more
#' informative purpose in some scenarios. (2) \code{var_name} can be passed
#' through \code{...} to specify the name of generated variable. (3) \code{simplify}
#' can be set to \code{FALSE} to convert a vector into a one-column data frame in returned
#' object. This happens for built-in random number generators, e.g., \code{rnorm},
#' \code{rbinom}, etc. These three arguments will not be passed into \code{fn}.
#'
#' @return a function to generate random number based on \code{fn} and arguments in
#' \code{...}. Specified arguments will be fixed and cannot be changed when invoking
#' \code{DynamicRNGFunction(fn, ...)()}. For example,
#' if \code{foo <- DynamicRNGFunction(rnorm, sd = 2)},
#' then \code{foo(n = 100)} will always generate data from normal distribution of
#' variance 4. \code{foo(n = 100, sd = 1)} will trigger an error. However,
#' if an argument is not specified in \code{DynamicRNGFunction}, then it can be specified
#' later. For example, \code{foo(n = 100, mean = -1)} will generate data from N(-1, 4).
#'
#' @examples
#' # example code
#' dfunc <- DynamicRNGFunction(rnorm, sd = 3.2)
#' x <- dfunc(1e3) # mean 0 and sd 3.2
#' hist(x)
#'
#' y <- dfunc(1e3, mean = 3.5) # mean can be changed
#' mean(y)
#'
#' try(z <- dfunc(1e3, sd = 1)) # error because sd is fixed in dfunc
#'
#' @export
DynamicRNGFunction <- function(fn, ...) {

  # Capture fixed arguments
  fixed_args <- list(...)

  ## name of generator
  if(is.null(fixed_args$rng)){
    fn_name <- deparse(substitute(fn))
  }else{
    fn_name <- fixed_args$rng
    fixed_args$rng <- NULL
  }

  var_name <- fixed_args$var_name
  fixed_args$var_name <- NULL

  simplify <- ifelse(is.null(fixed_args$simplify), FALSE, fixed_args$simplify)
  fixed_args$simplify <- NULL

  type <- fixed_args$type
  fixed_args$type <- NULL

  readout <- fixed_args$readout
  fixed_args$readout <- NULL

  # Validate fixed arguments against fn
  unused_args <- setdiff(names(fixed_args), names(formals(fn)))
  if (length(unused_args) > 0) {
    warning('Some arguments in ... are not valid for the function <',
            fn_name, '>: \n',
         paste0(unused_args, collapse = ', '))
    for(arg in unused_args){
      fixed_args[[arg]] <- NULL
    }
  }

  # Create the wrapper function
  wrapper <- function(...) {

    # Capture new arguments
    new_args <- list(...)

    # Prevent overriding fixed arguments
    if (any(names(new_args) %in% names(fixed_args))) {
      stop('Cannot override fixed arguments: ',
           paste(intersect(names(new_args), names(fixed_args)), collapse = ', '))
    }

    # Combine fixed and new arguments
    all_args <- c(fixed_args, new_args)

    # Call the original function
    dat <- do.call(fn, all_args)

    if(is.vector(dat)){
      if(simplify || is.null(type)){ ## useful when an Endpoint class is used to define enroller
        return(dat)
      }

      stopifnot(is.null(type) || (length(type) == 1))
      stopifnot(is.null(var_name) || (length(var_name) == 1))

      ## otherwise add column names
      if(type %in% 'tte'){ ## if tte, add event indicator
        dat <- data.frame(tte = dat, tte_event = 1)
        if(!is.null(var_name)){
          colnames(dat) <- paste0(var_name, c('', '_event'))
        }
      }else{

        stopifnot(is.null(readout) || (length(readout) == 1))
        dat <- data.frame(V1 = dat, V1_readout = unname(readout))
        if(!is.null(var_name)){
          colnames(dat) <- paste0(var_name, c('', '_readout'))
        }
      }
    }else{ ## user-defined rng function is received. It is user's responsibility
           ## to name all columns in data frame returned from their own function
      ## do nothing
      base_cols <- gsub('_event', '', names(dat))
      tte_cols <- base_cols[duplicated(base_cols)]
      non_tte_cols <- setdiff(base_cols, tte_cols)
      if(!setequal(non_tte_cols, names(readout))){
        cols1 <- setdiff(non_tte_cols, names(readout))
        if(length(cols1) > 0){
          stop('Readout may be missing for endpoints <',
               paste0(cols1, collapse = ', '),
               '>. If they are time-to-event endpoints, please add columns <',
               paste0(paste0(cols1, '_event'), collapse = ', '),
               '>, the event indicator, in custom random number generator. ')
        }

        cols2 <- setdiff(names(readout), non_tte_cols)
        if(length(cols2) > 0){
          stop('Readout are specified to endpoints <',
               paste0(cols2, collapse = ', '),
               '> that are not time-to-event endpoints. ')
        }
      }

      for(col in non_tte_cols){
        dat[, paste0(col, '_readout')] <- readout[col]
      }
    }
    dat
  }

  # Add fixed arguments as an attribute for printing
  attr(wrapper, 'args') <- fixed_args
  attr(wrapper, 'function_name') <- fn_name

  # Define a custom print method for the wrapper
  class(wrapper) <- c('dynamic_rng_function', class(wrapper))
  wrapper
}

# Custom print method for objects of class 'dynamic_rng_function'
#' @export
print.dynamic_rng_function <- function(x, ...) {
  ## use cat/print in print method
  cat(attr(x, 'function_name'), ':\n')
  print(attr(x, 'args'))
}

#' Calculate Parameters of Weibull Distribution as a Dropout Method
#'
#' @description
#' Fit scale and shape parameters of the Weibull distribution to match
#' dropout rates at two specified time points. Weibull distribution can be
#' used as a dropout distribution because it has two parameters.
#'
#' Note that It is users' responsibility to assure that the units of dropout
#' time, readout of non-tte endpoints, and trial duration are consistent.
#'
#' @param time a numeric vector of two time points at which dropout rates are
#' specified.
#' @param dropout_rate a numeric vector of dropout rates at \code{time}.
#'
#' @returns a named vector for scale and shape parameters.
#'
#' @examples
#' ## dropout rates are 8% and 18% at time 12 and 18.
#' weibullDropout(time = c(12, 18), dropout_rate = c(.08, .18))
#'
#'
#' @export
weibullDropout <- function(time, dropout_rate){

  if(length(time) != 2){
    stop('time should be of length 2. ')
  }

  if(length(dropout_rate) != 2){
    stop('dropout_rate should be of length 2. ')
  }

  if(!(all(time > 0))){
    stop('time should be positive')
  }

  if(!(all(dropout_rate > 0) && all(dropout_rate < 1))){
    stop('dropout_rate should be between 0 and 1')
  }

  shape <- diff(log(-log(1 - dropout_rate))) / diff(log(time))
  scale <- exp(log(time[1]) - log(-log(1 - dropout_rate[1]))/shape)

  stopifnot(max(abs(1 - exp(-(time / scale)^shape) - dropout_rate)) < 1e-4)
  c(shape = shape, scale = scale)

}

#' Resizing a Trial
#'
#' @description
#'
#' resize a trial with a greater sample size. This function is used to update
#' the maximum sample size adaptively after sample size reassessment. Note that
#' this function should be called within action functions. It is users'
#' responsibility to ensure it and \code{TrialSimulator} has no way to track this.
#'
#' This is a user-friendly wrapper of the member function of trial, i.e.,
#' \code{Trials$resize()}, which is used in vignettes. Users who are not
#' familiar with the concept of classes may consider using this wrapper
#' directly.
#'
#'
#' @param trial a trial object returned by \code{trial()}.
#' @param n_patients integer. Number of maximum sample size of a trial.
#'
#' @export
#'
resize <- function(trial, n_patients){

  trial$resize(n_patients = n_patients)

}

#' Updating Sampling Ratios of Existing Arms in a Trial
#'
#' @description
#'
#' update sample ratios of arms. This could be called after an arm is added or
#' removed. Sample ratios can be updated for any existing arms.
#'
#' This is a user-friendly wrapper of the member function of trial, i.e.,
#' \code{Trials$update_sample_ratio()}, which is used in vignettes.
#' Users who are not familiar with the concept of classes may consider using
#' this wrapper directly.
#'
#' @param trial a trial object returned by \code{trial()}.
#' @param arm_names character vector. Name of arms.
#' @param sample_ratios numeric vector. New sample ratios of arms. If sample
#' ratio is a whole number, the permuted block randomization is adopted;
#' otherwise, \code{sample()} will be used instead, which can cause imbalance
#' between arms by chance. However, this is usually fine for simulation.
#'
#' @export
#'
update_sample_ratio <- function(trial, arm_names, sample_ratios){

  trial$update_sample_ratio(arm_names = arm_names, sample_ratios = sample_ratios)

}

#' Summarize A Data Frame
#'
#' @description
#' A minimum alternative to \code{summarytools::dfSummary} to avoid package
#' dependency. This function is used to generate summary reports of endpoints
#' and arms. No meant to be used by end users. However, users may
#' find it helpful in their own applications if the interface is okay with them.
#'
#' @param data a data frame.
#' @param exclude_vars columns to be excluded from summary.
#' @param tte_vars character. Vector of time-to-event variables.
#' @param event_vars character. Vector of event indicators. Every time-to-event
#' variable should be corresponding to an event indicator.
#' @param categorical_vars character. Vector of categorical variables. This can
#' be used to specify variables with limited distinct values as categorical
#' variables in summary.
#' @param title character. Title of the summary report.
#' @param sub_title character. Sub-title.
#'
#' @returns a data frame of summary
#' @export
#'
#' @examples
#'
#' set.seed(123)
#'
#' n <- 1000
#' data <- data.frame(
#'   age = rnorm(n, 65, 10),
#'   gender = sample(c('M', 'F', NA), n, replace = TRUE, prob = c(.4, .4, .2)),
#'   time_to_death = rexp(n, .01),
#'   death = rbinom(n, 1, .6),
#'   type = sample(LETTERS[1:8], n, replace = TRUE)
#' )
#'
#' summarizeDataFrame(data, tte_vars = 'time_to_death', event_vars = 'death')
#'
summarizeDataFrame <- function(data,
                               exclude_vars = NULL,
                               tte_vars = NULL,
                               event_vars = NULL,
                               categorical_vars = NULL,
                               title = 'Summary',
                               sub_title = '') {

  if(length(tte_vars) != length(event_vars)){
    stop('tte_vars should be of same length as of event_vars. ')
  }

  var_summaries <- list()

  for(var_name in setdiff(names(data), exclude_vars)) {

    var_data <- data[[var_name]]

    is_tte <- var_name %in% tte_vars
    is_event <- var_name %in% event_vars
    is_cate <- var_name %in% categorical_vars

    if(is_tte) {
      event_var <- event_vars[which(tte_vars == var_name)]
      event_data <- data[[event_var]]

      valid_idx <- complete.cases(var_data, event_data)
      n_valid <- sum(valid_idx)
      n_missing <- sum(!valid_idx)

      surv_obj <- Surv(var_data[valid_idx], event_data[valid_idx])
      km_fit <- survfit(surv_obj ~ 1)

      median_surv <- summary(km_fit)$table["median"]
      median_surv <- ifelse(is.na(median_surv), "Not reached", round(median_surv, 2))

      n_events <- sum(event_data[valid_idx])

      temp_file <- tempfile(fileext = ".png")
      png(temp_file, width = 120, height = 80, res = 96, bg = "white")
      opar <- par(mar = c(1.5, 1.5, 0.5, 0.5), cex = 0.5)
      on.exit(par(opar))
      plot(km_fit,
           conf.int = FALSE,
           col = "black",
           lwd = 1,
           xlab = "",
           ylab = "",
           main = "",
           axes = FALSE,
           ylim = c(0, 1))
      axis(1, cex.axis = 0.4, tck = -0.03)
      axis(2, cex.axis = 0.4, tck = -0.03)
      dev.off()

      img_raw <- readBin(temp_file, "raw", file.info(temp_file)$size)
      img_base64 <- base64encode(img_raw)
      unlink(temp_file)

      graph_html <- paste0('<img src="data:image/png;base64,', img_base64, '" alt="KM curve">')

      stats_text <- paste0(
        "Median time: ", median_surv, "<br>",
        "Events: ", n_events, "<br>",
        "Missing: ", n_missing, " (", round(n_missing/nrow(data)*100, 1), "%)"
      )

      var_summaries[[var_name]] <- list(
        no = length(var_summaries) + 1,
        variable = paste0(var_name, "<br>[time-to-event]"),
        stats = stats_text,
        graph = graph_html
      )


    } else if(is_event || is_cate || is.factor(var_data) || is.character(var_data)) {
      cnt <- .count(as.character(var_data))
      inv_cnt <- rev(cnt)
      freq_text <- paste0(names(inv_cnt), ": ", inv_cnt, " (",
                          round(inv_cnt/sum(inv_cnt) * 100, 1), "%)", collapse = "<br>")

      n_text_lines <- length(cnt)

      line_height_px <- 20
      total_height <- n_text_lines * line_height_px

      temp_file <- tempfile(fileext = ".png")
      png(temp_file, width = 120, height = total_height, res = 96, bg = "white")

      opar <- par(mar = c(0, 0, 0, 0), oma = c(0, 0, 0, 0))
      on.exit(par(opar))

      barplot(cnt, horiz = TRUE,
              main = "", xlab = "", ylab = "",
              axes = FALSE, names.arg = rep("", length(cnt)),
              col = "lightgray", border = "gray60", lwd = 0.5,
              space = 0.2)

      dev.off()

      img_raw <- readBin(temp_file, "raw", file.info(temp_file)$size)
      img_base64 <- base64encode(img_raw)
      unlink(temp_file)

      graph_html <- paste0('<img src="data:image/png;base64,', img_base64,
                           '" alt="barplot" style="height: auto; width: 120px;">')

      var_summaries[[var_name]] <- list(
        no = length(var_summaries) + 1,
        variable = paste0(var_name, "<br>", ifelse(is_event, "[event indicator]", "[categorical]")),
        stats = freq_text,
        graph = graph_html
      )

    } else {
      stopifnot(is.numeric(var_data))

      n_valid <- sum(!is.na(var_data))
      n_missing <- sum(is.na(var_data))

      mean_val <- round(mean(var_data, na.rm = TRUE), 2)
      sd_val <- round(sd(var_data, na.rm = TRUE), 2)
      min_val <- round(min(var_data, na.rm = TRUE), 2)
      max_val <- round(max(var_data, na.rm = TRUE), 2)

      stats_text <- paste0(
        "Mean (sd): ", mean_val, " (", sd_val, ")<br>",
        "Min < median < max:<br>",
        min_val, " < ", round(median(var_data, na.rm = TRUE), 2), " < ", max_val, "<br>",
        "IQR (CV): ", round(IQR(var_data, na.rm = TRUE), 2), " (", round(sd_val/abs(mean_val), 2), ")<br>",
        "Missing: ", n_missing, " (", round(n_missing/nrow(data)*100, 1), "%)"
      )

      temp_file <- tempfile(fileext = ".png")
      png(temp_file, width = 120, height = 80, res = 96, bg = "white")
      opar <- par(mar = c(0, 0, 0, 0), cex = 0.5)
      on.exit(par(opar))
      hist(var_data, main = "", xlab = "", ylab = "", axes = FALSE,
           col = "lightgray", border = "gray60", breaks = 10, lwd = 0.5)
      dev.off()

      img_raw <- readBin(temp_file, "raw", file.info(temp_file)$size)
      img_base64 <- base64encode(img_raw)
      unlink(temp_file)

      graph_html <- paste0('<img src="data:image/png;base64,', img_base64, '" alt="histogram">')

      var_summaries[[var_name]] <- list(
        no = length(var_summaries) + 1,
        variable = paste0(var_name, "<br>[numeric]"),
        stats = stats_text,
        graph = graph_html
      )

    }
  }

  html_content <- paste0('
<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <title>', title, '</title>
    <style>
        body {
            font-family: Arial, sans-serif;
            margin: 20px;
            background-color: white;
            display: flex;
            flex-direction: column;
            align-items: center;
        }
        h1 {
            color: black;
            text-align: center;
            margin-bottom: 20px;
            font-size: 20px;
        }
        .subtitle {
            text-align: center;
            color: #666;
            margin-bottom: 20px;
            font-size: 16px;
        }
        table {
            border-collapse: collapse;
            font-size: 14px;
            border: 1px solid #999;
            width: auto;
            margin: 0 auto;
        }
        th {
            background-color: #f0f0f0;
            color: black;
            padding: 10px;
            text-align: left;
            font-weight: normal;
            border: 1px solid #999;
            white-space: nowrap;
            font-size: 14px;
        }
        td {
            padding: 10px;
            border: 1px solid #999;
            vertical-align: top;
            line-height: 1.4;
            font-size: 14px;
        }
        .no-col {
            text-align: center;
            white-space: nowrap;
        }
        .variable-col {
            white-space: nowrap;
        }
        .stats-col {
        }
        .freqs-col {
            line-height: 20px;
        }
        .graph-col {
            text-align: center;
            white-space: nowrap;
            vertical-align: top;
        }
        img {
            display: block;
            margin: 0 auto;
            vertical-align: top;
        }
    </style>
</head>
<body>
    <h1>', title, '</h1>
    <div class="subtitle" style="text-align: left;">
        ', sub_title, '<br>
    </div>

    <table>
        <thead>
            <tr>
                <th class="no-col">No</th>
                <th class="variable-col">Variable</th>
                <th class="stats-col">Stats / Freqs</th>
                <th class="graph-col">Graph</th>
            </tr>
        </thead>
        <tbody>')

  for(var_summary in var_summaries) {
    html_content <- paste0(html_content, '
            <tr>
                <td class="no-col">', var_summary$no, '</td>
                <td class="variable-col">', var_summary$variable, '</td>
                <td class="stats-col">', var_summary$stats, '</td>
                <td class="graph-col">', var_summary$graph, '</td>
            </tr>')
  }

  html_content <- paste0(html_content, '
        </tbody>
    </table>
</body>
</html>')

  if(requireNamespace("knitr", quietly = TRUE) &&
     isTRUE(getOption('knitr.in.progress'))) {
    return(invisible(html_content))
  } else if(requireNamespace("rstudioapi", quietly = TRUE) && rstudioapi::isAvailable()) {
    temp_html <- tempfile(fileext = ".html")
    writeLines(html_content, temp_html, useBytes = TRUE)
    rstudioapi::viewer(temp_html)
    return(invisible(html_content))
  } else {
    cat("Summary generated.\n")
    return(invisible(html_content))
  }
}

.count <- function(x, top_k = 5) {

  x_char <- as.character(x)

  # Frequency of non-NA values
  tbl <- table(x_char, useNA = "no")
  tbl_sorted <- sort(tbl, decreasing = TRUE)

  # Count missing
  n_missing <- sum(is.na(x))

  # Identify top-k values
  if (length(tbl_sorted) <= top_k) {
    main_items <- tbl_sorted
    n_others <- 0
    others_count <- 0
    others_label <- NULL
  } else {
    main_items <- tbl_sorted[1:top_k]
    other_items <- tbl_sorted[(top_k + 1):length(tbl_sorted)]
    n_others <- length(other_items)
    others_count <- sum(other_items)
    others_label <- sprintf("%d others", n_others)
  }

  # Build result in correct order for horiz = TRUE
  # Order: [most freq -> least freq] + [others] + [missing]
  result <- main_items  # least to most frequent

  if (n_others > 0) {
    result <- c(result, setNames(others_count, others_label))
  }

  result <- c(result, Missing = n_missing) |> rev()

  return(result)
}

# Output summaries
#
# Covers:
#   - summarizeMilestoneTime: extracts and summarizes milestone_time_<...> cols,
#     errors when none are present, attaches a `time` attribute usable by
#     plot(), plot.milestone_time_summary returns invisibly
#   - summarizeDataFrame: returns invisible HTML character string and handles
#     tte + categorical + numeric columns; validates tte/event vector lengths

test_that("summarizeMilestoneTime summarizes matching columns", {

  set.seed(11)
  output <- data.frame(
    seed = 1:10,
    `milestone_time_<interim>` = runif(10, 10, 20),
    `milestone_time_<final>`   = runif(10, 30, 40),
    extra_col                  = rnorm(10),
    check.names = FALSE
  )

  res <- summarizeMilestoneTime(output)
  expect_s3_class(res, "milestone_time_summary")
  expect_equal(sort(res$milestone), sort(c("interim", "final")))
  expect_named(res, c("milestone", "mean", "median", "sd", "n"))
  expect_true(all(res$n == 10))
  expect_equal(res$mean[res$milestone == "interim"],
               mean(output$`milestone_time_<interim>`))

  # `time` attribute must carry the original columns renamed
  tm <- attr(res, "time")
  expect_true(is.data.frame(tm))
  expect_equal(sort(colnames(tm)), sort(c("interim", "final")))
})

test_that("summarizeMilestoneTime errors when no matching columns exist", {
  expect_error(summarizeMilestoneTime(data.frame(a = 1:3, b = 4:6)),
               "milestone_time_")
  expect_error(summarizeMilestoneTime("not a data frame"))
})

test_that("plot.milestone_time_summary returns invisibly", {

  output <- data.frame(
    `milestone_time_<m1>` = 1:5,
    `milestone_time_<m2>` = 6:10,
    check.names = FALSE
  )
  res <- summarizeMilestoneTime(output)
  # just checking it runs
  pdf(NULL); on.exit(dev.off())
  expect_invisible(plot(res))
})

test_that("summarizeDataFrame produces HTML for mixed-type data", {

  skip_if_not_installed("survival")

  set.seed(123)
  n <- 50
  data <- data.frame(
    age = rnorm(n, 65, 10),
    gender = sample(c("M", "F"), n, replace = TRUE),
    time_to_death = rexp(n, .01),
    death = rbinom(n, 1, .6),
    type = sample(LETTERS[1:3], n, replace = TRUE)
  )

  html <- summarizeDataFrame(data,
                             tte_vars = "time_to_death",
                             event_vars = "death")
  expect_type(html, "character")
  expect_length(html, 1)
  expect_match(html, "<table")
  expect_match(html, "age")
  expect_match(html, "time-to-event")
})

test_that("summarizeDataFrame rejects mismatched tte/event vars", {
  expect_error(
    summarizeDataFrame(data.frame(a = 1:3, b = 1:3),
                       tte_vars = c("a", "b"),
                       event_vars = "b"),
    "same length"
  )
})

test_that("summarizeDataFrame respects exclude_vars and categorical_vars", {

  set.seed(5)
  data <- data.frame(
    x = rnorm(20),
    flag = rbinom(20, 1, 0.5),
    group = sample(c("A", "B"), 20, replace = TRUE),
    secret = 1:20
  )

  html <- summarizeDataFrame(data,
                             exclude_vars = "secret",
                             categorical_vars = "flag")
  expect_type(html, "character")
  expect_false(grepl("secret", html))
  expect_match(html, "flag")
})

#' Class of Endpoint
#' @description
#' Create a class of endpoint to specify its name, type, readout time
#' (optional) and assign a random number generator.
#'
#' Public methods in this R6 class are used in developing
#' this package. Thus, I have to export the whole R6 class which exposures all
#' public methods. However, none of the public methods is useful to end users
#' except for the one below.
#'
#' \itemize{
#' \item \code{$print()}
#' }
#'
#' @docType class
#'
#' @examples
#' # Instead of using Endpoints$new(), please use endpoint(), a user-friendly
#' # wrapper to define endpoints. See examples in ?endpoint.
#'
#' @export
Endpoints <- R6::R6Class(
  'Endpoints',

  public = list(

    #' @description
    #' initialize an endpoint.
    #' @param name character vector. Name(s) of endpoint(s)
    #' @param type character vector. Type(s) of endpoint(s). It supports
    #' \code{"tte"} for time-to-event endpoints, and \code{"non-tte"} for
    #' all other types of endpoints (e.g., continous, binary, categorical,
    #' or repeated measurement. \code{TrialSimulator} will do some verification if
    #' an endpoint is of type \code{"tte"}. However, no special
    #' manipulation is done for non-tte endpoints.
    #' @param generator a random number generation (RNG) function.
    #' It supports all built-in random number generators in \code{stats}, e.g.,
    #' \code{stats::rnorm}, \code{stats::rexp}, etc. that with \code{n} as the
    #' argument for number of observations and returns a vector.
    #' A custom RNG function is also supported.
    #' \code{generator} could be any functions as long as
    #' (1) its first argument is \code{n}; and (2) it returns a vector of
    #' length \code{n} (univariate endpoint) or a data frame of \code{n} rows
    #' (multiple endpoints), i.e., custom RNG can return data of more than one
    #' endpoint. This is useful when users need to simulate correlated endpoints
    #' or longitudinal data. The column names of returned data frame should
    #' match to \code{name} exactly, although order of columns does not matter.
    #' If an endpoint is of type \code{"tte"}, the custom \code{generator}
    #' should also return a column as its event indicator. For example,
    #' if \code{"pfs"} is \code{"tte"},
    #' then custom \code{generator} should return at least two columns
    #' \code{"pfs"} and \code{"pfs_event"}. Usually \code{pfs_event} can be
    #' all 1s if no censoring. Some RNG functions, e.g.,
    #' \code{TrialSimulator::PiecewiseConstantExponentialRNG()} and
    #' \code{TrialSimulator::CorrelatedPfsAndOs4()}, simulate TTE endpoint
    #' data with censoring simultaneously, thus 0 exists in the columns of
    #' event indicators. Users can implement censorship in their own RNG.
    #' Censoring can also be specified later when
    #' defining a trial object through argument \code{dropout}.
    #' See \code{?trial}.
    #' Note that if covariates, e.g., biomarker, subgroup, are needed in
    #' generating and analyzing trial data, they can and should be defined as
    #' endpoints in \code{endpoint()} as well.
    #' @param readout a named numeric vector with name to be non-tte endpoint(s).
    #' \code{readout} must be specified for every non-tte endpoint. For
    #' example, \code{c(endpoint1 = 6, endpoint2 = 3)}, which means that it
    #' takes 6 and 3 unit time to get readout of
    #' \code{endpoint1} and \code{endpoint2} of a patient since being
    #' randomized. Error message would be prompted if \code{readout} is not
    #' named or readout is not specified for some non-tte endpoint.
    #' If all endpoints are tte, \code{readout} should be \code{NULL} as default.
    #' @param ... optional arguments for \code{generator}.
    initialize = function(
      name,
      type,
      readout = NULL,
      generator,
      ...
    ){

      private$validate_arguments(name, type, readout, generator, ...)
      private$name <- name
      private$uid <- paste0(name, collapse = '/')

      if(length(type) == 1 && length(name) > 1){
        type <- rep(type, length(name))
      }
      private$type <- type

      private$readout <- readout

      if(!is.null(generator)){
        private$generator <- DynamicRNGFunction(
          generator, rng = deparse(substitute(generator)),
          var_name = self$get_name(),
          type = self$get_type(),
          readout = self$get_readout(), ...)
        ## ignore all other arguments in ... if generator is provided
        return()
      }

      stop('Fail to create an endpoint because of lack of RNG generator.')

      ## do something else
    },

    #' @description
    #' test random number generator of the endpoints. It returns an example
    #' dataset of an endpoint object. Note that users of \code{TrialSimulator}
    #' does not need to call this function to generate trial data; instead,
    #' the package will call this function at milestone automatically.
    #' Users may see example in vignette where this function is called.
    #' However, it is for illustration purpose only. In practice, this function
    #' may be used for debugging if users suspect some issues in custom
    #' generator, otherwise, this function should never been called in formal
    #' simulation.
    #' @param n integer. Number of random numbers generated from the generator.
    test_generator = function(n = 1e3){
      stopifnot(is.numeric(n) && is.wholenumber(n) && length(n) == 1)
      self$get_generator()(n)
    },

    #' @description
    #' return random number generator of an endpoint
    get_generator = function(){
      private$generator
    },

    #' @description
    #' update endpoint generator
    #'
    #' @param generator a random number generation (RNG) function.
    #' See \code{generator} of \code{endpoint()}.
    #' @param ... optional arguments for \code{generator}.
    update_generator = function(generator, ...){

      stopifnot(is.function(generator))
      private$generator <- DynamicRNGFunction(
        generator, rng = deparse(substitute(generator)),
        var_name = self$get_name(),
        type = self$get_type(),
        readout = self$get_readout(), ...)
    },

    #' @description
    #' return readout function
    get_readout = function(){
      private$readout
    },

    #' @description
    #' return uid
    get_uid = function(){
      private$uid
    },

    #' @description
    #' return endpoints' name
    get_name = function(){
      private$name
    },

    #' @description
    #' return endpoints' type
    get_type = function(){
      private$type
    },

    #' @description
    #' print an endpoint object
    #'
    #' @param categorical_vars a character vector of endpoints. This can
    #' be used to force variables with limited distinct values as categorical
    #' variables in summary report. For example, a numeric endpoint may take
    #' integer values 0, 1, 2. Instead of computing mean and standard derivation
    #' in the summary report, put this endpoint in \code{categorical_vars} can
    #' force it be a categorical variable and a barplot is generated in summary
    #' report instead.
    #'
    #' @examples
    #'
        #' rng <- function(n){
        #'   data.frame(x = sample(1:3, n, replace = TRUE),
        #'              y = sample(1:3, n, replace = TRUE)
        #'             )
        #' }
        #' ep <- endpoint(name = c('x', 'y'),
        #'                type = c('non-tte', 'non-tte'),
        #'                readout = c(x = 0, y = 0),
        #'                generator = rng)
        #'
        #' ## x and y as continuous endpoints, thus mean and sd are reported
        #' ep
        #'
        #' ## force y to be categorical to create barplot of it
        #' print(ep, categorical_vars = 'y')
        #'
    print = function(categorical_vars = NULL){
      white_text_blue_bg <- "" ## "\033[37;44m"
      reset <- "" ## "\033[0m"  # Reset to default color
      logo <- '\u2695\u2695' ## stringi::stri_escape_unicode('⚕')

      # cat(white_text_blue_bg, logo, 'Endpoint Name: ', paste0(self$get_name(), collapse = ', '), reset, '\n')
      # cat(white_text_blue_bg, logo, '# of Endpoints: ', length(self$get_name()), reset, '\n')

      title <- paste0('Endpoints (', length(self$get_name()), ')')
      sub_title <- paste0(self$get_name(), collapse = ', ')

      dat <- self$test_generator(n = 1e4)
      vars <- self$get_name()
      event_vars <- intersect(paste0(vars, '_event'), names(dat))
      tte_vars <- gsub('_event$', '', event_vars)
      exclude_vars <- grep('_readout$', names(dat), value = TRUE)

      if(requireNamespace("knitr", quietly = TRUE) &&
         isTRUE(getOption('knitr.in.progress'))) {
        summary_html <- summarizeDataFrame(dat, exclude_vars = exclude_vars,
                                           tte_vars = tte_vars, event_vars = event_vars,
                                           categorical_vars = categorical_vars,
                                           title = title, sub_title = sub_title)

        temp_file <- tempfile(fileext = ".html")
        writeLines(summary_html, temp_file, useBytes = TRUE)

        if(requireNamespace("htmltools", quietly = TRUE)) {
          iframe_html <- htmltools::tags$iframe(
            src = paste0("data:text/html;charset=utf-8;base64,", base64enc::base64encode(temp_file)),
            width = "100%",
            height = "500px",
            style = "border: 1px solid #ccc; border-radius: 4px;"
          )
          cat(as.character(iframe_html))
        } else {
          file_content <- paste(readLines(temp_file), collapse = "\n")
          file_b64 <- base64enc::base64encode(charToRaw(file_content))
          cat('<iframe src="data:text/html;charset=utf-8;base64,', file_b64,
              '" width="100%" height="500px" style="border: 1px solid #ccc;"></iframe>')
        }
      } else {
        summarizeDataFrame(dat, exclude_vars = exclude_vars,
                           tte_vars = tte_vars, event_vars = event_vars,
                           categorical_vars = categorical_vars,
                           title = title, sub_title = sub_title)
      }

      invisible(self)

    }
  ),

  private = list(
    ## @field uid a character for unique ID of an Endpoint object. This is needed
    ## because an object can probably consist of more than one endpoints, i.e.,
    ## \code{name} has length greater than 1.
    ## @field name a character vector for name(s) of endpoint(s), e.g., \code{"pfs"}, \code{"ORR"}.
    ## @field type a character vector for type(s) of endpoint(s), e.g., \code{"tte"},
    ## \code{"non-tte"}.
    ## @field generator a user-defined RNG function. Its first argument must be \code{n},
    ## number of patients. It must return a data frame of \code{n} rows. When `generator`
    ## is \code{NULL}, one can create one endpoint at a time, i.e., both \code{name} and \code{type}
    ## are of length 1. Otherwise, one can create multiple endpoints at a time as
    ## long as \code{generator} is capable to generate data for the endpoints.
    uid = NULL,
    name = NULL,
    type = NULL,
    generator = NULL,
    readout = NULL,

    validate_arguments = function(
      name,
      type,
      readout,
      generator,
      ...
    ){
      stopifnot(is.character(name))
      stopifnot(is.character(type))
      stopifnot((length(name) == length(type)) || (length(type) == 1))
      if(!all(type %in% c('tte', 'non-tte'))){
        stop('Type of endpoint can only be either "tte" or "non-tte". ',
             'You use <', paste0(setdiff(type, c('tte', 'non-tte')), collapse = ', '),
             '> in endpoint(). ')
      }

      stopifnot(!is.null(generator))

      if(is.null(readout)){
        if(any(type != 'tte')){
          stop('Readout cannot be NULL for <',
               paste0(name[type != 'tte'], collapse = ', '),
               '>. ')
        }
      }else{

        stopifnot(is.numeric(readout) && all(readout >= 0))
        if(is.null(names(readout)) || ('' %in% names(readout))){
          stop('readout should be a named numeric vector, ',
               'e.g. c(os = 1, pfs = 2). ')
        }

        if(!all(names(readout) %in% name[type != 'tte'])){
          stop('Readout is set for unknown or time-to-event endpoint(s) <',
               paste0(setdiff(names(readout), name[type != 'tte']), collapse = ', '),
               '>. ')
        }

        if(!setequal(names(readout), name[type != 'tte'])){
          stop('Readout is missing for endpoint(s) <',
               paste0(setdiff(name[type != 'tte'], names(readout)), collapse = ', '),
               '>. ')
        }
      }

      stopifnot(is.function(generator))

      # Check that the first argument of generator is "n"
      arg_names <- names(formals(generator))
      if (length(arg_names) == 0 || arg_names[1] != "n") {
        stop("The first argument of generator must be 'n'.")
      }

      n_ <- 2
      generator_ <- DynamicRNGFunction(generator,
                                       rng = deparse(substitute(generator)),
                                       var_name = name,
                                       type = type,
                                       readout = readout, ...)
      example_data <- generator_(n = n_)

      if(!is.data.frame(example_data) && !is.vector(example_data)){
        stop('generator must return a data frame (for multiple endpoints or a TTE) or a vector.')
      }

      if(is.vector(example_data)){
        example_data <- data.frame(v1 = example_data) %>%
          rename(!!name := .data$v1)
      }

      if(!all(name %in% colnames(example_data))){
        stop('Generator must return data for every endpoints in \'name\'. \n',
             'Endpoint being specified in \'name\' are: \n<',
             paste0(name, collapse = ', '), '>. \n',
             'Columns returned by generator are: \n<',
             paste0(colnames(example_data), collapse = ', '), '>. \n',
             'Consider renaming the output from your custom generator. ')
      }

      if(nrow(example_data) != n_){
        stop('\'n\' in generator does not work correctly.')
      }
    }
  )
)

#' Generate Correlated PFS, OS and Objective Response
#'
#' @description
#' Generate correlated PFS, OS and objective response using the
#' four-states model. It can be used as custom generator of \code{endpoint()}.
#'
#' @param n integer. Number of observations.
#' @param transition_probability a 4x4 matrix defining transition probabilities
#' between stable (initial state, 1), response (2), progression (3) and
#' death (absorbing, 4).
#' @param duration integer. Duration of trial. Set it to a sufficient large
#' integer in practice to cover the duration of the trial (potentially be
#' extended).
#' @param death_name column name of OS in returned data frame. It must be
#' consistent with \code{name} in the function \code{endpoint()}.
#' @param progression_name column name of PFS in returned data frame. It must be
#' consistent with \code{name} in the function \code{endpoint()}.
#' @param response_name column name of objective response in returned data frame. It must be
#' consistent with \code{name} in the function \code{endpoint()}.
#'
#' @returns
#' A data frame of \code{n} rows and 6 columns (response, progression,
#' death, and their event indicators with 1 means event and 0 means censored
#' at duration). The column names are
#' \code{<death_name>}, \code{<death_name>_event}, \code{<progression_name>},
#' \code{<progression_name>_event}, \code{<response_name>} and
#' \code{<response_name>_event}.
#'
#' Note that it returns time-to-response for each patients with status of
#' censoring at pre-set duration. If a binary indicator of response at
#' a time point is needed as an endpoint, we may write a wrapper function to
#' convert the column \code{<response_name>} to binary and remove the column
#' \code{<response_name>_event} from return value.
#'
#' @export
#'
#' @examples
#' m <- matrix(c(0.99, 0.0035, 0.0055, 0.0010,
#'                  0, 0.9900, 0.0052, 0.0048,
#'                  0,      0, 0.9960, 0.0040,
#'                  0,      0,      0,      1),
#'              nrow = 4, byrow = TRUE)
#'
#' ## use as function (if you don't use TrialSimulator for simulation)
#'
#' dat <- CorrelatedPfsAndOs4(1e4, m, 365 * 3)
#'
#' ## use as generator (if you use TrialSimulator for simulation)
#'
#' ep <- endpoint(name = c('pfs', 'os', 'or'),
#'                type = c('tte', 'tte', 'tte'), ## OR is TTE, not binary
#'                generator = CorrelatedPfsAndOs4,
#'                transition_probability = m,
#'                duration = 365 * 3,
#'                death_name = 'os', ## rename output from generator to match with "name"
#'                progression_name = 'pfs',
#'                response_name = 'or')
#'
#' ep # run it in console to see summary report
#'
CorrelatedPfsAndOs4 <- function(n, transition_probability, duration,
                                death_name = 'death',
                                progression_name = 'progression',
                                response_name = 'response') {

  if(!is.matrix(transition_probability) ||
     !all(dim(transition_probability) == c(4, 4))) {
    stop("transition_probability must be a 4x4 matrix.")
  }

  if(any(transition_probability < 0)) {
    stop("All elements of transition_probability must be non-negative.")
  }

  row_sums <- rowSums(transition_probability)
  if(any(abs(row_sums - 1) > 1e-8)) {
    stop("Each row of transition_probability must sum to 1.")
  }

  if(any(lower.tri(transition_probability) & transition_probability != 0)) {
    stop("transition_probability must have zeros in lower triangle (M[i, j] = 0 for i > j).")
  }

  simulate_one_patient <- function(){

    first_visit <- c(1, rep(NA_integer_, 3))

    next_state_from_stable <- sample(1:4, size = duration, replace = TRUE, prob = transition_probability[1, ])

    day <- which(next_state_from_stable != 1 &
                   seq_along(next_state_from_stable) > 1)[1]
    if(is.na(day)){ ## stay in stable state

      return(first_visit[-1])
    }

    second_state <- next_state_from_stable[day]
    first_visit[second_state] <- day

    if(second_state == 4){ ## death
      if(is.na(first_visit[3])){
        first_visit[3] <- day
      }


      return(first_visit[-1])
    }

    third_states <- sample(1:4, size = duration, replace = TRUE, prob = transition_probability[second_state, ])

    day <- which(!(third_states %in% c(1, second_state)) &
                   seq_along(third_states) > day)[1]
    if(is.na(day)){ ## stay in second_state

      return(first_visit[-1])
    }

    third_state <- third_states[day]
    first_visit[third_state] <- day

    if(third_state == 4){ ## death
      if(is.na(first_visit[3])){
        first_visit[3] <- day
      }

      return(first_visit[-1])
    }

    fourth_states <- sample(1:4, size = duration, replace = TRUE, prob = transition_probability[third_state, ])

    day <- which(!(fourth_states %in% c(1, second_state, third_state)) &
                   seq_along(fourth_states) > day)[1]
    if(is.na(day)){ ## stay in third_state

      return(first_visit[-1])
    }

    fourth_state <- fourth_states[day]
    stopifnot(fourth_state == 4)
    first_visit[fourth_state] <- day


    return(first_visit[-1])
  }

  dat <- replicate(n, simulate_one_patient())

  dat <- as.data.frame(t(dat))
  colnames(dat) <- c('response', 'progression', 'death')

  stopifnot(with(dat, all(is.na(death) | death >= progression)))
  stopifnot(with(dat, all(is.na(progression) |
                            is.na(response) |
                            progression > response)))

  event <- dat
  event[is.na(dat)] <- 0
  event[!is.na(dat)] <- 1
  names(event) <- paste0(names(event), '_event')

  dat[is.na(dat)] <- duration

  dat <- cbind(dat, event)

  dat <- dat %>%
    rename(!!paste0(response_name, '_event') := .data$response_event) %>%
    rename(!!response_name := .data$response) %>%
    rename(!!paste0(progression_name, '_event') := .data$progression_event) %>%
    rename(!!progression_name := .data$progression) %>%
    rename(!!paste0(death_name, '_event') := .data$death_event) %>%
    rename(!!death_name := .data$death)

  dat

}

#' Define an Arm
#'
#' @description
#' Define an arm in a trial. This is a user-friendly wrapper for
#' the class constructor \code{Arms$new()}. Users who are not familiar with
#' the concept of classes may consider using this wrapper directly.
#'
#' @param name character. Name of arm, which is the arm's label in generated
#' trial data, i.e., the one retrieved by calling \code{Trials$get_locked_data()}
#' in action functions.
#' @param ... subset condition that is compatible with \code{dplyr::filter}.
#' This can be used to specify inclusion criteria of an arm.
#' By default it is not specified, i.e. all data generated by the generator
#' will be used as trial data. More than one conditions can be
#' specified in \code{...}.
#'
#' @examples
#'
#' risk <- data.frame(
#'   end_time = c(1, 10, 26.0, 52.0),
#'   piecewise_risk = c(1, 1.01, 0.381, 0.150) * exp(-3.01)
#' )
#'
#' pfs <- endpoint(name = 'pfs', type='tte',
#' generator = PiecewiseConstantExponentialRNG,
#' risk = risk, endpoint_name = 'pfs')
#'
#' orr <- endpoint(
#'   name = 'orr', type = 'non-tte',
#'   readout = c(orr = 2), generator = rbinom,
#'   size = 1, prob = .4)
#'
#' placebo <- arm(name = 'pbo')
#'
#' placebo$add_endpoints(pfs, orr)
#'
#' ## try to generate some data from the arm
#' ## it is NOT a recommended way to use the package in simulation
#' head(placebo$get_endpoints()[[1]]$get_generator()(n = 1e3))
#'
#' ## get name of endpoints in the arm
#' ## for illustration only, NOT recommended
#' placebo$get_endpoints()[[2]]$get_name()
#'
#' ## run it in console to get summary report
#' ## It is the recommended way to view an arm
#' placebo
#'
#' @export
arm <- function(name, ...){

  Arms$new(name = name, ...)

}

#' Summary of Milestone Time from Simulated Trials
#'
#' @param output a data frame.
#' It assumes that triggering time of milestones
#' are store in columns \code{milestone_time_<...>}. It can be a data frame
#' returned by \code{controller$get_output()}, or row-binded from multiple
#' data frames returned by \code{controller$get_output()} (e.g., users may
#' run simulation under the \code{targets} framework).
#'
#' @returns
#' A data frame of class \code{milestone_time_summary}. It comes with a
#' \code{plot} method for visualization.
#'
#'
#' @examples
#'
#' # a minimum, meaningful, and executable example,
#' # where a randomized trial with two arms is simulated and analyzed.
#'
#' control <- arm(name = 'control arm')
#' active <- arm(name = 'active arm')
#'
#' pfs_in_control <- endpoint(name = 'PFS', type = 'tte', generator = rexp, rate = log(2) / 5)
#' control$add_endpoints(pfs_in_control)
#'
#' pfs_in_active <- endpoint(name = 'PFS', type = 'tte', generator = rexp, rate = log(2) / 6)
#' active$add_endpoints(pfs_in_active)
#'
#' accrual_rate <- data.frame(end_time = c(10, Inf), piecewise_rate = c(30, 50))
#' trial <- trial(name = 'trial',
#'                n_patients = 1000,
#'                duration = 40,
#'                enroller = StaggeredRecruiter,
#'                accrual_rate = accrual_rate,
#'                dropout = rweibull, shape = 2, scale = 38,
#'                silent = TRUE)
#'
#' trial$add_arms(sample_ratio = c(1, 1), control, active)
#'
#' action_at_final <- function(trial){
#'   locked_data <- trial$get_locked_data('final analysis')
#'   fitLogrank(Surv(PFS, PFS_event) ~ arm, placebo = 'control arm',
#'              data = locked_data, alternative = 'less')
#'   invisible(NULL)
#' }
#'
#' final <- milestone(name = 'final analysis',
#'                    action = action_at_final,
#'                    when = eventNumber(endpoint = 'PFS', n = 300))
#'
#' listener <- listener(silent = TRUE)
#' listener$add_milestones(final)
#'
#' controller <- controller(trial, listener)
#' controller$run(n = 10, plot_event = FALSE, silent = TRUE)
#'
#' output <- controller$get_output()
#' time <- summarizeMilestoneTime(output)
#' time
#'
#' plot(time)
#'
#'
#' @export
#'
summarizeMilestoneTime <- function(output){

  stopifnot(is.data.frame(output))
  cols <- grep('^milestone_time_<.*>', names(output), value = TRUE)

  if(length(cols) == 0){
    stop('No columns of milestone time can be found. ',
         'This function looks for columns milestone_time_<...>. ')
  }

  milestone_name <- sub('^milestone_time_<(.+)>$', '\\1', cols)

  res <- lapply(seq_along(cols),
                function(i){
                  vals <- output[[cols[i]]]
                  data.frame(
                    milestone = milestone_name[i],
                    mean = mean(vals, na.rm = TRUE),
                    median = median(vals, na.rm = TRUE),
                    sd = sd(vals, na.rm = TRUE),
                    n = sum(!is.na(vals))
                  )
                }) %>%
    bind_rows()

  class(res) <- c('milestone_time_summary', class(res))
  time <- output[cols]
  colnames(time) <- milestone_name
  attr(res, 'time') <- time
  res
}

#' Plot Triggering Time of Milestones in Simulated Trials
#'
#' @param x an object returned by \code{summarizeMilestoneTime()}.
#' @param ... currently not supported.
#'
#' @export
#'
plot.milestone_time_summary <- function(x, ...){

  df <- attr(x, 'time') %>% stack()

  p <- ggplot(df, aes(x = .data$ind, y = .data$values)) +
    geom_boxplot(fill = 'lightgreen') +
    labs(title = 'Distribution of Milestone Time',
         y = 'Time',
         x = 'Milestone') +
    theme_minimal() +
    theme(axis.text.x = element_text(angle = 45, hjust = 1))

  print(p)

}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/wrapper/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(dplyr)
library(TrialSimulator)

## ----ioeajfls-----------------------------------------------------------------
## time-to-event endpoint
pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = .07)
## continuous endpoint
cep <- endpoint(name = 'cep', type = 'non-tte', 
                readout = c(cep = 0), generator = rnorm)
## binary endpoint
bep <- endpoint(name = 'bep', type = 'non-tte', 
                readout = c(bep = 0), generator = rbinom, size = 1, prob = .1)

## biomarker
bm <- endpoint(name = 'biomarker', type = 'non-tte', 
               readout = c(biomarker = 0), generator = rbinom, 
               size = 1, prob = .7)

## covariate
covar <- endpoint(name = 'x', type = 'non-tte', 
                  readout = c(x = 0), generator = rnorm)

pbo <- arm(name = 'pbo')
pbo$add_endpoints(pfs, cep, bep, bm, covar)

## ----eualgha, echo=FALSE------------------------------------------------------
## time-to-event endpoint
pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = .06)
## continuous endpoint
cep <- endpoint(name = 'cep', type = 'non-tte', 
                readout = c(cep = 0), generator = rnorm, mean = 1.2)
## binary endpoint
bep <- endpoint(name = 'bep', type = 'non-tte', 
                readout = c(bep = 0), generator = rbinom, size = 1, prob = .2)

## biomarker
bm <- endpoint(name = 'biomarker', type = 'non-tte', 
               readout = c(biomarker = 0), generator = rbinom, 
               size = 1, prob = .7)

## covariate
covar <- endpoint(name = 'x', type = 'non-tte', 
                  readout = c(x = 0), generator = rnorm)

low <- arm(name = 'low')
low$add_endpoints(pfs, cep, bep, bm, covar)

## time-to-event endpoint
pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = .04)
## continuous endpoint
cep <- endpoint(name = 'cep', type = 'non-tte', 
                readout = c(cep = 0), generator = rnorm, mean = 1.3)
## binary endpoint
bep <- endpoint(name = 'bep', type = 'non-tte', 
                readout = c(bep = 0), generator = rbinom, size = 1, prob = .35)

## biomarker
bm <- endpoint(name = 'biomarker', type = 'non-tte', 
               readout = c(biomarker = 0), generator = rbinom, 
               size = 1, prob = .7)

## covariate
covar <- endpoint(name = 'x', type = 'non-tte', 
                  readout = c(x = 0), generator = rnorm)

high <- arm(name = 'high')
high$add_endpoints(pfs, cep, bep, bm, covar)

accrual_rate <- data.frame(end_time = c(10, Inf),
                           piecewise_rate = c(30, 50))
trial <- trial(
  name = 'Trial-3415', n_patients = 300,
  seed = 1727811904, duration = 1000,
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
  dropout = rexp, rate = -log(1 - 0.1)/18, ## 10% by month 18
  silent = TRUE
)

trial$add_arms(sample_ratio = c(1, 1, 1), pbo, low, high)

final <- milestone(name = 'final', action = doNothing, when = calendarTime(1000))

listener <- listener()
listener$add_milestones(final)

controller <- controller(trial, listener)

## ----ieoajf-------------------------------------------------------------------
controller$run(n = 1, plot_event = FALSE, silent = TRUE)
locked_data <- trial$get_locked_data('final')
head(locked_data)

table(locked_data$arm)

## ----ghkaljf------------------------------------------------------------------
## adjust for covariate x
fitCoxph(Surv(pfs, pfs_event) ~ arm + x, placebo = 'pbo', 
         data = locked_data, alternative = 'less', 
         scale = 'hazard ratio')

fitLogrank(Surv(pfs, pfs_event) ~ arm, placebo = 'pbo', 
           data = locked_data, alternative = 'less')

## more details
fitLogrank(Surv(pfs, pfs_event) ~ arm, placebo = 'pbo', 
           data = locked_data, alternative = 'less', tidy = FALSE)

## with strata
fitLogrank(Surv(pfs, pfs_event) ~ arm + strata(biomarker), placebo = 'pbo', 
           data = locked_data, alternative = 'less')

## analyze a subset
fitCoxph(Surv(pfs, pfs_event) ~ arm + strata(biomarker), placebo = 'pbo', 
         data = locked_data, alternative = 'less', 
         scale = 'log hazard ratio', 
         x > -2 & x < 3) ## define a subset

## ----saljfh-------------------------------------------------------------------
## ATE accounting for covariate x
fitLinear(cep ~ arm * x, placebo = 'pbo', 
          data = locked_data, alternative = 'greater')

## marginal model
fitLinear(cep ~ arm, placebo = 'pbo', 
          data = locked_data, alternative = 'greater')

## analyze a sub-group
fitLinear(cep ~ arm, placebo = 'pbo', 
          data = locked_data, alternative = 'greater', 
          biomarker == 1) ## define the subgroup

## ----pqeir--------------------------------------------------------------------
## compute regression coefficient of arm
fitLogistic(bep ~ arm * x + biomarker, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'coefficient')

## compute odds ratio (ATE)
fitLogistic(bep ~ arm + x*biomarker, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'odds ratio')

## compute risk ratio (ATE)
fitLogistic(bep ~ arm + x + biomarker, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'risk ratio')

## ----uyta---------------------------------------------------------------------
## compute risk difference (ATE)
fitLogistic(bep ~ arm + x * biomarker, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'risk difference')

## compute risk difference without covariate
fitLogistic(bep ~ arm, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'risk difference')

## analyze a sub-group
fitLogistic(bep ~ arm, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'risk difference', 
            x < 2 & biomarker != 1) ## define a subgroup

## analyze the same sub-group using the FM test,
## same estimate but different p-values
fitFarringtonManning(endpoint = 'bep', placebo = 'pbo', 
                     data = locked_data, alternative = 'greater', 
                     x < 2 & biomarker != 1)

#' Class of Regimens
#' @description
#' Create a class of regimen. A regimen defines the rules to select treatments for
#' patients switch, to determine the time of switching, and to update patients'
#' endpoint data.
#'
#' @docType class
#'
#' @export
Regimens <- R6::R6Class(
  'Regimens',

  public = list(

    #' @description
    #' initialize regimen
    #'
    #' @param what a function determining whether patients' data would be
    #' updated due to switching treatment. It takes \code{patient_data},
    #' a data frame as argument, and returns a data frame of two columns
    #' \code{patient_id} and \code{new_treatment}. Patients with \code{NA} in
    #' \code{new_treatment} will be skipped.
    #' The number of rows in the returned data frame may be smaller than the
    #' number of patients in the input data frame. This indicates that some
    #' patients' data will not be modifier.
    #' Note that the returned object will be passed into function `how()`, which
    #' is also provide by users. This argument can also be a
    #' list of functions that will be executed sequentially. No default value.
    #' @param when a function determining the time at which a patient switches
    #' to another treatment regimen, measured from the time of enrollment.
    #' It takes \code{patient_data}, a data frame as
    #' argument, and returns a data frame of two columns \code{patient_id} and
    #' \code{switch_time} (from \code{enroll_time}). No \code{NA} is allowed
    #' in \code{switch_time} and the number of rows in the returned data frame
    #' must equal the number of rows in \code{patient_data}, i.e., switching
    #' time must be specified to every patients.
    #' Note that the returned object will be passed into function `how()`, which
    #' is also provided by users. This argument can also be a
    #' list of functions that will be executed sequentially. No default value.
    #' @param how a function updating patients' data after treatment switching.
    #' Only modified columns and \code{patient_id} are returned. A cell will
    #' be omitted if \code{NA}, meaning no change to that patient for the endpoint
    #' or other variables. Equivalently, users can also fill the cell with
    #' its original value. This argument can also be a list of functions that
    #' will be executed sequentially. No default value.
    #' @param ... (optional) named arguments routed to one or more of
    #' \code{what}, \code{when}, and \code{how}.
    initialize =
      function(what, when, how, ...){

        dots <- list(...)
        if(length(dots) && (is.null(names(dots)) || any(names(dots) == ''))){
          stop('All extra arguments to regimen(...) must be named; ',
               'they are routed to `what`, `when`, and/or `how`.')
        }

        private$validate_arguments(what, when, how)

        if(is.function(what)){ ## all validation have been done in validate_arguments()
          what <- list(what)
          when <- list(when)
          how <- list(how)
        }

        private$treatment_allocator <- what
        private$time_selector <- when
        private$data_modifier <- how

        private$what_args <- private$route_args(what, dots, 'what')
        private$when_args <- private$route_args(when, dots, 'when')
        private$how_args  <- private$route_args(how,  dots, 'how')

        ## Every dot arg must match at least one parameter across all functions
        all_consumed <- unique(unlist(lapply(
          c(what, when, how),
          function(f){
            fnames <- names(formals(f))
            if('...' %in% fnames) names(dots) else intersect(names(dots), fnames)
          }
        )))
        unmatched <- setdiff(names(dots), all_consumed)
        if(length(unmatched)){
          stop('Unknown argument(s) <',
               paste0(unmatched, collapse = ', '),
               '> not found in any of `what`, `when`, or `how` functions. ')
        }

      },

    #' @description
    #' return number of treatment allocators for regimen
    get_number_treatment_allocator = function(){
      length(private$treatment_allocator)
    },

    #' @description
    #' return user-defined new treatment for a patient
    #' @param index integer. Index of allocator. Return all allocators if \code{NULL}.
    get_treatment_allocator = function(index = NULL){
      if(is.null(index)){
        return(private$treatment_allocator)
      }else{
        stopifnot(is.wholenumber(index) && index > 0)
        stopifnot(length(index) == 1)
        if(index > self$get_number_treatment_allocator()){
          stop('There are only <', self$get_number_treatment_allocator(),
               '> treatment allocators, and cannot request for the ',
               index, 'th. ')
        }
        return(private$treatment_allocator[[index]])
      }
    },

    #' @description
    #' return number of time selector for regimen
    get_number_time_selector = function(){
      length(private$time_selector)
    },

    #' @description
    #' return user-defined time selector
    #' @param index integer. Index of selector. Return all selectors if \code{NULL}.
    get_time_selector = function(index = NULL){
      if(is.null(index)){
        return(private$time_selector)
      }else{
        stopifnot(is.wholenumber(index) && index > 0)
        stopifnot(length(index) == 1)
        if(index > self$get_number_time_selector()){
          stop('There are only <', self$get_number_time_selector(),
               '> time selectors, and cannot request for the ',
               index, 'th. ')
        }
        return(private$time_selector[[index]])
      }
    },

    #' @description
    #' return number of data modifier for regimen
    get_number_data_modifier = function(){
      length(private$data_modifier)
    },

    #' @description
    #' return user-defined endpoint data modifier
    #' @param index integer. Index of selector. Return all modifiers if \code{NULL}.
    get_data_modifier = function(index = NULL){
      if(is.null(index)){
        return(private$data_modifier)
      }else{
        stopifnot(is.wholenumber(index) && index > 0)
        stopifnot(length(index) == 1)
        if(index > self$get_number_data_modifier()){
          stop('There are only <', self$get_number_data_modifier(),
               '> data modifiers, and cannot request for the ',
               index, 'th. ')
        }
        return(private$data_modifier[[index]])
      }
    },

    #' @description
    #' return pre-bound arguments for the i-th treatment allocator
    #' @param index integer.
    get_treatment_allocator_args = function(index){
      private$what_args[[index]]
    },

    #' @description
    #' return pre-bound arguments for the i-th time selector
    #' @param index integer.
    get_time_selector_args = function(index){
      private$when_args[[index]]
    },

    #' @description
    #' return pre-bound arguments for the i-th data modifier
    #' @param index integer.
    get_data_modifier_args = function(index){
      private$how_args[[index]]
    }
  ),

  private = list(
    treatment_allocator = NULL,
    time_selector = NULL,
    data_modifier = NULL,
    what_args = NULL,
    when_args = NULL,
    how_args  = NULL,

    validate_arguments = function(what, when, how){

      isValidFunction <- function(func, func_type, func_name){
        if(!is.function(func)){
          stop(func_name, ' is not a function. ')
        }

        if(length(formals(func)) == 0 || names(formals(func))[1] != 'patient_data'){
          stop('The <', func_type, '> function <', func_name, '> passed to regimen has arguments <',
               paste0(names(formals(func)), collapse = ', '), '>. \n',
               'The TrialSimulator convention requires that this function\'s first argument must be `patient_data`.')
        }
      }

      stopifnot(is.function(what) || is.list(what))
      stopifnot(is.function(when) || is.list(when))
      stopifnot(is.function(how) || is.list(how))

      if(is.function(what)){
        stopifnot(is.function(when) && is.function(how))
        what <- list(what)
        when <- list(when)
        how <- list(how)
      }else{
        stopifnot(is.list(when) && is.list(how))
        stopifnot(length(what) == length(when))
        stopifnot(length(what) == length(how))
      }

      for(i in seq_along(what)){
        isValidFunction(what[[i]], 'what', deparse(substitute(what[[i]])))
        isValidFunction(when[[i]], 'when', deparse(substitute(when))[[i]])
        isValidFunction(how[[i]], 'how', deparse(substitute(how))[[i]])
      }

    },

    ## For each function in func_list, extract matching args from dots.
    ## Validates that required args (no default) are present in dots.
    route_args = function(func_list, dots, slot_name){
      lapply(func_list, function(f){
        fnames <- names(formals(f))
        extra  <- setdiff(fnames, c('patient_data', '...'))

        if('...' %in% fnames){
          matched <- dots
        }else{
          idx <- intersect(names(dots), extra)
          matched <- if(length(idx)) dots[idx] else list()
        }

        ## check required args (no default value)
        req_mask <- vapply(formals(f)[extra],
                           function(x) identical(x, quote(expr=)), logical(1))
        required <- extra[req_mask]
        missing_req <- setdiff(required, names(matched))
        if(length(missing_req)){
          stop('Missing required argument(s) <',
               paste0(missing_req, collapse = ', '),
               '> for a `', slot_name, '` function in regimen(). ')
        }

        matched
      })
    }
  )

)

#' Class of Trial
#' @description
#' Create a class of trial.
#'
#' Public methods in this R6 class are used in developing
#' this package. Thus, we have to export the whole R6 class which exposures all
#' public methods. However, only the public methods in the list below are
#' useful to end users.
#'
#' \itemize{
#' \item \code{$set_duration()} set duration of a trial. This function can be
#' used to extend duration under adaptive designs.
#' \item \code{$resize()} set maximum sample size of a trial. This function can
#' be used to increase sample size under adaptive designs (e.g., sample size
#' reassessment).
#' \item \code{$remove_arms()} drop arms from a trial. This function can be
#' used in adaptive designs, e.g., dose selection, enrichment design, etc.
#' \item \code{$update_sample_ratio()} change sample ratio of arm. This function
#' can be used under adaptive designs, e.g., response-adaptive design, etc.
#' \item \code{$update_generator()} change endpoint generator of arm. This
#' function can be used in enrichment design.
#' \item \code{$add_arms()} add arms to a trial. This function is used to add
#' arms to a newly defined trial, or add arms under adaptive design, e.g.,
#' dose-ranging, etc.
#' \item \code{$add_regimen()} register a \code{regimen} object to a trial.
#' Must be called before \code{$add_arms()}.
#' \item \code{$get_locked_data()} request for data snapshot at a milestone.
#' Calling this function is recommended as the first action in any action
#' function as long as trial data is needed in statistical analysis or decision
#' making.
#' \item \code{$save()} save intermediate result for simulation summary.
#' Results across multiple replicates of simulation are saved, which can be
#' retrieved by calling \code{get_output()} anytime.
#' \item \code{$bind()} row bind and save intermediate results across
#' milestones if those results are data frames of similar formats. The life
#' cycle of the save results is within a single replicate of simulation and
#' is reset to NULL in next simulated trial. Saved results
#' can be retrieved by calling \code{get()} anytime.
#' \item \code{$save_custom_data()} save intermediate results of any format.
#' The life cycle of the saved result is within a single replicate of simulation
#' and is reset to NULL in next simulated trial. Saved results can be retrieved
#' by calling \code{get()} anytime.
#' \item \code{$get()} retrieve intermediate results saved by calling functions
#' \code{save_custom_data()} or \code{bind()}.
#' \item \code{$get_output()} retrieve intermediate results saved by calling
#' function \code{save()}.
#' \item \code{$dunnettTest()} perform Dunnett's test.
#' \item \code{$closedTest()} perform combination test based on Dunnett's test.
#' }
#'
#' @docType class
#'
#' @examples
#' # Instead of using Trials$new, please use trial(), a user-friendly
#' # wrapper. See examples in ?trial.
#'
#' @export
Trials <- R6::R6Class(
  'Trials',

  public = list(

    #' @description
    #' initialize a trial
    #' @param name character. Name of trial. Usually, hmm..., useless.
    #' @param n_patients integer. Maximum (and initial) number of patients
    #' could be enrolled when planning the trial. It can be altered adaptively
    #' during a trial.
    #' @param duration Numeric. Trial duration. It can be altered adaptively
    #' during a trial.
    #' @param description character. Optional for description of the trial. By
    #' default it is set to be trial's \code{name}. Usually useless.
    #' @param seed random seed. If \code{NULL}, seed is set for each simulated
    #' trial automatically and saved in output. It can be retrieved in the
    #' \code{seed} column in \code{$get_output()}. Setting it to be \code{NULL}
    #' is recommended. For debugging, set it to a specific integer.
    #' @param enroller a function returning a vector enrollment time for
    #' patients. Its first argument \code{n} is the number of enrolled patients.
    #' Set it to \code{StaggeredRecruiter} can handle most of the use cases.
    #' See \code{?TrialSimulator::StaggeredRecruiter} for more information.
    #' @param dropout a function returning a vector of dropout time for patients.
    #' It can be any random number generator with first argument \code{n},
    #' the number of enrolled patients. Usually \code{rexp} if dropout rate
    #' is set at a single time point, or \code{rweibull} if dropout rates are
    #' set at two time points. See \code{?TrialSimulator::weibullDropout}.
    #' @param stratification_factors character. Names of baseline characteristics
    #' to define stratums in stratified permuted block randomization.
    #' Stratification factors must be defined in \code{endpoint()} with
    #' \code{readout = 0}. As a natural assumption for randomized trial,
    #' \code{TrialSimulator} assumes that the baseline
    #' characteristics share the same distribution across arms, but endpoints
    #' can have same or different distributions given baseline characteristics.
    #' \code{NULL} by default, i.e., unstratified permuted block randomization is
    #' executed.
    #' @param silent logical. \code{TRUE} to mute messages. However, warning
    #' message is still displayed.
    #' @param ... (optional) arguments of \code{enroller} and \code{dropout}.
    initialize =
      function(
        name,
        n_patients,
        duration,
        description = name,
        seed = NULL,
        enroller,
        dropout = NULL,
        stratification_factors = NULL,
        silent = FALSE,
        ...
      ){

        # stratification_factors will be checked again when an arm is added
        private$validate_arguments(
          name, n_patients, duration, description, seed,
          enroller, dropout, stratification_factors, silent, ...)

        private$silent <- silent

        if(is.null(seed)){
          seed <- sample(.Machine$integer.max, 1)
          if(!private$silent){
            message('Seed is not specified. TrialSimulator sets it to ', seed)
          }
        }

        private$arms <- list()
        private$name <- name
        private$description <- description
        private$n_patients <- n_patients
        private$duration <- duration
        private$now <- 0
        private$trial_data <- NULL
        private$locked_data <- list()
        private$output <- data.frame(trial = self$get_name())
        private$custom_data <- list()

        private$seed <- seed
        self$save(seed, 'seed')
        self$save('', 'error_message')

        if(is.null(dropout)){
          self$set_dropout(rconst, value = Inf)
        }else{
          self$set_dropout(dropout, ...)
        }

        self$set_enroller(enroller, ...)

        private$stratification_factors <- stratification_factors

        self$make_snapshot()

        set.seed(private$seed)
        ## Why we need enroll_time_with_redundant?
        ## This is for resize(), i.e. resizing a trial adaptively.
        ## We generate enroll time more than enough for potential patients
        ## that are more than the planned size. Currently it is 10x
        ## The enroll_time is then initialized to be the first n_patients
        ## elements in enroll_time_with_redundant. If the trial is later resized
        ## to be more than 10 * n_patients, an error message will be prompted.
        ##
        ## Sort enrollment time
        private$enroll_time_with_redundant <-
          sort(self$get_enroller()(n = n_patients * 10), decreasing = FALSE)
        private$enroll_time <- head(private$enroll_time_with_redundant, n_patients)
        private$enroll_time_with_redundant <- private$enroll_time_with_redundant[-c(1:n_patients)]

      },

    #' @description
    #' return trial data of enrolled patients at the time of this
    #' function is called
    get_trial_data = function(){
      private$trial_data
    },

    #' @description
    #' return maximum duration of a trial
    get_duration = function(){
      private$duration
    },

    #' @description
    #' set trial duration in an adaptive designed trial. All patients enrolled
    #' before resetting the duration are truncated (non-tte endpoints) or
    #' censored (tte endpoints) at the original duration. Remaining patients
    #' are re-randomized. New duration must be longer than the old one.
    #' @param duration new duration of a trial. It must be greater than the
    #' current duration.
    set_duration = function(duration){

      if(duration <= self$get_duration()){
        stop('Trial duration can only be set to be longer. <', duration,
             '> is shorter than <', self$get_duration(), '>. ')
      }

      old_duration <- self$get_duration()

      ## update the duration
      private$duration <- duration

      if(!private$silent){
        message('Trial duration is updated <', old_duration,
                '> -> <', self$get_duration(), '>. ')
      }

      ## all patients enrolled before current milestone should be censored
      ## or truncated at old duration
      self$censor_trial_data(censor_at = old_duration,
                             enrolled_before = self$get_current_time())

      ## with trial duration is extended, unenrolled patient at current time
      ## should be randomized again.
      self$roll_back()

      ## update data for unrolled patients based on new trial duration
      self$enroll_patients()

    },

    #' @description
    #' set recruitment curve when initialize a trial.
    #' @param func function to generate enrollment time. It can be built-in
    #' function like `rexp` or customized functions like `StaggeredRecruiter`.
    #' @param ... (optional) arguments for \code{func}.
    set_enroller = function(func, ...){

      # Check that the first argument of enroller is "n"
      arg_names <- names(formals(func))
      if (length(arg_names) == 0 || arg_names[1] != "n") {
        stop("The first argument of enroller must be 'n'.")
      }

      n_ <- 2

      suppressWarnings(
        enroller_ <- DynamicRNGFunction(
          func, rng = deparse(substitute(func)), simplify = TRUE, ...)
      )

      example_data <- enroller_(n = n_)
      if(!is.vector(example_data)){
        stop('enroller must return a vector.')
      }

      if(length(example_data) != n_){
        stop('\'n\' in enroller does not work correctly.')
      }

      private$enroller <- enroller_
    },

    #' @description
    #' get function of recruitment curve
    get_enroller = function(){
      private$enroller
    },

    #' @description
    #' set distribution of drop out time. This can be done when initialize a
    #' trial, or when updating a trial in adaptive design.
    #' @param func function to generate dropout time. It can be built-in
    #' function like `rexp` or customized functions.
    #' @param ... (optional) arguments for \code{func}.
    set_dropout = function(func, ...){

      arg_names <- names(formals(func))
      if (length(arg_names) == 0 || arg_names[1] != "n") {
        stop("The first argument of random number generator for dropout time must be 'n'.")
      }

      suppressWarnings(
        dropout_ <- DynamicRNGFunction(func, rng = deparse(substitute(func)),
                                       simplify = TRUE, ...)
      )

      n_ <- 2
      example_data <- dropout_(n = n_)
      if(!is.vector(example_data)){
        stop('dropout must return a vector.')
      }

      if(length(example_data) != n_){
        stop('\'n\' in dropout does not work correctly.')
      }

      private$dropout <- dropout_

    },

    #' @description
    #' get generator of dropout time
    get_dropout = function(){
      private$dropout
    },

    #' @description
    #' roll back data to current time of trial. By doing so,
    #' \code{Trial$trial_data} will be cut at current time, and data after then
    #' are deleted. However, \code{Trial$enroll_time} after current time are
    #' kept unchanged because that is planned enrollment curve.
    roll_back = function(){

      current_time <- self$get_current_time()

      td <- self$get_trial_data()
      future_mask <- td$enroll_time > current_time
      private$enroll_time <- sort(td$enroll_time[future_mask])

      ## self$get_number_patients(), i.e. private$n_patients may have been
      ## updated by calling Trials$resize() to resize the trial, in this case
      ## we need to add more enroll time into private$enroll_time from
      ## private$enroll_time_with_redundant.
      ##
      if(nrow(self$get_trial_data()) != self$get_number_patients()){

        number_new_patients <- self$get_number_patients() - nrow(self$get_trial_data())
        if(length(private$enroll_time_with_redundant) < number_new_patients){
          stop('Debug roll_back(). ',
               'No sufficient redundant enroll time can be used when resizing the trial. ')
        }

        if(length(private$enroll_time) > 0 &&
           min(private$enroll_time_with_redundant) < max(private$enroll_time)){
          stop('Debug roll_back() if Trials$resize() is called. ',
               'Unlikely to trigger this error. ')
        }

        private$enroll_time <-
          c(private$enroll_time,
            head(private$enroll_time_with_redundant, number_new_patients))

        private$enroll_time_with_redundant <-
          private$enroll_time_with_redundant[-c(1:number_new_patients)]
      }

      past_mask <- td$enroll_time <= current_time
      past_td   <- td[past_mask, , drop = FALSE]
      private$trial_data <- past_td[order(past_td$enroll_time), , drop = FALSE]

      if(!private$silent){
        message('Trial data is rolling back to time = ', current_time, '. \n',
                'Randomization will be carried out again for unenrolled patients. \n')
      }

    },

    #' @description
    #' remove arms from a trial. \code{enroll_patients()} will be called
    #' at the end of this function to enroll all remaining patients after
    #' \code{Trials$get_current_time()}, i.e. no more unenrolled patients
    #' could be randomized to removed arms. This function may be used with
    #' futility analysis, dose selection, enrichment analysis (sub-population)
    #' or interim analysis (early stop for efficacy).
    #'
    #' Note that this function should only be called within action functions.
    #' It is users' responsibility to ensure it and \code{TrialSimulator} has
    #' no way to track this.
    #' In addition, data of the removed arms are censored or truncated by
    #' the time of arm removal.
    #' @param arms_name character vector. Name of arms to be removed.
    remove_arms = function(arms_name){
      stopifnot(is.character(arms_name))
      stopifnot(all(arms_name %in% self$get_arms_name()))

      private$sample_ratio <-
        private$sample_ratio[!(names(private$sample_ratio) %in% arms_name)]

      for(arm_name in arms_name){
        private$arms[[arm_name]] <- NULL

        ## I have to assume that the function Trials$remove_arms() can only be called in an action function.
        ## Thus, the time an arm is removed from the trial is the triggering time of the most recent milestone
        ## This assumption can be problematic but for now I do not have other solution.
        ## In Trials$dunnettTest(), I need to know at a given milestone whether an arm is still in the trial.
        ##
        existing_milestone_time <- self$get_milestone_time()
        if(is.null(existing_milestone_time)){
          latest_milestone_time <- 0
        }else{
          latest_milestone_time <- max(existing_milestone_time)
        }
        self$set_arm_removal_time(arm = arm_name,
                                  time = latest_milestone_time)
      }

      if(!private$silent){
        message('Arm <', paste0(arms_name, collapse = ', '), '> is removed. \n')

        message('Sample ratio is updated to be <',
                paste0(paste0(names(self$get_sample_ratio()),
                              ': ', self$get_sample_ratio()), collapse = ', '),
                '>. \n')
      }

      ## data of removed arms should be censored at milestone time
      ## so that number of events of those arms are fixed.
      ## Otherwise, number of events can possibly increase later and affect
      ## calculation of triggering condition based on event numbers.
      ## Ideally, number of events in removed arms should be flatten afterward,
      ## and can be seen through Trial$event_plot().
      self$censor_trial_data(censor_at = self$get_current_time(),
                             selected_arms = arms_name)
      ## with an arm is removed, unenrolled patient at current time should be
      ## randomized again.
      self$roll_back()

      ## update data for unrolled patients based on new arms and possibly
      ## new sample ratio.
      self$enroll_patients()


    },

    #' @description
    #' update sample ratios of arms. This could happen after an arm is added
    #' or removed. Note that we may update sample ratios of unaffected arms as
    #' well. Once sample ratio is updated, trial data should be rolled back with
    #' updated randomization queue. Data of unenrolled patients are
    #' re-sampled as well.
    #' @param arm_names character vector. Name of arms.
    #' @param sample_ratios numeric vector. New sample ratios of arms. If
    #' sample ratio is a whole number, the permuted block randomization is
    #' adopted; otherwise, \code{sample()} will be used instead, which can
    #' cause imbalance between arms by chance. However, this is fine for
    #' simulation.
    update_sample_ratio = function(arm_names, sample_ratios){

      stopifnot(is.character(arm_names))
      # stopifnot(length(arm_name) == 1)
      # stopifnot(is.numeric(sample_ratio) && all(is.wholenumber(sample_ratio)))
      stopifnot(is.numeric(sample_ratios) && all(sample_ratios > 0))
      # stopifnot(length(sample_ratio) == 1)

      for(arm_name in arm_names){
        if(is.null(private$arms[[arm_name]])){
          stop('Arm ', arm_name, ' is not in the trial. ')
        }

        if(!(arm_name %in% names(self$get_sample_ratio()))){
          stop('Sample ratio of arm ', arm_name, ' is not in the trial.')
        }
      }

      for(i in seq_along(arm_names)){
        private$sample_ratio[arm_names[i]] <- sample_ratios[i]
      }

      if(!private$silent){
        message('Sample ratio has been udpated to be <',
                paste0(paste0(names(self$get_sample_ratio()),
                              ': ', self$get_sample_ratio()), collapse = ', '),
                '>. ')
      }

      ## with sample ratio of an arm is updated, unenrolled patient at current
      ## time should be randomized again.
      self$roll_back()

      ## update data for unrolled patients based on new sample ratios.
      ## Note that if sample ratio is not whole number,
      ## the permuted block algorithm will be switched to sample() because
      ## it is not easy to specify a block with proper size automatically.
      self$enroll_patients()
    },

    #' @description
    #' update endpoint generator in an arm
    #'
    #' @param arm_name character. Name of an arm.
    #' @param endpoint_name character. A vector of endpoint names whose
    #' generator is updated.
    #' @param generator a random number generation (RNG) function.
    #' See \code{generator} of \code{endpoint()}.
    #' @param ... optional arguments for \code{generator}.
    update_generator = function(arm_name, endpoint_name, generator, ...){

      stopifnot(is.character(arm_name))
      if(length(arm_name) != 1){
        stop('Only one arm can be updated at a time. ',
             'You specified <', length(arm_name), '> arms: <',
             paste0(arm_name, collapse = ', '), '>. ')
      }

      if(!(arm_name %in% self$get_arms_name())){
        stop('The arm <', arm_name,
             '> is not in the trial or has been dropped from the trial. ')
      }

      selected_arm <- self$get_an_arm(arm_name)

      stopifnot(is.character(endpoint_name))

      unknown_endpoints <- setdiff(endpoint_name, selected_arm$get_endpoints_name())

      if(length(unknown_endpoints) > 0){
        stop('Endpoint(s) <', paste0(unknown_endpoints, collapse = ', '),
             '> are not in the arm <', selected_arm$get_name(), '>. ')
      }

      endpoint_name_ <- paste0(endpoint_name, collapse = '/')
      if(!(endpoint_name_ %in% names(selected_arm$get_endpoints()))){
        stop('Some more endpoint(s) are needed to define the arm <',
             selected_arm$get_name(), '> with <',
             paste0(endpoint_name, collapse = ', '), '>. ')
      }

      stopifnot(is.function(generator))

      private$arms[[arm_name]]$update_endpoint_generator(endpoint_name, generator, ...)

      if(!private$silent){
        message('Generator of endpoints <',
                paste0(endpoint_name, collapse = ', '),
                '> in the arm <', arm_name, '> has been udpated. ')
      }

      ## with generator of an arm is updated, unenrolled patient at current
      ## time should be randomized again.
      self$roll_back()

      ## update data for unrolled patients based on new generator in arm.
      self$enroll_patients()

    },

    #' @description
    #' resize a trial with a greater sample size. This function is used to
    #' update the maximum sample size adaptively after sample size reassessment.
    #' Note that this function should be called within action functions.
    #' It is users' responsibility to ensure it and \code{TrialSimulator} has
    #' no way to track this.
    #'
    #' @param n_patients integer. Number of maximum sample size of a trial.
    resize = function(n_patients){
      if(n_patients <= self$get_number_patients()){
        stop('TrialSimulator can only increase sample size of a trial. ',
             'When calling Trials$resize(n_patients), use n_patients > ',
             self$get_number_patients(), '. ')
      }

      private$n_patients <- n_patients

      self$roll_back()

      self$enroll_patients()

    },

    #' @description
    #' add one or more arms to the trial. \code{enroll_patients()} will be
    #' called at the end to enroll all remaining patients in
    #' \code{private$randomization_queue}. This function can be used in two
    #' scenarios:
    #' (1) add arms right after a trial is created (i.e., \code{Trials$new(...)}).
    #' \code{sample_ratio} and arms added through \code{...} should be of same
    #' length;
    #' (2) add arms to a trial already with arm(s).
    #'
    #' Note that this function should only be called within action functions.
    #' It is users' responsibility to ensure it and \code{TrialSimulator} has
    #' no way to track this.
    #'
    #' @param sample_ratio integer vector. Sample ratio for permuted block
    #' randomization. It will be appended to existing sample ratio in the trial.
    #' @param ... one or more objects returned from \code{arm()}.
    #' Randomization is carried out with updated
    #' sample ratio of newly added arm. It rolls back all patients after
    #' \code{Trials$get_current_time()}, i.e. redo randomization for those
    #' patients. This can be useful to add arms one by one when creating a trial.
    #' Note that we can run \code{Trials$add_arm(sample_ratio1, arm1)} followed
    #' by \code{Trials$add_arm(sample_ratio2, arm2)}.
    #' We would expected similar result with
    #' \code{Trials$add_arms(c(sample_ratio1, sample_ratio2), arm1, arm2)}. Note
    #' that these two method won't return exactly the same trial because
    #' randomization_queue were generated twice in the first approach but only
    #' once in the second approach. But statistically, they are equivalent and
    #' of the same distribution.
    add_arms = function(sample_ratio, ...){

      # stopifnot(is.numeric(sample_ratio) && all(is.wholenumber(sample_ratio)))
      stopifnot(is.numeric(sample_ratio))

      arm_list <- list(...)
      enforce <- arm_list$enforce
      if(is.null(enforce)){
        enforce <- FALSE
      }

      ## This line ensures that enforce = TRUE is no longer needed by end users.
      ## However, specifying enforce = TRUE when calling $add_arms() is still valid,
      ## it is simply unnecessary.
      if(self$has_arm()){
        enforce = TRUE
      }

      arm_list$enforce <- NULL
      stopifnot(length(arm_list) == length(sample_ratio))

      arm_names <- NULL
      for(arm in arm_list){
        stopifnot(inherits(arm, 'Arms'))

        if(!arm$has_endpoint()){
          stop('No endpoint in the arm <', arm$get_name(), '>. ',
               'Make sure that Arm$add_endpoints() has been executed before adding this arm into the trial. ')
        }

        if(arm$get_name() %in% self$get_arms_name()){
          stop('Arm <', arm$get_name(), '> already exists in the trial. ',
               'Do you want to update it instead? \n',
               'If so you need to revise your code, ',
               'currently updating an arm is not yet supported. ')
        }

        private$arms[[arm$get_name()]] <- arm
        arm_names <- c(arm_names, arm$get_name())

        ## I have to assume that the function Trials$add_arms() can only be called in an action function.
        ## Thus, the time an arm is added to the trial is the triggering time of the most recent milestone
        ## This assumption can be problematic but for now I do not have other solution.
        ## In Trials$dunnettTest(), I need to know at a given milestone whether an arm is still in the trial.
        ##
        existing_milestone_time <- self$get_milestone_time()
        if(is.null(existing_milestone_time)){
          latest_milestone_time <- 0
        }else{
          latest_milestone_time <- max(existing_milestone_time)
        }
        self$set_arm_added_time(arm = arm$get_name(),
                                time = latest_milestone_time)

      }

      endpoint_names <- NULL
      for(arm in self$get_arms()){
        if(is.null(endpoint_names)){
          endpoint_names <- arm$get_endpoints_name()
        }else{
          if(!setequal(endpoint_names, arm$get_endpoints_name())){
            stop('Arm <', arm$get_name(), '> ',
                 'has endpoint(s) that are different from other arm(s): \n',
                 '<', paste0(arm$get_endpoints_name(), collapse = ', '),
                 '> v.s. <',
                 paste0(endpoint_names, collapse = ', '), '>. \n',
                 'You may have typo when naming the endpoints in function endpoint(). ')
          }
        }
      }

      if(!private$silent){
        message('Arm(s) <', paste0(arm_names, collapse = ', '),
                '> are added to the trial. \n')
      }

      names(sample_ratio) <- arm_names
      private$sample_ratio <- c(private$sample_ratio, sample_ratio)
      rm(sample_ratio)

      if(enforce){
        self$roll_back()
      }

      self$enroll_patients()

    },

    #' @description
    #' register regimen to a trial. The regimen consists of three functions
    #' to determine the patients who may switch to other treatment during a
    #' a trial, to determine the switching time and how to update patients'
    #' endpoint data accordingly.
    #' @param regimen an object created by \code{regimen()}.
    add_regimen = function(regimen){
      if(self$has_arm()){
        stop('Member function trial$add_regimen() must be called before trial$add_arms(). ',
             'A good practice is to call trial$add_regimen() immediately after trial() is executed. ')
      }

      stopifnot(inherits(regimen, 'Regimens'))
      private$regimen <- regimen
    },

    #' @description
    #' return registered regimen.
    get_regimen = function(){
      private$regimen
    },

    #' @description
    #' return whether a regimen is registered
    has_regimen = function(){
      !is.null(private$regimen)
    },

    #' @description
    #' return name of trial
    get_name = function(){
      private$name
    },

    #' @description
    #' return description of trial
    get_description = function(){
      private$description
    },

    #' @description
    #' return a list of arms in the trial
    get_arms = function(){
      private$arms
    },

    #' @description
    #' return arms' name of trial
    get_arms_name = function(){
      lapply(private$arms, function(arm){arm$get_name()}) %>% unlist() %>% unname()
    },

    #' @description
    #' get number of arms in the trial
    get_number_arms = function(){
      length(private$arms)
    },

    #' @description
    #' check if the trial has any arm. Return \code{TRUE} or \code{FALSE}.
    has_arm = function(){
      self$get_number_arms() > 0
    },

    #' @description
    #' return an arm
    #' @param arm_name character, name of arm to be extracted
    get_an_arm = function(arm_name){
      if(!(arm_name %in% self$get_arms_name())){
        stop(arm_name, ' is not in the trial \'', self$get_name(), '\'')
      }

      self$get_arms()[[arm_name]]
    },

    #' @description
    #' return current sample ratio of the trial. The ratio can probably change
    #' during the trial (e.g., arm is removed or added)
    #' @param arm_names character vector of arms.
    get_sample_ratio = function(arm_names = NULL){
      if(is.null(arm_names)){
        arm_names <- names(private$sample_ratio)
      }
      stopifnot(all(arm_names %in% names(private$sample_ratio)))
      private$sample_ratio[arm_names]
    },

    #' @description
    #' return number of patients when planning the trial
    get_number_patients = function(){
      private$n_patients
    },

    #' @description
    #' return number of enrolled (randomized) patients
    get_number_enrolled_patients = function(){
      if(is.null(self$get_trial_data())){
        return(0)
      }
      nrow(self$get_trial_data())
    },

    #' @description
    #' return number of unenrolled patients
    get_number_unenrolled_patients = function(){
      self$get_number_patients() - self$get_number_enrolled_patients()
    },

    #' @description
    #' return stratum queue of planned but not yet enrolled patients.
    #' This function does not update stratum_queue, just return its value
    #' for debugging purpose.
    #' @param index index to be extracted. Return all queue if \code{NULL}.
    get_stratum_queue = function(index = NULL){
      if(length(private$stratum_queue) == 0){
        private$stratum_queue <- NULL
      }

      if(!is.null(index) && is.null(private$stratum_queue)){
        stop('Cannot randomize patients from empty list. ')
      }

      if(is.null(index)){ # return all
        return(private$stratum_queue)
      }
      stopifnot(max(abs(index)) <= length(private$stratum_queue))

      private$stratum_queue[index]

    },

    #' @description
    #' return randomization queue of planned but not yet enrolled patients.
    #' This function does not update randomization_queue, just return its value
    #' for debugging purpose.
    #' @param index index to be extracted. Return all queue if \code{NULL}.
    get_randomization_queue = function(index = NULL){
      if(length(private$randomization_queue) == 0){
        private$randomization_queue <- NULL
      }

      if(!is.null(index) && is.null(private$randomization_queue)){
        stop('Cannot randomize patients from empty list. ')
      }

      if(is.null(index)){ # return all
        return(private$randomization_queue)
      }
      stopifnot(max(abs(index)) <= length(private$randomization_queue))

      private$randomization_queue[index]

    },

    #' @description
    #' return enrollment time of planned but not yet enrolled patients.
    #' This function does not update enroll_time, just return its value
    #' for debugging purpose.
    #' @param index index to extract. Return all enroll time if \code{NULL}.
    get_enroll_time = function(index = NULL){

      if(length(private$enroll_time) == 0){
        private$enroll_time <- NULL
      }

      if(!is.null(index) && is.null(private$enroll_time)){
        stop('Cannot enroll patients from empty list. ')
      }

      if(is.null(index)){ # return all
        return(private$enroll_time)
      }
      stopifnot(max(abs(index)) <= length(private$enroll_time))

      private$enroll_time[index]
    },

    #' @description
    #' assign new patients to pre-planned randomization queue at pre-specified
    #' enrollment time.
    #' @param n_patients number of new patients to be enrolled. If \code{NULL},
    #' all remaining patients in plan are enrolled. Error may be triggered if
    #' n_patients is greater than remaining patients as planned.
    enroll_patients = function(n_patients = NULL){

      if(!self$has_arm()){
        stop('No arm is added in the trial yet. Patient cannot be enrolled. ')
      }

      if(self$get_number_unenrolled_patients() == 0){
        if(!private$silent){
          message('Maximum planned sample size has been reached. No more patient to be enrolled. ')
        }
        return(invisible(NULL))
      }

      if(is.null(n_patients)){
        n_patients <- self$get_number_unenrolled_patients()
      }

      if(n_patients > self$get_number_unenrolled_patients()){
        stop('Cannot enroll ', n_patients, ' patients for the trial. ',
             'Only ', self$get_number_unenrolled_patients(), ' left. ')
      }

      ## update randomization plan for unenrolled patients
      private$permuted_block_randomization()

      next_enroll_arms <- self$get_randomization_queue(1:n_patients)
      next_enroll_stratums <- self$get_stratum_queue(1:n_patients)

      ## update randomization_queue and stratum_queue after enrolling new patients.
      ## randomization_queue only keeps randomization queue for future patients
      ## stratum_queue only keeps stratum queue for future patients under
      ## stratified randomization. If simple randomization is used,
      ## stratum_queue is a constant vector of "all"
      private$randomization_queue <- self$get_randomization_queue(-c(1:n_patients))
      private$stratum_queue <- self$get_stratum_queue(-c(1:n_patients))

      next_enroll_time <- self$get_enroll_time(1:n_patients)
      private$enroll_time <- self$get_enroll_time(-c(1:n_patients))

      ## create patient pool
      arms_stratums_data <- list()
      arms_in_trial <- sort(unique(next_enroll_arms))
      stratums_in_trial <- sort(unique(next_enroll_stratums))

      make_stratum <- function(dat){
        if(!self$has_stratification_factors()){
          return(rep('all', nrow(dat)))
        }else{
          return(
            do.call(paste,
                    c(dat[, self$get_stratification_factors(),
                                   drop = FALSE],
                      sep = '|'))
          )
        }
      }

      for(i in seq_along(arms_in_trial)){
        arm <- arms_in_trial[i]
        n_patients_in_arm <- sum(next_enroll_arms %in% arm)
        tbl <- table(next_enroll_stratums[next_enroll_arms %in% arm])
        target_size_per_stratum <- data.frame(
          stratum     = names(tbl),
          target_size = as.integer(tbl),
          stringsAsFactors = FALSE
        )

        patient_pool <- NULL
        prop <- 1.1
        min_sample_size <- 20
        while(TRUE){
          new_sets <- self$get_an_arm(arm)$generate_data(max(ceiling(n_patients_in_arm * prop), min_sample_size))
          new_sets$stratum <- make_stratum(new_sets)
          patient_pool <- rbind(patient_pool, new_sets)

          tbl1 <- table(patient_pool$stratum)
          size_per_stratum <- data.frame(
            stratum      = names(tbl1),
            current_size = as.integer(tbl1),
            stringsAsFactors = FALSE
          )

          tmp <- merge(size_per_stratum, target_size_per_stratum, by = 'stratum', all.y = TRUE)
          tmp$current_size[is.na(tmp$current_size)] <- 0
          if(with(tmp, all(current_size >= target_size))){
            break
          }
          prop <- ifelse(any(tmp$current_size <= 10), 1, .2)
          min_sample_size <- with(tmp, max((target_size - current_size) * 3, 10))
        }

        col_idx <- which(names(patient_pool) == 'stratum')
        patient_pool <- split(
          patient_pool[, -col_idx, drop = FALSE],
          patient_pool$stratum
        )

        arms_stratums_data[[arm]] <- list()
        for(j in seq_along(stratums_in_trial)){
          stratum <- stratums_in_trial[j]
          patients_index <- which(next_enroll_arms %in% arm &
                                    next_enroll_stratums %in% stratum)

          n_patients_in_arm_stratum <- length(patients_index)
          if(n_patients_in_arm_stratum == 0){
            next
          }

          stopifnot(target_size_per_stratum$target_size[target_size_per_stratum$stratum == stratum] == n_patients_in_arm_stratum)

          arms_stratums_data[[arm]][[stratum]] <-
            data.frame(
              patient_id = self$get_number_enrolled_patients() + patients_index,
              arm = arm,
              enroll_time = next_enroll_time[patients_index],
              dropout_time = self$get_dropout()(n = n_patients_in_arm_stratum)
            )

          arms_stratums_data[[arm]][[stratum]] <-
            cbind(
              arms_stratums_data[[arm]][[stratum]],
              patient_pool[[stratum]][1:n_patients_in_arm_stratum, ]
            )

        }

        arms_stratums_data[[arm]] <- bind_rows(arms_stratums_data[[arm]])
      }

      patient_data <- do.call(rbind, arms_stratums_data)
      patient_data <- patient_data[order(patient_data$enroll_time), , drop = FALSE]
      rownames(patient_data) <- NULL

      if(self$has_regimen()){
        # message('Set regimen when there are ', self$get_number_enrolled_patients(), ' patients in the trial. ')
        ## real-time monitor to apply potential regimen switching over newly enrolled patients

        isValidOutput <- function(op, req_cols, func_name){
          if(!is.data.frame(op)){
            stop('The user-defined function ', func_name,
                 ' must return a data frame. ',
                 'Please set a breakpoint in ', func_name,
                 ' to debug it. ')
          }

          miss_cols <- setdiff(req_cols, names(op))
          if(length(miss_cols) > 0){
            stop('Column(s) <', paste0(miss_cols, collapse = ', '),
                 '> are missing in data frame returned from the user-defined function ',
                 func_name, ' for regimen. ',
                 'Please set a breakpoint in ', func_name,
                 ' to debug it. ')
          }
        }

        n_regimen <- self$get_regimen()$get_number_treatment_allocator()

        no_touch_cols <- c('patient_id', 'arm', 'enroll_time', 'dropout_time', 'regimen_trajectory')

        ## initialize trajectory; switches are appended inside the loop below
        patient_data$regimen_trajectory <- paste0(as.character(patient_data$arm), "@0")

        for(i in 1:n_regimen){

          new_treatment <- do.call(self$get_regimen()$get_treatment_allocator(i),
                                   c(list(patient_data),
                                     self$get_regimen()$get_treatment_allocator_args(i)))

          isValidOutput(new_treatment, c('patient_id', 'new_treatment'), 'what()')
          if(any(duplicated(new_treatment$patient_id))){
            stop('No duplicated patient ID is allowed in data frame returned from user-defined function what(). ',
                 'Set a breakpoint using browser() in your what() function to debug step-by-step. ')
          }
          new_treatment <- new_treatment[!is.na(new_treatment$new_treatment),
                                        c('patient_id', 'new_treatment'), drop = FALSE]

          pd_for_when <- patient_data[match(new_treatment$patient_id, patient_data$patient_id), , drop = FALSE]

          switch_time <- do.call(self$get_regimen()$get_time_selector(i),
                                  c(list(pd_for_when),
                                    self$get_regimen()$get_time_selector_args(i)))

          isValidOutput(switch_time, c('patient_id', 'switch_time'), 'when()')
          if(any(duplicated(switch_time$patient_id))){
            stop('No duplicated patient ID is allowed in data frame returned from user-defined function when(). ',
                 'Set a breakpoint using browser() in your when() function to debug step-by-step. ')
          }
          switch_time <- switch_time[!is.na(switch_time$switch_time),
                                     c('patient_id', 'switch_time'), drop = FALSE]
          if(nrow(switch_time) != nrow(new_treatment) ||
             !setequal(switch_time$patient_id, new_treatment$patient_id) ||
             any(is.na(switch_time$switch_time))){
            stop('In the user-defined function when(), ',
                 'switch_time must be specified to all patients who switch to new treatment regimen. ',
                 'NA is not allowed, too. ',
                 'Set a breakpoint using browser() in your when() function to debug step-by-step. ')
          }

          m <- match(new_treatment$patient_id, switch_time$patient_id)
          nr_pid <- new_treatment$patient_id
          nr_trt <- new_treatment$new_treatment
          nr_time <- switch_time$switch_time[m]

          ## append this round's switches to each patient's trajectory string
          idx <- match(nr_pid, patient_data$patient_id)
          patient_data$regimen_trajectory[idx] <- paste0(
            patient_data$regimen_trajectory[idx], ";", nr_trt, "@", nr_time
          )

          merged_patient_data <- patient_data[idx, , drop = FALSE]
          merged_patient_data$new_treatment <- nr_trt
          merged_patient_data$switch_time <- nr_time

          updated_data <- do.call(self$get_regimen()$get_data_modifier(i),
                                   c(list(merged_patient_data),
                                     self$get_regimen()$get_data_modifier_args(i)))
          isValidOutput(updated_data, 'patient_id', 'how()')
          if(any(duplicated(updated_data$patient_id))){
            stop('No duplicated patient ID is allowed in data frame returned from user-defined function how(). ',
                 'Set a breakpoint using browser() in your how() function to debug step-by-step. ')
          }

          touch_cols <- setdiff(names(updated_data), no_touch_cols)
          if(!all(touch_cols %in% names(patient_data))){
            stop('The columns below must not be returned from user-defined function < how() >: \n',
                 paste0(setdiff(touch_cols, names(patient_data)), collapse = ', '))
          }

          if(any(names(updated_data) %in% setdiff(no_touch_cols, 'patient_id'))){
            stop('The columns below must not be modified by user-defined function < how() >: \n',
                 paste0(intersect(names(updated_data), setdiff(no_touch_cols, 'patient_id')), collapse = ', '))
          }

          upd_idx <- match(updated_data$patient_id, patient_data$patient_id)
          for(col in touch_cols){
            new_vals <- updated_data[[col]]
            keep <- !is.na(new_vals)
            if(any(keep)) patient_data[[col]][upd_idx[keep]] <- new_vals[keep]
          }

        }

      }

      private$trial_data <- bind_rows(self$get_trial_data(), patient_data)
      ## newly updated trial data should be always censored at trial duration
      ## also, non-tte endpoints would be NA if readout time is after dropout time,
      ## and tte endpoints should be censored at dropout time.
      self$censor_trial_data()

      if(!private$silent){
        message('Data of ', n_patients,
                ' potential patients are generated for the trial with ',
                self$get_number_arms(), ' arm(s) <',
                paste0(self$get_arms_name(), collapse = ", "), '>. \n')#,
                # 'Depending on the scenarios, ',
                # 'some of those patients may be eventually enrolled \n',
                # 'and used in data lock, \n',
                # 'while some will be abandoned and re-generated ',
                # '(e.g. arm is removed or added). \n')
      }

    },

    #' @description
    #' set current time of a trial. Any data collected before could not be
    #' changed. private$now should be set after a milestone is triggered
    #' (through Milestones class, futility, interim, etc), an arm is added or
    #' removed at a milestone
    #' @param time current calendar time of a trial.
    set_current_time = function(time){
      stopifnot(time >= 0)
      attributes(time) <- NULL
      private$now <- time
    },

    #' @description
    #' return current time of a trial
    get_current_time = function(){
      private$now
    },

    #' @description
    #' count accumulative number of events (for TTE) or non-missing samples (otherwise) over
    #' calendar time (enroll time + tte for TTE, or enroll time + readout otherwise)
    #'
    #' @param arms a vector of arms' name on which the event tables are created.
    #' if \code{NULL}, all arms in the trial will be used.
    #' @param ... subset conditions compatible with \code{dplyr::filter}.
    #' Event tables will be counted on subset of trial data only.
    get_event_tables = function(arms = NULL, ...){

      if(is.null(arms)){
        arms <- self$get_arms_name()
      }

      if(!all(arms %in% c(self$get_arms_name(), names(private$.snapshot[['arms']])))){
        stop('Arm(s) <',
             paste0(setdiff(arms, self$get_arms_name()), collapse = ', '),
             '> cannot be found in the trial, debug Trial$get_event_tables. ')
      }

      trial_data <- self$get_trial_data()
      trial_data <- trial_data[trial_data$arm %in% arms, , drop = FALSE]

      ## Note: ...length() can report >0 when callers splice an empty list of
      ## quosures with !!!. Use enquos() to test for real filter expressions.
      if(length(rlang::enquos(...)) > 0L){
        trial_data <- tryCatch({
          trial_data %>% dplyr::filter(...)
        },
        error = function(e){
          self$save(e$message, 'error_message', overwrite = TRUE)
          stop('Error in filtering data for table of event count. ',
               'Please check condition in ..., ',
               'which should be compatible with dplyr::filter. ')
        })
      }

      n_rows <- nrow(trial_data)
      event_counts <- list()

      ## add event count for patient_id
      ## patient_id: sorted by enroll_time
      ord <- order(trial_data$enroll_time)
      event_counts[['patient_id']] <- data.frame(
        patient_id    = trial_data$patient_id[ord],
        arm           = trial_data$arm[ord],
        enroll_time   = trial_data$enroll_time[ord],
        calendar_time = trial_data$enroll_time[ord],
        n_events      = seq_len(n_rows),
        stringsAsFactors = FALSE
      )

      ## add event counts for time-to-event endpoints
      event_cols <- grep('_event$', names(trial_data), value = TRUE)
      for(event_col in event_cols){
        tte_col <- gsub('_event$', '', event_col)
        cal_time <- trial_data$enroll_time + trial_data[[tte_col]]
        ord      <- order(cal_time)
        event_counts[[tte_col]] <- data.frame(
          patient_id    = trial_data$patient_id[ord],
          arm           = trial_data$arm[ord],
          enroll_time   = trial_data$enroll_time[ord],
          calendar_time = cal_time[ord],
          n_events      = cumsum(trial_data[[event_col]][ord]),
          stringsAsFactors = FALSE
        )
      }

      ## add event counts for non-time-to-event endpoints
      readout_cols <- grep('_readout$', names(trial_data), value = TRUE)
      for(readout_col in readout_cols){
        ep_col <- gsub('_readout$', '', readout_col)
        valid    <- !is.na(trial_data[[ep_col]])
        td_sub   <- trial_data[valid, , drop = FALSE]
        cal_time <- td_sub$enroll_time + td_sub[[readout_col]]
        ord      <- order(cal_time)
        m        <- nrow(td_sub)
        event_counts[[ep_col]] <- data.frame(
          patient_id    = td_sub$patient_id[ord],
          arm           = td_sub$arm[ord],
          enroll_time   = td_sub$enroll_time[ord],
          calendar_time = cal_time[ord],
          n_events      = seq_len(m),
          stringsAsFactors = FALSE
        )
      }

      event_counts

    },

    #' @description
    #' given a set of endpoints and target number of events, determine the data
    #' lock time for a milestone (futility, interim, final, etc.). This function does
    #' not change trial object (e.g. rolling back not yet randomized patients after
    #' the found data lock time).
    #' @param endpoints character vector. Data lock time is determined by a set
    #' of endpoints.
    #' @param target_n_events target number of events for each of the
    #' \code{endpoints}.
    #' @param arms a vector of arms' name on which number of events will be
    #' counted.
    #' @param type \code{all} if all target number of events are reached.
    #' \code{any} if the any target number of events is reached.
    #' @param ... subset conditions compatible with \code{dplyr::filter}. Number
    #' Time of milestone is based on event counts on the subset of trial data.
    #' @return data lock time
    #'
    get_data_lock_time_by_event_number = function(endpoints, arms,
                                                  target_n_events,
                                                  type = c('all', 'any'),
                                                  ...){

      type <- match.arg(type)

      stopifnot(is.character(endpoints))
      stopifnot(all(is.wholenumber(target_n_events)))
      stopifnot(length(endpoints) == length(target_n_events))

      if(is.null(arms)){
        arms <- self$get_arms_name()
      }

      event_counts <- self$get_event_tables(arms, ...)

      missing_endpoints <- setdiff(endpoints, names(event_counts))
      if(length(missing_endpoints) > 0){
        stop('Endpoints <',
             paste0(missing_endpoints, collapse = ', '),
             '> are missing in event_counts when determining data lock time. ')
      }

      milestone_times <- NULL
      for(i in seq_along(endpoints)){
        if(max(event_counts[[endpoints[i]]]$n_events) < target_n_events[i]){
          warning('No enough events/samples for endpoint <', endpoints[i],
               '> to reach the target number <', target_n_events[i], '>. ',
               immediate. = TRUE)
          milestone_times <- c(milestone_times, Inf)
        }else{

          milestone_times <-
            c(milestone_times,
              min(event_counts[[endpoints[i]]]$calendar_time[
                event_counts[[endpoints[i]]]$n_events >= target_n_events[i]
              ]))
        }
      }

      lock_time <-
        case_when(
          type %in% 'all' ~ max(milestone_times),
          type %in% 'any' ~ min(milestone_times),
          TRUE ~ -Inf
        )

      if(is.infinite(lock_time)){
        stop('None of the endpoints can reach target event number during the trial. \n',
             'Consider reducing the target in milestone() and/or extending trial duration in trial(). ')
      }

      lock_time

    },

    #' @description
    #' given a target number of enrolled patients, determine the data
    #' lock time for a milestone (futility, interim, final, etc.). This function does
    #' not change trial object (e.g. rolling back not yet randomized patients after
    #' the found data lock time). It is similar to get_data_lock_time_by_event_number
    #' but only focus on patient_id.
    #' @param target_n_patients target number of enrolled patients.
    #' @param arms a vector of arms' name on which number of events will be
    #' counted.
    #' @param min_treatment_duration numeric. Zero or positive value.
    #' minimum treatment duration of enrolled patients.
    #' If 0, it looks for triggering time based on number of enrolled
    #' patients in population specified by \code{...} and \code{arms}. If positive,
    #' it means that milestone is triggered when a specific number of enrolled
    #' patients have received treatment for at least \code{min_treatment_duration}
    #' duration. It is users' responsibility to assure that the unit of
    #' \code{min_treatment_duration} are consistent with
    #' readout of non-tte endpoints, dropout time, and trial duration.
    #' @param ... subset conditions compatible with \code{dplyr::filter}. Number
    #' Time of milestone is based on event counts on the subset of trial data.
    #' @return data lock time
    #'
    get_data_lock_time_by_enrollment = function(arms,
                                                target_n_patients,
                                                min_treatment_duration,
                                                ...){

      stopifnot(all(is.wholenumber(target_n_patients)))
      stopifnot(min_treatment_duration >= 0)

      if(is.null(arms)){
        arms <- self$get_arms_name()
      }

      event_counts <- self$get_event_tables(arms, ...)

      if(max(event_counts[['patient_id']]$n_events) < target_n_patients){
        warning('No enough patients to reach the target number <', target_n_patients, '>. ',
                immediate. = TRUE)
        milestone_time <- Inf
      }else{
        milestone_time <- min(event_counts[['patient_id']]$calendar_time[
              event_counts[['patient_id']]$n_events >= target_n_patients
            ]) + min_treatment_duration
      }

      lock_time <- milestone_time

      if(is.infinite(lock_time)){
        stop('None of the endpoints can reach target event number during the trial. \n',
             'Consider reducing the target in milestone() and/or extending trial duration in trial(). ')
      }

      lock_time

    },

    #' @description
    #' given the calendar time to lock the data, return it with event counts of
    #' each of the endpoints.
    #' @param calendar_time numeric. Calendar time to lock the data
    #' @return data lock time
    #'
    get_data_lock_time_by_calendar_time = function(calendar_time){

      stopifnot(is.numeric(calendar_time) && length(calendar_time) && calendar_time >= 0)

      lock_time <- calendar_time
      lock_time

    },

    #' @description
    #' return locked data, i.e. snapshot at a milestone. TTE data is censored
    #' and non-TTE data is truncated accounting for readout time and dropout
    #' time simultaneously by the triggering time of milestone.
    #' @param milestone_name character. Milestone name of which the locked
    #' data to be extracted.
    get_locked_data = function(milestone_name){
      if(!(milestone_name %in% names(private$locked_data))){
        triggered_milestone_names <- self$get_locked_data_name()
        if(length(triggered_milestone_names) == 0){
          stop('Locked data for milestone <', milestone_name, '> cannot be found. ',
               'No milestone has been triggered yet. ',
               'Make sure the controller has run before calling get_locked_data(). ')
        }else{
          stop('Locked data for milestone <', milestone_name, '> cannot be found. ',
               'Triggered milestone(s): <',
               paste0(triggered_milestone_names, collapse = ', '), '>. ')
        }
      }

      private$locked_data[[milestone_name]]
    },

    #' @description
    #' return names of locked data
    get_locked_data_name = function(){
      names(private$locked_data)
    },

    #' @description
    #' return number of events at lock time of milestones
    #' @param milestone_name names of triggered milestones. Use all triggered milestones
    #' if \code{NULL}.
    get_event_number = function(milestone_name = NULL){
      if(is.null(milestone_name)){
        milestone_name <- self$get_locked_data_name()
      }

      n_events <- NULL
      lock_time <- NULL
      for(milestone in milestone_name){
        lock_time <- c(lock_time,
                       attr(self$get_locked_data(milestone), 'lock_time')[1])
        n_events <- bind_rows(n_events,
                              attr(attr(self$get_locked_data(milestone), 'lock_time'), 'n_events'))
      }

      n_events <- n_events %>%
        mutate(lock_time = lock_time) %>%
        mutate(milestone_name = milestone_name) %>%
        arrange(lock_time)

      n_events
    },

    #' @description
    #' save time of a new milestone.
    #' @param milestone_time numeric. Time of new milestone.
    #' @param milestone_name character. Name of new milestone.
    save_milestone_time = function(milestone_time, milestone_name){
      if(milestone_name %in% names(private$milestone_time)){
        stop('Time of milestone <', milestone_name, '> has already been saved before. ')
      }

      if(length(private$milestone_time) > 0){
        if(any(private$milestone_time > milestone_time)){
          en <- names(private$milestone_time)[private$milestone_time > milestone_time]
          et <- private$milestone_time[private$milestone_time > milestone_time]
          stop('New milestone <', milestone_name, '> (time = ', round(milestone_time, 2),
               ') happens before milestones <',
               paste0(en, ' (time = ', round(et, 2), ')', collapse = ', '), '>. \n',
               'A possible reason is mis-specification of milestone order or triggering conditions. \n',
               'Use seed = <', self$get_seed(), '> to debug it. ')
        }
      }

      private$milestone_time[milestone_name] <- milestone_time
    },

    #' @description
    #' return milestone time when triggering a given milestone
    #' @param milestone_name character. Name of milestone. If \code{NULL},
    #' time of all triggered milestones are returned.
    get_milestone_time = function(milestone_name = NULL){

      if(is.null(milestone_name)){
        return(private$milestone_time)
      }

      if(!all(milestone_name %in% names(private$milestone_time))){
        stop('Milestone(s) <',
             paste0(setdiff(milestone_name, names(private$milestone_time)), collapse = ', '),
             '> cannot be found. ',
             'Make sure that milestone(s) have be triggered ',
             'and their triggering time has been saved by calling get_milestone_time. ',
             'Usually this function is called automatically while locking a data. ')
      }

      private$milestone_time[milestone_name]
    },

    #' @description
    #' lock data at specific calendar time.
    #' For time-to-event endpoints, their event indicator \code{*_event} should be
    #' updated accordingly. Locked data should be stored separately.
    #' DO NOT OVERWRITE/UPDATE private$trial_data! which can lose actual
    #' time-to-event information. For example, a patient may be censored at
    #' the first data lock. However, he may have event being observed in a
    #' later data lock.
    #' @param at_calendar_time time point to lock trial data
    #' @param milestone_name assign milestone name as the name of locked data for
    #' future reference.
    lock_data = function(at_calendar_time, milestone_name){

      trial_data <- self$get_trial_data()

      event_cols <- grep('_event$', names(trial_data), value = TRUE)
      tte_cols   <- sub('_event$', '', event_cols)

      ## Censor TTE endpoints: if event falls after lock time, mark as censored.
      for(k in seq_along(event_cols)){
        event_col <- event_cols[k]
        tte_col   <- tte_cols[k]
        cal_time  <- trial_data$enroll_time + trial_data[[tte_col]]
        late      <- cal_time > at_calendar_time
        trial_data[[event_col]][late] <- 0L
        trial_data[[tte_col]][late]   <- at_calendar_time - trial_data$enroll_time[late]
      }

      readout_cols <- grep('_readout$', names(trial_data), value = TRUE)
      ep_cols      <- sub('_readout$', '', readout_cols)

      ## Null-out non-TTE readouts that have not yet occurred by lock time.
      for(k in seq_along(readout_cols)){
        readout_col <- readout_cols[k]
        ep_col      <- ep_cols[k]
        cal_time    <- trial_data$enroll_time + trial_data[[readout_col]]
        ## in locked data, some patients may have been enrolled, but
        ## their non-tte endpoints have no readout, thus set to be NA
        trial_data[[ep_col]][cal_time > at_calendar_time] <- NA
      }

      enrolled_mask <- trial_data$enroll_time <= at_calendar_time
      locked_data   <- trial_data[enrolled_mask, , drop = FALSE]
      locked_data   <- locked_data[order(locked_data$enroll_time), , drop = FALSE]
      rownames(locked_data) <- NULL

      if(!is.null(locked_data$regimen_trajectory)){
        ## only patients with switching events need filtering; non-switchers
        ## have a single "arm@0" segment that always survives the lock window
        switchers <- grep(';', locked_data$regimen_trajectory, fixed = TRUE)
        if(length(switchers) > 0){
          locked_data$regimen_trajectory[switchers] <- mapply(
            function(traj_str, enroll_time){
              entries <- strsplit(traj_str, ';', fixed = TRUE)[[1]]
              times <- as.numeric(sub('.*@', '', entries))
              paste(entries[enroll_time + times <= at_calendar_time], collapse = ';')
            },
            locked_data$regimen_trajectory[switchers],
            locked_data$enroll_time[switchers],
            SIMPLIFY = TRUE, USE.NAMES = FALSE
          )
        }
        ## number of switches = number of '@' minus 1 (the initial arm@0 entry)
        locked_data$n_switches <- lengths(gregexpr('@', locked_data$regimen_trajectory, fixed = TRUE)) - 1L
      }

      n_events_or_readouts <- NULL
      for(tte_col in tte_cols){
        n_events_or_readouts <- c(n_events_or_readouts, sum(locked_data[[tte_col]] != 0))
      }

      for(ep_col in ep_cols){
        n_events_or_readouts <- c(n_events_or_readouts, sum(!is.na(locked_data[[ep_col]])))
      }

      if(all(n_events_or_readouts == 0)){
        warning('No TTE event or non-TTE readout of endpoints <',
                paste0(c(tte_cols, ep_cols), collapse = ', '),
                '> in data snapshot locked at milestone <',
                milestone_name, '>. Check: \n',
                '(1) Is this milestone triggered too early?\n',
                '(2) Is the dropout rate too high?\n',
                '(3) Do you use the same unit for readout time, trial duration, and dropout time?')
      }

      ## Build per-arm event/readout counts without dplyr group_by+summarise.
      arms_in_locked <- sort(unique(locked_data$arm))
      arm_factor     <- factor(locked_data$arm, levels = arms_in_locked)

      event_counts_per_arm <- data.frame(arm = arms_in_locked, stringsAsFactors = FALSE)

      for(event_col in event_cols){
        tte_col <- sub('_event$', '', event_col)
        event_counts_per_arm[[tte_col]] <- as.integer(
          tapply(locked_data[[event_col]], arm_factor, sum)
        )
      }

      for(readout_col in readout_cols){
        ep_col <- sub('_readout$', '', readout_col)
        event_counts_per_arm[[ep_col]] <- as.integer(
          tapply(!is.na(locked_data[[ep_col]]), arm_factor, sum)
        )
      }

      event_counts_per_arm[['patient_id']] <- as.integer(tabulate(arm_factor))

      n_events <- as.data.frame(t(colSums(event_counts_per_arm[, -1])))
      n_events[['arms']] <- I(list(event_counts_per_arm))

      unenrolled_mask <- trial_data$enroll_time > at_calendar_time
      unenrolled_data <- trial_data[unenrolled_mask, c('enroll_time', 'arm'), drop = FALSE]
      unenrolled_data <- unenrolled_data[order(unenrolled_data$enroll_time), , drop = FALSE]
      rm(trial_data)

      attr(at_calendar_time, 'n_events') <- n_events
      attr(locked_data, 'lock_time') <- at_calendar_time
      attr(locked_data, 'n_enrolled_patients') <- length(unique(locked_data$patient_id))
      attr(locked_data, 'milestone_name') <- milestone_name
      private$locked_data[[milestone_name]] <- locked_data
      self$set_current_time(at_calendar_time)
      self$save_milestone_time(at_calendar_time, milestone_name)

      self$save(value = at_calendar_time, name = paste0('milestone_time_<', milestone_name, '>'))

      self$save(value = attr(at_calendar_time, 'n_events'),
                name = paste0('n_events_<', milestone_name, '>'))

      if(!private$silent){
        message('Data is locked at time = ', at_calendar_time, ' for milestone <',
                milestone_name, '>.\n',
                'Locked data can be accessed in Trial$get_locked_data(\'',
                milestone_name, '\'). \n',
                'Number of events at lock time: \n')
        out <- as.data.frame(attr(at_calendar_time, 'n_events'))
        if('patient_id' %in% names(out)){
          colnames(out)[names(out) == 'patient_id'] <- 'patient'
        }
        message(paste0(capture.output(out), collapse = "\n"))
        message('\n')
      }

      ## I am not sure about this part yet.
      ## Once data is locked for a milestone, it is not always necessary to
      ## roll back. For example, all arms are keeping moving without anything
      ## change is possible. This could happen except for futility analysis
      ## (early stop), or adding/removing arms. It seems like this part should
      ## be done in action() depending on the type of milestone.
      ##
      ## updated note: Yes, should be done by users in action function of milestone
      ## Actually, $add_arms, $remove_arms, and $update_sample_ratio can be
      ## called by users in action function. All these three functions will
      ## do randomization, and patient enrollment again. We possibly support
      ## update enrollment curve in the future.
      if(0){
      private$enroll_time <- unenrolled_data$enroll_time
      private$randomization_queue <- unenrolled_data$arm


      private$trial_data <- self$get_trial_data() %>%
        dplyr::filter(enroll_time <= at_calendar_time)

      self$enroll_patients()
      }

      NULL
    },


    #' @description
    #' plot of cumulative number of events/samples over calendar time.
    event_plot = function(){

      trial_data <- self$get_trial_data()

      event_number <- self$get_event_number()
      event_number$event_name <- paste0(1:nrow(event_number), ': ',
                                        event_number$event_name)

      event_cols <- grep('_event$', names(trial_data), value = TRUE)
      readout_cols <- grep('_readout$', names(trial_data), value = TRUE)

      all_data_list <- NULL
      for(col in c(event_cols, readout_cols)){
        tte_col <- gsub('_event$', '', col)
        ep_col <- gsub('_readout$', '', col)

        stopifnot((tte_col != col) || (ep_col != col))

        data_list <- list()

        if(ep_col == col){ ## a tte endpoint
          event_counts <- trial_data %>%
            dplyr::select(all_of(c('patient_id', 'arm', 'enroll_time', tte_col, col))) %>%
            mutate(calendar_time := enroll_time + !!sym(tte_col)) %>%
            arrange(calendar_time)
          col_ <- tte_col

          data_list[['0: overall']] <- event_counts %>%
            mutate(n_events = cumsum(get(col)))

          idx <- 0
          for(arm_ in sort(unique(trial_data$arm))){
            idx <- idx + 1
            data_list[[paste0(idx, ': ', arm_)]] <- event_counts %>%
              dplyr::filter(arm %in% arm_) %>%
              arrange(calendar_time) %>%
              mutate(n_events = cumsum(get(col)))
          }

        }else{ ## a non-tte endpoint
          event_counts <- trial_data %>%
            dplyr::select(all_of(c('patient_id', 'arm', 'enroll_time', ep_col, col))) %>%
            dplyr::filter(!is.na(!!sym(ep_col))) %>%
            mutate(calendar_time := enroll_time + !!sym(col)) %>%
            arrange(calendar_time)
          col_ <- ep_col

          data_list[['0: overall']] <- event_counts %>%
            mutate(n_events = row_number())

          idx <- 0
          for(arm_ in sort(unique(trial_data$arm))){
            idx <- idx + 1
            data_list[[paste0(idx, ': ', arm_)]] <- event_counts %>%
              dplyr::filter(arm %in% arm_) %>%
              arrange(calendar_time) %>%
              mutate(n_events = row_number())
          }

        }

        all_data_list <- bind_rows(
          all_data_list,
          lapply(names(data_list), function(name){
            data_list[[name]] %>%
              mutate(arm = name) %>%
              mutate(endpoint = col_)
          })
        )

      }


      ## prepare stacked area chart
      all_data <- all_data_list %>%
        dplyr::filter(!(arm %in% '0: overall'))

      endpoints <- sort(unique(all_data$endpoint))
      arms <- sort(unique(all_data$arm))
      ct <- sort(unique(all_data$calendar_time))

      new_data <- NULL
      for(col in c(event_cols, readout_cols)){
        tte_col <- gsub('_event$', '', col)
        ep_col <- gsub('_readout$', '', col)
        is_tte <- (ep_col == col)
        ep <- ifelse(is_tte, tte_col, ep_col)

        for(arm_ in arms){
          ## Using filter(endpoint %in% ep & ...) is dangerous because all_data
          ## has a column named ep. This will apply %in% between the two columns
          ## endpoint and ep in the data frame, rather than comparing column
          ## endpoint with the character vector ep. Using pronoun .env$ep
          ## forces R to look for ep in environment, not in the data frame columns
          ## Consider this example:
          ## x <- 'x'
          ## data.frame(x = rnorm(4), y = c('x', 'b', 'a', 'x'), z = 1:4) %>%
          ##   filter(y %in% x)
          dat <- all_data %>%
            dplyr::filter(endpoint %in% .env$ep & arm %in% .env$arm_) %>%
            dplyr::select(c('arm', col, 'calendar_time', 'endpoint')) %>%
            rename(has_event = !!sym(col))

          if(!is_tte){
            dat$has_event <- 1
          }

          time <- sort(setdiff(ct, dat$calendar_time))
          if(length(time) == 0){
            new_data <- bind_rows(new_data, dat)
            next
          }

          dat <- bind_rows(dat,
                           data.frame(
                             arm = arm_,
                             has_event = 0,
                             calendar_time = time,
                             endpoint = ep
                           ))
          dat <- dat[!duplicated(dat), ] %>%
            arrange(calendar_time) %>%
            mutate(n_events = cumsum(has_event)) %>%
            dplyr::select(-has_event)

          new_data <- bind_rows(new_data, dat)
        }
      }

      new_data$arm <- factor(new_data$arm, levels = arms)

      soft_colors <- function(n) {
        hcl(h = seq(0, 360 * (n-1)/n, length.out = n), c = 60, l = 70)
      }

      p <- ggplot(new_data, aes(x = calendar_time, y = n_events, fill = arm)) +
        xlim(0, self$get_duration() * 1.05) +
        labs(
          x = 'Calendar Time',
          y = 'Cumulative N'
        ) +
        geom_area() +
        scale_fill_manual(
          values = soft_colors(length(arms)),
          name = "Arm"
        ) +
        geom_vline(
          data = event_number,
          aes(xintercept = lock_time),
          linetype = 'dashed'
        ) +
        facet_wrap(~ endpoint, scales = 'free_x') +
        theme_minimal() +
        theme(legend.position = 'bottom')

      plot(p)

    },

    #' @description
    #' censor trial data at calendar time
    #' @param censor_at time of censoring. It is set to trial duration if
    #' \code{NULL}.
    #' @param selected_arms censoring is applied to selected arms (e.g.,
    #' removed arms) only. If \code{NULL}, it will be set to all available arms
    #' in trial data. Otherwise, censoring is applied to user-specified arms only.
    #' This is necessary because number of events/sample size in removed arms
    #' should be fixed unchanged since corresponding milestone is triggered. In that
    #' case, one can update trial data by something like
    #' \code{censor_trial_data(censor_at = milestone_time, selected_arms = removed_arms)}.
    #' @param enrolled_before censoring is applied to patients enrolled before
    #' specific time. This argument would be used when trial duration is
    #' updated by \code{set_duration}. Adaptation happens when \code{set_duration}
    #' is called so we fix duration for patients enrolled before adaptation
    #' to maintain independent increment. This should work when trial duration
    #' is updated for multiple times.
    censor_trial_data = function(censor_at = NULL, selected_arms = NULL, enrolled_before = Inf){

      if(is.null(censor_at)){
        censor_at <- self$get_duration()
      }

      trial_data <- self$get_trial_data()

      if(is.null(selected_arms)){
        selected_arms <- unique(trial_data$arm)
      }

      event_cols <- grep('_event$', names(trial_data), value = TRUE)
      readout_cols <- grep('_readout$', names(trial_data), value = TRUE)

      ## base R: precompute the arm/enroll mask once (shared by all endpoint loops)
      sel <- (trial_data$arm %in% selected_arms) & (trial_data$enroll_time <= enrolled_before)

      for(event_col in event_cols){
        tte_col <- gsub('_event$', '', event_col)
        ## dropout censoring: event falls after dropout
        drop_mask <- sel & (trial_data[[tte_col]] > trial_data$dropout_time)
        trial_data[[event_col]][drop_mask] <- 0L
        trial_data[[tte_col]][drop_mask]   <- trial_data$dropout_time[drop_mask]
        ## calendar-time censoring: event falls after data lock
        cal_time  <- trial_data$enroll_time + trial_data[[tte_col]]
        late_mask <- sel & (cal_time > censor_at)
        trial_data[[event_col]][late_mask] <- 0L
        trial_data[[tte_col]][late_mask]   <- censor_at - trial_data$enroll_time[late_mask]
        ## defensive clip (censor_at - enroll_time can be negative for patients not yet enrolled)
        neg_mask <- trial_data[[tte_col]] < 0
        trial_data[[tte_col]][neg_mask] <- 0
      }

      for(readout_col in readout_cols){
        ep_col <- gsub('_readout$', '', readout_col)
        ## dropout censoring
        drop_mask <- sel & (trial_data[[readout_col]] > trial_data$dropout_time)
        trial_data[[ep_col]][drop_mask] <- NA
        ## calendar-time censoring
        cal_time  <- trial_data$enroll_time + trial_data[[readout_col]]
        late_mask <- sel & (cal_time > censor_at)
        trial_data[[ep_col]][late_mask] <- NA
      }

      ## sort once at the end (dplyr version re-sorted inside every loop iteration)
      private$trial_data <- trial_data[order(trial_data$enroll_time), , drop = FALSE]
      rownames(private$trial_data) <- NULL
    },

    #' @description
    #' save a single value or a one-row data frame to trial's output
    #' for further analysis/summary later. Results saved by calling this
    #' function have a life cycle of the whole simulation. This means that
    #' all results are accumulated across multiple simulated trial and can be
    #' used for summary later.
    #' @param value value to be saved. It can be a scalar (vector of length 1)
    #' or a data frame (of one row).
    #' @param name character to name the saved object. It will be used to
    #' name a column in trial's output if \code{value} is a scalar.
    #' If \code{value} is a data frame, \code{name} will be the prefix pasted
    #' with the column name of \code{value} in trial's output.
    #' If user want to use
    #' \code{value}'s column name as is in trial's output, set \code{name}
    #' to be \code{''} as default. Otherwise, column name would be, e.g.,
    #' \code{"{<name>}_<{colnames(value)}>"}.
    #' @param overwrite logic. \code{TRUE} if overwriting existing entries
    #' with warning, otherwise, throwing an error and stop.
    save = function(value, name = '', overwrite = FALSE){

      is_vector_length1 <- function(x) {
        length(x) == 1 && is.atomic(unclass(x))
      }

      is_vector <- function(x) {
        is.atomic(unclass(x))
      }

      if(!(is.character(name) && length(name) == 1)){
        stop('The argument `name` should be a character of length 1',
             ' when calling Trial$save(). You specify it to be <',
             name, '> however, which is of class <',
             paste0(class(name), collapse = ', '), '>. ')
      }

      if(is.null(private$output)){
        private$output <- data.frame(trial = self$get_name())
      }

      if(!is_vector_length1(value) && !is.data.frame(value)){
        stop('For now only vector or data.frame can be saved during a trial. ')
      }

      if(is_vector(value)){
        if(length(value) > 1){
          stop('A vector object to be saved can only be of length one. ')
        }
        value <- data.frame(col = as.vector(value))
        colnames(value) <- name
      }else{ ## value is a data frame
        if(nrow(value) > 1){
          stop('A data frame to be saved can only contain one row. ')
        }

        if(name != ''){
          colnames(value) <- paste0(name, '_<', colnames(value), '>')
        }
      }

      for(cname in names(value)){
        if(cname %in% names(private$output)){
          if(!overwrite){
            stop(cname, ' has been used to name something in the output. ',
                 'Pick another name and try again. ')
          }else{
            if(cname != 'error_message'){
              warning(cname, ' exists in the output and is overwritten. ',
                      'Set overwrite = FALSE in save() if it is not intended. ',
                      immediate. = TRUE)
            }
          }
        }

        ## this is flexible to assign scale value or a data frame to a cell in output
        private$output[, cname] <- I(list(value[, cname]))

      }

      ## move error message to the last column for a better display
      error_message <- private$output$error_message
      private$output$error_message <- NULL
      private$output$error_message <- error_message

      invisible(NULL)

    },

    #' @description
    #' row bind a data frame to existing data frame. If a data frame \code{name}
    #' is not existing in a trial, then it is equivalent to
    #' calling \code{Trials$save_custom_data()}.
    #' Extra columns in \code{value} are ignored. Columns in
    #' \code{Trials$custom_data[[name]]} but not in \code{value} are filled
    #' with \code{NA}.
    #'
    #' This function can be used to save results across multiple milestones.
    #' For example, p-values and effect estimates of endpoints may be computed
    #' at multiple milestones. Users may want to bind them into a data frame
    #' for combination test or graphical test. In this case, this function
    #' can be called repeatedly in milestones. Once the data frame is fully
    #' conducted, statistical test can be performed on its final version
    #' retrieved by calling \code{Trials$get()}.
    #'
    #' Note that data saved by calling this function has a short life cycle
    #' within a single simulated trial. It will be reset to \code{NULL} before
    #' simulated another trial. Thus, it cannot be used to save results that
    #' are used for summarizing the simulation.
    #'
    #' @param value a data frame to be saved. It can consist of one or
    #' multiple rows.
    #' @param name character. Name of object to be saved.
    bind = function(value, name){
      if(!(is.character(name) && length(name) == 1)){
        stop('name should be a character of length 1')
      }

      if(!is.data.frame(value)){
        stop('value should be a data frame. ')
      }

      if(is.null(private$custom_data[[name]])){
        private$custom_data[[name]] <- value
        return(invisible(NULL))
      }

      new_cols <- names(value)
      old_cols <- names(private$custom_data[[name]])

      if(!any(new_cols %in% old_cols)){
        stop('None of columns in <value> can be found in <', name, '>. ',
             'Cannot bind data to <', name, '>. \n',
             'Columns in <value>: \n<', paste0(new_cols, collapse = ', '), '>. \n',
             'Columns in <', name, '>: \n<', paste0(old_cols, collapse = ', '), '>. ')
      }

      miss_cols <- setdiff(old_cols, new_cols)
      extra_cols <- setdiff(new_cols, old_cols)

      if(length(extra_cols) > 0){
        warning('Extra columns <', paste0(extra_cols, collapse = ', '),
                '> present in <', name, '>, and are omitted. ')
        value <- value %>%
          dplyr::select(-all_of(extra_cols))
        print(value)
      }

      if(length(miss_cols) > 0){
        warning('Columns <', paste0(miss_cols, collapse = ', '),
                '> are missing when binding to <', name, '>. ',
                'Entries are filled with NA')
      }

      private$custom_data[[name]] <- bind_rows(private$custom_data[[name]], value)
      invisible(NULL)
    },

    #' @description
    #' save arbitrary (number of) objects into a trial so that users can use
    #' those to control the workflow. Most common use case is to store
    #' simulation parameters to be used in action functions.
    #' @param value value to be saved. Any type.
    #' @param name character. Name of the value to be accessed later.
    #' @param overwrite logic. \code{TRUE} if overwriting existing entries
    #' with warning, otherwise, throwing an error and stop.
    save_custom_data = function(value, name, overwrite = FALSE){

      if(name == ''){
        stop('name in custom_data cannot be empty. ')
      }

      if(name %in% names(private$custom_data)){
        if(!overwrite){
          stop(name, ' has been used to name something in custom data ',
               'Pick another name and try again. ')
        }else{
          warning(name, ' exists in custom_data and is overwritten. ',
                  'Set overwrite = FALSE in save_custom_data() ',
                  'if it is not intended. ',
                  immediate. = TRUE)
        }
      }

      private$custom_data[[name]] <- value
      private$.snapshot[['custom_data']][[name]] <- value

      invisible(NULL)
    },

    #' @description
    #' return custom data saved by calling \code{Trials$save_custom_data()}
    #' or \code{Trials$bind()} with its name.
    #' @param name character. Name of custom data to be accessed.
    get_custom_data = function(name){
      if(!(name %in% names(private$custom_data))){
        stop(name, ' cannot be found in custom_data. ',
             'Check for bug or typo in data name. ',
             'Did you really save ', name,
             'by using Trial$save_custom_data(value, name) before? ')
      }

      private$custom_data[[name]]
    },

    #' @description
    #' alias of function \code{get_custom_data} to make it short and cool.
    #' @param name character. Name of custom data to be accessed.
    get = function(name){
      self$get_custom_data(name)
    },

    #' @description
    #' return a data frame of all current outputs saved by calling
    #' \code{Trials$save()}. Usually this function is call at the end of
    #' simulation for summary.
    #'
    #' @param cols columns to be returned from \code{Trial$output}. If
    #' \code{NULL}, all columns are returned.
    #' @param simplify logical. Return value rather than a data frame of one
    #' column when \code{length(col) == 1} and \code{simplify == TRUE}.
    #' @param tidy logical. \code{TrialSimulator} automatically records a set
    #' of standard outputs at milestones, even when \code{doNothing} is used
    #' as action functions. These includes time of triggering milestones,
    #' number of observed events for time-to-event endpoints, and number of
    #' non-missing readouts for non-TTE endpoints
    #' (see \code{vignette('actionFunctions')}). This usually mean a large
    #' number of columns in outputs. If users have no intent to summarize a
    #' trial on these columns, setting \code{tidy = TRUE} can eliminate these
    #' columns from \code{get_output()}. Note that currently we use regex
    #' \code{"^n_events_<.*?>_<.*?>$"} and
    #' \code{"^milestone_time_<.*?>$"} to match columns to be eliminated.
    #' If users plan to use \code{tidy = TRUE}, caution is needed when naming
    #' custom outputs in \code{save()}. Default \code{FALSE}.
    get_output = function(cols = NULL, simplify = TRUE, tidy = FALSE){
      if(is.null(cols)){
        cols <- colnames(private$output)
      }

      if(!all(cols %in% names(private$output))){
        stop('Columns <', paste0(setdiff(cols, names(private$output)), collapse = ', '),
             '> are not found in trial$output. Check if there is a typo. ')
      }
      ret <- private$output[, cols, drop = FALSE]

      if(tidy){
        ret <- ret %>%
          select(!matches("^n_events_<.*?>_<.*?>$")) %>%
          select(!matches("^milestone_time_<.*?>$"))
      }

      if(simplify && ncol(ret) == 1){
        return(ret[1, 1])
      }else{
        return(ret)
      }
    },

    #' @description
    #' mute all messages (not including warnings)
    #' @param silent logical.
    mute = function(silent){
      private$silent <- silent
    },

    #' @description
    #' save less information in trial output if no intent to use it in summary
    #' @param tidy logical. If \code{TRUE}, event count per arm per endpoint is
    #' not computed and saved in trial output. This can speed up simulation by
    #' up to 40\% under some circumstances.
    tidy_output = function(tidy){
      private$save_event_count_per_arm <- !tidy
    },

    #' @description
    #' calculate independent increments from a given set of milestones
    #' @param formula An object of class \code{formula} that can be used with
    #' \code{survival::coxph}. Must consist \code{arm} and endpoint in \code{data}.
    #' No covariate is allowed. Stratification variables are supported and can be
    #' added using \code{strata(...)}.
    #' @param placebo character. String of placebo in trial's locked data.
    #' @param milestones a character vector of milestone names in the trial, e.g.,
    #' \code{listener$get_milestone_names()}.
    #' @param alternative a character string specifying the alternative hypothesis,
    #' must be one of \code{"greater"} or \code{"less"}. No default value.
    #' \code{"greater"} means superiority of treatment over placebo is established
    #' by an hazard ratio greater than 1 when a log-rank test is used.
    #' @param planned_info a vector of planned accumulative number of event of
    #' time-to-event endpoint. It is named by milestone names.
    #' Note: \code{planned_info} can also be a character
    #' \code{"oracle"} so that planned number of events are set to be observed
    #' number of events, in that case inverse normal z statistics equal to
    #' one-sided logrank statistics. This is for the purpose of debugging only.
    #' In formal simulation, \code{"oracle"} should not be used if adaptation
    #' is present. Pre-fixed \code{planned_info} should be used to create
    #' weights in combination test that controls the family-wise error rate
    #' in the strong sense.
    #' @param ... subset condition that is compatible with \code{dplyr::filter}.
    #' \code{survdiff} will be fitted on this subset only to compute one-sided
    #' logrank statistics. It could be useful when a
    #' trial consists of more than two arms. By default it is not specified,
    #' all data will be used to fit the model.
    #'
    #' @return
    #' This function returns a data frame with columns:
    #' \describe{
    #' \item{\code{p_inverse_normal}}{one-sided p-value for inverse normal test
    #' based on logrank test (alternative hypothesis: risk is higher in placebo arm).
    #' Accumulative data is used. }
    #' \item{\code{z_inverse_normal}}{z statistics of \code{p_inverse_normal}.
    #' Accumulative data is used. }
    #' \item{\code{p_lr}}{one-sided p-value for logrank test
    #'  (alternative hypothesis: risk is higher in placebo arm).
    #' Accumulative data is used. }
    #' \item{\code{z_lr}}{z statistics of \code{p_lr}.
    #' Accumulative data is used. }
    #' \item{\code{info}}{observed accumulative event number. }
    #' \item{\code{planned_info}}{planned accumulative event number. }
    #' \item{\code{info_pbo}}{observed accumulative event number in placebo. }
    #' \item{\code{info_trt}}{observed accumulative event number in treatment arm. }
    #' \item{\code{wt}}{weights in \code{z_inverse_normal}. }
    #' }
    #'
    #' @examples
    #'
    #' \dontrun{
    #' trial$independentIncrement(Surv(pfs, pfs_event) ~ arm, 'pbo',
    #'                            listener$get_milestone_names(),
    #'                            'less', 'oracle')
    #' }
    independentIncrement = function(formula, placebo, milestones, alternative,
                                    planned_info,
                                    ...){

      if(!identical(planned_info, 'oracle') && length(milestones) != length(planned_info)){
        stop('milestones and planned_info should be of same length. ')
      }

      ## by doing this, milestones in function argument can be in arbitrary order
      milestone_time <- sort(self$get_milestone_time(milestones))
      milestones <- names(milestone_time)

      info <- c() ## observed accumulated events
      lr <- c() ## one-sided log rank statistics
      info_pbo <- c()
      info_trt <- c()
      plan_best_info <- ifelse(identical(planned_info, 'oracle'), TRUE, FALSE)
      if(plan_best_info){
        planned_info <- c()
      }else{
        ## make it accumulative
        planned_info <- planned_info[milestones] %>% cumsum()
      }


      ## Get label of treated arm from subset data (through ...)
      ## We need this label to call Trials$get_arm_removal_time

      # Prepare the data based on condition in ...
      analysis_set <- if(...length() == 0){
        self$get_locked_data(tail(milestones, 1))
      }else{
        tryCatch({
          self$get_locked_data(tail(milestones, 1)) %>% dplyr::filter(...)
        },
        error = function(e){
          self$save(e$message, 'error_message', overwrite = TRUE)
          stop('Error when filtering data in independentIncrement(). ',
               'Please check condition in ..., ',
               'which should be compatible with dplyr::filter. ')
        })
      }

      trt_arm <- setdiff(unique(analysis_set$arm), placebo)
      if(length(trt_arm) == 0){
        stop('No treatment arm can be used in independentIncrement(). ',
             'Check your ... argument. ')
      }

      if(length(trt_arm) > 1){
        stop('More than one treatment arm <',
             paste0(trt_arm, collapse = ', '),
             '> are passed into independentIncrement(). ',
             'Check your ... argument. ')
      }

      rm(analysis_set)

      n_pbo <- c()
      n_trt <- c()
      trt_str <- c()
      milestone_name <- c()
      for(i in seq_along(milestones)){
        milestone_name[i] <- milestones[i]

        ## We assume that placebo and only one treated arm are used in independentIncrement
        ## thus lr_fit should be a data frame of one row
        lr_fit <- fitLogrank(formula, placebo, self$get_locked_data(milestones[i]),
                             alternative, ..., tidy = FALSE)
        if(nrow(lr_fit) > 1){
          stop('Trials$independentIncrement() should be applied to one treated arm at a time. ',
               'Check the entry where you call independentIncrement(). ',
               'Usually you can use its subsetting argument ... to meet this assumption/requirement. ')
        }

        info[i] <- lr_fit$info
        lr[i] <- lr_fit$z
        info_pbo[i] <- lr_fit$info_pbo
        info_trt[i] <- lr_fit$info_trt

        n_pbo[i] <- lr_fit$n_pbo
        n_trt[i] <- lr_fit$n_trt
        trt_str[i] <- lr_fit$arm

        if(plan_best_info){
          planned_info[i] <- lr_fit$info
        }
      }

      names(info) <- milestones

      if(any(diff(planned_info) < 0)){
        stop('milestones and planned_info should be in the same order. ')
      }

      if(any(diff(info) < 0)){
        stop('Debug this as info should be non-decreasing. ')
      }

      ii <- c() ## independent increments
      wt <- c() ## weight in inverse normal statistics
      inverse_normal <- c() ## inverse normal test statistics

      stage_info <- c()
      stage_n_pbo <- c()
      stage_n_trt <- c()
      for(i in seq_along(info)){
        if(i == 1){
          wt[i] <- sqrt(planned_info[i])
          ii[i] <- lr[i]
          inverse_normal[i] <- lr[i]

          stage_info[i] <- info[i]
          stage_n_pbo[i] <- n_pbo[i]
          stage_n_trt[i] <- n_trt[i]

          next
        }

        stage_info[i] <- info[i] - info[i - 1]
        stage_n_pbo[i] <- n_pbo[i] - n_pbo[i - 1]
        stage_n_trt[i] <- n_trt[i] - n_trt[i - 1]
        wt[i] <- sqrt(planned_info[i] - planned_info[i - 1])

        arm_removal_time <- self$get_arm_removal_time(arm = trt_arm)
        milestone_triggering_time <- self$get_milestone_time(milestone_name = names(info)[i])


        ## Use "<", not "<="!!
        ## When arm_removal_time == milestone_triggering_time, it means the arm
        ## is just removed from the trial at that milestone. Thus, data from the
        ## arm can still be used to update testing statistics, no need to be
        ## specified as +/-Inf
        if(arm_removal_time < milestone_triggering_time){
          ## This means this treatment arm has been removed from the trial BEFORE milestone i
          ## Set z statistic of independent increment to be an extremely value
          ## so that inverse normal combination test would always accept
          ## the neutral null hypothesis (p-value is close or equal to 1.0)
          ii[i] <- ifelse(alternative == 'greater', -Inf, Inf) ## 10 * qnorm(.Machine$double.eps) ## about -81
        }else{
          ii[i] <- (sqrt(info[i]) * lr[i] - sqrt(info[i - 1]) * lr[i - 1]) /
            sqrt(stage_info[i])
        }
        inverse_normal[i] <- sum(wt * ii) / sqrt(sum(wt^2))
      }

      ret <-
        data.frame(
          milestone = milestone_name,
          milestone_time = unname(milestone_time),
          p_inverse_normal =
            if(alternative == 'greater'){
              1 - pnorm(inverse_normal)
            }else{
              pnorm(inverse_normal)
            },
          z_inverse_normal = inverse_normal,
          p_logrank =
            if(alternative == 'greater'){
              1 - pnorm(lr)
            }else{
              pnorm(lr)
            },
          z_logrank = lr,
          info = info,
          planned_info = planned_info,
          info_pbo = info_pbo,
          info_trt = info_trt,
          wt = wt,
          z_ii = ii, # stage-wise, independent increment
          n_pbo = n_pbo,
          n_trt = n_trt,
          stage_info = stage_info,
          stage_n_pbo = stage_n_pbo,
          stage_n_trt = stage_n_trt,
          trt_str = trt_str
        )

      if(any(ret$stage_info < 30)){
        ret_ <- ret %>%
          dplyr::filter(stage_info < 30) %>%
          dplyr::select(milestone, milestone_time, planned_info, info, stage_info, stage_n_pbo, stage_n_trt, trt_str)

        warning('In the arm(s) <',
                paste0(unique(ret_$trt_str), collapse = ', '),
                '>, stage-wise information (number of events) are lower than 30 (see stage_* below). \n',
                'Make sure that such a low stage-wise information is sufficient to maintain normality of independent increments of logrank statistics. ',
                immediate. = TRUE)
        message(paste0(capture.output(ret_), collapse = "\n"))
        message("\033[31m>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\033[0m\n")
      }

      ret
    },

    #' @description
    #' carry out closed test based on Dunnett method under group sequential
    #' design.
    #' @param formula An object of class \code{formula} that can be used with
    #' \code{survival::coxph}. Must consist \code{arm} and endpoint in \code{data}.
    #' No covariate is allowed. Stratification variables are supported and can be
    #' added using \code{strata(...)}.
    #' @param placebo character. Name of placebo arm.
    #' @param treatments character vector. Name of treatment arms to be used in
    #' comparison.
    #' @param milestones character vector. Names of triggered milestones at which either
    #' adaptation is applied or statistical testing for endpoint is performed.
    #' Milestones in \code{milestones} does not need to be sorted by their triggering time.
    #' @param alternative a character string specifying the alternative hypothesis,
    #' must be one of \code{"greater"} or \code{"less"}. No default value.
    #' \code{"greater"} means superiority of treatment over placebo is established
    #' by an hazard ratio greater than 1 when a log-rank test is used.
    #' @param planned_info a data frame of planned number of events of
    #' time-to-event endpoint in each stage and each arm. Milestone names, i.e.,
    #' \code{milestones} are row names of \code{planned_info}, and arm names, i.e.,
    #' \code{c(placebo, treatments)} are column names.
    #' Note that it is not the accumulative but stage-wise event numbers.
    #' It is usually not easy to determine these numbers in practice, simulation
    #' may be used to get estimates.
    #' Note: \code{planned_info} can also be a character
    #' \code{"default"} so that \code{planned_info} are set to be number
    #' of newly randomized patients in the control arm in each of the stages.
    #' This assumes that
    #' event rate do not change over time and/or sample ratio between placebo
    #' and a treatment arm does not change as well, which may not be true.
    #' It is for the purpose of debugging or rapid implementation
    #' only. Using simulation to pick \code{planned_info} is recommended in
    #' formal simulation study. Another issue with \code{planned_info} set to
    #' be \code{"default"} is that it is possible patient recruitment is done
    #' before a specific stage, as a result, \code{planned_info} is zero which
    #' can crash the program.
    #' @param ... subset condition that is compatible with \code{dplyr::filter}.
    #' \code{survdiff} will be fitted on this subset only to compute one-sided
    #' logrank statistics. It could be useful when comparison is made on a
    #' subset of treatment arms. By default it is not specified,
    #' all data (placebo plus one treatment arm at a time) in the locked data
    #' are used to fit the model.
    #'
    #' @details
    #' This function computes stage-wise p-values for each of the intersection
    #' hypotheses based on Dunnett test. If only one treatment arm is present,
    #' it is equivalent to compute the stage-wise p-values of elemental
    #' hypotheses. This function also computes inverse normal combination
    #' test statistics at each of the stages.
    #' The choice of \code{planned_info} can affect the calculation of
    #' stage-wise p-values. Specifically, it is used to compute
    #' the columns \code{observed_info} and \code{p_inverse_normal} in returned
    #' data frame, which will be used in \code{Trial$closedTest()}.
    #' The choice of \code{planned_info} can affect the result of
    #' \code{Trial$closedTest()} so user should chose it with caution.
    #'
    #' Note that in \code{Trial$closedTest()},
    #' \code{observed_info}, which is derived from \code{planned_info}, will
    #' lead to the same closed testing results up to a constant. This is because
    #' the closed test uses information fraction
    #' \code{observed_info/sum(observed_info)}. As a result, setting
    #' \code{planned_info} to, e.g., \code{10 * planned_info} should give same
    #' closed test results.
    #'
    #' Based on numerical study, setting \code{planned_info = "default"} leads
    #' to a much higher power (roughly 10\%) than setting \code{planned_info} to
    #' median of event numbers at stages, which can be determined by simulation.
    #' I am not sure if regulator would support such practice. For example,
    #' if a milestone (e.g., interim analysis) is triggered at a pre-specified
    #' calendar time, the number of randomized patients is random and is unknown
    #' when planning the trial. If I understand it correctly, regulator may want
    #' the information fraction in closed test (combined with Dunnett test) to
    #' be pre-fixed. In addition, this choice for \code{planned_info} assumes
    #' that the event rates does not change over time which is obviously not
    #' true. It is recommended to always use pre-fixed \code{planned_info} for
    #' restrict control of family-wise error rate. It should be pointed out
    #' that the choice of pre-fixed \code{planned_info} can affect statistical
    #' power significantly so fine-tuning may be required.
    #'
    #' @return a list with element names like \code{arm_name},
    #' \code{arm1_name|arm2_name}, \code{arm1_name|arm2_name|arm3_name}, etc.,
    #' i.e., all possible combination of treatment arms in comparison. Each
    #' element is a data frame, with its column names self-explained. Specifically,
    #' the columns \code{p_inverse_normal}, \code{observed_info},
    #' \code{is_final} can be used with \code{GroupSequentialTest} to perform
    #' significance test.
    #'
    #' @examples
    #' \dontrun{
    #' trial$dunnettTest(Surv(pfs, pfs_event) ~ arm, 'pbo', c('high dose', 'low dose'),
    #'                   listener$get_milestone_names(), 'default')
    #' }
    #'
    dunnettTest = function(formula, placebo, treatments, milestones, alternative,
                           planned_info, ...){

      alternative <- match.arg(alternative, choices = c('greater', 'less'))

      if(!identical(planned_info, 'default')){
        if(!('data.frame' %in% class(planned_info))){
          stop('planned_info should be a data frame of planned information at each of the stages. ')
        }
        if(nrow(planned_info) != length(milestones)){
          stop('milestones and planned_info should be of same length. ')
        }

        if(ncol(planned_info) != length(treatments) + 1){
          stop('length(planned_info) should be equal to length(treatments) + 1, i.e., ',
               length(treatment) + 1, '. ')
        }

        if(!setequal(names(planned_info), c(placebo, treatments))){
          stop('planned_info should use placebo and treatments\' names, <',
               paste0(c(placebo, treatments), collapse = ', '),
               '>, for its column names. ',
               paste0(setdiff(names(planned_info), c(placebo, treatments)), collapse = ', '),
               ' are not accepted. ')
        }

        planned_info <- planned_info[, c(placebo, treatments), drop = FALSE]

        if(!setequal(rownames(planned_info), milestones)){
          stop('planned_info should use milestone names for its row names. ')
        }

        planned_info <- planned_info[milestones, , drop = FALSE]
      }

      ## by doing this, milestones in function argument can be in arbitrary order

      if(!all(milestones %in% names(private$milestone_time))){
        stop('Milestone(s) <', paste0(setdiff(milestones, names(private$milestone_time)), collapse = ', '),
             '> haven\'t been triggered yet, so are unable to be used as "milestones" in dunnettTest(). ')
      }

      milestone_time <- sort(self$get_milestone_time(milestones))
      milestones <- names(milestone_time)

      ii <- list() ## calculate independent increments for each of treatment arms
      for(i in seq_along(treatments)){
        trt_str <- treatments[i]

        if(identical(planned_info, 'default')){
          planned_info_ <- 'oracle'
        }else{
          ## this is a vector named by milestone names
          planned_info_ <- planned_info[milestones, , drop = FALSE] %>%
            select(c(placebo, trt_str)) %>%
            rowSums()
        }

        ii[[trt_str]] <-
          self$independentIncrement(formula, placebo, milestones, alternative,
                                    ## it doesn't matter what is used for planned_info
                                    ## because we only use z_ii in returned object
                                    ## which is irrelevant to planned_info
                                    planned_info = planned_info_,
                                    arm %in% c(placebo, trt_str), ...)

      }


      all_trt <- names(ii)

      all_combn <-
        unlist(
          lapply(seq_along(all_trt),
                 function(k)
                   combn(all_trt, k, simplify = FALSE)
                 ),
          recursive = FALSE
        )

      createArmCombination <- function(comb){
        ## use sort() to allow user ignore order of treatments in argument
        paste0(sort(comb), collapse = '|')
      }

      stage_dunnett_pvalue <- list()
      inverse_normal_dunnett_pvalue <- list()
      for(comb in all_combn){
        inverse_normal_dunnett_pvalue[[createArmCombination(comb)]] <- NULL
        for(milestone_name in milestones){ ## milestones is already ordered by triggering time
          z_ii <- NULL
          ratio_trt <- NULL
          stage_n_pbo <- NULL
          available_trt <- NULL
          if(!identical(planned_info, 'default')){
            pinfo <- planned_info[milestone_name, placebo]
          }
          for(trt in comb){
            ii0 <- ii[[trt]] %>% dplyr::filter(milestone %in% milestone_name)

            if(!identical(planned_info, 'default')){
              pinfo <- pinfo + planned_info[milestone_name, trt]
            }

            ## we expect stage_n_pbo is a constant vector
            stage_n_pbo <- c(stage_n_pbo, ii0$stage_n_pbo)


            arm_removal_time <- self$get_arm_removal_time(arm = trt)
            milestone_triggering_time <- self$get_milestone_time(milestone_name = milestone_name)


            if(arm_removal_time >= milestone_triggering_time){
              z_ii <- c(z_ii, ii0$z_ii)
              ratio_trt <- c(ratio_trt,
                             sqrt(ii0$stage_n_trt / (ii0$stage_n_pbo + ii0$stage_n_trt)))

              available_trt <- c(available_trt, trt)
            }
          }

          name1 <- paste0(createArmCombination(comb), '@', milestone_name)
          if(length(available_trt) > 0){
            name2 <- paste0(paste0(sort(available_trt), collapse = '|'), '@', milestone_name)
            if(!is.null(stage_dunnett_pvalue[[name2]])){
              stage_dunnett_pvalue[[name1]] <- stage_dunnett_pvalue[[name2]]
            }else{
              if(alternative == 'greater'){
                lower <- rep(-Inf, length(available_trt))
                upper <- rep(max(z_ii), length(available_trt))
              }else{
                lower <- rep(min(z_ii), length(available_trt))
                upper <- rep(Inf, length(available_trt))
              }

              corr <- outer(ratio_trt, ratio_trt,
                            function(x, y) x * y)
              diag(corr) <- 1.

              if(length(available_trt) > 1){
                ## no matter what alternative is
                stage_dunnett_pvalue[[name1]] <-
                  1 - pmvnorm(lower = lower, upper = upper, sigma = corr)
              }else{
                if(alternative == 'greater'){
                  stage_dunnett_pvalue[[name1]] <- 1 - pnorm(upper)
                }else{
                  stage_dunnett_pvalue[[name1]] <- pnorm(lower)
                }
              }

              stage_dunnett_pvalue[[name2]] <- stage_dunnett_pvalue[[name1]]
            }

          }else{
            stage_dunnett_pvalue[[name1]] <- 1
          }

          if(identical(planned_info, 'default')){
            pinfo <- unique(stage_n_pbo)
            stopifnot(length(pinfo) == 1)
          }

          get_time_variable <- function(formula) {
            lhs <- formula[[2]]  # extract LHS of Surv(...)
            if(as.character(lhs[[1]]) != 'Surv'){
              stop('Left side is not a Surv() object')
            }

            time_var <- lhs[[2]] # the second element is column name of time (the first one is "Surv")
            as.character(time_var)
          }

          tmp <-
            data.frame(
              endpoint = get_time_variable(formula),
              milestone = milestone_name,
              milestone_time = unname(milestone_time[milestone_name]),
              p_logrank = ii0$p_logrank,
              stage_p = stage_dunnett_pvalue[[name1]],
              stage_planned_info = pinfo)

          inverse_normal_dunnett_pvalue[[createArmCombination(comb)]] <-
            rbind(inverse_normal_dunnett_pvalue[[createArmCombination(comb)]], tmp)
          rm(tmp)

          tmp_ <- inverse_normal_dunnett_pvalue[[createArmCombination(comb)]]
          if(any(tmp_$stage_planned_info == 0)){
            idx_ <- which(tmp_$stage_planned_info == 0)[1]
            if(idx_ == 1){
              stop('Using planned_info = "default" in dunnettTest() causes issues under your simulation settings. \n',
                   'It specified weights of closed test based on number of newly randomized patients between milestones. \n',
                   'In your case, no patient is recruited before milestone <', tmp_$milestone[idx_], '>. \n',
                   'Contact the package author if you do not know how to specify <planned_info> after reading manual of Trials$dunnettTest. \n')
            }else{
              stop('Using planned_info = "default" in dunnettTest() causes issues under your simulation settings. \n',
                   'It specified weights of closed test based on number of newly randomized patients between milestones. \n',
                   'In your case, no patient is recruited between milestones <', tmp_$milestone[idx_ - 1],
                   '> and <', tmp_$milestone[idx_], '>. \n',
                   'Contact the package author if you do not know how to specify <planned_info> after reading manual of Trials$dunnettTest. \n')
            }
          }

        }

      }

      for(i in seq_along(inverse_normal_dunnett_pvalue)){
        tmp <- inverse_normal_dunnett_pvalue[[i]]
        tmp$wt <- sqrt(tmp$stage_planned_info)

        tmp$z_inverse_normal <- cumsum(tmp$wt * qnorm(1 - tmp$stage_p)) / sqrt(cumsum(tmp$wt^2))
        tmp$p_inverse_normal <- 1 - pnorm(tmp$z_inverse_normal)
        tmp$planned_info <- cumsum(tmp$stage_planned_info) ## used as observed_info in GroupSequentialTest
        ## the last entry in is_final should be set to TRUE when calling GroupSequentialTest
        ## However, we don't do this here because not all rows in inverse_normal_dunnett_pvalue
        ## will be used to test a specific endpoint (e.g., PFS may not be tested
        ## at all milestone time). Instead, set the last entry to TRUE before
        ## performing the significance test
        # tmp$is_final <- FALSE
        inverse_normal_dunnett_pvalue[[i]] <- tmp
        rm(tmp)
      }

      class(inverse_normal_dunnett_pvalue) <- c('dunnett', class(inverse_normal_dunnett_pvalue))
      inverse_normal_dunnett_pvalue

    },

    #' @description
    #' perform closed test based on Dunnett test
    #' @param dunnett_test object returned by \code{Trial$dunnettTest()}.
    #' @param treatments character vector. Name of treatment arms to be used in
    #' comparison.
    #' @param milestones character vector. Names of triggered milestones at which
    #' significance testing for endpoint is performed in closed test.
    #' Milestones in \code{milestones} does not need to be sorted by their triggering time.
    #' @param alpha numeric. Allocated alpha.
    #' @param alpha_spending alpha spending function. It can be \code{"asP"} or
    #' \code{"asOF"}. Note that theoretically it can be \code{"asUser"}, but
    #' it is not tested. It may be supported in the future.
    #'
    #' @return a data frame of columns \code{arm}, \code{decision}
    #' (final decision on a hypothesis at the end of trial, \code{"accept"} or \code{"reject"}),
    #' \code{milestone_at_reject}, and \code{reject_time}.
    #' If a hypothesis is accepted at then end of a trial,
    #' \code{milestone_at_reject} is \code{NA}, and \code{reject_time} is \code{Inf}.
    #'
    #' Note that if a hypothesis is tested at multiple milestones, the final
    #' \code{decision} will be \code{"accept"} if it is accepted at at least
    #' one milestone. The \code{decision} is \code{"reject"} only if the hypothesis
    #' is rejected at all milestones.
    #'
    #' @examples
    #' \dontrun{
    #' dt <- trial$dunnettTest(
    #'   Surv(pfs, pfs_event) ~ arm,
    #'   placebo = 'pbo',
    #'   treatments = c('high dose', 'low dose'),
    #'   milestones = c('dose selection', 'interim', 'final'),
    #'   data.frame(pbo = c(100, 160, 80),
    #'              low = c(100, 160, 80),
    #'              high = c(100, 160, 80),
    #'              row.names = c('dose selection', 'interim', 'final'))
    #'
    #' trial$closedTest(dt, treatments = c('high dose', 'low dose'),
    #'                  milestones = c('interim', 'final'),
    #'                  alpha = 0.025, alpha_spending = 'asOF')
    #' }
    #'
    closedTest = function(dunnett_test, treatments, milestones, alpha, alpha_spending = c('asP', 'asOF')){

      alpha_spending <- match.arg(alpha_spending)

      if(!('dunnett' %in% class(dunnett_test))){
        stop('dunnett_test must be of class "dunnett". ',
             'Make sure that it is returned by dunnettTest(). ')
      }

      all_combn <-
        unlist(
          lapply(seq_along(treatments),
                 function(k)
                   combn(treatments, k, simplify = FALSE)
          ),
          recursive = FALSE
        )

      for(i in seq_along(dunnett_test)){

        if(all(milestones %in% dunnett_test[[i]]$milestone)){
          dunnett_test[[i]] <- dunnett_test[[i]] %>%
            dplyr::filter(milestone %in% milestones) %>%
            arrange(milestone_time) %>%
            mutate(is_final = FALSE)

          dunnett_test[[i]]$is_final[nrow(dunnett_test[[i]])] <- TRUE
        }else{
          stop('Milestones <',
               paste0(setdiff(milestones, dunnett_test[[i]]$milestone), collapse = ', '),
               '> are not in dunnett_test. ')
        }
      }

      createArmCombination <- function(comb){
        ## use sort() to allow user ignore order of treatments in argument
        paste0(sort(comb), collapse = '|')
      }

      for(comb in all_combn){
        name1 <- createArmCombination(comb)
        if(!(name1 %in% names(dunnett_test))){
          stop('Combination <', name1, '> is not found in dunnett_test. ',
               'Make sure that <', paste0(treatments, collapse = ', '),
               '> are all used with dunnettTest() when computing dunnett_test. ')
        }
      }

      gst <- list()
      gst_res <- list()
      for(i in seq_along(dunnett_test)){
        comb <- names(dunnett_test)[i]
        gst[[comb]] <- GroupSequentialTest$new(
          name = comb,
          alpha = alpha, alpha_spending = alpha_spending,
          planned_max_info = max(dunnett_test[[comb]]$planned_info)
        )

        gst[[comb]]$test(observed_info = dunnett_test[[comb]]$planned_info,
                         is_final = dunnett_test[[comb]]$is_final,
                         p_values = dunnett_test[[comb]]$p_inverse_normal)

        gst_res[[comb]] <- gst[[comb]]$get_trajectory() %>%
          mutate(milestone = dunnett_test[[comb]]$milestone) %>%
          mutate(milestone_time = dunnett_test[[comb]]$milestone_time)
      }

      # print(gst_res)

      tmp0 <-
        data.frame(
          arm = NA,
          comb = NA,
          reject = FALSE,
          milestone_at_reject = NA,
          reject_time = Inf,
          stageLevels = NA_real_,
          obs_p_value = NA_real_
        )

      ret <- list()
      for(trt in treatments){

        ret[[trt]] <- NULL
        for(comb in names(gst_res)){
          if(!(trt %in% unlist(strsplit(comb, split = '\\|')))){
            next
          }

          tmp <- tmp0
          tmp$arm <- trt
          tmp$comb <- comb
          if('reject' %in% gst_res[[comb]]$decision){ ## trial reach the endpoint
            tmp$reject <- TRUE
            idx <- which(gst_res[[comb]]$decision %in% 'reject')[1]
            tmp$milestone_at_reject <- gst_res[[comb]]$milestone[idx]
            tmp$reject_time <- gst_res[[comb]]$milestone_time[idx]
            tmp$stageLevels <- gst_res[[comb]]$stageLevels[idx]
            tmp$obs_p_value <- gst_res[[comb]]$obs_p_value[idx]
          }

          ret[[trt]] <- rbind(ret[[trt]], tmp)
          rm(tmp)
        }

      }


      # print(ret)
      ret_ <- NULL
      for(trt in names(ret)){
        if(all(ret[[trt]]$reject)){
          idx <- which.max(ret[[trt]]$reject_time)[1]
          ret_ <- rbind(ret_,
                        data.frame(
                          arm = trt,
                          decision = 'reject',
                          milestone_at_reject = ret[[trt]]$milestone_at_reject[idx],
                          reject_time = ret[[trt]]$reject_time[idx]))
        }else{
          ret_ <- rbind(ret_,
                        data.frame(
                          arm = trt,
                          decision = 'accept',
                          milestone_at_reject = NA,
                          reject_time = Inf))
        }
      }
      ret_
    },

    #' @description
    #' return random seed
    get_seed = function(){
      private$seed
    },

    #' @description
    #' print a trial
    print = function(){
      white_text_blue_bg <- "" ## "\033[37;44m"
      reset <- "" ## "\033[0m"  # Reset to default color
      logo <- '\u2695\u2695' ## stringi::stri_escape_unicode('⚕')

      cat(white_text_blue_bg, logo, '        Trial Name: ', self$get_name(), reset, '\n')
      cat(white_text_blue_bg, logo, '       Description: ', self$get_description(), reset, '\n')
      cat(white_text_blue_bg, logo, '    Number of Arms: ', self$get_number_arms(), reset, '\n')
      cat(white_text_blue_bg, logo, '   Registered Arms: ',
          paste0(self$get_arms_name(), collapse = ', '), reset, '\n')
      cat(white_text_blue_bg, logo, '      Sample Ratio: ',
          paste0(self$get_sample_ratio(), collapse = ', '), reset, '\n')
      cat(white_text_blue_bg, logo, 'Number of Patients: ', self$get_number_patients(), reset, '\n')
      cat(white_text_blue_bg, logo, '  Planned Duration: ', self$get_duration(), reset, '\n')
      cat(white_text_blue_bg, logo, '           Regimen: ', ifelse(is.null(self$get_regimen()), 'not set', 'set'), reset, '\n')
      cat(white_text_blue_bg, logo, '       Random Seed: ', self$get_seed(), reset, '\n')

      invisible(self)

    },

    #' @description
    #' return a snapshot of a trial before it is executed.
    get_snapshot_copy = function(){
      private$.snapshot
    },

    #' @description
    #' make a snapshot before running a trial. This can be useful when
    #' resetting a trial. This is only called when initializing a `Trial`
    #' object, when arms have not been added yet.
    make_snapshot = function() {

      private$.snapshot <- list()

      for(field in names(private)){
        if(field %in% c('.snapshot', 'permuted_block_randomization', 'validate_arguments')){
          next
        }
        private$.snapshot[[field]] <- private[[field]]
      }

    },

    #' @description
    #' make a snapshot of arms
    make_arms_snapshot = function(){
      arm_names <- self$get_arms_name()
      arms <- self$get_arms()
      sample_ratio <- self$get_sample_ratio()
      stopifnot(length(arms) == length(sample_ratio))
      stopifnot(length(arm_names) == length(sample_ratio))

      private$.snapshot$arms <- list()
      for(arm_name in arm_names){
        private$.snapshot$arms[[arm_name]] <- arms[[arm_name]]$clone(deep = TRUE)
      }
      private$.snapshot$sample_ratio <- sample_ratio

    },

    #' @description
    #' reset a trial to its snapshot taken before it was executed. Seed will be
    #' reassigned with a new one. Enrollment time are re-generated. If the trial
    #' already have arms when this function is called, they are added back to
    #' recruit patients again.
    reset = function() {

      arms <- private$.snapshot[['arms']]
      sample_ratio <- private$.snapshot[['sample_ratio']]

      if(is.null(arms) || is.null(sample_ratio)){
        warning('arms is not found in the snapshot. ',
                'There is nothing to be reset. ')
        return(invisible(NULL))
      }

      for (field in names(private$.snapshot)){
        private[[field]] <- private$.snapshot[[field]]
      }

      private$milestone_time <- c()
      private$trial_data <- NULL
      private$enroll_time <- NULL
      private$enroll_time_with_redundant <- NULL
      private$randomization_queue <- NULL
      private$n_enrolled_patients <- NULL
      private$sample_ratio <- NULL
      private$arms <- list()

      private$seed <- sample(.Machine$integer.max, 1)
      private$output$seed <- private$seed
      set.seed(private$seed)

      ## see comments in initialize()
      private$enroll_time_with_redundant <-
        sort(self$get_enroller()(n = private$n_patients * 10), decreasing = FALSE)
      private$enroll_time <- head(private$enroll_time_with_redundant, private$n_patients)
      private$enroll_time_with_redundant <- private$enroll_time_with_redundant[-c(1:private$n_patients)]

      arms <- private$.snapshot[['arms']]
      sample_ratio <- private$.snapshot[['sample_ratio']]

      if(length(arms) > 0){
        stopifnot(length(arms) == length(sample_ratio))
        ## must call add_arms to add arms back to the trial
        ## because this will generate some data as well.
        ## set self$arms <- ... does not fulfill the purpose of
        ## resetting a trial
        do.call(self$add_arms, c(list(sample_ratio), arms))
      }

    },

    #' @description
    #' save time when an arm is added to the trial
    #' @param arm name of added arm.
    #' @param time time when an arm is added.
    set_arm_added_time = function(arm, time){
      if(!is.null(private$arm_time[[arm]][['time_added']])){
        stop('The time the arm <', arm, '> was added to the trial has already been recorded <',
             private$arm_time[[arm]][['time_added']], '>. You cannot overwrite it. ',
             'Usually this indicates an error in your codes. ')
      }
      stopifnot(time >= 0)
      private$arm_time[[arm]][['time_added']] <- time
    },

    #' @description
    #' get time when an arm is added to the trial
    #' @param arm arm name.
    get_arm_added_time = function(arm){
      ## arm is not in the trial
      if(is.null(private$arm_time[[arm]][['time_added']])){
        if(!private$silent){
          message('Arm <', arm, '> is not in the trial. ')
        }
        return(Inf)
      }else{
        return(private$arm_time[[arm]][['time_added']])
      }
    },

    #' @description
    #' save time when an arm is removed to the trial
    #' @param arm name of removed arm.
    #' @param time time when an arm is removed.
    set_arm_removal_time = function(arm, time){
      if(!is.null(private$arm_time[[arm]][['time_removed']])){
        stop('The time the arm <', arm, '> was removed from the trial has already been recorded <',
             private$arm_time[[arm]][['time_removed']], '>. You cannot overwrite it. ',
             'Usually this indicates an error in your codes. ')
      }
      stopifnot(time >= 0)
      private$arm_time[[arm]][['time_removed']] <- time
    },

    #' @description
    #' get time when an arm is removed from the trial
    #' @param arm arm name.
    get_arm_removal_time = function(arm){
      ## arm is not in the trial
      if(is.null(private$arm_time[[arm]][['time_removed']])){
        if(!private$silent){
          # message('Arm <', arm, '> is still in the trial. ')
        }
        return(Inf)
      }else{
        return(private$arm_time[[arm]][['time_removed']])
      }
    },

    #' @description
    #' return stratification factors
    get_stratification_factors = function(){
      private$stratification_factors
    },

    #' @description
    #' has stratification factors
    has_stratification_factors = function(){
      !is.null(self$get_stratification_factors())
    }

  ),

  private = list(
    seed = NULL,
    name = NULL,
    description = NULL,
    n_patients = NULL,
    duration = NULL,
    n_enrolled_patients = NULL,
    sample_ratio = NULL,

    arms = list(),
    now = 0, # current time point of a trial. Change to the data of patients
             # enrolled before are not allowed. When a trial is created,
             # now = 0. If a milestone triggers a data lock (in Milestones class),
             # now will be set to the time of data lock (e.g. futility, interim).
             # When adding or removing a arm at a milestone,
             # private$randomizatioon_queue[private$enroll_time > now] will be
             # regenerated. This is important because randomization needs to be
             # done with possibly changed sample ratio. enroll_patients() is
             # then executed with updated randomizatioon_queue.
    randomization_queue = NULL,
    enroller = NULL,
    enroll_time = NULL,
    enroll_time_with_redundant = NULL,
    stratum_queue = NULL,

    dropout = NULL, # function to generate dropout time

    stratification_factors = NULL,

    trial_data = NULL,
    locked_data = list(),

    milestone_time = c(),
    arm_time = list(), # time when arms are added to or removed from the trial

    regimen = NULL,

    silent = FALSE,

    output = NULL,

    ## User can save whatever they want in an unstructured way (list)
    ## This is useful for simulation to store some setting parameters
    ## that could be used in action functions.
    custom_data = list(),

    validate_arguments =
      function(name, n_patients, duration, description, seed,
               enroller, dropout, stratification_factors, silent, ...){

      stopifnot(is.null(seed) || is.wholenumber(seed))
      stopifnot(is.character(name))
      stopifnot(is.character(description))
      stopifnot(is.null(stratification_factors) ||
                  is.character(stratification_factors))

      stopifnot(is.numeric(n_patients) &&
                  (length(n_patients) == 1) &&
                  is.wholenumber(n_patients))

      stopifnot(is.numeric(duration) &&
                  (length(duration) == 1) &&
                  duration > 0)

      stopifnot(is.function(enroller))
      stopifnot(is.null(dropout) || is.function(dropout))

      stopifnot(is.logical(silent))

    },

    ## Whenever arms are added or removed from the trial,
    ## permuted_block_randomization should be called to update the randomization
    ## plan for unenrolled patients, up to the maximum planned sample size.
    ## This function will be called by add_arms, remove_arms, etc.
    permuted_block_randomization = function(block_size = NULL){

      if(!is.null(self$get_randomization_queue())){
        message('condition check is triggered in permuted_block_randomization. ',
        ' Debug this.\n')
        stopifnot(
          length(self$get_randomization_queue()) ==
            self$get_number_unenrolled_patients())
      }

      if(self$get_number_unenrolled_patients() == 0){
        stop('All patients are enrolled. No further randomization is needed. \n',
             'If you see this message, there is probably an unexpected issue with your code. \n',
             'One known reason is that arms are added into the trial one right after one, \n',
             'e.g., calling $add_arms twice and no milestone happen in between. \n',
             'However, whenever a milestone is triggered during a trial, ',
             'patients being enrolled after the milestone time will be rolled back, ',
             'so that a new arm can be removed or added. \n',
             'Those patients will be randomized again. \n')
      }


      if(!all(is.wholenumber(private$sample_ratio))){
        if(!private$silent){
          message('Permuted block randomization is replaced with sample() ',
                  'as not all sample ratios are whole numbers. ',
                  'This is useful if response-adaptive randomization is adopted. ')
        }

        arm_names <- names(private$sample_ratio)
        private$randomization_queue <- sample(arm_names,
                                              size = self$get_number_unenrolled_patients(),
                                              replace = TRUE,
                                              prob = private$sample_ratio)
        private$stratum_queue <- rep('all', self$get_number_unenrolled_patients())

        if(!private$silent){
          message('Randomization is done for ',
                  self$get_number_unenrolled_patients(),
                  ' potential patients. \n')
        }

        return(invisible(NULL))
      }


      if(is.null(block_size)){
        block_size <- sum(self$get_sample_ratio())
      }

      ## create stratum and randomization queue
      if(!self$has_arm()){
        stop('No arm registered to the trial yet. Cannot randomize patients. ')
      }

      if(self$has_stratification_factors()){
        any_arm <- self$get_an_arm(self$get_arms_name()[1])
        stratum_queue <- any_arm$generate_data(self$get_number_unenrolled_patients())
        stratum_queue <-
          do.call(paste, c(stratum_queue[, self$get_stratification_factors(), drop = FALSE],
                           sep = '|')
          )

        size_per_stratum <- c(table(stratum_queue))
      }else{
        stratum_queue <- rep('all', self$get_number_unenrolled_patients())
        size_per_stratum <- c(all = self$get_number_unenrolled_patients())
      }

      arm_names <- names(private$sample_ratio)
      block <- rep(seq(private$sample_ratio), times = private$sample_ratio)
      randomization_queue <- list()
      for(str in names(size_per_stratum)){
        blocks <- rep(block, length.out = size_per_stratum[str])
        randomization_queue[[str]] <-
          lapply(
            split(blocks, ceiling(seq_along(blocks) / block_size)),
            sample
          ) %>%
          unlist() %>%
          unname()
        randomization_queue[[str]] <- arm_names[randomization_queue[[str]]]
      }

      private$randomization_queue <- rep(NA, self$get_number_unenrolled_patients())
      for(str in names(size_per_stratum)){
        private$randomization_queue[stratum_queue %in% str] <- randomization_queue[[str]]
      }

      private$stratum_queue <- stratum_queue

      stopifnot(length(private$stratum_queue) == length(private$randomization_queue))

      if(!private$silent){
        message('Randomization is done for ', length(randomization_queue),
                ' potential patients. \n')
      }
    },

    .snapshot = list()

  )
)

# Trials, Listeners, and plot-method coverage
#
# Covers:
#   - Trials: print, get_description, get_arms_name, has_arm, get_sample_ratio,
#     get_number_arms, get_seed, event_plot, get_output, get_milestone_time
#   - Listeners: add_milestones errors on duplicates, print, get_milestone_names
#   - plot.three_state_model: runs to completion without error
#   - Endpoints$print output: exercises non-tte with readout path

test_that("Trials accessors and print reflect the constructed trial", {

  pbo <- arm(name = "pbo")
  pbo$add_endpoints(endpoint(name = "pfs", type = "tte",
                             generator = rexp, rate = log(2) / 8))
  trt <- arm(name = "trt")
  trt$add_endpoints(endpoint(name = "pfs", type = "tte",
                             generator = rexp, rate = log(2) / 12))

  tr <- trial(name = "demo", n_patients = 200, duration = 30, seed = 42L,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 20),
              silent = TRUE)
  tr$add_arms(sample_ratio = c(1, 1), pbo, trt)

  expect_equal(tr$get_seed(), 42L)
  expect_equal(sort(tr$get_arms_name()), sort(c("pbo", "trt")))
  expect_equal(tr$get_number_arms(), 2)
  expect_true(tr$has_arm())

  ratio <- tr$get_sample_ratio()
  expect_true(is.numeric(ratio) || is.list(ratio))

  desc <- tr$get_description()
  expect_true(is.character(desc) || is.null(desc))

  out <- capture.output(print(tr))
  expect_true(length(out) > 0)
})

test_that("Trials event_plot runs without error after a run", {

  pbo <- arm(name = "pbo")
  pbo$add_endpoints(endpoint(name = "pfs", type = "tte",
                             generator = rexp, rate = log(2) / 6))
  trt <- arm(name = "trt")
  trt$add_endpoints(endpoint(name = "pfs", type = "tte",
                             generator = rexp, rate = log(2) / 10))

  tr <- trial(name = "t", n_patients = 200, duration = 30, seed = 1L,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 20),
              silent = TRUE)
  tr$add_arms(sample_ratio = c(1, 1), pbo, trt)

  lstn <- listener(silent = TRUE)
  lstn$add_milestones(milestone(name = "final",
                                when = calendarTime(time = 30)))

  ctrl <- controller(tr, lstn)
  ctrl$run(n = 1, silent = TRUE, plot_event = FALSE)

  pdf(NULL); on.exit(dev.off())
  expect_no_error(tr$event_plot())
  expect_gte(tr$get_milestone_time("final"), 0)
})

test_that("Listeners reject duplicate milestones and list their names", {

  lstn <- listener(silent = TRUE)
  m1 <- milestone(name = "m1", when = calendarTime(time = 10))
  m2 <- milestone(name = "m2", when = calendarTime(time = 20))
  dup <- milestone(name = "m1", when = calendarTime(time = 30))

  lstn$add_milestones(m1, m2)
  expect_equal(sort(lstn$get_milestone_names()),
               sort(c("m1", "m2")))

  expect_warning(lstn$add_milestones(dup), regexp = "over-write|already")

  out <- capture.output(print(lstn))
  expect_true(length(out) > 0)
})

test_that("plot.three_state_model runs to completion", {

  skip_if_not_installed("ggplot2")

  set.seed(7)
  ret <- solveThreeStateModel(median_pfs = 4.6, median_os = 9.6,
                              corr = c(.5, .6),
                              h12 = seq(.08, .2, length.out = 8))
  pdf(NULL); on.exit(dev.off())
  expect_no_error(plot(ret))
})

test_that("Endpoints print handles tte and non-tte with readout", {

  ep_tte <- endpoint(name = "pfs", type = "tte",
                     generator = rexp, rate = 0.1)
  ep_non <- endpoint(name = "orr", type = "non-tte",
                     readout = c(orr = 6), generator = rnorm)

  out_tte <- capture.output(print(ep_tte))
  out_non <- capture.output(print(ep_non))
  expect_true(length(out_tte) > 0)
  expect_true(length(out_non) > 0)
})

test_that("Trials save/get_custom_data and get_output round-trip", {

  pbo <- arm(name = "pbo")
  pbo$add_endpoints(endpoint(name = "pfs", type = "tte",
                             generator = rexp, rate = 0.1))

  tr <- trial(name = "t", n_patients = 100, duration = 20, seed = 3L,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 20),
              silent = TRUE)
  tr$add_arms(sample_ratio = 1, pbo)

  act <- function(trial) {
    trial$save(value = 42L, name = "answer")
    trial$save_custom_data(value = list(x = 1), name = "cfg")
  }

  lstn <- listener(silent = TRUE)
  lstn$add_milestones(milestone(name = "final",
                                action = act,
                                when = calendarTime(time = 20)))

  ctrl <- controller(tr, lstn)
  ctrl$run(n = 1, silent = TRUE, plot_event = FALSE)

  op <- ctrl$get_output()
  expect_true("answer" %in% names(op))
  expect_equal(op$answer[1], 42)

  expect_equal(tr$get_custom_data("cfg"), list(x = 1))
})

# Statistical model fitting
#
# Covers:
#   - fitLinear: coefficient and ATE in additive model with and without covariates
#   - fitLogistic: log OR, OR, RR, RD scales without covariates; regression
#                  coefficient with covariates and interaction terms
#   - fitLogrank: agreement with survdiff/coxph for one-sided test
#   - fitCoxph: main effect of arm with covariates and interaction terms

test_that('fitLinear works as expected', {


  ep <- endpoint(name = 'ep', type = 'tte', generator = rnorm)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'tte', generator = rnorm, mean = .1)
  trt <- arm(name = 'trt')
  trt$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 2), pbo, trt)

  act <- function(trial){

    locked_data <- trial$get_locked_data('final')

    n <- nrow(locked_data)
    locked_data$covar1 <- rnorm(n)
    locked_data$covar2 <- rbinom(n, 1, .4)

    ## ATE is equivalent to model coefficient when no interaction term in linear regression
    fit <- fitLinear(ep ~ arm + covar1 + covar2, placebo = 'pbo', data = locked_data, alternative = 'greater')
    trial$save(value = fit, name = 'fitLinear_output')
    fit_ <- lm(ep ~ I(arm != 'pbo') + covar1 + covar2, data =locked_data)
    trial$save(value = data.frame(z = summary(fit_)$coef[2, 't value']) %>%
                 mutate(p = 1 - pt(z, df = fit_$df.residual - 2)) %>%
                 mutate(info = fit_$df.residual + fit_$rank),
               name = 'lm_output')

  }

  final <- milestone(name = 'final',
                     action = act,
                     when = calendarTime(time = 40))

  listener <- listener(silent = TRUE)
  listener$add_milestones(final)

  controller <- controller(trial, listener)
  controller$run(n = 10, plot_event = FALSE, silent = TRUE)

  op <- controller$get_output()

  expect_equal(op$`fitLinear_output_<p>`, op$`lm_output_<p>`, tolerance = 1e-3)
  expect_equal(op$`fitLinear_output_<z>`, op$`lm_output_<z>`, tolerance = 1e-3)
  expect_equal(op$`fitLinear_output_<info>`, op$`lm_output_<info>`)
  expect_true(all(op$`fitLinear_output_<arm>` == 'trt'))
  expect_true(all(op$`fitLinear_output_<placebo>` == 'pbo'))

})

test_that('fitLinear can compute ATE as expected in additive model', {

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 0), generator = rnorm)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 0), generator = rnorm, mean = .1)
  trt <- arm(name = 'trt')
  trt$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 2), pbo, trt)

  act <- function(trial){

    locked_data <- trial$get_locked_data('final')

    n <- nrow(locked_data)
    locked_data$covar1 <- rnorm(n)
    locked_data$covar2 <- rbinom(n, 1, .4)

    ## ATE is equivalent to model coefficient when no interaction term in linear regression
    fit <- fitLinear(ep ~ arm + covar1 + covar2, placebo = 'pbo', data = locked_data, alternative = 'greater')
    trial$save(value = fit, name = 'fitLinear_output')

    fit_ <- lm(ep ~ I(arm != 'pbo') + covar1 + covar2, data = locked_data)
    trial$save(value = data.frame(z = summary(fit_)$coef[2, 't value'],
                                  estimate = summary(fit_)$coef[2, 'Estimate']) %>%
                 mutate(p = 1 - pt(z, df = fit_$df.residual - 2)) %>%
                 mutate(info = fit_$df.residual + fit_$rank),
               name = 'lm_output')

  }

  final <- milestone(name = 'final',
                     action = act,
                     when = calendarTime(time = 40))

  listener <- listener(silent = TRUE)
  listener$add_milestones(final)

  controller <- controller(trial, listener)
  controller$run(n = 10, plot_event = FALSE, silent = TRUE)

  op <- controller$get_output()

  expect_equal(op$`fitLinear_output_<p>`, op$`lm_output_<p>`, tolerance = 1e-3)
  expect_equal(op$`fitLinear_output_<z>`, op$`lm_output_<z>`, tolerance = 1e-3)
  expect_equal(op$`fitLinear_output_<estimate>`, op$`lm_output_<estimate>`, tolerance = 1e-3)
  expect_equal(op$`fitLinear_output_<info>`, op$`lm_output_<info>`)
  expect_true(all(op$`fitLinear_output_<arm>` == 'trt'))
  expect_true(all(op$`fitLinear_output_<placebo>` == 'pbo'))

})

test_that('fitLogistic can compute ATE as expected in model without covariates', {

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 0), generator = rnorm)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 0), generator = rnorm, mean = .1)
  trt <- arm(name = 'trt')
  trt$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 2), pbo, trt)

  act <- function(trial){

    locked_data <- trial$get_locked_data('final')

    trial$save(value = nrow(locked_data), name = 'n')

    ## ATE is equivalent to log OR estimate when no covariate in logistic regression
    fit_logOR <- fitLogistic(I(ep > 0) ~ arm, placebo = 'pbo',
                             data = locked_data, alternative = 'greater',
                             scale = 'log odds ratio')

    trial$save(value = fit_logOR, name = 'fit_logOR')

    fit_OR <- fitLogistic(I(ep > 0) ~ arm, placebo = 'pbo',
                          data = locked_data, alternative = 'greater',
                          scale = 'odds ratio')

    trial$save(value = fit_OR, name = 'fit_OR')

    ## ATE is equivalent to ratio of arm probabilities when no covariate in logistic regression
    fit_RR <- fitLogistic(I(ep > 0) ~ arm, placebo = 'pbo',
                          data = locked_data, alternative = 'greater',
                          scale = 'risk ratio')

    trial$save(value = fit_RR, name = 'fit_RR')

    ## ATE is equivalent to difference of arm probabilities when no covariate in logistic regression
    fit_RD <- fitLogistic(I(ep > 0) ~ arm, placebo = 'pbo',
                          data = locked_data, alternative = 'greater',
                          scale = 'risk difference')

    trial$save(value = fit_RD, name = 'fit_RD')

    fit <- glm(I(ep > 0) ~ I(arm != 'pbo'), data = locked_data, family = 'binomial')

    trial$save(value = data.frame(estimate = summary(fit)$coef[2, 'Estimate'],
                                  z = summary(fit)$coef[2, 'z value']) %>%
                 mutate(p = 1 - pnorm(z)) %>%
                 mutate(info = fit$df.residual + fit$rank),
               name = 'glm_logOR')

    probs <- locked_data %>%
      group_by(arm) %>%
      summarise(prob = mean(ep > 0))

    trial$save(value = exp(summary(fit)$coef[2, 'Estimate']),
               name = 'glm_OR')

    trial$save(value = probs$prob[probs$arm == 'trt'] / probs$prob[probs$arm == 'pbo'],
               name = 'glm_RR')

    trial$save(value = probs$prob[probs$arm == 'trt'] - probs$prob[probs$arm == 'pbo'],
               name = 'glm_RD')

  }

  final <- milestone(name = 'final',
                     action = act,
                     when = calendarTime(time = 40))

  listener <- listener(silent = TRUE)
  listener$add_milestones(final)

  controller <- controller(trial, listener)
  controller$run(n = 10, plot_event = FALSE, silent = TRUE)

  op <- controller$get_output()

  #################

  expect_true(all(op$`fit_logOR_<arm>` == 'trt'))
  expect_true(all(op$`fit_OR_<arm>` == 'trt'))
  expect_true(all(op$`fit_RR_<arm>` == 'trt'))
  expect_true(all(op$`fit_RD_<arm>` == 'trt'))

  expect_true(all(op$`fit_logOR_<placebo>` == 'pbo'))
  expect_true(all(op$`fit_OR_<placebo>` == 'pbo'))
  expect_true(all(op$`fit_RR_<placebo>` == 'pbo'))
  expect_true(all(op$`fit_RD_<placebo>` == 'pbo'))

  expect_equal(op$`fit_logOR_<estimate>`, op$`glm_logOR_<estimate>`, tolerance = 1e-3)

  expect_equal(op$`fit_logOR_<p>`, op$`glm_logOR_<p>`, tolerance = 1e-3)
  expect_equal(op$`fit_OR_<p>`, op$`glm_logOR_<p>`, tolerance = 1e-3)

  expect_equal(op$`fit_logOR_<z>`, op$`glm_logOR_<z>`, tolerance = 1e-3)
  expect_equal(op$`fit_OR_<z>`, op$`glm_logOR_<z>`, tolerance = 1e-3)

  expect_equal(op$`fit_OR_<estimate>`, op$`glm_OR`, tolerance = 1e-3)
  expect_equal(op$`fit_RR_<estimate>`, op$`glm_RR`, tolerance = 1e-3)
  expect_equal(op$`fit_RD_<estimate>`, op$`glm_RD`, tolerance = 1e-3)

  expect_identical(op$`fit_logOR_<info>`, op$n)
  expect_identical(op$`fit_logOR_<info>`, op$`fit_OR_<info>`)
  expect_identical(op$`fit_logOR_<info>`, op$`fit_RR_<info>`)
  expect_identical(op$`fit_logOR_<info>`, op$`fit_RD_<info>`)


})

test_that('fitLogistic can compute regression coefficient as expected in model with covariates', {

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 0), generator = rnorm)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 0), generator = rnorm, mean = .1)
  trt <- arm(name = 'trt')
  trt$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 2), pbo, trt)

  act <- function(trial){

    locked_data <- trial$get_locked_data('final')
    locked_data$x <- rnorm(nrow(locked_data))
    locked_data$y <- rnorm(nrow(locked_data))
    locked_data$z <- rbinom(nrow(locked_data), 1, .5)

    trial$save(value = nrow(locked_data), name = 'n')

    fit_coef <- fitLogistic(I(ep > 0) ~ x*arm + z + arm:y, placebo = 'pbo',
                            data = locked_data, alternative = 'greater',
                            scale = 'coefficient')
    fit <- glm(I(ep > 0) ~ arm*x + z + arm:y, data = locked_data,
               family = binomial)

    trial$save(value = fit_coef, name = 'fit_coef')
    trial$save(value = data.frame(estimate = summary(fit)$coef['armtrt', 'Estimate'],
                                  z = summary(fit)$coef['armtrt', 'z value']) %>%
                 mutate(p = 1 - pnorm(z)) %>%
                 mutate(info = fit$df.residual + fit$rank),
               name = 'glm_coef')

  }

  final <- milestone(name = 'final',
                     action = act,
                     when = calendarTime(time = 40))

  listener <- listener(silent = TRUE)
  listener$add_milestones(final)

  controller <- controller(trial, listener)
  controller$run(n = 10, plot_event = FALSE, silent = TRUE)

  op <- controller$get_output()

  #################

  expect_true(all(op$`fit_coef_<arm>` == 'trt'))

  expect_true(all(op$`fit_coef_<placebo>` == 'pbo'))

  expect_equal(op$`fit_coef_<estimate>`, op$`glm_coef_<estimate>`, tolerance = 1e-3)

  expect_equal(op$`fit_coef_<p>`, op$`glm_coef_<p>`, tolerance = 1e-3)

  expect_equal(op$`fit_coef_<z>`, op$`glm_coef_<z>`, tolerance = 1e-3)

  expect_identical(op$`fit_coef_<info>`, op$n)

})

test_that('fitLogrank works as expected', {

  ep <- endpoint(name = 'ep', type = 'tte', generator = rexp, rate = log(2)/10)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'tte', generator = rexp, rate = log(2)/12)
  trt <- arm(name = 'trt')
  trt$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 2), pbo, trt)

  act <- function(trial){

    locked_data <- trial$get_locked_data('final')

    fit <- fitLogrank(Surv(ep, ep_event) ~ arm, placebo = 'pbo', data = locked_data, alternative = 'less')

    lr <- survdiff(Surv(ep, ep_event) ~ arm, data = locked_data)
    fit_ <- coxph(Surv(ep, ep_event) ~ arm, data = locked_data)

    p <- fit$p
    z <- sqrt(lr$chisq) * ifelse(coef(fit_)['armtrt'] > 0, 1, -1)
    p_ <- pnorm(z)

    trial$save(value = p, name = 'logrank_p')
    trial$save(value = p_, name = 'survdiff_p')

  }

  final <- milestone(name = 'final',
                     action = act,
                     when = calendarTime(time = 40))

  listener <- listener(silent = TRUE)
  listener$add_milestones(final)

  controller <- controller(trial, listener)
  controller$run(n = 10, plot_event = FALSE, silent = TRUE)

  op <- controller$get_output()

  expect_equal(op$logrank_p, op$survdiff_p, tolerance = 1e-3)

})

test_that('fitCoxph can compute main effect of arm', {

  ep <- endpoint(name = 'ep', type = 'tte', generator = rexp, rate = log(2)/10)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'tte', generator = rexp, rate = log(2)/12)
  trt <- arm(name = 'trt')
  trt$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 2), pbo, trt)

  act <- function(trial){

    locked_data <- trial$get_locked_data('final')

    n <- nrow(locked_data)
    locked_data$covar1 <- rnorm(n)
    locked_data$covar2 <- rbinom(n, 1, .4)

    fit <- fitCoxph(Surv(ep, ep_event) ~ covar2 + arm*covar1 + covar1:covar2,
                    placebo = 'pbo', data = locked_data, alternative = 'less',
                    scale = 'hazard ratio')
    trial$save(value = fit, name = 'fitCoxph_output')

    fit_ <- coxph(Surv(ep, ep_event) ~ I(arm != 'pbo')*covar1 + covar1:covar2 + covar2, data = locked_data)

    trial$save(value = data.frame(z = summary(fit_)$coef[1, 'z'],
                                  estimate = exp(summary(fit_)$coef[1, 'coef'])) %>%
                 mutate(p = pnorm(z)) %>%
                 mutate(info = sum(locked_data$ep_event)),
               name = 'coxph_output')

  }

  final <- milestone(name = 'final',
                     action = act,
                     when = calendarTime(time = 40))

  listener <- listener(silent = TRUE)
  listener$add_milestones(final)

  controller <- controller(trial, listener)
  controller$run(n = 10, plot_event = FALSE, silent = TRUE)

  op <- controller$get_output()

  expect_equal(op$`fitCoxph_output_<p>`, op$`coxph_output_<p>`, tolerance = 1e-3)
  expect_equal(op$`fitCoxph_output_<z>`, op$`coxph_output_<z>`, tolerance = 1e-3)
  expect_equal(op$`fitCoxph_output_<estimate>`, op$`coxph_output_<estimate>`, tolerance = 1e-3)
  expect_equal(op$`fitCoxph_output_<info>`, op$`coxph_output_<info>`)
  expect_true(all(op$`fitCoxph_output_<arm>` == 'trt'))
  expect_true(all(op$`fitCoxph_output_<placebo>` == 'pbo'))

})

# Utility functions and censoring invariants
#
# Covers:
#   - solvePiecewiseConstantExponentialDistribution + qPiecewiseExponential:
#     round-trip accuracy and agreement with rpact reference implementation
#   - censor_trial_data: no negative TTE after dropout censoring
#   - censor_trial_data: enroll_time + TTE <= lock_time for all patients
#   - censor_trial_data: calendar-time invariant holds across all arms
#   - Stratified randomization: per-stratum arm balance with
#     stratification_factors
#   - enrollment(arms = ...): milestone counts only the specified arms

test_that('functions of piecewise exponential distribution work fine', {

  for(n in sample(5:10, 20, TRUE)){
    surv_prob <- sort(runif(n), decreasing = TRUE)
    times <- sort(runif(n, 0, 10), decreasing = FALSE)
    x <- solvePiecewiseConstantExponentialDistribution(
      surv_prob = surv_prob,
      times = times
    )

    times <- x$end_time
    rates <- x$piecewise_risk

    qs <- qPiecewiseExponential(
      1 - surv_prob,
      times = times, piecewise_risk = c(rates, .1)
    )

    expect_equal(qs, times, tolerance = 1e-4)

    p <- c(0, 1, runif(1e3))

    u <-
      data.frame(
        a = qPiecewiseExponential(
          p,
          times = x$end_time,
          piecewise_risk = c(x$piecewise_risk, .1)),

        b = rpact::getPiecewiseExponentialQuantile(
          p,
          piecewiseSurvivalTime = c(0,x$end_time),
          piecewiseLambda=c(x$piecewise_risk, .1))
      )
    expect_equal(u$a, u$b, tolerance = 1e-4)

  }

})

## ── censor_trial_data: dropout censoring ──────────────────────────────────────
test_that("censor_trial_data: dropout censoring sets event=0 and clips tte", {

  skip_if_not_installed("TrialSimulator")

  set.seed(42)
  pfs_ep <- endpoint(name = "pfs", type = "tte",
                     generator = rexp, rate = log(2) / 6)
  a <- arm(name = "trt")
  a$add_endpoints(pfs_ep)

  tr <- trial(name = "t", n_patients = 200, seed = 1,
              duration = 100, enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 10),
              dropout = rexp, rate = 0.5,   # heavy dropout
              silent = TRUE)
  tr$add_arms(sample_ratio = 1, a)

  lstn <- listener(silent = TRUE)
  lstn$add_milestones(milestone(name = "final",
                                when = calendarTime(time = 100)))
  ctrl <- controller(tr, lstn)
  ctrl$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data("final")
  expect_true(all(d$pfs >= 0))
  expect_true(all(d$pfs[d$pfs_event == 1] >= 0))

})

## ── censor_trial_data: calendar-time censoring ────────────────────────────────
test_that("censor_trial_data: no event time exceeds calendar lock time", {

  skip_if_not_installed("TrialSimulator")

  set.seed(7)
  pfs_ep <- endpoint(name = "pfs", type = "tte",
                     generator = rexp, rate = log(2) / 8)
  a <- arm(name = "trt")
  a$add_endpoints(pfs_ep)

  lock_time <- 30
  tr <- trial(name = "t", n_patients = 300, seed = 2,
              duration = lock_time,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 20),
              dropout = rweibull, shape = 1, scale = 1e6,  # essentially no dropout
              silent = TRUE)
  tr$add_arms(sample_ratio = 1, a)

  lstn <- listener(silent = TRUE)
  lstn$add_milestones(milestone(name = "final",
                                when = calendarTime(time = lock_time)))
  ctrl <- controller(tr, lstn)
  ctrl$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data("final")
  calendar_event_time <- d$enroll_time + d$pfs
  expect_true(all(calendar_event_time <= lock_time + 1e-9))

})

## ── censor_trial_data: selected_arms partial censoring ────────────────────────
test_that("censor_trial_data: selected_arms leaves other arms untouched", {

  skip_if_not_installed("TrialSimulator")

  set.seed(99)
  pfs_ep_pbo <- endpoint(name = "pfs", type = "tte",
                         generator = rexp, rate = log(2) / 5)
  pfs_ep_trt <- endpoint(name = "pfs", type = "tte",
                         generator = rexp, rate = log(2) / 10)
  pbo <- arm(name = "placebo")
  trt <- arm(name = "trt")
  pbo$add_endpoints(pfs_ep_pbo)
  trt$add_endpoints(pfs_ep_trt)

  lock_time <- 20
  tr <- trial(name = "t", n_patients = 200, seed = 3,
              duration = lock_time,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 15),
              dropout = rweibull, shape = 1, scale = 1e6,
              silent = TRUE)
  tr$add_arms(sample_ratio = c(1, 1), pbo, trt)

  lstn <- listener(silent = TRUE)
  lstn$add_milestones(milestone(name = "final",
                                when = calendarTime(time = lock_time)))
  ctrl <- controller(tr, lstn)
  ctrl$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data("final")
  expect_true(all(d$enroll_time + d$pfs <= lock_time + 1e-9))

})

## ── stratified randomization: per-stratum balance ─────────────────────────────
test_that("stratified randomization produces balanced strata", {

  skip_on_cran()
  skip_if_not_installed("TrialSimulator")

  set.seed(5)
  rng <- function(n) data.frame(sex = sample(c("M", "F"), n, replace = TRUE))
  strat_ep <- endpoint(name = "sex", type = "non-tte",
                       readout = c(sex = 0), generator = rng)

  pfs_ep <- endpoint(name = "pfs", type = "tte",
                     generator = rexp, rate = log(2) / 8)

  pbo <- arm(name = "placebo")
  trt <- arm(name = "trt")
  pbo$add_endpoints(strat_ep, pfs_ep)
  trt$add_endpoints(strat_ep, pfs_ep)

  tr <- trial(name = "t", n_patients = 400, seed = 10,
              duration = 100,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 30),
              dropout = rweibull, shape = 1, scale = 1e6,
              stratification_factors = "sex",
              silent = TRUE)
  tr$add_arms(sample_ratio = c(1, 1), pbo, trt)

  lstn <- listener(silent = TRUE)
  lstn$add_milestones(milestone(name = "final",
                                when = enrollment(n = 400)))
  ctrl <- controller(tr, lstn)
  ctrl$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data("final")
  for(s in unique(d$sex)){
    sub <- d[d$sex == s, ]
    counts <- table(sub$arm)
    ratio <- max(counts) / min(counts)
    expect_lt(ratio, 1.3,
              label = paste("stratum", s, "imbalance ratio", round(ratio, 2)))
  }

})

## ── enrollment(arms=...): counts only specified arms ──────────────────────────
test_that("enrollment(arms=...) milestone triggers on specified arms only", {

  skip_if_not_installed("TrialSimulator")

  set.seed(20)
  pfs_ep <- endpoint(name = "pfs", type = "tte",
                     generator = rexp, rate = log(2) / 8)

  pbo <- arm(name = "placebo"); pbo$add_endpoints(pfs_ep)
  low <- arm(name = "low");     low$add_endpoints(pfs_ep)
  hi  <- arm(name = "high");    hi$add_endpoints(pfs_ep)

  tr <- trial(name = "t", n_patients = 300, seed = 21,
              duration = 100,
              enroller = StaggeredRecruiter,
              accrual_rate = data.frame(end_time = Inf, piecewise_rate = 20),
              dropout = rweibull, shape = 1, scale = 1e6,
              silent = TRUE)
  tr$add_arms(sample_ratio = c(1, 1, 1), pbo, low, hi)

  n_target <- 100L
  lstn <- listener(silent = TRUE)
  lstn$add_milestones(
    milestone(name = "pbo_only",
              when = enrollment(n = n_target, arms = "placebo"))
  )
  ctrl <- controller(tr, lstn)
  ctrl$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data("pbo_only")
  n_pbo <- sum(d$arm == "placebo")
  expect_gte(n_pbo, n_target)

})

# Parallelization
#
# Covers:
#   - Reproducibility: running n_workers > 1 with seeds extracted from a
#     multi-replicate run reproduces identical results under n_workers = 1.
#     This verifies that the Mersenne-Twister stream reset (instead of
#     L'Ecuyer-CMRG) keeps single- and multi-process outputs in sync.

test_that('package behaves the same under single- and multi-process modes with same seed', {


  foo <- function(n, n_workers, seed = NULL){
    pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/10)
    os <- endpoint(name = 'os', type = 'tte', generator = rexp, rate = log(2)/17)
    or <- endpoint(name = 'or', type = 'non-tte', readout = c(or = 1), generator = rnorm)

    pbo <- arm(name = 'pbo')
    pbo$add_endpoints(pfs, os, or)

    pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/12)
    os <- endpoint(name = 'os', type = 'tte', generator = rexp, rate = log(2)/18)
    or <- endpoint(name = 'or', type = 'non-tte', readout = c(or = 1), generator = rnorm)

    trt1 <- arm(name = 'trt1')
    trt1$add_endpoints(pfs, os, or)


    pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/13)
    os <- endpoint(name = 'os', type = 'tte', generator = rexp, rate = log(2)/18.5)
    or <- endpoint(name = 'or', type = 'non-tte', readout = c(or = 1), generator = rnorm)

    trt2 <- arm(name = 'trt2')
    trt2$add_endpoints(pfs, os, or)


    accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                               piecewise_rate = c(2, 8, 20, 25, 50))

    trial <- trial(
      name = 'test', n_patients = 1000, duration = 40,
      enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
      dropout = rweibull, shape = 1.32, scale = 114.4,
      seed = seed,
      silent = TRUE
    )

    trial$add_arms(sample_ratio = c(1, 1, 2), pbo, trt1, trt2)

    interim1 <- milestone(name = 'interim1',
                          when = eventNumber(endpoint = 'or', n = 200),
                          action = function(trial){trial$remove_arms('trt1')})

    action2 <- function(trial){

      locked_data <- trial$get_locked_data('interim2')
      fit1 <- fitLogrank(Surv(pfs, pfs_event) ~ arm,
                         placebo = 'pbo',
                         data = locked_data,
                         alternative = 'less')
      fit2 <- fitCoxph(Surv(os, os_event) ~ arm,
                       placebo = 'pbo',
                       data = locked_data,
                       alternative = 'less',
                       scale = 'hazard ratio',
                       arm != 'trt1')
      fit3 <- fitLinear(or ~ arm,
                        placebo = 'pbo',
                        data = locked_data,
                        alternative = 'less', arm != 'trt2')
      trial$save(fit1$p[1], 'IA_pval11')
      trial$save(fit1$p[2], 'IA_pval12')
      trial$save(fit2$p[1], 'IA_pval2')
      trial$save(fit3$p[1], 'IA_pval3')
    }

    interim2 <- milestone(name = 'interim2',
                          when = eventNumber(endpoint = 'pfs', n = 240) &
                            eventNumber(endpoint = 'os', n = 170),
                          action = action2)

    action3 <- function(trial){

      locked_data <- trial$get_locked_data('final')
      fit1 <- fitLogrank(Surv(pfs, pfs_event) ~ arm,
                         placebo = 'pbo',
                         data = locked_data,
                         alternative = 'less')
      fit2 <- fitCoxph(Surv(os, os_event) ~ arm,
                       placebo = 'pbo',
                       data = locked_data,
                       alternative = 'less',
                       scale = 'hazard ratio',
                       arm != 'trt1')
      fit3 <- fitLinear(or ~ arm,
                        placebo = 'pbo',
                        data = locked_data,
                        alternative = 'less', arm != 'trt2')
      trial$save(fit1$p[1], 'FA_pval11')
      trial$save(fit1$p[2], 'FA_pval12')
      trial$save(fit2$p[1], 'FA_pval2')
      trial$save(fit3$p[1], 'FA_pval3')
    }

    final <- milestone(name = 'final',
                       when = calendarTime(time = 40),
                       action = action3)

    listener <- listener(silent = TRUE)
    listener$add_milestones(interim1, interim2, final)

    controller <- controller(trial, listener)
    controller$run(n = n, n_workers = n_workers, plot_event = FALSE, silent = TRUE)
    controller$get_output(c('seed', 'IA_pval11', 'IA_pval12', 'IA_pval2', 'IA_pval3',
                            'FA_pval11', 'FA_pval12', 'FA_pval2', 'FA_pval3'))
  }

  op2 <- foo(n = 10, n_workers = 2, seed = NULL)

  op1 <- NULL
  for(i in 1:10){
    op1 <- rbind(op1, foo(n = 1, n_workers = 1, seed = op2$seed[i]))
  }

  expect_equal(op1, op2)

})

# User-facing wrappers for adaptive trial mutations
#
# Covers: add_arms, remove_arms, resize, set_duration, update_generator,
# update_sample_ratio — each invoked inside an action function to verify
# they forward to the corresponding Trials$... method.

make_arm <- function(name, rate) {
  ep <- endpoint(name = 'pfs', type = 'tte', generator = rexp,
                 rate = log(2) / rate)
  a <- arm(name = name)
  a$add_endpoints(ep)
  a
}

make_trial <- function(seed = 1, n_patients = 400, duration = 30) {
  accrual <- data.frame(end_time = Inf, piecewise_rate = 30)
  trial(name = "t", n_patients = n_patients, duration = duration, seed = seed,
        enroller = StaggeredRecruiter, accrual_rate = accrual,
        dropout = rweibull, shape = 1, scale = 1e6,
        silent = TRUE)
}


test_that("add_arms wrapper adds an arm at trial creation", {

  pbo <- make_arm("pbo", 10)
  trt <- make_arm("trt", 12)
  tr <- make_trial()
  add_arms(tr, sample_ratio = c(1, 1), pbo, trt)

  lstn <- listener(silent = TRUE)
  lstn$add_milestones(milestone(name = "f", when = calendarTime(time = 30)))
  controller(tr, lstn)$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data("f")
  expect_equal(sort(unique(d$arm)), sort(c("pbo", "trt")))
})


test_that("remove_arms wrapper drops an arm after a milestone", {

  pbo <- make_arm("pbo", 10)
  trt1 <- make_arm("trt1", 12)
  trt2 <- make_arm("trt2", 14)
  tr <- make_trial()
  add_arms(tr, sample_ratio = c(1, 1, 1), pbo, trt1, trt2)

  drop <- milestone(name = "drop",
                    when = calendarTime(time = 10),
                    action = function(trial) { remove_arms(trial, "trt1") })
  final <- milestone(name = "final", when = calendarTime(time = 30))

  lstn <- listener(silent = TRUE)
  lstn$add_milestones(drop, final)
  controller(tr, lstn)$run(n = 1, silent = TRUE, plot_event = FALSE)

  d_before <- tr$get_locked_data("drop")
  d_after <- tr$get_locked_data("final")

  expect_true("trt1" %in% unique(d_before$arm))
  # trt1 patients present but censored at time of removal
  after_trt1 <- d_after[d_after$arm == "trt1", ]
  expect_true(nrow(after_trt1) > 0)
  expect_true(all(after_trt1$enroll_time + after_trt1$pfs <= 10 + 1e-6 |
                    after_trt1$pfs_event == 0))
})


test_that("resize wrapper increases maximum sample size mid-trial", {

  pbo <- make_arm("pbo", 10)
  trt <- make_arm("trt", 12)
  tr <- make_trial(n_patients = 200)
  add_arms(tr, sample_ratio = c(1, 1), pbo, trt)

  check <- milestone(name = "check",
                     when = calendarTime(time = 5),
                     action = function(trial) { resize(trial, 400) })
  final <- milestone(name = "final", when = calendarTime(time = 30))
  lstn <- listener(silent = TRUE)
  lstn$add_milestones(check, final)
  controller(tr, lstn)$run(n = 1, silent = TRUE, plot_event = FALSE)

  d <- tr$get_locked_data("final")
  # With a generous duration and piecewise_rate=30/month, we should comfortably
  # exceed the original 200-patient cap once resized.
  expect_gt(nrow(d), 200)
})


test_that("set_duration wrapper extends trial duration", {

  pbo <- make_arm("pbo", 10)
  trt <- make_arm("trt", 12)
  tr <- make_trial(duration = 15)
  add_arms(tr, sample_ratio = c(1, 1), pbo, trt)

  extend <- milestone(name = "extend",
                      when = calendarTime(time = 10),
                      action = function(trial) { set_duration(trial, 30) })
  final <- milestone(name = "final", when = calendarTime(time = 30))
  lstn <- listener(silent = TRUE)
  lstn$add_milestones(extend, final)
  controller(tr, lstn)$run(n = 1, silent = TRUE, plot_event = FALSE)

  # If set_duration had failed, the trial would have locked at time 15 and no
  # final milestone at time 30 would be reachable.
  t_final <- tr$get_milestone_time("final")
  expect_gte(t_final, 15)
})


test_that("update_generator wrapper swaps generator for an endpoint", {

  pbo <- make_arm("pbo", 10)
  trt <- make_arm("trt", 12)
  tr <- make_trial()
  add_arms(tr, sample_ratio = c(1, 1), pbo, trt)

  swap <- milestone(name = "swap",
                    when = calendarTime(time = 5),
                    action = function(trial) {
                      update_generator(trial,
                                       arm_name = "trt",
                                       endpoint_name = "pfs",
                                       generator = rexp,
                                       rate = log(2) / 50)
                    })
  final <- milestone(name = "final", when = calendarTime(time = 30))
  lstn <- listener(silent = TRUE)
  lstn$add_milestones(swap, final)
  expect_no_error(
    controller(tr, lstn)$run(n = 1, silent = TRUE, plot_event = FALSE)
  )
})


test_that("update_sample_ratio wrapper changes randomization ratio", {

  pbo <- make_arm("pbo", 10)
  trt <- make_arm("trt", 12)
  tr <- make_trial()
  add_arms(tr, sample_ratio = c(1, 1), pbo, trt)

  reallocate <- milestone(name = "reallocate",
                          when = calendarTime(time = 5),
                          action = function(trial) {
                            update_sample_ratio(trial,
                                                arm_names = c("pbo", "trt"),
                                                sample_ratios = c(1, 3))
                          })
  final <- milestone(name = "final", when = calendarTime(time = 30))
  lstn <- listener(silent = TRUE)
  lstn$add_milestones(reallocate, final)
  expect_no_error(
    controller(tr, lstn)$run(n = 1, silent = TRUE, plot_event = FALSE)
  )

  d <- tr$get_locked_data("final")
  # After reallocation trt should attract more enrollments than pbo in
  # patients enrolled after time 5.
  late <- d[d$enroll_time > 5, ]
  if(nrow(late) > 20){
    expect_gt(sum(late$arm == "trt"), sum(late$arm == "pbo"))
  }
})

# Core trial mechanics
#
# Covers:
#   - Milestone triggering conditions (eventNumber, calendarTime, enrollment, combined)
#   - Event and endpoint counts at each locked dataset
#   - Behavior when trial duration is extended mid-trial
#   - Behavior when an arm is removed adaptively
#   - Inclusion criteria on arms
#   - Custom data persistence across multiple controller$run() calls
#   - Reproducibility: re-running with a fixed seed yields identical output

test_that('trial milestone timing and endpoint event count work as expected', {

  pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/10)
  os <- endpoint(name = 'os', type = 'tte', generator = rexp, rate = log(2)/17)
  or <- endpoint(name = 'or', type = 'non-tte', readout = c(or = 1), generator = rnorm)

  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(pfs, os, or)

  pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/12)
  os <- endpoint(name = 'os', type = 'tte', generator = rexp, rate = log(2)/18)
  or <- endpoint(name = 'or', type = 'non-tte', readout = c(or = 1), generator = rnorm)

  trt <- arm(name = 'trt')
  trt$add_endpoints(pfs, os, or)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    dropout = rweibull, shape = 1.32, scale = 114.4,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 2), pbo, trt)

  interim1 <- milestone(name = 'interim1',
                        when = eventNumber(endpoint = 'or', n = 200))

  interim2 <- milestone(name = 'interim2',
                        when = eventNumber(endpoint = 'pfs', n = 240) &
                          eventNumber(endpoint = 'os', n = 170))

  final <- milestone(name = 'final',
                     when = calendarTime(time = 40))

  listener <- listener(silent = TRUE)
  listener$add_milestones(interim1, interim2, final)

  controller <- controller(trial, listener)
  controller$run(n = 1, plot_event = FALSE, silent = TRUE)

  ## check data

  dat1 <- trial$get_locked_data('interim1')
  dat2 <- trial$get_locked_data('interim2')
  dat3 <- trial$get_locked_data('final')

  time1 <- trial$get_milestone_time('interim1')
  time2 <- trial$get_milestone_time('interim2')


  expect_equal(sum(!is.na(dat1$or)), 200)

  expect_equal(
    dat2 %>%
      filter(enroll_time + or_readout <= time1 &
               or_readout < dropout_time &
               !is.na(or)) %>%
      nrow(),
    200)

  expect_equal(
    dat3 %>%
      filter(enroll_time + or_readout <= time1 &
               or_readout < dropout_time &
               !is.na(or)) %>%
      nrow(),
    200)

  expect_true(sum(dat2$pfs_event %in% 1) == 240 || sum(dat2$os_event %in% 1) == 170)

  expect_true(sum(dat2$pfs_event %in% 1) >= 240 && sum(dat2$os_event %in% 1) >= 170)

  n_pfs_events <- dat3 %>%
    filter(enroll_time + pfs <= time2 &
             pfs < dropout_time &
             pfs_event %in% 1) %>% nrow()

  n_os_events <- dat3 %>%
    filter(enroll_time + os <= time2 &
             os < dropout_time &
             os_event %in% 1) %>% nrow()

  expect_true(n_pfs_events == 240 || n_os_events == 170)
  expect_true(n_pfs_events >= 240 && n_os_events >= 170)

})

test_that('filters are supported when defining milestones', {

  #' random number generator (RNG) for OS, PFS, and PRO
  rng <- function(n, medians, prop_high){

    ## os
    median_os_low <- TrialSimulator::solveMixtureExponentialDistribution(
      weight1 = prop_high,
      median1 = medians['os_high'],
      overall_median = medians['os_all']
    )

    median_pfs_low <- TrialSimulator::solveMixtureExponentialDistribution(
      weight1 = prop_high,
      median1 = medians['pfs_high'],
      overall_median = medians['pfs_all']
    )

    median_pro_low <- TrialSimulator::solveMixtureExponentialDistribution(
      weight1 = prop_high,
      median1 = medians['pro_high'],
      overall_median = medians['pro_all']
    )

    dll3 <- sample(c('high', 'low'), size = n, replace = TRUE, prob = c(prop_high, 1 - prop_high))

    os_high <- rexp(n, rate = log(2) / medians['os_high'])
    os_low <- rexp(n, rate = log(2) / median_os_low)

    pfs_high <- rexp(n, rate = log(2) / medians['pfs_high'])
    pfs_low <- rexp(n, rate = log(2) / median_pfs_low)

    pro_high <- rexp(n, rate = log(2) / medians['pro_high'])
    pro_low <- rexp(n, rate = log(2) / median_pro_low)

    data.frame(dll3,
               x = rnorm(n),
               os = ifelse(dll3 %in% 'high', os_high, os_low),
               pfs = ifelse(dll3 %in% 'high', pfs_high, pfs_low),
               pro = ifelse(dll3 %in% 'high', pro_high, pro_low),
               os_event = 1,
               pfs_event = 1,
               pro_event = 1)
  }


  simulate <- function(fwer,
                       sample_size,
                       final_event_number,
                       interim_info_fraction,
                       prop_dll3_high,
                       median_soc,
                       median_trt){


    ep_soc <- endpoint(name = c('pfs', 'os', 'pro', 'dll3', 'x'),
                       type = c('tte', 'tte', 'tte', 'non-tte', 'non-tte'),
                       readout = c(dll3 = 0, x = 0),
                       generator = rng,
                       medians = median_soc,
                       prop_high = prop_dll3_high)

    soc <- arm(name = 'soc')
    soc$add_endpoints(ep_soc)

    ep_trt <- endpoint(name = c('pfs', 'os', 'pro', 'dll3', 'x'),
                       type = c('tte', 'tte', 'tte', 'non-tte', 'non-tte'),
                       readout = c(dll3 = 0, x = 0),
                       generator = rng,
                       medians = median_trt,
                       prop_high = prop_dll3_high)

    trt <- arm(name = 'trt')
    trt$add_endpoints(ep_trt)


    accrual_rate <- data.frame(end_time = c(1:17, Inf),
                               piecewise_rate = c(1,2,5,8,12,17,22,28,32,37,40,43,46,47,50,50,51,52))

    trial <- trial(
      name = '1438', n_patients = sample_size, duration = 200,
      enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
      dropout = rexp, rate = -log(1 - .15)/12,
      silent = TRUE
    )

    trial$add_arms(sample_ratio = c(1, 1), soc, trt)

    interim <- milestone(name = 'interim', when = enrollment(n = 100, dll3 == 'high', x > -2),
                         action = interim_action)

    final <- milestone(name = 'final',
                       when =
                         eventNumber('pfs', n = 100, dll3 == 'high', x > -2) &
                         eventNumber('pfs', n = 150) &
                         eventNumber('os', n = 25, dll3 != 'high'),
                       action = final_action)

    listener <- listener(silent = TRUE)
    listener$add_milestones(interim, final)

    controller <- controller(trial, listener)
    controller$run(n = 10, plot_event = FALSE, silent = TRUE)

    controller$get_output()

  }

  interim_action <- function(trial){

    locked_data <- trial$get_locked_data('interim')
    trial$save(value = locked_data %>% filter(dll3 == 'high' & x > -2) %>% nrow(),
               name = 'n_at_lock')
  }

  final_action <- function(trial){

    locked_data <- trial$get_locked_data('final')
    tmp1 <- locked_data %>% filter(dll3 == 'high' & x > -2)
    trial$save(value = sum(tmp1$pfs_event), name = 'n_pfs_high')
    trial$save(value = sum(locked_data$pfs_event), name = 'n_pfs_all')

    tmp2 <- locked_data %>% filter(dll3 != 'high')
    trial$save(value = sum(tmp2$os_event), name = 'n_os_low')
  }


  sims <-
    simulate(fwer = c(0.025), ## family-wise error rate for one-sided test
             sample_size = c(702), ## total planned sample size
             final_event_number = list(os_high = 343, os_all = 512),
             interim_info_fraction = c(.68),
             prop_dll3_high = c(.67),
             median_soc = list(c(os_high = 13,
                                 os_all = 13,
                                 pfs_high = 5,
                                 pfs_all = 5,
                                 pro_high = 7.5,
                                 pro_all = 7.5))[[1]],
             median_trt = list(c(os_high = 19,
                                 os_all = 17.6,
                                 pfs_high = 7,
                                 pfs_all = 6.5,
                                 pro_high = 10,
                                 pro_all = 9.7))[[1]])

  expect_true(all(sims$n_at_lock == 100))

  expect_true(all(sims$n_pfs_high >= 100))

  expect_true(all(sims$n_pfs_all >= 150))

  expect_true(all(sims$n_os_low >= 25))

  expect_true(all(sims$`milestone_time_<final>` > sims$`milestone_time_<interim>`))


})

test_that('endpoint event counts work as expected when duration is adapted', {

  pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/10)
  os <- endpoint(name = 'os', type = 'tte', generator = rexp, rate = log(2)/17)
  or <- endpoint(name = 'or', type = 'non-tte', readout = c(or = 1), generator = rnorm)

  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(pfs, os, or)

  pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/12)
  os <- endpoint(name = 'os', type = 'tte', generator = rexp, rate = log(2)/18)
  or <- endpoint(name = 'or', type = 'non-tte', readout = c(or = 1), generator = rnorm)

  trt <- arm(name = 'trt')
  trt$add_endpoints(pfs, os, or)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 30,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    dropout = rweibull, shape = 1.32, scale = 114.4,
    seed = 808715505,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 2), pbo, trt)

  action_at_interim <- function(trial){
    trial$set_duration(duration = 40)
  }

  interim <- milestone(name = 'interim',
                       when = eventNumber(endpoint = 'os', n = 150),
                       action = action_at_interim)

  listener <- listener(silent = TRUE)
  listener$add_milestones(interim)

  controller <- controller(trial, listener)
  controller$run(plot_event = FALSE, silent = TRUE)

  dat1_ <- trial$get_locked_data('interim')

  expect_equal(sum(dat1_$os_event %in% 1), 150)

  final <- milestone(name = 'final',
                     when = eventNumber(endpoint = 'os', n = 400) |
                       calendarTime(time = 40))


  listener$add_milestones(final)
  controller$run(plot_event = FALSE, silent = TRUE)

  dat1 <- trial$get_locked_data('interim')
  dat2 <- trial$get_locked_data('final')

  expect_identical(dat1_, dat1)

  expect_true(sum(dat2$os_event %in% 1) == 400 ||
                all((dat2$enroll_time + dat2$os_event)[dat2$os_event %in% 1] <= 40))

  tol <- 1e-6
  expect_true(
    dat2 %>%
      filter(patient_id %in% dat1$patient_id) %>%
      summarise(os = all((enroll_time + os < 30 &
                            os < dropout_time - tol) == (os_event == 1)),
                pfs = all((enroll_time + pfs < 30 &
                             pfs < dropout_time - tol) == (pfs_event == 1)),
                or = all((enroll_time + or_readout < 30 &
                            or_readout < dropout_time - tol) == !is.na(or))) %>%
      apply(1, function(x){all(x)})
  )

  expect_true(
    dat2 %>%
      filter(os_event %in% 1) %>%
      summarise(os = all(enroll_time + os <= 40)) %>%
      apply(1, function(x){all(x)}))

  expect_true(
    dat2 %>%
      filter(pfs_event %in% 1) %>%
      summarise(pfs = all(enroll_time + pfs <= 40)) %>%
      apply(1, function(x){all(x)}))

  expect_true(
    dat2 %>%
      filter(!is.na(or)) %>%
      summarise(or = all(enroll_time + or_readout <= 40)) %>%
      apply(1, function(x){all(x)}))

})

test_that('endpoint event counts work as expected when an arm is removed', {

  pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/10)
  os <- endpoint(name = 'os', type = 'tte', generator = rexp, rate = log(2)/17)
  or <- endpoint(name = 'or', type = 'non-tte', readout = c(or = 1), generator = rnorm)

  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(pfs, os, or)

  pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/12)
  os <- endpoint(name = 'os', type = 'tte', generator = rexp, rate = log(2)/18)
  or <- endpoint(name = 'or', type = 'non-tte', readout = c(or = 1), generator = rnorm)

  trt1 <- arm(name = 'trt1')
  trt1$add_endpoints(pfs, os, or)


  pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/13)
  os <- endpoint(name = 'os', type = 'tte', generator = rexp, rate = log(2)/18.5)
  or <- endpoint(name = 'or', type = 'non-tte', readout = c(or = 1), generator = rnorm)

  trt2 <- arm(name = 'trt2')
  trt2$add_endpoints(pfs, os, or)


  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    dropout = rweibull, shape = 1.32, scale = 114.4,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 1, 2), pbo, trt1, trt2)

  interim1 <- milestone(name = 'interim1',
                        when = eventNumber(endpoint = 'or', n = 200),
                        action = function(trial){trial$remove_arms('trt1')})

  interim2 <- milestone(name = 'interim2',
                        when = eventNumber(endpoint = 'pfs', n = 240) &
                          eventNumber(endpoint = 'os', n = 170))

  final <- milestone(name = 'final',
                 when = calendarTime(time = 40))

  listener <- listener(silent = TRUE)
  listener$add_milestones(interim1, interim2, final)

  controller <- controller(trial, listener)
  controller$run(n = 1, plot_event = FALSE, silent = TRUE)

  dat1 <- trial$get_locked_data('interim1')
  dat2 <- trial$get_locked_data('interim2')
  dat3 <- trial$get_locked_data('final')

  remove_attr <- function(locked_data){
    attr(locked_data, 'lock_time') <- NULL
    attr(locked_data, 'n_enrolled_patients') <- NULL
    attr(locked_data, 'milestone_name') <- NULL
    locked_data
  }

  dat1 <- remove_attr(dat1)
  dat2 <- remove_attr(dat2)
  dat3 <- remove_attr(dat3)

  expect_equal(sum(!is.na(dat1$or)), 200)

  expect_true(sum(dat2$pfs_event %in% 1 & dat2$arm %in% c('pbo', 'trt2')) >= 240 &&
                sum(dat2$os_event %in% 1 & dat2$arm %in% c('pbo', 'trt2')) >= 170 &&
                (sum(dat2$pfs_event %in% 1 & dat2$arm %in% c('pbo', 'trt2')) == 240 ||
                   sum(dat2$os_event %in% 1 & dat2$arm %in% c('pbo', 'trt2')) == 170))

  dat1_ <- dat2 %>%
    filter(patient_id %in% dat1$patient_id) %>%
    arrange(patient_id)

  expect_equal(dat1 %>% filter(arm == 'trt1'), dat1_ %>% filter(arm == 'trt1'))

  dat1__ <- dat3 %>%
    filter(patient_id %in% dat1$patient_id) %>%
    arrange(patient_id)

  expect_equal(dat1 %>% filter(arm == 'trt1'), dat1__ %>% filter(arm == 'trt1'))

})

test_that('inclusion criteria of arm work as expected', {

  rng <- function(n){

    data.frame(
      x = rnorm(n),
      y = rbinom(n, 1, .5),
      z = sample(LETTERS[1:3], n, replace = TRUE),
      w = rexp(n, rate = .01),
      w_event = rbinom(n, 1, .3)
    )

  }

  ep <- endpoint(name = c('x', 'y', 'z', 'w'),
                 type = c('non-tte', 'non-tte', 'non-tte', 'tte'),
                 readout = c(x = 0, y = 0, z = 0),
                 generator = rng)

  test <- arm(name = 'test', x > 0)
  test$add_endpoints(ep)

  dat <- test$generate_data(1e4)
  expect_true(all(dat$x > 0))

  test <- arm(name = 'test', x > 1, z %in% c('A', 'C'))
  test$add_endpoints(ep)
  dat <- test$generate_data(1e4)
  expect_true(all(dat$x > 1) && all(dat$z %in% c('A', 'C')))

  test <- arm(name = 'test', x > -0.5 | z %in% c('B', 'C'))
  test$add_endpoints(ep)
  dat <- test$generate_data(1e4)
  expect_true(all(dat$x > -0.5 | dat$z %in% c('B', 'C')))

})

test_that('custom data can be re-used in multiple trials', {

  ep <- endpoint(name = 'ep', type = 'tte', generator = rexp, rate = .1)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = 1, pbo)
  trial$save_custom_data(value = list(x = 1, y = 'a'), name = 'config')

  final <- milestone(name = 'final',
                     action = function(trial) {
                       trial$get_custom_data('config')},
                     when = calendarTime(time = 40))

  listener <- listener(silent = TRUE)
  listener$add_milestones(final)

  controller <- controller(trial, listener)
  expect_no_error(controller$run(n = 10, plot_event = FALSE, silent = TRUE))

})

test_that('trial data can be replicated', {

  ep <- endpoint(name = 'ep', type = 'tte', generator = rexp, rate = .1)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50))

  trial <- trial(
    name = 'test', n_patients = 1000, duration = 40,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = 1, pbo)

  act <- function(trial){

    locked_data <- trial$get_locked_data('final')
    trial$save(value = median(locked_data$ep), name = 'median')
    trial$save(value = mean(locked_data$ep), name = 'mean')
    trial$save(value = sd(locked_data$ep), name = 'sd')

  }

  final <- milestone(name = 'final',
                     action = act,
                     when = calendarTime(time = 40))

  listener <- listener(silent = TRUE)
  listener$add_milestones(final)

  controller <- controller(trial, listener)
  controller$run(n = 10, plot_event = FALSE, silent = TRUE)
  op10 <- controller$get_output()
  op10 <- op10[order(op10$seed), ]
  rownames(op10) <- NULL

  ## feed seeds in op10 one by one for testing purpose

  seeds <- sort(op10$seed)
  ops <- NULL
  for(seed in seeds){
    trial <- trial(
      name = 'test', n_patients = 1000, duration = 40,
      seed = seed,
      enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
      silent = TRUE
    )

    trial$add_arms(sample_ratio = 1, pbo)

    final <- milestone(name = 'final',
                       action = act,
                       when = calendarTime(time = 40))

    listener <- listener(silent = TRUE)
    listener$add_milestones(final)

    controller <- controller(trial, listener)
    controller$run(plot_event = FALSE, silent = TRUE)
    ops <- rbind(ops, controller$get_output())

  }

  expect_identical(op10, ops)

})

# Adaptive trial features
#
# Covers:
#   - update_sample_ratio: switching between permuted-block and sample()-based
#     randomization; balance checks at each stage of a response-adaptive design
#   - enrollment(min_treatment_duration): milestone triggered after all patients
#     have received treatment for a minimum duration
#   - add_arms(enforce): backward-compatible behavior of the enforce argument
#     when adding arms mid-trial
#   - update_generator: changing a data generator during a running trial and
#     verifying that only future patients are affected
#   - resize: increasing planned sample size at interim milestones

test_that('sample ratio can be updated to switch between permuted block and sample()', {

  skip_on_cran()

  ep <- endpoint(name = 'os', type = 'non-tte', readout = c(os = 0), generator = rexp, rate = log(2)/10)
  pbo <- arm(name = 'pbo')

  pbo$add_endpoints(ep)

  ep <- endpoint(name = 'os', type = 'non-tte', readout = c(os = 0), generator = rexp, rate = log(2)/11)
  low <- arm(name = 'low')

  low$add_endpoints(ep)

  ep <- endpoint(name = 'os', type = 'non-tte', readout = c(os = 0), generator = rexp, rate = log(2)/12)
  med <- arm(name = 'med')

  med$add_endpoints(ep)

  ep <- endpoint(name = 'os', type = 'non-tte', readout = c(os = 0), generator = rexp, rate = log(2)/13)
  high <- arm(name = 'high')

  high$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(10, Inf),
                             piecewise_rate = c(200, 200))

  trial <- trial(
    name = 'test', n_patients = 1e4, duration = 52,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 1, 1, 1), pbo, low, med, high)


  action1 <- function(trial){

    dat1 <- trial$get_locked_data('interim1')

    prob <- prop.table(trial$get_sample_ratio())

    trial$save(value = chisq.test(table(dat1$arm)[names(prob)], p = prob)$p.value,
               name = 'stage1')

    trial$update_sample_ratio(c('med', 'high'), c(2, 2))

  }

  interim1 <- milestone(name = 'interim1',
                        when = eventNumber('os', n = 2000),
                        action = action1
  )

  action2 <- function(trial){

    dat1 <- trial$get_locked_data('interim1')
    dat2 <- trial$get_locked_data('interim2') %>%
      filter(!(patient_id %in% dat1$patient_id))

    prob <- prop.table(trial$get_sample_ratio())

    trial$save(value = chisq.test(table(dat2$arm)[names(prob)], p = prob)$p.value,
               name = 'stage2')

    trial$update_sample_ratio(c('low', 'high'), c(.5, 2.5))

  }

  interim2 <- milestone(name = 'interim2',
                        when = eventNumber('os', 4000),
                        action = action2
  )

  action3 <- function(trial){

    dat2 <- trial$get_locked_data('interim2')
    dat3 <- trial$get_locked_data('interim3') %>%
      filter(!(patient_id %in% dat2$patient_id))

    prob <- prop.table(trial$get_sample_ratio())

    trial$save(value = chisq.test(table(dat3$arm)[names(prob)], p = prob)$p.value,
               name = 'stage3')

    trial$update_sample_ratio(c('low', 'high'), c(1.5, 3))
  }

  interim3 <- milestone(name = 'interim3',
                        when = eventNumber('os', 6000),
                        action = action3
  )


  action4 <- function(trial){

    dat3 <- trial$get_locked_data('interim3')
    dat4 <- trial$get_locked_data('final') %>%
      filter(!(patient_id %in% dat3$patient_id))

    prob <- prop.table(trial$get_sample_ratio())

    trial$save(value = chisq.test(table(dat4$arm)[names(prob)], p = prob)$p.value,
               name = 'stage4')

  }

  final <- milestone(name = 'final', when = calendarTime(time = 52),
                     action = action4)

  listener <- listener(silent = TRUE)
  listener$add_milestones(interim1, interim2, interim3, final)

  controller <- controller(trial, listener)
  set.seed(42)
  controller$run(n = 10, plot_event = FALSE, silen = TRUE)

  # p-values from chi-sq allocation checks; threshold 1e-4 to avoid flakiness
  expect_true(all(controller$get_output() %>% select(contains('stage')) > .1/10))

})

test_that('milestone can be triggerd when all patients have received treatment for a while', {
  ep <- endpoint(name = 'x', type = 'tte', generator = rexp, rate = log(2)/10)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)
  ep <- endpoint(name = 'x', type = 'tte', generator = rexp, rate = log(2)/12)
  trt <- arm(name = 'trt')
  trt$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(6, 12, 16, Inf),
                             piecewise_rate = c(20, 30, 40, 60))
  ## enrollment of 580 patients will be done by the 18th month
  ## if min treatment duration is 2 months, then triggering time would be 20

  trial <- trial(
    name = 'test', n_patients = 580, duration = 25,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    dropout = rexp, rate = -log(1 - .08)/2,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 2), pbo, trt)

  final <- milestone(name = 'final',
                     when = enrollment(n = 580, min_treatment_duration = 2))

  listener <- listener(silent = TRUE)
  listener$add_milestones(final)

  controller <- controller(trial, listener)
  controller$run(n = 10, plot_event = FALSE, silen = TRUE)

  expect_true(all(controller$get_output('milestone_time_<final>') - 19.98333 < .001))

})



test_that('enforce = TRUE is no longer needed in Trials$add_arms()', {

  ep <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/10)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  ep <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/10)
  trt3 <- arm(name = 'trt3')
  trt3$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(7, Inf),
                             piecewise_rate = c(50, 80))

  trial <- trial(name = '123', n_patients = 1000, duration = 20,
                 enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
                 seed = 31415926, silent = TRUE)

  trial$add_arms(sample_ratio = c(1, 1), pbo, trt3)

  action_at_interim1 <- function(trial){
    locked_data <- trial$get_locked_data('interim')

    ep <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/12)
    trt1 <- arm(name = 'trt1')
    trt1$add_endpoints(ep)

    ep <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/14)
    trt2 <- arm(name = 'trt2')
    trt2$add_endpoints(ep)

    trial$add_arms(sample_ratio = 2, enforce = TRUE, trt1)
    trial$add_arms(sample_ratio = 3, enforce = TRUE, trt2)
  }

  interim1 <- milestone(name = 'interim',
                        when = eventNumber(endpoint = 'pfs', n = 100),
                        action = action_at_interim1)

  final <- milestone(name = 'final',
                     when = eventNumber(endpoint = 'pfs', n = 400))

  listener <- listener(silent = TRUE)
  listener$add_milestones(interim1, final)

  controller <- controller(trial, listener)
  controller$run(n = 10, silent = TRUE)
  op1 <- controller$get_output()

  ####################
  ep <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/10)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  ep <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/10)
  trt3 <- arm(name = 'trt3')
  trt3$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(7, Inf),
                             piecewise_rate = c(50, 80))

  trial <- trial(name = '123', n_patients = 1000, duration = 20,
                 enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
                 seed = 31415926, silent = TRUE)

  trial$add_arms(sample_ratio = c(1, 1), pbo, trt3)

  action_at_interim2 <- function(trial){
    locked_data <- trial$get_locked_data('interim')

    ep <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/12)
    trt1 <- arm(name = 'trt1')
    trt1$add_endpoints(ep)

    ep <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2)/14)
    trt2 <- arm(name = 'trt2')
    trt2$add_endpoints(ep)

    trial$add_arms(sample_ratio = 2, trt1)
    trial$add_arms(sample_ratio = 3, trt2)
  }

  interim2 <- milestone(name = 'interim',
                        when = eventNumber(endpoint = 'pfs', n = 100),
                        action = action_at_interim2)

  final <- milestone(name = 'final',
                     when = eventNumber(endpoint = 'pfs', n = 400))

  listener <- listener(silent = TRUE)
  listener$add_milestones(interim2, final)

  controller <- controller(trial, listener)
  controller$run(n = 10, silent = TRUE)
  op2 <- controller$get_output()

  expect_true(identical(op1, op2))

})


test_that('generator of endpoint can be updated in Trials$update_generator()', {

  skip_on_cran()

  gen <- function(n, rate1, rate2){
    data.frame(ep11 = rexp(n, rate = rate1), ep11_event = 1,
               ep12 = rexp(n, rate = rate2), ep12_event = 1)
  }
  ep1 <- endpoint(name = c('ep11', 'ep12'), type = c('tte', 'tte'), generator = gen, rate1 = log(2)/5, rate2 = log(2)/8)

  ep2 <- endpoint(name = 'ep2', type = 'non-tte', readout = c(ep2 = 0), generator = rnorm)

  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep1, ep2)

  ep1 <- endpoint(name = c('ep11', 'ep12'), type = c('tte', 'tte'), generator = gen, rate1 = log(2)/6, rate2 = log(2)/10)

  ep2 <- endpoint(name = 'ep2', type = 'non-tte', readout = c(ep2 = 0), generator = rnorm)

  trt <- arm(name = 'trt')
  trt$add_endpoints(ep1, ep2)

  accrual_rate <- data.frame(end_time = c(1, 2, 6, 12, Inf),
                             piecewise_rate = c(2, 8, 20, 25, 50) * 1000)

  trial <- trial(
    name = 'test', n_patients = 1000000, duration = 100,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    seed = 31415926,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 1), pbo, trt)

  action_at_interim <- function(trial){

    locked_data <- trial$get_locked_data('interim')

    trial$update_generator(arm_name = 'pbo', endpoint_name = c('ep11', 'ep12'), generator = gen, rate1 = log(2)/2, rate2 = log(2)/3)

    fit11 <- survival::survfit(Surv(ep11, ep11_event) ~ arm, data = locked_data)
    medians11 <- summary(fit11)$table[, 'median']

    expect_equal(unname(medians11['arm=pbo']), 5, tolerance = .1)
    expect_equal(unname(medians11['arm=trt']), 6, tolerance = .1)

    fit12 <- survival::survfit(Surv(ep12, ep12_event) ~ arm, data = locked_data)
    medians12 <- summary(fit12)$table[, 'median']

    expect_equal(unname(medians12['arm=pbo']), 8, tolerance = .1)
    expect_equal(unname(medians12['arm=trt']), 10, tolerance = .1)

  }

  action_at_final <- function(trial){

    locked_data <- trial$get_locked_data('final') %>% filter(patient_id > 500000)
    fit11 <- survival::survfit(Surv(ep11, ep11_event) ~ arm, data = locked_data)
    medians11 <- summary(fit11)$table[, 'median']

    expect_equal(unname(medians11['arm=pbo']), 2, tolerance = .1)
    expect_equal(unname(medians11['arm=trt']), 6, tolerance = .1)

    fit12 <- survival::survfit(Surv(ep12, ep12_event) ~ arm, data = locked_data)
    medians12 <- summary(fit12)$table[, 'median']

    expect_equal(unname(medians12['arm=pbo']), 3, tolerance = .1)
    expect_equal(unname(medians12['arm=trt']), 10, tolerance = .1)

  }

  interim <- milestone(name = 'interim', when = enrollment(n = 500000), action = action_at_interim)
  final <- milestone(name = 'final', when = enrollment(n = 1000000, min_treatment_duration = 10), action = action_at_final)

  listener <- listener(silent = TRUE)
  listener$add_milestones(interim, final)

  controller <- controller(trial, listener)
  controller$run(n = 1, plot_event = FALSE, silent = TRUE)

})



test_that('trial can be resized by Trials$resize()', {

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 0), generator = rnorm)
  pbo <- arm(name = 'pbo')
  pbo$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 0), generator = rnorm, mean = .1)
  trt <- arm(name = 'trt')
  trt$add_endpoints(ep)

  accrual_rate <- data.frame(end_time = c(10, Inf),
                             piecewise_rate = c(50, 50))

  trial <- trial(
    name = 'test', n_patients = 500, duration = 50,
    enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
    silent = TRUE
  )

  trial$add_arms(sample_ratio = c(1, 2), pbo, trt)

  interim1_action <- function(trial){

    locked_data <- trial$get_locked_data('interim1')
    trial$save(nrow(locked_data), name = 'interim1_size')
    trial$resize(700)

  }

  interim1 <- milestone(name = 'interim1',
                        action = interim1_action,
                        when = enrollment(n = 400))

  interim2_action <- function(trial){

    locked_data <- trial$get_locked_data('interim2')
    trial$save(nrow(locked_data), name = 'interim2_size')
    trial$resize(900)

  }

  interim2 <- milestone(name = 'interim2',
                        action = interim2_action,
                        when = enrollment(n = 650))

  final_action <- function(trial){

    locked_data <- trial$get_locked_data('final')
    trial$save(nrow(locked_data), name = 'final_size')

  }

  final <- milestone(name = 'final',
                     action = final_action,
                     when = calendarTime(50))

  listener <- listener(silent = TRUE)
  listener$add_milestones(interim1, interim2, final)

  controller <- controller(trial, listener)
  controller$run(n = 10, plot_event = FALSE, silent = TRUE)

  expect_true(all(controller$get_output('interim1_size') == 400))
  expect_true(all(controller$get_output('interim2_size') == 650))
  expect_true(all(controller$get_output('final_size') == 900))
  expect_true(all(controller$get_output('milestone_time_<final>') == 50))

})

#' @title Farrington-Manning test for rate difference
#'
#' @description
#' Test rate difference by comparing it to a pre-specified value using the
#' Farrington-Manning test.
#'
#' Refer to \href{https://zhangh12.github.io/TrialSimulator/articles/wrappers.html}{this vignette}
#' for more information and examples.
#'
#' @param endpoint Character. Name of the endpoint in \code{data}.
#' @param placebo Character. String indicating the placebo in \code{data$arm}.
#' @param data Data frame. Usually it is a locked data set.
#' @param alternative a character string specifying the alternative hypothesis,
#' must be one of \code{"greater"} or \code{"less"}, i.e.,
#' one-sided test is enforced. No default value.
#' \code{"greater"} means superiority of treatment over placebo is established
#' by rate difference greater than `delta`.
#' @param ... Subset conditions compatible with \code{dplyr::filter}.
#' \code{glm} will be fitted on this subset only. This argument can be useful
#' to create a subset of data for analysis when a trial consists of more
#' than two arms. By default, it is not specified,
#' all data will be used to fit the model. More than one condition can be
#' specified in \code{...}, e.g.,
#' \code{fitFarringtonManning('remission', 'pbo', data, delta, arm \%in\% c('pbo', 'low dose'), cfb > 0.5)},
#' which is equivalent to:
#' \code{fitFarringtonManning('remission', 'pbo', data, delta, arm \%in\% c('pbo', 'low dose') & cfb > 0.5)}.
#' Note that if more than one treatment arm are present in the data after
#' applying filter in \code{...}, models are fitted for placebo verse
#' each of the treatment arms.
#' @param delta the rate difference between a treatment arm and placebo under
#' the null. 0 by default.
#'
#' @returns a data frame with three columns:
#' \describe{
#' \item{\code{arm}}{name of the treatment arm. }
#' \item{\code{placebo}}{name of the placebo arm. }
#' \item{\code{estimate}}{estimate of rate difference. }
#' \item{\code{p}}{one-sided p-value for log odds ratio (treated vs placebo). }
#' \item{\code{info}}{sample size in the subset with \code{NA} being removed. }
#' \item{\code{z}}{the z statistics of log odds ratio (treated vs placebo). }
#' }
#'
#' @references Farrington, Conor P., and Godfrey Manning. "Test statistics and sample size formulae for comparative binomial trials with null hypothesis of non-zero risk difference or non-unity relative risk." Statistics in medicine 9.12 (1990): 1447-1454.
#'
#' @export
#'
fitFarringtonManning <- function(endpoint, placebo, data, alternative, ..., delta = 0) {

  if(!is.character(endpoint) || length(endpoint) != 1){
    stop("endpoint must be a single character string")
  }

  if(!is.character(placebo) || length(placebo) != 1){
    stop("placebo must be a single character string")
  }

  if(!is.data.frame(data)){
    stop("data must be a data frame")
  }

  alternative <- match.arg(alternative, choices = c('greater', 'less'))

  required_cols <- c('arm', endpoint)
  if(!all(required_cols %in% names(data))){
    stop('Columns <',
         paste0(setdiff(required_cols, names(data)), collapse = ', '),
         '> are not present in locked data. ',
         'Please check endpoint\'s name. ')
  }

  if(delta < -1 || delta > 1){
    stop('delta should be in [-1, 1]. ')
  }

  # Prepare the data based on condition ...
  filtered_data <- if(...length() == 0){
    data
  }else{
    tryCatch({
      data %>% dplyr::filter(...)
    },
    error = function(e){
      stop('Error in filtering data for fitting Farrington-Manning test. ',
           'Please check condition in ..., ',
           'which should be compatible with dplyr::filter. ')
    })
  }

  # Check if any data remains after filtering
  if (nrow(filtered_data) == 0) {
    stop("No data remaining after applying subset condition. ")
  }

  treatment_arms <- setdiff(unique(filtered_data$arm), placebo) %>% sort()

  ret <- NULL

  for(trt_arm in treatment_arms){
    sub_data <- filtered_data %>%
      dplyr::filter(.data$arm %in% c(placebo, trt_arm)) %>%
      dplyr::filter(!is.na(.data[[endpoint]]))

    p1 <- mean(sub_data[[endpoint]][sub_data$arm %in% trt_arm])
    p2 <- mean(sub_data[[endpoint]][sub_data$arm %in% placebo])
    n1 <- sum(sub_data$arm %in% trt_arm)
    n2 <- sum(sub_data$arm %in% placebo)
    delta <- 0

    # standard deviation of the rate difference under the null hypothesis (risk difference = -delta)
    theta <- n2/n1
    d <- -p1 * delta * (1 + delta)
    c <- delta^2 + delta * (2 * p1 + theta + 1) + p1 + theta * p2
    b <- -(1 + theta + p1 + theta * p2 + delta * (theta + 2))
    a <- 1 + theta
    v <- b^3/(27*a^3) - b*c/(6*a^2) + d/(2*a)
    u <- sign(v)*sqrt(b^2/(9*a^2) - c/(3*a))
    w <- (pi + acos(   max(min(1, v/u^3), 0, na.rm = TRUE)  ))/3
    p1_null <- 2*u*cos(w) - b/(3*a)
    p2_null <- p1_null - delta
    sd_diff_null <- sqrt(p1_null*(1 - p1_null)/n1 + p2_null*(1 - p2_null)/n2)

    z <- (p1 - p2 - delta)/sd_diff_null
    p <- ifelse(alternative == 'greater', 1 - pnorm(z), pnorm(z))

    info <- nrow(sub_data)

    ret <- rbind(ret, data.frame(arm = trt_arm, placebo = placebo,
                                 estimate = p1 - p2,
                                 p = p, info = info, z = z)
    )
  }

  rownames(ret) <- NULL
  class(ret) <- c('fit_fm', class(ret))
  ret
}

# Farrington-Manning test
#
# Covers:
#   - return shape (arm, placebo, estimate, p, info, z) and class
#   - sign of z and p follow alternative under clear treatment benefit
#   - loops over multiple treatment arms
#   - ... filter subsets the data
#   - input validation

test_that("fitFarringtonManning returns expected shape and class", {

  set.seed(1)
  n <- 400
  data <- data.frame(
    arm = rep(c("pbo", "trt"), each = n),
    ep  = c(rbinom(n, 1, 0.20), rbinom(n, 1, 0.40))
  )
  res <- fitFarringtonManning(endpoint = "ep", placebo = "pbo",
                              data = data, alternative = "greater")

  expect_s3_class(res, "fit_fm")
  expect_named(res, c("arm", "placebo", "estimate", "p", "info", "z"))
  expect_equal(nrow(res), 1)
  expect_equal(res$arm, "trt")
  expect_equal(res$placebo, "pbo")
  expect_equal(res$info, 2 * n)
  # estimate is p1 - p2; under treatment benefit it should be positive
  expect_gt(res$estimate, 0)
  expect_gt(res$z, 0)
  expect_lt(res$p, 0.05)
})

test_that("fitFarringtonManning respects alternative='less'", {

  set.seed(2)
  n <- 400
  data <- data.frame(
    arm = rep(c("pbo", "trt"), each = n),
    ep  = c(rbinom(n, 1, 0.40), rbinom(n, 1, 0.20))
  )
  res <- fitFarringtonManning(endpoint = "ep", placebo = "pbo",
                              data = data, alternative = "less")
  expect_lt(res$estimate, 0)
  expect_lt(res$z, 0)
  expect_lt(res$p, 0.05)
})

test_that("fitFarringtonManning fits one model per non-placebo arm", {

  set.seed(3)
  n <- 300
  data <- data.frame(
    arm = rep(c("pbo", "low", "high"), each = n),
    ep  = c(rbinom(n, 1, 0.20),
            rbinom(n, 1, 0.30),
            rbinom(n, 1, 0.45))
  )
  res <- fitFarringtonManning(endpoint = "ep", placebo = "pbo",
                              data = data, alternative = "greater")
  expect_equal(sort(res$arm), sort(c("high", "low")))
  expect_true(all(res$placebo == "pbo"))
  # high arm should give a stronger effect (smaller p, larger estimate)
  hi <- res[res$arm == "high", ]
  lo <- res[res$arm == "low", ]
  expect_gt(hi$estimate, lo$estimate)
})

test_that("fitFarringtonManning ... filter restricts the sample", {

  set.seed(4)
  n <- 300
  data <- data.frame(
    arm    = rep(c("pbo", "trt"), each = n),
    region = sample(c("A", "B"), 2 * n, replace = TRUE),
    ep     = c(rbinom(n, 1, 0.2), rbinom(n, 1, 0.4))
  )
  full <- fitFarringtonManning("ep", "pbo", data, "greater")
  sub  <- fitFarringtonManning("ep", "pbo", data, "greater", region == "A")
  expect_lt(sub$info, full$info)
  expect_equal(sub$info, sum(data$region == "A"))
})

test_that("fitFarringtonManning input validation", {

  d <- data.frame(arm = c("pbo", "trt"), ep = c(1, 0))
  expect_error(fitFarringtonManning(endpoint = c("ep", "ep2"),
                                    placebo = "pbo", data = d,
                                    alternative = "greater"),
               "single character")
  expect_error(fitFarringtonManning("ep", c("pbo", "x"), d, "greater"),
               "single character")
  expect_error(fitFarringtonManning("ep", "pbo", list(), "greater"),
               "data frame")
  expect_error(fitFarringtonManning("ep", "pbo", d, "two-sided"),
               regexp = NULL)
  expect_error(fitFarringtonManning("missing", "pbo", d, "greater"),
               "Columns")
  expect_error(fitFarringtonManning("ep", "pbo", d, "greater", delta = 2),
               "delta")
  expect_error(fitFarringtonManning("ep", "pbo", d, "greater",
                                    nonexistent_var > 0),
               "Error in filtering")
})

test_that("fitFarringtonManning errors when filtered data is empty", {

  d <- data.frame(arm = rep(c("pbo", "trt"), each = 10),
                  ep  = rbinom(20, 1, .5),
                  flag = 0)
  expect_error(fitFarringtonManning("ep", "pbo", d, "greater", flag == 1),
               "No data remaining")
})

#' Fit linear regression model
#'
#' @description
#' Fit linear regression model on a continuous endpoint.
#'
#' Refer to \href{https://zhangh12.github.io/TrialSimulator/articles/wrappers.html}{this vignette}
#' for more information and examples.
#'
#' @param formula an object of class \code{formula}. Must include \code{arm} and
#' endpoint in \code{data}. Covariates can be adjusted.
#' @param placebo Character. String indicating the placebo arm in \code{data$arm}.
#' @param data Data frame. Usually it is a locked data set.
#' @param alternative a character string specifying the alternative hypothesis,
#' must be one of \code{"greater"} or \code{"less"},
#' i.e., one-sided test is enforced. No default value.
#' \code{"greater"} means superiority of treatment over placebo is established
#' by a greater mean in treated arm.
#' @param ... Subset conditions compatible with \code{dplyr::filter}.
#' \code{glm} will be fitted on this subset only. This argument can be useful
#' to create a subset of data for analysis when a trial consists of more
#' than two arms. By default, it is not specified,
#' all data will be used to fit the model. More than one condition can be
#' specified in \code{...}, e.g.,
#' \code{fitLinear(cfb ~ arm, 'pbo', data, 'greater', arm \%in\% c('pbo', 'low dose'), cfb > 0.5)},
#' which is equivalent to:
#' \code{fitLinear(cfb ~ arm, 'pbo', data, 'greater', arm \%in\% c('pbo', 'low dose') & cfb > 0.5)}.
#' Note that if more than one treatment arm are present in the data after
#' applying filter in \code{...}, models are fitted and tested for placebo verse
#' each of the treatment arms.
#'
#' @returns a data frame with columns:
#' \describe{
#' \item{\code{arm}}{name of the treatment arm. }
#' \item{\code{placebo}}{name of the placebo arm. }
#' \item{\code{estimate}}{estimate of average treatment effect of \code{arm}. }
#' \item{\code{p}}{one-sided p-value for between-arm difference (treated vs placebo). }
#' \item{\code{info}}{sample size used in model with \code{NA} being removed. }
#' \item{\code{z}}{z statistics of between-arm difference (treated vs placebo). }
#' }
#'
#' @export
#'
fitLinear <- function(formula, placebo, data, alternative, ...) {

  if(!inherits(formula, 'formula')){
    stop('formula must be a formula object with "arm" indicating the column arm in data. ')
  }

  if(!is.character(placebo) || length(placebo) != 1){
    stop("placebo must be a single character string")
  }

  if(!is.data.frame(data)){
    stop("data must be a data frame")
  }

  alternative <- match.arg(alternative, choices = c('greater', 'less'))

  vars_in_formula <- all.vars(formula)
  missing_vars <- setdiff(vars_in_formula, names(data))
  if(length(missing_vars) > 0){
    stop('The following variable(s) used in formula are missing from data: \n',
         paste0(missing_vars, collapse = ', '))
  }

  if(!'arm' %in% vars_in_formula){
    stop('formula must include main effect term for arm. ')
  }

  # Prepare the data based on condition ...
  filtered_data <- if(...length() == 0){
    data
  }else{
    tryCatch({
      data %>% dplyr::filter(...)
    },
    error = function(e){
      stop('Error in filtering data for fitting a linear regression model. ',
           'Please check condition in ..., ',
           'which should be compatible with dplyr::filter. ')
    })
  }

  # Check if any data remains after filtering
  if (nrow(filtered_data) == 0) {
    stop("No data remaining after applying subset condition. ")
  }

  treatment_arms <- setdiff(unique(filtered_data$arm), placebo) %>% sort()

  ret <- NULL

  for(trt_arm in treatment_arms){
    sub_data <- filtered_data %>% dplyr::filter(.data$arm %in% c(placebo, trt_arm))

    # Ensure arm is a factor with placebo and treatment
    sub_data$arm <- factor(sub_data$arm, levels = c(placebo, trt_arm))

    # Fit the linear regression model
    fit <- glm(formula, data = sub_data, family = 'gaussian')

    res <- fit %>%
      emmeans(~ arm) %>%
      contrast(method =
                 list('trt_vs_pbo' = setNames(c(-1, 1), c(placebo, trt_arm)))
               ) %>%
      summary()

    estimate <- res$estimate
    z <- res$t.ratio
    df <- res$df
    p <- ifelse(alternative == 'greater', 1 - pt(z, df = df), pt(z, df = df))

    info <- fit$df.residual + fit$rank

    ret <- rbind(ret, data.frame(arm = trt_arm, placebo = placebo,
                                 estimate = estimate,
                                 p = p, info = info, z = z
                                 )
                 )
  }

  rownames(ret) <- NULL
  class(ret) <- c('fit_linear', class(ret))
  ret
}

#' Class of Milestones
#' @description
#' Create a class of milestone. A milestone means the time point to take an action,
#' e.g., carrying out (futility, interim, final) analysis for
#' adding/removing arms, or stopping a trial early.
#' It can also be any more general time point where trial
#' data is used in decision making or adaptation. For example, one can define a
#' milestone for changing randomization scheme, sample size re-assessment, trial
#' duration extension etc.
#'
#' Public methods in this R6 class are used in developing
#' this package. Thus, we have to export the whole R6 class which exposures all
#' public methods. However, none of the public methods on this page is
#' useful to end users. Instead, refer to the
#' \href{https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html}{vignette}
#' to learn how to define milestones when performing simulation using
#' \code{TrialSimulator}.
#'
#' @docType class
#'
#' @export
Milestones <- R6::R6Class(
  'Milestones',

  private = list(
    name = NULL,
    type = NULL,
    trigger_condition = NULL,
    action = NULL,
    action_args = list(),
    triggered = FALSE, ## logical. Whether this milestone has been triggered
                      ## (to avoid repeated execution)
    silent = FALSE,
    is_dry_run = FALSE
  ),

  public = list(

    #' @description
    #' initialize milestone
    #' @param name character. Name of milestone.
    #' @param type character vector. Milestone type(s) (futility, interim, final),
    #' a milestone can be of multiple types. This is for information purpose so
    #' can be any string.
    #' @param trigger_condition function to check if this milestone should
    #' trigger. See vignette \code{Condition System for Triggering Milestones in a Trial}.
    #' @param action function to execute when the milestone triggers.
    #' @param ... (optional) arguments of \code{action}.
    initialize = function(name,
                          type = name,
                          trigger_condition,
                          action = doNothing,
                          ...){
      stopifnot(is.character(name) && (length(name) == 1))
      stopifnot(is.character(type))
      if(!('Condition' %in% class(trigger_condition))){
        stop('trigger_condition should be created by functions <',
             'eventNumber, calendarTime, enrollment',
             '> and their combination using (), & and |. ')
      }
      # allow no specified action, for testing purpose.
      stopifnot(is.function(action) || is.null(action))

      # Capture fixed arguments for action
      dots <- list(...)
      if(length(dots) && (is.null(names(dots)) || any(names(dots) == ''))){
        stop('All extra arguments to milestone(...) must be named; ',
             'they are passed to `action`.')
      }

      if(is.function(action)){
        args_in_action <- names(formals(action))
        if(length(args_in_action) == 0 || args_in_action[1] != 'trial'){
          stop('Action function ', deparse(substitute(action)), ' must have ',
               '\'trial\' as its first argument.')
        }
      }

      if(!('...' %in% args_in_action)){
        allowed_args <- setdiff(args_in_action, 'trial')
        unknown_args <- setdiff(names(dots), allowed_args)
        if(length(unknown_args)){
          stop('Unknown argument(s) <',
               paste0(unknown_args, collapse = ', '),
               '> in the action function of milestone <',
               name, '>. ')
        }

        req_mask <- vapply(formals(action)[allowed_args],
                           function(x)
                             identical(x, quote(expr=)), logical(1))
        required <- allowed_args[req_mask]
        missing_req <- setdiff(required, names(dots))
        if(length(missing_req)){
          stop('Missing required argument(s) <',
               paste0(missing_req, collapse = ', '),
               '> in action function of milestone <',
               name, '>. ')
        }
      }

      private$name <- name
      private$type <- type

      private$trigger_condition <- trigger_condition

      private$action <- action
      private$action_args <- dots
      private$triggered <- FALSE
      private$is_dry_run <- FALSE
    },

    #' @description
    #' return name of milestone
    get_name = function(){
      private$name
    },

    #' @description
    #' return type(s) of milestone
    get_type = function(){
      private$type
    },

    #' @description
    #' return trigger_condition function
    get_trigger_condition = function(){
      private$trigger_condition
    },

    #' @description
    #' return action function
    get_action = function(){
      private$action
    },

    #' @description
    #' set if dry run should be carried out for the milestone. For more details,
    #' refer to \code{Controller::run}.
    #' @param dry_run logical.
    set_dry_run = function(dry_run){
      private$is_dry_run <- dry_run
    },

    #' @description
    #' execute action function
    #' @param trial a \code{Trial} object.
    execute_action = function(trial){

      if(private$is_dry_run){
        action <- .default_action()
      }else{
        action <- do.call(self$get_action(), c(list(trial), private$action_args))
      }

      if(!private$silent && !is.null(action)){
        message('Action for milestone <', self$get_name(), '> is executed. ')
        # print(action)
      }

    },

    #' @description
    #' return trigger status
    get_trigger_status = function(){
      private$triggered
    },

    #' @description
    #' reset an milestone so that it can be triggered again. Usually, this is called
    #' before the controller of a trial can run additional replicates
    #' of simulation.
    reset = function(){
      private$triggered <- FALSE
    },

    #' @description
    #' trigger an milestone (always TRUE) and execute action accordingly. It calls
    #' Trial$get_data_lock_time() to lock data based on conditions implemented
    #' in Milestones$trigger_condition. If time that meets the condition cannot be
    #' found, Trial$get_data_lock_time() will throw an error and stop the
    #' program. This means that user needs to adjust their trigger_condition
    #' (e.g., target number of events (target_n_events) is impossible to
    #' reach).
    #' @param trial a \code{Trial} object.
    trigger_milestone = function(trial){
      if(self$get_trigger_status()){
        if(!private$silent){
          message('Milestone <', self$get_name(),
                  '> has already been triggered before, thus is skipped. ')
        }
        return(NULL)
      }

      ## find time that meets the trigger condition to lock data in trial,
      ## so that action can be executed on it.
      if(!private$silent){
        message('Condition of milestone <', self$get_name(), '> is being checked. \n')
      }

      data_lock_time <- self$get_trigger_condition()$get_trigger_time(trial)

      ## always lock data after an milestone and before taking actions
      trial$lock_data(data_lock_time, self$get_name())

      self$execute_action(trial)
      private$triggered <- TRUE
    },

    #' @description
    #' mute all messages (not including warnings)
    #' @param silent logical.
    mute = function(silent){
      private$silent <- silent
    }
  )

)

#' Extending Duration of a Trial
#'
#' @description
#'
#' set trial duration in an adaptive designed trial. New duration must be
#' longer than the old one. All patients enrolled
#' before resetting the duration are truncated (non-TTE endpoints) or
#' censored (TTE endpoints) at the original duration. This helps maintain
#' proper type I error or family-wise error rate and control multiplicity when
#' conducting testing statistics. For more
#' details of why this is necessary, please refer to
#' \href{https://onlinelibrary.wiley.com/doi/10.1002/pst.1926}{Jorgens et al. 2019}.
#'
#' Note that this function should only be called within action functions of
#' milestones. It is users' responsibility to ensure that and
#' \code{TrialSimulator} has no way to track it.
#'
#' This is a user-friendly wrapper of the member function of trial, i.e.,
#' \code{Trials$set_duration()}, which is used in vignettes. Users who are not
#' familiar with the concept of classes may consider using this wrapper
#' directly.
#'
#' @param trial a trial object returned by \code{trial()}.
#' @param duration new duration of a trial. It must be greater than the current
#' duration.
#'
#' @export
#'
set_duration <- function(trial, duration){

  trial$set_duration(duration = duration)

}

Condition <- R6::R6Class(
  'Condition',

  public = list(
    initialize = function() {},

    get_trigger_time = function(trial){
      stop('Abstract method: get_trigger_time must be implemented in subclass. ')
    },

    print = function(){
      cat('Condition object\n')
      invisible(self)
    }
  )
)

## operator overloading methods
Condition$set('public',
              '|',
              function(other){
                ConditionCombiner$new(list(self, other), operation = 'or')
              })

Condition$set('public',
              '&',
              function(other){
                ConditionCombiner$new(list(self, other), operation = 'and')
              })

## operator overloading for condition objects
#' @export
`|.Condition` <- function(cond1, cond2){
  cond1$`|`(cond2)
}

#' @export
`&.Condition` <- function(cond1, cond2){
  cond1$`&`(cond2)
}

EventCountCondition <- R6::R6Class(
  'EventCountCondition',

  inherit = Condition,

  public = list(
    endpoint = NULL,
    n = NULL,
    filter_conditions = NULL,
    arms = NULL,

    initialize = function(endpoint, n, ..., arms = NULL){
      stopifnot(is.character(endpoint))
      stopifnot(length(endpoint) == 1)

      stopifnot(is.wholenumber(n))
      stopifnot(length(n) == 1)

      stopifnot(is.null(arms) || is.character(arms))

      self$endpoint <- endpoint
      self$n <- n
      self$filter_conditions <- enquos(...)
      self$arms <- arms
    },

    get_trigger_time = function(trial){

      milestone_time <- trial$get_data_lock_time_by_event_number(
        endpoints = self$endpoint,
        arms = self$arms,
        target_n_events = self$n,
        !!!self$filter_conditions,
        type = 'all'
      )

      milestone_time
    },

    print = function(){
      cat('Number of events (', self$endpoint, ') >= ', self$n, sep = '')

      if(!is.null(self$arms)){
        cat(' in arms <', paste0(self$arms, collapse = ', '), '>. ', sep = '')
      }else{
        cat(' in all arms in a trial. ')
      }

      cat('\n')
      invisible(self)
    }
  )
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache = TRUE,
  cache.path = 'cache/conditionSystem/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, message = FALSE, echo=FALSE---------------------------------------
library(TrialSimulator)

## ----oiea---------------------------------------------------------------------
## Note: the unit of time depends on the context of a trial
##       It is users responsibility to align it with trial's parameters
calendarTime(time = 6)

## ----ioea---------------------------------------------------------------------
enrollment(n = 520)

## ----ghafl--------------------------------------------------------------------
enrollment(n = 400, min_treatment_duration = 3)

## ----uuea---------------------------------------------------------------------
## condition is based on number of event in the placebo arm
## Note: 'pfs' is the name of the endpoint, i.e., endpoints(name = 'pfs', ...)
## Note: 'pbo' is the name of the placebo arm, i.e., arm(name = 'pbo', ...)
eventNumber(endpoint = 'pfs', n = 340, arms = 'pbo')

## condition can be based on number of event in specific arms
eventNumber(endpoint = 'pfs', n = 340, arms = c('pbo', 'trt'))

## condition can be based on number of event in the trial
## Note: for example, a trial of more than two arms. 
##       Here the number of PFS events are counted on all arms in trial
eventNumber(endpoint = 'pfs', n = 340)

## ----eaiggo-------------------------------------------------------------------
## observe at least 200 OS events on at least 500 enrolled patients, or
## the trial has been running for at least 12 months
(enrollment(n = 500) & eventNumber(endpoint = 'os', n = 200)) | 
  calendarTime(time = 12)

## observe 320 OS or 480 PFS event when the trial has been running for 6 months
calendarTime(time = 6) & 
  (eventNumber('os', n = 320) | eventNumber('pfs', n = 480))

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/dynamicTreatmentSwitching/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, echo = FALSE, message = FALSE-------------------------------------
library(TrialSimulator)
library(mvtnorm)
library(dplyr)
library(kableExtra)

## ----eval=FALSE---------------------------------------------------------------
# treatment_allocator <- function(patient_data){...}
# time_selector <- function(patient_data){...}
# data_modifier <- function(patient_data){...}
# 
# regimen <- regimen(treatment_allocator, time_selector, data_modifier)

## ----eval=FALSE---------------------------------------------------------------
# trial <- trial(...)
# trial$add_regime(regimen)

## ----eval=FALSE---------------------------------------------------------------
# trial <- trial(...)
# trial$add_arms(sample_ratio, soc, low_dose, high_dose)
# trial$add_regime(regimen)

## ----echo=FALSE, error=TRUE---------------------------------------------------
try({
msg <- tryCatch(
  {
    stop(' Member function trial$add_regimen() must be called before trial$add_arms(). ', 
         'A good practice is to call trial$add_regimen() immediately after trial() is executed. ')
    NULL
  },
  error = function(e) {
    cat('Error in trial$add_regimen(regimen) :\n', 
        e$message)
  }
)
})

## ----eval=FALSE---------------------------------------------------------------
# treatment_allocator <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   switch_to <- sample(c('low', 'high', 'stay'), nrow(patient_data),
#                         replace = TRUE, prob = c(.3, .4, .3))
#   data.frame(
#     patient_id = patient_data$patient_id,
#     new_treatment =
#       dplyr::case_when(
#         # patient die before progression cannot switch
#         patient_data$os == patient_data$pfs ~ NA_character_,
#         patient_data$arm == 'placebo' & switch_to == 'low' ~ 'low dose',
#         patient_data$arm == 'placebo' & switch_to == 'high' ~ 'high dose',
#         TRUE ~ NA_character_
#       )
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# time_selector <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## all patient in patient_data progress before die
#     ## thus pfs < os and can switch.
#     ## See treatment_allocator()
#     switch_time = patient_data$pfs
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# time_selector <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## all patient in patient_data progress before die
#     ## thus pfs < os and can switch.
#     ## See treatment_allocator()
#     switch_time = runif(nrow(patient_data), min = patient_data$pfs, max = patient_data$os)
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# data_modifier <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   f <- ifelse(patient_data$new_treatment == 'low dose', 1.1, 1.15)
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## other_endpoint = ...,
#     os = patient_data$switch_time + f * (patient_data$os - patient_data$switch_time)
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# treatment_allocator <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   data.frame(
#     patient_id = patient_data$patient_id,
#     new_treatment =
#       dplyr::case_when(
#         patient_data$arm == 'low dose' & patient_data$response == 0 ~ 'high dose',
#         TRUE ~ NA_character_
#       )
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# time_selector <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## all patient in patient_data progress before die
#     ## thus pfs < os and can switch
#     switch_time = patient_data$response_readout
#   )
# 
# }

## -----------------------------------------------------------------------------
treatment_allocator <- function(patient_data){
  ## add break point to develop and debug
  # browser()
  data.frame(
    patient_id = patient_data$patient_id,
    new_treatment =
      dplyr::case_when(
        patient_data$arm == 'placebo' ~ 'new treatment',
        TRUE ~ NA_character_
      )
  )

}

## ----eval=FALSE---------------------------------------------------------------
# time_selector <- function(patient_data){
#   ## add break point to develop and debug
#   # browser()
#   data.frame(
#     patient_id = patient_data$patient_id,
#     ## all patient in patient_data progress before die
#     ## thus pfs < os and can switch
#     switch_time = ifelse(patient_data$os <= 1, .9 * patient_data$os, patient_data$os - 1)
#   )
# 
# }

## ----eval=FALSE---------------------------------------------------------------
# what <- list(allocator1, allocator2, allocator3)
# when <- list(selector1, selector2, selector3)
# how <- list(modifier1, modifier2, modifier3)
# 
# regimen <- regimen(what, when, how)
# 
# trial <- trial(...)
# trial$add_regimen(regimen)

## Compute cumulative alpha spent on the critical values at information
## fractions.
##
computeCumulativeAlphaSpent <- function(critical_values, information_fraction){

  corr <- outer(information_fraction, information_fraction,
                function(x, y) sqrt(pmin(x, y) / pmax(x, y)))

  alpha_spent <- c()
  for(k in seq_along(critical_values)){
    if(k == 1){
      lower <- critical_values[k]
      upper <- Inf
    }else{
      lower <- c(rep(-Inf, k - 1), critical_values[k])
      upper <- c(critical_values[1:(k-1)], Inf)
    }

    alpha_spent[k] <- pmvnorm(lower = lower, upper = upper, sigma = corr[1:k, 1:k, drop = FALSE])

  }

  cumsum(alpha_spent)

}

#' Define Endpoints
#'
#' @description
#' Define one or multiple endpoints. This is a user-friendly wrapper for
#' the class constructor \code{Endpoints$new}. Users who are not familiar with
#' the concept of classes may consider using this wrapper directly.
#'
#' Note that it is users' responsibility to assure that the units of
#' readout of non-tte endpoints, dropout time, and trial duration are consistent.
#'
#' @param name character vector. Name(s) of endpoint(s)
#' @param type character vector. Type(s) of endpoint(s) in \code{name}. It
#' supports \code{"tte"} for time-to-event endpoints, and \code{"non-tte"} for
#' all other types of endpoints (e.g., continuous, binary, categorical,
#' or repeated measurement. \code{TrialSimulator} will do some verification if
#' an endpoint is of type \code{"tte"}. However, no special
#' manipulation is done for non-tte endpoints.
#' @param readout numeric vector named by non-tte endpoint(s).
#' \code{readout} should be specified for every non-tte endpoint. For
#' example, \code{c(endpoint1 = 6, endpoint2 = 3)}, which means that it takes
#' 6 and 3 unit time to get readouts of \code{endpoint1} and \code{endpoint2}
#' of a patient since being randomized. For readouts of a longitudinal endpoint
#' being collected at baseline (\code{baseline}) and 2 (\code{ep1}), 4 (\code{ep2})
#' unit time, its \code{readout} can be set as
#' \code{c(baseline = 0, ep1 = 2, ep2 = 4)}.  Error message will be prompted
#' if \code{readout} is not named or is not specified for all non-tte endpoint,
#' or it is specified for any tte endpoints. If all
#' endpoints are tte, \code{readout} should be its default value \code{NULL}.
#' @param generator a RNG function. Its first argument must be \code{n},
#' number of patients. It must return a data frame of \code{n} rows.
#' It supports all univariate random number generators, like those in
#' \code{stats}, e.g., \code{stats::rnorm}, \code{stats::rexp}, etc.
#' that with \code{n} as the first argument for number of
#' observations. \code{generator} could be any custom functions as long as
#' (1) its first argument is \code{n}; and (2) it returns a vector of
#' length \code{n} or a data frame of \code{n} rows. Custom random number
#' generator can return data of more than one endpoint. This is useful
#' when users need to simulate correlated endpoints (e.g., longitudinal
#' endpoints, or PFS/OS). The column names of
#' returned data frame should match to the argument \code{name} exactly,
#' but order does not matter. If an endpoint
#' is of type \code{"tte"}, the custom \code{generator} should also return
#' a column as event indicator. The column name of event indicator is
#' \code{<endpoint name>_event}. For example, if \code{"pfs"} is \code{"tte"},
#' then custom \code{generator} should return at least two columns
#' \code{"pfs"} and \code{"pfs_event"}. Usually \code{pfs_event} can be
#' all 1s if no censoring. For other generators, e.g.,
#' \code{TrialSimulator::PiecewiseConstantExponentialRNG} and
#' \code{TrialSimulator::CorrelatedPfsAndOs4}, the event indicators could
#' take values 0/1 due to the nature of their algorithms.
#' Censoring can also be specified later in \code{trial()}
#' through its argument \code{dropout}. See \code{?Trials}.
#' Note that if covariates, e.g., biomarker, subgroup, are needed in
#' generating and analyzing trial data, they can and should be defined as
#' endpoints as well.
#' @param ... (optional) arguments of \code{generator}.
#'
#' @examples
#' set.seed(12345)
#' ## Example 1. Generate a time-to-event endpoint.
#' ## Two columns are returned, one for time, one for event (1/0, 0 for
#  ## censoring)
#' ## A built-in RNG function is used to handle piecewise constant exponential
#' ## distribution
#' risk <- data.frame(
#'   end_time = c(1, 10, 26.0, 52.0),
#'   piecewise_risk = c(1, 1.01, 0.381, 0.150) * exp(-3.01)
#' )
#'
#' pfs <- endpoint(name = 'pfs', type='tte',
#'                 generator = PiecewiseConstantExponentialRNG,
#'                 risk = risk, endpoint_name = 'pfs')
#'
#' # run it in R console to display a summary report
#' # event indicator takes values 0/1
#' pfs
#'
#' ## Example 2. Generate continuous and binary endpoints using R's built-in
#' ## RNG functions, e.g. rnorm, rexp, rbinom, etc.
#' ep1 <- endpoint(
#'          name = 'cd4', type = 'non-tte', generator = rnorm, readout = c(cd4=1),
#'          mean = 1.2)
#' ep2 <- endpoint(
#'          name = 'resp_time', type = 'non-tte', generator = rexp, readout = c(resp_time=0),
#'          rate = 4.5)
#' ep3 <- endpoint(
#'          name = 'orr', type = 'non-tte', readout = c(orr=3), generator = rbinom,
#'          size = 1, prob = .4)
#'
#' ep1 # run it in R console. Mean and sd should be comparable to (1.2, 1.0)
#'
#' ep2 # run it in R console. Median should be comparable to log(2)/4.5 = 0.154
#'
#' ep3 # run it in R console. Mean and sd should be comparable to 0.4 and 0.49
#'
#'
#' ## Example3: delayed effect
#' ## Use piecewise constant exponential random number generator
#' ## Baseline hazards are piecewise constant
#' ## Hazard ratios are piecewise constant, resulting a delayed effect.
#' ## Note that this example is for explaining the concept of "endpoint".
#' ## Generating endpoint data manually is not the recommended way to use this package.
#'
#' run <- TRUE
#'
#' if (!requireNamespace("survminer", quietly = TRUE)) {
#'   run <- FALSE
#'   message("Please install 'survminer' to run this example.")
#' }
#'
#' if (!requireNamespace("survival", quietly = TRUE)) {
#'   run <- FALSE
#'   message("Please install 'survival' to run this example.")
#' }
#'
#' if(run){
#' risk1 <- risk
#' ep1 <- endpoint(
#'   name = 'pfs', type='tte',
#'   generator = PiecewiseConstantExponentialRNG,
#'   risk=risk1, endpoint_name = 'pfs')
#'
#' risk2 <- risk1
#' risk2$hazard_ratio <- c(1, 1, .6, .4)
#' ep2 <- endpoint(
#'   name = 'pfs', type='tte',
#'   generator = PiecewiseConstantExponentialRNG,
#'   risk=risk2, endpoint_name = 'pfs')
#'
#' n <- 1000
#' tte <- rbind(ep1$get_generator()(n), ep2$get_generator()(n))
#' arm <- rep(0:1, each = n)
#' dat <- data.frame(tte, arm)
#' sfit <- survival::survfit(
#'   survival::Surv(time = pfs, event = pfs_event) ~ arm, dat)
#'
#' survminer::ggsurvplot(sfit,
#'            data = dat,
#'            pval = TRUE,  # Show p-value
#'            conf.int = TRUE,  # Show confidence intervals
#'            risk.table = TRUE,  # Add risk table
#'            palette = c("blue", "red"))
#'
#'
#' ## print summary reports for endpoint objects in console
#' ep1
#' ep2
#'
#' }
#'
#' ## Example 4: generate correlated pfs and os
#' ## See vignette('simulatePfsAndOsIdm') and vignette('simulatePfsAndOsGumbel')
#'
#' @export
endpoint = function(
    name,
    type,
    readout = NULL,
    generator,
    ...
){

  Endpoints$new(
    name = name,
    type = type,
    readout = readout,
    generator = generator,
    ...
  )

}

#' Get simulation output in the vignette adaptiveDesign.Rmd
#'
#' Internal function that retrieves precomputed simulation results.
#' Not meant for use by package users.
#'
#' @return A data frame containing simulation results of 1000 replicates.
#'
getAdaptiveDesignOutput <- function(){
  ## this is saved by calling
  ## usethis::use_data(adaptive_design_output, internal = TRUE, overwrite = TRUE)
  return(adaptive_design_output)
}

#' Triggering Condition by Number of Events or Non-missing Observations of an Endpoint
#'
#' @description
#' Define a condition to trigger trial milestone by the number of events of a
#' time-to-event endpoint or the number of non-missing observations of a
#' non-time-to-event endpoint. The milestone will be triggered when a trial has
#' observed at least the specified number of endpoint events (or non-missing
#' observations). It can be used combined with
#' conditions specified by \link[TrialSimulator]{calendarTime} and
#' \link[TrialSimulator]{enrollment}.
#'
#' Number of events for a time-to-event endpoint can vary at different
#' milestones as more patients are randomized into a trial, or more events
#' onset over time.
#'
#' Number of non-missing observations for a non-time-to-event endpoint can vary
#' at different milestones as more patients are randomized into a trial, or more
#' patients have been treated until their readout time (thus, \code{NA} turns
#' to a value).
#'
#' Both numbers are affected by dropout.
#'
#' Refer to the
#' \href{https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html}{vignette}
#' to learn how to define milestones when performing simulation using
#' \code{TrialSimulator}.
#'
#' @param endpoint character. Name of an endpoint. It should be something that
#' is specified in the argument \code{name} in \code{endpoint()}.
#' @param n integer. Targeted number of events or non-missing obervations,
#' depending on the type of endpoint.
#' @param arms vector of character. Name of arms on which the number of
#' events/observations is counted. If \code{NULL}, use all arms that are not yet
#' removed from the trial (using \code{remove_arms()})
#' by the time of calculation.
#' @param ... subset conditions compatible with \code{dplyr::filter}. Number
#' of events/observations will be counted on subset of trial data only.
#'
#' @returns an object of class `Condition`
#'
#' @export
eventNumber <- function(endpoint, n, ..., arms = NULL){

  EventCountCondition$new(endpoint, n, ..., arms = arms)

}

#' Class of Listener
#' @description
#' Create a class of listener. A listener monitors the trial while checking
#' condition of pre-defined milestones. Actions are triggered and executed
#' automatically.
#'
#' Public methods in this R6 class are used in developing
#' this package. Thus, we have to export the whole R6 class which exposures all
#' public methods. However, only the public methods in the list below are
#' useful to end users.
#'
#' \itemize{
#' \item \code{$add_milestones()}
#' }
#'
#' @docType class
#' @examples
#' ##
#' @export
Listeners <- R6::R6Class(
  'Listeners',

  private = list(
    milestones = list(),
    silent = FALSE
  ),

  public = list(

    #' @description
    #' initialize a listener
    #' @param silent logical. \code{TRUE} to mute messages.
    initialize = function(silent = FALSE){
      stopifnot(is.logical(silent))
      private$silent <- silent
      private$milestones <- list()
    },

    #' @description
    #' register milestones with listener. Order in \code{...} matter
    #' as they are scanned and triggered in that order. It is users'
    #' responsibility to use reasonable order when calling this function,
    #' otherwise, the result of \code{Listeners$monitor()} can be problematic.
    #' @param ... one or more objects returned from \code{milestone()}.
    #'
    #' @examples
        #' listener <- listener()
        #' interim <- milestone(name = 'interim',
        #'                      when = eventNumber('endpoint', n = 100)
        #'                     )
        #' final <- milestone(name = 'final',
        #'                    when = calendarTime(time = 24)
        #'                   )
        #' listener$add_milestones(interim, final)
        #'
    add_milestones = function(...){
      milestone_list <- list(...)

      for(milestone in milestone_list){
        stopifnot(inherits(milestone, 'Milestones'))
        if(milestone$get_name() %in% names(private$milestones)){
          warning('Listener has milestone <', milestone$get_name(), '> already. ',
                  'Do you want to over-write it? \n')
        }
        private$milestones[[milestone$get_name()]] <- milestone

        if(!private$silent){
          message('A milestone <', milestone$get_name(), '> is registered. ')
        }
      }
    },

    #' @description
    #' return registered milestones
    #' @param milestone_name return \code{Milestone} object with given name(s).
    #' If \code{NULL}, all registered milestones are returned.
    get_milestones = function(milestone_name = NULL){
      if(is.null(milestone_name)){
        return(private$milestones)
      }

      if(!(milestone_name %in% names(private$milestones))){
        stop('Milestone <', milestone_name, '> is not registered. ')
      }

      return(private$milestones[[milestone_name]])
    },

    #' @description
    #' return names of registered milestones
    get_milestone_names = function(){

      names <- NULL
      for(milestone in private$milestones){
        names <- c(names, milestone$get_name())
      }
      return(names)

    },

    #' @description
    #' scan, check, and trigger registered milestones.
    #' Milestones are triggered in the order when calling
    #' \code{Listener$add_milestones}.
    #' @param trial a \code{Trial} object.
    #' @param dry_run logical. See \code{Controller::run} for more information.
    monitor = function(trial, dry_run){

      if(!trial$has_arm()){
        stop('No arm is found in the trial. ',
             'Make sure that Trial$add_arms() has been executed before running the trial. ')
      }
      for(milestone in self$get_milestones()){
        milestone$set_dry_run(dry_run)
        tryCatch(
          {milestone$trigger_milestone(trial)},
          error = function(e){
            trial$save(e$message, 'error_message', overwrite = TRUE)
            stop('Error in executing action function of milestone <',
                 milestone$get_name(), '>: \n',
                 e$message, '\n\n',
                 'Please set a breakpoint in its action function to debug it. \n',
                 'The browser() function can be helpful for a step-by-step diagnosis. \n',
                 'To fully replicate the issue in debugging, set seed = ',
                 trial$get_output('seed'), ' in trial(...). ')
          }
        )
        milestone$set_dry_run(FALSE)
      }
    },

    #' @description
    #' mute all messages (not including warnings)
    #' @param silent logical.
    mute = function(silent){
      private$silent <- silent
      for(milestone in self$get_milestones()){
        milestone$mute(private$silent)
      }
    },

    #' @description
    #' reset all milestones registered to the listener. Usually, this is called
    #' before a controller can run additional replicates of simulation.
    reset = function(){
      milestones <- self$get_milestones()
      for(milestone in milestones){
        milestone$reset()
      }
    }
  )

)

#' Define a Listener
#'
#' @description
#' Define a listener. This is a user-friendly wrapper for
#' the class constructor \code{Listener$new()}. Users who are not familiar with
#' the concept of classes may consider using this wrapper directly.
#'
#' Listener is an important concept of \code{TrialSimulator}. Used with a
#' trial object in a controller, a listener can monitor a running trial to
#' execute user-defined actions when it determine condition of triggering a
#' milestone is met. This mechanism allows the package users to focus on
#' the development of action functions in a simulation.
#'
#' @param silent logical. \code{TRUE} to mute messages.
#'
#' @examples
#'
#' listener <- listener()
#'
#' @export
#'
listener <- function(silent = FALSE){
  Listeners$new(silent = silent)
}

EnrollmentCountCondition <- R6::R6Class(
  'EnrollmentCountCondition',

  inherit = Condition,

  public = list(
    n = NULL,
    filter_conditions = NULL,
    arms = NULL,
    min_treatment_duration = 0,

    initialize = function(n, ..., arms = NULL, min_treatment_duration = 0){

      stopifnot(is.wholenumber(n))
      stopifnot(length(n) == 1)

      stopifnot(is.null(arms) || is.character(arms))

      stopifnot(is.numeric(min_treatment_duration))
      stopifnot(min_treatment_duration >= 0)

      self$n <- n
      self$filter_conditions <- enquos(...) #filter_conditions
      self$arms <- arms
      self$min_treatment_duration <- min_treatment_duration
    },

    get_trigger_time = function(trial){

      milestone_time <- trial$get_data_lock_time_by_enrollment(
        arms = self$arms,
        target_n_patients = self$n,
        self$min_treatment_duration,
        !!!self$filter_conditions
      )

      milestone_time
    },

    print = function(){
      cat('Number of randomized patients >= ', self$n)

      if(!is.null(self$arms)){
        cat(' in arms <', paste0(self$arms, collapse = ', '), '> \n')
      }

      if(self$min_treatment_duration > 0){
        cat(' and all enrolled patients have been treated for <',
            self$min_treatment_duration, '> (unit time) \n')
      }

      if(length(self$filter_conditions) > 0) {
        cat(' with conditions: ')
        for(i in seq_along(self$filter_conditions)) {
          cat(deparse(self$filter_conditions[[i]]))
          if(i < length(self$filter_conditions)){
            cat(', ')
          }
        }
      }

      cat('\n')
      invisible(self)
    }
  )
)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache = TRUE,
  cache.path = 'cache/defineActionFunctions/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, message = FALSE, echo=FALSE---------------------------------------
library(dplyr)
library(TrialSimulator)
set.seed(12345)

## ----eval=FALSE, include=TRUE-------------------------------------------------
# listener <- listener()
# #' register milestones with listener
# listener$add_milestones(interim, final)
# controller <- controller(trial, listener)
# controller$run()

createSticker <- function(){

#   font_add_google('Caveat')
#
#   sticker(
#     subplot = image_read('~/Downloads/iStock-466742904.png'),
#     package = 'TrialSimulator',
#     s_width = 2.5,
#     s_height = 2.5,
#     s_x = 1,
#     s_y = 1.1,
#     p_y = .4,
#     p_size = 16,
#     #p_color = '#00AFFF',
#     p_family = 'Caveat',
#     white_around_sticker = TRUE,
#     h_size = 0,
#     h_color = "#FFFFFF",
#     filename = 'man/figures/logo.png',
#     dpi = 300
#   )

  invisible(NULL)

}

ConditionCombiner <- R6::R6Class(
  'ConditionCombiner',

  inherit = Condition,

  public = list(
    conditions = NULL,
    operation = NULL, ## 'and' or 'or'

    initialize = function(conditions, operation){

      if(!is.list(conditions)){
        conditions <- list(conditions)
      }

      operation <- match.arg(operation, choices = c('and', 'or'))

      self$conditions <- conditions
      self$operation <- operation

    },

    get_trigger_time = function(trial){

      trigger_times <- lapply(self$conditions,
                              function(condition){
                                condition$get_trigger_time(trial)
                              })

      if(self$operation == 'and'){
        idx <- which.max(unlist(trigger_times))
      }else{
        idx <- which.min(unlist(trigger_times))
      }

      time <- trigger_times[[idx]]

      return(time)
    },

    print = function(){
      cat('Combined Condition (', self$operation, '):\n', sep = '')
      for (i in seq_along(self$conditions)) {
        output <- capture.output(self$conditions[[i]]$print())
        cat(paste0(output, collapse = '\n    '), '\n', sep = '')
      }
      invisible(self)
    }

  )
)

#' Carry out log rank test
#' @description
#' Compute log rank test statistic on an endpoint.
#'
#' Refer to \href{https://zhangh12.github.io/TrialSimulator/articles/wrappers.html}{this vignette}
#' for more information and examples.
#'
#' @param formula An object of class \code{formula} that can be used with
#' \code{survival::coxph}. Must consist \code{arm} and endpoint in \code{data}.
#' No covariate is allowed. Stratification variables are supported and can be
#' added using \code{strata(...)}.
#' @param placebo character. String of placebo in \code{data$arm}.
#' @param data data frame. Usually it is a locked data.
#' @param alternative a character string specifying the alternative hypothesis,
#' must be one of \code{"greater"} or \code{"less"},
#' i.e., one-sided test is enforced. No default value.
#' \code{"greater"} means superiority of treatment over placebo is established
#' by an hazard ratio greater than 1.
#' @param tidy logical. \code{FALSE} if more information are returned.
#' Default \code{TRUE}.
#' @param ... subset condition that is compatible with \code{dplyr::filter}.
#' \code{survival::coxph} with \code{ties = "exact"} will be fitted on this
#' subset only. This argument could be useful to create a subset of data for
#' analysis when a trial consists of more than two arms. By default it is not
#' specified, all data will be used to fit the model. More than one conditions
#' can be specified in \code{...}, e.g.,
#' \code{fitLogrank(formula, data, arm \%in\% c('pbo', 'low dose'), x > 0.5)},
#' which is equivalent to
#' \code{fitLogrank(formula, data, arm \%in\% c('pbo', 'low dose') & x > 0.5)}.
#' Note that if more than one treatment arm are present in the data after
#' applying filter in \code{...}, models are fitted for placebo verse
#' each of the treatment arms.
#'
#' @returns a data frame with three columns:
#' \describe{
#' \item{\code{arm}}{name of the treatment arm. }
#' \item{\code{placebo}}{name of the placebo arm. }
#' \item{\code{p}}{one-sided p-value for log-rank test (treated vs placebo). }
#' \item{\code{info}}{the number of events of the endpoint in the subset. }
#' \item{\code{z}}{the z statistics of log hazard ratios. }
#' }
#'
#'
#' @export
#'
fitLogrank <- function(formula, placebo, data, alternative, ..., tidy = TRUE) {

  if(!inherits(formula, 'formula')){
    stop('formula must be a formula object with "arm" indicating the column arm in data. ')
  }

  is_valid_arm_formula <- function(formula, arm_var = "arm") {
    trms <- terms(formula, specials = "strata")
    labels <- attr(trms, "term.labels")
    strata_vars <- untangle.specials(trms, "strata", order = TRUE)$vars
    non_strata_vars <- setdiff(labels, strata_vars)
    length(non_strata_vars) == 1 && non_strata_vars == arm_var
  }

  if(!is_valid_arm_formula(formula, 'arm')){
    stop('formula should be in the format of Surv(time, event) ~ arm or Surv(time, event) ~ arm + strata(...) + ... + strata(...). ')
  }

  if(!is.character(placebo) || length(placebo) != 1){
    stop("placebo must be a single character string")
  }

  if(!is.data.frame(data)){
    stop("data must be a data frame")
  }

  alternative <- match.arg(alternative, choices = c('greater', 'less'))

  vars_in_formula <- all.vars(formula)
  missing_vars <- setdiff(vars_in_formula, names(data))
  if(length(missing_vars) > 0){
    stop('The following variable(s) used in formula are missing from data: \n',
         paste0(missing_vars, collapse = ', '))
  }

  if(!'arm' %in% vars_in_formula){
    stop('formula must include main effect term for arm. ')
  }

  # Prepare the data based on condition in ...
  filtered_data <- if(...length() == 0){
    data
  }else{
    tryCatch({
      data %>% dplyr::filter(...)
    },
    error = function(e){
      stop('Error in filtering data for logrank test. ',
           'Please check condition in ..., ',
           'which should be compatible with dplyr::filter. ')
    })
  }

  # Check if any data remains after filtering
  if (nrow(filtered_data) == 0) {
    stop("No data remaining after applying subset condition. ")
  }

  treatment_arms <- setdiff(unique(filtered_data$arm), placebo) %>% sort()

  ret <- NULL

  for(trt_arm in treatment_arms){
    sub_data <- filtered_data %>% dplyr::filter(.data$arm %in% c(placebo, trt_arm))

    # Ensure arm is a factor with placebo and treatment
    sub_data$arm <- factor(sub_data$arm, levels = c(placebo, trt_arm))

    fit_cox <- fitCoxph(formula, placebo, sub_data, alternative,
                        scale = 'log hazard ratio', tidy = tidy) ## ... is not needed

    # Fit the Cox model
    lr <- tryCatch({
      coxph(formula, data = sub_data, ties = 'exact')
    }, error = function(e){
      stop('coxph model fitting failed in fitLogrank: ', e$message)
    })

    # calculate log rank statistic
    z <- sqrt(summary(lr)$sctest['test']) * ifelse(fit_cox$z > 0, 1, -1)
    p <- ifelse(alternative == 'greater', 1 - pnorm(z), pnorm(z))
    info <- fit_cox$info

    res <- data.frame(arm = trt_arm, placebo = placebo,
                      p = p, info = info, z = z
                    )
    if(!tidy){
      res$info_pbo <- fit_cox$info_pbo
      res$info_trt <- fit_cox$info_trt
      res$n_pbo <- fit_cox$n_pbo
      res$n_trt <- fit_cox$n_trt
    }

    ret <- rbind(ret, res)
  }

  rownames(ret) <- NULL
  class(ret) <- c('fit_logrank', class(ret))
  ret
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  cache = TRUE,
  cache.path = 'cache/defineTimeToEventEndpoints/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)

## ----setup, message = FALSE, echo = FALSE-------------------------------------
library(dplyr)
library(survival)
library(survminer)
library(simdata)
library(TrialSimulator)
set.seed(12345)

## ----lkioie-------------------------------------------------------------------
pfs_pbo <- endpoint(name = 'PFS', type = 'tte', 
                    generator = rexp, rate = log(2)/5.6)

## ----adfla, message=FALSE-----------------------------------------------------
test_set <- pfs_pbo$test_generator(n = 1e5)
head(test_set)
median(test_set$PFS) ## should be close to 5.6

## ----ieaofdsaf, results='asis'------------------------------------------------
pfs_pbo

## ----aldj---------------------------------------------------------------------
pfs_trt <- endpoint(name = 'PFS', type = 'tte', 
                    generator = rexp, rate = log(2)/6.4)
median(pfs_trt$test_generator(n = 1e5)$PFS) ## should be close to 6.4

## ----alfjd--------------------------------------------------------------------
pbo <- arm(name = 'placebo')
pbo$add_endpoints(pfs_pbo)
trt <- arm(name = 'treatment')
trt$add_endpoints(pfs_trt)

## ----aldjfba------------------------------------------------------------------
risk_pbo <- data.frame(
  end_time = c(2, 8, 10), 
  piecewise_risk = c(1, 0.48, 0.25) * exp(-1)
)

pfs_pbo <- endpoint(name = 'PFS', type = 'tte', 
                    generator = PiecewiseConstantExponentialRNG, 
                    risk = risk_pbo, 
                    endpoint_name = 'PFS')
risk_trt <- risk_pbo %>% 
  mutate(hazard_ratio = c(1, .6, .7))

pfs_trt <- endpoint(name = 'PFS', type = 'tte', 
                    generator = PiecewiseConstantExponentialRNG, 
                    risk = risk_trt, 
                    endpoint_name = 'PFS')

test_set <- rbind(pfs_pbo$test_generator(n = 1e4) %>% mutate(arm = 'pbo'), 
                  pfs_trt$test_generator(n = 1e4) %>% mutate(arm = 'trt'))

sfit <- survfit(Surv(time = PFS, event = PFS_event) ~ arm, test_set)
ggsurvplot(sfit, data = test_set, palette = c("blue", "red"))

## ----ldao---------------------------------------------------------------------
head(test_set %>% slice_sample(prop = 1))

## ----eegj---------------------------------------------------------------------
pbo <- arm(name = 'placebo')
pbo$add_endpoints(pfs_pbo)

trt <- arm(name = 'treatment')
trt$add_endpoints(pfs_trt)

## ----laiojb-------------------------------------------------------------------
os_pbo <- endpoint(name = 'OS', type = 'tte', 
                   generator = rexp, rate = log(2)/7.2)
os_trt <- endpoint(name = 'OS', type = 'tte', 
                   generator = rexp, rate = log(2)/8.5)

median(os_pbo$test_generator(n = 1e5)$OS) ## should be close to 7.2
median(os_trt$test_generator(n = 1e5)$OS) ## should be close to 8.5

## add endpoint to existing arms
pbo$add_endpoints(os_pbo)
trt$add_endpoints(os_trt)

## ----albaiola-----------------------------------------------------------------
custom_generator <- function(n, pfs_rate, os_rate, corr){
  
  dist <- list()
  dist[['PFS']] <- function(x) qexp(x, rate = pfs_rate)
  dist[['OS']] <- function(x) qexp(x, rate = os_rate)
  dsgn = simdata::simdesign_norta(cor_target_final = corr, 
                                dist = dist, 
                                transform_initial = data.frame,
                                names_final = names(dist), 
                                seed_initial = 1)
  
  simdata::simulate_data(dsgn, n_obs = n) %>% 
    mutate(PFS_event = 1, OS_event = 1) ## event indicators
}

## ----ojhonln------------------------------------------------------------------
corr <- matrix(c(1, .6, .6, 1), nrow = 2)
eps_pbo <- endpoint(name = c('PFS', 'OS'), type = c('tte', 'tte'), 
                    generator = custom_generator, 
                    pfs_rate = log(2)/5.6, os_rate = log(2)/7.2, 
                    corr = corr)

eps_trt <- endpoint(name = c('OS', 'PFS'), type = c('tte', 'tte'), 
                    generator = custom_generator, 
                    pfs_rate = log(2)/6.4, os_rate = log(2)/8.5, 
                    corr = corr)

test_set <- rbind(eps_pbo$test_generator(n = 1e5) %>% mutate(arm = 'pbo'), 
                  eps_trt$test_generator(n = 1e5) %>% mutate(arm = 'trt'))

with(test_set, cor(PFS, OS)) ## should be close to 0.6

## sample medians match to the parameters well
test_set %>% 
  group_by(arm) %>% 
  summarise(PFS = median(PFS), OS = median(OS))


## ----daadidal, results='asis'-------------------------------------------------
eps_pbo

## ----ieaofafa, results='asis'-------------------------------------------------
eps_trt

## ----lioe---------------------------------------------------------------------
pbo <- arm(name = 'placebo')
pbo$add_endpoints(eps_pbo)

trt <- arm(name = 'treatment')
trt$add_endpoints(eps_trt)

#' Triggering Condition by Number of Randomized Patients
#'
#' @description
#' Define a condition to trigger trial milestone by the number of randomized
#' patients. The milestone will be triggered when a trial has enrolled
#' at least the specified number of patients. It can be used combined with
#' conditions specified by \link[TrialSimulator]{calendarTime} and
#' \link[TrialSimulator]{eventNumber}.
#'
#' Refer to the
#' \href{https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html}{vignette}
#' to learn how to define milestones when performing simulation using
#' \code{TrialSimulator}.
#'
#' @param n integer. Number of randomized patients.
#' @param ... subset conditions compatible with \code{dplyr::filter}. Number
#' of randomized patients will be counted on subset of trial data only.
#' @param arms vector of character. Name of arms on which the number of patients
#' is counted. If \code{NULL}, use all arms that are not yet removed from the
#' trial by the time of calculation.
#' @param min_treatment_duration numeric. Zero or positive value.
#' minimum treatment duration of enrolled patients.
#' Default is 0, i.e., looking for triggering time based on number of enrolled
#' patients in population specified by \code{...} and \code{arms}. If positive,
#' it means that milestone is triggered when a specific number of enrolled
#' patients have received treatment for at least \code{min_treatment_duration}
#' duration. It is users' responsibility to assure that the unit of
#' \code{min_treatment_duration} are consistent with
#' readout of non-tte endpoints, dropout time, and trial duration.
#'
#' @returns an object of class `Condition`
#'
#' @examples
#'
#' ## ensure sufficient sample size of whole trial
#' enrollment(n = 100)
#'
#' ## ensure sufficient sample size in sub-group of interest
#' enrollment(n = 100, biomarker1 == 'positive' & biomarker2 == 'high')
#'
#' ## ensure sufficient sample size in high dose + placebo
#' enrollment(n = 1000, arms = c('high dose', 'placebo'))
#'
#' ## ensure sufficient treatment duration
#' enrollment(n = 500, min_treatment_duration = 2)
#'
#'
#' @export
enrollment <- function(n, ..., arms = NULL, min_treatment_duration = 0){

  EnrollmentCountCondition$new(n, ...,
                               arms = arms,
                               min_treatment_duration = min_treatment_duration)

}

#' Define a Controller
#'
#' @description
#' Define a controller of a trial. This is a user-friendly wrapper for
#' the class constructor \code{Controller$new()}. Users who are not familiar with
#' the concept of classes may consider using this wrapper directly.
#'
#' \code{TrialSimulator} uses a controller to coordinate a trial object
#' and a listener object to run simulations, in which the trial object defines
#' endpoints, arms, and other settings of a trial, while the listener object
#' monitors trials to triggered pre-defined milestones and execute action
#' functions. See vignettes of this package for more examples.
#'
#' @param trial an object returned from \code{trial()}.
#' @param listener an object returned from \code{listener()}.
#'
#' @examples
#'
#' # a minimum, meaningful, and executable example,
#' # where a randomized trial with two arms is simulated and analyzed.
#'
#' control <- arm(name = 'control arm')
#' active <- arm(name = 'active arm')
#'
#' pfs_in_control <- endpoint(name = 'PFS', type = 'tte',
#'                            generator = rexp, rate = log(2) / 5)
#' control$add_endpoints(pfs_in_control)
#'
#' pfs_in_active <- endpoint(name = 'PFS', type = 'tte',
#'                           generator = rexp, rate = log(2) / 6)
#' active$add_endpoints(pfs_in_active)
#'
#' accrual_rate <- data.frame(end_time = c(10, Inf),
#'                            piecewise_rate = c(30, 50))
#' trial <- trial(name = 'trial',
#'                n_patients = 1000,
#'                duration = 40,
#'                enroller = StaggeredRecruiter,
#'                accrual_rate = accrual_rate,
#'                dropout = rweibull, shape = 2, scale = 38)
#'
#' trial$add_arms(sample_ratio = c(1, 1), control, active)
#'
#' action_at_final <- function(trial){
#'   locked_data <- trial$get_locked_data('final analysis')
#'   fitLogrank(Surv(PFS, PFS_event) ~ arm, placebo = 'control arm',
#'              data = locked_data, alternative = 'less')
#'   invisible(NULL)
#' }
#'
#' final <- milestone(name = 'final analysis',
#'                    action = action_at_final,
#'                    when = calendarTime(time = 40))
#'
#' listener <- listener()
#' listener$add_milestones(final)
#'
#' controller <- controller(trial, listener)
#' controller$run(n = 1)
#'
#' @export
#'
controller = function(trial, listener){

  Controllers$new(trial = trial, listener = listener)

}

#' Updating Data Generators of One or More Endpoints in an Arm
#'
#' @description
#'
#' update endpoint generator in an arm. This function can be useful in, e.g.,
#' enrichment design where generator is updated so that patients are enrolled
#' from sub-population afterwards. This function can also be used when data
#' model changes over time, i.e., generating data in a new way after a milestone.
#' This function can be called multiple times to update generators of endpoints
#' one by one.
#'
#' Note that this function should only be called within action functions of
#' milestones. It is users' responsibility to ensure that and
#' \code{TrialSimulator} has no way to track it.
#'
#' This is a user-friendly wrapper of the member function of trial, i.e.,
#' \code{Trials$update_generator()}, which is used in vignettes. Users who are
#' not familiar with the concept of classes may consider using this wrapper
#' directly.
#'
#' @param trial a trial object returned by \code{trial()}.
#' @param arm_name character. Name of an arm.
#' @param endpoint_name character. A vector of endpoint names whose generator
#' is updated.
#' @param generator a random number generation (RNG) function.
#' See generator of \code{endpoint()}.
#' @param ... optional arguments for generator.
#'
#' @export
#'
update_generator <- function(trial, arm_name, endpoint_name, generator, ...){

  trial$update_generator(arm_name = arm_name,
                         endpoint_name = endpoint_name,
                         generator = generator, ...)

}

---
title: "Crossover Design with Wash-out Periods"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Crossover Design with Wash-out Period}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
editor_options: 
  markdown: 
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/crossoverWashout/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(TrialSimulator)
library(mvtnorm)
library(dplyr)
library(kableExtra)
```

The `TrialSimulator` package can handle three types of crossover design.
This vignette focuses on a trial for studies on symptom reduction, e.g.,
for chronic conditions with short-lived and reversible treatment
effects. A crossover trial is a longitudinal design where participants
receive multiple treatments sequentially, acting as their own control if
the carry-over effect can be minimized by the use of untreated wash-out
period. We can use the similar idea in the
[vignette of longitudinal endpoints](https://zhangh12.github.io/TrialSimulator/articles/defineLongitudinalEndpoints.html) to implement simulation, where endpoints at
the end of each treatment in the pre-determined and randomized sequence
are defined in function `endpoint()` with its argument `readout` is set.

To simulate dynamic treatment switching, please refer to its [vignette](https://zhangh12.github.io/TrialSimulator/articles/dynamicTreatmentSwitching.html). 

## Simulation Settings

In simulation, we assume a balanced Latin square design of four arms

| Sequence | Treatment 1 | Treatment 2 | Treatment 3 | Treatment 4 |
|----------|-------------|-------------|-------------|-------------|
| ABCD     | A           | B           | C           | D           |
| BDAC     | B           | D           | A           | C           |
| CADB     | C           | A           | D           | B           |
| DCBA     | D           | C           | B           | A           |

-   duration of wash-out periods between treatments are 1, 2, 2.5
    months, respectively.
-   duration of treatments in sequence are 3, 4, 5, 2 months,
    respectively.

The figure shows the timeline of the trial

```{r dadfaiq}
#| out-width: 100%
#| echo: false
knitr::include_graphics('crossover_readout.png')
```

Additional settings are 

-   enroll 10 patients per week for 6 weeks. In total 60 patients are randomized into four arms evenly. 
-   no dropout.
-   analyze the data when planned treatment for all patients are
    completed, i.e., approximately 25.5 weeks. 
-   statistical analysis is not implemented in this example.

## Define Endpoints in Four Arms

We collect baselines at the beginning of a new treatment (i.e., at the
end of wash-out period), and endpoint value when the treatment is
completed. To do so, we define four baseline variables and four endpoint
variables as below

<!-- Hide this action function and display it later for better typesetting. -->
```{r rng, echo=FALSE}
rng <- function(n, means, vcov = diag(1, 8)){
  ret <- as.data.frame(rmvnorm(n, mean = means, sigma = vcov))
  colnames(ret) <- c('baseline1', 'ep1', 
                     'baseline2', 'ep2', 
                     'baseline3', 'ep3', 
                     'baseline4', 'ep4')
  ret
}
```

```{r results='asis'}
all_endpoint_name <- c('baseline1', 'ep1', 
                       'baseline2', 'ep2', 
                       'baseline3', 'ep3', 
                       'baseline4', 'ep4')

readouts <- c(baseline1 = 0, ep1 = 3, 
              baseline2 = 4, ep2 = 8, 
              baseline3 = 10, ep3 = 15, 
              baseline4 = 17.5, ep4 = 19.5)

eps <- endpoint(
  name = all_endpoint_name,
  type = rep('non-tte', 8), 
  readout = readouts, 
  generator = rng, means = rep(c(0, .5), 4)
)

arm1 <- arm(name = 'ABCD')
arm1$add_endpoints(eps)
arm1
```

Here, a custom random number generator, `rng`, is provided by users to describe how data of the treatment sequence `ABCD` is generated. For the purpose of illustration, we simply assume an independent structure between baselines and endpoints (`vcov = diag(1, 8)`). Users can adopt much complex models to simulate carry-over effect or other mechanism. Note that `rng` uses its default value for `vcov`, and a custom value for its argument `means`. We will use different value for `means` when defining other arms. 

```{r ref.label='rng', eval=FALSE}
```

Adopting `TrialSimulator` in development of simulation enjoys the convenience of writing similar codes with good readability. To see that, here is how we define the other three arms

```{r}
eps <- endpoint(
  name = all_endpoint_name,
  type = rep('non-tte', 8), 
  readout = readouts, 
  generator = rng, means = rep(c(0, .6), 4) # diff means
)

arm2 <- arm(name = 'BDAC')
arm2$add_endpoints(eps)

eps <- endpoint(
  name = all_endpoint_name,
  type = rep('non-tte', 8), 
  readout = readouts, 
  generator = rng, means = rep(c(0, .2), 4), vcov = diag(1.2, 8) # diff means/vcov
)

arm3 <- arm(name = 'CADB')
arm3$add_endpoints(eps)

eps <- endpoint(
  name = all_endpoint_name,
  type = rep('non-tte', 8), 
  readout = readouts, 
  generator = rng, means = rep(c(0, 0), 4) # diff means
)

arm4 <- arm(name = 'DCBA')
arm4$add_endpoints(eps)
```

## Define a Trial
As planned, we recruit 10 patients per week until 60 patients are randomized. A built-in enroller function `StaggeredRecruiter` is used to enroll patients. Note that the last patient would be randomized at the 6th week and complete all treatments at week 25.5. We can set `duration = 25.5` in function `trial()`, however, we set it to a greater number (28) so that we can illustrate some nice features of `TrialSimulator` later. 

```{r}
accrual_rate <- data.frame(end_time = c(6, Inf),
                           piecewise_rate = c(10, 10))

trial <- trial(name = 'crossover-trial', 
               n_patients = 60, 
               duration = 28,
               enroller = StaggeredRecruiter, accrual_rate = accrual_rate, 
               silent = TRUE)

trial$add_arms(sample_ratio = c(1, 1, 1, 1), arm1, arm2, arm3, arm4)
trial
```

## Define Milestone and Action

Next, we define the only milestone of the trial where data is analyzed. In this example, we have three different ways to specify the triggering condition of the milestone equivalently, using the three key functions of the condition system.

<!-- Hide this action function and display it later for better typesetting. -->
```{r action_function, echo=FALSE}
action <- function(trial){
  locked_data <- trial$get_locked_data('final')
  ## omit statistical analysis
  
  trial$save(value = 'anything', name = 'result')
  ## save more results for summary of simulation
  # trial$save(value = ..., name = ...)
}
```

The first one uses `calendarTime()` where we set the 25.5th weeks as the end of trial. Note that the simulate will stop at that time even if the duration of trial is set to be 28 weeks.

```{r}
final <- milestone(name = 'final', 
                   when = calendarTime(time = 25.5), 
                   action = action)
```

We can also define the milestone time by the number of readouts of `ep4`, i.e., endpoint when the last treatment is completed. In the code below, we trigger the milestone when all 60 patients have their `ep4` readouts. 

```{r}
final <- milestone(name = 'final', 
                   when = eventNumber(endpoint = 'ep4', n = 60), 
                   action = action)
```

Equivalently, the milestone can be define as the time point when all 60 patients are enrolled and have been treated for at least 19.5 weeks. 

```{r}
final <- milestone(name = 'final', 
                   when = enrollment(n = 60, min_treatment_duration = 19.5), 
                   action = action)
```

The action function, `action()`, is an essential component when defining a milestone. It takes the `trial` object as its first argument, and any optional arguments follow. Statistical analysis can be carried out in the function using data locked for milestone. The data cut can be requested by calling the member function `get_locked_data(milestone_name)`, followed by user-defined analysis, decisions, and actions. As the statistical methods for crossover design is out of the scope of the package, we omit it from implementation. 

```{r ref.label='action_function', eval=FALSE}
```

## Execute a Trial

Now we have all pieces to complete the implementation of simulation. Let's register the milestone with a listener, so that it can be monitored automatically when a controller is running a trial.

```{r}
listener <- listener()
listener$add_milestones(final)

controller <- controller(trial, listener)
controller$run(n = 1, plot_event = TRUE)
```

Simulation results are saved and can be retrieved by calling the member function `get_output()` of a controller. In this example, we can see the column `result` we saved manually in action function, along with many other columns saved by the controller automatically. To know more about the naming convention of auto saved columns, please refer to [vignette of action function](https://zhangh12.github.io/TrialSimulator/articles/actionFunctions.html). To run more replicates of simulation, simply set `n` to a greater integer in `controller$run()`. 

```{r}
output <- controller$get_output()

output %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")
```

---
title: "Dose-Ranging Study"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Dose-Ranging Study}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/doseRanging/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(TrialSimulator)
library(DoseFinding)
library(dplyr)
library(kableExtra)
set.seed(12345)
```

In this vignette, we implement simulation for a dose-ranging trial under 
adaptive design. We start from a placebo arm and a treatment arm with 
highest possible dose. At an interim, if we observe promising results, we add 
arms with doses in between to inform decision at final analysis.

For the sake of simplicity, we assume a binary endpoint of interest. The
response rates at doses 0 (placebo), 0.5, 1.5, 2.5, 4 (highest dose) are
given by a `sigEmax` model below, where the response rate is assume to
be 10% in placebo, and 25% in arm of the highest dose (dose = 4). For further 
background information, refer to [vignette](https://cran.r-project.org/package=DoseFinding/vignettes/binary_data.html) 
of the `DoseFinding` package. 

```{r flaf}
mods <- DoseFinding::Mods(sigEmax = rbind(c(1, 3)), 
                          placEff = log(.1/(1 - .1)), 
                          maxEff = log(.25/(1 - .25)) - log(.1/(1 - .1)),
                          doses = c(0, 0.5, 1.5, 2.5, 4))

DoseFinding::plotMods(mods, trafo = function(x) 1/(1+exp(-x)))

## response rates on curve
x <- DoseFinding::getResp(mods, doses = c(0, 0.5, 1.5, 2.5, 4))
1 / (1 + exp(-unclass(x)))
```

## Simulation Settings

In our simulation, we assume 

- the response rates of binary endpoint as

| Dose              | 0    | 0.5  | 1.5  | 2.5  | 4    |
|-------------------|------|------|------|------|------|
| Response Rate (%) | 10.0 | 11.2 | 20.8 | 24.1 | 25.0 |

- readout time is 1 week. 
- no dropout. 
- starting with a placebo arm and a treatment arm of the highest dose (= 4), 
with a 1:1 randomization ratio.
  - enroll 5 patients per week for 7 weeks.
- an interim is planed when we have collected 30 readouts in total from the 
two arms. 
  - if z statistic of odds ratio is greater than 1
    - add three arms of dose 0.5, 1.5, and 2.5 to trial
    - enroll 20 patients per week with a randomization ratio 1:2:2:2:1, until 
    in total 150 patients are enrolled in the trial
    - final analysis is performed when readouts are available for all enrolled patients. 
  - otherwise, stop the trial.
  
## Define Placebo and High Dose Arms

```{r}
ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
               generator = rbinom, size = 1, prob = 0.1)
pbo <- arm(name = 'dose = 0.0')
pbo$add_endpoints(ep)

ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
               generator = rbinom, size = 1, prob = 0.25)
trt4 <- arm(name = 'dose = 4.0')
trt4$add_endpoints(ep)
```

## Define a Trial

```{r}
accrual_rate <- data.frame(end_time = c(7, Inf),
                           piecewise_rate = c(5, 20))

trial <- trial(name = '123', n_patients = 150, duration = 14,
               enroller = StaggeredRecruiter, accrual_rate = accrual_rate, 
               silent = TRUE)

trial$add_arms(sample_ratio = c(1, 1), pbo, trt4)
trial
```

<!-- Hide this helper function and display it in Appendix for better typesetting. -->
```{r go_nogo, echo=FALSE}
go_nogo <- function(data){

  ## candidate models for MCPMod
  doses <- c(0, 0.5, 1.5, 2.5, 4)
  candidates <- Mods(emax = c(.25, 1),
                     sigEmax = rbind(c(1, 3), c(2.5, 4)),
                     betaMod = c(1.1, 1.1),
                     placEff = log(.1/(1 - .1)),
                     maxEff = log(.25/(1 - .25)) - log(.1/(1 - .1)),
                     doses = doses)

  fit <- glm(ep ~ factor(arm) + 0, data = data, family = binomial)
  mu_hat <- coef(fit)
  S_hat <- vcov(fit)

  ## multiple contrast test
  test <- DoseFinding::MCTtest(dose = doses,
                               mu_hat, S = S_hat,
                               models = candidates, type = "general")

  ## model averaging
  model <- DoseFinding::maFitMod(dose = doses,
                                 mu_hat, S = S_hat,
                                 models = c("emax", "sigEmax", "betaMod"))
  
  ## predict response rate per dose
  prd <- predict(model, summaryFct = median, doseSeq = doses)
  
  ## convert to scale of probability
  prd_rate <- 1 / (1 + exp(-prd))

  ## go/no-go rule: MCP test p-value < 0.05 and estimated effect > 10%
  ifelse(min(attr(test$tStat, 'pVal')) < .05 &
           max(prd_rate - prd_rate[1]) > .1, 'go', 'no-go')

}
```

<!-- Hide this action function and display it later for better typesetting. -->

```{r action_at_interim, echo=FALSE}
action_at_interim <- function(trial){

  ## get data snapshot
  locked_data <- trial$get_locked_data('interim')

  ## compare two arms
  ## Risk difference = response rate in high dose - response rate in placebo
  fit <- fitLogistic(ep ~ arm, placebo = 'dose = 0.0',
                     data = locked_data, alternative = 'greater',
                     scale = 'risk difference')

  ## for summary of early termination
  trial$save(value = fit$z, name = 'z_value')
  trial$save(value = ifelse(fit$z > 1.64, 'add dose arms', 'stop trial'), 
             name = 'interim_decision')

  ## create three dose arms
  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
                 generator = rbinom, size = 1, prob = .112)
  trt1 <- arm(name = 'dose = 0.5')
  trt1$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
                 generator = rbinom, size = 1, prob = .208)
  trt2 <- arm(name = 'dose = 1.5')
  trt2$add_endpoints(ep)

  ep <- endpoint(name = 'ep', type = 'non-tte', readout = c(ep = 1),
                 generator = rbinom, size = 1, prob = .241)
  trt3 <- arm(name = 'dose = 2.5')
  trt3$add_endpoints(ep)

  ## add three new arms to trial
  trial$add_arms(sample_ratio = c(2, 2, 2), trt1, trt2, trt3)
  
}
```

<!-- Hide this action function and display it later for better typesetting. -->

```{r action_at_final, echo=FALSE}
action_at_final <- function(trial){

  locked_data <- trial$get_locked_data('final')

  trial$save(value = go_nogo(locked_data), name = 'decision')

}
```


## Define Milestones and Actions

Two milestones are triggered when 30 and 150 readouts are collected, 
respectively. Note that the 30 readouts are from the placebo and the highest 
dose arm, while the 150 readouts are from all five arms. 

```{r}
interim <- milestone(name = 'interim',
                     when = eventNumber(endpoint = 'ep', n = 30),
                     action = action_at_interim)

final <- milestone(name = 'final',
                   when = eventNumber(endpoint = 'ep', n = 150),
                   action = action_at_final)
```

In the action function `action_at_interim()`, we don't actually terminate the 
trial in simulation but always add dose arms. Instead, we save the z statistic 
for summary later, e.g., computing the proportion of early termination. 

```{r ref.label='action_at_interim', eval=FALSE}
```

At final analysis, we analyze data from all five arms using MCPMod. It calls 
a helper function `go_nogo()` which can be found in the Appendix below. 

```{r ref.label='action_at_final', eval=FALSE}
```

## Execute a Trial
After registering the two milestones with a listener, we simulate a single trial 
using `controller$run()`. We can see that more patients are randomized to the 
three newly added arms after interim, reflecting the new randomization ratio 
1:2:2:2:1. 

```{r message=FALSE}
listener <- listener()
listener$add_milestones(interim, final)

controller <- controller(trial, listener)
controller$run(n = 1, plot_event = TRUE)
```

In this trial, the z statistic at interim is relatively low (`r round(controller$get_output('z_value'), 2)`), thus is early 
terminated even if in simulation we keep going with three newly added dose arms.

```{r}
output <- controller$get_output()

output %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")
```

We can call `controller$run(n = 10)` to simulate additional trials. 
in the function `action_at_final()`, the go/no-go decision (`decision` in output) 
is determined under the assumption that the dose has been expanded, 
regardless of the interim evidence—this simplification is made solely for the 
convenience of simulation. However, the overall probability of a `"go"` should 
incorporate the interim results—specifically, the trial must not be stopped at 
interim (i.e., `interim_decision == "add dose arms"`), 
and the function `go_nogo()` must return `"go"` at the final analysis. 

```{r eval=FALSE, message=FALSE}
## important: reset before calling run() again
controller$reset()
controller$run(n = 10, plot_event = FALSE, silent = TRUE)
res <- controller$get_output()
```

```{r echo=FALSE}
res <- TrialSimulator:::getDoseRangingOutput()
```

```{r}
res %>%
  kable(escape = FALSE) %>%
  kable_styling(bootstrap_options = "striped",
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")
```

```{r message=FALSE}
## Number of overall Go
sum(res$interim_decision == 'add dose arms' & 'decision' == 'go')

## Number of early trial termination
sum(res$interim_decision == 'stop trial')
```

## Appendix: Code of Helper Function
For completeness, the full code of the helper functions `go_nogo()` is included 
below, which informs go/no-go decision at the end of the trial.

```{r ref.label='go_nogo', eval=FALSE}
```

---
title: "Crossover at a Milestone"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Crossover at a Milestone}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
editor_options:
  markdown:
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/crossoverAtMilestone/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(TrialSimulator)
```

The
[dynamic treatment switching](https://zhangh12.github.io/TrialSimulator/articles/dynamicTreatmentSwitching.html)
vignette describes crossover that is decided *at enrollment*: every
patient's switching rule is fixed when they enter the trial, off their
own (fully simulated) outcome trajectory. That covers crossover-at-
progression, rescue medication, and similar patient-level designs.

This vignette covers a different situation: crossover that becomes
available only **at, or after, a milestone**, and is therefore a decision
the trial makes once it reaches that milestone. Typical examples include:

-   a promising efficacy signal is seen at an interim analysis, so control
    patients who are still on study are, on ethical grounds, offered the
    experimental treatment;
-   after a futile dose is dropped at an interim, patients on that dose are
    switched to the retained dose;
-   a protocol amendment opens crossover from a fixed calendar time
    onward.

The defining feature is that switching is anchored to a **calendar time**
that is not known until the trial runs (it may be event-driven), and it
applies only to patients who are still in the trial when that time is
reached. Already-observed outcomes must not change; only the future may.
`TrialSimulator` supports this through the member function
`trial$crossover()`.

`crossover()` reuses the same `what()` / `when()` / `how()` interface as
`regimen()`, so this vignette assumes familiarity with the three-function
design described in the
[dynamic treatment switching](https://zhangh12.github.io/TrialSimulator/articles/dynamicTreatmentSwitching.html)
vignette. Here we focus on what is specific to a milestone crossover: the
opening time, how the eligible population is determined, and how the
package validates the triplet you supply.

## Where `crossover()` is called

Unlike `add_regimen()`, which must be registered *before* `add_arms()`
and is applied at enrollment, `crossover()` is meant to be called
**inside the action function of a milestone**:

```{r eval=FALSE}
action <- function(trial){
  locked_data <- trial$get_locked_data('interim')
  ## ... interim analysis / decision making ...
  trial$crossover(what = what_fn, how = how_fn)
}

interim <- milestone(name = 'interim',
                     when = eventNumber(endpoint = 'pfs', n = 300),
                     action = action)
```

You do not compute the crossover time yourself. `crossover()` reads the
trial's current time internally and sets the earliest crossover time `T`
for you (see the next section) -- there is no need to call
`trial$get_current_time()` or to pass any time.

For context: when a milestone is triggered, `TrialSimulator` advances the
trial's current time to that milestone's (calendar) trigger time and
automatically stores a locked snapshot of the data as of that time. This
storing happens *whether or not* the action calls `get_locked_data()`,
which is why a later milestone's action can still retrieve a snapshot
taken at an earlier milestone. `get_locked_data()` does not edit the
evolving trial data; it only returns a time-cut view of it. Inside an
action, `trial$get_current_time()` therefore returns the current
milestone's time -- the same value `crossover()` uses as the basis for
`T`. (Calling `crossover()` before any milestone has fired, when the
current time is still `0`, is rejected with an informative error.)

The user-friendly wrapper `crossover(trial, ...)` is also available for
those who prefer not to call member functions directly:

```{r eval=FALSE}
crossover(trial, what = what_fn, how = how_fn)
```

## The earliest crossover time: `delay` and the opening time

The only milestone-specific argument is `delay`. From it, `crossover()`
computes the **earliest crossover time** -- the calendar time from which
switching may take effect:

```
T = (current milestone time) + delay
```

You set only `delay`; the milestone time is supplied automatically.

-   `delay = 0` (default): crossover opens *at* the milestone. This is the
    common "let eligible patients switch now" case.
-   `delay > 0`: crossover opens a fixed amount of time *after* the
    milestone. This is convenient for an administrative ramp-up or a
    wash-out window before switching becomes effective.

`T` is the reference time for everything that follows: it determines who
is eligible, it is the *floor* for the switching time, and it is the
boundary that separates already-observed outcomes (which are protected)
from future outcomes (which `how()` may modify).

Note that `T` is the *earliest* time a switch may take effect, not
necessarily when it does. Under the default timing every selected patient
switches exactly at `T`, but a `when()` you supply may place each
patient's switch at any time at or after `T` -- for instance at a
progression that occurs after the opening.

## How the eligible population is determined

A milestone crossover only makes sense for patients who still have
something left to change. Before calling `what()`, the package therefore
restricts the candidate pool to patients who are **still in the trial at
`T`**, i.e. patients with at least one endpoint whose value has not yet
been settled by `T`. Concretely, for each patient the package checks
whether any endpoint is still *open*, using the reference time
`ref = max(T, enroll_time)` (so that not-yet-enrolled patients are
handled correctly too):

-   a time-to-event endpoint is open when
    `enroll_time + min(event_time, dropout_time) > ref`, i.e. neither the
    event nor dropout has occurred by `ref`;
-   a non-time-to-event endpoint is open when
    `enroll_time + readout > ref` **and** the reading will actually be
    taken, that is `readout <= dropout_time` and the reading falls within
    the trial duration. A baseline endpoint (`readout = 0`) is measured at
    enrollment and is therefore never open.

A patient enters the pool passed to `what()` if **any** of their
endpoints is open. Patients who have died, dropped out, or completed all
of their readouts by `T` are excluded automatically -- they have no
post-`T` outcome that a crossover could alter. Eligibility does not depend
on the treatment arm; restricting the switch to particular arms is the job
of `what()`. This mirrors the way `regimen()` "figures out patients who are
eligible for the three functions"; the difference is only the reference
time `T`.

The three functions then receive progressively smaller data frames, which
keeps the logic clean and avoids unnecessary computation:

-   `what()` receives the **eligible** patients;
-   `when()` receives the patients that `what()` actually selected for
    switching;
-   `how()` receives those same switchers, with their assigned
    `switch_time`.

All three functions share the same signature: each takes the data frame
`patient_data` and returns a data frame.

```{r eval=FALSE}
what(patient_data)   # -> patient_id, new_treatment
when(patient_data)   # -> patient_id, switch_time
how(patient_data)    # -> patient_id, <modified endpoints>
```

Importantly, the `patient_data` they receive is already the filtered
subset shown below; there is no way to reach a non-eligible patient from
inside the triplet, because the package controls the input. This both
prevents accidental logic errors and avoids unnecessary computation.

```{r funnel, echo=FALSE, fig.width=7.2, fig.height=4.2, out.width="95%", fig.alt="Funnel: trial_data is filtered to eligible patients (passed to what()), then to switchers (passed to when()), then to switchers with their switch_time (passed to how())."}
op <- par(mar = c(0, 0, 0, 0))
plot.new(); plot.window(xlim = c(0, 10), ylim = c(0, 10))
cx <- 5
yb <- c(9.3, 7.1, 4.9, 2.7, 0.6)            # band boundaries (top -> bottom)
wt <- c(3.4, 2.6, 1.8, 1.0)                 # band top widths
wd <- c(2.6, 1.8, 1.0, 0.5)                 # band bottom widths
for (i in 1:4) {
  polygon(cx + c(-wt[i], wt[i], wd[i], -wd[i]) / 2,
          c(yb[i], yb[i], yb[i + 1], yb[i + 1]),
          col = "#e8eef5", border = "#33618f", lwd = 1.4)
}
ymid <- (head(yb, -1) + tail(yb, -1)) / 2
left <- c("all patients\n(trial_data)",
          "eligible\n(open endpoint at T)",
          "switchers\n(what() selects)",
          "+ switch_time")
text(cx - 2.0, ymid, left, adj = 1, cex = 0.76)
fn <- c("what()", "when()", "how()")
for (i in 1:3) {
  arrows(cx + wt[i + 1] / 2 + 0.1, ymid[i + 1], cx + 2.0, ymid[i + 1],
         length = 0.07, col = "#33618f")
  text(cx + 2.1, ymid[i + 1], paste0("passed to ", fn[i]), adj = 0, cex = 0.82)
}
par(op)
```

To make timing rules easy to express, the package injects two helper
columns into `patient_data` for the triplet:

-   `earliest_crossover_calendar_time`, equal to `T`;
-   `earliest_crossover_time_from_enrollment`, equal to
    `max(T - enroll_time, 0)`, i.e. the earliest admissible
    `switch_time` measured from enrollment.

These let `when()` express, for example, "switch at progression, but not
before crossover opens":

```{r eval=FALSE}
time_selector <- function(patient_data){
  data.frame(
    patient_id  = patient_data$patient_id,
    switch_time = pmax(patient_data$pfs,
                       patient_data$earliest_crossover_time_from_enrollment)
  )
}
```

If `when()` is not supplied, patients switch at the opening time `T` by
default (`switch_time = earliest_crossover_time_from_enrollment`).

## How the package monitors the triplet's output

Because the triplet is user-supplied, `TrialSimulator` validates what
each function returns and stops with an informative error when an input
would corrupt the simulation. This lets you develop the three functions
incrementally and catch mistakes early (a `browser()` inside any of them
is a convenient way to debug).

**`what()`** must return a data frame with columns `patient_id` and
`new_treatment`, with one row per switching patient. Patients you leave out
simply do not switch, so the returned set may be smaller than the input.
The package checks that:

-   the result is a data frame containing the required columns;
-   no `patient_id` is duplicated;
-   every selected patient belongs to the eligible pool. Because `what()`
    is only ever shown eligible patients, selecting a patient outside that
    pool indicates a logic error and is rejected.

`new_treatment` is a free-form label; it does not need to be an existing
arm. The randomized `arm` column is never changed -- the label is recorded
only in the patient's switching history (see below).

**`when()`** must return a data frame with columns `patient_id` and
`switch_time`, measured from enrollment. The package checks that:

-   the result is a data frame containing the required columns;
-   no `patient_id` is duplicated;
-   a non-`NA` `switch_time` is provided for *every* patient selected by
    `what()`;
-   the switch does not predate the opening time, i.e.
    `enroll_time + switch_time >= T` for every switcher. A milestone
    crossover cannot take effect before it opens, so an earlier time is
    rejected.

**`how()`** must return a data frame with `patient_id` and the endpoint
columns it modifies. You return only the endpoints you change, and within
an endpoint you change just the cells that should change and return the
rest at their original value -- the natural
`ifelse(condition, new_value, original)` pattern used in the examples. The
package checks that:

-   the result is a data frame with a `patient_id` column, no duplicated
    ids, and only columns that exist in the trial data;
-   protected columns (`arm`, `enroll_time`, `dropout_time`, and the
    internal switching-history column) are not modified;
-   **only post-switch outcomes are altered.** Returning a value that
    differs from the original for a cell whose readout or event falls at
    or before the patient's `switch_time` -- a pre-switch or
    already-observed outcome -- is rejected. This guarantees that the
    crossover never rewrites history: outcomes observed before the switch
    are preserved exactly, and the data locked at the milestone is left
    intact.

In practice this means a `how()` for a milestone crossover guards the
post-switch region explicitly, for example:

```{r eval=FALSE}
data_modifier <- function(patient_data){
  data.frame(
    patient_id = patient_data$patient_id,
    ## extend only the residual (post-switch) survival; leave os unchanged
    ## for patients whose event is at or before the switch
    os = ifelse(patient_data$os > patient_data$switch_time,
                patient_data$switch_time +
                  1.2 * (patient_data$os - patient_data$switch_time),
                patient_data$os)
  )
}
```

As with `regimen()`, never apply dropout or censoring manually in
`how()`; `TrialSimulator` re-applies dropout and calendar censoring
automatically after the crossover and when locking data at later
milestones.

## Developing the triplet

The most convenient way to develop the three functions is to put a
`browser()` at the top of each, register them through `crossover()` inside
the milestone action, and run the trial. Execution then pauses *inside*
your function, with the actual `patient_data` in scope -- the eligible pool
for `what()`, and the selected switchers for `when()` and `how()`. You can
inspect the available columns (including the injected helper columns), try
your selection, timing, and update logic on real data, and confirm the
package handed you the subset you expected.

```{r eval=FALSE}
what <- function(patient_data){
  browser()                       # pause here with the eligible pool in scope
  switchers <- patient_data[patient_data$arm == 'control', ]
  data.frame(patient_id = switchers$patient_id, new_treatment = 'experimental')
}
```

Comment out `browser()` before production runs; it is in any case a no-op
in non-interactive sessions, so it never interferes with vignette building
or parallel workers.

## A worked example

The following runs a small two-arm trial end to end. Control patients who
are still on study at an interim are offered the experimental treatment;
each switches at progression, but no earlier than the moment crossover
opens, and only their post-switch survival is extended. We first define
the triplet and the action, using the full `crossover()` signature:

```{r}
what <- function(patient_data){
  ## return only the patients who switch (here, everyone on control)
  switchers <- patient_data[patient_data$arm == 'control', ]
  data.frame(
    patient_id    = switchers$patient_id,
    new_treatment = 'experimental'
  )
}

when <- function(patient_data){
  data.frame(
    patient_id  = patient_data$patient_id,
    switch_time = pmax(patient_data$pfs,
                       patient_data$earliest_crossover_time_from_enrollment)
  )
}

how <- function(patient_data){
  data.frame(
    patient_id = patient_data$patient_id,
    os = ifelse(patient_data$os > patient_data$switch_time,
                patient_data$switch_time +
                  1.3 * (patient_data$os - patient_data$switch_time),
                patient_data$os)
  )
}

crossover_action <- function(trial){
  trial$get_locked_data('interim')        # interim decision making can go here
  ## delay = 0 opens crossover at the interim; use delay > 0 for a wash-out
  trial$crossover(what = what, when = when, how = how, delay = 0)
}
```

The remaining set-up (endpoints, arms, enrollment, the milestones, and the
run) is ordinary `TrialSimulator` code and is hidden here for brevity.

```{r echo=FALSE}
os_e  <- endpoint(name = 'os',  type = 'tte', generator = rexp, rate = log(2) / 12)
pfs_e <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = log(2) / 6)
control <- arm(name = 'control', pfs <= os); control$add_endpoints(pfs_e, os_e)
trt     <- arm(name = 'trt',     pfs <= os); trt$add_endpoints(pfs_e, os_e)

trial <- trial(name = 'demo', n_patients = 200, seed = 123, duration = 36,
               enroller = StaggeredRecruiter,
               accrual_rate = data.frame(end_time = Inf, piecewise_rate = 15),
               silent = TRUE)
trial$add_arms(sample_ratio = c(1, 1), control, trt)

lst <- listener(silent = TRUE)
lst$add_milestones(
  milestone(name = 'interim', when = calendarTime(time = 12), action = crossover_action),
  milestone(name = 'final',   when = calendarTime(time = 36))
)
invisible(controller(trial, lst)$run(n = 1, silent = TRUE, plot_event = FALSE))
```

After the run, each control switcher carries a recorded switching history
in the locked data. The recorded switch times show the timing rule at
work: a patient whose progression falls after the opening switches at
progression (a larger `@` time), while a patient who would have progressed
earlier is held at the opening, so the recorded time equals
`T - enroll_time` -- the `earliest_crossover_time_from_enrollment` that
`when()` used:

```{r}
final <- trial$get_locked_data('final')
head(final[final$arm == 'control',
           c('patient_id', 'arm', 'regimen_trajectory', 'n_switches')])
```

## Scenarios

**Delayed opening (wash-out).** Supplying `delay` opens crossover after
the milestone, which is useful when switching cannot be implemented
immediately:

```{r eval=FALSE}
trial$crossover(what = what, how = how, delay = 2) # opens 2 time units later
```

**Custom timing.** Provide `when()` to switch at a clinically meaningful
time rather than at the opening, using the injected helper column to
respect the opening floor (see the `time_selector` example above).

**Patients enrolled after the milestone.** A milestone crossover also
applies to patients who enroll later: they switch according to the same
triplet once they are in the trial. There is nothing extra to do.

**Several crossovers.** `crossover()` can be called in more than one
milestone. Each call stacks an additional switching opportunity, applied
in chronological order, so a patient may cross over more than once.

## Relationship to `add_regimen()`

A milestone crossover and an enrollment-time `regimen()` are the same
machinery under the hood: both are sequences of `what()` / `when()` /
`how()` triplets, each with an earliest crossover time. The enrollment
regimen is simply the special case where that time is `0` (switching is
open from the first enrollment); `crossover()` appends a triplet whose
earliest time is the milestone time plus `delay`. The eligibility filter,
the timing floor, and the post-switch protection therefore behave
consistently across both entry points.

## Inspecting the switches

Every switch is recorded in the `regimen_trajectory` column shown above, as
a compact `"arm@0;new_treatment@switch_time"` string. Use `expandRegimen()`
to expand it into one row per regimen segment per patient for summaries and
checks (continuing the worked example above):

```{r}
head(expandRegimen(final))
```

---
title: "Define Longitudinal Endpoints"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Define Longitudinal Endpoints}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache = TRUE,
  cache.path = 'cache/defineLongitudinalEndpoints/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, message = FALSE, echo=FALSE}
library(dplyr)
library(TrialSimulator)
set.seed(12345)
```

The `TrialSimulator` package provides a flexible framework for defining endpoints in clinical trial simulations. Users can choose to generate endpoints using either built-in random number generators from standard probability distributions or custom user-defined functions. The ability to specify custom generators is particularly useful when simulating correlated endpoints, which often arise in longitudinal settings.

Longitudinal endpoints refer to outcomes measured repeatedly over time for the same subject, such as blood pressure, biomarker levels, or symptom severity scores. These repeated measures typically exhibit within-subject correlation as well as correlation with baseline values. Accurately simulating such correlation structures is essential for realistic trial design and evaluation.

In this vignette, we demonstrate how to define longitudinal endpoints using the `endpoint()` function, focusing on its `generator` argument to specify the data-generating mechanism, and the `readout` argument to determine when each endpoint is observed. Note that longitudinal endpoints are non-TTE endpoints, thus `type` in `endpoint()` must be set as `"non-tte"` and there `readout` needs to be specified. 

## Example: Simulating Longitudinal Blood Pressure Endpoints

Suppose blood pressure is measured for each patient at baseline (week 0), and again at weeks 2 and 4 following randomization. We are interested in changes from baseline at weeks 2 and 4. A typical dataset at a milestone (e.g., after enrollment or follow-up) that can be accessed by calling `trial$get_locked_data(milestone_name)` may resemble the following:

```{r adfa, echo = FALSE}
locked_data_ <- data.frame(
  baseline = runif(6, 130, 150), 
  bp2 <- runif(6, 110, 140), 
  bp4 <- runif(6, 105, 130)
) %>% 
  mutate(
    bp_cfb2 = bp2 - baseline, 
    bp_cfb4 = bp4 - baseline
  ) %>% 
  select(baseline, bp_cfb2, bp_cfb4) %>% 
  tibble()
```

```{r dalfjal, echo = FALSE}
locked_data_
```

In this example, the variables `baseline`, `bp_cfb2`, and `bp_cfb4` are observed at weeks 0, 2, and 4, respectively. Therefore, we will define `readout = c(baseline = 0, bp_cfb2 = 2, bp_cfb4 = 4)` in the endpoint definition.

To simulate correlated values across time points, we assume that `baseline`, `bp2`, and `bp4` follow a multivariate normal distribution. The user must provide the mean vector and covariance matrix for these variables. Importantly, any custom generator function must accept `n` (number of observations) as its first argument, as required by TrialSimulator, otherwise it throws an error. 

```{r eioajf}
library(mvtnorm)
bp_generator <- function(n, bp_means, bp_vcov){
  dat <- rmvnorm(n, mean = bp_means, sigma = bp_vcov) %>% 
    as.data.frame()
  names(dat) <- c('baseline', 'bp2', 'bp4')
  dat %>% 
    mutate(
      bp_cfb2 = bp2 - baseline, 
      bp_cfb4 = bp4 - baseline
    ) %>% 
    select(baseline, bp_cfb2, bp_cfb4)
}
```

This function assumes a trivariate normal distribution for blood pressure at baseline, week 2, and week 4. Users may also extend this function to support more complex simulation logic as needed.

## Defining Endpoints for an Arm

We now use the `endpoint()` function to define the longitudinal endpoints in a treatment arm. The required parameters for the custom generator, `bp_means` and `bp_vcov`, are passed through the ellipsis (`...`) argument.

```{r adlfjadl}
vcov1 <- matrix(
  c(2, 1.5, 1, 
    1.5, 3, 1.5, 
    1, 1.5, 4),
  nrow = 3
)

ep_in_trt1 <- endpoint(
  name = c('bp_cfb2', 'baseline', 'bp_cfb4'), 
  type = rep('non-tte', 3), 
  readout = c(baseline = 0, bp_cfb4 = 4, bp_cfb2 = 2), 
  generator = bp_generator, 
  bp_means = c(140, 125, 120), 
  bp_vcov = vcov1
)
```

Note that we deliberately specify the `name` and `readout` arguments in an order different from the data frame returned by `bp_generator`, to illustrate that the `endpoint()` function correctly maps variable names to their respective time points. However, the user is responsible for ensuring the order of `name` and `type` are aligned; the function cannot automatically infer types from the generator output.

This example also demonstrates that `endpoint()` can be used to generate covariates. For instance, the baseline value is typically used as a covariate in statistical models, and should therefore be simulated and retained. More generally, users can define covariates, biomarker dynamics, or subgroup indicators using the same mechanism to simulate complex trial designs. 

A simple way to generate a report for an endpoint object is to print it in console directly
```{r eiieala, results='asis'}
ep_in_trt1
```

## Defining Endpoints for Another Arm

We now define another arm with slightly different means and correlation structure for blood pressure:

```{r adlfeqadl, results='asis'}
vcov2 <- matrix(
  c(2, 1.5, 1, 
    1.5, 3, 1, 
    1, 1, 4),
  nrow = 3
)

ep_in_trt2 <- endpoint(
  name = c('bp_cfb2', 'baseline', 'bp_cfb4'), 
  type = rep('non-tte', 3), 
  readout = c(baseline = 0, bp_cfb4 = 4, bp_cfb2 = 2), 
  generator = bp_generator, 
  bp_means = c(140, 127, 122), 
  bp_vcov = vcov2
)

ep_in_trt2
```

Note that user can even specify different generators across arms. For example, `bp_generator` can be replaced by another function with different mechanism and arguments. 

## Creating Treatment Arms and Adding Endpoints

Finally, we create two treatment arms and attach the respective endpoints:

```{r daliea, results='asis'}
trt1 <- arm(name = 'treatment 1')
trt2 <- arm(name = 'treatment 2')

trt1$add_endpoints(ep_in_trt1)
trt2$add_endpoints(ep_in_trt2)

trt1

trt2
```

---
title: "Define Time-to-Event Endpoints"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Define Time-to-Event Endpoints}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache = TRUE,
  cache.path = 'cache/defineTimeToEventEndpoints/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, message = FALSE, echo = FALSE}
library(dplyr)
library(survival)
library(survminer)
library(simdata)
library(TrialSimulator)
set.seed(12345)
```

The first step in simulating a randomized clinical trial using `TrialSimulator` is to define one or more endpoints for each treatment arm. This vignette demonstrates how to use the following key functions to define time-to-event endpoints. For non-time-to-event endpoints, please refer to the separate vignette [Define Non-Time-to-Event Endpoints in Clinical Trials](defineNonTimeToEventEndpoints.html).

- `endpoint`: Creates one or more endpoints
- `test_generator`: Generates an example dataset from an `Endpoints` object

Typically, endpoints follow the same distribution family across arms but with different parameters. Distribution parameters are specified through the `...` arguments in `endpoint`. However, please note that `endpoint` offers the flexibility to specify different distribution families across arms for endpoints. 

Note that the function `test_generator` is for helping users understanding how `TrialSimulator` works. In formal simulation, we do not need to call this function. 

For the sake of completeness, in this vignette, we also demonstrates how to define an arm with the created endpoints by using the following key functions: 

- `arm`: Creates one or more arms
- `add_endpoints`: Add one or more endpoints into an arm

## Define a univariate endpoint with random number generators from `stats`

To define a time-to-event endpoint such as progression-free survival (`PFS`) following an exponential distribution, we need to specify its `name` and `type.` This specification is crucial because `TrialSimulator` ensures all arms have the same set of endpoints and manages endpoint data based on `type`. For example, endpoints of type `"tte"` (time-to-event) automatically receive an additional column `(name)_event` to indicate censoring status.

```{r lkioie}
pfs_pbo <- endpoint(name = 'PFS', type = 'tte', 
                    generator = rexp, rate = log(2)/5.6)
```

Arguments for the generator function (in this case, `rate`) are passed through `...`. We can verify that the generator works as expected by requesting an example dataset:

```{r adfla, message=FALSE}
test_set <- pfs_pbo$test_generator(n = 1e5)
head(test_set)
median(test_set$PFS) ## should be close to 5.6
```

Note that data returned by `test_generator` is only for validation purposes. During actual trial simulation, `TrialSimulator` determines when to call `generator` from which how many samples to draw. To request access of a data locked at a milestone, we can use the member function `get_locked_data` of a `Trials` object which will be introduced in the vignette [An Example of Simulating a Trial with Adaptive Design](adaptiveDesign.html). 

We can get a summary report by printing an `endpoint` object
```{r ieaofdsaf, results='asis'}
pfs_pbo
```

Similarly, we can define a `PFS` endpoint for the treatment arm:

```{r aldj}
pfs_trt <- endpoint(name = 'PFS', type = 'tte', 
                    generator = rexp, rate = log(2)/6.4)
median(pfs_trt$test_generator(n = 1e5)$PFS) ## should be close to 6.4
```

Now we can define the placebo and treatment arms by adding the `PFS` endpoints: 

```{r alfjd}
pbo <- arm(name = 'placebo')
pbo$add_endpoints(pfs_pbo)
trt <- arm(name = 'treatment')
trt$add_endpoints(pfs_trt)
```

We will cover how to create a trial object based on arms `pbo` and `trt` in 
another vignette. 

## Define a univariate endpoint with custom random number generators

`endpoints` accepts custom random number generators for univariate random variables, as long as the generator's first argument is `n` (number of observations). Additional arguments can be passed through `...` in `endpoints`.

`TrialSimulator` provides a built-in custom generator, `PiecewiseConstantExponentialRNG`, to generate time-to-event endpoints from piecewise constant exponential distributions. This is particularly useful for simulating lagged or delayed treatment effects. The example below demonstrates a scenario where treatment effect begins at week 2:

```{r aldjfba}
risk_pbo <- data.frame(
  end_time = c(2, 8, 10), 
  piecewise_risk = c(1, 0.48, 0.25) * exp(-1)
)

pfs_pbo <- endpoint(name = 'PFS', type = 'tte', 
                    generator = PiecewiseConstantExponentialRNG, 
                    risk = risk_pbo, 
                    endpoint_name = 'PFS')
risk_trt <- risk_pbo %>% 
  mutate(hazard_ratio = c(1, .6, .7))

pfs_trt <- endpoint(name = 'PFS', type = 'tte', 
                    generator = PiecewiseConstantExponentialRNG, 
                    risk = risk_trt, 
                    endpoint_name = 'PFS')

test_set <- rbind(pfs_pbo$test_generator(n = 1e4) %>% mutate(arm = 'pbo'), 
                  pfs_trt$test_generator(n = 1e4) %>% mutate(arm = 'trt'))

sfit <- survfit(Surv(time = PFS, event = PFS_event) ~ arm, test_set)
ggsurvplot(sfit, data = test_set, palette = c("blue", "red"))
```

In this example, `PiecewiseConstantExponentialRNG` adds a column `PFS_event` because we specified `endpoint_name = 'PFS'`. Some patients have their `PFS` censored (`PFS_event = 0`). By default, when a custom generator is supplied and an event column is not provided for a time-to-event endpoint, `TrialSimulator` will add one and set all values to 1 (no censoring).

```{r ldao}
head(test_set %>% slice_sample(prop = 1))
```

To add endpoint to an arm, 
```{r eegj}
pbo <- arm(name = 'placebo')
pbo$add_endpoints(pfs_pbo)

trt <- arm(name = 'treatment')
trt$add_endpoints(pfs_trt)
```

## Define multiple endpoints

We can define more than one endpoint in a trial. Let's add overall survival (OS) as a second endpoint. We'll assume the median overall survival is 7.2 months in the placebo arm and 8.5 months in the treatment arm. 

```{r laiojb}
os_pbo <- endpoint(name = 'OS', type = 'tte', 
                   generator = rexp, rate = log(2)/7.2)
os_trt <- endpoint(name = 'OS', type = 'tte', 
                   generator = rexp, rate = log(2)/8.5)

median(os_pbo$test_generator(n = 1e5)$OS) ## should be close to 7.2
median(os_trt$test_generator(n = 1e5)$OS) ## should be close to 8.5

## add endpoint to existing arms
pbo$add_endpoints(os_pbo)
trt$add_endpoints(os_trt)
```

Note that by defining `PFS` and `OS` separately (i.e., calling `endpoints` for each endpoint), we are implicitly assuming that these endpoints are independent in the trial.

## An example of a custom random number generator for correlated endpoints

To define multiple correlated endpoints, we need to create a custom generator. In this example, we define one based on the R package `simdata`:

```{r albaiola}
custom_generator <- function(n, pfs_rate, os_rate, corr){
  
  dist <- list()
  dist[['PFS']] <- function(x) qexp(x, rate = pfs_rate)
  dist[['OS']] <- function(x) qexp(x, rate = os_rate)
  dsgn = simdata::simdesign_norta(cor_target_final = corr, 
                                dist = dist, 
                                transform_initial = data.frame,
                                names_final = names(dist), 
                                seed_initial = 1)
  
  simdata::simulate_data(dsgn, n_obs = n) %>% 
    mutate(PFS_event = 1, OS_event = 1) ## event indicators
}
```

Note that for custom generator, we need to add columns of event indicator for each of the time-to-event endpoints. 

```{r ojhonln}
corr <- matrix(c(1, .6, .6, 1), nrow = 2)
eps_pbo <- endpoint(name = c('PFS', 'OS'), type = c('tte', 'tte'), 
                    generator = custom_generator, 
                    pfs_rate = log(2)/5.6, os_rate = log(2)/7.2, 
                    corr = corr)

eps_trt <- endpoint(name = c('OS', 'PFS'), type = c('tte', 'tte'), 
                    generator = custom_generator, 
                    pfs_rate = log(2)/6.4, os_rate = log(2)/8.5, 
                    corr = corr)

test_set <- rbind(eps_pbo$test_generator(n = 1e5) %>% mutate(arm = 'pbo'), 
                  eps_trt$test_generator(n = 1e5) %>% mutate(arm = 'trt'))

with(test_set, cor(PFS, OS)) ## should be close to 0.6

## sample medians match to the parameters well
test_set %>% 
  group_by(arm) %>% 
  summarise(PFS = median(PFS), OS = median(OS))

```

Instead of use `test_generator`, a simpler way to summarize an endpoint object is to print it directly. For example, the two endpoints in the placebo arm

```{r daadidal, results='asis'}
eps_pbo
```

and the two endpoints in the treatment arm 

```{r ieaofafa, results='asis'}
eps_trt
```

To define arms with these correlated endpoints, simply do the following:

```{r lioe}
pbo <- arm(name = 'placebo')
pbo$add_endpoints(eps_pbo)

trt <- arm(name = 'treatment')
trt$add_endpoints(eps_trt)
```

---
title: "Simulate Correlated Progression-Free Survival and Overall Survival as Endpoints Under Illness-Death Model"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Simulate Correlated Progression-Free Survival and Overall Survival as Endpoints Under Illness-Death Model}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/simulatePfsAndOsIdm/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(dplyr)
library(knitr)
library(ggplot2)
library(TrialSimulator)
```

Progression-free survival (PFS) and overall survival (OS) are commonly used as confirmatory endpoints in clinical trials. These two endpoints are typically highly correlated, and it is always the case that PFS $\leq$ OS. A reliable algorithm for simulating both PFS and OS is therefore of great interest.

Several methods have been proposed to achieve this, but many rely on latent variables or copula-based approaches, making it difficult to interpret and specify model parameters in practical settings. In this vignette, we do not aim to provide a comprehensive review of existing methods. Instead, we focus on a three-state illness-death model consisting of the following states: initial (0), progression (1), and death (2). For more details, refer to [Meller, Beyersmann, and Rufibach (2019)](https://onlinelibrary.wiley.com/doi/abs/10.1002/sim.8295).

```{r pplloott}
#| fig-alt: |
#|   A three-state illness-death model for PFS and OS.
#| out-width: 80%
#| echo: false
knitr::include_graphics('three_state_ill_death_model.png')
```


We consider the simplest case of the illness-death model, where all transition hazards $h(t)$ are constant over time. A data generator of this model is implemented in `TrialSimulator::CorrelatedPfsAndOs3()`, which can be used to define endpoints in `TrialSimulator::endpoint()`.

Marginally, the survival functions of PFS and OS are given by

$$
\Pr(\mbox{PFS} > t) = \exp\{-(h_{01} + h_{02})t\}
$$

and 

$$
\Pr(\mbox{OS} > t)  = \exp\{-(h_{01} + h_{02})t\} + \frac{h_{01}}{h_{12}-h_{01}-h_{02}}\exp\{-(h_{01} + h_{02})t\} - \exp\{-h_{12}t\}
$$

Thus, PFS follows an exponential distribution with rate $h_{01} + h_{02}$, which is also referred to as the all-cause hazard. Given the transition hazards $h_{01}, h_{02}, h_{12}$, a patient’s trajectory can be simulated using the following algorithm:

- **Step 1**. Generate the time to progression $t_{01}$ from an exponential distribution with rate $h_{01} + h_{02}$.

- **Step 2**. Draw a Bernoulli sample with success probability $\frac{h_{02}}{h_{01} + h_{02}}$. If successful, the patient dies immediately, i.e., time to death $t_{02} = t_{01}$, and the simulation ends. Otherwise, proceed to Step 3.

- **Step 3**. Generate the time from progression to death $t_{12}$ from an exponential distribution with rate $h_{12}$. Then, time to death $t_{02} = t_{01} + t_{12}$, and the simulation ends.


## Reparameterization

Although statistical methods exist for estimating transition hazards when data are available (e.g., the R package `simIDM`), in simulation studies for trial planning, it is common to specify these hazard parameters based on limited information. A more intuitive way to define PFS and OS is through their medians and correlation coefficient. In this section, we describe a reparameterization strategy that maps the medians and correlation to the transition hazards.

Let $m_1$ and $m_2$ denote the medians of PFS and OS, respectively. Then we have


$$h_{01} + h_{02} = \frac{\log(2)}{m_1} \tag{1}$$

and

$$
\frac{1}{2} = \exp\{-(h_{01} + h_{02})m_2\} + \frac{h_{01}}{h_{12}-h_{01}-h_{02}}\exp\{-(h_{01} + h_{02})m_2\} - \exp\{-h_{12}m_2\}
$$
Through algebraic manipulation, we obtain

$$
h_{01} = \frac{(\frac{1}{2} - \exp\{-\log(2) \frac{m_2}{m_1}\}) (h_{12} - \log(2) \frac{1}{m_1})}{
      (\exp\{-\log(2) \frac{m_2}{m_1}\} - \exp\{-h_{12} m_2\})}
      \tag{2}
$$

This implies that, for given medians $m_1$ and $m_2$, we can perform a grid search over values of $h_{12}$. For each candidate value, we compute $h_{01}$ and $h_{02}$ using equations (1) and (2), respectively. Then, we simulate a large dataset using `TrialSimulator::CorrelatedPfsAndOs3()` and calculate the Pearson correlation between PFS and OS. We select the value of $h_{12}$ (and the corresponding $h_{01}, h_{02}$) that yields the desired correlation.

This reparameterization of the three-state illness-death model enables us to specify correlated PFS and OS based on their medians and correlation, which is more interpretable and practical in clinical trial design. The `TrialSimulator` package provides a dedicated function, `solveThreeStateModel`, to facilitate this process.


## Example

Suppose we aim to simulate PFS with a median of 5 and OS with a median of 12, targeting a correlation of 0.6 between them.

```{r laiela}
pars <- solveThreeStateModel(median_pfs = 5, median_os = 12, 
                             corr = seq(.55, .65, by = .05), 
                             h12 = seq(.05, .15, length.out = 50))
plot(pars)
```

The result suggests that using $h_{01} = 0.11$, $h_{02} = 0.03$, and $h_{12} = 0.10$ may achieve the desired specifications. To verify this, we simulate PFS and OS data using

```{r eiaoljf, results='asis'}
pfs_and_os <- endpoint(name = c('pfs', 'os'), 
                       type = c('tte', 'tte'), 
                       generator = CorrelatedPfsAndOs3, 
                       h01 = .11, h02 = .03, h12 = .10, 
                       pfs_name = 'pfs', os_name = 'os')
pfs_and_os
```

If you are not familiar with the `TrialSimulator` framework, you can generate PFS and OS directly by calling `CorrelatedPfsAndOs3()`:


```{r llea}
dat <- CorrelatedPfsAndOs3(n = 1e6, h01 = .11, h02 = .03, h12 = .10)
head(dat, 2)

## should be close to 0.6
with(dat, cor(pfs, os))

## should be close to 5.0
with(dat, median(pfs))

## should be close to 12.0
with(dat, median(os))
with(dat, all(pfs <= os))
```

## Further Discussion

In this vignette, we demonstrate how to derive the transition hazards from the medians of PFS and OS, along with their Pearson correlation coefficient. It is important to note that the correlation is computed based on data simulated from the specified hazard, and therefore can be substituted by any other measure that is relevant to the simulation objective.

For instance, one may simulate trial data for two treatment arms, each defined by their own PFS and OS medians and $h_{12}$. Instead of using the correlation between simulated PFS and OS times, one could compute the correlation between the $z$-statistics from the respective treatment comparisons of PFS and OS. This alternative approach may be more appropriate in scenarios where the joint behavior of test statistics, rather than raw survival times, is of primary interest—such as in the context of multiple testing or endpoint selection.

---
title: "Define Non-Time-to-Event Endpoints"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Define Non-Time-to-Event Endpoints}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache = TRUE,
  cache.path = 'cache/defineNonTimeToEventEndpoints/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, message = FALSE, echo=FALSE}
library(dplyr)
library(TrialSimulator)
set.seed(12345)
```

`TrialSimulator` provides a flexible framework for defining and simulating a variety of clinical trial endpoints by specifying the `type` parameter in `endpoint`. This vignette covers non-time-to-event (non-TTE) endpoints, demonstrating how they can be defined, integrated into trial arms, and analyzed at pre-specified milestones. For time-to-event endpoints, please refer to the vignette [Define Time-to-Event Endpoints in Clinical Trials](defineTimeToEventEndpoints.html). For longitudinal endpoints, please refer to the vignette [Define Longitudinal Endpoints in Clinical Trials](defineLongitudinalEndpoints.html)

This vignette demonstrates how to use the following key functions to define non-TTE endpoints. For the sake of completeness, we also demonstrates how to define arms and trial with the created endpoints

- `endpoint`: Creates one or more endpoints. It can also be used to define covariates, bio-markers, sub-group indicators, etc. 
- `arm`: Creates one or more arms
- `add_endpoints`: Add one or more endpoints to an arm
- `milestone`: Defines one or more milestones when data snapshots are needed for analysis


## Define endpoints with random number generators

Similar to time-to-event endpoints, non-TTE endpoints can be defined using any univariate random number generator that takes `n` (number of observations) as its first argument. The `stats` package provides a set of random number generators that can be assigned to `generator` in `endpoints`, where additional arguments required by `generator` can be passed through `...`. When creating non-TTE endpoints, the argument `type` should be set to `"non-tte"`, and the argument `readout` should be specified as a named numeric vector, indicating the time required for the endpoint to be available for analysis after patient enrollment.

In the example below, we define two types of endpoints:

- Continuous endpoint: Tumor size change from baseline (`cfb`), available after 6 months, assuming a normal distribution (`generator = rnorm`) with custom `mean` and `sd`. 

- Binary endpoint: Objective response rate (`orr`), available after 2 months, assuming a binomial distribution (`generator = rbinom`) with `size = 1` and custom `prob`. 

```{r lljalk}
## endpoints in placebo arm
tumor_cfb_pbo <- endpoint(name = 'cfb', type = 'non-tte', 
                          readout = c(cfb = 6),
                          generator = rnorm, mean = .8, sd = 3.2)
orr_pbo <- endpoint(name = 'orr', type = 'non-tte', 
                    readout = c(orr = 2),
                    generator = rbinom, size = 1, prob = .1)

## define the placebo arm
pbo <- arm(name = 'placebo')
pbo$add_endpoints(tumor_cfb_pbo, orr_pbo)

## endpoints in treatment arm
tumor_cfb_trt <- endpoint(name = 'cfb', type = 'non-tte', 
                          readout = c(cfb = 6), 
                          generator = rnorm, mean = -2.3, sd = 1.5)
orr_trt <- endpoint(name = 'orr', type = 'non-tte', 
                    readout = c(orr = 2), 
                    generator = rbinom, size = 1, prob = .25)

## define the treatment arm
trt <- arm(name = 'treatment')
trt$add_endpoints(tumor_cfb_trt, orr_trt)
```

### Baseline endpoints

Some variables are observed at randomization rather than after a delay, for example a baseline covariate, biomarker, or subgroup indicator. Such a variable is a non-TTE endpoint with a readout of `0`. Instead of writing `type = 'non-tte', readout = c(biomarker = 0)`, you can set `type = 'baseline'`, in which case the readout is `0` automatically and must be omitted from `readout`:

```{r baseline-ep}
## a baseline biomarker available at randomization
biomarker <- endpoint(name = 'biomarker', type = 'baseline',
                      generator = rbinom, size = 1, prob = .3)
```

`type = 'baseline'` is a convenience alias for a readout-`0` non-TTE endpoint; the two definitions below are equivalent. Specifying a `readout` for a `'baseline'` endpoint (even `0`) raises an error.

```{r baseline-equiv, eval = FALSE}
## these are equivalent
endpoint(name = 'biomarker', type = 'baseline',
         generator = rbinom, size = 1, prob = .3)
endpoint(name = 'biomarker', type = 'non-tte', readout = c(biomarker = 0),
         generator = rbinom, size = 1, prob = .3)
```

With the treatment arms defined, we can proceed to create a trial. Patients are recruited at a piecewise constant rate, with an accrual pattern as follows:

- First 6 months: 10 patients per month.
- After 6 months: 20 patients per month until 420 patients are randomized 1:1 into the two arms.

We also specify a dropout process with a Weibull distribution. The dropout rates are set as follows:

- 15% dropout at 12 months
- 30% dropout at 18 months

These constraints are resolved using the Weibull dropout function:

$$
\begin{split}
0.15 & = & 1 - e^{-(12/\lambda)^k} \\
0.30 & = & 1 - e^{-(18/\lambda)^k}
\end{split}
$$

```{r oheahs}
dropout_pars <- weibullDropout(c(12, 18), c(.15, .30))
dropout_pars
```

Using the computed scale parameter $\lambda=$ `r round(dropout_pars['scale'], 3)` and shape parameter $k=$ `r round(dropout_pars['shape'], 3)`, we specify the trial setup: 

```{r lihoehf}
accrual_rate <- data.frame(end_time = c(6, Inf), 
                           piecewise_rate = c(10, 20))

trial <- trial(
  name = 'Trial-31415', description = 'Example Clinical Trial', 
  n_patients = 420, duration = 30, 
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate, 
  dropout = rweibull, scale = 30.636, shape = 1.939
)

## add arms to the trial
trial$add_arms(sample_ratio = c(1, 1), trt, pbo)
trial
```

Here `accrual_rate` is an argument of `TrialSimulator::StaggeredRecruiter` controlling how patients are recruited into the trial. Similarly, `scale` and `shape` are arguments of `rweibull`. All these arguments are passed through `...` of `trial()`. 

`TrialSimulator` allows defining trial milestones at specific time points when data is locked for analysis. Here, we define three key milestones:

1. Interim Analysis: Triggered when `orr` has been observed for 60 patients.
2. Random Checkpoint: For illustration purpose only. Triggered when the trial has reached at least 10 months, and at least one of the following conditions is met:
    - `cfb` has been observed for at least 100 patients, 
    - `orr` has been observed for at least 180 patients. 
3. Final Analysis: Occurs when the trial reaches 30 months.

```{r llieh}
interim <- milestone(name = 'interim', 
                     when = eventNumber(endpoint = 'orr', n = 60), 
                     action = doNothing)

random <- milestone(name = 'random', 
                    when = 
                      calendarTime(time = 10) & 
                      (eventNumber(endpoint = 'cfb', n = 100) | 
                         eventNumber(endpoint = 'orr', n = 180)
                       ), 
                    action = doNothing)

final <- milestone(name = 'final', 
                   when = calendarTime(time = 30), 
                   action = doNothing)
```

Here `action = doNothing` in `milestone` means we don't expect any action at the time of triggered milestones. In practice, instead of `doNothing`, custom action function can be adopted to add or remove arms (e.g., dose selection), adjust sample ratio per arm, or carry out statistical analysis based on locked data. These advanced setups are covered in other vignettes.

Next, we register the milestones with a listener and create a controller to monitor and execute the trial.

```{r ihhf}
## register milestones to the listener
listener <- listener()
listener$add_milestones(interim, random, final)

## run the trial
controller <- controller(trial, listener)
controller$run()
```

We can inspect the dataset locked at different milestone by calling member function `get_locked_data` with milestone names. Ideally, this should be done within custom action function, where decision is made based on data locked at the time of a milestone. 

```{r ihegea}
interim_data <- trial$get_locked_data(milestone_name = 'interim')
random_data <- trial$get_locked_data(milestone_name = 'random')
final_data <- trial$get_locked_data(milestone_name = 'final')
head(interim_data)
```

Since `cfb` has a 6-month readout time, at interim analysis, some patients’ `cfb` values are still unavailable, appearing as `NA` in `interim_data.` However, these values become available in `random_data` collected at a later time point. This demonstrates how `TrialSimulator` properly and automatically handles endpoint availability at different milestones

```{r eqfee}
not_ready_at_interim <- 
  interim_data %>% 
  dplyr::filter(is.na(cfb) & 
                  is.na(orr) & 
                  enroll_time + 6 < dropout_time) %>% 
  head() %>% 
  print()

random_data %>% 
  dplyr::filter(patient_id %in% not_ready_at_interim$patient_id) %>% 
  print()
```

In this example, we simulate tumor size change from baseline (`cfb`). However, in many trials, it is more appropriate to simulate tumor size at both baseline and follow-up separately to allow for more complex modeling, such as longitudinal or repeated measures analysis. This will be covered in another vignette.

With this flexible setup, `TrialSimulator` enables efficient endpoint definition, adaptive trial execution, and data monitoring—allowing users to design and simulate clinical trials tailored to specific research needs.

---
title: "Simulate Correlated Progression-Free Survival and Overall Survival Using a Gumbel Copula"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Simulate Correlated Progression-Free Survival and Overall Survival Using a Gumbel Copula}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/simulatePfsAndOsGumbel/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(dplyr)
library(knitr)
library(ggplot2)
library(TrialSimulator)

```

Progression-free survival (PFS) and overall survival (OS) are common time-to-event endpoints in oncology trials. Because progression or death defines a PFS event, simulated trial data should always satisfy $\mbox{PFS} \leq \mbox{OS}$. At the same time, the two endpoints are often highly correlated, so independent marginal simulation is usually not appropriate.

This vignette describes the copula-based generator implemented in `TrialSimulator::CorrelatedPfsAndOs2()`. The function is intended for simulation settings where the user wants to specify three interpretable quantities:

- the marginal median PFS,
- the marginal median OS,
- Kendall's tau between the observed, uncensored PFS and OS times.

The method uses a Gumbel survival copula for a latent time to progression (TTP) and OS, and then defines

$$
\mbox{PFS} = \min(\mbox{TTP}, \mbox{OS}).
$$

This construction guarantees $\mbox{PFS} \leq \mbox{OS}$ for every simulated patient. It also keeps both marginal PFS and OS exponential, so the inputs can be specified directly through their medians.

This is a useful distinction from the illness-death model in `TrialSimulator::CorrelatedPfsAndOs3()`. The illness-death model gives a clinically interpretable transition structure, but OS generated from that model can have a time-varying hazard ratio between treatment arms. Therefore, if OS will be analyzed using a proportional hazards Cox model and the simulation should be consistent with that analysis model, the copula-based method described here is often more appropriate.

## Model

Let $X$ denote latent TTP and let $Y$ denote OS. The model assumes exponential marginal survival functions

$$
\Pr(X > x) = \exp(-\lambda_X x), \qquad
\Pr(Y > y) = \exp(-\lambda_Y y),
$$

and a Gumbel--Hougaard survival copula

$$
\Pr(X > x, Y > y)
=
\exp\left[
-\left\{(\lambda_X x)^\theta + (\lambda_Y y)^\theta\right\}^{1/\theta}
\right],
\qquad \theta \geq 1.
$$

The observed PFS time is

$$
P = \min(X, Y).
$$

Under this model, OS is exponential with rate $\lambda_Y$. PFS is also exponential, with survival function

$$
\Pr(P > t)
=
\Pr(X > t, Y > t)
=
\exp\left[
-\left\{(\lambda_X^\theta + \lambda_Y^\theta)t^\theta\right\}^{1/\theta}
\right]
=
\exp(-\lambda_P t),
$$

where

$$
\lambda_P = \left(\lambda_X^\theta + \lambda_Y^\theta\right)^{1/\theta}.
$$

Therefore, if the target medians are $m_P$ for PFS and $m_Y$ for OS,

$$
\lambda_P = \frac{\log(2)}{m_P},
\qquad
\lambda_Y = \frac{\log(2)}{m_Y}.
$$

For a candidate value of $\theta$, the latent TTP rate is then

$$
\lambda_X =
\left(\lambda_P^\theta - \lambda_Y^\theta\right)^{1/\theta}.
$$

This requires $m_P < m_Y$, as expected for PFS and OS.

## Choosing the Copula Parameter

For the Gumbel copula, Kendall's tau between latent TTP and OS is

$$
\tau(X,Y) = 1 - \frac{1}{\theta}.
$$

However, `CorrelatedPfsAndOs2()` asks for Kendall's tau between the observed, uncensored endpoints $P=\min(X,Y)$ and $Y$, not between latent TTP and OS. These are not the same quantity. The formula below is derived in the appendix. Under the model above,

$$
\tau(P,Y)
=
1 - \frac{1}{\theta}
\left[
1 - \left(\frac{m_P}{m_Y}\right)^\theta
\right].
$$

Equivalently, writing $\tau_X = \tau(X,Y)$ and $\theta = 1/(1-\tau_X)$,

$$
\tau(P,Y)
=
1 -
(1-\tau_X)
\left[
1 -
\left(\frac{m_P}{m_Y}\right)^{1/(1-\tau_X)}
\right].
$$

`CorrelatedPfsAndOs2()` solves this equation numerically for the latent Kendall's tau $\tau_X$, converts it to $\theta$, and then simulates from the Gumbel copula.

For any finite $\theta$, $\tau(P,Y)$ is larger than $\tau(X,Y)$:

$$
\tau(P,Y)-\tau(X,Y)
=
\frac{1}{\theta}
\left(\frac{m_P}{m_Y}\right)^\theta
> 0.
$$

Intuitively, even if latent TTP and OS have a weaker association, observed PFS contains deaths by construction, which increases the association between observed PFS and OS.

The requested value of $\tau(P,Y)$ has a lower bound. When $\theta=1$, TTP and OS are independent, but PFS still contains OS deaths through the definition $P=\min(X,Y)$. In this limiting case,

$$
\tau(P,Y) = \frac{m_P}{m_Y}.
$$

Thus, for fixed medians $m_P$ and $m_Y$, `CorrelatedPfsAndOs2()` can only target

$$
\frac{m_P}{m_Y} \leq \tau(P,Y) < 1.
$$

If the requested Kendall's tau is too small for the two medians, the function stops with an error.

## Example

Suppose we want to simulate PFS with median 5 months and OS with median 11 months, targeting Kendall's tau of 0.6 between observed, uncensored PFS and OS times.

```{r example-endpoint, results = 'asis'}
pfs_and_os <- endpoint(name = c('pfs', 'os'),
                       type = c('tte', 'tte'),
                       generator = CorrelatedPfsAndOs2,
                       median_pfs = 5,
                       median_os = 11,
                       kendall = 0.6,
                       pfs_name = 'pfs',
                       os_name = 'os')

pfs_and_os
```

For verification only, we can call the generator directly. This is not the recommended way to use `TrialSimulator` for simulation studies; in practice, the generator should usually be supplied to `endpoint()` and used inside the trial simulation workflow. Direct calls are helpful here because they let us check the marginal medians, the observed Kendall's tau, and the ordering $\mbox{PFS} \leq \mbox{OS}$ before adding enrollment, censoring, treatment arms, and analyses.

```{r direct-generator}
set.seed(123)
dat <- CorrelatedPfsAndOs2(n = 10000,
                           median_pfs = 5,
                           median_os = 11,
                           kendall = 0.6)

head(dat, 2)
```

The simulation should approximately recover the requested medians and Kendall's tau between observed, uncensored PFS and OS times.

```{r validation}
with(dat, median(pfs))
with(dat, median(os))
with(dat, cor(pfs, os, method = 'kendall'))
with(dat, all(pfs <= os))
```

Because this generator returns event indicators equal to 1 for both endpoints, censoring and staggered enrollment should be handled by the broader trial simulation workflow rather than by this endpoint generator.

## Practical Guidance

This generator is most useful when the simulation objective is to specify marginal PFS and OS medians and a rank-based association between the two observed endpoints. Kendall's tau is often more stable and interpretable than Pearson correlation for skewed time-to-event variables, and it is directly connected to the Gumbel copula parameter.

There are still modeling assumptions to keep in mind:

- PFS and OS are both marginally exponential.
- Dependence is induced through a Gumbel survival copula on latent TTP and OS.
- The Gumbel copula only allows non-negative dependence.
- The requested Kendall's tau is for observed PFS and OS event times before censoring is applied.
- The attainable Kendall's tau between observed PFS and OS is constrained by the PFS/OS median ratio.

For simulations that require transition-specific hazards or a mechanistic disease process, the illness-death model implemented in `TrialSimulator::CorrelatedPfsAndOs3()` may be more appropriate. For simulations where the main requirement is a direct medians-plus-Kendall's-tau specification, especially when OS will be analyzed under a proportional hazards Cox model, `CorrelatedPfsAndOs2()` provides a compact alternative.

## Appendix: Kendall's Tau for Observed PFS and OS

This appendix gives the derivation behind the formula used in `CorrelatedPfsAndOs2()`.

Let $X$ and $Y$ be nonnegative random variables with exponential margins

$$
X \sim \operatorname{Exp}(\lambda_X),
\qquad
Y \sim \operatorname{Exp}(\lambda_Y),
$$

and joint survival function

$$
S_{X,Y}(x,y)
=
\Pr(X>x, Y>y)
=
\exp\left[
-\left\{(\lambda_X x)^\theta + (\lambda_Y y)^\theta\right\}^{1/\theta}
\right],
\qquad \theta \ge 1.
$$

Define $P=\min(X,Y)$. We want Kendall's tau between $P$ and $Y$.

Standardize the margins by setting

$$
A=\lambda_X X, \qquad B=\lambda_Y Y.
$$

Then the joint survival function of $(A,B)$ is

$$
\Pr(A>a, B>b)
=
\exp\left\{-(a^\theta+b^\theta)^{1/\theta}\right\}.
$$

Use the polar-type transformation

$$
R=(A^\theta+B^\theta)^{1/\theta},
\qquad
W=\frac{A^\theta}{A^\theta+B^\theta}.
$$

A Jacobian calculation gives the joint density

$$
f_{R,W}(r,w)=\frac{1}{\theta} e^{-r}(r+\theta-1),
\qquad r>0,\; 0<w<1.
$$

Thus, $R$ and $W$ are independent and $W \sim \operatorname{Unif}(0,1)$. The inverse transformation is

$$
A = R W^{1/\theta},
\qquad
B = R(1-W)^{1/\theta}.
$$

Let

$$
a=\lambda_X^\theta,
\qquad
b=\lambda_Y^\theta,
\qquad
c=\frac{a}{a+b},
\qquad
d=\frac{b}{a+b}.
$$

The event $X \le Y$ is equivalent to $W \le c$:

$$
X \le Y
\iff
\frac{A}{\lambda_X} \le \frac{B}{\lambda_Y}
\iff
W \le c,
$$

so

$$
P=
\begin{cases}
X, & W \le c,\\
Y, & W > c.
\end{cases}
$$

Using the survival-copula representation of Kendall's tau,

$$
\tau(P,Y)=4E\{S_{P,Y}(P,Y)\}-1.
$$

When $W \le c$, $P=X$ and

$$
S_{P,Y}(P,Y)=S_{X,Y}(X,Y)=e^{-R}.
$$

When $W>c$, $P=Y$. Since

$$
S_{P,Y}(y,y)=\Pr(P>y,Y>y)=\Pr(X>y,Y>y),
$$

we have, using $Y=B/\lambda_Y=R(1-W)^{1/\theta}/\lambda_Y$,

$$
S_{P,Y}(Y,Y)
=
\exp\left[
-R\left(\frac{1-W}{d}\right)^{1/\theta}
\right].
$$

Therefore,

$$
E\{S_{P,Y}(P,Y)\}
=
E\{e^{-R}\mathbf 1(W\le c)\}
+
E\left[
e^{-R((1-W)/d)^{1/\theta}}
\mathbf 1(W>c)
\right].
$$

By independence of $R$ and $W$,

$$
E\{S_{P,Y}(P,Y)\}
=
cL_R(1)
+
\int_c^1
L_R\left\{\left(\frac{1-w}{d}\right)^{1/\theta}\right\}
dw,
$$

where $L_R(s)=E(e^{-sR})$. From the density of $R$,

$$
L_R(s)
=
\frac{1}{\theta}
\int_0^\infty e^{-(1+s)r}(r+\theta-1)\,dr
=
\frac{\theta+(\theta-1)s}{\theta(1+s)^2}.
$$

In particular,

$$
L_R(1)=\frac{2\theta-1}{4\theta}.
$$

For the integral term, use the substitution

$$
t=\left(\frac{1-w}{d}\right)^{1/\theta},
\qquad
dw=-d\theta t^{\theta-1}dt.
$$

Then

$$
\int_c^1
L_R\left\{\left(\frac{1-w}{d}\right)^{1/\theta}\right\}
dw
=
d\theta\int_0^1 t^{\theta-1}L_R(t)\,dt.
$$

Substituting $L_R(t)$ gives

$$
d\int_0^1
\frac{\theta t^{\theta-1}+(\theta-1)t^\theta}{(1+t)^2}
dt.
$$

Because

$$
\frac{d}{dt}\left(\frac{t^\theta}{1+t}\right)
=
\frac{\theta t^{\theta-1}+(\theta-1)t^\theta}{(1+t)^2},
$$

the integral is

$$
d\left[\frac{t^\theta}{1+t}\right]_0^1
=
\frac{d}{2}.
$$

Thus,

$$
E\{S_{P,Y}(P,Y)\}
=
c\frac{2\theta-1}{4\theta}+\frac{d}{2}
=
\frac{2\theta-1+d}{4\theta},
$$

using $c+d=1$. Finally,

$$
\tau(P,Y)
=
4E\{S_{P,Y}(P,Y)\}-1
=
\frac{\theta-1+d}{\theta}
=
1-\frac{1}{\theta}
\left(
\frac{\lambda_X^\theta}{\lambda_X^\theta+\lambda_Y^\theta}
\right).
$$

Since $P$ has rate

$$
\lambda_P =
\left(\lambda_X^\theta+\lambda_Y^\theta\right)^{1/\theta},
$$

this can also be written as

$$
\tau(P,Y)
=
1-\frac{1}{\theta}
\left(
\frac{\lambda_X^\theta}{\lambda_P^\theta}
\right)
=
1-\frac{1}{\theta}
\left[
1-\left(\frac{\lambda_Y}{\lambda_P}\right)^\theta
\right].
$$

Because $\lambda_Y/\lambda_P = m_P/m_Y$, the formula used by `CorrelatedPfsAndOs2()` is

$$
\tau(P,Y)
=
1-\frac{1}{\theta}
\left[
1-\left(\frac{m_P}{m_Y}\right)^\theta
\right].
$$

---
title: "Wrapper Functions of Common Statistical Methods in TrialSimulator"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Wrapper Functions of Common Statistical Methods in TrialSimulator}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/wrapper/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(dplyr)
library(TrialSimulator)
```

The `TrialSimulator` package provides a unified set of wrapper functions that encapsulate statistical methods commonly used in clinical trial simulations. These functions facilitate model fitting, treatment comparisons, and covariate adjustments within a standardized interface.

When multiple active treatment arms are present, each wrapper function automatically performs pairwise comparisons between each active arm and the designated reference (e.g., placebo, control, or standard-of-care). All wrapper functions share a consistent syntax and output structure. Most of them support model specification via an R formula interface, and covariate adjustment is available where appropriate.

Pairwise average treatment effects (ATEs) are estimated using the `emmeans` package under the hood. All tests are one-sided, and the ellipsis (`...`) argument can be used to define data subsets, enabling flexible analyses such as those needed in enrichment designs.

Below is a summary of the available wrapper functions included in this vignette, along with their corresponding statistical methods, output metrics, and support for covariate adjustment.

+------------------------+-------------------------+------------------------+------------------------+
| Function               | Method                  | Statistics in Outputs  | Covariate adjustment   |
+:=======================+:========================+:=======================+:=======================+
| `fitLinear`            | Linear model            | ATE                    | Yes                    |
+------------------------+-------------------------+------------------------+------------------------+
| `fitLogistic`          | Logistic model          | regression coefficient | Yes                    |
|                        |                         |                        |                        |
|                        |                         | log odds ratio         |                        |
|                        |                         |                        |                        |
|                        |                         | odds ratio             |                        |
|                        |                         |                        |                        |
|                        |                         | risk ratio             |                        |
|                        |                         |                        |                        |
|                        |                         | risk difference        |                        |
+------------------------+-------------------------+------------------------+------------------------+
| `fitCoxph`             | Cox PH model            | log hazard ratio       | Yes                    |
|                        |                         |                        |                        |
|                        |                         | hazard ratio           |                        |
+------------------------+-------------------------+------------------------+------------------------+
| `fitLogrank`           | logrank test            |                        | No but supports strata |
+------------------------+-------------------------+------------------------+------------------------+
| `fitFarringtonManning` | Farrington-Manning test |                        | No                     |
+------------------------+-------------------------+------------------------+------------------------+

## Example

To demonstrate the usage of the wrapper functions, we simulate a hypothetical three-arm trial with one control (`pbo`) and two active doses (`low` and `high`). The trial includes a continuous covariate `x`, three endpoint types (time-to-event, continuous, and binary), and a binary biomarker used to define subgroups.

The placebo arm is constructed as follows:

```{r ioeajfls}
## time-to-event endpoint
pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = .07)
## continuous endpoint
cep <- endpoint(name = 'cep', type = 'non-tte', 
                readout = c(cep = 0), generator = rnorm)
## binary endpoint
bep <- endpoint(name = 'bep', type = 'non-tte', 
                readout = c(bep = 0), generator = rbinom, size = 1, prob = .1)

## biomarker
bm <- endpoint(name = 'biomarker', type = 'non-tte', 
               readout = c(biomarker = 0), generator = rbinom, 
               size = 1, prob = .7)

## covariate
covar <- endpoint(name = 'x', type = 'non-tte', 
                  readout = c(x = 0), generator = rnorm)

pbo <- arm(name = 'pbo')
pbo$add_endpoints(pfs, cep, bep, bm, covar)
```

For brevity, the code for the low and high dose arms and the trial definition are hidden in this vignette. Refer to the source of this vignette for full code. A single milestone for final analysis is defined, with an empty action `doNothing.` This is because we will explicitly request locked data outside of the action function when demonstrating the wrapper functions.


```{r eualgha, echo=FALSE}
## time-to-event endpoint
pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = .06)
## continuous endpoint
cep <- endpoint(name = 'cep', type = 'non-tte', 
                readout = c(cep = 0), generator = rnorm, mean = 1.2)
## binary endpoint
bep <- endpoint(name = 'bep', type = 'non-tte', 
                readout = c(bep = 0), generator = rbinom, size = 1, prob = .2)

## biomarker
bm <- endpoint(name = 'biomarker', type = 'non-tte', 
               readout = c(biomarker = 0), generator = rbinom, 
               size = 1, prob = .7)

## covariate
covar <- endpoint(name = 'x', type = 'non-tte', 
                  readout = c(x = 0), generator = rnorm)

low <- arm(name = 'low')
low$add_endpoints(pfs, cep, bep, bm, covar)

## time-to-event endpoint
pfs <- endpoint(name = 'pfs', type = 'tte', generator = rexp, rate = .04)
## continuous endpoint
cep <- endpoint(name = 'cep', type = 'non-tte', 
                readout = c(cep = 0), generator = rnorm, mean = 1.3)
## binary endpoint
bep <- endpoint(name = 'bep', type = 'non-tte', 
                readout = c(bep = 0), generator = rbinom, size = 1, prob = .35)

## biomarker
bm <- endpoint(name = 'biomarker', type = 'non-tte', 
               readout = c(biomarker = 0), generator = rbinom, 
               size = 1, prob = .7)

## covariate
covar <- endpoint(name = 'x', type = 'non-tte', 
                  readout = c(x = 0), generator = rnorm)

high <- arm(name = 'high')
high$add_endpoints(pfs, cep, bep, bm, covar)

accrual_rate <- data.frame(end_time = c(10, Inf),
                           piecewise_rate = c(30, 50))
trial <- trial(
  name = 'Trial-3415', n_patients = 300,
  seed = 1727811904, duration = 1000,
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
  dropout = rexp, rate = -log(1 - 0.1)/18, ## 10% by month 18
  silent = TRUE
)

trial$add_arms(sample_ratio = c(1, 1, 1), pbo, low, high)

final <- milestone(name = 'final', action = doNothing, when = calendarTime(1000))

listener <- listener()
listener$add_milestones(final)

controller <- controller(trial, listener)
```

Now we execute the trial. After simulation, locked data can be retrieved using the `get_locked_data()` method with the milestone name `"final"`.

```{r ieoajf}
controller$run(n = 1, plot_event = FALSE, silent = TRUE)
locked_data <- trial$get_locked_data('final')
head(locked_data)

table(locked_data$arm)
```

## Analyze Time-to-Event Endpoint

We begin by analyzing the time-to-event endpoint `pfs` using both a Cox proportional hazards model and a log-rank test. When performing analysis on a subset defined via the `...` argument, the syntax must be compatible with that of `dplyr::filter()`.

```{r ghkaljf}
## adjust for covariate x
fitCoxph(Surv(pfs, pfs_event) ~ arm + x, placebo = 'pbo', 
         data = locked_data, alternative = 'less', 
         scale = 'hazard ratio')

fitLogrank(Surv(pfs, pfs_event) ~ arm, placebo = 'pbo', 
           data = locked_data, alternative = 'less')

## more details
fitLogrank(Surv(pfs, pfs_event) ~ arm, placebo = 'pbo', 
           data = locked_data, alternative = 'less', tidy = FALSE)

## with strata
fitLogrank(Surv(pfs, pfs_event) ~ arm + strata(biomarker), placebo = 'pbo', 
           data = locked_data, alternative = 'less')

## analyze a subset
fitCoxph(Surv(pfs, pfs_event) ~ arm + strata(biomarker), placebo = 'pbo', 
         data = locked_data, alternative = 'less', 
         scale = 'log hazard ratio', 
         x > -2 & x < 3) ## define a subset
```

## Analyze Continuous Endpoint

We analyze the continuous endpoint `cep` using linear models, with and without covariate adjustment.

```{r saljfh}
## ATE accounting for covariate x
fitLinear(cep ~ arm * x, placebo = 'pbo', 
          data = locked_data, alternative = 'greater')

## marginal model
fitLinear(cep ~ arm, placebo = 'pbo', 
          data = locked_data, alternative = 'greater')

## analyze a sub-group
fitLinear(cep ~ arm, placebo = 'pbo', 
          data = locked_data, alternative = 'greater', 
          biomarker == 1) ## define the subgroup
```

## Analyze Binary Endpoint

We analyze the binary endpoint `bep` using logistic regression. Multiple estimands (e.g., odds ratio, risk ratio, risk difference) can be computed by specifying the `scale` argument.

```{r pqeir}
## compute regression coefficient of arm
fitLogistic(bep ~ arm * x + biomarker, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'coefficient')

## compute odds ratio (ATE)
fitLogistic(bep ~ arm + x*biomarker, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'odds ratio')

## compute risk ratio (ATE)
fitLogistic(bep ~ arm + x + biomarker, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'risk ratio')
```

The risk difference can also be estimated using logistic regression or the Farrington-Manning test. Note that the latter does not support covariate adjustment.

```{r uyta}
## compute risk difference (ATE)
fitLogistic(bep ~ arm + x * biomarker, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'risk difference')

## compute risk difference without covariate
fitLogistic(bep ~ arm, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'risk difference')

## analyze a sub-group
fitLogistic(bep ~ arm, placebo = 'pbo', 
            data = locked_data, alternative = 'greater', 
            scale = 'risk difference', 
            x < 2 & biomarker != 1) ## define a subgroup

## analyze the same sub-group using the FM test,
## same estimate but different p-values
fitFarringtonManning(endpoint = 'bep', placebo = 'pbo', 
                     data = locked_data, alternative = 'greater', 
                     x < 2 & biomarker != 1)
```

---
title: "An Example of Simulating a Trial with Adaptive Design"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{An Example of Simulating a Trial with Adaptive Design}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/adaptiveDesign/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(dplyr)
library(R6)
library(kableExtra)
library(rlang)
library(survival)
library(survminer)
library(ggplot2)
library(TrialSimulator)
```

In this vignette, we illustrate how to use `TrialSimulator` to simulate
a trial with seamless adaptive design of multiple endpoints, where dose
selection, futility analysis, interim analysis are triggered by
user-defined milestones. Family-wise error rate are controlled by closed
test with Dunnett's test, with $\alpha$ split for the endpoints. Please
refer to another vignette where graphical testing procedure is adopted
instead.

## Simulation Settings

-   Trial consists of two active arms of high or low dose, and a placebo
    arm.
-   Patients are randomized into the trial with ratio `1:1:1`.
-   30 patients are randomized per month for the first 10 months, and 50
    patients per month after then until 1000 patients.
-   Dropout rate is 8% at month 12, and 18% at month 18, which is
    modeled by a Weibull distribution. The scale and shape parameters of
    the Weibull distribution can be computed using helper function

```{r daoif, class.source='fold-show'}
weibullDropout(time = c(12, 18), dropout_rate = c(.08, .18))
```

-   Two time-to-event endpoints (`PFS` and `OS`) and a binary
    `surrogate` are under consideration, where a higher rate of
    `surrogate` is better.
-   `surrogate` has a readout time of 5 weeks.
-   `surrogate` is used for dose selection using a hypothetical rule.
    Let $z_h$ (or $z_l$) be the z statistics of `surrogate` between high
    (or low) dose and placebo.
    -   low dose arm is selected if $z_l > 1.28$;
    -   high dose arm is selected if $z_h > 1.28 > z_l$;
    -   keep both arms otherwise.
-   Rates of `surrogate` are
    -   0.05 in placebo
    -   0.12 in low dose arm
    -   0.13 in high dose arm
-   Medians of PFS are
    -   5 months in placebo
    -   6.7 months in low dose arm (hazard ratio 0.75)
    -   7.1 months in high dose arm (hazard ratio 0.70) Exponential
        distribution is assumed.
-   Medians of OS are
    -   14 months in placebo
    -   17.5 months in low dose arm (hazard ratio 0.80)
    -   18.2 months high dose arms (hazard ratio 0.77)
-   Dose selection is done when 300 patients are randomized with readout
    of `surrogate` are ready for analysis.
-   An interim is set to test `PFS` when 300 `PFS` events are observed.
    A non-binding futility analysis is also done with a boundary 0.5.
-   Final analysis is set for `PFS` and `OS` when all planned 1000
    patients are randomized and at least 300 `OS` events are observed.
    Meanwhile, the trial has reached at least 28 months or at least 520
    events are observed for `PFS`.
-   To control the family-wise error rate accounting for `PFS` and `OS`,
    as well as adaptation of dose-selection, we split $\alpha$ between
    `PFS` (0.5%) and `OS` (2%), and adopt closed test with Dunnett's
    test for intersection hypotheses.

## Define Three Arms

In this example, we call `endpoint()` twice for each arm to define `PFS` 
and `OS`. This implicitly assumes independence between the two time-to-event 
endpoints. `TrialSimulator` offers a built-in generator `CorrelatedPfsAndOs3()` 
to generate `PFS` and `OS` with correlation based on the illness-death model, 
which enjoys several nice properties. For selection of its arguments (i.e. 
transition hazards), refer to vignette 
[Simulate Correlated Progression-Free Survival and Overall Survival as Endpoints Under Illness-Death Model](simulatePfsAndOsIdm.html). Note that data generated under the illness-death
model may induce a time-varying OS hazard ratio between treatment arms,
violating the proportional hazards (PH) assumption in a Cox model. For a data
model that is more suitable for PH Cox analysis, refer to the built-in
generator `CorrelatedPfsAndOs2()` and vignette of the 
[Gumbel copula method](simulatePfsAndOsGumbel.html).

```{r ljgai, class.source="fold-show"}
#' define three arms
pbo <- arm(name = 'placebo')
low <- arm(name = 'low dose')
high <- arm(name = 'high dose')

#' define endpoints in placebo
pfs <- endpoint(name = 'pfs', type = 'tte',
                generator = rexp, rate = log(2) / 5)

os <- endpoint(name = 'os', type = 'tte',
               generator = rexp, rate = log(2) / 14)

five_weeks <- 5 / 52 * 12 ## convert it in months
surrogate <- endpoint(name = 'surrogate', type = 'non-tte',
                      readout = c(surrogate = five_weeks),
                      generator = rbinom, size = 1, prob = .05)
pbo$add_endpoints(pfs, os, surrogate)

#' define endpoints in low dose arm
pfs <- endpoint(name = 'pfs', type = 'tte',
                generator = rexp, rate = log(2) / 6.7)

os <- endpoint(name = 'os', type = 'tte',
               generator = rexp, rate = log(2) / 17.5)

surrogate <- endpoint(name = 'surrogate', type = 'non-tte',
                      readout = c(surrogate = five_weeks),
                      generator = rbinom, size = 1, prob = .12)
low$add_endpoints(pfs, os, surrogate)

#' define endpoints in high dose arm
pfs <- endpoint(name = 'pfs', type = 'tte',
                generator = rexp, rate = log(2) / 7.1)

os <- endpoint(name = 'os', type = 'tte',
               generator = rexp, rate = log(2) / 18.2)

surrogate <- endpoint(name = 'surrogate', type = 'non-tte',
                      readout = c(surrogate = five_weeks),
                      generator = rbinom, size = 1, prob = .13)
high$add_endpoints(pfs, os, surrogate)
```

## Define a Trial

With three arms, we can define a trial of class `Trial`. Recruitment
curve are specified through `enroller` with a built-in function
`StaggeredRecruiter` of piecewise constant rate. We set `duration` to be
an arbitrary large number (40) but controlling the end of trial through
pre-defined milestones later. Note that if `seed = NULL`, `TrialSimulator`
will pick a seed for the purpose of reproducibility.

```{r lagieg, class.source="fold-show"}
accrual_rate <- data.frame(end_time = c(10, Inf),
                           piecewise_rate = c(30, 50))
trial <- trial(
  name = 'Trial-3415', n_patients = 1000,
  seed = 1727811904, duration = 40,
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
  dropout = rweibull, shape = 2.139, scale = 38.343
)

trial$add_arms(sample_ratio = c(1, 1, 1), low, high, pbo)
```

## Define Trial Milestones and Action Functions

Next, we define action functions for trial milestones, i.e., dose selection,
interim and final analysis. Note that an action function should always
has argument `trial` returned from `trial()`. We
calculate z statistics of the Farrington-Manning test for binary
endpoint `surrogate` using helper function `fitFarringtonManning`, which
returns a data frame (see `?fitFarringtonManning`).

```{r ieaong, class.source='fold-show'}
action1 <- function(trial){
  
  locked_data <- trial$get_locked_data('dose selection')
  
  fit <- fitFarringtonManning(endpoint = 'surrogate', placebo = 'placebo',
                              data = locked_data, alternative = 'greater')
  
  # browser() ## if you want to see what does fit look like
  z_l <- fit$z[fit$arm == 'low dose']
  z_h <- fit$z[fit$arm == 'high dose']
  if(z_l > 1.28){
    trial$remove_arms('high dose')
    trial$save(value = 'low', name = 'kept_arm')
  }else if(z_h > 1.28){
    trial$remove_arms('low dose')
    trial$save(value = 'high', name = 'kept_arm')
  }else{
    trial$save(value = 'both', name = 'kept_arm')
  }

}
```

At the interim, we test `PFS` using the logrank test to carry out a
non-binding futility analysis in the following action function. Depending
on how many arms are carried forward in dose selection, `fit` may
consists of testing results of one or two dose arms. Note that a formal
test for `PFS` will be done at the end of the trial when we have all
p-values. If no futility analysis is planned, we can simply set
`action = doNothing` in `milestone`.

```{r aotel, class.source="fold-show"}
action2 <- function(trial){
  
  locked_data <- trial$get_locked_data('interim')

  fit <- fitLogrank(Surv(pfs, pfs_event) ~ arm, placebo = 'placebo', 
                    data = locked_data, alternative = 'less')

  ## futility analysis
  if(max(fit$z) < .5){
    trial$save(value = 'negative', name = 'futility')
    
    ## extend duration
    ## trial$set_duration(45)
  }else{
    trial$save(value = 'positive', name = 'futility')
  }

}
```

At final analysis, we conduct a closed test using Dunnett's test for
intersection hypotheses. The action function is as follows.

```{r alkdae, class.source="fold-show"}
action3 <- function(trial){
  
  locked_data <- trial$get_locked_data('final')

  ## test PFS
  dt_pfs <- trial$dunnettTest(Surv(pfs, pfs_event) ~ arm, placebo = 'placebo',
                              treatments = c('high dose', 'low dose'),
                              milestones = c('dose selection', 'interim', 'final'),
                              alternative = 'less',
                              planned_info = 'default')
  
  ct_pfs <- trial$closedTest(dt_pfs, treatments = c('high dose', 'low dose'),
                             milestones = c('interim', 'final'),
                             alpha = .005, alpha_spending = 'asOF')

  ## test OS
  dt_os <- trial$dunnettTest(Surv(os, os_event) ~ arm, placebo = 'placebo',
                             treatments = c('high dose', 'low dose'),
                             milestones = c('dose selection', 'final'),
                             alternative = 'less', 
                             planned_info = 'default')
  
  ct_os <- trial$closedTest(dt_os, treatments = c('high dose', 'low dose'),
                            milestones = c('final'),
                            alpha = .02, alpha_spending = 'asOF')
  
  ## we only save testing decision here
  ## You can save whatever you want for summarizing things later, e.g. reject time
  trial$save(value = ct_pfs$decision[ct_pfs$arm == 'high dose'], 
             name = 'pfs_high_dose_decision')
  
  trial$save(value = ct_pfs$decision[ct_pfs$arm == 'low dose'], 
             name = 'pfs_low_dose_decision')
  
  trial$save(value = ct_os$decision[ct_os$arm == 'high dose'], 
             name = 'os_high_dose_decision')
  
  trial$save(value = ct_os$decision[ct_os$arm == 'low dose'], 
             name = 'os_low_dose_decision')
}
```

Next, we register three trial milestones to a listener

```{r}
dose_selection <- milestone(name = 'dose selection', action = action1,   
                            when = eventNumber(endpoint = 'surrogate', n = 300)
                            )

interim <- milestone(name = 'interim', action = action2, 
                     when = eventNumber(endpoint = 'pfs', n = 300)
                     )

final <- milestone(name = 'final', action = action3, 
                   when = enrollment(n = 1000, arms = c('placebo', 'low dose', 'high dose')) & 
                     eventNumber(endpoint = 'os', n = 300) & (
                       calendarTime(time = 28) | 
                         eventNumber(endpoint = 'pfs', n = 520)
                       )
                   )

listener <- listener()
#' register milestones with listener
listener$add_milestones(
  dose_selection,
  interim,
  final
)
```

## Execute a Trial

We can run a trial as follows. By default, `TrialSimulator` generates a
plot showing cumulative number of patients or endpoint events over time
for each of the arms. We can set it to `FALSE` if a massive number of
simulation replicates is run to save time.

```{r eiaaf, dpi = 1200}
controller <- controller(trial, listener)
controller$run(plot_event = TRUE)
```

In custom action functions, we can use `Trial$save()` to save
intermediate results for summary purpose, which can be accessed anytime
and anywhere by calling `get_output()`

```{r alojfqoitl, class.source="fold-show"}
controller$get_output() %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")
```

Here we dive into the action function (`action3`) for the final
analysis. We can literally execute the function line by line with locked
data loaded. Note that the member functions `Trial$dunnettTest` and
`Trial$closedTest` can make use of locked data at multiple stages
(through the `milestones` argument) automatically, thus, unlike in
`action3`, an explicit call of `Trial$get_locked_data('final')` is
unnecessary.

```{r aief, class.source='fold-show'}
## test PFS
dt_pfs <- trial$dunnettTest(Surv(pfs, pfs_event) ~ arm, placebo = 'placebo',
                            treatments = c('high dose', 'low dose'),
                            milestones = c('dose selection', 'interim', 'final'),
                            alternative = 'less', 
                            planned_info = 'default')
ct_pfs <- trial$closedTest(dt_pfs, treatments = c('high dose', 'low dose'),
                           milestones = c('interim', 'final'),
                           alpha = .005, alpha_spending = 'asOF')

## test OS
dt_os <- trial$dunnettTest(Surv(os, os_event) ~ arm, placebo = 'placebo',
                           treatments = c('high dose', 'low dose'),
                           milestones = c('dose selection', 'final'),
                           alternative = 'less', 
                           planned_info = 'default')
ct_os <- trial$closedTest(dt_os, treatments = c('high dose', 'low dose'),
                          milestones = c('final'),
                          alpha = .02, alpha_spending = 'asOF')

print(ct_pfs)
print(ct_os)
```

The two null hypotheses of `PFS` and `OS` are both accepted for the high
dose because it is dropped at dose selection. Thus, `milestone_at_reject` is
`NA`, and the `reject_time` is infinite. In contrast, `PFS` and `OS` are
significant in low does. Note that even if `PFS` are tested at interim
and final, one cannot claim its significance until the final analysis
(`milestone_at_reject`). The `reject_time` (`r ct_os$reject_time[2]`) can be
saved to the output using the member function `Trial$save`.

`TrialSimulator` abstracts the data generation and management to allow
user focus on implementing the statistical analysis. It simulates a
trial on patient level, and provide flexibility in adaptive design.

## Execute Trial Simulation

We can run a massive number of replicates in simulation to study
operating characteristics of a trial design by specifying `n` in
`Controller$run()`. Parallelization is also supported by setting `n_workers`. 

```{r ioeinaf, eval=FALSE, cache=TRUE, message=FALSE, warning=FALSE, results='hide'}
## reset a controller if $run has been executed before
controller$reset()
controller$run(n = 1000, plot_event = FALSE, silent = TRUE)
output <- controller$get_output()
```

```{r eaiofj, echo=FALSE}
output <- TrialSimulator:::getAdaptiveDesignOutput()
```

```{r liefa}
output %>% 
  head(5) %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")
```

```{r iojalf, class.source = "fold-hide"}
output %>% 
  summarise(
    time_dose_selection = mean(`milestone_time_<dose selection>`), 
    time_interim = mean(`milestone_time_<interim>`), 
    time_final = mean(`milestone_time_<final>`), 
    n_dose_selection = mean(`n_events_<dose selection>_<patient_id>`), 
    n_interim = mean(`n_events_<interim>_<patient_id>`), 
    n_final = mean(`n_events_<final>_<patient_id>`), 
    low = mean(kept_arm == 'low') * 100, 
    high = mean(kept_arm == 'high') * 100, 
    both = mean(kept_arm == 'both') * 100
  ) %>% 
  kable(col.names = NULL, digits = 1, align = 'r', 
        caption = 'Number of Randomized Patients at Stages') %>% 
  add_header_above(c(rep(c('Dose Selection', 'Interim', 'Final'), 2), 
                     'Low Dose', 'High Dose', 'Both'), align = 'r') %>% 
  add_header_above(c('Time' = 3, 'Number of Patients' = 3, 'Selected Dose (%)' = 3)) %>% 
  kable_styling(full_width = TRUE)
```

```{r adalf, class.source = "fold-hide"}
output %>% 
  summarise(
    n_pfs_dose_selection = mean(`n_events_<dose selection>_<pfs>`), 
    n_pfs_interim = mean(`n_events_<interim>_<pfs>`),
    n_pfs_final = mean(`n_events_<final>_<pfs>`),
    n_os_dose_selection = mean(`n_events_<dose selection>_<os>`), 
    n_os_interim = mean(`n_events_<interim>_<os>`),
    n_os_final = mean(`n_events_<final>_<os>`)
  ) %>% 
  kable(col.names = NULL, digits = 1, align = 'r', 
        caption = 'Number of Events of PFS and OS at Stages') %>% 
  add_header_above(rep(c('Dose Selection', 'Interim', 'Final'), 2), align = 'r') %>% 
  add_header_above(c('PFS' = 3, 'OS' = 3)) %>% 
  kable_styling(full_width = TRUE)
```

```{r dgslja, class.source = "fold-hide"}
output %>% 
  summarise(
    power_pfs_low = mean(pfs_low_dose_decision == 'reject') * 100, 
    power_pfs_high = mean(pfs_high_dose_decision == 'reject') * 100, 
    power_pfs_or = mean(pfs_low_dose_decision == 'reject' | pfs_high_dose_decision == 'reject') * 100, 
    power_pfs_and = mean(pfs_low_dose_decision == 'reject' & pfs_high_dose_decision == 'reject') * 100, 
    power_os_low = mean(os_low_dose_decision == 'reject') * 100, 
    power_os_high = mean(os_high_dose_decision == 'reject') * 100, 
    power_os_or = mean(os_low_dose_decision == 'reject' | os_high_dose_decision == 'reject') * 100,
    power_os_and = mean(os_low_dose_decision == 'reject' & os_high_dose_decision == 'reject') * 100
  ) %>% 
  kable(col.names = NULL, digits = 1, align = 'r', 
        caption = 'Power of Testing PFS and OS') %>% 
  add_header_above(rep(c('Low Dose', 'High Dose', 'Low or High', 'Low and High'), 2), align = 'r') %>% 
  add_header_above(c('PFS (%)' = 4, 'OS (%)' = 4)) %>% 
  kable_styling(full_width = TRUE)
```

```{r eioajf, class.source = "fold-hide"}
output %>% 
  summarise(
    power_pfs_not_os = mean((pfs_low_dose_decision == 'reject' | pfs_high_dose_decision == 'reject') & 
                              os_low_dose_decision == 'accept' & os_high_dose_decision == 'accept') * 100, 
    power_os_not_pfs = mean((os_low_dose_decision == 'reject' | os_high_dose_decision == 'reject') & 
                              pfs_low_dose_decision == 'accept' & pfs_high_dose_decision == 'accept') * 100, 
    power_pfs_and_os = mean((pfs_low_dose_decision == 'reject' | pfs_high_dose_decision == 'reject') & 
                              (os_low_dose_decision == 'reject' | os_high_dose_decision == 'reject')) * 100,
    power_pfs_or_os = mean((pfs_low_dose_decision == 'reject' | pfs_high_dose_decision == 'reject') | 
                              (os_low_dose_decision == 'reject' | os_high_dose_decision == 'reject')) * 100
  ) %>% 
  kable(col.names = NULL, digits = 1, align = 'r', 
        caption = 'Power of Testing PFS and OS (Cont.)') %>% 
  add_header_above(c('Reject PFS and Accept OS', 'Accept PFS and Reject OS', 
                     'Reject PFS and OS', 'Reject PFS or OS'), align = 'r') %>% 
  kable_styling(full_width = TRUE)
```

---
title: "An Example of Fixed Design with Two Correlated Endpoints"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{An Example of Fixed Design with Two Correlated Endpoints}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/fixedDesign/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(dplyr)
library(kableExtra)
library(TrialSimulator)
set.seed(12345)
```

In this vignette, we illustrate how to use `TrialSimulator` to simulate a trial 
of two correlated endpoints. Under a fixed design, only one milestone is specified 
for final analysis. 

## Simulation Settings

-   Trial consists of two active arms of high and low dose, 
    and a standard-of-care arm.
-   Patients are randomized into the trial with ratio `1:1:1`.
-   30 patients are randomized per month for the first 10 months, and 50
    patients per month after then until 1000 patients.
-   Dropout rate is 10% at month 18, which is modeled by an exponential 
    distribution, with rate parameter `-log(1 - 0.1)/18`. 

-   Modeled by a three-state illness-death model, two endpoints `PFS` and `OS` 
    are simulated.
    -   Primary endpoint `OS` has a medians of 15, 18.5, 20 months in standard-of-care, 
        low dose and high dose arms, respectively. 
    -   Key secondary endpoint `PFS` has a median of 7, 9, 10 months in the 
        three arms. 
    -   Pearson's correlation between `OS` and `PFS` are 0.68, 0.65, and 0.60 in 
        the three arms. 

-   To ensures sufficient powers for both endpoints, final analysis is set when 
    we have at least a total of 800 `PFS` events in the standard-of-care and 
    high dose arm, meanwhile a total of 550 `OS` events are observed in three 
    arms
    -   `OS` is tested using one-sided logrank test. 
    -   `PFS` is tested using one-sided p-value from the Cox proportional hazard 
        model as the PH assumption is assumed. 
    -   The Bonferroni correction is adopted to split overall $\alpha = 5\%$, 
        i.e., claiming significant effect for an endpoint when p-value is lower 
        than $0.05/4$. 
    
## Transition Hazards of `PFS` and `OS`

We adopt the illness-death model to simulate the two endpoints. This ensures `PFS` 
$\leq$ `OS` with probability one, and makes no assumption on latent variables or 
copula parameters. `TrialSimulator` offers a function `solveThreeStateModel()` 
to convert endpoints' medians and correlation to the transition hazards, which 
are required by the built-in generator `CorrelatedPfsAndOs3()`. Refer to the 
vignette [Simulate Correlated Progression-Free Survival and Overall Survival as Endpoints Under Illness-Death Model](simulatePfsAndOsIdm.html). Note that data generated under the illness-death
model may induce a time-varying OS hazard ratio between treatment arms,
violating the proportional hazards (PH) assumption in a Cox model. For a data
model that is more suitable for PH Cox analysis, refer to the built-in
generator `CorrelatedPfsAndOs2()` and vignette of the 
[Gumbel copula method](simulatePfsAndOsGumbel.html).

```{r iioeal}
pars_soc <- solveThreeStateModel(median_pfs = 7, median_os = 15, corr = .68, 
                                 h12 = seq(.07, .10, length.out = 50))
pars_soc
```

```{r eaioie}
pars_low <- solveThreeStateModel(median_pfs = 9, median_os = 18.5, corr = .65, 
                                 h12 = seq(.04, .07, length.out = 50))
pars_low
```

```{r aealga}
pars_high <- solveThreeStateModel(median_pfs = 10, median_os = 20, corr = .60, 
                                  h12 = seq(.02, .06, length.out = 50))
pars_high
```

```{r eaald, echo=FALSE, eval=TRUE}
rbind(pars_soc, pars_low, pars_high) %>% 
  structure(class = 'data.frame') %>% 
  mutate(arm = c('soc', 'low', 'high')) %>% 
  select(arm, h01, h02, h12) %>% 
  kable(format = 'html', digits = 3, 
        caption = 'Transition hazards in three treatment arms')
```

## Define Treatment Arms

We use generator `CorrelatedPfsAndOs3()` with `endpoint()` and `arm()` to define 
the three treatment arms. 

```{r ieaof}
#' define SoC
pfs_os_in_soc <- endpoint(name = c('pfs', 'os'), 
                          type = c('tte', 'tte'), 
                          generator = CorrelatedPfsAndOs3, 
                          h01 = 0.075, h02 = 0.024, h12 = 0.090)

soc <- arm(name = 'soc')
soc$add_endpoints(pfs_os_in_soc)

#' define low dose arm
pfs_os_in_low <- endpoint(name = c('pfs', 'os'), 
                          type = c('tte', 'tte'), 
                          generator = CorrelatedPfsAndOs3, 
                          h01 = 0.051, h02 = 0.026, h12 = 0.062)

low <- arm(name = 'low')
low$add_endpoints(pfs_os_in_low)

#' define high dose arm
pfs_os_in_high <- endpoint(name = c('pfs', 'os'), 
                           type = c('tte', 'tte'), 
                           generator = CorrelatedPfsAndOs3, 
                          h01 = 0.040, h02 = 0.030, h12 = 0.047)

high <- arm(name = 'high')
high$add_endpoints(pfs_os_in_high)
```

We can request for a summary report of, e.g., the high dose arm, by printing 
the arm object in R console. The medians of `PFS` and `OS` matches to the settings 
very well. 

```{r ioeafaea, results='asis'}
high
```

## Define a Trial

With three arms, we can define a trial using the function `trial()`. Recruitment curve are specified through `enroller` with a built-in function `StaggeredRecruiter` of piecewise constant rate. We set `duration` to be an arbitrary large number (500) but controlling the end of trial through a pre-defined milestone later. Note that if `seed = NULL`, `TrialSimulator` will pick a seed for the purpose of reproducibility.

```{r ioeeaf}
accrual_rate <- data.frame(end_time = c(10, Inf),
                           piecewise_rate = c(30, 50))
trial <- trial(
  name = 'Trial-3415', n_patients = 1000,
  seed = 1727811904, duration = 500,
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
  dropout = rexp, rate = -log(1 - 0.1)/18, ## 10% by month 18
  silent = TRUE
)

trial$add_arms(sample_ratio = c(1, 1, 1), soc, low, high) ## 1:1:1
trial
```


## Define Milestone and Action for Final Analysis

To ensure sufficient powers of testing `PFS` and `OS`, final analysis is performed 
when we have at least 700 events for `OS` and 800 events for `PFS`. In the 
action function, we compute one-sided p-value of `PFS` using the proportional 
hazard Cox model, and one-sided p-value of `OS` using the logrank test. This is 
consistent with the assumption of illness-death model implemented in data generator 
`CorrelatedPfsAndOs3()`. Note that five columns are available in locked data: 
`arm`, `pfs`, 'os', 'pfs_event', and 'os_event', which are used to construct 
model formula. Estimates of hazard ratio are also computed. Built-in functions 
`fitCoxph` and `fitLogrank` return data frames. Refer to their help documants 
for more details. 

```{r wewfal}
action <- function(trial){
  
  locked_data <- trial$get_locked_data('final')
  
  pfs <- fitCoxph((Surv(pfs, pfs_event) ~ arm), placebo = 'soc', 
                  data = locked_data, alternative = 'less', 
                  scale = 'hazard ratio')
  
  os <- fitLogrank((Surv(os, os_event) ~ arm), placebo = 'soc', 
                   data = locked_data, alternative = 'less')
  
  ## Bonferroni test is applied to four hypotheses: 
  ## PFS_low, PFS_high, OS_low, and OS_high
  pfs$decision <- ifelse(pfs$p < .05/4, 'reject', 'accept')
  os$decision <- ifelse(os$p < .05/4, 'reject', 'accept')
  
  trial$save(
    value = pfs %>% filter(arm == 'low') %>% select(estimate, decision, info), 
    name = 'pfs_low')
  
  trial$save(
    value = pfs %>% filter(arm == 'high') %>% select(estimate, decision, info), 
    name = 'pfs_high')
  
  trial$save(
    value = os %>% filter(arm == 'low') %>% select(decision, info), 
    name = 'os_low')
  
  trial$save(
    value = os %>% filter(arm == 'high') %>% select(decision, info), 
    name = 'os_high')
  
}
```

Now we can define and register the milestone to a listener, which monitors 
the trial for us through a controller

```{r tuyo}
final <- milestone(name = 'final', action = action, 
                   when = eventNumber(endpoint = 'pfs', n = 450, 
                                      arms = c('soc', 'high')) & 
                     eventNumber(endpoint = 'os', n = 550)
                   )

listener <- listener()
listener$add_milestones(final)

controller <- controller(trial, listener)
```

We can run a massive number of replicates in simulation to study
operating characteristics of a trial design by specifying `n` in
`Controller$run()`. The simulation results can be accessed by calling the 
member function `get_output()` of the controller. 

```{r hgajffal, eval=FALSE, cache=TRUE, message=FALSE, warning=FALSE, results='hide'}
controller$run(n = 1000, plot_event = FALSE, silent = TRUE)
output <- controller$get_output()
```


```{r eaioalafj, echo=FALSE}
output <- TrialSimulator:::getFixedDesignOutput()
```

```{r eeioafla}
output %>% 
  head(5) %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")
```

For example, we can compute the powers and summarize the estimates of 
hazard ratio for `PFS`. 

```{r hghgale}
output %>% 
  summarise(
    across(matches('_<decision>$'), ~ mean(. == 'reject') * 100, .names = 'Power_{.col}'), 
    across(matches('_<estimate>$'), ~ mean(.x), .names = 'HR_{.col}')
  ) %>%
  rename_with(~ sub('_<decision>$', '', .), starts_with('Power_')) %>%
  rename_with(~ sub('_<estimate>$', '', .), starts_with('HR_')) %>% 
  kable(col.name = NULL, digits = 3, align = 'r') %>% 
  add_header_above(c('Low', 'High', 'Low', 'High', 'Low', 'High'), align = 'r') %>% 
  add_header_above(c('PFS' = 2, 'OS' = 2, 'PFS' = 2)) %>% 
  add_header_above(c('Power (%)' = 4, 'Hazard Ratio' = 2)) %>% 
  kable_styling(full_width = TRUE)
```

Note that `OS` does not satisfy the proportional hazard assumption, and a 
composite condition on event numbers is used to trigger the final analysis. 
Thus, the powers in the table above would not match to the output from power 
calculation packages, which is as expected. 

---
title: "Designs with Response-Adaptive Randomization"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Designs with Response-Adaptive Randomization}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/responseAdaptive/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(dplyr)
library(kableExtra)
library(DoseFinding)
library(TrialSimulator)
set.seed(12345)
```

Response-adaptive randomization (RAR) can be a powerful strategy in Phase II dose-finding trials. It allows sponsors to dynamically update the randomization scheme at one or more interim analyses based on accumulating data. By shifting allocation toward more promising treatment arms, RAR can enhance the ethical and statistical efficiency of the trial.

This vignette demonstrates how to simulate a trial with response-adaptive design using the `TrialSimulator` package. For further background, refer to [this](https://www.mediana.us/MedianaDesigner/ADRand.pdf) document from the `MedianaDesigner` package. Dr. Alex Dmitrienko also provides a series of excellent online lectures on this topic:

- [Part 1](https://www.youtube.com/watch?v=-ADsHASLimM&list=PL_qOo99xv_nRJU8MEBO_VzBjH8duFy01n&index=3&ab_channel=MedianaOnline)

- [Part 2](https://www.youtube.com/watch?v=2agD_9qP1cU&t=1447s&ab_channel=MedianaOnline)

However, the original `MedianaDesigner::ADRand()` function is no longer functional, even for examples provided on [this](https://medianasoft.github.io/CaseStudyF1) page. Therefore, this vignette focuses on implementing a similar response-adaptive design using `TrialSimulator`. The core algorithm for updating the randomization ratio is re-implemented based on the logic of the `DoseFinding` package and may differ slightly from that used in Dr. Dmitrienko's materials.

## Simulation Settings

- We assume an `Emax` model for the endpoint `fev1` (forced expiratory volume in 1 second) measured after 4 months of treatment. The maximum effect (0.1) is achieved at dose 100.

- The trial includes one placebo arm and five active arms with doses: 20, 25, 30, and 35.

  - Patients are initially randomized equally across all five arms.
  
- A total of 200 patients are recruited over 36 months, with 50% of enrollment expected by 24 months.

- Two interim analyses are planned after 50 and 120 patients have non-missing `fev1` readouts, i.e. pipeline patients are excluded. 
  
- The final analysis is performed when data from all 200 patients are available.

- At each interim: 

  - Candidate dose-response models `Emax`, `sigEmax`, and `quadratic` are fitted.
  
  - Bootstrap estimates from `DoseFinding::maFitMod()` are used to calculate, for each dose $d \in \{20, 25, 30, 35\}$, the probability $p_d$ that the estimated treatment effect exceeds 0.08.
  
  - The randomization ratio for each active dose is set proportional to $p_d$
  
  - The placebo ratio remains fixed at 20%.

- At the final analysis, a multiple contrast test is conducted using data from all 200 patients.

## Define Data Generator of `fev1`

The following function generates `fev1` outcomes using the assumed `Emax` model. It is later assigned as the generator function when defining endpoints.

```{r eaioajf}
rng <- function(n, dose){

  model <- DoseFinding::Mods(
    emax = c(2.6, 12.5),
    placEff = 1.25, maxEff = 0.1,
    doses = c(0, 20, 25, 50, 100))

  data.frame(
    fev1 = rnorm(n, mean = DoseFinding::getResp(model, doses = dose), sd = .05)
  )

}
```

## Define `fev1` Endpoints for Each Arm

Each treatment arm is associated with an endpoint definition, specifying the dose and data generator.

```{r lalglafj}
fev1 <- endpoint(name = 'fev1', type = 'non-tte', readout = c(fev1 = 4),
                 generator = rng, dose = 0)
pbo <- arm(name = '0.0')
pbo$add_endpoints(fev1)

fev1 <- endpoint(name = 'fev1', type = 'non-tte', readout = c(fev1 = 4),
                 generator = rng, dose = 20.0)
dose1 <- arm(name = '20.0')
dose1$add_endpoints(fev1)

fev1 <- endpoint(name = 'fev1', type = 'non-tte', readout = c(fev1 = 4),
                 generator = rng, dose = 25.0)
dose2 <- arm(name = '25.0')
dose2$add_endpoints(fev1)

fev1 <- endpoint(name = 'fev1', type = 'non-tte', readout = c(fev1 = 4),
                 generator = rng, dose = 30.0)
dose3 <- arm(name = '30.0')
dose3$add_endpoints(fev1)

fev1 <- endpoint(name = 'fev1', type = 'non-tte', readout = c(fev1 = 4),
                 generator = rng, dose = 35.0)
dose4 <- arm(name = '35.0')
dose4$add_endpoints(fev1)
```


## Define a Trial

Here we define the trial object with 200 patients and an accrual period of 36 months. The total trial duration is extended to 40 months to account for a 4-month follow-up after last enrollment.

```{r weoieaf}
accrual_rate <- data.frame(end_time = c(24, Inf),
                           piecewise_rate = c(100/24, 100/12))
trial <- trial(
  name = 'Trial-3415', n_patients = 200,
  seed = 1727811904, duration = 40,
  enroller = StaggeredRecruiter, accrual_rate = accrual_rate,
  silent = TRUE
)

trial$add_arms(sample_ratio = rep(1, 5), pbo, dose1, dose2, dose3, dose4)
trial
```

<!-- Hide this helper function and display it in Appendix for better typesetting. -->
```{r helpers, echo=FALSE}
compute_sample_ratio <- function(data){

  data$dose <- as.numeric(data$arm)
  fit <- lm(fev1 ~ factor(dose) - 1, data = data)
  dose <- unique(sort(data$dose))
  mu_hat <- coef(fit)
  S_hat <- vcov(fit)

  suppressMessages(
    ma_fit <- DoseFinding::maFitMod(dose, mu_hat, S = S_hat,
                                    models = c("emax", "sigEmax", "quadratic"))
  )

  pred <- predict(ma_fit, doseSeq = c(0, 20, 25, 30, 35), summaryFct = NULL)
  prob <- apply(pred[, -1] - pred[, 1], 2, function(x){mean(x > .08)})
  sample_ratio <- c(.2, (1 - .2) * prob / sum(prob)) %>% unname()

  sample_ratio
}

multiple_contrast_test <- function(data){
  
  candidate_models <- DoseFinding::Mods(
    emax = c(2.6, 12.5), sigEmax = c(30.5, 3.5), quadratic = -0.00776,
    placEff = 1.25, maxEff = 0.15, doses = c(0, 20, 25, 30, 35))

  data$dose <- as.numeric(data$arm)
  test <- DoseFinding::MCTtest(dose = dose, resp = fev1,
                               models = candidate_models, data = data)
  
  ## at least one dose shows significant non-flatten pattern
  any(attr(test$tStat, 'pVal') < .05)

}
```

<!-- Hide this action function and display it later for better typesetting. -->

```{r stage_action, echo=FALSE}
stage_action <- function(trial, milestone_name){

  locked_data <- trial$get_locked_data(milestone_name)
  new_sample_ratio <- compute_sample_ratio(locked_data)

  trial$update_sample_ratio(arm_names = c('0.0', '20.0', '25.0', '30.0', '35.0'),
                            sample_ratios = new_sample_ratio)
  
  message(milestone_name, ': ')
  data.frame(table(locked_data$arm), new_sample_ratio) %>%
    setNames(c('dose', 'total_n', 'new_ratio')) %>% print()

}
```

<!-- Hide this action function and display it later for better typesetting. -->

```{r final_action, echo=FALSE}
final_action <- function(trial){

  locked_data <- trial$get_locked_data('final')
  
  message('final: ')
  data.frame(table(locked_data$arm)) %>%
    setNames(c('dose', 'total_n')) %>% print()

  trial$save(value = multiple_contrast_test(locked_data),
             name = 'MC_test')
}
```


## Define Milestones and Associated Actions

Three milestones are defined: two interim analyses and one final analysis. The same action is used for both interims, while a separate one is used for the final.

```{r gfdassfl}
stage1 <- milestone(name = 'stage 1',
                    when = eventNumber('fev1', n = 50),
                    action = stage_action, milestone_name = 'stage 1')

stage2 <- milestone(name = 'stage 2',
                    when = eventNumber('fev1', n = 120),
                    action = stage_action, milestone_name = 'stage 2')

final <- milestone(name = 'final',
                   when = eventNumber('fev1', n = 200),
                   action = final_action)
```

The `stage_action()` function is called at each interim milestone to lock current data and update sample ratios based on model-based probabilities. It utilities a helper function `compute_sample_ratio()` which can be found in the Appendix below. 


```{r ref.label='stage_action', eval=FALSE}
```


At the final milestone, the function `final_action()` performs the multiple contrast test and stores the result. It calls a helper function `multiple_contrast_test()`, which can be found in the Appendix below. 

```{r ref.label='final_action', eval=FALSE}
```

## Execute a Trial

After registering all milestones with a listener object, we simulate the trial using `controller$run()`.

```{r sdlatiow}
listener <- listener()
listener$add_milestones(stage1, stage2, final)

controller <- controller(trial, listener)
controller$run(n = 1, silent = TRUE)

output <- controller$get_output()

output %>% 
  kable(escape = FALSE) %>% 
  kable_styling(bootstrap_options = "striped", 
                full_width = FALSE,
                position = "left") %>%
  scroll_box(width = "100%")
```

In the output, the columns `n_event_<milestone>_<arms>` contain detailed information on observed events or sample sizes per arm at each milestone. It is evident that we have pipeline patients at both interim. 

```{r qeaeal}
output[, 'n_events_<stage 1>_<arms>']

output[, 'n_events_<stage 2>_<arms>']

output[, 'n_events_<final>_<arms>']
```


## Appendix: Codes of Helper Functions

For completeness, the full code of the helper functions `compute_sample_ratio()` and `multiple_contrast_test()` is included below, which determine the new sample ratio and performs the multiple contrast test. Note that the implementation to these two functions are completely project-specific. 

```{r ref.label='helpers', eval=FALSE}
```

---
title: "Simulate Correlated Progression-Free Survival and Overall Survival Using a Gumbel Copula"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Simulate Correlated Progression-Free Survival and Overall Survival Using a Gumbel Copula}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/simulatePfsAndOsGumbel/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(dplyr)
library(knitr)
library(ggplot2)
library(TrialSimulator)

if (!requireNamespace('copula', quietly = TRUE)) {
  knitr::opts_chunk$set(eval = FALSE)
}
```

Progression-free survival (PFS) and overall survival (OS) are common time-to-event endpoints in oncology trials. Because progression or death defines a PFS event, simulated trial data should always satisfy $\mbox{PFS} \leq \mbox{OS}$. At the same time, the two endpoints are often highly correlated, so independent marginal simulation is usually not appropriate.

This vignette describes the copula-based generator implemented in `TrialSimulator::CorrelatedPfsAndOs2()`. The function is intended for simulation settings where the user wants to specify three interpretable quantities:

- the marginal median PFS,
- the marginal median OS,
- Kendall's tau between the observed, uncensored PFS and OS times.

The method uses a Gumbel survival copula for a latent time to progression (TTP) and OS, and then defines

$$
\mbox{PFS} = \min(\mbox{TTP}, \mbox{OS}).
$$

This construction guarantees $\mbox{PFS} \leq \mbox{OS}$ for every simulated patient. It also keeps both marginal PFS and OS exponential, so the inputs can be specified directly through their medians.

This is a useful distinction from the illness-death model in `TrialSimulator::CorrelatedPfsAndOs3()`. The illness-death model gives a clinically interpretable transition structure, but OS generated from that model can have a time-varying hazard ratio between treatment arms. Therefore, if OS will be analyzed using a proportional hazards Cox model and the simulation should be consistent with that analysis model, the copula-based method described here is often more appropriate.

## Model

Let $X$ denote latent TTP and let $Y$ denote OS. The model assumes exponential marginal survival functions

$$
\Pr(X > x) = \exp(-\lambda_X x), \qquad
\Pr(Y > y) = \exp(-\lambda_Y y),
$$

and a Gumbel--Hougaard survival copula

$$
\Pr(X > x, Y > y)
=
\exp\left[
-\left\{(\lambda_X x)^\theta + (\lambda_Y y)^\theta\right\}^{1/\theta}
\right],
\qquad \theta \geq 1.
$$

The observed PFS time is

$$
P = \min(X, Y).
$$

Under this model, OS is exponential with rate $\lambda_Y$. PFS is also exponential, with survival function

$$
\Pr(P > t)
=
\Pr(X > t, Y > t)
=
\exp\left[
-\left\{(\lambda_X^\theta + \lambda_Y^\theta)t^\theta\right\}^{1/\theta}
\right]
=
\exp(-\lambda_P t),
$$

where

$$
\lambda_P = \left(\lambda_X^\theta + \lambda_Y^\theta\right)^{1/\theta}.
$$

Therefore, if the target medians are $m_P$ for PFS and $m_Y$ for OS,

$$
\lambda_P = \frac{\log(2)}{m_P},
\qquad
\lambda_Y = \frac{\log(2)}{m_Y}.
$$

For a candidate value of $\theta$, the latent TTP rate is then

$$
\lambda_X =
\left(\lambda_P^\theta - \lambda_Y^\theta\right)^{1/\theta}.
$$

This requires $m_P < m_Y$, as expected for PFS and OS.

## Choosing the Copula Parameter

For the Gumbel copula, Kendall's tau between latent TTP and OS is

$$
\tau(X,Y) = 1 - \frac{1}{\theta}.
$$

However, `CorrelatedPfsAndOs2()` asks for Kendall's tau between the observed, uncensored endpoints $P=\min(X,Y)$ and $Y$, not between latent TTP and OS. These are not the same quantity. The formula below is derived in the appendix. Under the model above,

$$
\tau(P,Y)
=
1 - \frac{1}{\theta}
\left[
1 - \left(\frac{m_P}{m_Y}\right)^\theta
\right].
$$

Equivalently, writing $\tau_X = \tau(X,Y)$ and $\theta = 1/(1-\tau_X)$,

$$
\tau(P,Y)
=
1 -
(1-\tau_X)
\left[
1 -
\left(\frac{m_P}{m_Y}\right)^{1/(1-\tau_X)}
\right].
$$

`CorrelatedPfsAndOs2()` solves this equation numerically for the latent Kendall's tau $\tau_X$, converts it to $\theta$, and then simulates from the Gumbel copula.

For any finite $\theta$, $\tau(P,Y)$ is larger than $\tau(X,Y)$:

$$
\tau(P,Y)-\tau(X,Y)
=
\frac{1}{\theta}
\left(\frac{m_P}{m_Y}\right)^\theta
> 0.
$$

Intuitively, even if latent TTP and OS have a weaker association, observed PFS contains deaths by construction, which increases the association between observed PFS and OS.

The requested value of $\tau(P,Y)$ has a lower bound. When $\theta=1$, TTP and OS are independent, but PFS still contains OS deaths through the definition $P=\min(X,Y)$. In this limiting case,

$$
\tau(P,Y) = \frac{m_P}{m_Y}.
$$

Thus, for fixed medians $m_P$ and $m_Y$, `CorrelatedPfsAndOs2()` can only target

$$
\frac{m_P}{m_Y} \leq \tau(P,Y) < 1.
$$

If the requested Kendall's tau is too small for the two medians, the function stops with an error.

## Example

Suppose we want to simulate PFS with median 5 months and OS with median 11 months, targeting Kendall's tau of 0.6 between observed, uncensored PFS and OS times.

```{r example-endpoint, results = 'asis'}
pfs_and_os <- endpoint(name = c('pfs', 'os'),
                       type = c('tte', 'tte'),
                       generator = CorrelatedPfsAndOs2,
                       median_pfs = 5,
                       median_os = 11,
                       kendall = 0.6,
                       pfs_name = 'pfs',
                       os_name = 'os')

pfs_and_os
```

For verification only, we can call the generator directly. This is not the recommended way to use `TrialSimulator` for simulation studies; in practice, the generator should usually be supplied to `endpoint()` and used inside the trial simulation workflow. Direct calls are helpful here because they let us check the marginal medians, the observed Kendall's tau, and the ordering $\mbox{PFS} \leq \mbox{OS}$ before adding enrollment, censoring, treatment arms, and analyses.

```{r direct-generator}
set.seed(123)
dat <- CorrelatedPfsAndOs2(n = 10000,
                           median_pfs = 5,
                           median_os = 11,
                           kendall = 0.6)

head(dat, 2)
```

The simulation should approximately recover the requested medians and Kendall's tau between observed, uncensored PFS and OS times.

```{r validation}
with(dat, median(pfs))
with(dat, median(os))
with(dat, cor(pfs, os, method = 'kendall'))
with(dat, all(pfs <= os))
```

Because this generator returns event indicators equal to 1 for both endpoints, censoring and staggered enrollment should be handled by the broader trial simulation workflow rather than by this endpoint generator.

## Practical Guidance

This generator is most useful when the simulation objective is to specify marginal PFS and OS medians and a rank-based association between the two observed endpoints. Kendall's tau is often more stable and interpretable than Pearson correlation for skewed time-to-event variables, and it is directly connected to the Gumbel copula parameter.

There are still modeling assumptions to keep in mind:

- PFS and OS are both marginally exponential.
- Dependence is induced through a Gumbel survival copula on latent TTP and OS.
- The Gumbel copula only allows non-negative dependence.
- The requested Kendall's tau is for observed PFS and OS event times before censoring is applied.
- The attainable Kendall's tau between observed PFS and OS is constrained by the PFS/OS median ratio.

For simulations that require transition-specific hazards or a mechanistic disease process, the illness-death model implemented in `TrialSimulator::CorrelatedPfsAndOs3()` may be more appropriate. For simulations where the main requirement is a direct medians-plus-Kendall's-tau specification, especially when OS will be analyzed under a proportional hazards Cox model, `CorrelatedPfsAndOs2()` provides a compact alternative.

## Appendix: Kendall's Tau for Observed PFS and OS

This appendix gives the derivation behind the formula used in `CorrelatedPfsAndOs2()`.

Let $X$ and $Y$ be nonnegative random variables with exponential margins

$$
X \sim \operatorname{Exp}(\lambda_X),
\qquad
Y \sim \operatorname{Exp}(\lambda_Y),
$$

and joint survival function

$$
S_{X,Y}(x,y)
=
\Pr(X>x, Y>y)
=
\exp\left[
-\left\{(\lambda_X x)^\theta + (\lambda_Y y)^\theta\right\}^{1/\theta}
\right],
\qquad \theta \ge 1.
$$

Define $P=\min(X,Y)$. We want Kendall's tau between $P$ and $Y$.

Standardize the margins by setting

$$
A=\lambda_X X, \qquad B=\lambda_Y Y.
$$

Then the joint survival function of $(A,B)$ is

$$
\Pr(A>a, B>b)
=
\exp\left\{-(a^\theta+b^\theta)^{1/\theta}\right\}.
$$

Use the polar-type transformation

$$
R=(A^\theta+B^\theta)^{1/\theta},
\qquad
W=\frac{A^\theta}{A^\theta+B^\theta}.
$$

A Jacobian calculation gives the joint density

$$
f_{R,W}(r,w)=\frac{1}{\theta} e^{-r}(r+\theta-1),
\qquad r>0,\; 0<w<1.
$$

Thus, $R$ and $W$ are independent and $W \sim \operatorname{Unif}(0,1)$. The inverse transformation is

$$
A = R W^{1/\theta},
\qquad
B = R(1-W)^{1/\theta}.
$$

Let

$$
a=\lambda_X^\theta,
\qquad
b=\lambda_Y^\theta,
\qquad
c=\frac{a}{a+b},
\qquad
d=\frac{b}{a+b}.
$$

The event $X \le Y$ is equivalent to $W \le c$:

$$
X \le Y
\iff
\frac{A}{\lambda_X} \le \frac{B}{\lambda_Y}
\iff
W \le c,
$$

so

$$
P=
\begin{cases}
X, & W \le c,\\
Y, & W > c.
\end{cases}
$$

Using the survival-copula representation of Kendall's tau,

$$
\tau(P,Y)=4E\{S_{P,Y}(P,Y)\}-1.
$$

When $W \le c$, $P=X$ and

$$
S_{P,Y}(P,Y)=S_{X,Y}(X,Y)=e^{-R}.
$$

When $W>c$, $P=Y$. Since

$$
S_{P,Y}(y,y)=\Pr(P>y,Y>y)=\Pr(X>y,Y>y),
$$

we have, using $Y=B/\lambda_Y=R(1-W)^{1/\theta}/\lambda_Y$,

$$
S_{P,Y}(Y,Y)
=
\exp\left[
-R\left(\frac{1-W}{d}\right)^{1/\theta}
\right].
$$

Therefore,

$$
E\{S_{P,Y}(P,Y)\}
=
E\{e^{-R}\mathbf 1(W\le c)\}
+
E\left[
e^{-R((1-W)/d)^{1/\theta}}
\mathbf 1(W>c)
\right].
$$

By independence of $R$ and $W$,

$$
E\{S_{P,Y}(P,Y)\}
=
cL_R(1)
+
\int_c^1
L_R\left\{\left(\frac{1-w}{d}\right)^{1/\theta}\right\}
dw,
$$

where $L_R(s)=E(e^{-sR})$. From the density of $R$,

$$
L_R(s)
=
\frac{1}{\theta}
\int_0^\infty e^{-(1+s)r}(r+\theta-1)\,dr
=
\frac{\theta+(\theta-1)s}{\theta(1+s)^2}.
$$

In particular,

$$
L_R(1)=\frac{2\theta-1}{4\theta}.
$$

For the integral term, use the substitution

$$
t=\left(\frac{1-w}{d}\right)^{1/\theta},
\qquad
dw=-d\theta t^{\theta-1}dt.
$$

Then

$$
\int_c^1
L_R\left\{\left(\frac{1-w}{d}\right)^{1/\theta}\right\}
dw
=
d\theta\int_0^1 t^{\theta-1}L_R(t)\,dt.
$$

Substituting $L_R(t)$ gives

$$
d\int_0^1
\frac{\theta t^{\theta-1}+(\theta-1)t^\theta}{(1+t)^2}
dt.
$$

Because

$$
\frac{d}{dt}\left(\frac{t^\theta}{1+t}\right)
=
\frac{\theta t^{\theta-1}+(\theta-1)t^\theta}{(1+t)^2},
$$

the integral is

$$
d\left[\frac{t^\theta}{1+t}\right]_0^1
=
\frac{d}{2}.
$$

Thus,

$$
E\{S_{P,Y}(P,Y)\}
=
c\frac{2\theta-1}{4\theta}+\frac{d}{2}
=
\frac{2\theta-1+d}{4\theta},
$$

using $c+d=1$. Finally,

$$
\tau(P,Y)
=
4E\{S_{P,Y}(P,Y)\}-1
=
\frac{\theta-1+d}{\theta}
=
1-\frac{1}{\theta}
\left(
\frac{\lambda_X^\theta}{\lambda_X^\theta+\lambda_Y^\theta}
\right).
$$

Since $P$ has rate

$$
\lambda_P =
\left(\lambda_X^\theta+\lambda_Y^\theta\right)^{1/\theta},
$$

this can also be written as

$$
\tau(P,Y)
=
1-\frac{1}{\theta}
\left(
\frac{\lambda_X^\theta}{\lambda_P^\theta}
\right)
=
1-\frac{1}{\theta}
\left[
1-\left(\frac{\lambda_Y}{\lambda_P}\right)^\theta
\right].
$$

Because $\lambda_Y/\lambda_P = m_P/m_Y$, the formula used by `CorrelatedPfsAndOs2()` is

$$
\tau(P,Y)
=
1-\frac{1}{\theta}
\left[
1-\left(\frac{m_P}{m_Y}\right)^\theta
\right].
$$

---
title: "Specify Actions to Execute at Trial Milestones"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Specify Actions to Execute at Trial Milestones}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache = TRUE,
  cache.path = 'cache/defineActionFunctions/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, message = FALSE, echo=FALSE}
library(dplyr)
library(TrialSimulator)
set.seed(12345)
```

In `TrialSimulator`, we can define custom action functions that will be executed automatically whenever a trial milestone is reached. This mechanism provides a flexible way to simulate adaptive and dynamic trial behaviors, such as:

-   Conducting interim analyses

-   Early stopping for efficacy or futility

-   Dropping, adding or selecting treatment arms

-   Resizing based on sample size reassessment

-   extending trial duration

-   Adjusting enrollment rules

-   Logging or tracking intermediate results

-   Locking and exporting trial data snapshot at specific points

This vignette demonstrates how to define such actions, associate them with milestones, and make use of them to enrich trial simulations.

## Define Trial Milestones

A milestone represents a pre-specified time point or condition during the trial at which a snapshot of accumulated data becomes available for custom analysis. Each milestone consists of three key elements:

-   `name`: the label of the milestone. A data snapshot at the triggering time can be retrieved by calling `trial$get_locked_data(<name>)`, where `<name>` is a placeholder of `name`.

-   `when`: the logical conditions that trigger the milestone. Refer to the vignette [Condition System for Triggering Milestones in a Trial](conditionSystem.html) for more information.

-   `action`: a function to be executed once the milestone is reached. Within this function, we can analyze data snapshot, make decisions, and store analysis results for later summary.

For example, consider an interim analysis that is triggered once the trial has been running for at least 20 months and at least 450 events have occurred for the endpoint `PFS`.

``` r
interim <- milestone(name = 'interim analysis', 
                     action = doNothing, 
                     when = calendarTime(time = 20) & 
                       eventNumber('PFS', n = 450)
                     )
```

Note that in this example, `action` is set to the default `doNothing` function. In such cases, `TrialSimulator` will not perform additional computations but will still record the triggering time of the milestone. This can be useful when the sole interest is in timing of interim analysis given recruitment and dropout assumptions.

Importantly, whenever a milestone is reached, `TrialSimulator` **automatically** records a set of standard outputs, even when `doNothing` is used. These are summarized in the following table.

+-----------------------------------------------------------------------+------------+------------------------------------------------------------+-----------------------------------+
| Type of Results                                                       | Format     | Column Naming Convention                                   | Examples                          |
+=======================================================================+============+============================================================+===================================+
| Time of triggering a milestone                                        | scalar     | `milestone_time_<...>`                                     | `milestone_time_<interim>`        |
|                                                                       |            |                                                            |                                   |
|                                                                       |            | where `...` is milestone name                              |                                   |
+-----------------------------------------------------------------------+------------+------------------------------------------------------------+-----------------------------------+
| Number of observed events for a time-to-event endpoint                | scalar     | `n_events_<...1>_<...2>`                                   | `n_events_<interim>_<PFS>`        |
|                                                                       |            |                                                            |                                   |
|                                                                       |            | where `...1` is milestone name and `...2` is endpoint name |                                   |
+-----------------------------------------------------------------------+------------+------------------------------------------------------------+-----------------------------------+
| Number of observed non-missing readouts for a non-TTE endpoint        | scalar     | `n_events_<...1>_<...2>`                                   | `n_events_<final>_<ORR>`          |
|                                                                       |            |                                                            |                                   |
|                                                                       |            | where `...1` is milestone name and `...2` is endpoint name |                                   |
+-----------------------------------------------------------------------+------------+------------------------------------------------------------+-----------------------------------+
| Number of enrolled patients                                           | scalar     | `n_events_<...>_<patient_id>`                              | `n_events_<interim>_<patient_id>` |
|                                                                       |            |                                                            |                                   |
|                                                                       |            | where `...` is milestone name                              |                                   |
+-----------------------------------------------------------------------+------------+------------------------------------------------------------+-----------------------------------+
| Number of observed events/readouts per arm for a TTE/non-TTE endpoint | data frame | `n_events_<...>_<arms>`                                    | `n_events_<final>_<arms>`         |
|                                                                       |            |                                                            |                                   |
|                                                                       |            | where `...` is milestone name                              |                                   |
+-----------------------------------------------------------------------+------------+------------------------------------------------------------+-----------------------------------+

: Automatically Saved Results At Triggered Milestones

Note that these automatically saved columns can be eliminated from outputs by setting `tidy = TRUE` in `controller$get_output()`. Currently, we use regex `"^n_events_<.*?>_<.*?>$"` and `"^milestone_time_<.*?>$"` to match columns to be eliminated. If users plan to use `tidy = TRUE`, caution is needed when naming \#' custom outputs in `save()`.

We can define arbitrary number of milestones in a trial. For example, the final analysis might be triggered when both `PFS` and `OS` events reach their required numbers:

``` r
final <- milestone(name = 'final analysis', 
                   action = doNothing, 
                   when = eventNumber('PFS', n = 800) & 
                     eventNumber('OS', n = 500))
```

Here the trial is considered complete once sufficient events are observed for both endpoints to ensure targeted statistical power.

Note that a milestone is only triggered if

-   its `when` condition is satisfied, **and**

-   the milestone has been registered to a `listener`.

The following code snippet shows how to register milestones and run a trial with a controller. You may have seen it in many of the documents of this package:

```{r eval=FALSE, include=TRUE}
listener <- listener()
#' register milestones with listener
listener$add_milestones(interim, final)
controller <- controller(trial, listener)
controller$run()
```

## Custom Action Functions

In most realistic scenarios, we do not simply want to record the timing of a milestone and number of events/readouts. Instead, we want to analyze the data snapshot available at that time and possibly take actions based on the results. To achieve this, we define a custom action function and assign it to the `action` argument of a milestone. A custom action function must always takes `trial` as its first argument, which is returned from the function `trial()`.

The first step inside every action function should be calling `trial$get_locked_data()` to retrieve data snapshot using milestone name. After this, we are free to perform any analyses or decision-making steps.

For example, the following action function illustrates how one might analyze survival data at an interim milestone. Here, hazard ratios are estimated from proportional hazard models, and p-values (or other statistics) can also be calculated if desired.

``` r
action_at_interim <- function(trial){
  
  locked_data <- trial$get_locked_data('interim analysis')
  
  ## write your own codes to fit coxph model on locked_data
  ## extract estimate and p-value of hazard ratio
  
  ## no return value is needed in an action function
  
}
```

Once the functions is defined, it can be passed to the `action` argument of a milestone:

``` r
interim <- milestone(name = 'interim analysis', 
                     action = action_at_interim, 
                     when = calendarTime(time = 20) & 
                       eventNumber('PFS', n = 450)
                     )
```

Thus, whenever the milestone is triggered, the action function will be executed.

Within a custom action function, we can go beyond simple data access. The main possibilities include

-   performing statistical analyses;

-   saving intermediate results;

-   altering the trial through adaptation.

Each of these topics will be discussed in the following sections.

### Perform statistical analyses on data snapshot

Statisticians rarely stop at just locking the data–we want to analyze it!

A milestone provide an opportunity to perform formal analysis on the accumulated data, guiding decisions such as continuing, adapting, or stopping the trial.

Let's revisit the interim milestone. The following example shows how one might compute hazard ratios for two active arms (low and high dose) compared against placebo. The action function extracts the snapshot of patient-level data, fits separate Cox proportional hazards models, and then computes the hazard ratios.

``` r
action_at_interim <- function(trial){
  
  locked_data <- trial$get_locked_data('interim analysis')
  
  ## write your own codes to fit coxph model on locked_data
  fit_high <- coxph(Surv(PFS, PFS_event) ~ arm, 
                    data = locked_data %>% 
                      filter(arm %in% c('pbo', 'high'))
                    )
  
  fit_low <- coxph(Surv(PFS, PFS_event) ~ arm, 
                   data = locked_data %>% 
                     filter(arm %in% c('pbo', 'low'))
                   )
  
  hr <- data.frame(high = exp(coef(fit_high)), 
                   low = exp(coef(fit_low)))
  
  ## write more codes to compute one- or two-sided p-values, etc. 
  
}
```

This function illustrates the flexibility of action functions: any valid R code can be executed at the milestone. However, writing such functions from scratch may become **cumbersome** and **error-prone**, especially when:

-   multiple treatment arms exist,

-   one-sided tests are required, or

-   different trial scenarios have varying arm names or endpoints

To reduce coding burden and promote standardization, `TrialSimulator` provides a collection of **helper functions** for common statistical analyses. These wrappers ensure consistent inputs and outputs, making downstream use (such as saving or combining results) much easier.

Here is a rewritten version of the interim action function using these helper functions:

``` r
action_at_interim <- function(trial){
  
  locked_data <- trial$get_locked_data('interim analysis')
  
  fit_PFS <- fitCoxph(Surv(PFS, PFS_event) ~ arm, 
                      placebo = 'pbo', 
                      data = locked_data, 
                      alternative = 'less', 
                      scale = 'hazard ratio') # also also request other scales
  
  fit_OS <- fitLogrank(Surv(OS, OS_event) ~ arm, 
                       placebo = 'pbo', 
                       data = locked_data, 
                       alternative = 'less', 
                       biomarker1 == 'positive' & 
                         biomarker2 == 'negative') # specify subgroup through ...
  
}
```

In this example,

-   `fitCoxph` estimates hazard ratio for `PFS` against placebo, with a one-sided test.

-   `figLogrank` performs logrank test for `OS`, restricted to a biomarker-defined subgroup through the argument `...` in helper functions.

Both helper functions return results in a standardized format, which integrates seamlessly with other utilities of `TrialSimulator`. This is particularly useful when saving outputs across milestones or performing multiplicity adjustments.

For more complete description of these helper functions for common statistical analyses, see the vignette [Wrapper Functions of Common Statistical Methods in TrialSimulator](wrappers.html).

At this stage, we are only calculating statistics. In the next section, we will show how to save these results for later use—either to summarize operating characteristics or to carry information forward into future milestones.

### Save intermediate results

`TrialSimulator` offers dedicated member functions for saving intermediate results during a trial. The stored information can be broadly classified into two categories:

-   **Trial output**—results that directly contribute to the summary of trial operating characteristics (e.g., effect estimates, test decisions) or anything that we need for each simulated trial. They are stored in a private single-row data frame within the `trial` object. We cannot access or modify it directly. Instead, they should be inserted and accessed using member functions `trial$save()` and `trial$get_output()`. Since outputs are stored in a single-row data frame, only scalars or single-row data frame are accepted by `trial$save()`

-   **auxiliary information**—results that may be useful in subsequent analysis (e.g., in action functions of other milestones). Any results that are not directly part of trial-level summaries should be considered as auxiliary information. These are stored using member function `trial$save_custom_data()`. To retrieve it later, simply call `trial$get()`. This mechanism provides a safe way to pass data across milestones without relying on global variables. Unlike `trial$save()`, `trial$save_custom_data()` can save any objects, which provides flexibility.

In addition, a convenience function `trial$bind()` is provided for sequentially appending rows to a stored data frame—useful when accumulating results across milestones.

Here we give more details of those member functions. We can also refer to their manuals by running `?Trials` in R console.

-   `save(value, name)`: saves a scalar or a single-row data frame into the trial's output.

    -   later retrieved with `trial$get_output(cols = name)`.

    -   calling `trial$get_output()` without argument returns all saved outputs available so far.

-   `save_custom_data(value, name)`: saves an object of any type as auxiliary information as a temporary and private list within the trial.

    -   retrieved later by calling `trial$get(name)`.

-   `bind(value, name)`: a special case of `save_custom_data` for data frames.

    -   If the named data frame already exists, the new `value` is row-bound to it. Otherwise, a new data frame is created and saved.

    -   retrieved later by `trial$get_custom_data(name)`.

    -   particularly useful for accumulating results across multiple milestones in group sequential designs. For example, we may want to save estimates, p-values, etc. across milestones to a data frame, so that we can test it with a graphical testing strategy when all p-values are collected (i.e. at the last milestone).

#### Example 1: saving hazard ratio and p-value

Below, the interim action function computes a hazard ratio `hr` and its p-value `pval`, then saves them as part of the trial output:

``` r
action_at_interim <- function(trial){
  
  locked_data <- trial$get_locked_data('interim analysis')
  
  ## fit coxph model on locked_data
  ## extract estimate and p-value of hazard ratio
  ## assume that hr is estimate, and pval is p-value
  trial$save(value = hr, name = 'pfs_interim_hazard_ratio')
  trial$save(value = pval, name = 'pfs_interim_p_value')
  
  ## values of hr and pval can be accessed anywhere later by calling
  ## trial$get_output(cols = 'pfs_interim_hazard_ratio')
  ## trial$get_output(cols = 'pfs_interim_p_value')
  ## trial$get_output(cols = c('pfs_interim_hazard_ratio', 'pfs_interim_hazard_ratio'))
  ## If a colum specified in cols does not exist (e.g., typo in codes), 
  ## an informative error message will be prompted. 
  
}
```

#### Example 2: passing complex objects between milestones

Sometimes we want to carry more complex information forward, such as lists of results, which cannot be stored in the single-row output. In this case, we use `save_custom_data()`.

``` r
action_at_interim <- function(trial){
  
  locked_data <- trial$get_locked_data('interim analysis')
  
  ## fit coxph model on locked_data
  ## extract estimate and p-value of hazard ratio
  ## assume that hr is estimate, and pval is p-value
  trial$save(value = hr, name = 'pfs_interim_hazard_ratio')
  trial$save(value = pval, name = 'pfs_interim_p_value')
  
  ## assume that we also compute other values that may be useful in later milestone, 
  ## we save them in a list and call
  more_results <- list(numeric = 1, 
                       string = 'abc', 
                       list = list(e1 = 100, e2 = 'AA'))
  
  trial$save_custom_data(value = more_results, name = 'more_results')
  ## value of more_results can be accessed anywhere later by calling
  ## tmp <- trial$get(name = 'more_results')
  
  ## no return value is needed in an action function
  
}
```

In the final analysis, we can then retrieve both the saved trial outputs and the auxiliary information:

``` r
action_at_final <- function(trial){
  
  locked_data <- trial$get_locked_data('final analysis')
  
  ## extract outputs
  pfs_hr_interim <- trial$get_output(cols = 'pfs_interim_hazard_ratio')
  pfs_hr_pval <- trial$get_output(cols = 'pfs_interim_p_value')
  
  ## extract auxiliary information
  custom_list <- trial$get('more_results')
  
}
```

#### Example 3: defining auxiliary information before the trial runs

All member functions discussed in this section can be called before any milestone is triggered, as long as the `trial` object has been created by calling the function `trial()`. For example, we may want to define the level of family-wise error rate (FWER) once, and then use it later in interim and final analysis:

``` r
trial <- trial(...) ## initialize a trial with necessary arguments in ...
trial$add_arms(sample_ratio = c(1, 2, 1), pbo, low, high)

trial$save_custom_data(value = 0.025, name = 'fwer')
```

This auxiliary information can then be retrieved in later action functions to adjust boundaries or multiplicity procedures:

``` r
action_at_interim <- function(trial){
  
  locked_data <- trial$get_locked_data('interim analysis')
  
  ## compute p-values of PFS and OS at interim
  ## then extract FWER
  alpha <- trial$get(name = 'fwer')
  
  ## compute decision boundaries at interim based on FWER
  ## then test PFS and OS
  
  ## save testing results for interim
  # trial$save(...)
  
}

action_at_final <- function(trial){
  
  locked_data <- trial$get_locked_data('final analysis')
  
  ## compute p-values of PFS and OS at final
  ## then extract FWER and testing result at interim to adjust boundaries
  alpha <- trial$get(name = 'fwer')
  # interim_results <- trial$get_output(...)
  
  ## test PFS and OS again
  
  ## save testing results for final
  # trial$save(...)
  
}
```

In summary, `TrialSimulator` provides a structured saving system that allows users to:

-   record scalar results directly into trial outputs;

-   carry forward complex objects across milestones, and

-   accumulate data frames across multiple milestones.

This ensures smooth information flow between milestones and avoids ad-hoc workarounds such as global variables.

### Alter a trial via adaptation

Beyond analyzing data and saving results, one of the most powerful uses of action functions is the ability to adapt the ongoing trial.

Adaptation means that trial features—such as randomization ratios, sample size, trial duration, or active arms—can be modified in response to accumulated data. This reflects how many modern clinical trials are designed in practice, aiming to increase efficiency, ethical and cost balance.

In `TrialSimulator`, such adaptations are implemented directly within action functions. At a milestone, after retrieving the locked data and performing statistical analyses, the action function can call methods of the `trial` object to alter its internal state. These changes will then influence the subsequent course of the trial simulation.

Adaptations that are currently supported are summarized below:

+--------------------------+--------------------------------------------------------+---------------------------------------------------------+
| Adaptation               | Member Function                                        | Use Case                                                |
+==========================+========================================================+=========================================================+
| Remove an arm            | `trial$remove_arms()`                                  | dose selection; seamless design                         |
+--------------------------+--------------------------------------------------------+---------------------------------------------------------+
| Add an arm               | `trial$add_arms()`                                     | adaptive platform trials                                |
+--------------------------+--------------------------------------------------------+---------------------------------------------------------+
| Update sample ratio      | `trial$update_sample_ratio()`                          | response-adaptive design                                |
+--------------------------+--------------------------------------------------------+---------------------------------------------------------+
| Extend trial duration    | `trial$set_duration()` or add a event-driven milestone | actual patient or event accrual is slower than expected |
+--------------------------+--------------------------------------------------------+---------------------------------------------------------+
| Increase sample size[^1] | `trial$resize()`                                       | sample size reassessment                                |
+--------------------------+--------------------------------------------------------+---------------------------------------------------------+
| Eliminate sub-population | `trial$update_generator()`                             | enrichment design; data model changes over time         |
+--------------------------+--------------------------------------------------------+---------------------------------------------------------+

[^1]: experimental

Note that functions in the table above are member functions of `Trials` object. Users who are not familiar with the concept of classes may consider using their wrapper functions. The wrapper function of `trial$foo(...)` is `foo(trial, ...)`.

These adaptive features allow users to simulate complex, data-driven designs where trial conduct is not static but evolves in response to accumulating evidence. At the time of writing, the vignette dedicated to trial adaptation is still under development. Detailed examples will be provided in that vignette.

---
title: "Simulate Trials with Dynamic Treatment Switching"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Simulate Trials with Dynamic Treatment Switching}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
editor_options: 
  markdown: 
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  cache.path = 'cache/dynamicTreatmentSwitching/',
  comment = '#>',
  dpi = 300,
  out.width = '100%'
)
```

```{r setup, echo = FALSE, message = FALSE}
library(TrialSimulator)
library(mvtnorm)
library(dplyr)
library(kableExtra)
```

The `TrialSimulator` package can handle three types of crossover
designs, of which dynamic treatment switching is common in clinical
trial, but simulating it is usually an unmet need due to its complexity.
This vignette demonstrates how to simulate a trial where all or a subset
of patients are offered the option of treatment switching over time.
This happens in settings such as:

-   crossover after progression, for example control patients switching
    to active treatment;
-   rescue medication after non-response;
-   dose escalation or dose reduction based on response or toxicity;
-   switching after a adverse event to a different regimen.

These examples differ clinically, but they share the same simulation
challenge: the patient does not follow a single static treatment path
from baseline to the end of follow-up. In many cases, switching is not
just a change in treatment label. It changes the patient’s future
outcome trajectory. For example, switching may alter post-switch
survival, response probability, or adverse-event risk, while leaving
earlier outcomes unchanged. To address this challenge in simulation,
`TrialSimulator` offers to define a `regimen` object in which we can
implement custom mechanism for crossover.

Note that there are many other types of crossover that cannot be
simulated using the approach discussed in this vignette. For example,
patients may be reassigned after the arm is dropped. In such a scenario,
the availability of alternative regimen depend on analysis result of
unblinded data and can only happen at interim. We will show how to do
that in other vignette. For crossover design with wash-out periods,
please refer to this
[vignette](https://zhangh12.github.io/TrialSimulator/articles/crossoverWashout.html).

## A Common Structure

Most switching designs can be reduced to three questions:

-   Who switches and what's the new regimen?
-   When does switching occur?
-   What changes after switching?

This leads naturally to a two-stage simulation strategy:

1.  generate patient data under the originally assigned arm for all
    enrolled patients;
2.  apply a patient-level switching mechanism that updates post-switch
    outcomes.

This is the core idea behind `regimen` in `TrialSimulator`.

## Why `regimen` uses three functions?

The `regimen` interface splits treatment switching into three
components:

-   `what()`, a treatment allocator for switchers
-   `when()`, a time selector for switching
-   `how()`, a data modifier altering post-switch outcome trajectory

A single large switching function could do all of this, but it would mix
different responsibilities and become difficult to reuse. In practice,
these components often vary separately across scenarios. For example,
one may keep the same switching population but compare different
switching times, or keep selection and timing fixed while testing
different post-switch assumptions. The three-function design keeps these
choices explicit.

All the three functions take a data frame `patient_data` as input
argument. It is the data generated automatically by calling `generator`
of endpoints for the whole life-cycle of the trial. The package will
figure out patients who are eligible for the three functions, and the
right timing to call.

## The Role of `what()`

This custom function is responsible for identifying switchers and
assigning their new treatment regimen. Some typical rules that we can
consider include:

-   only patients in a control arm can switch;
-   only non-responders receive rescue therapy;
-   switching destination depends on subgroup, biomarker, etc.

The output of `what()` consists of two columns `patient_id` and
`new_treatment`. Only switchers are returned. Any patients with `NA` as
their `new_treatment` are omitted. Patients not selected for switching
simply remain under their original treatment path and outcome
trajectory.

## The Role of `when()`

This custom function assigns a switching time to each selected patient.
A patient may switch:

-   at progression,
-   if not respond,
-   at a scheduled visit,
-   once a composite condition is met, etc.

Switching time can depend on the full outcome trajectory. For example,
patient may have a higher chance of switching (e.g., for rescue) when
being closer to the onset of major event (e.g., death). Keeping timing
in `when()` makes scenario analysis much easier. The same switching
population can be reused under different timing assumptions.

Note that patients whose data is passed to `when()` is a subset of
patients whose data is passed to `what()`. The package governs this to
prevent logic error and avoid unnecessary computation.

This function returns a data frame of two columns `patient_id` and
`switch_time` measured from enrollment. All selected patients must be
assigned a switching time, otherwise package will throw an error.

## The Role of `how()`

This custom function modifies patient data to reflect the consequences
of switching. In practice, switching does not necessarily mean replacing
the patient with a whole new patient from another arm. More often, we
wants to

-   keep pre-switch history unchanged;
-   modify only what happens after the switch;
-   update only selected endpoints;
-   allow different post-switch behavior for different new treatments,
    etc.

So `how()` is not just a label change. It is a controlled post-switch
data update. Depending on the simulation goal, `how()` could:

-   extend residual survival time after switching;
-   change the post-switch hazard rate;
-   replace a future event time with a draw from another model;
-   modify response probability after switching;
-   leave some endpoints unchanged if they are not used in formal
    testing or decision making, etc.

This function returns a data frame of `patient_id` and columns of
endpoints that are altered. We do not have to return an endpoint if it
is not updated for any patient. For an endpoint that is updated for a
subset of switchers, simply set the unchanged cells to `NA` and the
package will ignore them. We can also fill those cells with their
original values when being passed to `how()`. Note that we should never 
apply dropout or censoring manually in `how()` because `TrialSimulator` 
will handle that automatically when triggering milestones and prepare 
locked data in action functions. 

The three-functions design in `regimen` provides a useful balance
between structure and flexibility. The interface mirrors the way
investigators naturally describe switching designs. It encourages
modular thinking because each part can be changed independently.
Sensitivity analysis becomes straightforward under this framework as we
can modify one function at a time. Another benefit is that more than one
set of treatment allocator, time selector and data modifier are accepted
for simulating multi-stage crossover. It also avoids overloading the arm
definitions and supports partial data updates.

This flowchart illustrates how `regimen` is integrated with other
modules in `TrialSimulator`.

![](simulation_flowchart.svg)

## Implement Dynamic Treatment Switching

Dynamic treatment switching can be added to any simulation codes
implemented with `TrialSimulator`. So we only show code snippets in this
vignette to illustrate what do the three functions look like in several
examples.

In all cases, we need to implement functions to initialize an `regimen`
object

```{r eval=FALSE}
treatment_allocator <- function(patient_data){...}
time_selector <- function(patient_data){...}
data_modifier <- function(patient_data){...}

regimen <- regimen(treatment_allocator, time_selector, data_modifier)
```

All three functions must have `patient_data` as argument otherwise
`TrialSimulator` will throw exception. The regime is then registered to
a trial before any arm is added to it.

```{r eval=FALSE}
trial <- trial(...)
trial$add_regime(regimen)
```

Any codes like below will trigger an error message because
`trial$add_arms(...)` enrolls patients into a trial immediately but we
need to make `regimen` ready before then.

```{r eval=FALSE}
trial <- trial(...)
trial$add_arms(sample_ratio, soc, low_dose, high_dose)
trial$add_regime(regimen)
```

```{r echo=FALSE, error=TRUE}
msg <- tryCatch(
  {
    stop(' Member function trial$add_regimen() must be called before trial$add_arms(). ', 
         'A good practice is to call trial$add_regimen() immediately after trial() is executed. ')
    NULL
  },
  error = function(e) {
    cat('Error in trial$add_regimen(regimen) :\n', 
        e$message)
  }
)
```

#### Example 1: Crossover after progression

In oncology trials, patients initially randomized to standard of care
(`soc`) may be allowed to switch to active treatment after disease
progression, conditional on a patient-level event. If we assume 30% of
the patients are assigned to low dose while 40% of the patients are
assigned to high dose, we can implement the `what()` function like

```{r eval=FALSE}
treatment_allocator <- function(patient_data){
  ## add break point to develop and debug
  # browser()
  switch_to <- sample(c('low', 'high', 'stay'), nrow(patient_data), 
                        replace = TRUE, prob = c(.3, .4, .3))
  data.frame(
    patient_id = patient_data$patient_id,
    new_treatment =
      dplyr::case_when(
        # patient die before progression cannot switch
        patient_data$os == patient_data$pfs ~ NA_character_, 
        patient_data$arm == 'placebo' & switch_to == 'low' ~ 'low dose',
        patient_data$arm == 'placebo' & switch_to == 'high' ~ 'high dose',
        TRUE ~ NA_character_
      )
  )

}
```

The `when()` function simply returned the progression time, i.e., switching does not relies on event after the switching time. 

```{r eval=FALSE}
time_selector <- function(patient_data){
  ## add break point to develop and debug
  # browser()
  data.frame(
    patient_id = patient_data$patient_id,
    ## all patient in patient_data progress before die
    ## thus pfs < os and can switch. 
    ## See treatment_allocator()
    switch_time = patient_data$pfs
  )
  
}
```

Another common way to specify the switching time is picking a time point between `pfs` and `os`, i.e.,

```{r eval=FALSE}
time_selector <- function(patient_data){
  ## add break point to develop and debug
  # browser()
  data.frame(
    patient_id = patient_data$patient_id,
    ## all patient in patient_data progress before die
    ## thus pfs < os and can switch. 
    ## See treatment_allocator()
    switch_time = runif(nrow(patient_data), min = patient_data$pfs, max = patient_data$os)
  )
  
}
```

In the `how()` function, we extend residual survival time by a factor
using the causal accelerated failure time model depending on the
assigned dose. Here we only update the overall survival `os` and leave
other endpoints (e.g., response status) or variables unchanged, assuming
that we won't use their values after switching when summarizing the
simulation.

```{r eval=FALSE}
data_modifier <- function(patient_data){
  ## add break point to develop and debug
  # browser()
  f <- ifelse(patient_data$new_treatment == 'low dose', 1.1, 1.15)
  data.frame(
    patient_id = patient_data$patient_id,
    ## other_endpoint = ..., 
    os = patient_data$switch_time + f * (patient_data$os - patient_data$switch_time)
  )

}
```


#### Example 2: Crossover for non-responsers

Here we consider a trial of a placebo arm and two active treatment arms (low and high dose). Patients in the low dose arm who do not respond are switched to a higher dose. 

```{r eval=FALSE}
treatment_allocator <- function(patient_data){
  ## add break point to develop and debug
  # browser()
  data.frame(
    patient_id = patient_data$patient_id,
    new_treatment =
      dplyr::case_when(
        patient_data$arm == 'low dose' & patient_data$response == 0 ~ 'high dose',
        TRUE ~ NA_character_
      )
  )

}
```

When implementing time selector for the function `when()`, the switching time is 
set to the readout time

```{r eval=FALSE}
time_selector <- function(patient_data){
  ## add break point to develop and debug
  # browser()
  data.frame(
    patient_id = patient_data$patient_id,
    ## all patient in patient_data progress before die
    ## thus pfs < os and can switch
    switch_time = patient_data$response_readout
  )
  
}
```

### Example 3: Crossover when condition deteriorates

Now we assume that all patients in the placebo arm can crossover when their condition deteriorate significantly. Specifically, the switching time is 1 month before a patient dies. If the overall survival is shorter than a month, the switching time is set to `0.9 * os`. 

```{r}
treatment_allocator <- function(patient_data){
  ## add break point to develop and debug
  # browser()
  data.frame(
    patient_id = patient_data$patient_id,
    new_treatment =
      dplyr::case_when(
        patient_data$arm == 'placebo' ~ 'new treatment',
        TRUE ~ NA_character_
      )
  )

}
```

```{r eval=FALSE}
time_selector <- function(patient_data){
  ## add break point to develop and debug
  # browser()
  data.frame(
    patient_id = patient_data$patient_id,
    ## all patient in patient_data progress before die
    ## thus pfs < os and can switch
    switch_time = ifelse(patient_data$os <= 1, .9 * patient_data$os, patient_data$os - 1)
  )
  
}
```

### Example 4: Crossover multiple times

We can implement multiple rounds of crossover for each patient. To do this, simply provide `regimen()` with a list of treatment allocators for `what`, a list of time selectors for `when`, and a list of data modifiers for `how`. 

```{r eval=FALSE}
what <- list(allocator1, allocator2, allocator3)
when <- list(selector1, selector2, selector3)
how <- list(modifier1, modifier2, modifier3)

regimen <- regimen(what, when, how)

trial <- trial(...)
trial$add_regimen(regimen)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GroupSequentialTest.R
\docType{class}
\name{GroupSequentialTest}
\alias{GroupSequentialTest}
\title{Class of GroupSequentialTest}
\description{
Perform group sequential test for a single endpoint based on sequential
one-sided p-values at each stages. Selected alpha spending functions,
including user-defined functions, are supported. Boundaries are calculated
with `rpact`. At the final analysis, adjustment can be applied for
over-running or under-running trial where observed final information is
greater or lower than the planned maximum information. See
Wassmer & Brannath, 2016, p78f. The test is based on p-values not z
statistics because it is easier to not handling direction of alternative
hypothesis in current implementation. In addition, only one-sided test is
supported which should be sufficient for common use in clinical design.
}
\examples{
## Note: examples showed here replicate the results from
## https://www.rpact.org/vignettes/planning/rpact_boundary_update_example/

## Example 1. Generate boundaries for a pre-fix group sequential design
gst <- GroupSequentialTest$new(
  alpha = .025, alpha_spending = 'asOF',
  planned_max_info = 387)

## without giving p-values, boundaries are returned without actual testing
gst$test(observed_info = c(205, 285, 393), is_final = c(FALSE, FALSE, TRUE))
gst

## Example 2. Calculate boundaries with observed information at stages
## No p-values are provided

## get an error without resetting an used object
try( gst$test(observed_info = 500, is_final = FALSE) )

## reset the object for re-use
gst$reset()
gst$test(observed_info = c(205, 285, 393), is_final = c(FALSE, FALSE, TRUE))
gst

## Example 3. Test stagewise p-values sequentially
gst$reset()

gst$test(observed_info = 205, is_final = FALSE, p_values = .09)
gst$test(285, FALSE, .006)

## print testing trajectory by now
gst

gst$test(393, TRUE, .002)

## print all testing trajectory
gst

## you can also test all stages at once
## the result is the same as calling test() for each of the stages
gst$reset()
gst$test(c(205, 285, 393), c(FALSE, FALSE, TRUE), c(.09, .006, .002))
gst

## Example 4. use user-define alpha spending
gst <- GroupSequentialTest$new(
  alpha = .025, alpha_spending = 'asUser',
  planned_max_info = 387)

gst$test(
  observed_info = c(205, 285, 393),
  is_final = c(FALSE, FALSE, TRUE),
  alpha_spent = c(.005, .0125, .025))
gst

}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GroupSequentialTest-new}{\code{GroupSequentialTest$new()}}
\item \href{#method-GroupSequentialTest-get_name}{\code{GroupSequentialTest$get_name()}}
\item \href{#method-GroupSequentialTest-get_alpha}{\code{GroupSequentialTest$get_alpha()}}
\item \href{#method-GroupSequentialTest-set_alpha_spending}{\code{GroupSequentialTest$set_alpha_spending()}}
\item \href{#method-GroupSequentialTest-get_alpha_spending}{\code{GroupSequentialTest$get_alpha_spending()}}
\item \href{#method-GroupSequentialTest-get_max_info}{\code{GroupSequentialTest$get_max_info()}}
\item \href{#method-GroupSequentialTest-set_max_info}{\code{GroupSequentialTest$set_max_info()}}
\item \href{#method-GroupSequentialTest-get_stage}{\code{GroupSequentialTest$get_stage()}}
\item \href{#method-GroupSequentialTest-reset}{\code{GroupSequentialTest$reset()}}
\item \href{#method-GroupSequentialTest-set_trajectory}{\code{GroupSequentialTest$set_trajectory()}}
\item \href{#method-GroupSequentialTest-get_trajectory}{\code{GroupSequentialTest$get_trajectory()}}
\item \href{#method-GroupSequentialTest-get_stage_level}{\code{GroupSequentialTest$get_stage_level()}}
\item \href{#method-GroupSequentialTest-test_one}{\code{GroupSequentialTest$test_one()}}
\item \href{#method-GroupSequentialTest-test}{\code{GroupSequentialTest$test()}}
\item \href{#method-GroupSequentialTest-print}{\code{GroupSequentialTest$print()}}
\item \href{#method-GroupSequentialTest-clone}{\code{GroupSequentialTest$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-new"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-new}{}}}
\subsection{Method \code{new()}}{
initialize a group sequential test. Now only support one-sided test
based on p-values.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$new(
  alpha = 0.025,
  alpha_spending = c("asP", "asOF", "asUser"),
  planned_max_info,
  name = "H0",
  silent = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{alpha}}{familywise error rate}

\item{\code{alpha_spending}}{alpha spending function. Use \code{"asUser"}
if custom alpha spending schedule is used.}

\item{\code{planned_max_info}}{integer. Planned maximum number of patients for
non-tte endpoints or number of events for tte endpoints}

\item{\code{name}}{character. Name of the hypothesis, e.g. endpoint, subgroup,
etc. Optional.}

\item{\code{silent}}{\code{TRUE} if muting all messages.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-get_name"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-get_name}{}}}
\subsection{Method \code{get_name()}}{
get name of hypothesis
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$get_name()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-get_alpha"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-get_alpha}{}}}
\subsection{Method \code{get_alpha()}}{
get overall alpha
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$get_alpha()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-set_alpha_spending"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-set_alpha_spending}{}}}
\subsection{Method \code{set_alpha_spending()}}{
set alpha spending function. This is useful when set 'asUser' at the
final stage to adjust for an under- or over-running trial.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$set_alpha_spending(asf)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{asf}}{character of alpha spending function.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-get_alpha_spending"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-get_alpha_spending}{}}}
\subsection{Method \code{get_alpha_spending()}}{
return character of alpha spending function
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$get_alpha_spending()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-get_max_info"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-get_max_info}{}}}
\subsection{Method \code{get_max_info()}}{
return planned maximum information
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$get_max_info()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-set_max_info"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-set_max_info}{}}}
\subsection{Method \code{set_max_info()}}{
set planned maximum information. This is used at the final stage
to adjust for an under- or over-running trial.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$set_max_info(obs_max_info)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{obs_max_info}}{integer. Maximum information, which could be
observed number of patients or events at the final stage.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-get_stage"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-get_stage}{}}}
\subsection{Method \code{get_stage()}}{
get current stage.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$get_stage()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-reset"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-reset}{}}}
\subsection{Method \code{reset()}}{
an object of class \code{GroupSequentialTest} is designed to be used
sequentially by calling \code{GroupSequentialTest$test}. When all
planned tests are performed, no further analysis could be done. In that
case keep calling \code{GroupSequentialTest$test} will trigger an error.
To reuse the object for a new set of staged p-values, call this function
to reset the status to stage 1. See examples. This implementation can
prevent the error that more than the planned number of stages are tested.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$reset()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-set_trajectory"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-set_trajectory}{}}}
\subsection{Method \code{set_trajectory()}}{
save testing result at current stage
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$set_trajectory(result, is_final = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{result}}{a data frame storing testing result at a stage.}

\item{\code{is_final}}{logical. \code{TRUE} if final test for the hypothesis,
\code{FALSE} otherwise.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-get_trajectory"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-get_trajectory}{}}}
\subsection{Method \code{get_trajectory()}}{
return testing trajectory until current stage. This function can be
called at any stage. See examples.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$get_trajectory()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-get_stage_level"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-get_stage_level}{}}}
\subsection{Method \code{get_stage_level()}}{
compute boundaries given current (potentially updated) settings. It
returns different values if settings are changed over time.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$get_stage_level()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-test_one"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-test_one}{}}}
\subsection{Method \code{test_one()}}{
test a hypothesis with the given p-value at current stage
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$test_one(
  p_value,
  is_final,
  observed_info,
  alpha_spent = NA_real_
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{p_value}}{numeric. A p-value.}

\item{\code{is_final}}{logical. \code{TRUE} if this test is carried out for
the final analysis.}

\item{\code{observed_info}}{integer. Observed information at current stage. It
can be the number of samples (non-tte) or number of events (tte) at test.
If the current stage is final, observed_info will be used to update
planned_max_info, the alpha spending function (\code{typeOfDesign}
in \code{rpact}) will be updated to \code{'asUser'}, and the argument
\code{userAlphaSpending} will be used when calling
\code{rpact::getDesignGroupSequential}.}

\item{\code{alpha_spent}}{numeric if \code{alpha_spending = "asUser"}. It must
be between 0 and \code{alpha}, the overall alpha of the test.
\code{NA_real_} for other alpha spending functions \code{"asOF"} and
\code{"asP"}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-test"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-test}{}}}
\subsection{Method \code{test()}}{
Carry out test based on group sequential design. If \code{p_values}
is \code{NULL}, dummy values will be use and boundaries are calculated
for users to review.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$test(
  observed_info,
  is_final,
  p_values = NULL,
  alpha_spent = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{observed_info}}{a vector of integers, observed information at stages.}

\item{\code{is_final}}{logical vector. \code{TRUE} if the test is for the final
analysis.}

\item{\code{p_values}}{a vector of p-values. If specified, its length should
equal to the length of \code{observed_info}.}

\item{\code{alpha_spent}}{accumulative alpha spent at observed information.
It is a numeric vector of values between 0 and 1, and of length that
equals \code{length(observed_info)} if alpha-spending
function is \code{"asUser"}. Otherwise \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-print"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-print}{}}}
\subsection{Method \code{print()}}{
generic function for \code{print}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GroupSequentialTest-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GroupSequentialTest-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GroupSequentialTest$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_duration.R
\name{set_duration}
\alias{set_duration}
\title{Extending Duration of a Trial}
\usage{
set_duration(trial, duration)
}
\arguments{
\item{trial}{a trial object returned by \code{trial()}.}

\item{duration}{new duration of a trial. It must be greater than the current
duration.}
}
\description{
set trial duration in an adaptive designed trial. New duration must be
longer than the old one. All patients enrolled
before resetting the duration are truncated (non-TTE endpoints) or
censored (TTE endpoints) at the original duration. This helps maintain
proper type I error or family-wise error rate and control multiplicity when
conducting testing statistics. For more
details of why this is necessary, please refer to
\href{https://onlinelibrary.wiley.com/doi/10.1002/pst.1926}{Jorgens et al. 2019}.

Note that this function should only be called within action functions of
milestones. It is users' responsibility to ensure that and
\code{TrialSimulator} has no way to track it.

This is a user-friendly wrapper of the member function of trial, i.e.,
\code{Trials$set_duration()}, which is used in vignettes. Users who are not
familiar with the concept of classes may consider using this wrapper
directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Trials.R
\docType{class}
\name{Trials}
\alias{Trials}
\title{Class of Trial}
\description{
Create a class of trial.

Public methods in this R6 class are used in developing
this package. Thus, we have to export the whole R6 class which exposures all
public methods. However, only the public methods in the list below are
useful to end users.

\itemize{
\item \code{$set_duration()} set duration of a trial. This function can be
used to extend duration under adaptive designs.
\item \code{$resize()} set maximum sample size of a trial. This function can
be used to increase sample size under adaptive designs (e.g., sample size
reassessment).
\item \code{$remove_arms()} drop arms from a trial. This function can be
used in adaptive designs, e.g., dose selection, enrichment design, etc.
\item \code{$update_sample_ratio()} change sample ratio of arm. This function
can be used under adaptive designs, e.g., response-adaptive design, etc.
\item \code{$update_generator()} change endpoint generator of arm. This
function can be used in enrichment design.
\item \code{$add_arms()} add arms to a trial. This function is used to add
arms to a newly defined trial, or add arms under adaptive design, e.g.,
dose-ranging, etc.
\item \code{$add_regimen()} register a \code{regimen} object to a trial.
Must be called before \code{$add_arms()}.
\item \code{$get_locked_data()} request for data snapshot at a milestone.
Calling this function is recommended as the first action in any action
function as long as trial data is needed in statistical analysis or decision
making.
\item \code{$save()} save intermediate result for simulation summary.
Results across multiple replicates of simulation are saved, which can be
retrieved by calling \code{get_output()} anytime.
\item \code{$bind()} row bind and save intermediate results across
milestones if those results are data frames of similar formats. The life
cycle of the save results is within a single replicate of simulation and
is reset to NULL in next simulated trial. Saved results
can be retrieved by calling \code{get()} anytime.
\item \code{$save_custom_data()} save intermediate results of any format.
The life cycle of the saved result is within a single replicate of simulation
and is reset to NULL in next simulated trial. Saved results can be retrieved
by calling \code{get()} anytime.
\item \code{$get()} retrieve intermediate results saved by calling functions
\code{save_custom_data()} or \code{bind()}.
\item \code{$get_output()} retrieve intermediate results saved by calling
function \code{save()}.
\item \code{$dunnettTest()} perform Dunnett's test.
\item \code{$closedTest()} perform combination test based on Dunnett's test.
}
}
\examples{
# Instead of using Trials$new, please use trial(), a user-friendly
# wrapper. See examples in ?trial.


## ------------------------------------------------
## Method `Trials$independentIncrement`
## ------------------------------------------------


\dontrun{
trial$independentIncrement(Surv(pfs, pfs_event) ~ arm, 'pbo',
                           listener$get_milestone_names(),
                           'less', 'oracle')
}

## ------------------------------------------------
## Method `Trials$dunnettTest`
## ------------------------------------------------

\dontrun{
trial$dunnettTest(Surv(pfs, pfs_event) ~ arm, 'pbo', c('high dose', 'low dose'),
                  listener$get_milestone_names(), 'default')
}


## ------------------------------------------------
## Method `Trials$closedTest`
## ------------------------------------------------

\dontrun{
dt <- trial$dunnettTest(
  Surv(pfs, pfs_event) ~ arm,
  placebo = 'pbo',
  treatments = c('high dose', 'low dose'),
  milestones = c('dose selection', 'interim', 'final'),
  data.frame(pbo = c(100, 160, 80),
             low = c(100, 160, 80),
             high = c(100, 160, 80),
             row.names = c('dose selection', 'interim', 'final'))

trial$closedTest(dt, treatments = c('high dose', 'low dose'),
                 milestones = c('interim', 'final'),
                 alpha = 0.025, alpha_spending = 'asOF')
}

}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Trials-new}{\code{Trials$new()}}
\item \href{#method-Trials-get_trial_data}{\code{Trials$get_trial_data()}}
\item \href{#method-Trials-get_duration}{\code{Trials$get_duration()}}
\item \href{#method-Trials-set_duration}{\code{Trials$set_duration()}}
\item \href{#method-Trials-set_enroller}{\code{Trials$set_enroller()}}
\item \href{#method-Trials-get_enroller}{\code{Trials$get_enroller()}}
\item \href{#method-Trials-set_dropout}{\code{Trials$set_dropout()}}
\item \href{#method-Trials-get_dropout}{\code{Trials$get_dropout()}}
\item \href{#method-Trials-roll_back}{\code{Trials$roll_back()}}
\item \href{#method-Trials-remove_arms}{\code{Trials$remove_arms()}}
\item \href{#method-Trials-update_sample_ratio}{\code{Trials$update_sample_ratio()}}
\item \href{#method-Trials-update_generator}{\code{Trials$update_generator()}}
\item \href{#method-Trials-resize}{\code{Trials$resize()}}
\item \href{#method-Trials-add_arms}{\code{Trials$add_arms()}}
\item \href{#method-Trials-add_regimen}{\code{Trials$add_regimen()}}
\item \href{#method-Trials-get_regimen}{\code{Trials$get_regimen()}}
\item \href{#method-Trials-has_regimen}{\code{Trials$has_regimen()}}
\item \href{#method-Trials-get_name}{\code{Trials$get_name()}}
\item \href{#method-Trials-get_description}{\code{Trials$get_description()}}
\item \href{#method-Trials-get_arms}{\code{Trials$get_arms()}}
\item \href{#method-Trials-get_arms_name}{\code{Trials$get_arms_name()}}
\item \href{#method-Trials-get_number_arms}{\code{Trials$get_number_arms()}}
\item \href{#method-Trials-has_arm}{\code{Trials$has_arm()}}
\item \href{#method-Trials-get_an_arm}{\code{Trials$get_an_arm()}}
\item \href{#method-Trials-get_sample_ratio}{\code{Trials$get_sample_ratio()}}
\item \href{#method-Trials-get_number_patients}{\code{Trials$get_number_patients()}}
\item \href{#method-Trials-get_number_enrolled_patients}{\code{Trials$get_number_enrolled_patients()}}
\item \href{#method-Trials-get_number_unenrolled_patients}{\code{Trials$get_number_unenrolled_patients()}}
\item \href{#method-Trials-get_stratum_queue}{\code{Trials$get_stratum_queue()}}
\item \href{#method-Trials-get_randomization_queue}{\code{Trials$get_randomization_queue()}}
\item \href{#method-Trials-get_enroll_time}{\code{Trials$get_enroll_time()}}
\item \href{#method-Trials-enroll_patients}{\code{Trials$enroll_patients()}}
\item \href{#method-Trials-set_current_time}{\code{Trials$set_current_time()}}
\item \href{#method-Trials-get_current_time}{\code{Trials$get_current_time()}}
\item \href{#method-Trials-get_event_tables}{\code{Trials$get_event_tables()}}
\item \href{#method-Trials-get_data_lock_time_by_event_number}{\code{Trials$get_data_lock_time_by_event_number()}}
\item \href{#method-Trials-get_data_lock_time_by_enrollment}{\code{Trials$get_data_lock_time_by_enrollment()}}
\item \href{#method-Trials-get_data_lock_time_by_calendar_time}{\code{Trials$get_data_lock_time_by_calendar_time()}}
\item \href{#method-Trials-get_locked_data}{\code{Trials$get_locked_data()}}
\item \href{#method-Trials-get_locked_data_name}{\code{Trials$get_locked_data_name()}}
\item \href{#method-Trials-get_event_number}{\code{Trials$get_event_number()}}
\item \href{#method-Trials-save_milestone_time}{\code{Trials$save_milestone_time()}}
\item \href{#method-Trials-get_milestone_time}{\code{Trials$get_milestone_time()}}
\item \href{#method-Trials-lock_data}{\code{Trials$lock_data()}}
\item \href{#method-Trials-event_plot}{\code{Trials$event_plot()}}
\item \href{#method-Trials-censor_trial_data}{\code{Trials$censor_trial_data()}}
\item \href{#method-Trials-save}{\code{Trials$save()}}
\item \href{#method-Trials-bind}{\code{Trials$bind()}}
\item \href{#method-Trials-save_custom_data}{\code{Trials$save_custom_data()}}
\item \href{#method-Trials-get_custom_data}{\code{Trials$get_custom_data()}}
\item \href{#method-Trials-get}{\code{Trials$get()}}
\item \href{#method-Trials-get_output}{\code{Trials$get_output()}}
\item \href{#method-Trials-mute}{\code{Trials$mute()}}
\item \href{#method-Trials-tidy_output}{\code{Trials$tidy_output()}}
\item \href{#method-Trials-independentIncrement}{\code{Trials$independentIncrement()}}
\item \href{#method-Trials-dunnettTest}{\code{Trials$dunnettTest()}}
\item \href{#method-Trials-closedTest}{\code{Trials$closedTest()}}
\item \href{#method-Trials-get_seed}{\code{Trials$get_seed()}}
\item \href{#method-Trials-print}{\code{Trials$print()}}
\item \href{#method-Trials-get_snapshot_copy}{\code{Trials$get_snapshot_copy()}}
\item \href{#method-Trials-make_snapshot}{\code{Trials$make_snapshot()}}
\item \href{#method-Trials-make_arms_snapshot}{\code{Trials$make_arms_snapshot()}}
\item \href{#method-Trials-reset}{\code{Trials$reset()}}
\item \href{#method-Trials-set_arm_added_time}{\code{Trials$set_arm_added_time()}}
\item \href{#method-Trials-get_arm_added_time}{\code{Trials$get_arm_added_time()}}
\item \href{#method-Trials-set_arm_removal_time}{\code{Trials$set_arm_removal_time()}}
\item \href{#method-Trials-get_arm_removal_time}{\code{Trials$get_arm_removal_time()}}
\item \href{#method-Trials-get_stratification_factors}{\code{Trials$get_stratification_factors()}}
\item \href{#method-Trials-has_stratification_factors}{\code{Trials$has_stratification_factors()}}
\item \href{#method-Trials-clone}{\code{Trials$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-new"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-new}{}}}
\subsection{Method \code{new()}}{
initialize a trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$new(
  name,
  n_patients,
  duration,
  description = name,
  seed = NULL,
  enroller,
  dropout = NULL,
  stratification_factors = NULL,
  silent = FALSE,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{character. Name of trial. Usually, hmm..., useless.}

\item{\code{n_patients}}{integer. Maximum (and initial) number of patients
could be enrolled when planning the trial. It can be altered adaptively
during a trial.}

\item{\code{duration}}{Numeric. Trial duration. It can be altered adaptively
during a trial.}

\item{\code{description}}{character. Optional for description of the trial. By
default it is set to be trial's \code{name}. Usually useless.}

\item{\code{seed}}{random seed. If \code{NULL}, seed is set for each simulated
trial automatically and saved in output. It can be retrieved in the
\code{seed} column in \code{$get_output()}. Setting it to be \code{NULL}
is recommended. For debugging, set it to a specific integer.}

\item{\code{enroller}}{a function returning a vector enrollment time for
patients. Its first argument \code{n} is the number of enrolled patients.
Set it to \code{StaggeredRecruiter} can handle most of the use cases.
See \code{?TrialSimulator::StaggeredRecruiter} for more information.}

\item{\code{dropout}}{a function returning a vector of dropout time for patients.
It can be any random number generator with first argument \code{n},
the number of enrolled patients. Usually \code{rexp} if dropout rate
is set at a single time point, or \code{rweibull} if dropout rates are
set at two time points. See \code{?TrialSimulator::weibullDropout}.}

\item{\code{stratification_factors}}{character. Names of baseline characteristics
to define stratums in stratified permuted block randomization.
Stratification factors must be defined in \code{endpoint()} with
\code{readout = 0}. As a natural assumption for randomized trial,
\code{TrialSimulator} assumes that the baseline
characteristics share the same distribution across arms, but endpoints
can have same or different distributions given baseline characteristics.
\code{NULL} by default, i.e., unstratified permuted block randomization is
executed.}

\item{\code{silent}}{logical. \code{TRUE} to mute messages. However, warning
message is still displayed.}

\item{\code{...}}{(optional) arguments of \code{enroller} and \code{dropout}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_trial_data"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_trial_data}{}}}
\subsection{Method \code{get_trial_data()}}{
return trial data of enrolled patients at the time of this
function is called
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_trial_data()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_duration"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_duration}{}}}
\subsection{Method \code{get_duration()}}{
return maximum duration of a trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_duration()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-set_duration"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-set_duration}{}}}
\subsection{Method \code{set_duration()}}{
set trial duration in an adaptive designed trial. All patients enrolled
before resetting the duration are truncated (non-tte endpoints) or
censored (tte endpoints) at the original duration. Remaining patients
are re-randomized. New duration must be longer than the old one.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$set_duration(duration)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{duration}}{new duration of a trial. It must be greater than the
current duration.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-set_enroller"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-set_enroller}{}}}
\subsection{Method \code{set_enroller()}}{
set recruitment curve when initialize a trial.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$set_enroller(func, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{func}}{function to generate enrollment time. It can be built-in
function like `rexp` or customized functions like `StaggeredRecruiter`.}

\item{\code{...}}{(optional) arguments for \code{func}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_enroller"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_enroller}{}}}
\subsection{Method \code{get_enroller()}}{
get function of recruitment curve
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_enroller()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-set_dropout"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-set_dropout}{}}}
\subsection{Method \code{set_dropout()}}{
set distribution of drop out time. This can be done when initialize a
trial, or when updating a trial in adaptive design.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$set_dropout(func, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{func}}{function to generate dropout time. It can be built-in
function like `rexp` or customized functions.}

\item{\code{...}}{(optional) arguments for \code{func}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_dropout"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_dropout}{}}}
\subsection{Method \code{get_dropout()}}{
get generator of dropout time
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_dropout()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-roll_back"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-roll_back}{}}}
\subsection{Method \code{roll_back()}}{
roll back data to current time of trial. By doing so,
\code{Trial$trial_data} will be cut at current time, and data after then
are deleted. However, \code{Trial$enroll_time} after current time are
kept unchanged because that is planned enrollment curve.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$roll_back()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-remove_arms"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-remove_arms}{}}}
\subsection{Method \code{remove_arms()}}{
remove arms from a trial. \code{enroll_patients()} will be called
at the end of this function to enroll all remaining patients after
\code{Trials$get_current_time()}, i.e. no more unenrolled patients
could be randomized to removed arms. This function may be used with
futility analysis, dose selection, enrichment analysis (sub-population)
or interim analysis (early stop for efficacy).

Note that this function should only be called within action functions.
It is users' responsibility to ensure it and \code{TrialSimulator} has
no way to track this.
In addition, data of the removed arms are censored or truncated by
the time of arm removal.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$remove_arms(arms_name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arms_name}}{character vector. Name of arms to be removed.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-update_sample_ratio"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-update_sample_ratio}{}}}
\subsection{Method \code{update_sample_ratio()}}{
update sample ratios of arms. This could happen after an arm is added
or removed. Note that we may update sample ratios of unaffected arms as
well. Once sample ratio is updated, trial data should be rolled back with
updated randomization queue. Data of unenrolled patients are
re-sampled as well.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$update_sample_ratio(arm_names, sample_ratios)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arm_names}}{character vector. Name of arms.}

\item{\code{sample_ratios}}{numeric vector. New sample ratios of arms. If
sample ratio is a whole number, the permuted block randomization is
adopted; otherwise, \code{sample()} will be used instead, which can
cause imbalance between arms by chance. However, this is fine for
simulation.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-update_generator"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-update_generator}{}}}
\subsection{Method \code{update_generator()}}{
update endpoint generator in an arm
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$update_generator(arm_name, endpoint_name, generator, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arm_name}}{character. Name of an arm.}

\item{\code{endpoint_name}}{character. A vector of endpoint names whose
generator is updated.}

\item{\code{generator}}{a random number generation (RNG) function.
See \code{generator} of \code{endpoint()}.}

\item{\code{...}}{optional arguments for \code{generator}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-resize"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-resize}{}}}
\subsection{Method \code{resize()}}{
resize a trial with a greater sample size. This function is used to
update the maximum sample size adaptively after sample size reassessment.
Note that this function should be called within action functions.
It is users' responsibility to ensure it and \code{TrialSimulator} has
no way to track this.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$resize(n_patients)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{n_patients}}{integer. Number of maximum sample size of a trial.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-add_arms"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-add_arms}{}}}
\subsection{Method \code{add_arms()}}{
add one or more arms to the trial. \code{enroll_patients()} will be
called at the end to enroll all remaining patients in
\code{private$randomization_queue}. This function can be used in two
scenarios:
(1) add arms right after a trial is created (i.e., \code{Trials$new(...)}).
\code{sample_ratio} and arms added through \code{...} should be of same
length;
(2) add arms to a trial already with arm(s).

Note that this function should only be called within action functions.
It is users' responsibility to ensure it and \code{TrialSimulator} has
no way to track this.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$add_arms(sample_ratio, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{sample_ratio}}{integer vector. Sample ratio for permuted block
randomization. It will be appended to existing sample ratio in the trial.}

\item{\code{...}}{one or more objects returned from \code{arm()}.
Randomization is carried out with updated
sample ratio of newly added arm. It rolls back all patients after
\code{Trials$get_current_time()}, i.e. redo randomization for those
patients. This can be useful to add arms one by one when creating a trial.
Note that we can run \code{Trials$add_arm(sample_ratio1, arm1)} followed
by \code{Trials$add_arm(sample_ratio2, arm2)}.
We would expected similar result with
\code{Trials$add_arms(c(sample_ratio1, sample_ratio2), arm1, arm2)}. Note
that these two method won't return exactly the same trial because
randomization_queue were generated twice in the first approach but only
once in the second approach. But statistically, they are equivalent and
of the same distribution.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-add_regimen"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-add_regimen}{}}}
\subsection{Method \code{add_regimen()}}{
register regimen to a trial. The regimen consists of three functions
to determine the patients who may switch to other treatment during a
a trial, to determine the switching time and how to update patients'
endpoint data accordingly.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$add_regimen(regimen)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{regimen}}{an object created by \code{regimen()}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_regimen"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_regimen}{}}}
\subsection{Method \code{get_regimen()}}{
return registered regimen.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_regimen()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-has_regimen"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-has_regimen}{}}}
\subsection{Method \code{has_regimen()}}{
return whether a regimen is registered
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$has_regimen()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_name"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_name}{}}}
\subsection{Method \code{get_name()}}{
return name of trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_name()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_description"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_description}{}}}
\subsection{Method \code{get_description()}}{
return description of trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_description()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_arms"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_arms}{}}}
\subsection{Method \code{get_arms()}}{
return a list of arms in the trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_arms()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_arms_name"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_arms_name}{}}}
\subsection{Method \code{get_arms_name()}}{
return arms' name of trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_arms_name()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_number_arms"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_number_arms}{}}}
\subsection{Method \code{get_number_arms()}}{
get number of arms in the trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_number_arms()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-has_arm"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-has_arm}{}}}
\subsection{Method \code{has_arm()}}{
check if the trial has any arm. Return \code{TRUE} or \code{FALSE}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$has_arm()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_an_arm"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_an_arm}{}}}
\subsection{Method \code{get_an_arm()}}{
return an arm
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_an_arm(arm_name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arm_name}}{character, name of arm to be extracted}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_sample_ratio"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_sample_ratio}{}}}
\subsection{Method \code{get_sample_ratio()}}{
return current sample ratio of the trial. The ratio can probably change
during the trial (e.g., arm is removed or added)
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_sample_ratio(arm_names = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arm_names}}{character vector of arms.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_number_patients"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_number_patients}{}}}
\subsection{Method \code{get_number_patients()}}{
return number of patients when planning the trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_number_patients()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_number_enrolled_patients"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_number_enrolled_patients}{}}}
\subsection{Method \code{get_number_enrolled_patients()}}{
return number of enrolled (randomized) patients
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_number_enrolled_patients()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_number_unenrolled_patients"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_number_unenrolled_patients}{}}}
\subsection{Method \code{get_number_unenrolled_patients()}}{
return number of unenrolled patients
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_number_unenrolled_patients()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_stratum_queue"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_stratum_queue}{}}}
\subsection{Method \code{get_stratum_queue()}}{
return stratum queue of planned but not yet enrolled patients.
This function does not update stratum_queue, just return its value
for debugging purpose.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_stratum_queue(index = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{index}}{index to be extracted. Return all queue if \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_randomization_queue"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_randomization_queue}{}}}
\subsection{Method \code{get_randomization_queue()}}{
return randomization queue of planned but not yet enrolled patients.
This function does not update randomization_queue, just return its value
for debugging purpose.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_randomization_queue(index = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{index}}{index to be extracted. Return all queue if \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_enroll_time"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_enroll_time}{}}}
\subsection{Method \code{get_enroll_time()}}{
return enrollment time of planned but not yet enrolled patients.
This function does not update enroll_time, just return its value
for debugging purpose.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_enroll_time(index = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{index}}{index to extract. Return all enroll time if \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-enroll_patients"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-enroll_patients}{}}}
\subsection{Method \code{enroll_patients()}}{
assign new patients to pre-planned randomization queue at pre-specified
enrollment time.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$enroll_patients(n_patients = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{n_patients}}{number of new patients to be enrolled. If \code{NULL},
all remaining patients in plan are enrolled. Error may be triggered if
n_patients is greater than remaining patients as planned.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-set_current_time"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-set_current_time}{}}}
\subsection{Method \code{set_current_time()}}{
set current time of a trial. Any data collected before could not be
changed. private$now should be set after a milestone is triggered
(through Milestones class, futility, interim, etc), an arm is added or
removed at a milestone
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$set_current_time(time)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{time}}{current calendar time of a trial.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_current_time"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_current_time}{}}}
\subsection{Method \code{get_current_time()}}{
return current time of a trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_current_time()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_event_tables"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_event_tables}{}}}
\subsection{Method \code{get_event_tables()}}{
count accumulative number of events (for TTE) or non-missing samples (otherwise) over
calendar time (enroll time + tte for TTE, or enroll time + readout otherwise)
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_event_tables(arms = NULL, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arms}}{a vector of arms' name on which the event tables are created.
if \code{NULL}, all arms in the trial will be used.}

\item{\code{...}}{subset conditions compatible with \code{dplyr::filter}.
Event tables will be counted on subset of trial data only.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_data_lock_time_by_event_number"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_data_lock_time_by_event_number}{}}}
\subsection{Method \code{get_data_lock_time_by_event_number()}}{
given a set of endpoints and target number of events, determine the data
lock time for a milestone (futility, interim, final, etc.). This function does
not change trial object (e.g. rolling back not yet randomized patients after
the found data lock time).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_data_lock_time_by_event_number(
  endpoints,
  arms,
  target_n_events,
  type = c("all", "any"),
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{endpoints}}{character vector. Data lock time is determined by a set
of endpoints.}

\item{\code{arms}}{a vector of arms' name on which number of events will be
counted.}

\item{\code{target_n_events}}{target number of events for each of the
\code{endpoints}.}

\item{\code{type}}{\code{all} if all target number of events are reached.
\code{any} if the any target number of events is reached.}

\item{\code{...}}{subset conditions compatible with \code{dplyr::filter}. Number
Time of milestone is based on event counts on the subset of trial data.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
data lock time
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_data_lock_time_by_enrollment"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_data_lock_time_by_enrollment}{}}}
\subsection{Method \code{get_data_lock_time_by_enrollment()}}{
given a target number of enrolled patients, determine the data
lock time for a milestone (futility, interim, final, etc.). This function does
not change trial object (e.g. rolling back not yet randomized patients after
the found data lock time). It is similar to get_data_lock_time_by_event_number
but only focus on patient_id.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_data_lock_time_by_enrollment(
  arms,
  target_n_patients,
  min_treatment_duration,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arms}}{a vector of arms' name on which number of events will be
counted.}

\item{\code{target_n_patients}}{target number of enrolled patients.}

\item{\code{min_treatment_duration}}{numeric. Zero or positive value.
minimum treatment duration of enrolled patients.
If 0, it looks for triggering time based on number of enrolled
patients in population specified by \code{...} and \code{arms}. If positive,
it means that milestone is triggered when a specific number of enrolled
patients have received treatment for at least \code{min_treatment_duration}
duration. It is users' responsibility to assure that the unit of
\code{min_treatment_duration} are consistent with
readout of non-tte endpoints, dropout time, and trial duration.}

\item{\code{...}}{subset conditions compatible with \code{dplyr::filter}. Number
Time of milestone is based on event counts on the subset of trial data.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
data lock time
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_data_lock_time_by_calendar_time"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_data_lock_time_by_calendar_time}{}}}
\subsection{Method \code{get_data_lock_time_by_calendar_time()}}{
given the calendar time to lock the data, return it with event counts of
each of the endpoints.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_data_lock_time_by_calendar_time(calendar_time)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{calendar_time}}{numeric. Calendar time to lock the data}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
data lock time
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_locked_data"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_locked_data}{}}}
\subsection{Method \code{get_locked_data()}}{
return locked data, i.e. snapshot at a milestone. TTE data is censored
and non-TTE data is truncated accounting for readout time and dropout
time simultaneously by the triggering time of milestone.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_locked_data(milestone_name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{milestone_name}}{character. Milestone name of which the locked
data to be extracted.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_locked_data_name"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_locked_data_name}{}}}
\subsection{Method \code{get_locked_data_name()}}{
return names of locked data
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_locked_data_name()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_event_number"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_event_number}{}}}
\subsection{Method \code{get_event_number()}}{
return number of events at lock time of milestones
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_event_number(milestone_name = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{milestone_name}}{names of triggered milestones. Use all triggered milestones
if \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-save_milestone_time"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-save_milestone_time}{}}}
\subsection{Method \code{save_milestone_time()}}{
save time of a new milestone.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$save_milestone_time(milestone_time, milestone_name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{milestone_time}}{numeric. Time of new milestone.}

\item{\code{milestone_name}}{character. Name of new milestone.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_milestone_time"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_milestone_time}{}}}
\subsection{Method \code{get_milestone_time()}}{
return milestone time when triggering a given milestone
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_milestone_time(milestone_name = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{milestone_name}}{character. Name of milestone. If \code{NULL},
time of all triggered milestones are returned.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-lock_data"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-lock_data}{}}}
\subsection{Method \code{lock_data()}}{
lock data at specific calendar time.
For time-to-event endpoints, their event indicator \code{*_event} should be
updated accordingly. Locked data should be stored separately.
DO NOT OVERWRITE/UPDATE private$trial_data! which can lose actual
time-to-event information. For example, a patient may be censored at
the first data lock. However, he may have event being observed in a
later data lock.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$lock_data(at_calendar_time, milestone_name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{at_calendar_time}}{time point to lock trial data}

\item{\code{milestone_name}}{assign milestone name as the name of locked data for
future reference.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-event_plot"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-event_plot}{}}}
\subsection{Method \code{event_plot()}}{
plot of cumulative number of events/samples over calendar time.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$event_plot()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-censor_trial_data"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-censor_trial_data}{}}}
\subsection{Method \code{censor_trial_data()}}{
censor trial data at calendar time
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$censor_trial_data(
  censor_at = NULL,
  selected_arms = NULL,
  enrolled_before = Inf
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{censor_at}}{time of censoring. It is set to trial duration if
\code{NULL}.}

\item{\code{selected_arms}}{censoring is applied to selected arms (e.g.,
removed arms) only. If \code{NULL}, it will be set to all available arms
in trial data. Otherwise, censoring is applied to user-specified arms only.
This is necessary because number of events/sample size in removed arms
should be fixed unchanged since corresponding milestone is triggered. In that
case, one can update trial data by something like
\code{censor_trial_data(censor_at = milestone_time, selected_arms = removed_arms)}.}

\item{\code{enrolled_before}}{censoring is applied to patients enrolled before
specific time. This argument would be used when trial duration is
updated by \code{set_duration}. Adaptation happens when \code{set_duration}
is called so we fix duration for patients enrolled before adaptation
to maintain independent increment. This should work when trial duration
is updated for multiple times.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-save"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-save}{}}}
\subsection{Method \code{save()}}{
save a single value or a one-row data frame to trial's output
for further analysis/summary later. Results saved by calling this
function have a life cycle of the whole simulation. This means that
all results are accumulated across multiple simulated trial and can be
used for summary later.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$save(value, name = "", overwrite = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{value}}{value to be saved. It can be a scalar (vector of length 1)
or a data frame (of one row).}

\item{\code{name}}{character to name the saved object. It will be used to
name a column in trial's output if \code{value} is a scalar.
If \code{value} is a data frame, \code{name} will be the prefix pasted
with the column name of \code{value} in trial's output.
If user want to use
\code{value}'s column name as is in trial's output, set \code{name}
to be \code{''} as default. Otherwise, column name would be, e.g.,
\code{"{<name>}_<{colnames(value)}>"}.}

\item{\code{overwrite}}{logic. \code{TRUE} if overwriting existing entries
with warning, otherwise, throwing an error and stop.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-bind"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-bind}{}}}
\subsection{Method \code{bind()}}{
row bind a data frame to existing data frame. If a data frame \code{name}
is not existing in a trial, then it is equivalent to
calling \code{Trials$save_custom_data()}.
Extra columns in \code{value} are ignored. Columns in
\code{Trials$custom_data[[name]]} but not in \code{value} are filled
with \code{NA}.

This function can be used to save results across multiple milestones.
For example, p-values and effect estimates of endpoints may be computed
at multiple milestones. Users may want to bind them into a data frame
for combination test or graphical test. In this case, this function
can be called repeatedly in milestones. Once the data frame is fully
conducted, statistical test can be performed on its final version
retrieved by calling \code{Trials$get()}.

Note that data saved by calling this function has a short life cycle
within a single simulated trial. It will be reset to \code{NULL} before
simulated another trial. Thus, it cannot be used to save results that
are used for summarizing the simulation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$bind(value, name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{value}}{a data frame to be saved. It can consist of one or
multiple rows.}

\item{\code{name}}{character. Name of object to be saved.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-save_custom_data"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-save_custom_data}{}}}
\subsection{Method \code{save_custom_data()}}{
save arbitrary (number of) objects into a trial so that users can use
those to control the workflow. Most common use case is to store
simulation parameters to be used in action functions.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$save_custom_data(value, name, overwrite = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{value}}{value to be saved. Any type.}

\item{\code{name}}{character. Name of the value to be accessed later.}

\item{\code{overwrite}}{logic. \code{TRUE} if overwriting existing entries
with warning, otherwise, throwing an error and stop.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_custom_data"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_custom_data}{}}}
\subsection{Method \code{get_custom_data()}}{
return custom data saved by calling \code{Trials$save_custom_data()}
or \code{Trials$bind()} with its name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_custom_data(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{character. Name of custom data to be accessed.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get}{}}}
\subsection{Method \code{get()}}{
alias of function \code{get_custom_data} to make it short and cool.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{character. Name of custom data to be accessed.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_output"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_output}{}}}
\subsection{Method \code{get_output()}}{
return a data frame of all current outputs saved by calling
\code{Trials$save()}. Usually this function is call at the end of
simulation for summary.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_output(cols = NULL, simplify = TRUE, tidy = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{cols}}{columns to be returned from \code{Trial$output}. If
\code{NULL}, all columns are returned.}

\item{\code{simplify}}{logical. Return value rather than a data frame of one
column when \code{length(col) == 1} and \code{simplify == TRUE}.}

\item{\code{tidy}}{logical. \code{TrialSimulator} automatically records a set
of standard outputs at milestones, even when \code{doNothing} is used
as action functions. These includes time of triggering milestones,
number of observed events for time-to-event endpoints, and number of
non-missing readouts for non-TTE endpoints
(see \code{vignette('actionFunctions')}). This usually mean a large
number of columns in outputs. If users have no intent to summarize a
trial on these columns, setting \code{tidy = TRUE} can eliminate these
columns from \code{get_output()}. Note that currently we use regex
\code{"^n_events_<.*?>_<.*?>$"} and
\code{"^milestone_time_<.*?>$"} to match columns to be eliminated.
If users plan to use \code{tidy = TRUE}, caution is needed when naming
custom outputs in \code{save()}. Default \code{FALSE}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-mute"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-mute}{}}}
\subsection{Method \code{mute()}}{
mute all messages (not including warnings)
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$mute(silent)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{silent}}{logical.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-tidy_output"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-tidy_output}{}}}
\subsection{Method \code{tidy_output()}}{
save less information in trial output if no intent to use it in summary
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$tidy_output(tidy)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{tidy}}{logical. If \code{TRUE}, event count per arm per endpoint is
not computed and saved in trial output. This can speed up simulation by
up to 40\% under some circumstances.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-independentIncrement"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-independentIncrement}{}}}
\subsection{Method \code{independentIncrement()}}{
calculate independent increments from a given set of milestones
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$independentIncrement(
  formula,
  placebo,
  milestones,
  alternative,
  planned_info,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{formula}}{An object of class \code{formula} that can be used with
\code{survival::coxph}. Must consist \code{arm} and endpoint in \code{data}.
No covariate is allowed. Stratification variables are supported and can be
added using \code{strata(...)}.}

\item{\code{placebo}}{character. String of placebo in trial's locked data.}

\item{\code{milestones}}{a character vector of milestone names in the trial, e.g.,
\code{listener$get_milestone_names()}.}

\item{\code{alternative}}{a character string specifying the alternative hypothesis,
must be one of \code{"greater"} or \code{"less"}. No default value.
\code{"greater"} means superiority of treatment over placebo is established
by an hazard ratio greater than 1 when a log-rank test is used.}

\item{\code{planned_info}}{a vector of planned accumulative number of event of
time-to-event endpoint. It is named by milestone names.
Note: \code{planned_info} can also be a character
\code{"oracle"} so that planned number of events are set to be observed
number of events, in that case inverse normal z statistics equal to
one-sided logrank statistics. This is for the purpose of debugging only.
In formal simulation, \code{"oracle"} should not be used if adaptation
is present. Pre-fixed \code{planned_info} should be used to create
weights in combination test that controls the family-wise error rate
in the strong sense.}

\item{\code{...}}{subset condition that is compatible with \code{dplyr::filter}.
\code{survdiff} will be fitted on this subset only to compute one-sided
logrank statistics. It could be useful when a
trial consists of more than two arms. By default it is not specified,
all data will be used to fit the model.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
This function returns a data frame with columns:
\describe{
\item{\code{p_inverse_normal}}{one-sided p-value for inverse normal test
based on logrank test (alternative hypothesis: risk is higher in placebo arm).
Accumulative data is used. }
\item{\code{z_inverse_normal}}{z statistics of \code{p_inverse_normal}.
Accumulative data is used. }
\item{\code{p_lr}}{one-sided p-value for logrank test
 (alternative hypothesis: risk is higher in placebo arm).
Accumulative data is used. }
\item{\code{z_lr}}{z statistics of \code{p_lr}.
Accumulative data is used. }
\item{\code{info}}{observed accumulative event number. }
\item{\code{planned_info}}{planned accumulative event number. }
\item{\code{info_pbo}}{observed accumulative event number in placebo. }
\item{\code{info_trt}}{observed accumulative event number in treatment arm. }
\item{\code{wt}}{weights in \code{z_inverse_normal}. }
}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{
\dontrun{
trial$independentIncrement(Surv(pfs, pfs_event) ~ arm, 'pbo',
                           listener$get_milestone_names(),
                           'less', 'oracle')
}
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-dunnettTest"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-dunnettTest}{}}}
\subsection{Method \code{dunnettTest()}}{
carry out closed test based on Dunnett method under group sequential
design.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$dunnettTest(
  formula,
  placebo,
  treatments,
  milestones,
  alternative,
  planned_info,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{formula}}{An object of class \code{formula} that can be used with
\code{survival::coxph}. Must consist \code{arm} and endpoint in \code{data}.
No covariate is allowed. Stratification variables are supported and can be
added using \code{strata(...)}.}

\item{\code{placebo}}{character. Name of placebo arm.}

\item{\code{treatments}}{character vector. Name of treatment arms to be used in
comparison.}

\item{\code{milestones}}{character vector. Names of triggered milestones at which either
adaptation is applied or statistical testing for endpoint is performed.
Milestones in \code{milestones} does not need to be sorted by their triggering time.}

\item{\code{alternative}}{a character string specifying the alternative hypothesis,
must be one of \code{"greater"} or \code{"less"}. No default value.
\code{"greater"} means superiority of treatment over placebo is established
by an hazard ratio greater than 1 when a log-rank test is used.}

\item{\code{planned_info}}{a data frame of planned number of events of
time-to-event endpoint in each stage and each arm. Milestone names, i.e.,
\code{milestones} are row names of \code{planned_info}, and arm names, i.e.,
\code{c(placebo, treatments)} are column names.
Note that it is not the accumulative but stage-wise event numbers.
It is usually not easy to determine these numbers in practice, simulation
may be used to get estimates.
Note: \code{planned_info} can also be a character
\code{"default"} so that \code{planned_info} are set to be number
of newly randomized patients in the control arm in each of the stages.
This assumes that
event rate do not change over time and/or sample ratio between placebo
and a treatment arm does not change as well, which may not be true.
It is for the purpose of debugging or rapid implementation
only. Using simulation to pick \code{planned_info} is recommended in
formal simulation study. Another issue with \code{planned_info} set to
be \code{"default"} is that it is possible patient recruitment is done
before a specific stage, as a result, \code{planned_info} is zero which
can crash the program.}

\item{\code{...}}{subset condition that is compatible with \code{dplyr::filter}.
\code{survdiff} will be fitted on this subset only to compute one-sided
logrank statistics. It could be useful when comparison is made on a
subset of treatment arms. By default it is not specified,
all data (placebo plus one treatment arm at a time) in the locked data
are used to fit the model.}
}
\if{html}{\out{</div>}}
}
\subsection{Details}{
This function computes stage-wise p-values for each of the intersection
hypotheses based on Dunnett test. If only one treatment arm is present,
it is equivalent to compute the stage-wise p-values of elemental
hypotheses. This function also computes inverse normal combination
test statistics at each of the stages.
The choice of \code{planned_info} can affect the calculation of
stage-wise p-values. Specifically, it is used to compute
the columns \code{observed_info} and \code{p_inverse_normal} in returned
data frame, which will be used in \code{Trial$closedTest()}.
The choice of \code{planned_info} can affect the result of
\code{Trial$closedTest()} so user should chose it with caution.

Note that in \code{Trial$closedTest()},
\code{observed_info}, which is derived from \code{planned_info}, will
lead to the same closed testing results up to a constant. This is because
the closed test uses information fraction
\code{observed_info/sum(observed_info)}. As a result, setting
\code{planned_info} to, e.g., \code{10 * planned_info} should give same
closed test results.

Based on numerical study, setting \code{planned_info = "default"} leads
to a much higher power (roughly 10\%) than setting \code{planned_info} to
median of event numbers at stages, which can be determined by simulation.
I am not sure if regulator would support such practice. For example,
if a milestone (e.g., interim analysis) is triggered at a pre-specified
calendar time, the number of randomized patients is random and is unknown
when planning the trial. If I understand it correctly, regulator may want
the information fraction in closed test (combined with Dunnett test) to
be pre-fixed. In addition, this choice for \code{planned_info} assumes
that the event rates does not change over time which is obviously not
true. It is recommended to always use pre-fixed \code{planned_info} for
restrict control of family-wise error rate. It should be pointed out
that the choice of pre-fixed \code{planned_info} can affect statistical
power significantly so fine-tuning may be required.
}

\subsection{Returns}{
a list with element names like \code{arm_name},
\code{arm1_name|arm2_name}, \code{arm1_name|arm2_name|arm3_name}, etc.,
i.e., all possible combination of treatment arms in comparison. Each
element is a data frame, with its column names self-explained. Specifically,
the columns \code{p_inverse_normal}, \code{observed_info},
\code{is_final} can be used with \code{GroupSequentialTest} to perform
significance test.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{\dontrun{
trial$dunnettTest(Surv(pfs, pfs_event) ~ arm, 'pbo', c('high dose', 'low dose'),
                  listener$get_milestone_names(), 'default')
}

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-closedTest"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-closedTest}{}}}
\subsection{Method \code{closedTest()}}{
perform closed test based on Dunnett test
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$closedTest(
  dunnett_test,
  treatments,
  milestones,
  alpha,
  alpha_spending = c("asP", "asOF")
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{dunnett_test}}{object returned by \code{Trial$dunnettTest()}.}

\item{\code{treatments}}{character vector. Name of treatment arms to be used in
comparison.}

\item{\code{milestones}}{character vector. Names of triggered milestones at which
significance testing for endpoint is performed in closed test.
Milestones in \code{milestones} does not need to be sorted by their triggering time.}

\item{\code{alpha}}{numeric. Allocated alpha.}

\item{\code{alpha_spending}}{alpha spending function. It can be \code{"asP"} or
\code{"asOF"}. Note that theoretically it can be \code{"asUser"}, but
it is not tested. It may be supported in the future.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
a data frame of columns \code{arm}, \code{decision}
(final decision on a hypothesis at the end of trial, \code{"accept"} or \code{"reject"}),
\code{milestone_at_reject}, and \code{reject_time}.
If a hypothesis is accepted at then end of a trial,
\code{milestone_at_reject} is \code{NA}, and \code{reject_time} is \code{Inf}.

Note that if a hypothesis is tested at multiple milestones, the final
\code{decision} will be \code{"accept"} if it is accepted at at least
one milestone. The \code{decision} is \code{"reject"} only if the hypothesis
is rejected at all milestones.
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{\dontrun{
dt <- trial$dunnettTest(
  Surv(pfs, pfs_event) ~ arm,
  placebo = 'pbo',
  treatments = c('high dose', 'low dose'),
  milestones = c('dose selection', 'interim', 'final'),
  data.frame(pbo = c(100, 160, 80),
             low = c(100, 160, 80),
             high = c(100, 160, 80),
             row.names = c('dose selection', 'interim', 'final'))

trial$closedTest(dt, treatments = c('high dose', 'low dose'),
                 milestones = c('interim', 'final'),
                 alpha = 0.025, alpha_spending = 'asOF')
}

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_seed"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_seed}{}}}
\subsection{Method \code{get_seed()}}{
return random seed
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_seed()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-print"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-print}{}}}
\subsection{Method \code{print()}}{
print a trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$print()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_snapshot_copy"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_snapshot_copy}{}}}
\subsection{Method \code{get_snapshot_copy()}}{
return a snapshot of a trial before it is executed.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_snapshot_copy()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-make_snapshot"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-make_snapshot}{}}}
\subsection{Method \code{make_snapshot()}}{
make a snapshot before running a trial. This can be useful when
resetting a trial. This is only called when initializing a `Trial`
object, when arms have not been added yet.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$make_snapshot()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-make_arms_snapshot"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-make_arms_snapshot}{}}}
\subsection{Method \code{make_arms_snapshot()}}{
make a snapshot of arms
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$make_arms_snapshot()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-reset"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-reset}{}}}
\subsection{Method \code{reset()}}{
reset a trial to its snapshot taken before it was executed. Seed will be
reassigned with a new one. Enrollment time are re-generated. If the trial
already have arms when this function is called, they are added back to
recruit patients again.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$reset()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-set_arm_added_time"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-set_arm_added_time}{}}}
\subsection{Method \code{set_arm_added_time()}}{
save time when an arm is added to the trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$set_arm_added_time(arm, time)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arm}}{name of added arm.}

\item{\code{time}}{time when an arm is added.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_arm_added_time"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_arm_added_time}{}}}
\subsection{Method \code{get_arm_added_time()}}{
get time when an arm is added to the trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_arm_added_time(arm)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arm}}{arm name.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-set_arm_removal_time"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-set_arm_removal_time}{}}}
\subsection{Method \code{set_arm_removal_time()}}{
save time when an arm is removed to the trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$set_arm_removal_time(arm, time)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arm}}{name of removed arm.}

\item{\code{time}}{time when an arm is removed.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_arm_removal_time"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_arm_removal_time}{}}}
\subsection{Method \code{get_arm_removal_time()}}{
get time when an arm is removed from the trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_arm_removal_time(arm)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{arm}}{arm name.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-get_stratification_factors"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-get_stratification_factors}{}}}
\subsection{Method \code{get_stratification_factors()}}{
return stratification factors
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$get_stratification_factors()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-has_stratification_factors"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-has_stratification_factors}{}}}
\subsection{Method \code{has_stratification_factors()}}{
has stratification factors
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$has_stratification_factors()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Trials-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Trials-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Trials$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Controller.R
\name{controller}
\alias{controller}
\title{Define a Controller}
\usage{
controller(trial, listener)
}
\arguments{
\item{trial}{an object returned from \code{trial()}.}

\item{listener}{an object returned from \code{listener()}.}
}
\description{
Define a controller of a trial. This is a user-friendly wrapper for
the class constructor \code{Controller$new()}. Users who are not familiar with
the concept of classes may consider using this wrapper directly.

\code{TrialSimulator} uses a controller to coordinate a trial object
and a listener object to run simulations, in which the trial object defines
endpoints, arms, and other settings of a trial, while the listener object
monitors trials to triggered pre-defined milestones and execute action
functions. See vignettes of this package for more examples.
}
\examples{

# a minimum, meaningful, and executable example,
# where a randomized trial with two arms is simulated and analyzed.

control <- arm(name = 'control arm')
active <- arm(name = 'active arm')

pfs_in_control <- endpoint(name = 'PFS', type = 'tte',
                           generator = rexp, rate = log(2) / 5)
control$add_endpoints(pfs_in_control)

pfs_in_active <- endpoint(name = 'PFS', type = 'tte',
                          generator = rexp, rate = log(2) / 6)
active$add_endpoints(pfs_in_active)

accrual_rate <- data.frame(end_time = c(10, Inf),
                           piecewise_rate = c(30, 50))
trial <- trial(name = 'trial',
               n_patients = 1000,
               duration = 40,
               enroller = StaggeredRecruiter,
               accrual_rate = accrual_rate,
               dropout = rweibull, shape = 2, scale = 38)

trial$add_arms(sample_ratio = c(1, 1), control, active)

action_at_final <- function(trial){
  locked_data <- trial$get_locked_data('final analysis')
  fitLogrank(Surv(PFS, PFS_event) ~ arm, placebo = 'control arm',
             data = locked_data, alternative = 'less')
  invisible(NULL)
}

final <- milestone(name = 'final analysis',
                   action = action_at_final,
                   when = calendarTime(time = 40))

listener <- listener()
listener$add_milestones(final)

controller <- controller(trial, listener)
controller$run(n = 1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Milestones.R
\docType{class}
\name{Milestones}
\alias{Milestones}
\title{Class of Milestones}
\description{
Create a class of milestone. A milestone means the time point to take an action,
e.g., carrying out (futility, interim, final) analysis for
adding/removing arms, or stopping a trial early.
It can also be any more general time point where trial
data is used in decision making or adaptation. For example, one can define a
milestone for changing randomization scheme, sample size re-assessment, trial
duration extension etc.

Public methods in this R6 class are used in developing
this package. Thus, we have to export the whole R6 class which exposures all
public methods. However, none of the public methods on this page is
useful to end users. Instead, refer to the
\href{https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html}{vignette}
to learn how to define milestones when performing simulation using
\code{TrialSimulator}.
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Milestones-new}{\code{Milestones$new()}}
\item \href{#method-Milestones-get_name}{\code{Milestones$get_name()}}
\item \href{#method-Milestones-get_type}{\code{Milestones$get_type()}}
\item \href{#method-Milestones-get_trigger_condition}{\code{Milestones$get_trigger_condition()}}
\item \href{#method-Milestones-get_action}{\code{Milestones$get_action()}}
\item \href{#method-Milestones-set_dry_run}{\code{Milestones$set_dry_run()}}
\item \href{#method-Milestones-execute_action}{\code{Milestones$execute_action()}}
\item \href{#method-Milestones-get_trigger_status}{\code{Milestones$get_trigger_status()}}
\item \href{#method-Milestones-reset}{\code{Milestones$reset()}}
\item \href{#method-Milestones-trigger_milestone}{\code{Milestones$trigger_milestone()}}
\item \href{#method-Milestones-mute}{\code{Milestones$mute()}}
\item \href{#method-Milestones-clone}{\code{Milestones$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-new"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-new}{}}}
\subsection{Method \code{new()}}{
initialize milestone
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$new(name, type = name, trigger_condition, action = doNothing, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{character. Name of milestone.}

\item{\code{type}}{character vector. Milestone type(s) (futility, interim, final),
a milestone can be of multiple types. This is for information purpose so
can be any string.}

\item{\code{trigger_condition}}{function to check if this milestone should
trigger. See vignette \code{Condition System for Triggering Milestones in a Trial}.}

\item{\code{action}}{function to execute when the milestone triggers.}

\item{\code{...}}{(optional) arguments of \code{action}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-get_name"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-get_name}{}}}
\subsection{Method \code{get_name()}}{
return name of milestone
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$get_name()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-get_type"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-get_type}{}}}
\subsection{Method \code{get_type()}}{
return type(s) of milestone
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$get_type()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-get_trigger_condition"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-get_trigger_condition}{}}}
\subsection{Method \code{get_trigger_condition()}}{
return trigger_condition function
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$get_trigger_condition()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-get_action"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-get_action}{}}}
\subsection{Method \code{get_action()}}{
return action function
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$get_action()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-set_dry_run"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-set_dry_run}{}}}
\subsection{Method \code{set_dry_run()}}{
set if dry run should be carried out for the milestone. For more details,
refer to \code{Controller::run}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$set_dry_run(dry_run)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{dry_run}}{logical.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-execute_action"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-execute_action}{}}}
\subsection{Method \code{execute_action()}}{
execute action function
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$execute_action(trial)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{trial}}{a \code{Trial} object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-get_trigger_status"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-get_trigger_status}{}}}
\subsection{Method \code{get_trigger_status()}}{
return trigger status
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$get_trigger_status()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-reset"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-reset}{}}}
\subsection{Method \code{reset()}}{
reset an milestone so that it can be triggered again. Usually, this is called
before the controller of a trial can run additional replicates
of simulation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$reset()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-trigger_milestone"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-trigger_milestone}{}}}
\subsection{Method \code{trigger_milestone()}}{
trigger an milestone (always TRUE) and execute action accordingly. It calls
Trial$get_data_lock_time() to lock data based on conditions implemented
in Milestones$trigger_condition. If time that meets the condition cannot be
found, Trial$get_data_lock_time() will throw an error and stop the
program. This means that user needs to adjust their trigger_condition
(e.g., target number of events (target_n_events) is impossible to
reach).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$trigger_milestone(trial)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{trial}}{a \code{Trial} object.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-mute"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-mute}{}}}
\subsection{Method \code{mute()}}{
mute all messages (not including warnings)
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$mute(silent)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{silent}}{logical.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Milestones-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Milestones-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Milestones$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resize.R
\name{resize}
\alias{resize}
\title{Resizing a Trial}
\usage{
resize(trial, n_patients)
}
\arguments{
\item{trial}{a trial object returned by \code{trial()}.}

\item{n_patients}{integer. Number of maximum sample size of a trial.}
}
\description{
resize a trial with a greater sample size. This function is used to update
the maximum sample size adaptively after sample size reassessment. Note that
this function should be called within action functions. It is users'
responsibility to ensure it and \code{TrialSimulator} has no way to track this.

This is a user-friendly wrapper of the member function of trial, i.e.,
\code{Trials$resize()}, which is used in vignettes. Users who are not
familiar with the concept of classes may consider using this wrapper
directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Endpoints.R
\docType{class}
\name{Endpoints}
\alias{Endpoints}
\title{Class of Endpoint}
\description{
Create a class of endpoint to specify its name, type, readout time
(optional) and assign a random number generator.

Public methods in this R6 class are used in developing
this package. Thus, I have to export the whole R6 class which exposures all
public methods. However, none of the public methods is useful to end users
except for the one below.

\itemize{
\item \code{$print()}
}
}
\examples{
# Instead of using Endpoints$new(), please use endpoint(), a user-friendly
# wrapper to define endpoints. See examples in ?endpoint.


## ------------------------------------------------
## Method `Endpoints$print`
## ------------------------------------------------


rng <- function(n){
  data.frame(x = sample(1:3, n, replace = TRUE),
             y = sample(1:3, n, replace = TRUE)
            )
}
ep <- endpoint(name = c('x', 'y'),
               type = c('non-tte', 'non-tte'),
               readout = c(x = 0, y = 0),
               generator = rng)

## x and y as continuous endpoints, thus mean and sd are reported
ep

## force y to be categorical to create barplot of it
print(ep, categorical_vars = 'y')

}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Endpoints-new}{\code{Endpoints$new()}}
\item \href{#method-Endpoints-test_generator}{\code{Endpoints$test_generator()}}
\item \href{#method-Endpoints-get_generator}{\code{Endpoints$get_generator()}}
\item \href{#method-Endpoints-update_generator}{\code{Endpoints$update_generator()}}
\item \href{#method-Endpoints-get_readout}{\code{Endpoints$get_readout()}}
\item \href{#method-Endpoints-get_uid}{\code{Endpoints$get_uid()}}
\item \href{#method-Endpoints-get_name}{\code{Endpoints$get_name()}}
\item \href{#method-Endpoints-get_type}{\code{Endpoints$get_type()}}
\item \href{#method-Endpoints-print}{\code{Endpoints$print()}}
\item \href{#method-Endpoints-clone}{\code{Endpoints$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Endpoints-new"></a>}}
\if{latex}{\out{\hypertarget{method-Endpoints-new}{}}}
\subsection{Method \code{new()}}{
initialize an endpoint.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Endpoints$new(name, type, readout = NULL, generator, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{character vector. Name(s) of endpoint(s)}

\item{\code{type}}{character vector. Type(s) of endpoint(s). It supports
\code{"tte"} for time-to-event endpoints, and \code{"non-tte"} for
all other types of endpoints (e.g., continous, binary, categorical,
or repeated measurement. \code{TrialSimulator} will do some verification if
an endpoint is of type \code{"tte"}. However, no special
manipulation is done for non-tte endpoints.}

\item{\code{readout}}{a named numeric vector with name to be non-tte endpoint(s).
\code{readout} must be specified for every non-tte endpoint. For
example, \code{c(endpoint1 = 6, endpoint2 = 3)}, which means that it
takes 6 and 3 unit time to get readout of
\code{endpoint1} and \code{endpoint2} of a patient since being
randomized. Error message would be prompted if \code{readout} is not
named or readout is not specified for some non-tte endpoint.
If all endpoints are tte, \code{readout} should be \code{NULL} as default.}

\item{\code{generator}}{a random number generation (RNG) function.
It supports all built-in random number generators in \code{stats}, e.g.,
\code{stats::rnorm}, \code{stats::rexp}, etc. that with \code{n} as the
argument for number of observations and returns a vector.
A custom RNG function is also supported.
\code{generator} could be any functions as long as
(1) its first argument is \code{n}; and (2) it returns a vector of
length \code{n} (univariate endpoint) or a data frame of \code{n} rows
(multiple endpoints), i.e., custom RNG can return data of more than one
endpoint. This is useful when users need to simulate correlated endpoints
or longitudinal data. The column names of returned data frame should
match to \code{name} exactly, although order of columns does not matter.
If an endpoint is of type \code{"tte"}, the custom \code{generator}
should also return a column as its event indicator. For example,
if \code{"pfs"} is \code{"tte"},
then custom \code{generator} should return at least two columns
\code{"pfs"} and \code{"pfs_event"}. Usually \code{pfs_event} can be
all 1s if no censoring. Some RNG functions, e.g.,
\code{TrialSimulator::PiecewiseConstantExponentialRNG()} and
\code{TrialSimulator::CorrelatedPfsAndOs4()}, simulate TTE endpoint
data with censoring simultaneously, thus 0 exists in the columns of
event indicators. Users can implement censorship in their own RNG.
Censoring can also be specified later when
defining a trial object through argument \code{dropout}.
See \code{?trial}.
Note that if covariates, e.g., biomarker, subgroup, are needed in
generating and analyzing trial data, they can and should be defined as
endpoints in \code{endpoint()} as well.}

\item{\code{...}}{optional arguments for \code{generator}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Endpoints-test_generator"></a>}}
\if{latex}{\out{\hypertarget{method-Endpoints-test_generator}{}}}
\subsection{Method \code{test_generator()}}{
test random number generator of the endpoints. It returns an example
dataset of an endpoint object. Note that users of \code{TrialSimulator}
does not need to call this function to generate trial data; instead,
the package will call this function at milestone automatically.
Users may see example in vignette where this function is called.
However, it is for illustration purpose only. In practice, this function
may be used for debugging if users suspect some issues in custom
generator, otherwise, this function should never been called in formal
simulation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Endpoints$test_generator(n = 1000)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{n}}{integer. Number of random numbers generated from the generator.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Endpoints-get_generator"></a>}}
\if{latex}{\out{\hypertarget{method-Endpoints-get_generator}{}}}
\subsection{Method \code{get_generator()}}{
return random number generator of an endpoint
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Endpoints$get_generator()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Endpoints-update_generator"></a>}}
\if{latex}{\out{\hypertarget{method-Endpoints-update_generator}{}}}
\subsection{Method \code{update_generator()}}{
update endpoint generator
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Endpoints$update_generator(generator, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{generator}}{a random number generation (RNG) function.
See \code{generator} of \code{endpoint()}.}

\item{\code{...}}{optional arguments for \code{generator}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Endpoints-get_readout"></a>}}
\if{latex}{\out{\hypertarget{method-Endpoints-get_readout}{}}}
\subsection{Method \code{get_readout()}}{
return readout function
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Endpoints$get_readout()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Endpoints-get_uid"></a>}}
\if{latex}{\out{\hypertarget{method-Endpoints-get_uid}{}}}
\subsection{Method \code{get_uid()}}{
return uid
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Endpoints$get_uid()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Endpoints-get_name"></a>}}
\if{latex}{\out{\hypertarget{method-Endpoints-get_name}{}}}
\subsection{Method \code{get_name()}}{
return endpoints' name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Endpoints$get_name()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Endpoints-get_type"></a>}}
\if{latex}{\out{\hypertarget{method-Endpoints-get_type}{}}}
\subsection{Method \code{get_type()}}{
return endpoints' type
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Endpoints$get_type()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Endpoints-print"></a>}}
\if{latex}{\out{\hypertarget{method-Endpoints-print}{}}}
\subsection{Method \code{print()}}{
print an endpoint object
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Endpoints$print(categorical_vars = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{categorical_vars}}{a character vector of endpoints. This can
be used to force variables with limited distinct values as categorical
variables in summary report. For example, a numeric endpoint may take
integer values 0, 1, 2. Instead of computing mean and standard derivation
in the summary report, put this endpoint in \code{categorical_vars} can
force it be a categorical variable and a barplot is generated in summary
report instead.}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{
rng <- function(n){
  data.frame(x = sample(1:3, n, replace = TRUE),
             y = sample(1:3, n, replace = TRUE)
            )
}
ep <- endpoint(name = c('x', 'y'),
               type = c('non-tte', 'non-tte'),
               readout = c(x = 0, y = 0),
               generator = rng)

## x and y as continuous endpoints, thus mean and sd are reported
ep

## force y to be categorical to create barplot of it
print(ep, categorical_vars = 'y')

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Endpoints-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Endpoints-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Endpoints$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Regimens.R
\docType{class}
\name{Regimens}
\alias{Regimens}
\title{Class of Regimens}
\description{
Create a class of regimen. A regimen defines the rules to select treatments for
patients switch, to determine the time of switching, and to update patients'
endpoint data.
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Regimens-new}{\code{Regimens$new()}}
\item \href{#method-Regimens-get_number_treatment_allocator}{\code{Regimens$get_number_treatment_allocator()}}
\item \href{#method-Regimens-get_treatment_allocator}{\code{Regimens$get_treatment_allocator()}}
\item \href{#method-Regimens-get_number_time_selector}{\code{Regimens$get_number_time_selector()}}
\item \href{#method-Regimens-get_time_selector}{\code{Regimens$get_time_selector()}}
\item \href{#method-Regimens-get_number_data_modifier}{\code{Regimens$get_number_data_modifier()}}
\item \href{#method-Regimens-get_data_modifier}{\code{Regimens$get_data_modifier()}}
\item \href{#method-Regimens-get_treatment_allocator_args}{\code{Regimens$get_treatment_allocator_args()}}
\item \href{#method-Regimens-get_time_selector_args}{\code{Regimens$get_time_selector_args()}}
\item \href{#method-Regimens-get_data_modifier_args}{\code{Regimens$get_data_modifier_args()}}
\item \href{#method-Regimens-clone}{\code{Regimens$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-new"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-new}{}}}
\subsection{Method \code{new()}}{
initialize regimen
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$new(what, when, how, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{what}}{a function determining whether patients' data would be
updated due to switching treatment. It takes \code{patient_data},
a data frame as argument, and returns a data frame of two columns
\code{patient_id} and \code{new_treatment}. Patients with \code{NA} in
\code{new_treatment} will be skipped.
The number of rows in the returned data frame may be smaller than the
number of patients in the input data frame. This indicates that some
patients' data will not be modifier.
Note that the returned object will be passed into function `how()`, which
is also provide by users. This argument can also be a
list of functions that will be executed sequentially. No default value.}

\item{\code{when}}{a function determining the time at which a patient switches
to another treatment regimen, measured from the time of enrollment.
It takes \code{patient_data}, a data frame as
argument, and returns a data frame of two columns \code{patient_id} and
\code{switch_time} (from \code{enroll_time}). No \code{NA} is allowed
in \code{switch_time} and the number of rows in the returned data frame
must equal the number of rows in \code{patient_data}, i.e., switching
time must be specified to every patients.
Note that the returned object will be passed into function `how()`, which
is also provided by users. This argument can also be a
list of functions that will be executed sequentially. No default value.}

\item{\code{how}}{a function updating patients' data after treatment switching.
Only modified columns and \code{patient_id} are returned. A cell will
be omitted if \code{NA}, meaning no change to that patient for the endpoint
or other variables. Equivalently, users can also fill the cell with
its original value. This argument can also be a list of functions that
will be executed sequentially. No default value.}

\item{\code{...}}{(optional) named arguments routed to one or more of
\code{what}, \code{when}, and \code{how}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-get_number_treatment_allocator"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-get_number_treatment_allocator}{}}}
\subsection{Method \code{get_number_treatment_allocator()}}{
return number of treatment allocators for regimen
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$get_number_treatment_allocator()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-get_treatment_allocator"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-get_treatment_allocator}{}}}
\subsection{Method \code{get_treatment_allocator()}}{
return user-defined new treatment for a patient
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$get_treatment_allocator(index = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{index}}{integer. Index of allocator. Return all allocators if \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-get_number_time_selector"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-get_number_time_selector}{}}}
\subsection{Method \code{get_number_time_selector()}}{
return number of time selector for regimen
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$get_number_time_selector()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-get_time_selector"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-get_time_selector}{}}}
\subsection{Method \code{get_time_selector()}}{
return user-defined time selector
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$get_time_selector(index = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{index}}{integer. Index of selector. Return all selectors if \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-get_number_data_modifier"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-get_number_data_modifier}{}}}
\subsection{Method \code{get_number_data_modifier()}}{
return number of data modifier for regimen
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$get_number_data_modifier()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-get_data_modifier"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-get_data_modifier}{}}}
\subsection{Method \code{get_data_modifier()}}{
return user-defined endpoint data modifier
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$get_data_modifier(index = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{index}}{integer. Index of selector. Return all modifiers if \code{NULL}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-get_treatment_allocator_args"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-get_treatment_allocator_args}{}}}
\subsection{Method \code{get_treatment_allocator_args()}}{
return pre-bound arguments for the i-th treatment allocator
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$get_treatment_allocator_args(index)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{index}}{integer.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-get_time_selector_args"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-get_time_selector_args}{}}}
\subsection{Method \code{get_time_selector_args()}}{
return pre-bound arguments for the i-th time selector
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$get_time_selector_args(index)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{index}}{integer.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-get_data_modifier_args"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-get_data_modifier_args}{}}}
\subsection{Method \code{get_data_modifier_args()}}{
return pre-bound arguments for the i-th data modifier
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$get_data_modifier_args(index)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{index}}{integer.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Regimens-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Regimens-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Regimens$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitCoxph.R
\name{fitCoxph}
\alias{fitCoxph}
\title{Fit Cox Proportional Hazard Ratio model}
\usage{
fitCoxph(formula, placebo, data, alternative, scale, ..., tidy = TRUE)
}
\arguments{
\item{formula}{An object of class \code{formula} that can be used with
\code{survival::coxph}. The data frame \code{data} must consist a column
\code{arm} and a column of the endpoint specified in \code{formula}.
Covariates can be adjusted. Interactions between \code{arm} and covariates are
allowed in \code{formula}, but \code{arm} must has a term of main effect,
and only estimate of that main effect is tested.}

\item{placebo}{Character. String indicating the placebo in \code{data$arm}.}

\item{data}{Data frame. Usually it is a data snapshot locked at a milestone.}

\item{alternative}{a character string specifying the alternative hypothesis,
must be one of \code{"greater"} or \code{"less"}, i.e., one-sided test is
enforced. No default value.
\code{"greater"} means superiority of treatment over placebo is established
by an hazard ratio greater than 1.}

\item{scale}{character. The type of estimate in the output. Must be one
of \code{"log hazard ratio"} or \code{"hazard ratio"}. No default value.}

\item{...}{(optional) subset conditions compatible with \code{dplyr::filter}.
\code{coxph} will be fitted on this subset only. This argument can be useful
to create a subset of data for analysis when a trial consists of more
than two arms. By default, it is not specified,
all data will be used to fit the model. More than one condition can be
specified in \code{...}, e.g.,
\code{fitCoxph(formula, 'pbo', data, 'less', 'log hazard ratio', arm \%in\% c('pbo', 'low dose'), x > 0.5)},
which is equivalent to:
\code{fitCoxph(formula, 'pbo', data, 'less', 'log hazard ratio', arm \%in\% c('pbo', 'low dose') & x > 0.5)}.
Note that if more than one treatment arm are present in the data after
applying filter in \code{...}, models are fitted and tested for placebo verse
each of the treatment arms.}

\item{tidy}{logical. \code{FALSE} if more information are returned.
Default: \code{TRUE}.}
}
\value{
a data frame with three columns:
\describe{
\item{\code{arm}}{name of the treatment arm. }
\item{\code{placebo}}{name of the placebo arm. }
\item{\code{estimate}}{estimate of main effect of arm, depending on \code{scale}. }
\item{\code{p}}{one-sided p-value for log hazard ratio (treated vs placebo). }
\item{\code{info}}{the number of events of the endpoint in the subset. }
\item{\code{z}}{the z statistics of log hazard ratios. }
}
}
\description{
Fit Cox proportional hazards model on an time-to-event endpoint.

Refer to \href{https://zhangh12.github.io/TrialSimulator/articles/wrappers.html}{this vignette}
for more information and examples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitLogrank.R
\name{fitLogrank}
\alias{fitLogrank}
\title{Carry out log rank test}
\usage{
fitLogrank(formula, placebo, data, alternative, ..., tidy = TRUE)
}
\arguments{
\item{formula}{An object of class \code{formula} that can be used with
\code{survival::coxph}. Must consist \code{arm} and endpoint in \code{data}.
No covariate is allowed. Stratification variables are supported and can be
added using \code{strata(...)}.}

\item{placebo}{character. String of placebo in \code{data$arm}.}

\item{data}{data frame. Usually it is a locked data.}

\item{alternative}{a character string specifying the alternative hypothesis,
must be one of \code{"greater"} or \code{"less"},
i.e., one-sided test is enforced. No default value.
\code{"greater"} means superiority of treatment over placebo is established
by an hazard ratio greater than 1.}

\item{...}{subset condition that is compatible with \code{dplyr::filter}.
\code{survival::coxph} with \code{ties = "exact"} will be fitted on this
subset only. This argument could be useful to create a subset of data for
analysis when a trial consists of more than two arms. By default it is not
specified, all data will be used to fit the model. More than one conditions
can be specified in \code{...}, e.g.,
\code{fitLogrank(formula, data, arm \%in\% c('pbo', 'low dose'), x > 0.5)},
which is equivalent to
\code{fitLogrank(formula, data, arm \%in\% c('pbo', 'low dose') & x > 0.5)}.
Note that if more than one treatment arm are present in the data after
applying filter in \code{...}, models are fitted for placebo verse
each of the treatment arms.}

\item{tidy}{logical. \code{FALSE} if more information are returned.
Default \code{TRUE}.}
}
\value{
a data frame with three columns:
\describe{
\item{\code{arm}}{name of the treatment arm. }
\item{\code{placebo}}{name of the placebo arm. }
\item{\code{p}}{one-sided p-value for log-rank test (treated vs placebo). }
\item{\code{info}}{the number of events of the endpoint in the subset. }
\item{\code{z}}{the z statistics of log hazard ratios. }
}
}
\description{
Compute log rank test statistic on an endpoint.

Refer to \href{https://zhangh12.github.io/TrialSimulator/articles/wrappers.html}{this vignette}
for more information and examples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doNothing.R
\name{doNothing}
\alias{doNothing}
\title{An Action Function that Does Nothing}
\usage{
doNothing(trial, ...)
}
\arguments{
\item{trial}{an object returned from \code{trial()}.}

\item{...}{(optional) arguments. This is for capturing redundant arguments
in \code{milestone()} only.}
}
\value{
This function returns \code{NULL}. Actually, nothing is done in this function.
}
\description{
This is an action function that does nothing when the corresponding milestone
is triggered. When the listener is monitoring a trial and determining the
time to trigger a milestone, data is automatically locked with other necessary
data manipulations (censoring, truncation, etc.) are executed.
If the users have no intent to modify the
trial adaptively at the milestone, e.g., adding (\code{add_arms()}) or
removing (\code{remove_arms()}) arm(s),
changing sampling ratio(s) (\code{update_sample_ratio()}),
modifying trial duration (\code{set_duration()}), carrying out statistical
testing, or saving intermediate results (\code{save()}, etc.),
then this function
can be used to set the argument \code{action} when creating a new milestone.
Note that the triggering time and number of observations/events of endpoints
at a milestone with \code{action = doNothing} is still recorded in output
automatically.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GraphicalTesting.R
\docType{class}
\name{GraphicalTesting}
\alias{GraphicalTesting}
\title{Class of GraphicalTesting}
\description{
Perform graphical testing under group sequential design for one or multiple
endpoints. See Maurer & Bretz (2013).
}
\examples{

## Example 1
## dry-run to study the behavior of a graph
## without group sequential design
if(interactive()){
eps <- .01
alpha <- c(.01, .04, 0, 0, 0)
transition <- matrix(c(
  0, 0, 0, 0, 1,
  0, 0, .75, 0, .25,
  0, 1/2-eps/2, 0, eps, 1/2-eps/2,
  0, 0, 0, 0, 0,
  0, 1/2, 1/2, 0, 0
), nrow = 5, byrow = TRUE)

## dummy can be anything, we don't actually use it
asf <- rep('asOF', 5)
## dummy can be anything, we don't actually use it
max_info <- c(300, 1100, 1100, 1100, 500)

hs <- c('H1: UPCR IgA', 'H2: eGFR GN', 'H3: eGFR GN 10wk', 'H5: 2nd Endpoints', 'H4: eGFR IgA')

## initialize an object
gt <- GraphicalTesting$new(alpha, transition, asf, max_info, hs)
print(gt)

## reject hypotheses based on customized order
## to understand the behavior of a testing strategy
## Any other rejection order is possible
gt$reject_a_hypothesis('H1: UPCR IgA')
print(gt)

gt$reject_a_hypothesis('H2: eGFR GN')
print(gt)

gt$reject_a_hypothesis('H4: eGFR IgA')
print(gt)

gt$reject_a_hypothesis('H3: eGFR GN 10wk')
print(gt)

gt$reset()
}

## Example 2
## Example modified from vignettes in gMCPLite:
## Graphical testing for group sequential design
if(interactive()){
## initial alpha split to each of the hypotheses
alpha <- c(.01, .01, .004, .0, .0005, .0005)

## transition matrix of the initial graph
transition <- matrix(c(
  0, 1, 0, 0, 0, 0,
  0, 0, .5, .5, 0, 0,
  0, 0, 0, 1, 0, 0,
  0, 0, 0, 0, .5, .5,
  0, 0, 0, 0, 0, 1,
  .5, .5, 0, 0, 0, 0
), nrow = 6, byrow = TRUE)

## alpha spending functions per hypothesis
asf <- c('asUser', 'asOF', 'asUser', 'asOF', 'asOF', 'asOF')

## planned maximum number of events per hypothesis
max_info <- c(295, 800, 310, 750, 500, 1100)

## name of hypotheses
hs <- c('H1: OS sub',
        'H2: OS all',
        'H3: PFS sub',
        'H4: PFS all',
        'H5: ORR sub',
        'H6: ORR all')

gt <- GraphicalTesting$new(alpha, transition, asf, max_info, hs)

## print initial graph
gt

## nominal p-values at each stage
## Note: p-values with same order are calculated with the same locked data
## Note: alpha_spent is only specified for hypotheses using custom alpha
##       spending function "asUser"
stats <-
  data.frame(
    order = c(1:3, 1:3, 1:2, 1:2, 1, 1),
    hypotheses = c(rep('H1: OS sub', 3), rep('H2: OS all', 3),
                   rep('H3: PFS sub', 2), rep('H4: PFS all', 2),
                   'H5: ORR sub', 'H6: ORR all'),
    p = c(.03, .0001, .000001, .2, .15, .1, .2, .001, .3, .2, .00001, .1),
    info = c(185, 245, 295, 529, 700, 800, 265, 310, 675, 750, 490, 990),
    is_final = c(F, F, T, F, F, T, F, T, F, T, T, T),
    max_info = c(rep(295, 3), rep(800, 3), rep(310, 2), rep(750, 2), 490, 990),
    alpha_spent = c(c(.1, .4, 1), rep(NA, 3), c(.3, 1), rep(NA, 2), NA, NA)
  )

## test the p-values from the first analysis, plot the updated graph
gt$test(stats \%>\% dplyr::filter(order==1))

## test the p-values from the second analysis, plot the updated graph
gt$test(stats \%>\% dplyr::filter(order==2))

## test the p-values from the third analysis, plot the updated graph
## because no futher test would be done, displayed results are final
gt$test(stats \%>\% dplyr::filter(order==3))


## plot the final status of the graph
print(gt, trajectory = FALSE)

## you can get final testing results as follow
gt$get_current_testing_results()

## if you want to see step-by-step details
print(gt$get_trajectory())

## equivalently, you can call gt$test(stats) for only once to get same results.
gt$reset()
gt$test(stats)

## if you only want to get the final testing results
gt$get_current_decision()
}

}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GraphicalTestingProcedure-new}{\code{GraphicalTesting$new()}}
\item \href{#method-GraphicalTestingProcedure-reset}{\code{GraphicalTesting$reset()}}
\item \href{#method-GraphicalTestingProcedure-is_valid_hid}{\code{GraphicalTesting$is_valid_hid()}}
\item \href{#method-GraphicalTestingProcedure-get_hypothesis_name}{\code{GraphicalTesting$get_hypothesis_name()}}
\item \href{#method-GraphicalTestingProcedure-get_weight}{\code{GraphicalTesting$get_weight()}}
\item \href{#method-GraphicalTestingProcedure-set_weight}{\code{GraphicalTesting$set_weight()}}
\item \href{#method-GraphicalTestingProcedure-get_alpha}{\code{GraphicalTesting$get_alpha()}}
\item \href{#method-GraphicalTestingProcedure-set_alpha}{\code{GraphicalTesting$set_alpha()}}
\item \href{#method-GraphicalTestingProcedure-get_hypotheses_ids}{\code{GraphicalTesting$get_hypotheses_ids()}}
\item \href{#method-GraphicalTestingProcedure-get_number_hypotheses}{\code{GraphicalTesting$get_number_hypotheses()}}
\item \href{#method-GraphicalTestingProcedure-get_hids_not_in_graph}{\code{GraphicalTesting$get_hids_not_in_graph()}}
\item \href{#method-GraphicalTestingProcedure-get_testable_hypotheses}{\code{GraphicalTesting$get_testable_hypotheses()}}
\item \href{#method-GraphicalTestingProcedure-has_testable_hypotheses}{\code{GraphicalTesting$has_testable_hypotheses()}}
\item \href{#method-GraphicalTestingProcedure-is_in_graph}{\code{GraphicalTesting$is_in_graph()}}
\item \href{#method-GraphicalTestingProcedure-is_testable}{\code{GraphicalTesting$is_testable()}}
\item \href{#method-GraphicalTestingProcedure-get_hid}{\code{GraphicalTesting$get_hid()}}
\item \href{#method-GraphicalTestingProcedure-reject_a_hypothesis}{\code{GraphicalTesting$reject_a_hypothesis()}}
\item \href{#method-GraphicalTestingProcedure-set_trajectory}{\code{GraphicalTesting$set_trajectory()}}
\item \href{#method-GraphicalTestingProcedure-get_trajectory}{\code{GraphicalTesting$get_trajectory()}}
\item \href{#method-GraphicalTestingProcedure-test_hypotheses}{\code{GraphicalTesting$test_hypotheses()}}
\item \href{#method-GraphicalTestingProcedure-test}{\code{GraphicalTesting$test()}}
\item \href{#method-GraphicalTestingProcedure-get_current_testing_results}{\code{GraphicalTesting$get_current_testing_results()}}
\item \href{#method-GraphicalTestingProcedure-get_current_decision}{\code{GraphicalTesting$get_current_decision()}}
\item \href{#method-GraphicalTestingProcedure-print}{\code{GraphicalTesting$print()}}
\item \href{#method-GraphicalTestingProcedure-clone}{\code{GraphicalTesting$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-new"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-new}{}}}
\subsection{Method \code{new()}}{
Initialize an object for graphical testing procedure.
Group sequential design is also supported.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$new(
  alpha,
  transition,
  alpha_spending,
  planned_max_info,
  hypotheses = NULL,
  silent = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{alpha}}{initial alpha allocated to each of the hypotheses.}

\item{\code{transition}}{matrix of transition weights. Its diagonals should
be all 0s. The row sums should be 1s (for better power) or
0s (if no outbound edge from a node).}

\item{\code{alpha_spending}}{character vector of same length of \code{alpha}.
Currently it supports \code{'asP'}, \code{'asOF'}, and \code{'asUser'}.}

\item{\code{planned_max_info}}{vector of integers. Maximum numbers of
events (tte endpoints) or patients (non-tte endpoints) at the final
analysis of each hypothesis when planning a trial. The actual numbers
could be different, which can be specified elsewhere.}

\item{\code{hypotheses}}{vector of characters. Names of hypotheses.}

\item{\code{silent}}{\code{TRUE} if muting all messages and not generating
plots.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-reset"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-reset}{}}}
\subsection{Method \code{reset()}}{
reset an object of class \code{GraphicalTesting} to original status
so that it can be reused.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$reset()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-is_valid_hid"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-is_valid_hid}{}}}
\subsection{Method \code{is_valid_hid()}}{
determine if index of a hypothesis is valid
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$is_valid_hid(hid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{hid}}{an integer}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_hypothesis_name"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_hypothesis_name}{}}}
\subsection{Method \code{get_hypothesis_name()}}{
get name of a hypothesis given its index.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_hypothesis_name(hid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{hid}}{an integer}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_weight"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_weight}{}}}
\subsection{Method \code{get_weight()}}{
return weight between two nodes
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_weight(hid1, hid2)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{hid1}}{an integer}

\item{\code{hid2}}{an integer}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-set_weight"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-set_weight}{}}}
\subsection{Method \code{set_weight()}}{
update weight between two nodes
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$set_weight(hid1, hid2, value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{hid1}}{an integer}

\item{\code{hid2}}{an integer}

\item{\code{value}}{numeric value to be set as a weight two nodes}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_alpha"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_alpha}{}}}
\subsection{Method \code{get_alpha()}}{
return alpha allocated to a hypothesis when calling this function.
Note that a function can be called several time with the graph is
updated dynamically. Thus, returned alpha can be different even for
the same \code{hid}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_alpha(hid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{hid}}{an integer}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-set_alpha"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-set_alpha}{}}}
\subsection{Method \code{set_alpha()}}{
update alpha of a hypothesis
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$set_alpha(hid, value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{hid}}{integer. Index of a hypothesis}

\item{\code{value}}{numeric value to be allocated}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_hypotheses_ids"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_hypotheses_ids}{}}}
\subsection{Method \code{get_hypotheses_ids()}}{
return all valid \code{hid}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_hypotheses_ids()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_number_hypotheses"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_number_hypotheses}{}}}
\subsection{Method \code{get_number_hypotheses()}}{
return number of hypotheses, including those been rejected.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_number_hypotheses()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_hids_not_in_graph"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_hids_not_in_graph}{}}}
\subsection{Method \code{get_hids_not_in_graph()}}{
return index of hypotheses that are rejected.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_hids_not_in_graph()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_testable_hypotheses"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_testable_hypotheses}{}}}
\subsection{Method \code{get_testable_hypotheses()}}{
return index of hypotheses with non-zero alphas, thus can be tested
at the current stage.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_testable_hypotheses()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-has_testable_hypotheses"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-has_testable_hypotheses}{}}}
\subsection{Method \code{has_testable_hypotheses()}}{
determine whether at least one hypothesis is testable.
If return \code{FALSE}, the testing procedure is completed.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$has_testable_hypotheses()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-is_in_graph"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-is_in_graph}{}}}
\subsection{Method \code{is_in_graph()}}{
determine whether a hypothesis is not yet rejected (in graph).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$is_in_graph(hid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{hid}}{integer. Index of a hypothesis}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-is_testable"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-is_testable}{}}}
\subsection{Method \code{is_testable()}}{
determine whether a hypothesis has a non-zero alpha allocated.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$is_testable(hid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{hid}}{integer. Index of a hypothesis}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_hid"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_hid}{}}}
\subsection{Method \code{get_hid()}}{
convert hypothesis's name into (unique) index.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_hid(hypothesis)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{hypothesis}}{character. Name of a hypothesis. It is different from
\code{hid}, which is an index.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-reject_a_hypothesis"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-reject_a_hypothesis}{}}}
\subsection{Method \code{reject_a_hypothesis()}}{
remove a node from graph when a hypothesis is rejected
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$reject_a_hypothesis(hypothesis)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{hypothesis}}{name of a hypothesis. It is different from
\code{hid}, which is an index.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-set_trajectory"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-set_trajectory}{}}}
\subsection{Method \code{set_trajectory()}}{
save new testing results at current stage
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$set_trajectory(result)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{result}}{a data frame of specific columns.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_trajectory"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_trajectory}{}}}
\subsection{Method \code{get_trajectory()}}{
return testing results by the time this function is called.
Note that graphical test is carried out in a sequential manner.
Users may want to review the results anytime. Value returned
by this function can possibly vary over time.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_trajectory()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-test_hypotheses"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-test_hypotheses}{}}}
\subsection{Method \code{test_hypotheses()}}{
test hypotheses using p-values (and other information in \code{stats})
base on the current graph. All rows should have the same order
number.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$test_hypotheses(stats)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{stats}}{a data frame. It must contain the following columns:
\describe{
\item{\code{order}}{integer. P-values (among others) of hypotheses that
can be tested at the same time (e.g., an interim, or final analysis)
should be labeled with the same order number.
If a hypothesis is not tested at a stage,
simply don't put it in \code{stats} with that order number.}
\item{\code{hypotheses}}{character. Name of hypotheses to be tested. They
should be identical to those when calling \code{GraphicalTesting$new}.}
\item{\code{p}}{nominal p-values.}
\item{\code{info}}{observed number of events or samples at test. These will
be used to compute information fractions in group sequential design.}
\item{\code{max_info}}{integers. Maximum information at test. At interim,
\code{max_info} should be equal to \code{planned_max_info} when
calling \code{GraphicalTesting$new}. At the final stage of a
hypothesis, one can update it with observed numbers.}
}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-test"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-test}{}}}
\subsection{Method \code{test()}}{
test hypotheses using p-values (and other information in \code{stats})
base on the current graph. Users can call this function multiple times.
P-values of the same order should be passed through \code{stats}
together. P-values of multiple orders can be passed together as well.
For example, if users only have p-values at current stage, they can call
this function and update the graph accordingly. In this case, column
\code{order} in \code{stats} is a constant. They can call this
function again when p-values of next stage is available, where
\code{order} is another integer. In simulation, if p-values of all
stages are on hand, users can call this function to
test them all in a single pass. In this case, column \code{order} in
\code{stats} can have different values.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$test(stats)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{stats}}{a data frame. It must contain the following columns:
\describe{
\item{\code{order}}{integer. P-values (among others) of hypotheses that
can be tested at the same time (e.g., an interim, or final analysis)
should be labeled with the same order number.
If a hypothesis is not tested at a stage,
simply don't put it in \code{stats} with that order number.
If all p-values in \code{stats} are tested at the same stage, \code{order}
can be absent.}
\item{\code{hypotheses}}{character. Name of hypotheses to be tested. They
should be identical to those when calling
\code{GraphicalTesting$new}.}
\item{\code{p}}{nominal p-values.}
\item{\code{info}}{observed number of events or samples at test. These will
be used to compute information fractions in group sequential design.}
\item{\code{max_info}}{integers. Maximum information at test. At interim,
\code{max_info} should be equal to \code{planned_max_info} when
calling \code{GraphicalTesting$new}. At the final stage of a
hypothesis, one can update it with observed numbers.}
\item{\code{alpha_spent}}{accumulative proportion of allocated alpha
to be spent if \code{alpha_spending = "asUser"}. Set it to
\code{NA_real_} otherwise. If no hypothesis uses \code{"asUser"} in
\code{stats}, this column could be ignored. }
}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
a data frame returned by \code{get_current_testing_results}.
It contains details of each of the testing steps.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_current_testing_results"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_current_testing_results}{}}}
\subsection{Method \code{get_current_testing_results()}}{
return testing results with details by the time this function
is called. This function can be called by users by multiple
times, thus the returned value varies over time.
This function is called by \code{GraphicalTesting::test},
and returns a data frame consisting of columns
\describe{
\item{\code{hypothesis}}{name of hypotheses.}
\item{\code{obs_p_value}}{observed p-values.}
\item{\code{max_allocated_alpha}}{maximum allocated alpha for the hypothesis.}
\item{\code{decision}}{\code{'reject'} or \code{'accept'} the hypotheses.}
\item{\code{stages}}{stage of a hypothesis. }
\item{\code{order}}{order number that this hypothesis is tested for the last time.
It is different from \code{stages}.}
\item{\code{typeOfDesign}}{name of alpha spending functions.}
}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_current_testing_results()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-get_current_decision"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-get_current_decision}{}}}
\subsection{Method \code{get_current_decision()}}{
get current decisions for all hypotheses. Returned value could
changes over time because it depends on the stages being tested already.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$get_current_decision()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
a named vector of values \code{"accept"} or \code{"reject"}.
Note that if a hypothesis is not yet tested when calling this function,
the decision for that hypothesis would be \code{"accept"}.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-print"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-print}{}}}
\subsection{Method \code{print()}}{
generic function for \code{print}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$print(graph = TRUE, trajectory = TRUE, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{graph}}{logic. \code{TRUE} if visualizing the current graph,
which can vary over time.}

\item{\code{trajectory}}{logic. \code{TRUE} if print the current data frame of
trajectory, which can vary over time.}

\item{\code{...}}{other arguments supported in \code{gMCPLite::hGraph},
e.g., \code{trhw} and \code{trhh} to control the size of transition box,
and \code{trdigits} to control the digits displayed for transition
weights.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GraphicalTestingProcedure-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GraphicalTestingProcedure-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GraphicalTesting$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitLinear.R
\name{fitLinear}
\alias{fitLinear}
\title{Fit linear regression model}
\usage{
fitLinear(formula, placebo, data, alternative, ...)
}
\arguments{
\item{formula}{an object of class \code{formula}. Must include \code{arm} and
endpoint in \code{data}. Covariates can be adjusted.}

\item{placebo}{Character. String indicating the placebo arm in \code{data$arm}.}

\item{data}{Data frame. Usually it is a locked data set.}

\item{alternative}{a character string specifying the alternative hypothesis,
must be one of \code{"greater"} or \code{"less"},
i.e., one-sided test is enforced. No default value.
\code{"greater"} means superiority of treatment over placebo is established
by a greater mean in treated arm.}

\item{...}{Subset conditions compatible with \code{dplyr::filter}.
\code{glm} will be fitted on this subset only. This argument can be useful
to create a subset of data for analysis when a trial consists of more
than two arms. By default, it is not specified,
all data will be used to fit the model. More than one condition can be
specified in \code{...}, e.g.,
\code{fitLinear(cfb ~ arm, 'pbo', data, 'greater', arm \%in\% c('pbo', 'low dose'), cfb > 0.5)},
which is equivalent to:
\code{fitLinear(cfb ~ arm, 'pbo', data, 'greater', arm \%in\% c('pbo', 'low dose') & cfb > 0.5)}.
Note that if more than one treatment arm are present in the data after
applying filter in \code{...}, models are fitted and tested for placebo verse
each of the treatment arms.}
}
\value{
a data frame with columns:
\describe{
\item{\code{arm}}{name of the treatment arm. }
\item{\code{placebo}}{name of the placebo arm. }
\item{\code{estimate}}{estimate of average treatment effect of \code{arm}. }
\item{\code{p}}{one-sided p-value for between-arm difference (treated vs placebo). }
\item{\code{info}}{sample size used in model with \code{NA} being removed. }
\item{\code{z}}{z statistics of between-arm difference (treated vs placebo). }
}
}
\description{
Fit linear regression model on a continuous endpoint.

Refer to \href{https://zhangh12.github.io/TrialSimulator/articles/wrappers.html}{this vignette}
for more information and examples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solveMixtureExponentialDistribution.R
\name{solveMixtureExponentialDistribution}
\alias{solveMixtureExponentialDistribution}
\title{Solve Parameters in a Mixture Exponential Distribution}
\usage{
solveMixtureExponentialDistribution(
  weight1,
  median1,
  median2 = NULL,
  overall_median = NULL
)
}
\arguments{
\item{weight1}{numeric. The proportion of the first component.}

\item{median1}{numeric. Median of the first component.}

\item{median2}{numeric. Median of the second component. If \code{NULL},
then \code{overall_median} must be specified, and this function will
calculate and return \code{median2}.}

\item{overall_median}{numeric. Median of the overall population. If
\code{NULL}, then \code{median2} must be specified, and this function
will calculate and return \code{overall_median}.}
}
\value{
a named vector of \code{median2} or \code{overall_median}.
}
\description{
This is a helper function to explore parameters for endpoint generator,
likely in an enrichment design.

Assume that the overall population in an arm is a mixture of two exponential
distributions with medians \code{median1} (\eqn{m_1}) and
\code{median2} (\eqn{m_2}). Given the proportion of the first component
(\eqn{p_1}) and the overall median \eqn{m}, we have

\deqn{p_1 (1 - e^{-\log(2)m/m_1}) + (1 - p_1) (1 - e^{-\log(2)m/m_2}) = 1/2}

This function computes \eqn{m_2} or \eqn{m} given \eqn{p_1} and \eqn{m_1}.
These parameters can be used in custom random number generator to define
exponential distributed endpoints.

Note that the math formula above may not be displayed correctly on a
html page. You can read it with better format by running
\code{?solveMixtureExponentialDistribution}.
}
\examples{

library(dplyr)

median2 <-
  solveMixtureExponentialDistribution(
    weight1 = .3,
    median1 = 10,
    overall_median = 8)

median2

n <- 1e6
ifelse(
  runif(n) < .3,
  rexp(n, rate=log(2)/10),
  rexp(n, rate=log(2)/median2)) \%>\%
  median() ## should be close to 8

overall_median <-
  solveMixtureExponentialDistribution(
    weight1 = .4,
    median1 = 12,
    median2 = 4)

overall_median

ifelse(
  runif(n) < .4,
  rexp(n, rate=log(2)/12),
  rexp(n, rate=log(2)/4)) \%>\%
  median() ## should be close to overall_median

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Trial.R
\name{trial}
\alias{trial}
\title{Define a Trial}
\usage{
trial(
  name,
  n_patients,
  duration,
  description = name,
  seed = NULL,
  enroller,
  dropout = NULL,
  stratification_factors = NULL,
  silent = FALSE,
  ...
)
}
\arguments{
\item{name}{character. Name of trial. Usually, hmm..., useless.}

\item{n_patients}{integer. Maximum (and initial) number of patients
could be enrolled when planning the trial. It can be altered adaptively
during a trial.}

\item{duration}{Numeric. Trial duration. It can be altered adaptively
during a trial.}

\item{description}{character. Optional for description of the trial. By
default it is set to be trial's \code{name}. Usually useless.}

\item{seed}{random seed. If \code{NULL}, seed is set for each simulated
trial automatically and saved in output. It can be retrieved in the
\code{seed} column in \code{$get_output()}. Setting it to be \code{NULL}
is recommended. For debugging, set it to a specific integer.}

\item{enroller}{a function returning a vector enrollment time for
patients. Its first argument \code{n} is the number of enrolled patients.
Set it to \code{StaggeredRecruiter} can handle most of the use cases.
See \code{?TrialSimulator::StaggeredRecruiter} for more information.}

\item{dropout}{a function returning a vector of dropout time for patients.
It can be any random number generator with first argument \code{n},
the number of enrolled patients. Usually \code{rexp} if dropout rate
is set at a single time point, or \code{rweibull} if dropout rates are
set at two time points. See \code{?TrialSimulator::weibullDropout}.}

\item{stratification_factors}{character. Names of baseline characteristics
to define stratums in stratified permuted block randomization.
Stratification factors must be defined in \code{endpoint()} with
\code{readout = 0}. As a natural assumption for randomized trial,
\code{TrialSimulator} assumes that the baseline
characteristics share the same distribution across arms, but endpoints
can have same or different distributions given baseline characteristics.
\code{NULL} by default, i.e., unstratified permuted block randomization is
executed.}

\item{silent}{logical. \code{TRUE} to mute messages. However, warning
message is still displayed. Usually set it to \code{TRUE} in formal
simulation. Default: \code{FALSE}.}

\item{...}{(optional) arguments of \code{enroller} and \code{dropout}.}
}
\description{
Define a trial. This is a user-friendly wrapper for
the class constructor \code{Trial$new()}. Users who are not familiar with
the concept of classes may consider using this wrapper directly.

Trial's name, planned size/duration, enrollment plan,
dropout mechanism and seeding are specified in this function. Note that
many of these parameters can be altered adaptively during a trial.

Note that it is users' responsibility to assure that the units of dropout
time, trial duration, and readout of non-tte endpoints are consistent.
}
\examples{
risk1 <- data.frame(
  end_time = c(1, 10, 26.0, 52.0),
  piecewise_risk = c(1, 1.01, 0.381, 0.150) * exp(-3.01)
)

pfs1 <- endpoint(name = 'pfs', type='tte',
          generator = PiecewiseConstantExponentialRNG,
          risk = risk1, endpoint_name = 'pfs')

orr1 <- endpoint(
  name = 'orr', type = 'non-tte',
  readout = c(orr=1), generator = rbinom,
  size = 1, prob = .4)

placebo <- arm(name = 'pbo')

placebo$add_endpoints(pfs1, orr1)

risk2 <- risk1
risk2$hazard_ratio <- .8

pfs2 <- endpoint(name = 'pfs', type='tte',
          generator = PiecewiseConstantExponentialRNG,
          risk = risk2, endpoint_name = 'pfs')

orr2 <- endpoint(
  name = 'orr', type = 'non-tte',
  generator = rbinom, readout = c(orr=3),
  size = 1, prob = .6)

active <- arm(name = 'ac')

active$add_endpoints(pfs2, orr2)

## Plan a trial, Trial-3415, of up to 100 patients.
## Enrollment time follows an exponential distribution, with median 5
trial <- trial(
  name = 'Trial-3415', n_patients = 100,
  seed = 31415926, duration = 100,
  enroller = rexp, rate = log(2) / 5)

trial

trial$add_arms(sample_ratio = c(1, 2), placebo, active)

## updated information after arms are registered
trial

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/StaggeredRecruiter.R
\name{StaggeredRecruiter}
\alias{StaggeredRecruiter}
\title{Generate Enrollment Time from Piecewise Constant Uniform Distribution}
\usage{
StaggeredRecruiter(n, accrual_rate)
}
\arguments{
\item{n}{integer. Number of random numbers.}

\item{accrual_rate}{a data frame of columns
\describe{
      \item{\code{end_time}}{
          End time for a constant rate in a time window. The start time of
          the first time window is 0.}
      \item{\code{piecewise_rate}}{
          A constant rate in a time window. So the number of patients
          being recruited in that window is window length x
          \code{piecewise_rate}. }
}}
}
\description{
It assumes a uniform enrollment with constant rate in each of the time
windows. This function can be used as the \code{enroller} when calling
\code{trial()} to define a trial.
}
\examples{
accrual_rate <- data.frame(
  end_time = c(12, 13:17, Inf),
  piecewise_rate = c(15, 15 + 6 * (1:5), 45)
)

StaggeredRecruiter(30, accrual_rate)

accrual_rate <- data.frame(
  end_time = c(3, 4, 5, 8, Inf),
  piecewise_rate = c(1, 2, 2, 3, 4)
)

StaggeredRecruiter(30, accrual_rate)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eventNumber.R
\name{eventNumber}
\alias{eventNumber}
\title{Triggering Condition by Number of Events or Non-missing Observations of an Endpoint}
\usage{
eventNumber(endpoint, n, ..., arms = NULL)
}
\arguments{
\item{endpoint}{character. Name of an endpoint. It should be something that
is specified in the argument \code{name} in \code{endpoint()}.}

\item{n}{integer. Targeted number of events or non-missing obervations,
depending on the type of endpoint.}

\item{...}{subset conditions compatible with \code{dplyr::filter}. Number
of events/observations will be counted on subset of trial data only.}

\item{arms}{vector of character. Name of arms on which the number of
events/observations is counted. If \code{NULL}, use all arms that are not yet
removed from the trial (using \code{remove_arms()})
by the time of calculation.}
}
\value{
an object of class `Condition`
}
\description{
Define a condition to trigger trial milestone by the number of events of a
time-to-event endpoint or the number of non-missing observations of a
non-time-to-event endpoint. The milestone will be triggered when a trial has
observed at least the specified number of endpoint events (or non-missing
observations). It can be used combined with
conditions specified by \link[TrialSimulator]{calendarTime} and
\link[TrialSimulator]{enrollment}.

Number of events for a time-to-event endpoint can vary at different
milestones as more patients are randomized into a trial, or more events
onset over time.

Number of non-missing observations for a non-time-to-event endpoint can vary
at different milestones as more patients are randomized into a trial, or more
patients have been treated until their readout time (thus, \code{NA} turns
to a value).

Both numbers are affected by dropout.

Refer to the
\href{https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html}{vignette}
to learn how to define milestones when performing simulation using
\code{TrialSimulator}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFixedDesignOutput.R
\name{getFixedDesignOutput}
\alias{getFixedDesignOutput}
\title{Get simulation output in the vignette fixedDesign.Rmd}
\usage{
getFixedDesignOutput()
}
\value{
A data frame containing simulation results of 1000 replicates.
}
\description{
Internal function that retrieves precomputed simulation results.
Not meant for use by package users.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regimen.R
\name{regimen}
\alias{regimen}
\title{Define a Regimen}
\usage{
regimen(what, when, how, ...)
}
\arguments{
\item{what}{a function determining whether patients' data would be
updated due to switching treatment. It takes \code{patient_data},
a data frame as argument, and returns a data frame of two columns
\code{patient_id} and \code{new_treatment}. Patients with \code{NA} in
\code{new_treatment} will be skipped.
The number of rows in the returned data frame may be smaller than the
number of patients in the input data frame. This indicates that some
patients' data will not be modifier.
Note that the returned object will be passed into function `how()`, which
is also provide by users. This argument can also be a
list of functions that will be executed sequentially. No default value.}

\item{when}{a function determining the time at which a patient switches
to another treatment regimen, measured from the time of enrollment.
It takes \code{patient_data}, a data frame as
argument, and returns a data frame of two columns \code{patient_id} and
\code{switch_time} (from \code{enroll_time}). No \code{NA} is allowed
in \code{switch_time} and the number of rows in the returned data frame
must equal the number of rows in \code{patient_data}, i.e., switching
time must be specified to every patients.
Note that the returned object will be passed into function `how()`, which
is also provided by users. This argument can also be a
list of functions that will be executed sequentially. No default value.}

\item{how}{a function updating patients' data after treatment switching.
Only modified columns and \code{patient_id} are returned. A cell will
be omitted if \code{NA}, meaning no change to that patient for the endpoint
or other variables. Equivalently, users can also fill the cell with
its original value. This argument can also be a list of functions that
will be executed sequentially. No default value.}

\item{...}{(optional) named arguments to be passed to one or more of
\code{what}, \code{when}, and \code{how}. Each argument is routed to every
function whose formal parameter list contains that name. All arguments must
be named, and every name must match at least one parameter of at least one
function in \code{what}, \code{when}, or \code{how}.}
}
\description{
Define a regimen of a trial. This is a user-friendly wrapper for
the class constructor \code{Regimens$new()}. Users who are not familiar with
the concept of classes may consider using this wrapper directly.

A regimen defines the rules to select patients who
switch treatments, to determine the time of switching, and to update patients'
endpoint data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Listeners.R
\docType{class}
\name{Listeners}
\alias{Listeners}
\title{Class of Listener}
\description{
Create a class of listener. A listener monitors the trial while checking
condition of pre-defined milestones. Actions are triggered and executed
automatically.

Public methods in this R6 class are used in developing
this package. Thus, we have to export the whole R6 class which exposures all
public methods. However, only the public methods in the list below are
useful to end users.

\itemize{
\item \code{$add_milestones()}
}
}
\examples{
##

## ------------------------------------------------
## Method `Listeners$add_milestones`
## ------------------------------------------------

listener <- listener()
interim <- milestone(name = 'interim',
                     when = eventNumber('endpoint', n = 100)
                    )
final <- milestone(name = 'final',
                   when = calendarTime(time = 24)
                  )
listener$add_milestones(interim, final)

}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Listeners-new}{\code{Listeners$new()}}
\item \href{#method-Listeners-add_milestones}{\code{Listeners$add_milestones()}}
\item \href{#method-Listeners-get_milestones}{\code{Listeners$get_milestones()}}
\item \href{#method-Listeners-get_milestone_names}{\code{Listeners$get_milestone_names()}}
\item \href{#method-Listeners-monitor}{\code{Listeners$monitor()}}
\item \href{#method-Listeners-mute}{\code{Listeners$mute()}}
\item \href{#method-Listeners-reset}{\code{Listeners$reset()}}
\item \href{#method-Listeners-clone}{\code{Listeners$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Listeners-new"></a>}}
\if{latex}{\out{\hypertarget{method-Listeners-new}{}}}
\subsection{Method \code{new()}}{
initialize a listener
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Listeners$new(silent = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{silent}}{logical. \code{TRUE} to mute messages.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Listeners-add_milestones"></a>}}
\if{latex}{\out{\hypertarget{method-Listeners-add_milestones}{}}}
\subsection{Method \code{add_milestones()}}{
register milestones with listener. Order in \code{...} matter
as they are scanned and triggered in that order. It is users'
responsibility to use reasonable order when calling this function,
otherwise, the result of \code{Listeners$monitor()} can be problematic.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Listeners$add_milestones(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{one or more objects returned from \code{milestone()}.}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{listener <- listener()
interim <- milestone(name = 'interim',
                     when = eventNumber('endpoint', n = 100)
                    )
final <- milestone(name = 'final',
                   when = calendarTime(time = 24)
                  )
listener$add_milestones(interim, final)

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Listeners-get_milestones"></a>}}
\if{latex}{\out{\hypertarget{method-Listeners-get_milestones}{}}}
\subsection{Method \code{get_milestones()}}{
return registered milestones
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Listeners$get_milestones(milestone_name = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{milestone_name}}{return \code{Milestone} object with given name(s).
If \code{NULL}, all registered milestones are returned.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Listeners-get_milestone_names"></a>}}
\if{latex}{\out{\hypertarget{method-Listeners-get_milestone_names}{}}}
\subsection{Method \code{get_milestone_names()}}{
return names of registered milestones
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Listeners$get_milestone_names()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Listeners-monitor"></a>}}
\if{latex}{\out{\hypertarget{method-Listeners-monitor}{}}}
\subsection{Method \code{monitor()}}{
scan, check, and trigger registered milestones.
Milestones are triggered in the order when calling
\code{Listener$add_milestones}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Listeners$monitor(trial, dry_run)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{trial}}{a \code{Trial} object.}

\item{\code{dry_run}}{logical. See \code{Controller::run} for more information.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Listeners-mute"></a>}}
\if{latex}{\out{\hypertarget{method-Listeners-mute}{}}}
\subsection{Method \code{mute()}}{
mute all messages (not including warnings)
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Listeners$mute(silent)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{silent}}{logical.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Listeners-reset"></a>}}
\if{latex}{\out{\hypertarget{method-Listeners-reset}{}}}
\subsection{Method \code{reset()}}{
reset all milestones registered to the listener. Usually, this is called
before a controller can run additional replicates of simulation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Listeners$reset()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Listeners-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Listeners-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Listeners$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDoseRangingOutput.R
\name{getDoseRangingOutput}
\alias{getDoseRangingOutput}
\title{Get simulation output in the vignette doseRanging.Rmd}
\usage{
getDoseRangingOutput()
}
\value{
A data frame containing simulation results of 10 replicates.
}
\description{
Internal function that retrieves precomputed simulation results.
Not meant for use by package users.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_arms.R
\name{remove_arms}
\alias{remove_arms}
\title{Removing One or More Arms From a Trial}
\usage{
remove_arms(trial, arms_name)
}
\arguments{
\item{trial}{a trial object returned by \code{trial()}.}

\item{arms_name}{character vector. Name of arms to be removed.}
}
\description{
remove arms from a trial. The application of this function includes, but is
not limited to, dose selection, enrichment analysis
(select sub-population).

Note that this function should only be called within action functions of
milestones. It is users' responsibility to ensure that and
\code{TrialSimulator} has no way to track it. In addition, data of the
removed arms are censored or truncated by the time of arm removal.

This is a user-friendly wrapper of the member function of trial, i.e.,
\code{Trials$remove_arms()}, which is used in vignettes. Users who are not
familiar with the concept of classes may consider using this wrapper
directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getAdaptiveDesignOutput.R
\name{getAdaptiveDesignOutput}
\alias{getAdaptiveDesignOutput}
\title{Get simulation output in the vignette adaptiveDesign.Rmd}
\usage{
getAdaptiveDesignOutput()
}
\value{
A data frame containing simulation results of 1000 replicates.
}
\description{
Internal function that retrieves precomputed simulation results.
Not meant for use by package users.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarizeTable.R
\name{summarizeDataFrame}
\alias{summarizeDataFrame}
\title{Summarize A Data Frame}
\usage{
summarizeDataFrame(
  data,
  exclude_vars = NULL,
  tte_vars = NULL,
  event_vars = NULL,
  categorical_vars = NULL,
  title = "Summary",
  sub_title = ""
)
}
\arguments{
\item{data}{a data frame.}

\item{exclude_vars}{columns to be excluded from summary.}

\item{tte_vars}{character. Vector of time-to-event variables.}

\item{event_vars}{character. Vector of event indicators. Every time-to-event
variable should be corresponding to an event indicator.}

\item{categorical_vars}{character. Vector of categorical variables. This can
be used to specify variables with limited distinct values as categorical
variables in summary.}

\item{title}{character. Title of the summary report.}

\item{sub_title}{character. Sub-title.}
}
\value{
a data frame of summary
}
\description{
A minimum alternative to \code{summarytools::dfSummary} to avoid package
dependency. This function is used to generate summary reports of endpoints
and arms. No meant to be used by end users. However, users may
find it helpful in their own applications if the interface is okay with them.
}
\examples{

set.seed(123)

n <- 1000
data <- data.frame(
  age = rnorm(n, 65, 10),
  gender = sample(c('M', 'F', NA), n, replace = TRUE, prob = c(.4, .4, .2)),
  time_to_death = rexp(n, .01),
  death = rbinom(n, 1, .6),
  type = sample(LETTERS[1:8], n, replace = TRUE)
)

summarizeDataFrame(data, tte_vars = 'time_to_death', event_vars = 'death')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarizeMilestoneTime.R
\name{plot.milestone_time_summary}
\alias{plot.milestone_time_summary}
\title{Plot Triggering Time of Milestones in Simulated Trials}
\usage{
\method{plot}{milestone_time_summary}(x, ...)
}
\arguments{
\item{x}{an object returned by \code{summarizeMilestoneTime()}.}

\item{...}{currently not supported.}
}
\description{
Plot Triggering Time of Milestones in Simulated Trials
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DynamicRNGFunction.R
\name{DynamicRNGFunction}
\alias{DynamicRNGFunction}
\title{A wrapper of random number generator.}
\usage{
DynamicRNGFunction(fn, ...)
}
\arguments{
\item{fn}{random number generator, e.g., \code{rnorm}, \code{rchisq},
etc. It can be user-defined random number generator as well, e.g.,
\code{PiecewiseConstantExponentialRNG}.}

\item{...}{arguments for \code{fn}. Specifying invalid arguments can trigger error and
be stopped. There are three exceptions. (1) \code{rng} can be passed through
\code{...} to give true name of \code{fn}. This could be necessary as it may be
hard to parse it accurately in \code{DynamicRNGFunction}, or simply for a more
informative purpose in some scenarios. (2) \code{var_name} can be passed
through \code{...} to specify the name of generated variable. (3) \code{simplify}
can be set to \code{FALSE} to convert a vector into a one-column data frame in returned
object. This happens for built-in random number generators, e.g., \code{rnorm},
\code{rbinom}, etc. These three arguments will not be passed into \code{fn}.}
}
\value{
a function to generate random number based on \code{fn} and arguments in
\code{...}. Specified arguments will be fixed and cannot be changed when invoking
\code{DynamicRNGFunction(fn, ...)()}. For example,
if \code{foo <- DynamicRNGFunction(rnorm, sd = 2)},
then \code{foo(n = 100)} will always generate data from normal distribution of
variance 4. \code{foo(n = 100, sd = 1)} will trigger an error. However,
if an argument is not specified in \code{DynamicRNGFunction}, then it can be specified
later. For example, \code{foo(n = 100, mean = -1)} will generate data from N(-1, 4).
}
\description{
This function may be useful to advanced users of \code{TrialSimulator}. It
creates a wrapper function of a random number generator, while fixing a
subset or all of arguments. This function is design to prevent inadvertent
changing to arguments of random number generator. See examples below.
}
\examples{
# example code
dfunc <- DynamicRNGFunction(rnorm, sd = 3.2)
x <- dfunc(1e3) # mean 0 and sd 3.2
hist(x)

y <- dfunc(1e3, mean = 3.5) # mean can be changed
mean(y)

try(z <- dfunc(1e3, sd = 1)) # error because sd is fixed in dfunc

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Arm.R
\name{arm}
\alias{arm}
\title{Define an Arm}
\usage{
arm(name, ...)
}
\arguments{
\item{name}{character. Name of arm, which is the arm's label in generated
trial data, i.e., the one retrieved by calling \code{Trials$get_locked_data()}
in action functions.}

\item{...}{subset condition that is compatible with \code{dplyr::filter}.
This can be used to specify inclusion criteria of an arm.
By default it is not specified, i.e. all data generated by the generator
will be used as trial data. More than one conditions can be
specified in \code{...}.}
}
\description{
Define an arm in a trial. This is a user-friendly wrapper for
the class constructor \code{Arms$new()}. Users who are not familiar with
the concept of classes may consider using this wrapper directly.
}
\examples{

risk <- data.frame(
  end_time = c(1, 10, 26.0, 52.0),
  piecewise_risk = c(1, 1.01, 0.381, 0.150) * exp(-3.01)
)

pfs <- endpoint(name = 'pfs', type='tte',
generator = PiecewiseConstantExponentialRNG,
risk = risk, endpoint_name = 'pfs')

orr <- endpoint(
  name = 'orr', type = 'non-tte',
  readout = c(orr = 2), generator = rbinom,
  size = 1, prob = .4)

placebo <- arm(name = 'pbo')

placebo$add_endpoints(pfs, orr)

## try to generate some data from the arm
## it is NOT a recommended way to use the package in simulation
head(placebo$get_endpoints()[[1]]$get_generator()(n = 1e3))

## get name of endpoints in the arm
## for illustration only, NOT recommended
placebo$get_endpoints()[[2]]$get_name()

## run it in console to get summary report
## It is the recommended way to view an arm
placebo

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CorrelatedPfsAndOs2.R
\name{CorrelatedPfsAndOs2}
\alias{CorrelatedPfsAndOs2}
\title{Generate Correlated PFS and OS Using Gumbel Copula}
\usage{
CorrelatedPfsAndOs2(
  n,
  median_pfs,
  median_os,
  kendall,
  pfs_name = "pfs",
  os_name = "os"
)
}
\arguments{
\item{n}{integer. Number of observations.}

\item{median_pfs}{numeric. Median of PFS.}

\item{median_os}{numeric. Median of OS.}

\item{kendall}{numeric. Kendall's tau between observed, uncensored PFS and OS.
Must be positive and usually away from zero. Note that
this argument is not the Kendall's tau between TTP and OS.}

\item{pfs_name}{column name of PFS in returned data frame. It must be
consistent with name in the function \code{endpoint()}.}

\item{os_name}{column name of OS in returned data frame. It must be
consistent with name in the function \code{endpoint()}.}
}
\value{
A data frame of \code{n} rows and four columns, including PFS, OS and their
event indicators. The event indicators are all 1s. The column names are
\code{<pfs_name>}, \code{<pfs_name>_event}, \code{<os_name>},
and \code{<os_name>_event}.
}
\description{
Generate correlated PFS and OS endpoints using the Gumbel copula. Marginally,
both PFS and OS follow exponential distributions. This function can be used
as custom \code{generator} in the function \code{endpoint()}.

Note that the Gumbel copula is applied to the survival functions of OS and
time-to-progression (TTP). PFS is defined as min(TTP, OS), which also
follows an exponential distribution.

For more information, refer to
\href{https://zhangh12.github.io/TrialSimulator/articles/simulatePfsAndOsGumbel.html}{this vignette}.
}
\examples{
pfs_and_os <- endpoint(name = c('PFS', 'Os'),
                       type = c('tte', 'tte'),
                       generator = CorrelatedPfsAndOs2,
                       median_pfs = 5,
                       median_os = 11,
                       kendall = .6,
                       pfs_name = 'PFS',
                       os_name = 'Os')

pfs_and_os # run it in console to see summary report

## for validation purpose only
## not the recommended way to use TrialSimulator
dat <- pfs_and_os$test_generator(n = 1e4)
cor(dat[, 1:2], method = 'kendall') ## close to 0.6

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calendarTime.R
\name{calendarTime}
\alias{calendarTime}
\title{Triggering Condition by Calendar Time}
\usage{
calendarTime(time)
}
\arguments{
\item{time}{numeric. Calendar time to trigger a milestone of a trial.}
}
\value{
an object of class `Condition`
}
\description{
Define a condition to trigger trial milestone by calendar time. The
milestone will be triggered when a trial has been running for at least the
specified duration since the first patient is enrolled.
It can be used combined with conditions specified by
\link[TrialSimulator]{enrollment} and \link[TrialSimulator]{eventNumber}.

Refer to the
\href{https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html}{vignette}
to learn how to define milestones when performing simulation using
\code{TrialSimulator}.
}
\examples{
milestone(name = 'end of trial', when = calendarTime(time = 12))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rconst.R
\name{rconst}
\alias{rconst}
\title{Generate Constant Variable}
\usage{
rconst(n, value)
}
\arguments{
\item{n}{integer. Number of observations.}

\item{value}{scalar. Value of constant observations.}
}
\description{
A random number generator returning only a constant. This can be used to
set dropout time. Currently it is the default value of dropout time, with
\code{value = Inf}.

This function can also be used as a generator of \code{endpoint()} if a
constant endpoint is needed.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expandRegimen.R
\name{expandRegimen}
\alias{expandRegimen}
\title{Expand regimen trajectory into long format}
\usage{
expandRegimen(data)
}
\arguments{
\item{data}{a data frame returned by \code{trial$get_locked_data()}.}
}
\value{
a long-format data frame: one row per regimen segment per patient,
with \code{regimen} and \code{switch_time_from_enrollment} appended and
\code{regimen_trajectory} removed.
}
\description{
Expands the compact \code{regimen_trajectory} column in locked data
(returned by \code{trial$get_locked_data()}) into a long-format data frame
with one row per regimen segment per patient.

The \code{regimen_trajectory} column stores each patient's treatment history
as a semicolon-separated string of \code{"name\@time"} entries, e.g.
\code{"placebo\@0;low dose\@8.5"}.  \code{expandRegimen} parses this into
two additional columns:
\itemize{
  \item \code{regimen} — name of the treatment regimen
  \item \code{switch_time_from_enrollment} — time from enrollment at which
        the patient switched to this regimen
}
The \code{regimen_trajectory} column is dropped from the result.
}
\examples{
\dontrun{
locked <- trial$get_locked_data('final')
long   <- expandRegimen(locked)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_generator.R
\name{update_generator}
\alias{update_generator}
\title{Updating Data Generators of One or More Endpoints in an Arm}
\usage{
update_generator(trial, arm_name, endpoint_name, generator, ...)
}
\arguments{
\item{trial}{a trial object returned by \code{trial()}.}

\item{arm_name}{character. Name of an arm.}

\item{endpoint_name}{character. A vector of endpoint names whose generator
is updated.}

\item{generator}{a random number generation (RNG) function.
See generator of \code{endpoint()}.}

\item{...}{optional arguments for generator.}
}
\description{
update endpoint generator in an arm. This function can be useful in, e.g.,
enrichment design where generator is updated so that patients are enrolled
from sub-population afterwards. This function can also be used when data
model changes over time, i.e., generating data in a new way after a milestone.
This function can be called multiple times to update generators of endpoints
one by one.

Note that this function should only be called within action functions of
milestones. It is users' responsibility to ensure that and
\code{TrialSimulator} has no way to track it.

This is a user-friendly wrapper of the member function of trial, i.e.,
\code{Trials$update_generator()}, which is used in vignettes. Users who are
not familiar with the concept of classes may consider using this wrapper
directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Arms.R
\docType{class}
\name{Arms}
\alias{Arms}
\title{Class of Arm}
\description{
Create a class of arm.

Public methods in this R6 class are used in developing
this package. Thus, we have to export the whole R6 class which exposures all
public methods. However, only the public methods in the list below are
useful to end users.

\itemize{
\item \code{$add_endpoints()}
\item \code{$print()}
}
}
\examples{
# Instead of using Arms$new(), please use arm(), a user-friendly
# wrapper. See examples in ?arm


## ------------------------------------------------
## Method `Arms$add_endpoints`
## ------------------------------------------------


a <- arm(name = 'trt')
x <- endpoint(name = 'x', type = 'tte',
              generator = rexp) # median = log(2)/1 = 0.7
y <- endpoint(name = 'y', type = 'non-tte', readout = c(y = 0),
              generator = rnorm, sd = 1.4, mean = 0.7)

a$add_endpoints(y, x)

## run it in console to see the summary report
a

print(a) # use the print method

}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Arms-new}{\code{Arms$new()}}
\item \href{#method-Arms-add_endpoints}{\code{Arms$add_endpoints()}}
\item \href{#method-Arms-get_name}{\code{Arms$get_name()}}
\item \href{#method-Arms-get_number_endpoints}{\code{Arms$get_number_endpoints()}}
\item \href{#method-Arms-has_endpoint}{\code{Arms$has_endpoint()}}
\item \href{#method-Arms-get_endpoints}{\code{Arms$get_endpoints()}}
\item \href{#method-Arms-get_endpoints_name}{\code{Arms$get_endpoints_name()}}
\item \href{#method-Arms-update_endpoint_generator}{\code{Arms$update_endpoint_generator()}}
\item \href{#method-Arms-generate_data}{\code{Arms$generate_data()}}
\item \href{#method-Arms-print}{\code{Arms$print()}}
\item \href{#method-Arms-clone}{\code{Arms$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-new"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-new}{}}}
\subsection{Method \code{new()}}{
initialize an arm
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$new(name, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name of arm, which is the arm's label in generated data}

\item{\code{...}}{subset condition that is compatible with \code{dplyr::filter}.
This can be used to specify inclusion criteria of an arm.
By default it is not specified, i.e. all data generated by the generator
will be used as trial data. More than one conditions can be
specified in \code{...}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-add_endpoints"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-add_endpoints}{}}}
\subsection{Method \code{add_endpoints()}}{
add one or multiple endpoints to the arm.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$add_endpoints(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{one or more objects returned from \code{endpoint()}.}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{
a <- arm(name = 'trt')
x <- endpoint(name = 'x', type = 'tte',
              generator = rexp) # median = log(2)/1 = 0.7
y <- endpoint(name = 'y', type = 'non-tte', readout = c(y = 0),
              generator = rnorm, sd = 1.4, mean = 0.7)

a$add_endpoints(y, x)

## run it in console to see the summary report
a

print(a) # use the print method

}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-get_name"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-get_name}{}}}
\subsection{Method \code{get_name()}}{
return name of arm.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$get_name()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-get_number_endpoints"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-get_number_endpoints}{}}}
\subsection{Method \code{get_number_endpoints()}}{
return number of endpoints in the arm.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$get_number_endpoints()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-has_endpoint"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-has_endpoint}{}}}
\subsection{Method \code{has_endpoint()}}{
check if the arm has any endpoint. Return \code{TRUE} or \code{FALSE}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$has_endpoint()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-get_endpoints"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-get_endpoints}{}}}
\subsection{Method \code{get_endpoints()}}{
return a list of endpoints in the arm.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$get_endpoints()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-get_endpoints_name"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-get_endpoints_name}{}}}
\subsection{Method \code{get_endpoints_name()}}{
return name of endpoints registered to the arm.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$get_endpoints_name()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-update_endpoint_generator"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-update_endpoint_generator}{}}}
\subsection{Method \code{update_endpoint_generator()}}{
update generator of an endpoint object
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$update_endpoint_generator(endpoint_name, generator, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{endpoint_name}}{character. A vector of endpoint names whose
generator is updated.}

\item{\code{generator}}{a random number generation (RNG) function.
See \code{generator} of \code{endpoint()}.}

\item{\code{...}}{optional arguments for \code{generator}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-generate_data"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-generate_data}{}}}
\subsection{Method \code{generate_data()}}{
generate arm data.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$generate_data(n_patients_in_arm)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{n_patients_in_arm}}{integer.
Number of patients randomized to the arm.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-print"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-print}{}}}
\subsection{Method \code{print()}}{
print an arm.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$print(categorical_vars = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{categorical_vars}}{character vector of categorical variables.
This can be used to specify variables with limited distinct (numeric)
values as categorical variables in summary report.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Arms-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Arms-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Arms$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qPiecewiseExponential.R
\name{qPiecewiseExponential}
\alias{qPiecewiseExponential}
\title{Quantile Function of Piecewise Exponential Distribution}
\usage{
qPiecewiseExponential(p, times, piecewise_risk)
}
\arguments{
\item{p}{numeric. A vector of probabilities.}

\item{times}{numeric. A vector of time points where risk (event
rates) change. 0 shouldn't be in \code{times}.}

\item{piecewise_risk}{numeric. A vector of constant risk (event rates) in
a time window. \code{length(piecewise_risk) = length(times) + 1}. To fully
specify a piecewise exponential distribution, the number of risk parameters
is one greater than the number of changepoints in \code{times}.}
}
\value{
A vector of quantiles.
}
\description{
To generate endpoint that follows a piecewise exponential distribution and
is correlated with other endpoints, the copula method is commonly used. The
quantile function needs to be specified (e.g., in the \code{simdata} package).
If a piecewise exponential distributed endpoint is independent to other
endpoints, one can simply use
\code{TrialSimulator::PiecewiseConstantExponentialRNG()}
to specify the \code{generator} argument in \code{endpoint()}.

There are many R packages implementing the quantile function of the
piecewise exponential distributed random variable. Why do I implement it
again? The reason is that this function is extremely important for simulating
time-to-event endpoint in clinical trial simulation, thus the speed matters.
\code{qPiecewiseExponential()} is implemented purely in R for
code transparency, and is much faster than other packages.
}
\examples{
## This code snippet can take > 10s to execute
if(interactive()){
library(TrialSimulator)

run <- TRUE
if(!requireNamespace("rpact", quietly = TRUE)){
  run <- FALSE
  message("Please install 'rpact' to run this example.")
}

if(!requireNamespace("PWEXP", quietly = TRUE)){
  run <- FALSE
  message("Please install 'PWEXP' to run this example.")
}

if(!requireNamespace("PWEALL", quietly = TRUE)){
  run <- FALSE
  message("Please install 'PWEALL' to run this example.")
}

if(run){

x <- solvePiecewiseConstantExponentialDistribution(
  surv_prob = c(.9, .75, .64, .42, .28),
  times = c(.4, 1.2, 4, 5.5, 9)
)

p <- stats::runif(1e5)

## fast, and only five lines of R codes
message('TrialSimulator::qPiecewiseExponential(): ')
system.time(
  a <- qPiecewiseExponential(
    p, times = x$end_time, piecewise_risk = c(x$piecewise_risk, .1)
  )
) |> print()

## > 10s
message('rpact::getPiecewiseExponentialQuantile(): ')
system.time(
  b <- rpact::getPiecewiseExponentialQuantile(
    p, piecewiseSurvivalTime = c(0, x$end_time),
    piecewiseLambda=c(x$piecewise_risk, .1)
  )
) |> print()

## equally fast, but implemented in Fortran
message('PWEALL::qpwe(): ')
system.time(
  c <- PWEALL::qpwe(p, rate = c(x$piecewise_risk, .1), tchange = c(0, x$end_time))$q
) |> print()

## equally fast, long codes in R (maybe more versatile?)
message('PWEXP::qpwexp(): ')
system.time(
  d <- PWEXP::qpwexp(p, rate = c(x$piecewise_risk, .1), breakpoint = x$end_time)
) |> print()

message('a == b: ')
all.equal(a, b) |> print()
message('a == c: ')
all.equal(a, c) |> print()
message('a == d: ')
all.equal(a, d) |> print()

}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Controllers.R
\docType{class}
\name{Controllers}
\alias{Controllers}
\title{Class of Controller}
\description{
Create a class of controller to run a trial.

Public methods in this R6 class are used in developing
this package. Thus, we have to export the whole R6 class which exposures all
public methods. However, only the public methods in the list below are
useful to end users.

\itemize{
\item \code{$run()}
\item \code{$get_output()}
\item \code{$reset()}
}
}
\examples{
##
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-Controllers-new}{\code{Controllers$new()}}
\item \href{#method-Controllers-get_listener}{\code{Controllers$get_listener()}}
\item \href{#method-Controllers-get_trial}{\code{Controllers$get_trial()}}
\item \href{#method-Controllers-mute}{\code{Controllers$mute()}}
\item \href{#method-Controllers-reset}{\code{Controllers$reset()}}
\item \href{#method-Controllers-get_output}{\code{Controllers$get_output()}}
\item \href{#method-Controllers-run}{\code{Controllers$run()}}
\item \href{#method-Controllers-clone}{\code{Controllers$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Controllers-new"></a>}}
\if{latex}{\out{\hypertarget{method-Controllers-new}{}}}
\subsection{Method \code{new()}}{
initialize a controller of the trial
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Controllers$new(trial, listener)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{trial}}{a trial object returned from \code{trial()}.}

\item{\code{listener}}{a listener object returned from \code{listener()}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Controllers-get_listener"></a>}}
\if{latex}{\out{\hypertarget{method-Controllers-get_listener}{}}}
\subsection{Method \code{get_listener()}}{
return listener in a controller.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Controllers$get_listener()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Controllers-get_trial"></a>}}
\if{latex}{\out{\hypertarget{method-Controllers-get_trial}{}}}
\subsection{Method \code{get_trial()}}{
return trial in a controller.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Controllers$get_trial()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Controllers-mute"></a>}}
\if{latex}{\out{\hypertarget{method-Controllers-mute}{}}}
\subsection{Method \code{mute()}}{
mute all messages (not including warnings).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Controllers$mute()}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{silent}}{logical.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Controllers-reset"></a>}}
\if{latex}{\out{\hypertarget{method-Controllers-reset}{}}}
\subsection{Method \code{reset()}}{
reset the trial and listener registered to the controller before running
additional replicate of simulation. This is usually done between two
calls of \code{controller$run()}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Controllers$reset()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Controllers-get_output"></a>}}
\if{latex}{\out{\hypertarget{method-Controllers-get_output}{}}}
\subsection{Method \code{get_output()}}{
return a data frame of all current outputs saved by calling \code{save()}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Controllers$get_output(cols = NULL, simplify = TRUE, tidy = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{cols}}{character vector. Columns to be returned from the data frame
of simulation outputs. If \code{NULL}, all columns are returned.}

\item{\code{simplify}}{logical. Return vector rather than a data frame of one
column when \code{length(cols) == 1} and \code{simplify == TRUE}.}

\item{\code{tidy}}{logical. \code{TrialSimulator} automatically records a set
of standard outputs at milestones, even when \code{doNothing} is used
as action functions. These includes time of triggering milestones,
number of observed events for time-to-event endpoints, and number of
non-missing readouts for non-TTE endpoints
(see \code{vignette('actionFunctions')}). This usually mean a large
number of columns in outputs. If users have no intent to summarize a
trial on these columns, setting \code{tidy = TRUE} can eliminate these
columns from \code{get_output()}. This is useful to reduced the size of
output data frame when a large number of replicates are done for
simulation. Note that currently we use regex
\code{"^n_events_<.*?>_<.*?>$"} and
\code{"^milestone_time_<.*?>$"} to match columns to be eliminated.
If users plan to use \code{tidy = TRUE}, caution is needed when naming
custom outputs in \code{save()}. Default \code{FALSE}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Controllers-run"></a>}}
\if{latex}{\out{\hypertarget{method-Controllers-run}{}}}
\subsection{Method \code{run()}}{
run trial simulation.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Controllers$run(
  n = 1,
  n_workers = 1,
  plot_event = TRUE,
  silent = FALSE,
  dry_run = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{n}}{integer. Number of replicates of simulation.
\code{n = 1} by default. Simulation results can be accessed by
\code{controller$get_output()}.}

\item{\code{n_workers}}{integer. Number of parallel workers. When
\code{n_workers = 1} (default), replicates are run sequentially.
When \code{n_workers > 1}, replicates are distributed across
parallel workers using the \code{mirai} package, which must be
installed separately. Each worker receives a serialized copy of
the trial and listener objects and runs its share of replicates
independently. If any replicate encounters an error, execution
stops and already-collected results are preserved in
\code{$get_output()}. To debug, manually set \code{seed} in
\code{trial()} and \code{n_workers = 1} in \code{run()} for reproduced
results. Note that optimal \code{n_workers} may not be
\code{parallel::detectCores()}. For example, Macbook with M1/M2/M3 chips
may have performance cores and efficiency cores. To achieve the best
parallel performance, one may want to use the performance cores only.
For a M1 laptop with 4 performance cores, \code{n_workers = 3} may give
the best performance.}

\item{\code{plot_event}}{logical. Create event plot if \code{TRUE}. Forced to
\code{FALSE} when \code{n > 1} or \code{n_workers > 1}.}

\item{\code{silent}}{logical. \code{TRUE} if muting all messages during a
trial. Note that warning messages are still displayed.}

\item{\code{dry_run}}{logical. We are considering retire this argument.
\code{TRUE} if action function provided by users is
ignored and an internal default action \code{.default_action} is called
instead. This default function only locks data when the milestone is
triggered. Milestone time and number of endpoints' events or sample sizes
are saved. It is suggested to set \code{dry_run = TRUE} to estimate
distributions of triggering time and number of events before formally
using custom action functions if a fixed design is in use.
This helps determining planned maximum
information for group sequential design and reasonable time of milestone
of interest when planning a trial. Set it to \code{FALSE} for formal
simulations. However, for an adaptive design where arm(s) could
possibly be added or removed, setting \code{dry_run} to \code{TRUE}
is usually not helpful because adaption should be executed
before estimating the milestone time.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-Controllers-clone"></a>}}
\if{latex}{\out{\hypertarget{method-Controllers-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{Controllers$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarizeMilestoneTime.R
\name{summarizeMilestoneTime}
\alias{summarizeMilestoneTime}
\title{Summary of Milestone Time from Simulated Trials}
\usage{
summarizeMilestoneTime(output)
}
\arguments{
\item{output}{a data frame.
It assumes that triggering time of milestones
are store in columns \code{milestone_time_<...>}. It can be a data frame
returned by \code{controller$get_output()}, or row-binded from multiple
data frames returned by \code{controller$get_output()} (e.g., users may
run simulation under the \code{targets} framework).}
}
\value{
A data frame of class \code{milestone_time_summary}. It comes with a
\code{plot} method for visualization.
}
\description{
Summary of Milestone Time from Simulated Trials
}
\examples{

# a minimum, meaningful, and executable example,
# where a randomized trial with two arms is simulated and analyzed.

control <- arm(name = 'control arm')
active <- arm(name = 'active arm')

pfs_in_control <- endpoint(name = 'PFS', type = 'tte', generator = rexp, rate = log(2) / 5)
control$add_endpoints(pfs_in_control)

pfs_in_active <- endpoint(name = 'PFS', type = 'tte', generator = rexp, rate = log(2) / 6)
active$add_endpoints(pfs_in_active)

accrual_rate <- data.frame(end_time = c(10, Inf), piecewise_rate = c(30, 50))
trial <- trial(name = 'trial',
               n_patients = 1000,
               duration = 40,
               enroller = StaggeredRecruiter,
               accrual_rate = accrual_rate,
               dropout = rweibull, shape = 2, scale = 38,
               silent = TRUE)

trial$add_arms(sample_ratio = c(1, 1), control, active)

action_at_final <- function(trial){
  locked_data <- trial$get_locked_data('final analysis')
  fitLogrank(Surv(PFS, PFS_event) ~ arm, placebo = 'control arm',
             data = locked_data, alternative = 'less')
  invisible(NULL)
}

final <- milestone(name = 'final analysis',
                   action = action_at_final,
                   when = eventNumber(endpoint = 'PFS', n = 300))

listener <- listener(silent = TRUE)
listener$add_milestones(final)

controller <- controller(trial, listener)
controller$run(n = 10, plot_event = FALSE, silent = TRUE)

output <- controller$get_output()
time <- summarizeMilestoneTime(output)
time

plot(time)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PiecewiseConstantExponentialRNG.R
\name{PiecewiseConstantExponentialRNG}
\alias{PiecewiseConstantExponentialRNG}
\title{Generate Time-to-Event Endpoint from Piecewise Constant Exponential Distribution}
\usage{
PiecewiseConstantExponentialRNG(n, risk, endpoint_name)
}
\arguments{
\item{n}{integer. Number of random numbers}

\item{risk}{a data frame of columns
\describe{
\item{\code{end_time}}{ End time for a constant risk in a time window.
The start time of the first time window is 0.}
\item{\code{piecewise_risk}}{ A constant risk in a time window, which is
absolute risk * relative risk, or (h0 * g) in the link.}
\item{\code{hazard_ratio}}{ An optional column for simulating an active arm.
If absent, a column of 1s will be added. Equivalently, user can multiply
\code{piecewise_risk} by \code{hazard_ratio} manually and ignore this column.}
}}

\item{endpoint_name}{character. Name of endpoint. This should be the same as
the \code{name} argument when calling function \code{endpoint()}.}
}
\value{
a data frame of \code{n} rows and two columns
\describe{
\item{\code{<endpoint_name>}}{name of endpoint specified by users in
\code{endpoint_name}. }
\item{<endpoint_name>_event}{event indicator with 0/1 as censoring and
event, respectively. Note that due to the nature of the algorithm to
generate data from this distribution, it is possible to have the endpoint
being censoring at the last \code{end_time} unless it is set to \code{Inf}.}
}
}
\description{
This function can be used as generator to define endpoint.
Implementation is based on
\href{https://www.demogr.mpg.de/papers/technicalreports/tr-2010-003.pdf}{this algorithm}.
This distribution can be used to simulate delayed treatment effect.
}
\examples{
# example code
# In this example, absolute risk in [0, 1) and [26, 52] are 0.0181 and
# 0.0027, respectively.
risk <- data.frame(
  end_time = c(1, 4.33, 26.0, 52.0),
  piecewise_risk = c(1, 1.01, 0.381, 0.150) * exp(-4.01)
)
PiecewiseConstantExponentialRNG(10, risk, 'PFS')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/milestone.R
\name{milestone}
\alias{milestone}
\title{Define a Milestone}
\usage{
milestone(name, when, action = doNothing, ...)
}
\arguments{
\item{name}{character. Name of milestone.}

\item{when}{condition to check if this milestone should be
triggered. It taks value returned from functions \code{calendarTime()},
\code{enrollment()}, \code{eventNumber()} or their logic combinations.}

\item{action}{function to execute when the milestone triggers.
If no action to be executed but simply need to record triggering time and
number of events/non-missing observations of endpoints at
a milestone, \code{action} can be its default value, a built-in function
\code{doNothing}.}

\item{...}{(optional) arguments of \code{action}.}
}
\description{
Define a milestone of a trial. This is a user-friendly wrapper for
the class constructor \code{Milestones$new()}. Users who are not familiar with
the concept of classes may consider using this wrapper directly.

A milestone means the time point to take an action,
e.g., carrying out (futility, interim, final) analysis for
adding/removing arms, or stopping a trial early.
It can also be any more general time point where trial
data is used in decision making or adaptation. For example, one can define a
milestone for changing randomization scheme, sample size re-assessment, trial
duration extension etc.

Refer to the
\href{https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html}{vignette}
to learn how to define milestones when performing simulation using
\code{TrialSimulator}.
}
\examples{
## See vignette('conditionSystem')


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitLogistic.R
\name{fitLogistic}
\alias{fitLogistic}
\title{Fit logistic regression model}
\usage{
fitLogistic(formula, placebo, data, alternative, scale, ...)
}
\arguments{
\item{formula}{An object of class \code{formula}. Must include \code{arm}
and endpoint in \code{data}. Covariates can be adjusted.}

\item{placebo}{Character. String indicating the placebo in \code{data$arm}.}

\item{data}{Data frame. Usually it is a locked data set.}

\item{alternative}{a character string specifying the alternative hypothesis,
must be one of \code{"greater"} or \code{"less"},
i.e., one-sided test is enforced. No default value.
\code{"greater"} means superiority of treatment over placebo is established
by an odds ratio greater than 1.}

\item{scale}{character. The type of estimate in the output. Must be one
of \code{"coefficient"}, \code{"log odds ratio"}, \code{"odds ratio"}, \code{"risk ratio"},
or \code{"risk difference"}. No default value.}

\item{...}{Subset conditions compatible with \code{dplyr::filter}.
\code{glm} will be fitted on this subset only. This argument can be useful
to create a subset of data for analysis when a trial consists of more
than two arms. By default, it is not specified,
all data will be used to fit the model. More than one condition can be
specified in \code{...}, e.g.,
\code{fitLogistic(remission ~ arm, 'pbo', data, 'greater', 'odds ratio', arm \%in\% c('pbo', 'low dose'), cfb > 0.5)},
which is equivalent to:
\code{fitLogistic(remission ~ arm, 'pbo', data, 'greater', 'odds ratio', arm \%in\% c('pbo', 'low dose') & cfb > 0.5)}.
Note that if more than one treatment arm are present in the data after
applying filter in \code{...}, models are fitted for placebo verse
each of the treatment arms.}
}
\value{
a data frame with columns:
\describe{
\item{\code{arm}}{name of the treatment arm. }
\item{\code{placebo}}{name of the placebo arm. }
\item{\code{estimate}}{estimate depending on \code{scale}. }
\item{\code{p}}{one-sided p-value for log odds ratio (treated vs placebo). }
\item{\code{info}}{sample size used in model with \code{NA} being removed. }
\item{\code{z}}{z statistics of log odds ratio (treated vs placebo). }
}
}
\description{
Fit logistic regression model on an binary endpoint.

Refer to \href{https://zhangh12.github.io/TrialSimulator/articles/wrappers.html}{this vignette}
for more information and examples.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Endpoint.R
\name{endpoint}
\alias{endpoint}
\title{Define Endpoints}
\usage{
endpoint(name, type, readout = NULL, generator, ...)
}
\arguments{
\item{name}{character vector. Name(s) of endpoint(s)}

\item{type}{character vector. Type(s) of endpoint(s) in \code{name}. It
supports \code{"tte"} for time-to-event endpoints, and \code{"non-tte"} for
all other types of endpoints (e.g., continuous, binary, categorical,
or repeated measurement. \code{TrialSimulator} will do some verification if
an endpoint is of type \code{"tte"}. However, no special
manipulation is done for non-tte endpoints.}

\item{readout}{numeric vector named by non-tte endpoint(s).
\code{readout} should be specified for every non-tte endpoint. For
example, \code{c(endpoint1 = 6, endpoint2 = 3)}, which means that it takes
6 and 3 unit time to get readouts of \code{endpoint1} and \code{endpoint2}
of a patient since being randomized. For readouts of a longitudinal endpoint
being collected at baseline (\code{baseline}) and 2 (\code{ep1}), 4 (\code{ep2})
unit time, its \code{readout} can be set as
\code{c(baseline = 0, ep1 = 2, ep2 = 4)}.  Error message will be prompted
if \code{readout} is not named or is not specified for all non-tte endpoint,
or it is specified for any tte endpoints. If all
endpoints are tte, \code{readout} should be its default value \code{NULL}.}

\item{generator}{a RNG function. Its first argument must be \code{n},
number of patients. It must return a data frame of \code{n} rows.
It supports all univariate random number generators, like those in
\code{stats}, e.g., \code{stats::rnorm}, \code{stats::rexp}, etc.
that with \code{n} as the first argument for number of
observations. \code{generator} could be any custom functions as long as
(1) its first argument is \code{n}; and (2) it returns a vector of
length \code{n} or a data frame of \code{n} rows. Custom random number
generator can return data of more than one endpoint. This is useful
when users need to simulate correlated endpoints (e.g., longitudinal
endpoints, or PFS/OS). The column names of
returned data frame should match to the argument \code{name} exactly,
but order does not matter. If an endpoint
is of type \code{"tte"}, the custom \code{generator} should also return
a column as event indicator. The column name of event indicator is
\code{<endpoint name>_event}. For example, if \code{"pfs"} is \code{"tte"},
then custom \code{generator} should return at least two columns
\code{"pfs"} and \code{"pfs_event"}. Usually \code{pfs_event} can be
all 1s if no censoring. For other generators, e.g.,
\code{TrialSimulator::PiecewiseConstantExponentialRNG} and
\code{TrialSimulator::CorrelatedPfsAndOs4}, the event indicators could
take values 0/1 due to the nature of their algorithms.
Censoring can also be specified later in \code{trial()}
through its argument \code{dropout}. See \code{?Trials}.
Note that if covariates, e.g., biomarker, subgroup, are needed in
generating and analyzing trial data, they can and should be defined as
endpoints as well.}

\item{...}{(optional) arguments of \code{generator}.}
}
\description{
Define one or multiple endpoints. This is a user-friendly wrapper for
the class constructor \code{Endpoints$new}. Users who are not familiar with
the concept of classes may consider using this wrapper directly.

Note that it is users' responsibility to assure that the units of
readout of non-tte endpoints, dropout time, and trial duration are consistent.
}
\examples{
set.seed(12345)
## Example 1. Generate a time-to-event endpoint.
## Two columns are returned, one for time, one for event (1/0, 0 for
## A built-in RNG function is used to handle piecewise constant exponential
## distribution
risk <- data.frame(
  end_time = c(1, 10, 26.0, 52.0),
  piecewise_risk = c(1, 1.01, 0.381, 0.150) * exp(-3.01)
)

pfs <- endpoint(name = 'pfs', type='tte',
                generator = PiecewiseConstantExponentialRNG,
                risk = risk, endpoint_name = 'pfs')

# run it in R console to display a summary report
# event indicator takes values 0/1
pfs

## Example 2. Generate continuous and binary endpoints using R's built-in
## RNG functions, e.g. rnorm, rexp, rbinom, etc.
ep1 <- endpoint(
         name = 'cd4', type = 'non-tte', generator = rnorm, readout = c(cd4=1),
         mean = 1.2)
ep2 <- endpoint(
         name = 'resp_time', type = 'non-tte', generator = rexp, readout = c(resp_time=0),
         rate = 4.5)
ep3 <- endpoint(
         name = 'orr', type = 'non-tte', readout = c(orr=3), generator = rbinom,
         size = 1, prob = .4)

ep1 # run it in R console. Mean and sd should be comparable to (1.2, 1.0)

ep2 # run it in R console. Median should be comparable to log(2)/4.5 = 0.154

ep3 # run it in R console. Mean and sd should be comparable to 0.4 and 0.49


## Example3: delayed effect
## Use piecewise constant exponential random number generator
## Baseline hazards are piecewise constant
## Hazard ratios are piecewise constant, resulting a delayed effect.
## Note that this example is for explaining the concept of "endpoint".
## Generating endpoint data manually is not the recommended way to use this package.

run <- TRUE

if (!requireNamespace("survminer", quietly = TRUE)) {
  run <- FALSE
  message("Please install 'survminer' to run this example.")
}

if (!requireNamespace("survival", quietly = TRUE)) {
  run <- FALSE
  message("Please install 'survival' to run this example.")
}

if(run){
risk1 <- risk
ep1 <- endpoint(
  name = 'pfs', type='tte',
  generator = PiecewiseConstantExponentialRNG,
  risk=risk1, endpoint_name = 'pfs')

risk2 <- risk1
risk2$hazard_ratio <- c(1, 1, .6, .4)
ep2 <- endpoint(
  name = 'pfs', type='tte',
  generator = PiecewiseConstantExponentialRNG,
  risk=risk2, endpoint_name = 'pfs')

n <- 1000
tte <- rbind(ep1$get_generator()(n), ep2$get_generator()(n))
arm <- rep(0:1, each = n)
dat <- data.frame(tte, arm)
sfit <- survival::survfit(
  survival::Surv(time = pfs, event = pfs_event) ~ arm, dat)

survminer::ggsurvplot(sfit,
           data = dat,
           pval = TRUE,  # Show p-value
           conf.int = TRUE,  # Show confidence intervals
           risk.table = TRUE,  # Add risk table
           palette = c("blue", "red"))


## print summary reports for endpoint objects in console
ep1
ep2

}

## Example 4: generate correlated pfs and os
## See vignette('simulatePfsAndOsIdm') and vignette('simulatePfsAndOsGumbel')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_arms.R
\name{add_arms}
\alias{add_arms}
\title{Adding One or More Arms to a Trial}
\usage{
add_arms(trial, sample_ratio, ...)
}
\arguments{
\item{trial}{a trial object returned by \code{trial()}.}

\item{sample_ratio}{integer vector. Sample ratio for permuted block
randomization. It will be appended to existing sample ratio in the trial.}

\item{...}{one or more objects returned from \code{arm()}.
Randomization is carried out with updated
sample ratio of newly added arm.
Note that we can run \code{Trials$add_arm(sample_ratio1, arm1)} followed
by \code{add_arms(trial, sample_ratio2, arm2)}.
We would expected similar result with
\code{add_arms(trial, c(sample_ratio1, sample_ratio2), arm1, arm2)}. Note
that these two methods won't return exactly the same trial because
randomization queue are generated twice in the first approach but only
once in the second approach. But statistically, they are equivalent and
of the same distribution.}
}
\description{
add one or more arms to a trial. This function can be used in two
scenarios: (1) adding arms right after a trial is created
(i.e., \code{trial(...)}). \code{sample_ratio} and arms added through
\code{...} must be of same length; (2) adding arms to an existing trial in
action functions of milestones.

This is a user-friendly wrapper of the member function of trial, i.e.,
\code{Trials$add_arms()}, which is used in vignettes. Users who are not
familiar with the concept of classes may consider using this wrapper
directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitFarringtonManning.R
\name{fitFarringtonManning}
\alias{fitFarringtonManning}
\title{Farrington-Manning test for rate difference}
\usage{
fitFarringtonManning(endpoint, placebo, data, alternative, ..., delta = 0)
}
\arguments{
\item{endpoint}{Character. Name of the endpoint in \code{data}.}

\item{placebo}{Character. String indicating the placebo in \code{data$arm}.}

\item{data}{Data frame. Usually it is a locked data set.}

\item{alternative}{a character string specifying the alternative hypothesis,
must be one of \code{"greater"} or \code{"less"}, i.e.,
one-sided test is enforced. No default value.
\code{"greater"} means superiority of treatment over placebo is established
by rate difference greater than `delta`.}

\item{...}{Subset conditions compatible with \code{dplyr::filter}.
\code{glm} will be fitted on this subset only. This argument can be useful
to create a subset of data for analysis when a trial consists of more
than two arms. By default, it is not specified,
all data will be used to fit the model. More than one condition can be
specified in \code{...}, e.g.,
\code{fitFarringtonManning('remission', 'pbo', data, delta, arm \%in\% c('pbo', 'low dose'), cfb > 0.5)},
which is equivalent to:
\code{fitFarringtonManning('remission', 'pbo', data, delta, arm \%in\% c('pbo', 'low dose') & cfb > 0.5)}.
Note that if more than one treatment arm are present in the data after
applying filter in \code{...}, models are fitted for placebo verse
each of the treatment arms.}

\item{delta}{the rate difference between a treatment arm and placebo under
the null. 0 by default.}
}
\value{
a data frame with three columns:
\describe{
\item{\code{arm}}{name of the treatment arm. }
\item{\code{placebo}}{name of the placebo arm. }
\item{\code{estimate}}{estimate of rate difference. }
\item{\code{p}}{one-sided p-value for log odds ratio (treated vs placebo). }
\item{\code{info}}{sample size in the subset with \code{NA} being removed. }
\item{\code{z}}{the z statistics of log odds ratio (treated vs placebo). }
}
}
\description{
Test rate difference by comparing it to a pre-specified value using the
Farrington-Manning test.

Refer to \href{https://zhangh12.github.io/TrialSimulator/articles/wrappers.html}{this vignette}
for more information and examples.
}
\references{
Farrington, Conor P., and Godfrey Manning. "Test statistics and sample size formulae for comparative binomial trials with null hypothesis of non-zero risk difference or non-unity relative risk." Statistics in medicine 9.12 (1990): 1447-1454.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CorrelatedPfsAndOs4.R
\name{CorrelatedPfsAndOs4}
\alias{CorrelatedPfsAndOs4}
\title{Generate Correlated PFS, OS and Objective Response}
\usage{
CorrelatedPfsAndOs4(
  n,
  transition_probability,
  duration,
  death_name = "death",
  progression_name = "progression",
  response_name = "response"
)
}
\arguments{
\item{n}{integer. Number of observations.}

\item{transition_probability}{a 4x4 matrix defining transition probabilities
between stable (initial state, 1), response (2), progression (3) and
death (absorbing, 4).}

\item{duration}{integer. Duration of trial. Set it to a sufficient large
integer in practice to cover the duration of the trial (potentially be
extended).}

\item{death_name}{column name of OS in returned data frame. It must be
consistent with \code{name} in the function \code{endpoint()}.}

\item{progression_name}{column name of PFS in returned data frame. It must be
consistent with \code{name} in the function \code{endpoint()}.}

\item{response_name}{column name of objective response in returned data frame. It must be
consistent with \code{name} in the function \code{endpoint()}.}
}
\value{
A data frame of \code{n} rows and 6 columns (response, progression,
death, and their event indicators with 1 means event and 0 means censored
at duration). The column names are
\code{<death_name>}, \code{<death_name>_event}, \code{<progression_name>},
\code{<progression_name>_event}, \code{<response_name>} and
\code{<response_name>_event}.

Note that it returns time-to-response for each patients with status of
censoring at pre-set duration. If a binary indicator of response at
a time point is needed as an endpoint, we may write a wrapper function to
convert the column \code{<response_name>} to binary and remove the column
\code{<response_name>_event} from return value.
}
\description{
Generate correlated PFS, OS and objective response using the
four-states model. It can be used as custom generator of \code{endpoint()}.
}
\examples{
m <- matrix(c(0.99, 0.0035, 0.0055, 0.0010,
                 0, 0.9900, 0.0052, 0.0048,
                 0,      0, 0.9960, 0.0040,
                 0,      0,      0,      1),
             nrow = 4, byrow = TRUE)

## use as function (if you don't use TrialSimulator for simulation)

dat <- CorrelatedPfsAndOs4(1e4, m, 365 * 3)

## use as generator (if you use TrialSimulator for simulation)

ep <- endpoint(name = c('pfs', 'os', 'or'),
               type = c('tte', 'tte', 'tte'), ## OR is TTE, not binary
               generator = CorrelatedPfsAndOs4,
               transition_probability = m,
               duration = 365 * 3,
               death_name = 'os', ## rename output from generator to match with "name"
               progression_name = 'pfs',
               response_name = 'or')

ep # run it in console to see summary report

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solveThreeStateModel.R
\name{plot.three_state_model}
\alias{plot.three_state_model}
\title{Plot result of three-state illness-death model}
\usage{
\method{plot}{three_state_model}(x, ...)
}
\arguments{
\item{x}{an object returned by \code{solveThreeStateModel()}.}

\item{...}{currently not supported.}
}
\description{
Plot result of three-state illness-death model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CorrelatedPfsAndOs3.R
\name{CorrelatedPfsAndOs3}
\alias{CorrelatedPfsAndOs3}
\title{Generate Correlated PFS and OS Using the Three-States Illness-Death Model}
\usage{
CorrelatedPfsAndOs3(n, h01, h02, h12, pfs_name = "pfs", os_name = "os")
}
\arguments{
\item{n}{integer. Number of observations.}

\item{h01}{constant transition hazard from state "initial" to state "progression".}

\item{h02}{constant transition hazard from state "initial" to state "death".}

\item{h12}{constant transition hazard from state "progression" to state "death".}

\item{pfs_name}{column name of PFS in returned data frame. It must be
consistent with \code{name} in the function \code{endpoint()}.}

\item{os_name}{column name of OS in returned data frame. It must be
consistent with \code{name} in the function \code{endpoint()}.}
}
\value{
A data frame of \code{n} rows and four columns, including PFS, OS and their
event indicators. The event indicators are all 1s. The column names are
\code{<pfs_name>}, \code{<pfs_name>_event}, \code{<os_name>},
and \code{<os_name>_event}.
}
\description{
Generate correlated PFS and OS endpoints using the three-states model. This
function can be used as custom \code{generator} in the function
\code{endpoint()}.
}
\examples{
## use as function (if you don't use TrialSimulator for simulation)
pfs_and_os_trt <- CorrelatedPfsAndOs3(1e4, 0.06, 0.30, 0.30, 'PFS', 'OS')
pfs_and_os_pbo <- CorrelatedPfsAndOs3(1e4, 0.10, 0.40, 0.30, 'PFS', 'OS')

## use as generator (if you use TrialSimulator for simulation)

pfs_and_os <- endpoint(name = c('PFS', 'os'),
                       type = c('tte', 'tte'),
                       generator = CorrelatedPfsAndOs3,
                       h01 = .06, h02 = .30, h12 = .30,
                       pfs_name = 'PFS', os_name = 'os')

pfs_and_os # run it in console to see summary report

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weibullDropout.R
\name{weibullDropout}
\alias{weibullDropout}
\title{Calculate Parameters of Weibull Distribution as a Dropout Method}
\usage{
weibullDropout(time, dropout_rate)
}
\arguments{
\item{time}{a numeric vector of two time points at which dropout rates are
specified.}

\item{dropout_rate}{a numeric vector of dropout rates at \code{time}.}
}
\value{
a named vector for scale and shape parameters.
}
\description{
Fit scale and shape parameters of the Weibull distribution to match
dropout rates at two specified time points. Weibull distribution can be
used as a dropout distribution because it has two parameters.

Note that It is users' responsibility to assure that the units of dropout
time, readout of non-tte endpoints, and trial duration are consistent.
}
\examples{
## dropout rates are 8\% and 18\% at time 12 and 18.
weibullDropout(time = c(12, 18), dropout_rate = c(.08, .18))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_sample_ratio.R
\name{update_sample_ratio}
\alias{update_sample_ratio}
\title{Updating Sampling Ratios of Existing Arms in a Trial}
\usage{
update_sample_ratio(trial, arm_names, sample_ratios)
}
\arguments{
\item{trial}{a trial object returned by \code{trial()}.}

\item{arm_names}{character vector. Name of arms.}

\item{sample_ratios}{numeric vector. New sample ratios of arms. If sample
ratio is a whole number, the permuted block randomization is adopted;
otherwise, \code{sample()} will be used instead, which can cause imbalance
between arms by chance. However, this is usually fine for simulation.}
}
\description{
update sample ratios of arms. This could be called after an arm is added or
removed. Sample ratios can be updated for any existing arms.

This is a user-friendly wrapper of the member function of trial, i.e.,
\code{Trials$update_sample_ratio()}, which is used in vignettes.
Users who are not familiar with the concept of classes may consider using
this wrapper directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solvePiecewiseConstantExponentialDistribution.R
\name{solvePiecewiseConstantExponentialDistribution}
\alias{solvePiecewiseConstantExponentialDistribution}
\title{Compute Constant Rates of Piecewise Exponential Distribution}
\usage{
solvePiecewiseConstantExponentialDistribution(surv_prob, times)
}
\arguments{
\item{surv_prob}{numeric. A vector of survival probabilities at \code{times}.}

\item{times}{numeric. A vector of time points where event rates change.
\code{times} and \code{surv_prob} must be of equal length.}
}
\value{
a data frame of two columns
\describe{
\item{\code{end_time}}{End time for a constant event rate. The start time of
the first time window is 0. }
\item{piecewise_risk}{A constant event rate in the time window ending with
\code{end_time} on the same row. }
}
}
\description{
This function computes the rate parameters \code{lambda} in each of the time
windows. \code{lambda} are natural parameters of piecewise exponential
distribution, but in practice, users may define the distribution by
specifying the survival probabilities at time points where event rates
change.

This function returns a data frame, which can be used as input of the
argument \code{risk} of the data generator \code{PiecewiseConstantExponentialRNG}.
}
\examples{
solvePiecewiseConstantExponentialDistribution(
  surv_prob = c(.9, .75, .64, .42, .28),
  times = c(.4, 1.2, 4, 5.5, 9)
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Listener.R
\name{listener}
\alias{listener}
\title{Define a Listener}
\usage{
listener(silent = FALSE)
}
\arguments{
\item{silent}{logical. \code{TRUE} to mute messages.}
}
\description{
Define a listener. This is a user-friendly wrapper for
the class constructor \code{Listener$new()}. Users who are not familiar with
the concept of classes may consider using this wrapper directly.

Listener is an important concept of \code{TrialSimulator}. Used with a
trial object in a controller, a listener can monitor a running trial to
execute user-defined actions when it determine condition of triggering a
milestone is met. This mechanism allows the package users to focus on
the development of action functions in a simulation.
}
\examples{

listener <- listener()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solveThreeStateModel.R
\name{solveThreeStateModel}
\alias{solveThreeStateModel}
\title{Solve Parameters in a Three-State Illness-death Model}
\usage{
solveThreeStateModel(
  median_pfs,
  median_os,
  corr,
  h12 = seq(0.05, 0.2, length.out = 50)
)
}
\arguments{
\item{median_pfs}{numeric. Median of PFS.}

\item{median_os}{numeric. Median of OS.}

\item{corr}{numeric vector. Pearson correlation coefficients between PFS and OS.}

\item{h12}{numeric vector. A set of hazard from progression to
death that may induce the target correlation \code{corr} given \code{median_pfs}
and \code{median_os}. \code{solveThreeStateModel()} will do a grid search to
find the best hazard parameters that matches to the medians of PFS and OS,
and their correlations.}
}
\value{
a data frame with columns:
\describe{
\item{\code{corr}}{target Peason's correlation coefficients. }
\item{\code{h01}}{hazard from stable to progression. }
\item{\code{h02}}{hazard from stable to death.  }
\item{\code{h12}}{hazard from progression to death. }
\item{\code{error}}{absolute error between target correlation and correlation
derived from \code{h01}, \code{h02}, and \code{h12}. }
}
}
\description{
The illness-death model consists of three states, \code{initial}, \code{progression},
and \code{death}. It can be used to model the progression-free survival (PFS)
and overall survival (OS) in clinical trial simulation. It models the
correlation between PFS and OS without assumptions on latent status and copula.
Also, it does not assume PFS and OS satisfy the proportional hazard assumption
simultaneously. The three-state illness-death model ensures a nice property that
PFS <= OS with probability one. However, it requires three hazard parameters
under the homogeneous Markov assumption. In practice, hazard parameters are
hard to specify intuitively especially when no trial data is available at
the planning stage.

This function reparametrizes the illness-death model in term of three parameters,
i.e. median of PFS, median of OS, and correlation between PFS and OS. The
output of this function, which consists of the three hazard parameters, can
be used to generate PFS and OS with desired property. It can be used with
the built-in data generator \code{CorrelatedPfsAndOs3()} when defining
endpoints in \code{TrialSimulator}.

For more information, refer to
\href{https://zhangh12.github.io/TrialSimulator/articles/simulatePfsAndOsIdm.html}{this vignette}.
}
\examples{

dat <- CorrelatedPfsAndOs3(1e6, h01 = .1, h02 = .05, h12 = .12)

cor(dat$pfs, dat$os) ## 0.65

median(dat$pfs) ## 4.62

median(dat$os) ## 9.61

## find h01, h02, h12 that can match to median_pfs, median_os and corr
## should be close to h01 = 0.10, h02 = 0.05, h12 = 0.12 when corr = 0.65
ret <- solveThreeStateModel(median_pfs = 4.6, median_os = 9.6,
                            corr = seq(.5, .7, length.out=5))
ret

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrollment.R
\name{enrollment}
\alias{enrollment}
\title{Triggering Condition by Number of Randomized Patients}
\usage{
enrollment(n, ..., arms = NULL, min_treatment_duration = 0)
}
\arguments{
\item{n}{integer. Number of randomized patients.}

\item{...}{subset conditions compatible with \code{dplyr::filter}. Number
of randomized patients will be counted on subset of trial data only.}

\item{arms}{vector of character. Name of arms on which the number of patients
is counted. If \code{NULL}, use all arms that are not yet removed from the
trial by the time of calculation.}

\item{min_treatment_duration}{numeric. Zero or positive value.
minimum treatment duration of enrolled patients.
Default is 0, i.e., looking for triggering time based on number of enrolled
patients in population specified by \code{...} and \code{arms}. If positive,
it means that milestone is triggered when a specific number of enrolled
patients have received treatment for at least \code{min_treatment_duration}
duration. It is users' responsibility to assure that the unit of
\code{min_treatment_duration} are consistent with
readout of non-tte endpoints, dropout time, and trial duration.}
}
\value{
an object of class `Condition`
}
\description{
Define a condition to trigger trial milestone by the number of randomized
patients. The milestone will be triggered when a trial has enrolled
at least the specified number of patients. It can be used combined with
conditions specified by \link[TrialSimulator]{calendarTime} and
\link[TrialSimulator]{eventNumber}.

Refer to the
\href{https://zhangh12.github.io/TrialSimulator/articles/conditionSystem.html}{vignette}
to learn how to define milestones when performing simulation using
\code{TrialSimulator}.
}
\examples{

## ensure sufficient sample size of whole trial
enrollment(n = 100)

## ensure sufficient sample size in sub-group of interest
enrollment(n = 100, biomarker1 == 'positive' & biomarker2 == 'high')

## ensure sufficient sample size in high dose + placebo
enrollment(n = 1000, arms = c('high dose', 'placebo'))

## ensure sufficient treatment duration
enrollment(n = 500, min_treatment_duration = 2)


}
